(* PlotDistributionsWealthLevSCF_CastanedaAndDistSevenNoAggShock.m *)

(* Comparions of wealth levels (SCF (from Castaneda) and Dist (with time pref factors approximated by 7 PointNoAggShocks) etc. *)



SetDirectory[NotebookDirectory[]];

If[Directory[] != NoAggShockDir, SetDirectory[NoAggShockDir]]; (* If this file is run in DoAll file and current folder is "Mathematica", current folder is changed to "NoAggShock". *)

(*

ClearAll["Global`*"];

*)





Maxx = 100;



(* SCF from Castaneda. *)

WSCFCum  = {0,  0, (100-99.0)/100, (100-94.7)/100, (100-82.9)/100, (100-69.7)/100, (100-57.9)/100, (100-33.9)/100, 1} ; (* SCF2004 *)

(*

WSCFCum  = {0,  0, 0.013, 0.071, 0.205, 0.339, 0.465, 0.705, 1} ; (* SCF1992 from castaneda 2003 *)

*)

yAxisSCF = {0, 20,    40,    60,    80,    90,    95,    99, 100};



CumSCFPlot = ListLinePlot[Transpose[{yAxisSCF, WSCFCum}]

   , InterpolationOrder -> 3

   , PlotRange  -> {{0, Maxx}, {0, 1}}



   , PlotStyle  -> {Black, Thickness[0.004]}

];



(* Plot simulated data *)

kLevListKSNoAggShock       = Import["kLevListKSNoAggShock.txt","List"]/100; (* Import data *)

kLevtsortedCumKS = 1-kLevListKSNoAggShock; 

(*

kLevtsortedCumKS  = {0,  1-0.900, 1-0.747, 1-0.655, 1-0.554, 1-0.382, 1-0.318, 1-0.177, 1-0.097, 1-0.023,   1};

*)

yAxisKS           = {0,       20,      40,      50,      60,      75,      80,      90,      95, 99, 100};



CumKSPlot     = ListLinePlot[Transpose[{yAxisKS, kLevtsortedCumKS}] 

   , InterpolationOrder -> 3

   , PlotRange  -> {{0, Maxx}, {0, 1}}

   , PlotStyle  -> {Black, Dashing[{0.04, 0.04}],Thickness[0.004]}

   ];



kLevtsortedCumKSDist  = {1-0.92, 1-0.88, 1-0.73, 1-0.55, 1-0.24};

yAxisKSDist           = {    70,     80,     90,     95,     99};



CumKSHeteroPlot     = ListPlot[Transpose[{yAxisKSDist, kLevtsortedCumKSDist}] 



   , Joined     -> True

   , PlotRange  -> {{0, Maxx}, {0, 1}}



   , PlotStyle  -> {Black, Dashing[{0.04, 0.04}],Thickness[0.004]}



   ];



kLevListPointNoAggShock       = Import["kLevListPointNoAggShock.txt","List"]/100; (* Import data *)

kLevtsortedCumPointNoAggShock = 1-kLevListPointNoAggShock; 

(*

kLevtsortedCumPointNoAggShock  = {0,  1-0.970, 1-0.888, 1-0.829, 1-0.756, 1-0.610, 1-0.548, 1-0.385, 1-0.266, 1-0.107,   1};

*) 

yAxisPointNoAggShock           = {0,       20,      40,      50,      60,      75,      80,      90,      95,      99, 100};



CumPointNoAggShockPlot     = ListLinePlot[Transpose[{yAxisPointNoAggShock, kLevtsortedCumPointNoAggShock}] 

   , InterpolationOrder -> 3

   , PlotRange  -> {{0, Maxx}, {0, 1}}

   , PlotStyle  -> {Black, Dashing[{0.02, 0.02}],Thickness[0.004]}

   ];



(* Stats produced matching 20%, 40% and 60%. *)

kLevListDistSevenNoAggShock       = Import["kLevListDistSevenNoAggShock.txt","List"]/100; (* Import data *)

kLevtsortedCumDistSevenNoAggShock = 1-kLevListDistSevenNoAggShock; 

(*

kLevtsortedCumDistSevenNoAggShock  = {0,  1-0.993, 1-0.971, 1-0.953, 1-0.925, 1-0.847, 1-0.802, 1-0.649, 1-0.499, 1-0.243,   1};

*)

yAxisDistSevenNoAggShock           = {0,       20,      40,      50,      60,      75,      80,      90,      95,      99, 100};



CumDistSevenNoAggShockPlot     = ListLinePlot[Transpose[{yAxisDistSevenNoAggShock, kLevtsortedCumDistSevenNoAggShock}] 

   , InterpolationOrder -> 3

   , PlotRange  -> {{0, Maxx}, {0, 1}}

(*

   , PlotStyle  -> {Black, Thickness[0.004]}

*)

   , PlotStyle  -> {Black, Dashing[{0.01, 0.01}],Thickness[0.004]}



   ];





CumWLevSCFCastanedaAndDistSevenNoAggShockPlot = Show[

   {CumSCFPlot, CumKSPlot, (* CumKSHeteroPlot, *) CumPointNoAggShockPlot, CumDistSevenNoAggShockPlot (*,  CumDistPlot *)}



   , Graphics[Text["  US data (SCF, solid line) "              ,{58, 0.028},{-1,0}]]

   , Graphics[Text[" KS-JEDC "                  ,{55, 0.6}, {-1,0}]]

(*

   , Graphics[Text["  KS-Orig Hetero"                   ,{78, 0.09}, {-1,0}]]

*)

   , Graphics[Text["  -Point"             ,{70, 0.3}, {-1,0}]]

   , Graphics[Text[" -Dist"              ,{28, 0.27}, {-1,0}]]

   , Graphics[Text[" Percentile" , {87, -0.1}, {-1,0}]]



   , Graphics[{Arrowheads[.02],Arrow[{{35,0.2},{35,0.02}}]}]

   , PlotRangeClipping -> False

   , ImagePadding -> 29

   , AxesLabel -> {None, \[ScriptCapitalF]}

   , ImageSize -> {72. 6.5,72. 6.5/GoldenRatio}



   , Ticks     -> {{0, 25, 50, 75, 100}, {0, .25, .50, .75, 1}}

   , TextStyle -> {FontSize -> 14}

   ]



Export[ParentDirectory[ParentDirectory[ParentDirectory[]]] <> "/figures/CumWLevSCFCastanedaAndDistSevenNoAggShockPlot.EPS", CumWLevSCFCastanedaAndDistSevenNoAggShockPlot, "EPS"];

Export[ParentDirectory[ParentDirectory[ParentDirectory[]]] <> "/figures/CumWLevSCFCastanedaAndDistSevenNoAggShockPlot.PNG", CumWLevSCFCastanedaAndDistSevenNoAggShockPlot, "PNG"];

Export[ParentDirectory[ParentDirectory[ParentDirectory[]]] <> "/figures/CumWLevSCFCastanedaAndDistSevenNoAggShockPlot.pdf", CumWLevSCFCastanedaAndDistSevenNoAggShockPlot, "pdf"];





CumWLevSCFAndDistSevenNoAggShockPlot = Show[

   {CumSCFPlot, CumDistSevenNoAggShockPlot (*,  CumDistPlot *)}



   , Graphics[Text["  US data (SCF, solid line) "              ,{58, 0.028},{-1,0}]]



   , Graphics[Text[" -Dist"              ,{28, 0.27}, {-1,0}]]

   , Graphics[Text[" Percentile" , {87, -0.1}, {-1,0}]]



   , Graphics[{Arrowheads[.02],Arrow[{{35,0.2},{35,0.02}}]}]

   , PlotRangeClipping -> False

   , ImagePadding -> 29

   , AxesLabel -> {None, \[ScriptCapitalF]}

   , ImageSize -> {72. 6.5,72. 6.5/GoldenRatio}



   , Ticks     -> {{0, 25, 50, 75, 100}, {0, .25, .50, .75, 1}}

   , TextStyle -> {FontSize -> 14}

   ]



Export[ParentDirectory[ParentDirectory[ParentDirectory[]]] <> "/figures/CumWLevSCFAndDistSevenNoAggShockPlot.EPS", CumWLevSCFAndDistSevenNoAggShockPlot, "EPS"];

Export[ParentDirectory[ParentDirectory[ParentDirectory[]]] <> "/figures/CumWLevSCFAndDistSevenNoAggShockPlot.PNG", CumWLevSCFAndDistSevenNoAggShockPlot, "PNG"];

Export[ParentDirectory[ParentDirectory[ParentDirectory[]]] <> "/figures/CumWLevSCFAndDistSevenNoAggShockPlot.pdf", CumWLevSCFAndDistSevenNoAggShockPlot, "pdf"];

