(* SolveAndSimDistSeven.m *)
(* This file runs \[Beta]-Dist (uniformly distributed \[Beta]) model with time pref factors approximated by seven points. In the model, there are seven types of households. *)

SetDirectory[NotebookDirectory[]];
If[Directory[] != WithAggShocksDir, SetDirectory[WithAggShocksDir]];
(*
ClearAll["Global`*"];
*)

Print["============================================================="];
Print["Run \[Beta]-Dist model with time pref factors approximated by seven points"];

Print["========================================="];
(* Solve and simulate rep agent model with aggregate shocks to obtain initial agg process *)
<<SolveAndSimRepAgent.m; 

TimeStart = SessionTime[]; 

Print["========================================="];
Print["Running \[Beta]-Dist model with time pref factors approximated by seven points... "];
ModelType         = Dist;   (* Indicate that the model is Dist *) 
NumOfApproxPoints = 7;      (* Time pref factors approximated by seven points *)
Rep               = False;

(* Setup routines and set parameter values specific to this file *)
<<PrepareEverything.m;        (* Setup everything (routines etc.) *)
\[Beta]middle = Import["BetamiddleWithAggShocks.txt","List"][[1]]; (* Mean of time pref factors *)
diff          = Import["diffWithAggShocks.txt","List"][[1]];       (* Diff between approximated points (of time pref factors) *)
Print["\[EmptyDownTriangle]/3.5, \[Beta]middle: "];
Print[{diff, \[Beta]middle}];
TimesToEstimateSmall = 5;       (* Number of times to estimate agg process with small(er) number of people *) 

NumPeopleToSim       = 5600;    (* Number of people to simulate *)
NumPeopleToSimLarge  = NumPeopleToSim*10;   (* Number of people to simulate in large simulation(s) (needs to be a multiple of NumPeopleToSim). This param can be increased to further reduce sim error. *)

(*
NumPeopleToSim       = 7000;    (* Number of people to simulate *)
NumPeopleToSimLarge  = 70000;   (* Number of people to simulate in large simulation(s) (needs to be a multiple of NumPeopleToSim). This param can be increased to further reduce sim error. *)
*)

(* Generate steady state distribution of Pt (PtSSDist) *)
PeriodsToGenPtSSDist   = 5000;            (* Periods to use in generating steady state distribution of Pt *)
NumPeopleToGenPtSSDist = NumPeopleToSim;  (* Number of people to simulate to gen steady state distribution of Pt *)
GenPtSSDist;

(* Construct converged consumption function and run simulation *)
GapCoeff  = 1; (* Initial value of gap *)
Estimatet = 1;
RunModelt  = Import["RunModelt.txt","List"][[1]]; (* This starts with 1 when we run the Dist model. *)
While[GapCoeff > MaxGapCoeff || Estimatet <= TimesToEstimateSmall+2 (* Iterate while the max gap of the estiamted coeff is hither than 1%. Iterate at least TimesToEstimateSmall+2 times. *), 

   Print["Iteration ", Estimatet];   
       
    (* Construct consumption function *)
    StartConstructingFunc = SessionTime[]; 
(*
    Print[" Solving ind consumption function(s)..."];
*)


    \[Beta] = (\[Beta]middle + 3 diff)  (1-ProbOfDeath);   (* Effective time preference factor *)  
    If[Estimatet>1 || RunModelt>1, cInterpFuncList = cInterpFuncList1];
    ConstructKSIndFunc; 
    KSIndcFunc[mRatt_,AggStatet_,EmpStatet_,KRatt_,1] = KSIndcFunc[mRatt,AggStatet,EmpStatet,KRatt];
    cInterpFuncList1 = cInterpFuncList; 

    \[Beta] = (\[Beta]middle + 2 diff) (1-ProbOfDeath);   (* Effective time preference factor *)  
    If[Estimatet>1 || RunModelt>1, cInterpFuncList = cInterpFuncList2];
    ConstructKSIndFunc; 
    KSIndcFunc[mRatt_,AggStatet_,EmpStatet_,KRatt_,2] = KSIndcFunc[mRatt,AggStatet,EmpStatet,KRatt];
    cInterpFuncList2 = cInterpFuncList; 

    \[Beta] = (\[Beta]middle +   diff) (1-ProbOfDeath);   (* Effective time preference factor *)  
    If[Estimatet>1 || RunModelt>1, cInterpFuncList = cInterpFuncList3];
    ConstructKSIndFunc; 
    KSIndcFunc[mRatt_,AggStatet_,EmpStatet_,KRatt_,3] = KSIndcFunc[mRatt,AggStatet,EmpStatet,KRatt];
    cInterpFuncList3 = cInterpFuncList; 

    \[Beta] = \[Beta]middle (1-ProbOfDeath);   (* Effective time preference factor *)  
    If[Estimatet>1 || RunModelt>1, cInterpFuncList = cInterpFuncList4];
    ConstructKSIndFunc; 
    KSIndcFunc[mRatt_,AggStatet_,EmpStatet_,KRatt_,4] = KSIndcFunc[mRatt,AggStatet,EmpStatet,KRatt];
    cInterpFuncList4 = cInterpFuncList; 

    \[Beta] = (\[Beta]middle -  diff) (1-ProbOfDeath);   (* Effective time preference factor *)  
    If[Estimatet>1 || RunModelt>1, cInterpFuncList = cInterpFuncList5];
    ConstructKSIndFunc; 
    KSIndcFunc[mRatt_,AggStatet_,EmpStatet_,KRatt_,5] = KSIndcFunc[mRatt,AggStatet,EmpStatet,KRatt];
    cInterpFuncList5 = cInterpFuncList; 

    \[Beta] = (\[Beta]middle - 2 diff) (1-ProbOfDeath);   (* Effective time preference factor *)  
    If[Estimatet>1 || RunModelt>1, cInterpFuncList = cInterpFuncList6];
    ConstructKSIndFunc; 
    KSIndcFunc[mRatt_,AggStatet_,EmpStatet_,KRatt_,6] = KSIndcFunc[mRatt,AggStatet,EmpStatet,KRatt];
    cInterpFuncList6 = cInterpFuncList; 

    \[Beta] = (\[Beta]middle - 3 diff) (1-ProbOfDeath);   (* Effective time preference factor *)  
    If[Estimatet>1 || RunModelt>1, cInterpFuncList = cInterpFuncList7];
    ConstructKSIndFunc; 
    KSIndcFunc[mRatt_,AggStatet_,EmpStatet_,KRatt_,7] = KSIndcFunc[mRatt,AggStatet,EmpStatet,KRatt];
    cInterpFuncList7 = cInterpFuncList; 

    EndConstructingFunc   = SessionTime[]; 

    If[Estimatet==1 || TimesToEstimateSmall>3, 
    Print[" Time spent to solve consumption function(s) (minutes): ", (EndConstructingFunc - StartConstructingFunc)/60]
    ]; 

   (* Run simulation, show results, and estimate agg process *)
   SimulateKSInd;
       
   Estimatet++;

   ]; (* End While *)

(* Plot consumption function *)
maxisSeven = Table[0.2 m, {m, 0, 400}];
mlist = maxisSeven wSS/
   AdjustedLByAggState[[1]];(*Horizonal axis is cash on hand normalized by p (t)*wSS *)
caxisSevenHigh =  KSIndcFunc[mlist, 1, 1, kSS, 1] AdjustedLByAggState[[1]]/wSS;
caxisSevenLow  =  KSIndcFunc[mlist, 1, 1, kSS, 7] AdjustedLByAggState[[1]]/wSS;

Print[" Consumption function"]

Export["maxisSeven.txt", maxisSeven, "Table"]; 
maxisSeven = Import["maxisSeven.txt","List"]; 
Export["caxisSevenHigh.txt", caxisSevenHigh, "Table"]; 
caxisSevenHigh = Import["caxisSevenHigh.txt","List"]; 
Export["caxisSevenLow.txt", caxisSevenLow, "Table"]; 
caxisSevenLow = Import["caxisSevenLow.txt","List"]; 

<<PlotCFuncsDistSeven.m;

(* Export data on wealth distribution *)
kLevListDistSevenWithAggShock = {100, kLevTop80PercentMean, kLevTop60PercentMean, kLevTop50PercentMean, kLevTop40PercentMean, kLevTop25PercentMean, kLevTop20PercentMean, kLevTop10PercentMean, kLevTop5PercentMean, kLevTop1PercentMean, 0}; 
Export["kLevListDistSeven.txt", kLevListDistSevenWithAggShock, "Table"];
Export[ParentDirectory[] <> "/Results/kLevListDistSevenWithAggShock.txt", kLevListDistSevenWithAggShock, "Table"];

Export[ParentDirectory[] <> "/Results/AggStatsDistSevenWithAggShock.txt", AggStatsWithAggShock, "Table"];
Export["MPCsList.txt", MPCt, "Table"];

Export[ParentDirectory[] <> "/Results/MPCListDistSevenWithAggShock.txt",             MPCListWithAggShock, "Table"];
Export[ParentDirectory[] <> "/Results/MPCListRecessionDistSevenWithAggShock.txt",    MPCListRecessionWithAggShock, "Table"];
Export[ParentDirectory[] <> "/Results/MPCListExpansionDistSevenWithAggShock.txt",    MPCListExpansionWithAggShock, "Table"];
Export[ParentDirectory[] <> "/Results/MPCListEntRecessionDistSevenWithAggShock.txt", MPCListEntRecessionWithAggShock, "Table"];
Export[ParentDirectory[] <> "/Results/MPCListEntExpansionDistSevenWithAggShock.txt", MPCListEntExpansionWithAggShock, "Table"];

Export[ParentDirectory[] <> "/Results/MPCDistSevenWithAggShock.tex", Round[100 MeanMPCAnnual]/100//N, "Table"];
Export[ParentDirectory[] <> "/Results/MPCDistSevenBottomHalf.tex", Round[100 MeanMPCBottomHalfBykAnnual]/100//N, "Table"];

Export[ParentDirectory[] <> "/Results/kAR1Coeff1GoodDistSeven.tex", Round[1000 kAR1ByAggState[[1,1]]]/1000//N, "Table"];
Export[ParentDirectory[] <> "/Results/kAR1Coeff2GoodDistSeven.tex", Round[1000 kAR1ByAggState[[1,2]]]/1000//N, "Table"];
Export[ParentDirectory[] <> "/Results/kAR1Coeff1BadDistSeven.tex",  Round[1000 kAR1ByAggState[[3,1]]]/1000//N, "Table"];
Export[ParentDirectory[] <> "/Results/kAR1Coeff2BadDistSeven.tex",  Round[1000 kAR1ByAggState[[3,2]]]/1000//N, "Table"];

(* Display time spent *)  
TimeEnd = SessionTime[];  
Print[" Time spent to run \[Beta]-Dist model with time pref factors approximated by seven points (minutes): ",(TimeEnd - TimeStart)/60]; 