
#delimit;
** This program produces LaTeX code for the data tables in cstMPCxc;

clear; clear matrix; clear mata; program drop _all; discard;
set mem 3000m; set maxvar 30000; set matsize 155; set scheme s2color; set more off;

local basePath "U:\Slacalek\Laptop_X230\20130627\Research\BSinKS\20130408\cstMPCEurope\Latest\Code\Stata\";
* Current directory needs to be changed depending on working environment;
local stataPath "`basePath'";
local logPath "`stataPath'";
local dataPath "`basePath'data";
global docPath "`dataPath'";
adopath + "`stataPath'";
global eolString "_char(92) _char(92)";		* string denoting new line in LaTeX tables \\;

capture log close;
cd "`stataPath'";
log using "`logPath'\DoAllHFCStables.log", replace;
cd "`dataPath'";
*************************************************************************************;

use "`dataPath'\D_compl_normalIncome.dta";

local allCountries "AT BE CY DE ES FI FR GR IT LU MT NL PT SI SK";
local allCountriesFull "AT BE CY DE ES FI FR GR IT LU MT NL PT SI SK ALL";
local allWealthVars "wy lqy";
local allImplicates "1 2 3 4 5";
local allPercentiles "10 25 50 75";

* Data transformations;
*************************************************************************************;
* Drop data, if age < 25 or >60;
drop if age < 25|age > 60;
replace hg0700 = 2 if country=="FR"|country=="FI"; * for FR and FI indicator of permanent income is not available;
*keep if hg0700 == 2; ** keep if income is normal;
	
* Wealth ratios by pecentile; 
*************************************************************************************;
*gen nlqRatio = netliquidassets/(pincome/4);

gen wyRatio = networth/(pincome/4);  	* wyRatio is wealth ratio to quarterly permanent income; 
gen lqyRatio = liquidAssets/(pincome/4);

gen wyRatioSample = 1; replace wyRatioSample = 0 if hg0700~= 2|networth<0; * Sample indicator: Keep if income normal and net worth nonnegative;
gen lqyRatioSample = 1; replace lqyRatioSample = 0 if hg0700~= 2|liquidAssets<0;

foreach var of local allWealthVars {;
	gen `var'RatioHist = 1+`var'Ratio;
	gen  frac`var'RatioHistLess2=0;
	replace frac`var'RatioHistLess2=1 if `var'RatioHist<=2;

	foreach impl of local allImplicates {;
		foreach cntry of local allCountries {;
			sum `var'RatioHist [weight=wgt] if `var'RatioSample==1&`var'RatioHist>=0&`var'RatioHist<200&country=="`cntry'"&im0100==`impl', det;
			foreach ptile of local allPercentiles  {;
				sca `var'Rat_p`ptile'_`cntry'_`impl'=r(p`ptile');
			};
			sca `var'Rat_mean_`cntry'_`impl'=r(mean);

			sum frac`var'RatioHistLess2 [weight=wgt] if `var'RatioSample==1&country=="`cntry'"&im0100==`impl';
			sca frac`var'RatioLess2_`cntry'_`impl'=r(mean);
		};	
		
		** All countries (have to be done separately);
		sum `var'RatioHist [weight=wgt] if `var'RatioSample==1&`var'RatioHist>=0&`var'RatioHist<200&im0100==`impl', det;
		foreach ptile of local allPercentiles  {;
			sca `var'Rat_p`ptile'_ALL_`impl'=r(p`ptile');
		};		
		sca `var'Rat_mean_ALL_`impl'=r(mean);

		sum frac`var'RatioHistLess2 [weight=wgt] if `var'RatioSample==1;
		sca frac`var'RatioLess2_ALL_`impl'=r(mean);

	};	

	foreach cntry of local allCountriesFull {;   * Average across implicates;

		foreach ptile of local allPercentiles  {;
			sca `var'Rat_p`ptile'_`cntry'=0.2*(`var'Rat_p`ptile'_`cntry'_1+`var'Rat_p`ptile'_`cntry'_2+`var'Rat_p`ptile'_`cntry'_3+`var'Rat_p`ptile'_`cntry'_4+`var'Rat_p`ptile'_`cntry'_5);
		};
		sca `var'Rat_mean_`cntry'=0.2*(`var'Rat_mean_`cntry'_1+`var'Rat_mean_`cntry'_2+`var'Rat_mean_`cntry'_3+`var'Rat_mean_`cntry'_4+`var'Rat_mean_`cntry'_5);
		sca frac`var'RatioLess2_`cntry'=0.2*(frac`var'RatioLess2_`cntry'_1+frac`var'RatioLess2_`cntry'_2+frac`var'RatioLess2_`cntry'_3+frac`var'RatioLess2_`cntry'_4+frac`var'RatioLess2_`cntry'_5);

	};
};
	
local wealthRatios "wyRatio lqyRatio";
sca cutoffRatios=10000;	* upper bound to cut off the wealth ratios;

foreach var of local wealthRatios {;
	sum `var' [weight=wgt] if `var'Sample==1&`var'<cutoffRatios, det;
	pctile pct`var' = `var' [weight=wgt] if  `var'Sample==1&`var'<cutoffRatios, nq(20) genp(percent`var');
	list percent`var' pct`var' in 1/20;
};

* Cumulative distribution of wealth;
*************************************************************************************;
* normalize weights;
egen float wgt_norm = pc(wgt);
replace wgt_norm = wgt_norm/100;

gen tempVar = pincome*wgt_norm;
egen float meanPincome = total(tempVar);
drop tempVar;

local wealthRatios "networth netliquidassets liquidAssets";

foreach impl of local allImplicates {;
	foreach cntry of local allCountriesFull {;
		preserve;

		if "`cntry'" ~= "ALL" {;
			keep if country == "`cntry'"&im0100==`impl';
		};
		else {;
			keep if im0100==`impl';
		};
		
		sca nobs_`cntry'_`impl'=_N;

		foreach var of local wealthRatios {;
			gsort -`var'; 

			* calculate cumulative distribution from the top; 
			gen `var'_temp = `var'*wgt_norm;
			egen float dist`var' = pc(`var'_temp);
			gen cum`var' = dist`var'[1];
			replace cum`var' = dist`var'[_n] + cum`var'[_n-1] if _n>1;

			* show wealth held by top x% (note that the order is reversed from RATIOS (above));
			sum cum`var' [weight=wgt_norm], det;
			sca `var'_top1_`cntry'_`impl' = r(p1);
			sca `var'_top10_`cntry'_`impl' = r(p10);
			
			pctile pctcum`var' = cum`var' [weight=wgt_norm], nq(20) genp(percent`var');
			list percent`var' pctcum`var' in 1/20; 
			sca `var'_top20_`cntry'_`impl' = pctcum`var'[4];
			sca `var'_top40_`cntry'_`impl' = pctcum`var'[8];
			sca `var'_top60_`cntry'_`impl' = pctcum`var'[12];
			sca `var'_top80_`cntry'_`impl' = pctcum`var'[16];

			* AGGREGATE Wealth (net worth) / perm inc ratio (note divided by quarterly perm inc);  
			gen tempVar = `var'*wgt_norm;
			egen float mean`var' = total(tempVar);
			drop tempVar;

			gen agg`var'yRatio = mean`var'/(meanPincome/4);
			mean agg`var'yRatio; /* show aggregate WYRATIO */;
			*log close;
			
		};
		
		restore;
	};
};
	
local allPercentiles "1 10 20 40 60 80";
	
foreach var of local wealthRatios {;
	foreach cntry of local allCountries {;	
		foreach ptile of local allPercentiles  {;
			sca `var'_top`ptile'_`cntry'=0.2*(`var'_top`ptile'_`cntry'_1+`var'_top`ptile'_`cntry'_2+`var'_top`ptile'_`cntry'_3+`var'_top`ptile'_`cntry'_4+`var'_top`ptile'_`cntry'_5);
		};
		sca nobs_`cntry'=0.2*(nobs_`cntry'_1+nobs_`cntry'_2+nobs_`cntry'_3+nobs_`cntry'_4+nobs_`cntry'_5);
	};
};
	
* Gini coefficients;	
local allWealthVars "networth liquidAssets";
foreach var of local allWealthVars {;	
	foreach impl of local allImplicates {;	
		foreach cntry of local allCountries {;

			disp "Country: `cntry'";
			ainequal `var' [weight=wgt] if country=="`cntry'"&im0100==`impl';
			mat tempMat = r(b);
			sca gini_`var'_`cntry'_`impl' = tempMat[1,1];
		};

		disp "Country: All countries";
		ainequal `var' [weight=wgt] if im0100==`impl';
		mat tempMat = r(b);
		sca gini_`var'_ALL_`impl' = tempMat[1,1];

	};

	foreach cntry of local allCountriesFull {;   * Average across implicates;
		sca gini_`var'_`cntry'=0.2*(gini_`var'_`cntry'_1+gini_`var'_`cntry'_2+gini_`var'_`cntry'_3+gini_`var'_`cntry'_4+gini_`var'_`cntry'_5);;
	};
};

cd "`basePath'";
do writeWealthPropTable.do;
do writeWYRatTable.do;

*************************************************************************************;	
log close;
