
#delimit;
** This program draws the box and whiskers plots;

clear; clear matrix; clear mata; program drop _all; discard;
set mem 3000m; set maxvar 30000; set matsize 155; set scheme s2color; set more off;

local basePath "U:\Slacalek\Laptop_X230\20130627\Research\BSinKS\20130408\cstMPCEurope\Latest\Code\Stata\";
* Current directory needs to be changed depending on working environment;
local stataPath "`basePath'";
local logPath "`stataPath'";
local dataPath "`basePath'data";
global docPath "`dataPath'";
adopath + "`stataPath'";
global eolString "_char(92) _char(92)";		* string denoting new line in LaTeX tables \\;

capture log close;
cd "`stataPath'";
log using "`logPath'\DoAllHFCS.log", replace;
cd "`dataPath'";
*************************************************************************************;
use "`dataPath'\D_compl_normalIncome.dta";

replace hg0700 = 2 if country=="FR"|country=="FI"; * for FR and FI indicator of permanent income is not available;
keep if hg0700 == 2; ** keep if income is normal;

local allCountries "AT BE CY DE ES FI FR GR IT LU MT NL PT SI SK";
local allGraphFormats "wmf png eps pdf";	

* Data transformations;
*************************************************************************************;
* Drop data, if age < 25 or >60;
drop if age < 25|age > 60;

* Fraction of households whose wealth level is between 0.5*mean and 1.5*mean;
*************************************************************************************;
sum networth [weight=wgt], det;
gen networth_norm = networth/r(mean);
sum networth_norm [weight=wgt]; 
gen countNetworth=0; 
replace countNetworth=1 if networth_norm>0.5 & networth_norm<1.5; /* countNetworth is 1 if net worth is between is between 0.5*mean and 1.5*mean*/;
*log using "results-fraction_between_05_and_15.txt", append text;
* disp "SCF `year'";
sum countNetworth [weight=wgt]; /* mean of dum is fraction of households whose wealth level is between 0.5*mean and 1.5*mean*/;
*log close;

* Wealth ratios; 
*************************************************************************************;
gen wyRatio = networth/(pincome/4);  	* wyRatio is wealth ratio to quraterly permanent income; 
gen nlqRatio = netliquidassets/(pincome/4);
gen lqyRatio = liquidAssets/(pincome/4);

gen wyRatioHist = 1+wyRatio;
gen  fracwyRatioHistLess2=0;
replace fracwyRatioHistLess2=1 if wyRatioHist<=2;
	
gen lqyRatioHist = 1+lqyRatio;
gen  fraclqyRatioHistLess2=0;
replace fraclqyRatioHistLess2=1 if lqyRatioHist<=2;

* Wealth ratios -- graphs; 
*************************************************************************************;
gen countryNum = 0;
replace countryNum = 1 if country == "AT"; * Austria;
replace countryNum = 2 if country == "BE"; * Belgium;
replace countryNum = 3 if country == "CY"; * Cyprus;
replace countryNum = 4 if country == "DE"; * Germany;
replace countryNum = 5 if country == "ES"; * Spain;
replace countryNum = 6 if country == "FI"; * Finland;
replace countryNum = 7 if country == "FR"; * France;
replace countryNum = 8 if country == "GR"; * Greece;
replace countryNum = 9 if country == "IT"; * Italy;
replace countryNum = 10 if country == "LU"; * Luxembourg;
replace countryNum = 11 if country == "MT"; * Malta;
replace countryNum = 12 if country == "NL"; * Netherlands;
replace countryNum = 13 if country == "PT"; * Portugal;
replace countryNum = 14 if country == "SI"; * Slovenia;
replace countryNum = 15 if country == "SK"; * Slovakia;

label defin countryLabel 1 "Austria" 2 "Belgium" 3 "Cyprus" 4 "Germany" 5 "Spain" 6 "Finland" 7 "France" 8 "Greece" 9 "Italy" 10 "Luxembourg" 11 "Malta" 12 "Netherlands" 13 "Portugal" 14 "Slovenia" 15 "Slovakia";
label values countryNum countryLabel;

graph box wyRatio if wyRatio>=0 [weight=wgt], over(countryNum, label(angle(45))) noout 
 graphregion(color(white)) ytitle("Net Wealth-Income Ratio (Quarterly)"); 
 graph save "`dataPath'\boxNwhiskers_netWealth", asis replace;
foreach frmt of local allGraphFormats {;
	graph export "`dataPath'\boxNwhiskers_netWealth.`frmt'", replace;
};	
	
graph box lqyRatio if lqyRatio>=0 [weight=wgt], over(countryNum, label(angle(45))) noout 
 graphregion(color(white)) ytitle("Liquid Assets-Income Ratio (Quarterly)"); 
graph save "`dataPath'\boxNwhiskers_liquidAssets", asis replace;
foreach frmt of local allGraphFormats {;
	graph export "`dataPath'\boxNwhiskers_liquidAssets.`frmt'", replace;
};	

graph box wyRatio lqyRatio if wyRatio>=0&lqyRatio>=0 [weight=wgt], over(countryNum, label(angle(45))) noout 
 graphregion(color(white))  box(1, color(maroon*.7)) box(2, color(red*5))
 legend(lab(1 "Net Wealth") lab(2 "Liquid Assets")) ytitle("Wealth-Income Ratios (Quarterly)"); 
graph save "`dataPath'\boxNwhiskers_netWealth_liquidAssets", asis replace;
foreach frmt of local allGraphFormats {;
	graph export "`dataPath'\boxNwhiskers_netWealth_liquidAssets.`frmt'", replace;
};

graph box wyRatio lqyRatio if wyRatio>=0&lqyRatio>=0 [weight=wgt], horizontal over(countryNum) noout 
 graphregion(color(white))  box(1, color(maroon*.7)) box(2, color(red*5))
 legend(lab(1 "Net Wealth") lab(2 "Liquid Assets")) ytitle("Wealth-Income Ratios (Quarterly)"); 
graph save "`dataPath'\boxNwhiskers_netWealth_liquidAssets_horizon", asis replace;
foreach frmt of local allGraphFormats {;
	graph export "`dataPath'\boxNwhiskers_netWealth_liquidAssets_horizon.`frmt'", replace;
};

*************************************************************************************;
log close;
