(* DoAll.nb *)
(* This file runs all the files *)
(* Net worth data are used unless otherwise noted *)

ClearAll["Global`*"];


(* Models without aggregate shock *)
Print["==========================================================================="];
Print["Models without aggregate shock"];

SetDirectory[NotebookDirectory[]];
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggShocksDir]; 
<<SolveAndSimPF.m;              (* Run PF rep agent and partial eq PF models *)

ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
NoAggShockDir    = Directory[] <> "/NoAggShock";    (* Directory where files without aggregate shock are saved *)
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[NoAggShockDir]; 
<<FindBeta.m;                    (* Run model with point (unique) time pref factor and estimate \[Beta] *)
<<FindBetaDistSeven.m;           (* Run \[Beta]-Dist model with time pref factors approximated by seven points and estimate params *)

<<PlotCFuncandDeltaMZero.m;      (* Plots c func of the most patient and Delta m = 0 line *)
CFuncandDeltaMZeroPlot            (* Plot figure *)

ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
NoAggShockDir    = Directory[] <> "/NoAggShock";    (* Directory where files without aggregate shock are saved *)
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[NoAggShockDir]; 
<<SolveAndSimKSNoAggShock.m;     (* Run KS-JEDC model without aggregate shock *)
<<PlotDistributionsWealthLevSCF_CastanedaAndDistSevenNoAggShock.m; 
Print[" Figure of wealth distribution (without aggregate shock) "]; 
CumWLevSCFCastanedaAndDistSevenNoAggShockPlot (* Plot *)

<<FindBetaDistSevenReiterOnly.m; (* Run \[Beta]-Dist model without simulations and estimate params *)
(*
<<PlotWYRatioSCF_AndDistSeven.m; (* Plot US data and data generated using Reiter's (transition matrix) method *)
Print[" Plot wealth perm income ratios"];
WYRatioPlot (* Plot *)
*)

(*
(* FINANCIAL assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
NoAggShockDir    = Directory[] <> "/NoAggShock";    (* Directory where files without aggregate shock are saved *)
SetDirectory[NoAggShockDir]; 
<<FindBetaDistSevenFin.m;           (* Run \[Beta]-Dist model with time pref factors approximated by seven points and estimate params. Matching FINANCIAL assets. *)

*)

(* Liquid FINANCIAL assets plus retirement assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
NoAggShockDir    = Directory[] <> "/NoAggShock";    (* Directory where files without aggregate shock are saved *)
SetDirectory[NoAggShockDir]; 
<<FindBetaDistSevenLiqFinPlsRet.m;           (* Run \[Beta]-Dist model with time pref factors approximated by seven points and estimate params. Matching liquid FINANCIAL assets plus retirement assets. *)

<<PlotCFuncandDeltaMZeroLiqFinPlsRet.m; 
PlotCFuncandDeltaMZeroLiqFinPlsRetPlot        (* plot *)

(*
(* Liquid FINANCIAL assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
NoAggShockDir    = Directory[] <> "/NoAggShock";    (* Directory where files without aggregate shock are saved *)
SetDirectory[NoAggShockDir]; 
<<FindBetaDistSevenLiqFin.m;           (* Run \[Beta]-Dist model with time pref factors approximated by seven points and estimate params. Matching liquid FINANCIAL assets. *)
*)

ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
NoAggShockDir    = Directory[] <> "/NoAggShock";    (* Directory where files without aggregate shock are saved *)
SetDirectory[NoAggShockDir]; 
<<EstimateMoffittGottschalkProcess.m; 



(* Models with aggregate shocks *)
Print["==========================================================================="];
Print["Models with aggregate shocks"];
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggShocksDir]; 
<<SolveAndSimKS.m;             (* Run rep agent and KS-JEDC model. Rep agent model is run as a first step. *)
Print[" Figure fCrossSectionVar "]
<<PlotfCrossSectionVar.m;      (* Plot fCrossSectionVar *)  
fCrossSectionVar

<<SolveAndSimKSHetero.m;       (* Run rep agent and KS-Hetero model. Rep agent model is run as a first step. *)
<<SolveAndSimPoint.m;          (* Run \[Beta]-Point model *)
Print[" Figure of consumption function"];
CFuncPointPlotALL 

CFuncPointAndHistNetWorthPlot

<<FindBetaDistSeven.m;         (* Estimates parameter values for \[Beta]-Dist model (with aggregate shocks) *)
Print[" Matrix for \[EmptyDownTriangle]/3.5, \[Beta]middle, KLevMean, SumOfDevSq:"]; 
StatsMat // MatrixForm
<<SolveAndSimDistSeven.m;      (* Run \[Beta]-Dist model with time pref factors approximated by seven points *)
Print[" Figure of consumption function"];
CFuncDistSevenPointAndHistNetWorthPlot

CFuncDistSevenAndHistNetWorthLiqFinPlsRetPlot

CFuncDistSevenPointAndHistNetWorthPlot

<<SolveAndSimKS_experiments.m; (* Run an experiment using KS-JEDC model *)
Print[" Figure of interst rate"]; 
RPlot      
Print[" Figure of consumption growth"]; 
dlAggCtPlot 
<<PlotDistributionsWealthLevSCF_CastanedaAndDistSeven.m; 
Print[" Figure of wealth distribution (with aggregate shocks) "]; 
CumWLevSCFCastanedaAndDistSevenPlot   (* Plot *)

(*
(* FINANCIAL assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggShocksDir]; 
<<FindBetaDistSevenFin.m;         (* Estimates parameter values for \[Beta]-Dist model (with aggregate shocks). Matching FINANCIAL assets. *)
Print[" Matrix for \[EmptyDownTriangle]/3.5, \[Beta]middle, KLevMean, SumOfDevSq:"]; 
StatsMat // MatrixForm
<<SolveAndSimDistSevenFin.m;      (* Run \[Beta]-Dist model with time pref factors approximated by seven points. Using parameter values estimated by matching FINANCIAL assets. *)
*)

(* Liquid FINANCIAL assets plus retirement assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggShocksDir]; 
<<FindBetaDistSevenLiqFinPlsRet.m;         (* Estimates parameter values for \[Beta]-Dist model (with aggregate shocks). Matching liquid FINANCIAL assets. *)
Print[" Matrix for \[EmptyDownTriangle]/3.5, \[Beta]middle, KLevMean, SumOfDevSq:"]; 
StatsMat // MatrixForm
<<SolveAndSimDistSevenLiqFinPlsRet.m;      (* Run \[Beta]-Dist model with time pref factors approximated by seven points. Using parameter values estimated by matching liquid FINANCIAL assets plus retirement assets. *)


(*
(* Liquid FINANCIAL assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggShocksDir]; 
<<FindBetaDistSevenLiqFin.m;         (* Estimates parameter values for \[Beta]-Dist model (with aggregate shocks). Matching liquid FINANCIAL assets. *)
Print[" Matrix for \[EmptyDownTriangle]/3.5, \[Beta]middle, KLevMean, SumOfDevSq:"]; 
StatsMat // MatrixForm
<<SolveAndSimDistSevenLiqFin.m;      (* Run \[Beta]-Dist model with time pref factors approximated by seven points. Using parameter values estimated by matching liquid FINANCIAL assets. *)
*)

<<PlotDistributionsMPCsDistSevenKSAggShocks.m;
Print[" Figure of MPC comparison"];       (* Show distribution of MPCs when matching net worth and liq financial assets *)
DistributionsMPCsDistSevenAndKSKSAggShocksLiqFinPlsRetPlot (* Plot *)

(* Solve rep agent model with aggregate shocks *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggShocksDir]; 
<<SolveAndSimRepAgent.m; 

(* Models with aggregate PERMANENT shock *)
Print["==========================================================================="];
Print["Models with aggregate permanent shocks"];
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggPermShocksDir = Directory[] <> "/WithAggPermShocks"; (* Directory where files with PERMANENT aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggPermShocksDir]; 

<<SolveAndSimPoint.m;      (* Run \[Beta]-Point model *)
<<SolveAndSimDistSeven.m;  (* Run \[Beta]-Dist model with time pref factors approximated by seven points *)

Print[" Figure of consumption function"];
CFuncDistSevenPointPermAndHistNetWorthPlot

CFuncDistSevenPermAndHistNetWorthLiqFinPlot

CFuncDistSevenPointPermAndHistNetWorthPlot

<<PlotDistributionsWealthLevSCF_CastanedaAndDistSevenPerm.m; 
Print[" Figure of wealth distribution"]; 
CumWLevSCFCastanedaAndDistSevenPermPlot (* Plot *)

(*
(* FINANCIAL assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggPermShocksDir = Directory[] <> "/WithAggPermShocks"; (* Directory where files with PERMANENT aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggPermShocksDir]; 
<<SolveAndSimDistSevenFin.m;  (* Run \[Beta]-Dist model with time pref factors approximated by seven points. Using parameter values estimated by matching FINANCIAL assets. *)
*)

(* Liquid FINANCIAL assets plus retirement assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggPermShocksDir = Directory[] <> "/WithAggPermShocks"; (* Directory where files with PERMANENT aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggPermShocksDir]; 
<<SolveAndSimDistSevenLiqFinPlsRet.m;  (* Run \[Beta]-Dist model with time pref factors approximated by seven points. Using parameter values estimated by matching liquid FINANCIAL assets plus retirement assets. *)

(*
(* Liquid FINANCIAL assets *)
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggPermShocksDir = Directory[] <> "/WithAggPermShocks"; (* Directory where files with PERMANENT aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggPermShocksDir]; 
<<SolveAndSimDistSevenLiqFin.m;  (* Run \[Beta]-Dist model with time pref factors approximated by seven points. Using parameter values estimated by matching liquid FINANCIAL assets. *)
*)

<<PlotDistributionsMPCsDistSevenPerm.m;
Print[" Figure of MPC comparison"]; (* Show distribution of MPCs when matching net worth and liq financial assets *)
DistributionsMPCsDistSevenAndKSPermShocksLiqFinPlsRetPlot (* Plot *)

(*
(* Run \[Beta]-Dist model with an ALTERNATIVE parameter for income shocks. Specifically: 
 \[Sigma]Psi     = (0.01/4)^0.5; (* ALTERNATIVE Standard deviation of lognormal distribution of permanent shocks *)
*)

Print["==========================================================================="];
Print["==========================================================================="];
Print["Robustness checks: Run \[Beta]-Dist model with an ALTERNATIVE parameter for income shocks"]
Print["Annual variance of permanent income shocks is 0.01"]

(* Model without aggregate shock *)
Print["==========================================================================="];
Print["Model without aggregate shock"];
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
NoAggShockDir = Directory[] <> "/NoAggShock"; (* Directory where files without aggregate shock are saved *)
SetDirectory[NoAggShockDir]; 
<<FindBetaDistSevenAltParams.m;              (* Run \[Beta]-Dist model with time pref factors approximated by seven points and estimate params *) 

(* Model with aggregate shocks *)
Print["==========================================================================="];
Print["Model with aggregate shocks"];
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggShocksDir = Directory[] <> "/WithAggShocks"; (* Directory where files with aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggShocksDir]; 
<<SolveAndSimDistSevenAltParams.m;    (* Run \[Beta]-Dist model with time pref factors approximated by seven points *)

(* Model with aggregate PERMANENT shock *)
Print["==========================================================================="];
Print["Model with aggregate permanent shocks"];
ClearAll["Global`*"];
SetDirectory[NotebookDirectory[]];
WithAggPermShocksDir = Directory[] <> "/WithAggPermShocks"; (* Directory where files with PERMANENT aggregate shocks + file for PF rep agent are saved *)
SetDirectory[WithAggPermShocksDir]; 
<<SolveAndSimDistSevenAltParams.m;  (* Run \[Beta]-Dist model with time pref factors approximated by seven points *)

*)

(* Generate tables *)
Print["==========================================================================="];
Print["Generate tables "];
ClearAll["Global`*"];

SetDirectory[NotebookDirectory[]];
ResultsDir = Directory[] <> "/Results"; 


(* produce tables for "cstKS" paper and save the tables in folder "cstKS\Latest\Tables" *)
SetDirectory[ResultsDir];
<<GenWDistcstKS.m;

SetDirectory[ResultsDir];
<<GenAggStatscstKS.m;

SetDirectory[ResultsDir];
<<GenMGComparison.m;


(* produce tables for "cstMPC" paper and save the tables in folder "cstMPC\Latest\Tables" *)
SetDirectory[ResultsDir]; 
<<GenMPC.m;

SetDirectory[ResultsDir]; 
<<GenMPC_scenarios.m;


(* produce other tables including for slides *)
SetDirectory[ResultsDir];
<<GenWDistslides.m;

SetDirectory[ResultsDir];
<<GenMPCslides.m;

SetDirectory[ResultsDir];
(*
<<GenMPCslidesLiqFin.m;
*)
<<GenMPCslidesLiqFinPlsRet.m

SetDirectory[ResultsDir]; 
<<GenAggStatsPlausibleAggShocks.m;

(* copy figures *)
SetDirectory[NotebookDirectory[]];
<<CopyFigures.m;