(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[   1717995,      45131]
NotebookOptionsPosition[   1504073,      39979]
NotebookOutlinePosition[   1504416,      39994]
CellTagsIndexPosition[   1504373,      39991]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
(* DoAllxc.nb *)

(* This file estimates\\beta and diff across European economies (in cstMPCxc \
project) *)

(* Data on i) net worth and ii) liq fin pls ret assets are used  *)





ClearAll[\"Global`*\"];





Print[\"cstMPC across European economies, using net worth data \"]



(* Models without aggregate shock *)

Print[\"======================================================================\
=====\"];

Print[\"Models without aggregate shock\"];



ClearAll[\"Global`*\"];

SetDirectory[NotebookDirectory[]];

NoAggShockDir    = Directory[] <> \"/NoAggShock\";    (* Directory where \
files without aggregate shock are saved *)

WithAggShocksDir = Directory[] <> \"/WithAggShocks\"; (* Directory where \
files with aggregate shocks + file for PF rep agent are saved *)

SetDirectory[NoAggShockDir]; 





TimeStartEurope = SessionTime[];  





(* ********************** *)

(* All countries *)

Country = AllBaseline; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockAllBaseline.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenNoAggShockAllBaseline.txt\", AggMPCMeanAnnual, \
\"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffAllBaseline.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleAllBaseline.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffAllBaseline.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleAllBaseline.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffAllBaseline.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleAllBaseline.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

 (* low psi case *)

Country = AllLowPsi; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockAllLowPsi.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenNoAggShockAllLowPsi.txt\", AggMPCMeanAnnual, \
\"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffAllLowPsi.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleAllLowPsi.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffAllLowPsi.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleAllLowPsi.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffAllLowPsi.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleAllLowPsi.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

 (* high theta case *)

Country = AllHighTheta; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockAllHighTheta.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenNoAggShockAllHighTheta.txt\", AggMPCMeanAnnual, \
\"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffAllHighTheta.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleAllHighTheta.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffAllHighTheta.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \
\"/WithAggPermShocks/BetamiddleAllHighTheta.txt\", \[Beta]middleSolution, \
\"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffAllHighTheta.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleAllHighTheta.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

 (* very high theta case *)

Country = AllVeryHighTheta; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockAllVeryHighTheta.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenNoAggShockAllVeryHighTheta.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffAllVeryHighTheta.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \
\"/WithAggShocks/BetamiddleAllVeryHighTheta.txt\", \[Beta]middleSolution, \
\"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffAllVeryHighTheta.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \
\"/WithAggPermShocks/BetamiddleAllVeryHighTheta.txt\", \[Beta]middleSolution, \
\"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffAllVeryHighTheta.txt\",      \
 diff, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/BetamiddleAllVeryHighTheta.txt\", \[Beta]middleSolution, \
\"Table\"];







(* ********************** *)

(* Austria *)

Country = AT; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockAT.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockAT.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffAT.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleAT.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffAT.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleAT.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffAT.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleAT.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Belgium *)

Country = BE; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockBE.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockBE.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffBE.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleBE.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffBE.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleBE.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffBE.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleBE.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Cyprus *)

Country = CY; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockCY.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockCY.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffCY.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleCY.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffCY.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleCY.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffCY.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleCY.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Germany *)

Country = DE; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockDE.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockDE.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffDE.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleDE.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffDE.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleDE.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffDE.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleDE.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Spain *)

Country = ES; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockES.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockES.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffES.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleES.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffES.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleES.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffES.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleES.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Finland *)

Country = FI; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockFI.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockFI.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffFI.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleFI.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffFI.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleFI.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffFI.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleFI.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* France *)

Country = FR; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockFR.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockFR.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffFR.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleFR.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffFR.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleFR.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffFR.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleFR.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Greece *)

Country = GR; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockGR.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockGR.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffGR.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleGR.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffGR.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleGR.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffGR.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleGR.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Italy *)

Country = IT; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockIT.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockIT.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffIT.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleIT.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffIT.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleIT.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffIT.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleIT.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Luxembourg *)

Country = LU; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockLU.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockLU.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLU.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLU.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLU.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLU.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLU.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLU.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Malta *)

Country = MT; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockMT.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockMT.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffMT.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleMT.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffMT.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleMT.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffMT.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleMT.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Netherland *)

Country = NL; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockNL.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockNL.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffNL.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleNL.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffNL.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleNL.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffNL.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleNL.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Portugal *)

Country = PT; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockPT.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockPT.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffPT.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddlePT.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffPT.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddlePT.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffPT.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddlePT.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Portugal Slovenia *)

Country = SI; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockSI.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockSI.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffSI.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleSI.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffSI.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleSI.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffSI.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleSI.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Portugal Slovakia *)

Country = SK; 

<<FindBetaDistSevenEuropeCommon.m; (* Run \[Beta]-Dist model with time pref \
factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenNoAggShockSK.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/MPCDistSevenNoAggShockSK.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffSK.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleSK.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffSK.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleSK.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffSK.txt\",       diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleSK.txt\", \
\[Beta]middleSolution, \"Table\"];



Print[\" Time spent to estimate params across Euroepan economies:  \", \
(SessionTime[]-TimeStartEurope)/60];







(* Models with aggregate PERMANENT shock *)

Print[\"======================================================================\
=====\"];

Print[\"Models with aggregate permanent shocks\"];

ClearAll[\"Global`*\"];

SetDirectory[NotebookDirectory[]];

WithAggPermShocksDir = Directory[] <> \"/WithAggPermShocks\"; (* Directory \
where files with PERMANENT aggregate shocks + file for PF rep agent are saved \
*)

SetDirectory[WithAggPermShocksDir]; 





TimeStartEurope = SessionTime[]; 



Print[\"=============================================================\"];

Print[\"Run \[Beta]-Dist model with time pref factors approximated by seven \
points\"];



Print[\"==============================================\"]; 

(* All countries *)

Country = AllBaseline; 

Print[\"Country: All countries\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockAllBaseline.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksAllBaseline.txt\",          \
   MPCListWithAggShock, \"Table\"];





Print[\"==============================================\"]; 

(* All countries *)

Country = AllLowPsi; 

Print[\"Country: All (low psi)\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockAllLowPsi.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksAllLowPsi.txt\",            \
 MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* All countries *)

Country = AllHighTheta; 

Print[\"Country: All (high theta)\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockAllHighTheta.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksAllHighTheta.txt\",         \
    MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* All countries *)

Country = AllVeryHighTheta; 

Print[\"Country: All (very high theta)\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockAllVeryHighTheta.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksAllVeryHighTheta.txt\",     \
        MPCListWithAggShock, \"Table\"];







Print[\"==============================================\"]; 

(* Austria *)

Country = AT; 

Print[\"Country: Austria\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockAT.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksAT.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Belgium *)

Country = BE; 

Print[\"Country: Belgium\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockBE.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksBE.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Cyprus *)

Country = CY; 

Print[\"Country: Cyprus\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockCY.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksCY.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Germany *)

Country = DE; 

Print[\"Country: Germany\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockDE.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksDE.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Spain *)

Country = ES; 

Print[\"Country: Spain\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockES.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksES.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Finland *)

Country = FI; 

Print[\"Country: Finland\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockFI.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksFI.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* France *)

Country = FR; 

Print[\"Country: France\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockFR.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksFR.txt\",             \
MPCListWithAggShock, \"Table\"];





Print[\"==============================================\"]; 

(* Greece *)

Country = GR; 

Print[\"Country: Greece\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockGR.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksGR.txt\",             \
MPCListWithAggShock, \"Table\"];





Print[\"==============================================\"]; 

(* Italy *)

Country = IT; 

Print[\"Country: Italy\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockIT.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksIT.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Luxembourg *)

Country = LU; 

Print[\"Country: Luxembourg\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockLU.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksLU.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Malta *)

Country = MT; 

Print[\"Country: Malta\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockMT.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksMT.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Netherland *)

Country = NL; 

Print[\"Country: Netherland\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockNL.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksNL.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Portugal *)

Country = PT; 

Print[\"Country: Portugal\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockPT.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksPT.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Slovenia *)

Country = SI; 

Print[\"Country: Slovenia\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockSI.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksSI.txt\",             \
MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Slovakia *)

Country = SK; 

Print[\"Country: Slovakia\"]; 



<<SolveAndSimDistSevenEuropeCommon.m;  (* Run \[Beta]-Dist model with time \
pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenWithAggPermShockSK.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenWithAggPermShocksSK.txt\",             \
MPCListWithAggShock, \"Table\"];





Print[\" Time spent to run \[Beta]-Dist model across Euroepan economies \
(minutes):  \", (SessionTime[]-TimeStartEurope)/60];















Print[\"======================================================================\
=====\"];

Print[\"======================================================================\
=====\"];





Print[\"cstMPC across European economies, using data on liq fin and \
retirement assets \"]



(* Models without aggregate shock *)

Print[\"======================================================================\
=====\"];

Print[\"Models without aggregate shock\"];



ClearAll[\"Global`*\"];

SetDirectory[NotebookDirectory[]];

NoAggShockDir    = Directory[] <> \"/NoAggShock\";    (* Directory where \
files without aggregate shock are saved *)

WithAggShocksDir = Directory[] <> \"/WithAggShocks\"; (* Directory where \
files with aggregate shocks + file for PF rep agent are saved *)

SetDirectory[NoAggShockDir]; 



TimeStartEuropeLiqFinPlsRet = SessionTime[];  



(* ********************** *)

(* All countries *)

Country = AllBaseline; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockAllBaseline.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockAllBaseline.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \
\"/WithAggShocks/diffLiqFinPlsRetAllBaseline.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \
\"/WithAggShocks/BetamiddleLiqFinPlsRetAllBaseline.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \
\"/WithAggPermShocks/diffLiqFinPlsRetAllBaseline.txt\", diff, \"Table\"];

Export[ParentDirectory[] <> \
\"/WithAggPermShocks/BetamiddleLiqFinPlsRetAllBaseline.txt\", \
\[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \
\"/ResultsEurope/diffLiqFinPlsRetAllBaseline.txt\",       diff, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/BetamiddleLiqFinPlsRetAllBaseline.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Austria *)

Country = AT; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockAT.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockAT.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetAT.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetAT.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetAT.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetAT.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetAT.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetAT.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Belgium *)

Country = BE; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockBE.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockBE.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetBE.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetBE.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetBE.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetBE.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetBE.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetBE.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Cyprus *)

Country = CY; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockCY.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockCY.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetCY.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetCY.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetCY.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetCY.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetCY.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetCY.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Germany *)

Country = DE; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockDE.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockDE.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetDE.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetDE.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetDE.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetDE.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetDE.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetDE.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Spain *)

Country = ES; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockES.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockES.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetES.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetES.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetES.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetES.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetES.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetES.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Finland *)

Country = FI; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockFI.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockFI.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetFI.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetFI.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetFI.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetFI.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetFI.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetFI.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* France *)

Country = FR; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockFR.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockFR.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetFR.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetFR.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetFR.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetFR.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetFR.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetFR.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Greece *)

Country = GR; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockGR.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockGR.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetGR.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetGR.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetGR.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetGR.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetGR.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetGR.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Italy *)

Country = IT; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockIT.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockIT.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetIT.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetIT.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetIT.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetIT.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetIT.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetIT.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Luxembourg *)

Country = LU; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockLU.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockLU.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetLU.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetLU.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetLU.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetLU.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetLU.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetLU.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Malta *)

Country = MT; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockMT.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockMT.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetMT.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetMT.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetMT.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetMT.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetMT.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetMT.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Netherland *)

Country = NL; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockNL.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockNL.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetNL.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetNL.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetNL.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetNL.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetNL.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetNL.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Portugal *)

Country = PT; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockPT.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockPT.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetPT.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetPT.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetPT.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetPT.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetPT.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetPT.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Portugal Slovenia *)

Country = SI; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockSI.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockSI.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetSI.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetSI.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetSI.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetSI.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetSI.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetSI.txt\", \
\[Beta]middleSolution, \"Table\"];



(* ********************** *)

(* Portugal Slovakia *)

Country = SK; 

<<FindBetaDistSevenLiqFinPlsRetEuropeCommon.m; (* Run \[Beta]-Dist model with \
time pref factors approximated by seven points and estimate params *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetNoAggShockSK.txt\", \
kLevListDistSevenNoAggShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCDistSevenLiqFinPlsRetNoAggShockSK.txt\", \
AggMPCMeanAnnual, \"Table\"];



(*

Export[ParentDirectory[] <> \"/WithAggShocks/diffLiqFinPlsRetSK.txt\", diff, \
\"Table\"];

Export[ParentDirectory[] <> \"/WithAggShocks/BetamiddleLiqFinPlsRetSK.txt\", \
\[Beta]middleSolution, \"Table\"];

*)

Export[ParentDirectory[] <> \"/WithAggPermShocks/diffLiqFinPlsRetSK.txt\", \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/WithAggPermShocks/BetamiddleLiqFinPlsRetSK.txt\
\", \[Beta]middleSolution, \"Table\"];



Export[ParentDirectory[] <> \"/ResultsEurope/diffLiqFinPlsRetSK.txt\",       \
diff, \"Table\"];

Export[ParentDirectory[] <> \"/ResultsEurope/BetamiddleLiqFinPlsRetSK.txt\", \
\[Beta]middleSolution, \"Table\"];





Print[\" Time spent to run \[Beta]-Dist model (using liq fin and retirement \
assets) across Euroepan economies (minutes):  \", \
(SessionTime[]-TimeStartEuropeLiqFinPlsRet)/60];









(* Models with aggregate PERMANENT shock *)

Print[\"======================================================================\
=====\"];

Print[\"Models with aggregate permanent shocks\"];

ClearAll[\"Global`*\"];

SetDirectory[NotebookDirectory[]];

WithAggPermShocksDir = Directory[] <> \"/WithAggPermShocks\"; (* Directory \
where files with PERMANENT aggregate shocks + file for PF rep agent are saved \
*)

SetDirectory[WithAggPermShocksDir]; 



TimeStartEurope = SessionTime[]; 



Print[\"=============================================================\"];

Print[\"Run \[Beta]-Dist model with time pref factors approximated by seven \
points\"];



Print[\"==============================================\"]; 

(* All countries *)

Country = AllBaseline; 

Print[\"Country: All countries\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockAllBaseline.txt\
\", kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksAllBaseline.txt\
\",             MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Austria *)

Country = AT; 

Print[\"Country: Austria\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockAT.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksAT.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Belgium *)

Country = BE; 

Print[\"Country: Belgium\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockBE.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksBE.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Cyprus *)

Country = CY; 

Print[\"Country: Cyprus\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockCY.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksCY.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Germany *)

Country = DE; 

Print[\"Country: Germany\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockDE.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksDE.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Spain *)

Country = ES; 

Print[\"Country: Spain\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockES.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksES.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Finland *)

Country = FI; 

Print[\"Country: Finland\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockFI.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksFI.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* France *)

Country = FR; 

Print[\"Country: France\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockFR.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksFR.txt\",       \
      MPCListWithAggShock, \"Table\"];





Print[\"==============================================\"]; 

(* Greece *)

Country = GR; 

Print[\"Country: Greece\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockGR.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksGR.txt\",       \
      MPCListWithAggShock, \"Table\"];





Print[\"==============================================\"]; 

(* Italy *)

Country = IT; 

Print[\"Country: Italy\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockIT.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksIT.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Luxembourg *)

Country = LU; 

Print[\"Country: Luxembourg\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockLU.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksLU.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Malta *)

Country = MT; 

Print[\"Country: Malta\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockMT.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksMT.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Netherland *)

Country = NL; 

Print[\"Country: Netherland\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockNL.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksNL.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Portugal *)

Country = PT; 

Print[\"Country: Portugal\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockPT.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksPT.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Slovenia *)

Country = SI; 

Print[\"Country: Slovenia\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockSI.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksSI.txt\",       \
      MPCListWithAggShock, \"Table\"];



Print[\"==============================================\"]; 

(* Slovakia *)

Country = SK; 

Print[\"Country: Slovakia\"]; 



<<SolveAndSimDistSevenLiqFinPlsRetEuropeCommon.m;  (* Run \[Beta]-Dist model \
with time pref factors approximated by seven points *)



(* export data *)

Export[ParentDirectory[] <> \
\"/ResultsEurope/kLevListDistSevenLiqFinPlsRetWithAggPermShockSK.txt\", \
kLevListDistSevenWithAggPermShock, \"Table\"];

Export[ParentDirectory[] <> \
\"/ResultsEurope/MPCListDistSevenLiqFinPlsRetWithAggPermShocksSK.txt\",       \
      MPCListWithAggShock, \"Table\"];





Print[\" Time spent to run \[Beta]-Dist model across Euroepan economies \
(minutes):  \", (SessionTime[]-TimeStartEurope)/60];









(* Generate tables *)

Print[\"======================================================================\
=====\"];

Print[\"Generate tables \"];

ClearAll[\"Global`*\"];



SetDirectory[NotebookDirectory[]];

ResultsEuropeDir = Directory[] <> \"/ResultsEurope\"; 



SetDirectory[ResultsEuropeDir];

<<GenWealthDistSim.m;     (* gen wealth dist table (simulated) *)



SetDirectory[ResultsEuropeDir];

<<GenMPCallxc.m;             (* gen MPC table for the net worth case *)



SetDirectory[ResultsEuropeDir];

<<GenMPCallLiqFinPlsRetxc.m; (* gen MPC table for the liq fin + ret assets \
case *)



SetDirectory[ResultsEuropeDir];

<<GenMPCAltParamsxc.m; (* gen MPC table across alt params *)



SetDirectory[ResultsEuropeDir];

<<GenMPCAltParamsxc_slides.m; (* gen MPC table across alt params *)

\
\>", "Input",
 EmphasizeSyntaxErrors->True],

Cell[CellGroupData[{

Cell[BoxData["\<\"cstMPC across European economies, using net worth data \
\"\>"], "Print",
 CellChangeTimes->{3.5933759096455264`*^9}],

Cell[BoxData["\<\"============================================================\
===============\"\>"], "Print",
 CellChangeTimes->{3.5933759096455264`*^9}],

Cell[BoxData["\<\"Models without aggregate shock\"\>"], "Print",
 CellChangeTimes->{3.593375909661126*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933759096767263`*^9}],

Cell[BoxData["\<\"Country: All (baseline)\"\>"], "Print",
 CellChangeTimes->{3.5933759096923265`*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5933759120167303`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5933759120167303`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5933759141539345`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933759141539345`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933759883160653`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933759883316655`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933759991112843`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593376016006114*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593376016006114*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.59337601914172*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59337601914172*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933760626189957`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376062634596*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593376073336215*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9894949999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9894949999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376191802823*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933762451237164`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.901824120743676`", ",", "50.685314110731504`", ",", 
   "67.78494264219957`", ",", "73.57874281352336`", ",", "85.35399320099718`",
    ",", "90.32555001031578`", ",", "93.89394653509068`", ",", 
   "98.32141796180723`", ",", "67.78494264219957`", ",", 
   "0.6426492998429182`", ",", "0.6404047021196912`", ",", 
   "0.3037787476322962`", ",", "Indeterminate", ",", "1.398489827825512`", 
   ",", "0.616696109278796`", ",", "0.11603702323053378`", ",", 
   "35.24987175425219`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933762451237164`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933762451237164`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933762451237164`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593376288600993*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593376288741393*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376288741393*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593376299489812*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376394025978*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593376446800871*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.22581046242899`", ",", "53.09345917081628`", ",", "70.04924225732518`", 
   ",", "75.68022290636581`", ",", "86.82570570760187`", ",", 
   "91.38215962685126`", ",", "94.58537520896321`", ",", "98.50836412178914`",
    ",", "70.04924225732518`", ",", "0.6651338228328249`", ",", 
   "0.6630325511782942`", ",", "0.3037787476322962`", ",", "Indeterminate", 
   ",", "1.4431755311418473`", ",", "0.616696109278796`", ",", 
   "0.12312842629119325`", ",", "22.333873649397386`"}], "}"}]], "Print",
 CellChangeTimes->{3.593376446800871*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376446800871*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593376446800871*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376490652548*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933765014321666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9891949999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9891949999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933765938467293`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593376647354823*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.43893910627357`", ",", "55.27826771892077`", ",", "72.06178255542683`", 
   ",", "77.52408460833914`", ",", "88.07176054323833`", ",", 
   "92.25139868160309`", ",", "95.14078771321125`", ",", "98.65455202264701`",
    ",", "72.06178255542683`", ",", "0.6846463248900094`", ",", 
   "0.6826690266144343`", ",", "0.3037787476322962`", ",", "Indeterminate", 
   ",", "1.4841220712668204`", ",", "0.616696109278796`", ",", 
   "0.13257090902365054`", ",", "24.473445058721552`"}], "}"}]], "Print",
 CellChangeTimes->{3.593376647354823*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933766629392505`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593376905519676*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.671714776792278`", ",", "53.42612508257222`", ",", "70.27904298095036`",
    ",", "75.8680054290195`", ",", "86.90936246363877`", ",", 
   "91.43056886558193`", ",", "94.61960637307783`", ",", "98.51453572205796`",
    ",", "70.27904298095036`", ",", "0.66737666451935`", ",", 
   "0.6652769785577272`", ",", "0.3068820852136237`", ",", "Indeterminate", 
   ",", "1.4395628084739724`", ",", "0.6241798468045496`", ",", 
   "0.12289718205595768`", ",", "22.449938474764984`"}], "}"}]], "Print",
 CellChangeTimes->{3.593376905519676*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0008`", ",", "0.989375`", ",", "0.986575`", ",", "0.992175`", ",", 
     "39.751255517301416`", ",", "10.338812590697389`", ",", 
     "1.2486436738004034`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0008, 
   0.989375, 0.986575, 0.992175, 39.751255517301416`, 10.338812590697389`, 
   1.2486436738004034`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376905519676*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "16.59738249833333333333333333333333333333`10.148705850944678"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   16.59738249833333333333333333333333333333`10.148705850944678],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376905784877*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933769067364783`*^9}],

Cell[BoxData["\<\"Country: All (low psi)\"\>"], "Print",
 CellChangeTimes->{3.5933769067832785`*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.593376907188879*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593376907188879*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5933769095132837`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933769095132837`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933769840190144`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933769840190144`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593376994907833*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5933770124266644`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5933770124422646`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933770156402698`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933770156402698`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593377059039546*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593377059055146*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593377069959565*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9901999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9901999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933772119510145`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933772659115095`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.141076009166273`", ",", "53.31718499785823`", ",", "70.33516186416607`",
    ",", "75.92565403157136`", ",", "86.89504862686232`", ",", 
   "91.36617484366461`", ",", "94.5343803964018`", ",", "98.46932141914326`", 
   ",", "70.33516186416607`", ",", "0.6655701146418146`", ",", 
   "0.6634821474416897`", ",", "0.21938825472405096`", ",", "Indeterminate", 
   ",", "1.438627405424835`", ",", "0.43184666073424177`", ",", 
   "0.11915448117970073`", ",", "23.471779643645746`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933772659271097`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933772659271097`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933772659271097`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933773092015853`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933773092171855`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933773092327857`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593377320059204*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9899299999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9899299999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933774625654545`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59337751679115*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.20380006047503`", ",", "55.548311982213825`", ",", "72.43303575114783`",
    ",", "77.84465739972377`", ",", "88.13438418337086`", ",", 
   "92.17632579027207`", ",", "95.01664429682883`", ",", "98.57416005172227`",
    ",", "72.43303575114783`", ",", "0.6848709734105412`", ",", 
   "0.6829046330226793`", ",", "0.21938825472405096`", ",", "Indeterminate", 
   ",", "1.4697353911195499`", ",", "0.43184666073424177`", ",", 
   "0.1320853187965455`", ",", "28.11910455725513`"}], "}"}]], "Print",
 CellChangeTimes->{3.59337751679115*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.59337751679115*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59337751679115*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593377560720827*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593377571547246*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9902149999999998`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9902149999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.593377710730691*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593377767920391*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.533919062849193`", ",", "50.35792048277866`", ",", "67.57930528751508`",
    ",", "73.3879041711315`", ",", "85.12942387577687`", ",", 
   "90.10088492817215`", ",", "93.70672692539868`", ",", "98.24663502811538`",
    ",", "67.57930528751508`", ",", "0.6383151557195619`", ",", 
   "0.6360514202510391`", ",", "0.21938825472405096`", ",", "Indeterminate", 
   ",", "1.3892609216111933`", ",", "0.43184666073424177`", ",", 
   "0.11490912641601791`", ",", "39.19689101481279`"}], "}"}]], "Print",
 CellChangeTimes->{3.593377767920391*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933777836296186`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593378037910065*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.67148573093769`", ",", "53.65781722299558`", ",", "70.6174734194921`", 
   ",", "76.17779379323756`", ",", "87.05642361441502`", ",", 
   "91.48091084185802`", ",", "94.61054460325448`", ",", "98.48984308667899`",
    ",", "70.6174734194921`", ",", "0.668595218267543`", ",", 
   "0.6665153970092172`", ",", "0.21930161199081408`", ",", "Indeterminate", 
   ",", "1.4410233976143547`", ",", "0.4313327693168539`", ",", 
   "0.11912407704434524`", ",", "23.293971287228214`"}], "}"}]], "Print",
 CellChangeTimes->{3.593378037910065*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0007`", ",", "0.9901999999999999`", ",", "0.9877499999999999`", ",", 
     "0.9926499999999998`", ",", "42.27299999942482`", ",", 
     "10.753912129435545`", ",", "7.64508513911955`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0007, 
   0.9901999999999999, 0.9877499999999999, 0.9926499999999998, 
   42.27299999942482, 10.753912129435545`, 7.64508513911955}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593378037910065*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "18.85289311333333333333333333333333333333`10.204044253513121"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   18.85289311333333333333333333333333333333`10.204044253513121],
  Editable->False]], "Print",
 CellChangeTimes->{3.593378037910065*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593378038003666*^9}],

Cell[BoxData["\<\"Country: All (high theta)\"\>"], "Print",
 CellChangeTimes->{3.593378038003666*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.593378038237666*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593378038237666*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.59337804053087*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.59337804053087*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933781149898005`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933781150054007`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933781258162193`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.59337814311665*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.59337814311665*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933781463302555`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933781463458557`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933781897295322`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933781897295322`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933782004467506`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9893449999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9893449999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933782929861135`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933783492710123`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.27377660959025`", ",", "47.53272630250176`", ",", "64.55464390710397`", 
   ",", "70.51733749860644`", ",", "83.077183912981`", ",", 
   "88.66065775005313`", ",", "92.81553613963077`", ",", "98.10123448749115`",
    ",", "64.55464390710397`", ",", "0.6105463381023536`", ",", 
   "0.6080933111960304`", ",", "0.30001884417353064`", ",", "Indeterminate", 
   ",", "1.4943573186400145`", ",", "0.6202014161687857`", ",", 
   "0.13037041634096302`", ",", "83.47997427462688`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933783492710123`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933783492710123`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933783492710123`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933783927638884`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593378392795089*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593378392795089*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933784035747075`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989225`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989225],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933784976740727`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593378551899768*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.456389955041391`", ",", "49.728982643045924`", ",", 
   "66.63006344109004`", ",", "72.42691017567189`", ",", "84.39677944481691`",
    ",", "89.60620789952792`", ",", "93.43808757904263`", ",", 
   "98.26831048478616`", ",", "66.63006344109004`", ",", "0.630907380788847`",
    ",", "0.6285833906556247`", ",", "0.30001884417353064`", ",", 
   "Indeterminate", ",", "1.5260290194973751`", ",", "0.6202014161687857`", 
   ",", "0.13584112406140258`", ",", "50.437173024702695`"}], "}"}]], "Print",
 CellChangeTimes->{3.593378551899768*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.593378551915368*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593378551915368*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593378596047846*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933786068742647`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989045`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989045],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933786766219873`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933787326104856`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.511527323504815`", ",", "51.67238719663266`", ",", "68.44301040569235`",
    ",", "74.0875187195494`", ",", "85.51527098111146`", ",", 
   "90.39714525759643`", ",", "93.95523771491263`", ",", "98.40831060509184`",
    ",", "68.44301040569235`", ",", "0.6485102128375511`", ",", 
   "0.6462975198080383`", ",", "0.30001884417353064`", ",", "Indeterminate", 
   ",", "1.55585530794189`", ",", "0.6202014161687857`", ",", 
   "0.14322846944810053`", ",", "32.62691730969942`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933787326104856`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.001`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933787326104856`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933787326104856`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98886034399981716711661150892672610822`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98886034399981716711661150892672610822`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933792001433067`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593379210985326*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9888753439998171`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9888753439998171],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933792574422073`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933793139455066`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.612010354693425`", ",", "53.59730620435785`", ",", "70.18807679602205`",
    ",", "75.67647487491948`", ",", "86.56335396387577`", ",", 
   "91.13020497660035`", ",", "94.43391356664272`", ",", "98.53927220865363`",
    ",", "70.18807679602205`", ",", "0.6654364136718474`", ",", 
   "0.6633308041931207`", ",", "0.30001884417353064`", ",", "Indeterminate", 
   ",", "1.5868946831047852`", ",", "0.6202014161687857`", ",", 
   "0.15060498294010283`", ",", "24.88440025853383`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933793139455066`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0011`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0011],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933793139455066`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933793139455066`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98868156201542470623826786874288713804`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98868156201542470623826786874288713804`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933796158528366`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933796266636558`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9886965620154247`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9886965620154247],
  Editable->False]], "Print",
 CellChangeTimes->{3.593379673416938*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933797282666345`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.68955742778541`", ",", "55.424184174178556`", ",", "71.8010832145995`", 
   ",", "77.1300931580468`", ",", "87.50529237306935`", ",", 
   "91.78072939467826`", ",", "94.85440816994797`", ",", "98.6545004856654`", 
   ",", "71.8010832145995`", ",", "0.6810071912555568`", ",", 
   "0.6790001204675113`", ",", "0.30001884417353064`", ",", "Indeterminate", 
   ",", "1.6171868383982657`", ",", "0.6202014161687857`", ",", 
   "0.15833248442995063`", ",", "25.772777993252035`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933797282666345`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933797438822613`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593379994013101*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.109027180389077`", ",", "53.76363210459567`", ",", "70.38577534736568`",
    ",", "75.82918254502252`", ",", "86.64100542285446`", ",", 
   "91.1757487157118`", ",", "94.46136291808331`", ",", "98.54469546361683`", 
   ",", "70.38577534736568`", ",", "0.6664054299779648`", ",", 
   "0.664319253490929`", ",", "0.305158874311712`", ",", "Indeterminate", ",",
    "1.5883613878499163`", ",", "0.623970729691157`", ",", 
   "0.15032195593451014`", ",", "24.97388482745515`"}], "}"}]], "Print",
 CellChangeTimes->{3.593379994013101*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.001`", ",", "0.9888753439998171`", ",", "0.9853753439998172`", ",", 
     "0.9923753439998171`", ",", "39.21551680197559`", ",", 
     "10.249417931066269`", ",", "0.8997656279985663`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.001, 
   0.9888753439998171, 0.9853753439998172, 0.9923753439998171, 
   39.21551680197559, 10.249417931066269`, 0.8997656279985663}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933799940287013`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "32.60015725833333333333333333333333333333`10.441885943260923"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   32.60015725833333333333333333333333333333`10.441885943260923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933799940287013`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593379994106701*^9}],

Cell[BoxData["\<\"Country: All (very high theta)\"\>"], "Print",
 CellChangeTimes->{3.593379994122301*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5933799943407016`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5933799943407016`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593379996665106*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933799966807055`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98894662986751645056179153337593742114`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98894662986751645056179153337593742114`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933804303146677`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.98894662986751645056179153337593742114`10."}],
  SequenceForm[
  " Evaluate with good guess of \[Beta]middle =  ", 
   0.98894662986751645056179153337593742114`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933804412346864`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593380458129516*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593380458129516*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.593380461327522*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593380461327522*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933805368940544`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98899999999999999023003738329862244427`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98899999999999999023003738329862244427`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933805368940544`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933805475956736`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.98909`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.98909],
  Editable->False]], "Print",
 CellChangeTimes->{3.593380618450998*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593380672161892*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.389389565401293`", ",", "45.067128883757164`", ",", 
   "61.85703528647747`", ",", "67.90525543588319`", ",", "81.10081731922548`",
    ",", "87.21817370042837`", ",", "91.8808241642507`", ",", 
   "97.88944385520688`", ",", "61.85703528647747`", ",", 
   "0.5840020748404319`", ",", "0.5813925418789448`", ",", 
   "0.308674814415216`", ",", "Indeterminate", ",", "1.5335331214071999`", 
   ",", "0.629326377405982`", ",", "0.14171505904762816`", ",", 
   "151.27586384802967`"}], "}"}]], "Print",
 CellChangeTimes->{3.593380672161892*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593380672161892*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593380672161892*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98894664021650166283806698808068782358`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98894664021650166283806698808068782358`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933811676031623`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593381178398381*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9889516402165017`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9889516402165017],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933812246212626`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933812773181553`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.326038915885311`", ",", "46.91400758717164`", ",", 
   "63.595823935198695`", ",", "69.50976950586953`", ",", 
   "82.22077784972596`", ",", "88.02211105810152`", ",", "92.41025686086873`",
    ",", "98.03140279434037`", ",", "63.595823935198695`", ",", 
   "0.6011234775787223`", ",", "0.5986220881722324`", ",", 
   "0.308674814415216`", ",", "Indeterminate", ",", "1.556594625808436`", ",",
    "0.629326377405982`", ",", "0.14664994967964284`", ",", 
   "107.6372163321916`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933812773181553`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933812773181553`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933812773181553`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98879079172226847545628015697617999992`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98879079172226847545628015697617999992`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593381580863488*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593381591611907*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9887957917222685`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9887957917222685],
  Editable->False]], "Print",
 CellChangeTimes->{3.593381637772388*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593381692294484*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.257780970257787`", ",", "48.69991747362485`", ",", "65.24584281688063`",
    ",", "71.0220888004824`", ",", "83.25817482986668`", ",", 
   "88.76443092473416`", ",", "92.89889785178144`", ",", "98.16521212626094`",
    ",", "65.24584281688063`", ",", "0.6173215952534912`", ",", 
   "0.6149226303483926`", ",", "0.308674814415216`", ",", "Indeterminate", 
   ",", "1.5807246779741673`", ",", "0.629326377405982`", ",", 
   "0.15221862941789077`", ",", "75.12630205623607`"}], "}"}]], "Print",
 CellChangeTimes->{3.593381692294484*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.001`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.001],
  Editable->False]], "Print",
 CellChangeTimes->{3.593381692294484*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933816923100843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98862484306844397027234308825953325657`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98862484306844397027234308825953325657`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933819287908993`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593381939492518*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.988629843068444`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.988629843068444],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933819857153993`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933820449175034`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.202307526541514`", ",", "50.4406316530807`", ",", "66.82563735559344`", 
   ",", "72.45899778195371`", ",", "84.22538456661687`", ",", 
   "89.45076436379762`", ",", "93.35051747061105`", ",", "98.28974878987619`",
    ",", "66.82563735559344`", ",", "0.6327410959583706`", ",", 
   "0.6304396595741399`", ",", "0.308674814415216`", ",", "Indeterminate", 
   ",", "1.60521609815705`", ",", "0.629326377405982`", ",", 
   "0.15817954362884856`", ",", "52.011136710689065`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933820449175034`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0011`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0011],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933820449175034`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933820449175034`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98845087015674028914058107534460041504`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98845087015674028914058107534460041504`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593382314158376*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593382324906795*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9884558701567403`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9884558701567403],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933823709736757`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593382424980971*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.152142381248396`", ",", "52.12047181354242`", ",", "68.32529496902947`",
    ",", "73.81448442752165`", ",", "85.12220174442604`", ",", 
   "90.0809063724568`", ",", "93.76365182575032`", ",", "98.40398824397117`", 
   ",", "68.32529496902947`", ",", "0.6473077011672576`", ",", 
   "0.6450984191006767`", ",", "0.308674814415216`", ",", "Indeterminate", 
   ",", "1.6296905069510577`", ",", "0.629326377405982`", ",", 
   "0.164425019741526`", ",", "37.135200932382084`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933824249965706`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0012000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0012000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933824249965706`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933824249965706`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98826625693681802786502970379536670027`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98826625693681802786502970379536670027`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933826606817846`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933826713522034`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.988271256936818`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.988271256936818],
  Editable->False]], "Print",
 CellChangeTimes->{3.593382718214686*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593382772518381*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.10326736685062`", ",", "53.72800250600036`", ",", "69.73711718165423`", 
   ",", "75.07913792713452`", ",", "85.9456703203428`", ",", 
   "90.65499108822088`", ",", "94.13761773484926`", ",", "98.50739585942443`",
    ",", "69.73711718165423`", ",", "0.6609583666136534`", ",", 
   "0.6588353817695065`", ",", "0.308674814415216`", ",", "Indeterminate", 
   ",", "1.6541584166320302`", ",", "0.629326377405982`", ",", 
   "0.17097174205141508`", ",", "29.303522989021598`"}], "}"}]], "Print",
 CellChangeTimes->{3.593382772518381*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0013000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0013000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593382772518381*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933827725339813`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98807559653038125186541745441411775999`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98807559653038125186541745441411775999`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933830382648478`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933830488728666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.98807559653038125186541745441411775999`10."}],
  SequenceForm[
  " Solution of \[Beta]middle: ", 
   0.98807559653038125186541745441411775999`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593383095189348*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593383149711444*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.03360039157444`", ",", "55.24050724580013`", ",", "71.04783715653186`", 
   ",", "76.24277192674751`", ",", "86.6910007029562`", ",", 
   "91.17128377542832`", ",", "94.47245965027724`", ",", "98.59948693540036`",
    ",", "71.04783715653186`", ",", "0.673561792833376`", ",", 
   "0.6715184796116587`", ",", "0.308674814415216`", ",", "Indeterminate", 
   ",", "1.6781594342058213`", ",", "0.629326377405982`", ",", 
   "0.17777999989886728`", ",", "27.272480923898648`"}], "}"}]], "Print",
 CellChangeTimes->{3.593383149727044*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0014000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593383149727044*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593383149727044*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98787686323313327830999021222980527107`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98787686323313327830999021222980527107`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593383416269112*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593383426970731*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9878718632331333`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9878718632331333],
  Editable->False]], "Print",
 CellChangeTimes->{3.593383473178012*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933835269513063`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.943060430991142`", ",", "56.67080190746313`", ",", "72.26919757015989`",
    ",", "77.31877126151461`", ",", "87.36734716076231`", ",", 
   "91.63804177043251`", ",", "94.77401684341991`", ",", "98.68201294654772`",
    ",", "72.26919757015989`", ",", "0.6852563762093175`", ",", 
   "0.6832870523407326`", ",", "0.308674814415216`", ",", "Indeterminate", 
   ",", "1.7016349402834543`", ",", "0.629326377405982`", ",", 
   "0.18475237973465486`", ",", "29.825679008174728`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933835269513063`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933835423485336`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933837959737787`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.002928241941643`", ",", "55.370368569956`", ",", "71.23057909799336`", 
   ",", "76.41235732241431`", ",", "86.78608172190916`", ",", 
   "91.23712802183627`", ",", "94.5103512241105`", ",", "98.60735537836568`", 
   ",", "71.23057909799336`", ",", "0.6748776303694207`", ",", 
   "0.6728234803580163`", ",", "0.3048960388939765`", ",", "Indeterminate", 
   ",", "1.686911534102779`", ",", "0.6208720275397176`", ",", 
   "0.17853766982559371`", ",", "27.487749950965608`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933837959737787`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0013000000000000002`", ",", 
     "0.98807559653038125186541745441411775999`10.", ",", 
     "0.9835255965303812`", ",", "0.9926255965303813`", ",", 
     "39.31279283084515`", ",", "10.265682154392847`", ",", 
     "0.3619334224724991`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0013000000000000002`, 0.98807559653038125186541745441411775999`10., 
    0.9835255965303812, 0.9926255965303813, 39.31279283084515, 
    10.265682154392847`, 0.3619334224724991}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933837959737787`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "63.36445129333333333333333333333333333333`10.73051192664916"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   63.36445129333333333333333333333333333333`10.73051192664916],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933837959737787`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.59338379642618*^9}],

Cell[BoxData["\<\"Country: Austria\"\>"], "Print",
 CellChangeTimes->{3.59338379642618*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5933837966757803`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5933837966757803`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5933837990157843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933837990313845`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933838732875147`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593383873303115*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593383884207534*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5933839017575645`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5933839017575645`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338390493997*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59338390493997*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593383948167646*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593383948167646*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593383959056465*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933840535302315`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593384108083527*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.481938563466395`", ",", "50.36924985526128`", ",", "67.52551411032923`",
    ",", "73.34615896187115`", ",", "85.2602066021465`", ",", 
   "90.2947314231868`", ",", "93.89505654228827`", ",", "98.32607610774`", 
   ",", "67.52551411032923`", ",", "0.6398018212553555`", ",", 
   "0.6375513541437643`", ",", "0.30374133686373783`", ",", "Indeterminate", 
   ",", "1.3998851517983617`", ",", "0.624250250723705`", ",", 
   "0.11289947368126718`", ",", "200.56786066664787`"}], "}"}]], "Print",
 CellChangeTimes->{3.593384108083527*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593384108083527*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593384108083527*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593384151826004*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593384151841604*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593384151841604*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593384162652423*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933842339445477`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593384287780243*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.52611490403316`", ",", "52.57520862799989`", ",", "69.68276180435801`", 
   ",", "75.35473201347716`", ",", "86.66496639603014`", ",", 
   "91.28837171384929`", ",", "94.53450985185829`", ",", "98.49126308502062`",
    ",", "69.68276180435801`", ",", "0.6607422646720776`", ",", 
   "0.6586223521220188`", ",", "0.30374133686373783`", ",", "Indeterminate", 
   ",", "1.4403964667321858`", ",", "0.624250250723705`", ",", 
   "0.12279820605784508`", ",", "134.23677457559918`"}], "}"}]], "Print",
 CellChangeTimes->{3.593384287780243*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.593384287795843*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593384287795843*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338433186592*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593384342676739*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9893`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9893],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338441136366*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933844652929544`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.847958098913`", ",", "54.95305740712323`", ",", "71.87165106595404`", 
   ",", "77.35899609694336`", ",", "88.04007561303987`", ",", 
   "92.26275030073364`", ",", "95.17188437284251`", ",", "98.67085290104137`",
    ",", "71.87165106595404`", ",", "0.6822930198955344`", ",", 
   "0.6803097801468357`", ",", "0.30374133686373783`", ",", "Indeterminate", 
   ",", "1.4880405543974562`", ",", "0.624250250723705`", ",", 
   "0.12898922713015792`", ",", "81.9783016245056`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933844653085546`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.001`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933844653085546`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933844653085546`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593384509441032*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593384520314251*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989075`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989075],
  Editable->False]], "Print",
 CellChangeTimes->{3.593384589500373*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933846464404726`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.880802592376778`", ",", "56.9116206779119`", ",", "73.68760630157746`", 
   ",", "79.00590932945722`", ",", "89.1021930347532`", ",", 
   "92.9781975266105`", ",", "95.61574027788375`", ",", "98.78735344022373`", 
   ",", "73.68760630157746`", ",", "0.6992983428705816`", ",", 
   "0.6974215861852524`", ",", "0.30374133686373783`", ",", "Indeterminate", 
   ",", "1.52661841031836`", ",", "0.624250250723705`", ",", 
   "0.14011686423020253`", ",", "50.88596492412562`"}], "}"}]], "Print",
 CellChangeTimes->{3.593384646456073*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0011`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0011],
  Editable->False]], "Print",
 CellChangeTimes->{3.593384646456073*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593384646456073*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98885619329404980498218536272351794396`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98885619329404980498218536272351794396`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593385244358323*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933852551223416`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9888711932940498`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9888711932940498],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933853023436246`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593385357208921*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.939587153888187`", ",", "58.783952277281735`", ",", 
   "75.36440232723173`", ",", "80.51264244295118`", ",", "90.03565679764955`",
    ",", "93.59730526370488`", ",", "96.00015613009123`", ",", 
   "98.89052885376131`", ",", "75.36440232723173`", ",", 
   "0.7149141991943944`", ",", "0.7131354907567156`", ",", 
   "0.30374133686373783`", ",", "Indeterminate", ",", "1.5637694970961944`", 
   ",", "0.624250250723705`", ",", "0.15071667471388184`", ",", 
   "30.93231075454664`"}], "}"}]], "Print",
 CellChangeTimes->{3.593385357208921*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0012000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0012000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593385357208921*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593385357208921*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98866680810724426312519016658804716724`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98866680810724426312519016658804716724`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933856268709946`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933856377130136`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9886818081072443`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9886818081072443],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933856844194956`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933857390975914`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.04108589990406`", ",", "60.60063823965074`", ",", "76.9344754653232`", 
   ",", "81.90380808413254`", ",", "90.87114891064947`", ",", 
   "94.14139120743809`", ",", "96.33790593897254`", ",", "98.98333721611367`",
    ",", "76.9344754653232`", ",", "0.7294762112293063`", ",", 
   "0.7277891901461833`", ",", "0.30374133686373783`", ",", "Indeterminate", 
   ",", "1.5993410780607265`", ",", "0.624250250723705`", ",", 
   "0.1608198482056259`", ",", "19.570255048683013`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933857391131916`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0013000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0013000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933857391131916`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933857391131916`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98847187544300668996615372927301366595`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98847187544300668996615372927301366595`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933860081044645`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593386018946483*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9884768754430068`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9884768754430068],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933860652005644`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59338611980066*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.09970126875061`", ",", "62.27190231033345`", ",", "78.35337464888357`", 
   ",", "83.14987515247523`", ",", "91.58858025169647`", ",", 
   "94.59346354633436`", ",", "96.6137738952511`", ",", "99.05982169930101`", 
   ",", "78.35337464888357`", ",", "0.7424408809349625`", ",", 
   "0.740835393708191`", ",", "0.30374133686373783`", ",", "Indeterminate", 
   ",", "1.6317168736901744`", ",", "0.624250250723705`", ",", 
   "0.17125875834199378`", ",", "15.290612659504276`"}], "}"}]], "Print",
 CellChangeTimes->{3.59338611980066*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0014000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338611980066*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59338611980066*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9882718016050017843047785083395937046`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9882718016050017843047785083395937046`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593386387824731*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59338639860435*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9882718016050017843047785083395937046`10."}],
  SequenceForm[
  " Solution of \[Beta]middle: ", 0.9882718016050017843047785083395937046`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933864446088305`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933864985693254`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.152245748689246`", ",", "63.8448250401775`", ",", "79.66011133148241`", 
   ",", "84.28551180332519`", ",", "92.22536698821516`", ",", 
   "94.99056673548915`", ",", "96.85315278539652`", ",", "99.12578935717177`",
    ",", "79.66011133148241`", ",", "0.7543504231293942`", ",", 
   "0.7528200886905405`", ",", "0.30374133686373783`", ",", "Indeterminate", 
   ",", "1.6621031128989687`", ",", "0.624250250723705`", ",", 
   "0.1814663142690658`", ",", "16.11946916804191`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933864985849257`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933865143097534`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933867672641973`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.70527874737915`", ",", "63.00044711276163`", ",", "78.85129943876794`", 
   ",", "83.57784972732621`", ",", "91.78832379126463`", ",", 
   "94.71111355380684`", ",", "96.67974015452853`", ",", "99.07305017334525`",
    ",", "78.85129943876794`", ",", "0.747861891009853`", ",", 
   "0.746273961358335`", ",", "0.3051268737192862`", ",", "Indeterminate", 
   ",", "1.6305468673118493`", ",", "0.6219190219977687`", ",", 
   "0.17110413338632346`", ",", "15.420090034024911`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933867672641973`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0013000000000000002`", ",", "0.9884768754430068`", ",", 
     "0.9839268754430067`", ",", "0.9930268754430068`", ",", 
     "39.241007266242214`", ",", "10.253681253484348`", ",", 
     "1.1348994050240782`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0013000000000000002`, 0.9884768754430068, 0.9839268754430067, 
   0.9930268754430068, 39.241007266242214`, 10.253681253484348`, 
   1.1348994050240782`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933867672797976`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "49.51396696666666666666666666666666666667`10.623393970804768"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   49.51396696666666666666666666666666666667`10.623393970804768],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933867672797976`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933867676697984`*^9}],

Cell[BoxData["\<\"Country: Belgium\"\>"], "Print",
 CellChangeTimes->{3.593386767685398*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5933867679349985`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5933867679349985`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5933867702594028`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933867702594028`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593386845373535*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593386845373535*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593386856309154*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5933868735159845`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5933868735159845`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338687668279*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59338687668279*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593386920581267*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593386920581267*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933869314856863`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593387026380653*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933870815891495`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.953095785751128`", ",", "51.56103653698479`", ",", "68.31250643324819`",
    ",", "74.00321358660393`", ",", "85.58952212004574`", ",", 
   "90.48303492270597`", ",", "94.00115914032217`", ",", "98.35327127149708`",
    ",", "68.31250643324819`", ",", "0.6484228760528891`", ",", 
   "0.6461789684697785`", ",", "0.3091122964693249`", ",", "Indeterminate", 
   ",", "1.3990435286049985`", ",", "0.6330159884539527`", ",", 
   "0.11329263242058618`", ",", "60.54391239910649`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933870815891495`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933870815891495`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933870815891495`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593387125409627*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593387125784027*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593387125784027*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593387136828847*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.593387230522611*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593387283796705*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.045881312214878`", ",", "53.73645223017953`", ",", "70.42759643013635`",
    ",", "75.96959477540662`", ",", "86.95500757500422`", ",", 
   "91.45466200304476`", ",", "94.62761510314017`", ",", "98.51691448086525`",
    ",", "70.42759643013635`", ",", "0.6689638235524433`", ",", 
   "0.6668486443722185`", ",", "0.3091122964693249`", ",", "Indeterminate", 
   ",", "1.4394555990938112`", ",", "0.6330159884539527`", ",", 
   "0.12331020365132905`", ",", "99.93650058479797`"}], "}"}]], "Print",
 CellChangeTimes->{3.593387283796705*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.593387283796705*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593387283796705*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933873280539827`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933873388804016`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9896299999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9896299999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593387453618603*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593387508109499*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.43275021209233`", ",", "48.8732937856211`", ",", "65.7524683350932`", 
   ",", "71.6268639169933`", ",", "83.88142297923706`", ",", 
   "89.22666570292225`", ",", "93.15832165703544`", ",", "98.1245960263533`", 
   ",", "65.7524683350932`", ",", "0.6226059581005315`", ",", 
   "0.6202001035268316`", ",", "0.3091122964693249`", ",", "Indeterminate", 
   ",", "1.3527315599532683`", ",", "0.6330159884539527`", ",", 
   "0.10939962050184104`", ",", "31.473026241190283`"}], "}"}]], "Print",
 CellChangeTimes->{3.593387508109499*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0004999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0004999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593387508109499*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593387508109499*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933875522887764`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933875631151953`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9897499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9897499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933876796162*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593387737648302*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.150287163291395`", ",", "46.31138773630655`", ",", "63.23520767567592`",
    ",", "69.26118380545525`", ",", "82.14865781211009`", ",", 
   "87.96171731298668`", ",", "92.33011683853299`", ",", "97.93485072246779`",
    ",", "63.23520767567592`", ",", "0.5974991351170547`", ",", 
   "0.5949367922562079`", ",", "0.3091122964693249`", ",", "Indeterminate", 
   ",", "1.31784251107275`", ",", "0.6330159884539527`", ",", 
   "0.10412711237732486`", ",", "22.854341898990405`"}], "}"}]], "Print",
 CellChangeTimes->{3.593387737648302*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0003999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0003999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593387737648302*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593387737648302*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593387781624779*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933877924823985`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933879081254015`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593387962397897*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.985378215520473`", ",", "43.90203414243522`", ",", "60.85273909198122`",
    ",", "67.01839279106137`", ",", "80.53203096755671`", ",", 
   "86.82036758968037`", ",", "91.62599924774608`", ",", "97.81149468053589`",
    ",", "60.85273909198122`", ",", "0.5744527785134672`", ",", 
   "0.5717480575539384`", ",", "0.3091122964693249`", ",", "Indeterminate", 
   ",", "1.2963663827197403`", ",", "0.6330159884539527`", ",", 
   "0.09934078831962778`", ",", "32.16196102196607`"}], "}"}]], "Print",
 CellChangeTimes->{3.593387962397897*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933879781227245`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933882292675657`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.862120049000769`", ",", "45.51935936415108`", ",", "62.70837191434659`",
    ",", "68.81667211934005`", ",", "81.88231711675652`", ",", 
   "87.79783789231759`", ",", "92.23907836123868`", ",", "97.90842304112846`",
    ",", "62.70837191434659`", ",", "0.5916712881622229`", ",", 
   "0.5891099448769259`", ",", "0.30373849003805686`", ",", "Indeterminate", 
   ",", "1.3184940948014485`", ",", "0.6229980134062226`", ",", 
   "0.10397286364140346`", ",", "22.594346347636126`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933882292675657`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0004999999999999999`", ",", "0.9897499999999999`", ",", 
     "0.9879999999999999`", ",", "0.9914999999999999`", ",", 
     "39.955387103063394`", ",", "10.37276020875299`", ",", 
     "1.7395920010473385`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0004999999999999999, 0.9897499999999999, 0.9879999999999999, 
   0.9914999999999999, 39.955387103063394`, 10.37276020875299, 
   1.7395920010473385`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933882292675657`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "24.35996278666666666666666666666666666667`10.315342868729932"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   24.35996278666666666666666666666666666667`10.315342868729932],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933882292675657`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933882293299656`*^9}],

Cell[BoxData["\<\"Country: Cyprus\"\>"], "Print",
 CellChangeTimes->{3.593388229345566*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.593388229579566*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593388229579566*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5933882319819703`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933882319819703`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593388306394101*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593388306394101*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338831725172*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5933883344117503`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5933883344117503`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.593388337703356*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593388337703356*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593388381492633*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593388381492633*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933883923190517`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593388486262417*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59338853920891*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.751208133563804`", ",", "50.66988213111059`", ",", "67.8448959512934`", 
   ",", "73.66426413059676`", ",", "85.43441856609407`", ",", 
   "90.39605210642068`", ",", "93.95536752166011`", ",", "98.344173826834`", 
   ",", "67.8448959512934`", ",", "0.6433388441104247`", ",", 
   "0.6411015366012611`", ",", "0.30802439165178436`", ",", "Indeterminate", 
   ",", "1.402558705616933`", ",", "0.6283446055324015`", ",", 
   "0.11300857855376978`", ",", "24.270368089503666`"}], "}"}]], "Print",
 CellChangeTimes->{3.59338853920891*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338853920891*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59338853920891*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593388582920187*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933885829357867`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933885829513865`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933885937778053`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933886885167723`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933887445364704`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.7846572490389`", ",", "52.82914535946662`", ",", "69.94301806522101`", 
   ",", "75.62982294370369`", ",", "86.81264518810137`", ",", 
   "91.37392112835411`", ",", "94.58844744208086`", ",", "98.50682631545986`",
    ",", "69.94301806522101`", ",", "0.6638443406208058`", ",", 
   "0.6617374995688383`", ",", "0.30802439165178436`", ",", "Indeterminate", 
   ",", "1.4429434459923371`", ",", "0.6283446055324015`", ",", 
   "0.12286006127830684`", ",", "6.20072073853179`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933887445364704`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933887445364704`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933887445364704`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593388788700148*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933887995265675`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989225`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989225],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338889256513*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593388947492827*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.943636142916148`", ",", "55.011932755662215`", ",", 
   "71.98520837008488`", ",", "77.50385666484495`", ",", "88.09264041756809`",
    ",", "92.26745613694649`", ",", "95.16520663828639`", ",", 
   "98.6641677903138`", ",", "71.98520837008488`", ",", "0.6836581082003538`",
    ",", "0.6816783988834243`", ",", "0.30802439165178436`", ",", 
   "Indeterminate", ",", "1.4859959497726325`", ",", "0.6283446055324015`", 
   ",", "0.13129487766531922`", ",", "1.4748797341150612`"}], "}"}]], "Print",
 CellChangeTimes->{3.593388947492827*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.001`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933889475084267`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933889475084267`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593388991781305*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593389002670124*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.98903`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.98903],
  Editable->False]], "Print",
 CellChangeTimes->{3.593389049392206*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593389103695901*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.042416573879013`", ",", "56.973854853992776`", ",", "73.8003340151814`",
    ",", "79.15301350479524`", ",", "89.16638350941889`", ",", 
   "92.99714457921355`", ",", "95.6222884261305`", ",", "98.78613339812976`", 
   ",", "73.8003340151814`", ",", "0.7008174228884476`", ",", 
   "0.6989477400984723`", ",", "0.30802439165178436`", ",", "Indeterminate", 
   ",", "1.5249839896398811`", ",", "0.6283446055324015`", ",", 
   "0.14147043316383923`", ",", "7.408713376407715`"}], "}"}]], "Print",
 CellChangeTimes->{3.593389103711501*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933891192647285`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933893743407764`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.545089340516377`", ",", "55.120090985334315`", ",", 
   "72.09796877594955`", ",", "77.56699355769847`", ",", "88.08311342074619`",
    ",", "92.25045129591635`", ",", "95.14118009587258`", ",", 
   "98.65347812546322`", ",", "72.09796877594955`", ",", 
   "0.6837613438318308`", ",", "0.6817794270583842`", ",", 
   "0.30539323450627176`", ",", "Indeterminate", ",", "1.4827784076106976`", 
   ",", "0.6251681882650834`", ",", "0.13123649150129568`", ",", 
   "1.5304909644291214`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933893743407764`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0009000000000000001`", ",", "0.989225`", ",", "0.986075`", ",", 
     "0.992375`", ",", "39.77592942624679`", ",", "10.342919255282522`", ",", 
     "1.4180076992592234`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0009000000000000001, 0.989225, 0.986075, 0.992375, 39.77592942624679, 
   10.342919255282522`, 1.4180076992592234`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933893743563766`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "19.08351351833333333333333333333333333333`10.209324585100825"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   19.08351351833333333333333333333333333333`10.209324585100825],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933893743563766`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933893747463775`*^9}],

Cell[BoxData["\<\"Country: Germany\"\>"], "Print",
 CellChangeTimes->{3.5933893747463775`*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5933893749959784`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5933893749959784`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593389377351582*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593389377351582*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593389451872913*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593389451872913*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593389462699332*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5933894800153627`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5933894800153627`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933894832289677`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933894832289677`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933895267842445`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933895267842445`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933895376262636`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9893299999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9893299999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933896079979873`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59338966102248*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.39758529185803`", ",", "47.73803830735875`", ",", "64.702664053609`", 
   ",", "70.57006554000412`", ",", "83.02044628077918`", ",", 
   "88.61169768070401`", ",", "92.79194557993738`", ",", "98.09957740116855`",
    ",", "64.702664053609`", ",", "0.610460176747713`", ",", 
   "0.6080256062472778`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.495449548751394`", ",", "0.6169429824850287`", ",", 
   "0.13047760984405543`", ",", "383.2719718909422`"}], "}"}]], "Print",
 CellChangeTimes->{3.59338966102248*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.59338966102248*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59338966102248*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593389704967757*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933897049989576`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933897049989576`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933897158721766`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989185`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989185],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933898108451433`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933898643688374`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.380120685102968`", ",", "49.84083432588481`", ",", "66.69284916738178`",
    ",", "72.40524857481448`", ",", "84.2847249399515`", ",", 
   "89.51156634850962`", ",", "93.37961299374304`", ",", "98.25414046725028`",
    ",", "66.69284916738178`", ",", "0.6297959076952061`", ",", 
   "0.6274839612222546`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.525039861053345`", ",", "0.6169429824850287`", ",", 
   "0.13660397555634474`", ",", "294.8363688109815`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933898643688374`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933898643844376`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933898643844376`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933899084701147`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593389919327734*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.98903`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.98903],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933899664242167`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933900221475143`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.366541221025724`", ",", "51.852933762030965`", ",", 
   "68.56078311680936`", ",", "74.11936328143886`", ",", "85.449403430949`", 
   ",", "90.3347170781047`", ",", "93.9150408167491`", ",", 
   "98.39786905744397`", ",", "68.56078311680936`", ",", 
   "0.6479309499634455`", ",", "0.6457341376520755`", ",", 
   "0.30192963525919114`", ",", "Indeterminate", ",", "1.5557731308603273`", 
   ",", "0.6169429824850287`", ",", "0.14335857945801722`", ",", 
   "223.1826161067856`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933900221631145`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.001`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933900221631145`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933900221631145`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98886034399970748195042634631840456457`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98886034399970748195042634631840456457`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933904885259333`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593390499367953*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9888653439997075`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9888653439997075],
  Editable->False]], "Print",
 CellChangeTimes->{3.593390545731234*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593390602967735*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.339870001541893`", ",", "53.76126623100842`", ",", "70.29755401907025`",
    ",", "75.69741007483307`", ",", "86.49854292754874`", ",", 
   "91.07012107101579`", ",", "94.39219218057865`", ",", "98.52705886976932`",
    ",", "70.29755401907025`", ",", "0.6646679453766737`", ",", 
   "0.6625775986052289`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.586451929784972`", ",", "0.6169429824850287`", ",", 
   "0.1505862833486573`", ",", "166.5450597779112`"}], "}"}]], "Print",
 CellChangeTimes->{3.593390602967735*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0011`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0011],
  Editable->False]], "Print",
 CellChangeTimes->{3.593390602967735*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593390602967735*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98868156201541795034045492068518606862`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98868156201541795034045492068518606862`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933909057954664`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933909166842856`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9886865620154179`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9886865620154179],
  Editable->False]], "Print",
 CellChangeTimes->{3.593390962657566*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933910190204654`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.29729083698867`", ",", "55.56426121172737`", ",", "71.90451985971264`", 
   ",", "77.14611210939563`", ",", "87.43487831020407`", ",", 
   "91.71913912185524`", ",", "94.81099445574887`", ",", "98.64072331660762`",
    ",", "71.90451985971264`", ",", "0.6800316701903271`", ",", 
   "0.6780392404253639`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.6162394398876019`", ",", "0.6169429824850287`", ",", 
   "0.1582895719859958`", ",", "122.5989825167987`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933910190204654`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0012000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0012000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933910190360656`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933910190360656`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98849493161211321595442083092378643923`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98849493161211321595442083092378643923`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933912887137394`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933912993997583`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9884999316121132`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9884999316121132],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933913458722396`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593391400191535*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.247682055804667`", ",", "57.26487756111044`", ",", "73.38543656602745`",
    ",", "78.47533993860296`", ",", "88.27218542067624`", ",", 
   "92.29170152131796`", ",", "95.17791115450746`", ",", "98.74070576418734`",
    ",", "73.38543656602745`", ",", "0.6941384140236303`", ",", 
   "0.6922360235521406`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.6380056282974569`", ",", "0.6169429824850287`", ",", 
   "0.16627055928597811`", ",", "89.03486627283176`"}], "}"}]], "Print",
 CellChangeTimes->{3.593391400207135*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0013000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0013000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593391400207135*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593391400207135*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98830061853988835983409275105010463472`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98830061853988835983409275105010463472`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59339163676595*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933916475455694`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.98830061853988835983409275105010463472`10."}],
  SequenceForm[
  " Solution of \[Beta]middle: ", 
   0.98830061853988835983409275105010463472`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59339169372165*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593391747510545*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.17903976858591`", ",", "58.85439655875857`", ",", "74.74174226335857`", 
   ",", "79.68028115473419`", ",", "89.01675790858076`", ",", 
   "92.7931976910549`", ",", "95.49694875184676`", ",", "98.82750958583071`", 
   ",", "74.74174226335857`", ",", "0.7069869364364696`", ",", 
   "0.7051665632572465`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.6643097376274238`", ",", "0.6169429824850287`", ",", 
   "0.1745775831581532`", ",", "64.00248360615137`"}], "}"}]], "Print",
 CellChangeTimes->{3.593391747510545*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0014000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593391747526145*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593391747526145*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98810024549206546797055868184877132219`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98810024549206546797055868184877132219`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593392016236617*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933920269538355`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9880952454920654`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9880952454920654],
  Editable->False]], "Print",
 CellChangeTimes->{3.593392073270317*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933921282916136`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.105477435612073`", ",", "60.361298581067004`", ",", 
   "76.00114688994854`", ",", "80.78688300817501`", ",", "89.68930503637077`",
    ",", "93.2416415759866`", ",", "95.78039261863903`", ",", 
   "98.9042334150179`", ",", "76.00114688994854`", ",", "0.7188789887792745`",
    ",", "0.7171345097086312`", ",", "0.30192963525919114`", ",", 
   "Indeterminate", ",", "1.6903765487161015`", ",", "0.6169429824850287`", 
   ",", "0.1829864352837951`", ",", "45.51313738727209`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933921282916136`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0015000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933921282916136`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593392128307214*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9878938799946133587555629669477836376`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9878938799946133587555629669477836376`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933923635868273`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593392374179246*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9878788799946133`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9878788799946133],
  Editable->False]], "Print",
 CellChangeTimes->{3.593392419856126*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593392474128621*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.003966930368478`", ",", "61.770544930573`", ",", "77.14985905942922`", 
   ",", "81.78720440283483`", ",", "90.28545493945433`", ",", 
   "93.63611861926768`", ",", "96.02822060811884`", ",", "98.97097725582135`",
    ",", "77.14985905942922`", ",", "0.7297056886168377`", ",", 
   "0.7280302053571513`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.7154603989304398`", ",", "0.6169429824850287`", ",", 
   "0.19157484292120852`", ",", "32.56919184860084`"}], "}"}]], "Print",
 CellChangeTimes->{3.593392474128621*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016000000000000003`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0016000000000000003`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593392474128621*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593392474128621*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98768203889775203394092273174639881046`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98768203889775203394092273174639881046`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593392708035432*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933927187682505`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9876520388977521`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9876520388977521],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933928106836123`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933928642385063`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.860688845445836`", ",", "63.081901390915704`", ",", 
   "78.19454898146377`", ",", "82.68616035384971`", ",", "90.80976551604572`",
    ",", "93.982124502052`", ",", "96.24452650308083`", ",", 
   "99.02919655471352`", ",", "78.19454898146377`", ",", 
   "0.7395246775378654`", ",", "0.7379116828115525`", ",", 
   "0.30192963525919114`", ",", "Indeterminate", ",", "1.739015129599473`", 
   ",", "0.6169429824850287`", ",", "0.2002848721324555`", ",", 
   "24.00380267191034`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933928642385063`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000003`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000003`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933928642385063`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933928642385063`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98746554554428634792512372494419937619`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98746554554428634792512372494419937619`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933931295169725`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933931400625906`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9874355455442864`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9874355455442864],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933931852870703`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933932405111675`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.7646782623117`", ",", "64.38580563164983`", ",", "79.21544900361098`", 
   ",", "83.55622161514519`", ",", "91.31225698902443`", ",", 
   "94.31532854300745`", ",", "96.4537699436043`", ",", "99.08545946446698`", 
   ",", "79.21544900361098`", ",", "0.7491678509935673`", ",", 
   "0.7476163658046704`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.7623602403875864`", ",", "0.6169429824850287`", ",", 
   "0.2084849519732771`", ",", "18.441248829747117`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933932405111675`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000004`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000004`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933932405111675`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933932405111675`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98724415098652193349262528730026437028`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98724415098652193349262528730026437028`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593393474324378*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933934849479966`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.987199150986522`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.987199150986522],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933935534477167`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593393608749814*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.592200633978617`", ",", "65.55974821822363`", ",", "80.11816516131273`",
    ",", "84.31061751168716`", ",", "91.73323978045971`", ",", 
   "94.59264569540828`", ",", "96.62747253298684`", ",", "99.13242802813988`",
    ",", "80.11816516131273`", ",", "0.7575963901911775`", ",", 
   "0.7560984999042842`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.7839139901934442`", ",", "0.6169429824850287`", ",", 
   "0.21708558326781935`", ",", "15.940589404312002`"}], "}"}]], "Print",
 CellChangeTimes->{3.593393608749814*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0019000000000000004`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0019000000000000004`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593393608749814*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593393608749814*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98701889412353396325843872822970646281`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98701889412353396325843872822970646281`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933938412994223`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933938517826405`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.986958894123534`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.986958894123534],
  Editable->False]], "Print",
 CellChangeTimes->{3.593393920797162*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933939766140604`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.409951835162097`", ",", "66.67861990045462`", ",", "80.97049102928075`",
    ",", "85.01290913092227`", ",", "92.11542524972913`", ",", 
   "94.84370645876177`", ",", "96.78514721805878`", ",", "99.17530520019427`",
    ",", "80.97049102928075`", ",", "0.7655122062352857`", ",", 
   "0.7640646583256941`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.8045219606456575`", ",", "0.6169429824850287`", ",", 
   "0.22557986985287382`", ",", "15.559391201700286`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933939766140604`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0020000000000000005`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0020000000000000005`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933939766140604`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933939766140604`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98679051248513660260176035899857429036`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98679051248513660260176035899857429036`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593394207775266*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933942182740846`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9867155124851366`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9867155124851366],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933943094718447`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593394364040741*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.201080818875493`", ",", "67.73243015683846`", ",", "81.76748240413603`",
    ",", "85.66493125983412`", ",", "92.46147644023779`", ",", 
   "95.06996734480607`", ",", "96.92750275092898`", ",", "99.21409789218075`",
    ",", "81.76748240413603`", ",", "0.7728830991489704`", ",", 
   "0.7714823907499145`", ",", "0.30192963525919114`", ",", "Indeterminate", 
   ",", "1.8244868152573033`", ",", "0.6169429824850287`", ",", 
   "0.23396189427171876`", ",", "16.896717491139952`"}], "}"}]], "Print",
 CellChangeTimes->{3.593394364040741*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593394379562768*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593394645839636*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.356097181841232`", ",", "67.11675181553464`", ",", "81.26459690359144`",
    ",", "85.2419777272031`", ",", "92.25573052658285`", ",", 
   "94.94557464039796`", ",", "96.85356039707396`", ",", "99.1951417583245`", 
   ",", "81.26459690359144`", ",", "0.7697057383797602`", ",", 
   "0.7682518992883766`", ",", "0.303921160325122`", ",", "Indeterminate", 
   ",", "1.8048965982275957`", ",", "0.6242700890597831`", ",", 
   "0.22557506807104566`", ",", "15.730465921214531`"}], "}"}]], "Print",
 CellChangeTimes->{3.593394645839636*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0019000000000000004`", ",", "0.986958894123534`", ",", 
     "0.980308894123534`", ",", "0.993608894123534`", ",", 
     "39.20306517574723`", ",", "10.247335013256636`", ",", 
     "1.7998320788814914`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0019000000000000004`, 0.986958894123534, 0.980308894123534, 
   0.993608894123534, 39.20306517574723, 10.247335013256636`, 
   1.7998320788814914`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593394645839636*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "87.85155430333333333333333333333333333333`10.872415697819164"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   87.85155430333333333333333333333333333333`10.872415697819164],
  Editable->False]], "Print",
 CellChangeTimes->{3.593394645839636*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933946462764363`*^9}],

Cell[BoxData["\<\"Country: Spain\"\>"], "Print",
 CellChangeTimes->{3.593394646292036*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5933946465416365`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5933946465416365`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593394648850441*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593394648850441*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933947234029717`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593394723418572*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933947344789915`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5933947518262215`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5933947518262215`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933947550554276`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933947550554276`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933947986263037`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593394798641904*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933948093591228`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9893149999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9893149999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593394881197249*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933949345805426`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.14011860670204`", ",", "48.41701183853905`", ",", "65.15783684618287`", 
   ",", "71.00364775786466`", ",", "83.3254769155447`", ",", 
   "88.8194877425164`", ",", "92.91820474547868`", ",", "98.12797696358172`", 
   ",", "65.15783684618287`", ",", "0.6165048728139011`", ",", 
   "0.6140659584175239`", ",", "0.30292932170737713`", ",", "Indeterminate", 
   ",", "1.496970881888443`", ",", "0.6308912456528353`", ",", 
   "0.13119470407824085`", ",", "44.2840296692236`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933949345805426`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933949345805426`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933949345805426`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933949783542194`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933949783698196`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933949783698196`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593394989180639*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.98918`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.98918],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933950609563646`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593395116274062*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.302487346199303`", ",", "50.55090182502235`", ",", "67.1693537676199`", 
   ",", "72.85697214944005`", ",", "84.60566619989646`", ",", 
   "89.73383726417701`", ",", "93.5182287745643`", ",", "98.28909182781163`", 
   ",", "67.1693537676199`", ",", "0.636260652039866`", ",", 
   "0.6339463275787319`", ",", "0.30292932170737713`", ",", "Indeterminate", 
   ",", "1.527701885832494`", ",", "0.6308912456528353`", ",", 
   "0.13713916329040665`", ",", "80.39445905603709`"}], "}"}]], "Print",
 CellChangeTimes->{3.593395116274062*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.593395116274062*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593395116289662*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933951602973394`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593395171186158*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9894499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9894499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933952636007204`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593395318622017*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.015961106525769`", ",", "46.243745159647624`", ",", 
   "63.07585445214172`", ",", "69.07502205231246`", ",", "81.980538974301`", 
   ",", "87.8557403558602`", ",", "92.2872156936433`", ",", 
   "97.96472010189714`", ",", "63.07585445214172`", ",", "0.59609884096286`", 
   ",", "0.5935315894026127`", ",", "0.30292932170737713`", ",", 
   "Indeterminate", ",", "1.4691224244293577`", ",", "0.6308912456528353`", 
   ",", "0.12571294214048556`", ",", "19.628868755326195`"}], "}"}]], "Print",
 CellChangeTimes->{3.593395318622017*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0004999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0004999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593395318622017*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593395318622017*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933953625048943`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933953733625135`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593395466510277*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933955215471735`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.956746188368049`", ",", "44.11324795393377`", ",", "61.02189660164754`",
    ",", "67.17305329229815`", ",", "80.65350103099539`", ",", 
   "86.91005107697863`", ",", "91.67577319011518`", ",", "97.82496232404068`",
    ",", "61.02189660164754`", ",", "0.5761873735764116`", ",", 
   "0.5734954994268938`", ",", "0.30292932170737713`", ",", "Indeterminate", 
   ",", "1.4477539188168818`", ",", "0.6308912456528353`", ",", 
   "0.12051378453937245`", ",", "8.06602074348181`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933955215471735`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0003999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0003999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933955215471735`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933955215471735`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593395565445651*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59339557627207*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9896449999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9896449999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933956938338766`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933957485119724`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "11.782629195600041`", ",", "41.76580235928682`", ",", 
   "58.762476452367466`", ",", "65.07789681702518`", ",", 
   "79.18755640843797`", ",", "85.86384886247546`", ",", "90.99986228192`", 
   ",", "97.66981503140106`", ",", "58.762476452367466`", ",", 
   "0.5541891269238245`", ",", "0.5513592833834716`", ",", 
   "0.30292932170737713`", ",", "Indeterminate", ",", "1.425992793979931`", 
   ",", "0.6308912456528353`", ",", "0.11819178563529609`", ",", 
   "10.101212451034844`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933957485119724`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933957642524*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933960292500653`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.249844760414366`", ",", "43.66326538323222`", ",", 
   "60.763555622548914`", ",", "66.96017973722817`", ",", 
   "80.51626128453717`", ",", "86.8166224086161`", ",", "91.61848023002491`", 
   ",", "97.81277236530384`", ",", "60.763555622548914`", ",", 
   "0.5729945482885188`", ",", "0.5703073860872753`", ",", 
   "0.3039338577470521`", ",", "Indeterminate", ",", "1.4465697496171026`", 
   ",", "0.6213046843731937`", ",", "0.12049913017671843`", ",", 
   "7.453122985359945`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933960292500653`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0004999999999999999`", ",", "0.9895999999999999`", ",", 
     "0.9878499999999999`", ",", "0.99135`", ",", "40.266905455702364`", ",", 
     "10.424446436693868`", ",", "2.5086795029119213`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0004999999999999999, 0.9895999999999999, 0.9878499999999999, 0.99135, 
   40.266905455702364`, 10.424446436693868`, 2.5086795029119213`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933960292500653`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "23.04956048333333333333333333333333333333`10.29132889674898"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   23.04956048333333333333333333333333333333`10.29132889674898],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933960292500653`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933960296868663`*^9}],

Cell[BoxData["\<\"Country: Finland\"\>"], "Print",
 CellChangeTimes->{3.5933960296868663`*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5933960299364667`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5933960299364667`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593396032323271*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593396032323271*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593396107406203*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593396107406203*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933961183418217`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5933961357358522`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5933961357358522`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.593396138933858*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593396138933858*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593396182442334*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593396182442334*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933961932843533`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933962871965184`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933963406734123`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.172148776266802`", ",", "50.41766791408694`", ",", "67.58419957393458`",
    ",", "73.45134011218406`", ",", "85.33059044221272`", ",", 
   "90.32258145292806`", ",", "93.90329499665314`", ",", "98.32486886699968`",
    ",", "67.58419957393458`", ",", "0.6409487892881611`", ",", 
   "0.638716163335389`", ",", "0.3045922115937591`", ",", "Indeterminate", 
   ",", "1.3973189072518621`", ",", "0.6247408290335674`", ",", 
   "0.11280490185545111`", ",", "78.79411205702549`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933963406734123`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933963406734123`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933963406734123`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593396384291089*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933963843066893`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933963843066893`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593396395211108*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933964891700735`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933965428809676`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.175924072437294`", ",", "52.63828529902841`", ",", "69.74193169174909`",
    ",", "75.44237675286408`", ",", "86.72640836356864`", ",", 
   "91.31532179051199`", ",", "94.54194275247959`", ",", "98.4864309523133`", 
   ",", "69.74193169174909`", ",", "0.6618191566051695`", ",", 
   "0.6597163900722557`", ",", "0.3045922115937591`", ",", "Indeterminate", 
   ",", "1.436984313955218`", ",", "0.6247408290335674`", ",", 
   "0.12270107648943895`", ",", "60.042782902801314`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933965428809676`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933965428809676`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933965428809676`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933965870758452`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593396597902264*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9893`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9893],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933966673691864`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593396722390483*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.443411267288763`", ",", "55.01333493709393`", ",", "71.92730346913653`",
    ",", "77.44221531071211`", ",", "88.08527490539092`", ",", 
   "92.28595142439477`", ",", "95.17817723608307`", ",", "98.66550783445071`",
    ",", "71.92730346913653`", ",", "0.6832575268388482`", ",", 
   "0.6812907938978725`", ",", "0.3045922115937591`", ",", "Indeterminate", 
   ",", "1.4839071959287387`", ",", "0.6247408290335674`", ",", 
   "0.12887561552160232`", ",", "55.69298188126119`"}], "}"}]], "Print",
 CellChangeTimes->{3.593396722390483*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.001`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.001],
  Editable->False]], "Print",
 CellChangeTimes->{3.593396722390483*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593396722390483*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933967665697603`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933967774741793`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989035`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989035],
  Editable->False]], "Print",
 CellChangeTimes->{3.593396847159502*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593396901884398*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.314442245591266`", ",", "56.901459372358275`", ",", 
   "73.67727108182149`", ",", "79.03028139054838`", ",", "89.0920813807965`", 
   ",", "92.95408939017065`", ",", "95.5904090090881`", ",", 
   "98.77164297191136`", ",", "73.67727108182149`", ",", 
   "0.6993715670696428`", ",", "0.6975054000115848`", ",", 
   "0.3045922115937591`", ",", "Indeterminate", ",", "1.5197428004003015`", 
   ",", "0.6247408290335674`", ",", "0.14120988516180477`", ",", 
   "63.72849501173762`"}], "}"}]], "Print",
 CellChangeTimes->{3.593396901884398*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933969176092257`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933971787380843`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.126255020024633`", ",", "55.42341890285554`", ",", "72.20884225000938`",
    ",", "77.6423399950556`", ",", "88.15978915429754`", ",", 
   "92.33079218732028`", ",", "95.21089747995426`", ",", "98.67854638173918`",
    ",", "72.20884225000938`", ",", "0.6858783170094739`", ",", 
   "0.6839042609424174`", ",", "0.30551937992037786`", ",", "Indeterminate", 
   ",", "1.486613185262642`", ",", "0.6271271881753016`", ",", 
   "0.129036417389349`", ",", "57.199529644907464`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933971787380843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0009000000000000001`", ",", "0.9893`", ",", "0.98615`", ",", 
     "0.9924499999999999`", ",", "41.24477794718754`", ",", 
     "10.585765269354726`", ",", "4.959653453038679`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0009000000000000001, 0.9893, 0.98615, 0.9924499999999999, 
   41.24477794718754, 10.585765269354726`, 4.959653453038679}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933971787380843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "19.15085363666666666666666666666666666667`10.210854385341749"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   19.15085363666666666666666666666666666667`10.210854385341749],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933971787380843`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933971788316846`*^9}],

Cell[BoxData["\<\"Country: France\"\>"], "Print",
 CellChangeTimes->{3.5933971788316846`*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.593397179081285*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593397179081285*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593397181452489*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593397181452489*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933972561610203`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933972561610203`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.59339726711224*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.59339728444387*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.59339728444387*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933972876574755`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933972876574755`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933973313531523`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933973313531523`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933973421171713`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9894499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9894499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933974378701396`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593397493499837*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.727432861382209`", ",", "48.97780387501498`", ",", "66.07200077060017`",
    ",", "71.92658952825413`", ",", "84.03498878023625`", ",", 
   "89.33966618439568`", ",", "93.2492856846826`", ",", "98.19551012480932`", 
   ",", "66.07200077060017`", ",", "0.6244995470733115`", ",", 
   "0.6221409342724525`", ",", "0.30473955331809033`", ",", "Indeterminate", 
   ",", "1.396930364631812`", ",", "0.6210848348721715`", ",", 
   "0.12217385429747207`", ",", "60.915702872032924`"}], "}"}]], "Print",
 CellChangeTimes->{3.593397493499837*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593397493499837*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593397493499837*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593397537304714*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933975373203144`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933975373203144`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933975482715335`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9893`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9893],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933976190488577`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933976736177535`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.792709452495718`", ",", "51.108511082957165`", ",", 
   "68.15041954061012`", ",", "73.87009072939178`", ",", "85.39680244296034`",
    ",", "90.31070755922093`", ",", "93.88756068018883`", ",", 
   "98.36163675433916`", ",", "68.15041954061012`", ",", 
   "0.6448940437461409`", ",", "0.642666011575221`", ",", 
   "0.30473955331809033`", ",", "Indeterminate", ",", "1.4332354951289628`", 
   ",", "0.6210848348721715`", ",", "0.12898359981354945`", ",", 
   "39.32136532237274`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933976736177535`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933976736177535`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933976736177535`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593397717843831*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933977287326508`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.98915`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.98915],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933977979187717`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933978534548693`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.901639964211753`", ",", "53.17723503690869`", ",", "70.102437059473`", 
   ",", "75.67349506538457`", ",", "86.63900858536908`", ",", 
   "91.18591320233146`", ",", "94.45904434457758`", ",", "98.51623278872911`",
    ",", "70.102437059473`", ",", "0.6639951244378353`", ",", 
   "0.6618894940173901`", ",", "0.30473955331809033`", ",", "Indeterminate", 
   ",", "1.47028315862291`", ",", "0.6210848348721715`", ",", 
   "0.13625010175376828`", ",", "31.49477548138397`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933978534548693`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.001`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933978534548693`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933978534548693`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.98894057094905572323996447636008976304`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.98894057094905572323996447636008976304`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593398487252783*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593398498094802*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9889705709490557`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9889705709490557],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933985673277235`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933986210698175`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.966408275385287`", ",", "55.085030582902824`", ",", "71.8562881418886`",
    ",", "77.27482932996833`", ",", "87.71036058172741`", ",", 
   "91.93049236334764`", ",", "94.93818113379974`", ",", "98.64638453917452`",
    ",", "71.8562881418886`", ",", "0.6809371922488736`", ",", 
   "0.6789399753010468`", ",", "0.30473955331809033`", ",", "Indeterminate", 
   ",", "1.5058980325733278`", ",", "0.6210848348721715`", ",", 
   "0.14467922060486316`", ",", "34.594031350611296`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933986210698175`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5933986366854453`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933988949126987`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.97015136033933`", ",", "53.58761215425743`", ",", "70.32654134524144`", 
   ",", "75.82854975712976`", ",", "86.72579905312786`", ",", 
   "91.2591516122029`", ",", "94.5155306863015`", ",", "98.53332998917452`", 
   ",", "70.32654134524144`", ",", "0.6662965244181832`", ",", 
   "0.6641963974115909`", ",", "0.3057020369791104`", ",", "Indeterminate", 
   ",", "1.4733022035523495`", ",", "0.623772118313609`", ",", 
   "0.13621270818668707`", ",", "31.6741744005557`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933988949126987`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0009000000000000001`", ",", "0.98915`", ",", "0.986`", ",", "0.9923`", 
     ",", "39.65889890525076`", ",", "10.323432830715028`", ",", 
     "1.2712828747610583`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0009000000000000001, 0.98915, 0.986, 0.9923, 39.65889890525076, 
   10.323432830715028`, 1.2712828747610583`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933988949126987`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", "28.601350235`10.385052784343554"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   28.601350235`10.385052784343554],
  Editable->False]], "Print",
 CellChangeTimes->{3.593398894928299*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5933988953182993`*^9}],

Cell[BoxData["\<\"Country: Greece\"\>"], "Print",
 CellChangeTimes->{3.5933988953338995`*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5933988955835*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5933988955835*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593398897970304*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593398897970304*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593398972678835*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593398972678835*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933989835676546`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593399000509284*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593399000509284*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.59339900372289*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59339900372289*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933990477617674`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933990477617674`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933990587753863`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9894949999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9894949999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593399200018035*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5933992536509285`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.565209774718086`", ",", "50.6516081734889`", ",", "67.81310818014397`", 
   ",", "73.6201268803099`", ",", "85.41074019959063`", ",", 
   "90.37155458944642`", ",", "93.92774443314536`", ",", "98.32976615815143`",
    ",", "67.81310818014397`", ",", "0.6427335424716148`", ",", 
   "0.6405017123345628`", ",", "0.30950429982245564`", ",", "Indeterminate", 
   ",", "1.397601554760802`", ",", "0.6294346656022581`", ",", 
   "0.11565648868794143`", ",", "167.3265708982059`"}], "}"}]], "Print",
 CellChangeTimes->{3.5933992536665287`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933992536665287`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5933992536665287`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5933992975650063`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593399297580606*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593399297580606*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593399308531825*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.593399403879193*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593399459742891*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.82119943100314`", ",", "53.02198096822563`", ",", "70.07078220113395`", 
   ",", "75.71346918031449`", ",", "86.88741190558869`", ",", 
   "91.43000770739506`", ",", "94.62132281092248`", ",", "98.51634676758044`",
    ",", "70.07078220113395`", ",", "0.6650653500142324`", ",", 
   "0.6629749480113752`", ",", "0.30950429982245564`", ",", "Indeterminate", 
   ",", "1.442884094460837`", ",", "0.6294346656022581`", ",", 
   "0.1227449094171692`", ",", "243.46140455032403`"}], "}"}]], "Print",
 CellChangeTimes->{3.593399459742891*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.593399459742891*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593399459742891*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593399503797368*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933995146705875`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9896149999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9896149999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933996080211515`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59339966416565*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.337652180593407`", ",", "48.1951302701808`", ",", "65.40244905995348`", 
   ",", "71.36196335530849`", ",", "83.78010229804471`", ",", 
   "89.18272064923458`", ",", "93.14744440844187`", ",", "98.12880160109148`",
    ",", "65.40244905995348`", ",", "0.6187270245572574`", ",", 
   "0.6163435099239183`", ",", "0.30950429982245564`", ",", "Indeterminate", 
   ",", "1.3548953737060925`", ",", "0.6294346656022581`", ",", 
   "0.10935734984373324`", ",", "102.8610576584576`"}], "}"}]], "Print",
 CellChangeTimes->{3.59339966418125*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0004999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0004999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.59339966418125*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59339966418125*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5933997082513275`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5933997191401463`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9897499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9897499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.59339983486115*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59339989191045*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.171129344300889`", ",", "45.711107988633074`", ",", 
   "62.91266844978472`", ",", "69.0168661672333`", ",", "82.06048053996366`", 
   ",", "87.93560273568488`", ",", "92.34053125177807`", ",", 
   "97.94073004140654`", ",", "62.91266844978472`", ",", 
   "0.5941037832288281`", ",", "0.5915658028233214`", ",", 
   "0.30950429982245564`", ",", "Indeterminate", ",", "1.32093692763034`", 
   ",", "0.6294346656022581`", ",", "0.10377956999962323`", ",", 
   "55.18807996193439`"}], "}"}]], "Print",
 CellChangeTimes->{3.59339989192605*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0003999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0003999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.59339989192605*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59339989192605*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593399935793327*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593399946572946*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593400063354751*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934001186100483`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.092131117893201`", ",", "43.33476835947518`", ",", "60.53661252905066`",
    ",", "66.7745172722021`", ",", "80.446768577172`", ",", 
   "86.79882850748004`", ",", "91.6359335075423`", ",", "97.81696535585291`", 
   ",", "60.53661252905066`", ",", "0.5712233053179507`", ",", 
   "0.568542977358886`", ",", "0.30950429982245564`", ",", "Indeterminate", 
   ",", "1.298909110467414`", ",", "0.6294346656022581`", ",", 
   "0.09905982434598604`", ",", "27.564972511317777`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934001186100483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0002999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0002999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934001186100483`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934001186100483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934001626645255`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934001734753447`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593400288213546*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934003447012453`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "10.974181536667963`", ",", "40.9045719286447`", ",", "58.12304759330641`", 
   ",", "64.52249277007968`", ",", "78.85370439084734`", ",", 
   "85.68263906725095`", ",", "90.93564469095152`", ",", "97.66711326368237`",
    ",", "58.12304759330641`", ",", "0.5480842045232719`", ",", 
   "0.5452587535253006`", ",", "0.30950429982245564`", ",", "Indeterminate", 
   ",", "1.2745610753128462`", ",", "0.6294346656022581`", ",", 
   "0.09790973304014472`", ",", "17.376327013774993`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934003447012453`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.00019999999999999993`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.00019999999999999993`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934003447012453`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934003447012453`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934003887245226`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593400399582142*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593400492839105*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934005507308073`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "10.05783244689266`", ",", "38.90536208597391`", ",", "56.132248969942054`",
    ",", "62.67663499947348`", ",", "77.60668105229311`", ",", 
   "84.82321454090003`", ",", "90.40402910994983`", ",", "97.55524149253291`",
    ",", "56.132248969942054`", ",", "0.5294891546584851`", ",", 
   "0.5265479434299407`", ",", "0.30950429982245564`", ",", "Indeterminate", 
   ",", "1.2574571470772273`", ",", "0.6294346656022581`", ",", 
   "0.09712910899381166`", ",", "21.670877845838234`"}], "}"}]], "Print",
 CellChangeTimes->{3.593400550746407*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593400566377634*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593400833309703*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "10.930178394468093`", ",", "40.799414191338585`", ",", 
   "57.97780857630658`", ",", "64.378850002664`", ",", "78.72769652550991`", 
   ",", "85.57590239215489`", ",", "90.85227584156628`", ",", 
   "97.64700257848395`", ",", "57.97780857630658`", ",", 
   "0.5465204778071754`", ",", "0.5436734951605559`", ",", 
   "0.30458609364233435`", ",", "Indeterminate", ",", "1.2731058121077679`", 
   ",", "0.6240761714318745`", ",", "0.09807114142002593`", ",", 
   "17.68443610818872`"}], "}"}]], "Print",
 CellChangeTimes->{3.593400833309703*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0002999999999999999`", ",", "0.9898999999999999`", ",", 
     "0.9888499999999999`", ",", "0.9909499999999999`", ",", 
     "40.29291440768115`", ",", "10.428755244201476`", ",", 
     "2.58437640802056`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0002999999999999999, 0.9898999999999999, 0.9888499999999999, 
   0.9909499999999999, 40.29291440768115, 10.428755244201476`, 
   2.58437640802056}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593400833309703*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "32.29985673166666666666666666666666666667`10.437866844206654"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   32.29985673166666666666666666666666666667`10.437866844206654],
  Editable->False]], "Print",
 CellChangeTimes->{3.593400833309703*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593400833746504*^9}],

Cell[BoxData["\<\"Country: Italy\"\>"], "Print",
 CellChangeTimes->{3.593400833746504*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5934008340117044`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934008340117044`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593400836351709*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593400836351709*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.59340091101344*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.59340091101344*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934009218554587`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593400939062289*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5934009390778894`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.593400942291495*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593400942291495*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593400986111972*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593400986111972*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934009968915906`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989225`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989225],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401066810913*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593401124453015*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.124258676721137`", ",", "46.162165869707`", ",", "63.09102661831746`", 
   ",", "69.07700737782446`", ",", "81.95135037504761`", ",", 
   "87.84775130260786`", ",", "92.3003370253977`", ",", "97.98884260460731`", 
   ",", "63.09102661831746`", ",", "0.5952074750420957`", ",", 
   "0.5926590438848489`", ",", "0.3039974841061279`", ",", "Indeterminate", 
   ",", "1.5080160391798316`", ",", "0.626310304951054`", ",", 
   "0.13648738555351403`", ",", "21.99544978844008`"}], "}"}]], "Print",
 CellChangeTimes->{3.593401124453015*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401124453015*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593401124453015*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593401168398292*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593401168413892*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401168413892*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934011793807116`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.98909`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.98909],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401250423236*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934013088141384`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.116026289475329`", ",", "48.148023845102315`", ",", 
   "64.96126121499738`", ",", "70.79722789145667`", ",", "83.13698907388117`",
    ",", "88.69766488062155`", ",", "92.86132413835062`", ",", 
   "98.13814601368792`", ",", "64.96126121499738`", ",", 
   "0.6135043072733255`", ",", "0.6110732993095392`", ",", 
   "0.3039974841061279`", ",", "Indeterminate", ",", "1.5347581001464123`", 
   ",", "0.626310304951054`", ",", "0.1417629228146784`", ",", 
   "24.97791315709226`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934013088141384`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934013088141384`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934013088141384`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401352853016*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593401363741835*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9893449999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9893449999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401456686798*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934015120200953`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.157850491709837`", ",", "44.13955982700332`", ",", "61.16071168386156`",
    ",", "67.29384696916809`", ",", "80.71448756253959`", ",", 
   "86.95372920546728`", ",", "91.71299557645713`", ",", "97.83983249584224`",
    ",", "61.16071168386156`", ",", "0.5763671930941698`", ",", 
   "0.573697406164688`", ",", "0.3039974841061279`", ",", "Indeterminate", 
   ",", "1.484079913620631`", ",", "0.626310304951054`", ",", 
   "0.13191273693058636`", ",", "30.033124568931793`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934015120200953`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593401527635723*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934017937877903`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.743242310758832`", ",", "46.57109882650859`", ",", "63.43408076950405`",
    ",", "69.39323615566573`", ",", "82.16720970388583`", ",", 
   "87.98344601644979`", ",", "92.37712190872223`", ",", "98.0073590379569`", 
   ",", "63.43408076950405`", ",", "0.5989351747120342`", ",", 
   "0.596405164123699`", ",", "0.30739024636225515`", ",", "Indeterminate", 
   ",", "1.511318893694447`", ",", "0.6246131008857908`", ",", 
   "0.1364763524782342`", ",", "21.98529504606081`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934017937877903`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0007`", ",", "0.989225`", ",", "0.9867750000000001`", ",", "0.991675`",
      ",", "39.26964208849894`", ",", "10.258469286129676`", ",", 
     "0.3701437032199215`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0007, 
   0.989225, 0.9867750000000001, 0.991675, 39.26964208849894, 
   10.258469286129676`, 0.3701437032199215}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934017937877903`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "16.00068810333333333333333333333333333333`10.132804907937482"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   16.00068810333333333333333333333333333333`10.132804907937482],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934017938033905`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593401794177791*^9}],

Cell[BoxData["\<\"Country: Luxembourg\"\>"], "Print",
 CellChangeTimes->{3.593401794193391*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.593401794427391*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593401794427391*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593401796829796*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401796829796*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934018710547256`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934018710547256`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401881881145*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593401898994375*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593401898994375*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401902207981*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593401902207981*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593401946059658*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593401946059658*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593401956995277*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9894949999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9894949999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593402073589882*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593402127331976*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.704829182955859`", ",", "50.76154050691605`", ",", "67.86899005742309`",
    ",", "73.68814634758746`", ",", "85.43059404413367`", ",", 
   "90.38676236144495`", ",", "93.93687933413332`", ",", "98.32473818380143`",
    ",", "67.86899005742309`", ",", "0.6434093226578892`", ",", 
   "0.6411745117805436`", ",", "0.3066530697761206`", ",", "Indeterminate", 
   ",", "1.3992519519378082`", ",", "0.6265172269132824`", ",", 
   "0.11603760870839774`", ",", "22.791715208823586`"}], "}"}]], "Print",
 CellChangeTimes->{3.593402127331976*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593402127331976*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593402127331976*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593402171168053*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593402171183653*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593402171183653*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934021821660724`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934022774822397`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593402332035536*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.88833751449603`", ",", "53.14361460162198`", ",", "70.12233608851642`", 
   ",", "75.76968919406191`", ",", "86.89547912072483`", ",", 
   "91.44063360032682`", ",", "94.63033088786972`", ",", "98.5157919011585`", 
   ",", "70.12233608851642`", ",", "0.6656679850610266`", ",", 
   "0.6635754112362547`", ",", "0.3066530697761206`", ",", "Indeterminate", 
   ",", "1.444815953356832`", ",", "0.6265172269132824`", ",", 
   "0.12313603247998506`", ",", "12.87017068319334`"}], "}"}]], "Print",
 CellChangeTimes->{3.593402332035536*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009000000000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.593402332035536*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593402332035536*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934023762928133`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934023871816325`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9891949999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9891949999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934024803449965`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934025379558973`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.956059104073937`", ",", "55.2859406626226`", ",", "72.12683263109247`", 
   ",", "77.60152877475699`", ",", "88.13781345533053`", ",", 
   "92.30284410521834`", ",", "95.1826754210596`", ",", "98.66577214473241`", 
   ",", "72.12683263109247`", ",", "0.6849362031588845`", ",", 
   "0.6829666343140003`", ",", "0.3066530697761206`", ",", "Indeterminate", 
   ",", "1.4870590520220288`", ",", "0.6265172269132824`", ",", 
   "0.13256639723123564`", ",", "17.35142675015556`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934025379558973`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5934025535715246`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934028203631935`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.49194052196925`", ",", "53.419362648216115`", ",", "70.22885987514906`",
    ",", "75.81002258055592`", ",", "86.88724246842445`", ",", 
   "91.4194702096055`", ",", "94.61290776187431`", ",", "98.51230642107075`", 
   ",", "70.22885987514906`", ",", "0.6669522811528827`", ",", 
   "0.6648436382870802`", ",", "0.30665627699852616`", ",", "Indeterminate", 
   ",", "1.4399297059312268`", ",", "0.6270474494025664`", ",", 
   "0.12291891122303189`", ",", "13.225838856183088`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934028203631935`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0008`", ",", "0.989375`", ",", "0.986575`", ",", "0.992175`", ",", 
     "39.82879390917546`", ",", "10.35171480351054`", ",", 
     "1.447276500064733`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0008, 
   0.989375, 0.986575, 0.992175, 39.82879390917546, 10.35171480351054, 
   1.447276500064733}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934028203631935`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", "17.10309004`10.161740830308613"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   17.10309004`10.161740830308613],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934028203631935`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593402820815594*^9}],

Cell[BoxData["\<\"Country: Malta\"\>"], "Print",
 CellChangeTimes->{3.593402820815594*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5934028210651946`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934028210651946`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593402823498799*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593402823498799*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.59340289814493*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.59340289814493*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593402909064949*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593402926115779*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593402926115779*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934029293137846`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934029293137846`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934029730094614`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934029730094614`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934029838514805`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9894949999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9894949999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593403105344494*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934031607401915`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.416354910482873`", ",", "51.24259769274069`", ",", "68.14657506890316`",
    ",", "73.86982373402434`", ",", "85.47703294592198`", ",", 
   "90.38410848331824`", ",", "93.9262424512922`", ",", "98.32570482472615`", 
   ",", "68.14657506890316`", ",", "0.6459899309945276`", ",", 
   "0.6437436731211587`", ",", "0.3042868648871089`", ",", "Indeterminate", 
   ",", "1.3937818560548088`", ",", "0.6292115418733614`", ",", 
   "0.11565442398443315`", ",", "47.27703644793442`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934031608181915`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934031610989923`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593403161145792*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.59340320543427*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.59340320559027*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.59340320565267*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934032165882893`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934033097516527`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934033640553484`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.623238947001376`", ",", "53.58690328709539`", ",", "70.39841489018072`",
    ",", "75.96737915770981`", ",", "86.96368116050198`", ",", 
   "91.44810812045621`", ",", "94.62338753505486`", ",", "98.51545629997436`",
    ",", "70.39841489018072`", ",", "0.668282156486828`", ",", 
   "0.6661787193528542`", ",", "0.3042868648871089`", ",", "Indeterminate", 
   ",", "1.4401208634661347`", ",", "0.6292115418733614`", ",", 
   "0.12277627457319684`", ",", "92.34145649575348`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934033640865483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934033641489487`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934033641489487`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593403408281426*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934034192170453`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9896149999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9896149999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593403512442809*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934035682753067`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.209579707646103`", ",", "48.78604507186732`", ",", "65.72107882497532`",
    ",", "71.59179665964638`", ",", "83.8255047442467`", ",", 
   "89.18619721584645`", ",", "93.13777010783099`", ",", "98.11753587921682`",
    ",", "65.72107882497532`", ",", "0.6218233926899827`", ",", 
   "0.6194230219550401`", ",", "0.3042868648871089`", ",", "Indeterminate", 
   ",", "1.3502005764384395`", ",", "0.6292115418733614`", ",", 
   "0.10935086026705476`", ",", "15.795047734184125`"}], "}"}]], "Print",
 CellChangeTimes->{3.593403568306507*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0004999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0004999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593403568368907*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593403568368907*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934036127041855`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934036237022047`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9897499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9897499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593403739470008*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593403796644108*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.035383368250093`", ",", "46.28779935942179`", ",", "63.19995763899876`",
    ",", "69.20492503597738`", ",", "82.08776134354866`", ",", 
   "87.92941112379746`", ",", "92.31905439916164`", ",", "97.92573617194363`",
    ",", "63.19995763899876`", ",", "0.5968821903361934`", ",", 
   "0.5943235094530236`", ",", "0.3042868648871089`", ",", "Indeterminate", 
   ",", "1.3158372724905298`", ",", "0.6292115418733614`", ",", 
   "0.10380988704082417`", ",", "2.6423428566930562`"}], "}"}]], "Print",
 CellChangeTimes->{3.593403796644108*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0003999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0003999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593403797018509*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593403797034109*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593403840916986*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593403851790205*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934039664816065`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593404024856909*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.901187868956262`", ",", "43.8821064511719`", ",", "60.769943799449656`",
    ",", "66.91603307538632`", ",", "80.46380281232844`", ",", 
   "86.78251587511397`", ",", "91.60247596793005`", ",", "97.79887565864924`",
    ",", "60.769943799449656`", ",", "0.5736171255913626`", ",", 
   "0.5709124167564222`", ",", "0.3042868648871089`", ",", "Indeterminate", 
   ",", "1.2941035206406968`", ",", "0.6292115418733614`", ",", 
   "0.09911683427409279`", ",", "8.86483277611525`"}], "}"}]], "Print",
 CellChangeTimes->{3.593404024888109*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5934040404881363`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934043067026043`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.696256087160839`", ",", "45.96186016903216`", ",", "63.00496346350026`",
    ",", "69.07606767514125`", ",", "82.07329119514768`", ",", 
   "87.94183139800461`", ",", "92.33097270375403`", ",", "97.93270395323698`",
    ",", "63.00496346350026`", ",", "0.5954044805870689`", ",", 
   "0.5928441236611122`", ",", "0.3054554077127643`", ",", "Indeterminate", 
   ",", "1.3176108099539432`", ",", "0.622261942339956`", ",", 
   "0.10391504834268872`", ",", "2.794824131095527`"}], "}"}]], "Print",
 CellChangeTimes->{3.593404306718204*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0004999999999999999`", ",", "0.9897499999999999`", ",", 
     "0.9879999999999999`", ",", "0.9914999999999999`", ",", 
     "39.9976019635113`", ",", "10.379772852320913`", ",", 
     "1.8004274282626478`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0004999999999999999, 0.9897499999999999, 0.9879999999999999, 
   0.9914999999999999, 39.9976019635113, 10.379772852320913`, 
   1.8004274282626478`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593404306765004*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "24.75958348833333333333333333333333333333`10.322409582818963"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   24.75958348833333333333333333333333333333`10.322409582818963],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934043067806044`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5934043068742046`*^9}],

Cell[BoxData["\<\"Country: Netherland\"\>"], "Print",
 CellChangeTimes->{3.5934043069366045`*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.593404307279805*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593404307279805*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593404309651009*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934043096666093`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593404384687141*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593404384687141*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.59340439557596*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5934044129387903`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5934044129387903`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934044161679964`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934044161679964`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593404459785673*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934044601132736`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593404470674492*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593404564758257*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934046190463524`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.385146753810558`", ",", "50.272000741410686`", ",", 
   "67.63447598649903`", ",", "73.49008368809663`", ",", "85.33317773885885`",
    ",", "90.34264637891586`", ",", "93.9313675966676`", ",", 
   "98.33901261251573`", ",", "67.63447598649903`", ",", 
   "0.6403655726372021`", ",", "0.6381194662537153`", ",", 
   "0.3007373223294215`", ",", "Indeterminate", ",", "1.4031357482772489`", 
   ",", "0.6146606506648209`", ",", "0.11306840551166575`", ",", 
   "144.7396020201777`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934046194051533`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593404619436353*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593404619436353*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934046632256303`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593404663693631*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593404663693631*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934046747072506`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.593404767730214*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593404824327113*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.45094021831046`", ",", "52.49240561843788`", ",", "69.80284185271245`", 
   ",", "75.49135379102997`", ",", "86.73126325114028`", ",", 
   "91.32270572489332`", ",", "94.55831303666687`", ",", "98.50054868910321`",
    ",", "69.80284185271245`", ",", "0.6612339743605504`", ",", 
   "0.6591176612819589`", ",", "0.3007373223294215`", ",", "Indeterminate", 
   ",", "1.4439336651803754`", ",", "0.6146606506648209`", ",", 
   "0.12303856504292665`", ",", "148.71013598349253`"}], "}"}]], "Print",
 CellChangeTimes->{3.593404824389513*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.593404824389513*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593404824389513*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593404868974392*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593404879878811*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9896299999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9896299999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934049732917747`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593405029015073*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.042267711110933`", ",", "47.60169505894239`", ",", "65.06191650777838`",
    ",", "71.10036971685945`", ",", "83.6121937846608`", ",", 
   "89.08828695955395`", ",", "93.09179667535939`", ",", "98.10975006390481`",
    ",", "65.06191650777838`", ",", "0.6146212383455733`", ",", 
   "0.6122127604917265`", ",", "0.3007373223294215`", ",", "Indeterminate", 
   ",", "1.3564217880409302`", ",", "0.6146606506648209`", ",", 
   "0.10921833744996812`", ",", "161.28044435391402`"}], "}"}]], "Print",
 CellChangeTimes->{3.593405029030673*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593405044973901*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934053055411587`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.497238985010894`", ",", "50.784190928397166`", ",", 
   "67.83011808410298`", ",", "73.61959705437148`", ",", "85.38273037294826`",
    ",", "90.36046510237651`", ",", "93.93547022369064`", ",", 
   "98.34055781709532`", ",", "67.83011808410298`", ",", 
   "0.6431005881811576`", ",", "0.6408605043539617`", ",", 
   "0.30594905707451653`", ",", "Indeterminate", ",", "1.4002213713436045`", 
   ",", "0.6253230747589799`", ",", "0.11294477647708845`", ",", 
   "146.1350838583679`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934053055411587`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0007`", ",", "0.9895999999999999`", ",", "0.98715`", ",", 
     "0.9920499999999999`", ",", "41.31577012378442`", ",", 
     "10.597422872991922`", ",", "5.148824729790583`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0007, 
   0.9895999999999999, 0.98715, 0.9920499999999999, 41.31577012378442, 
   10.597422872991922`, 5.148824729790583}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405305946759*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", "16.644449235`10.14993567713711"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   16.644449235`10.14993567713711],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405305977959*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593405306071559*^9}],

Cell[BoxData["\<\"Country: Portugal\"\>"], "Print",
 CellChangeTimes->{3.593405306071559*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5934053063367596`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.59340530635236*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593405308723564*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405308723564*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593405383182495*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405383198095*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405394164914*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5934054116213446`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5934054116213446`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934054148193502`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934054148193502`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934054587022276`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934054587022276`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934054696066465`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934055629572105`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593405618025307*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.765491491671574`", ",", "51.32680272553362`", ",", "68.18787508426367`",
    ",", "73.88450374730589`", ",", "85.4980123759055`", ",", 
   "90.43328458044624`", ",", "93.97444260531202`", ",", "98.34824426301786`",
    ",", "68.18787508426367`", ",", "0.6461636664281755`", ",", 
   "0.6439266334895213`", ",", "0.3048378619926592`", ",", "Indeterminate", 
   ",", "1.4004033067813126`", ",", "0.6227958420991739`", ",", 
   "0.11315602005408654`", ",", "16.581123170558577`"}], "}"}]], "Print",
 CellChangeTimes->{3.593405618025307*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405618384108*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593405618384108*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934056620953846`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934056624541855`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405662469785*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934056734522047`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405767255169*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934058231500673`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.805139646960914`", ",", "53.54697857896357`", ",", "70.34263923916498`",
    ",", "75.89945143151216`", ",", "86.8915170533824`", ",", 
   "91.41427563929253`", ",", "94.6071737543139`", ",", "98.50993174045438`", 
   ",", "70.34263923916498`", ",", "0.6670239315215024`", ",", 
   "0.6649183874349069`", ",", "0.3048378619926592`", ",", "Indeterminate", 
   ",", "1.4408172558436687`", ",", "0.6227958420991739`", ",", 
   "0.12308200071971886`", ",", "36.15307208428117`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934058231500673`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934058231500673`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934058231500673`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593405867563346*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934058783741646`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9896299999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9896299999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934059926755652`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934060478840623`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.45580574697716`", ",", "48.646330867325304`", ",", "65.61634384132647`",
    ",", "71.49090111565017`", ",", "83.78935305492952`", ",", 
   "89.17738371471756`", ",", "93.13395507311395`", ",", "98.12082819673672`",
    ",", "65.61634384132647`", ",", "0.620484834038045`", ",", 
   "0.6180831525096656`", ",", "0.3048378619926592`", ",", "Indeterminate", 
   ",", "1.3543295470980725`", ",", "0.6227958420991739`", ",", 
   "0.10932984556626324`", ",", "12.252498585690034`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934060478840623`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0004999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0004999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593406048211663*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934060482272625`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934060923129406`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934061032329597`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9897499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9897499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593406219063163*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593406274708461*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.29364085208245`", ",", "46.0504932885593`", ",", "63.062395533816954`", 
   ",", "69.10234738358886`", ",", "82.05677655221204`", ",", 
   "87.91376807518606`", ",", "92.30894719397426`", ",", "97.92916233282668`",
    ",", "63.062395533816954`", ",", "0.5952899813770096`", ",", 
   "0.5927288906152488`", ",", "0.3048378619926592`", ",", "Indeterminate", 
   ",", "1.3191938732051`", ",", "0.6227958420991739`", ",", 
   "0.10408579195266421`", ",", "28.314048979667763`"}], "}"}]], "Print",
 CellChangeTimes->{3.593406274708461*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593406290355288*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593406555228153*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.981013321084628`", ",", "48.15541099346633`", ",", "65.34475034889189`",
    ",", "71.29800369327195`", ",", "83.69453727674373`", ",", 
   "89.12270318538357`", ",", "93.1091296391898`", ",", "98.11611436921936`", 
   ",", "65.34475034889189`", ",", "0.617763075465837`", ",", 
   "0.6153619025683843`", ",", "0.30433502280301933`", ",", "Indeterminate", 
   ",", "1.3548296607768662`", ",", "0.6240375587274245`", ",", 
   "0.10912551100100054`", ",", "12.361801548770071`"}], "}"}]], "Print",
 CellChangeTimes->{3.593406555571354*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0006`", ",", "0.9896299999999999`", ",", "0.9875299999999999`", ",", 
     "0.9917299999999999`", ",", "39.51075373377305`", ",", 
     "10.2987358779973`", ",", "0.8363411320220178`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0006, 
   0.9896299999999999, 0.9875299999999999, 0.9917299999999999, 
   39.51075373377305, 10.2987358779973, 0.8363411320220178}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593406555633754*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "20.81927656666666666666666666666666666667`10.247131882681057"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   20.81927656666666666666666666666666666667`10.247131882681057],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934065557897544`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5934065557897544`*^9}],

Cell[BoxData["\<\"Country: Slovenia\"\>"], "Print",
 CellChangeTimes->{3.593406555805354*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.593406556039355*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593406556039355*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593406558472959*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593406558472959*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934066331814904`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.59340663319709*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934066441951094`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.59340666168274*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.59340666168274*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.593406664880746*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593406664880746*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934067084984226`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934067088416233`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593406719418442*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9894799999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9894799999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593406837339049*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934068918767443`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.861748012743575`", ",", "52.10758027688388`", ",", "68.62683436922063`",
    ",", "74.2537137576858`", ",", "85.6945546242245`", ",", 
   "90.53370529937118`", ",", "94.01798859389876`", ",", "98.34237853000758`",
    ",", "68.62683436922063`", ",", "0.6524662671323633`", ",", 
   "0.650226674802487`", ",", "0.3171764923824781`", ",", "Indeterminate", 
   ",", "1.3879143586596376`", ",", "0.6242424378402551`", ",", 
   "0.11633381610013138`", ",", "160.18628809279355`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934068918767443`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934068918923445`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593406891923545*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934069361340227`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593406936165222*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593406936165222*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934069471164417`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989335`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989335],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934070416058073`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934070962371035`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.98238451546509`", ",", "54.34022555306035`", ",", "70.77254170526834`", 
   ",", "76.24672432138419`", ",", "87.10866912435566`", ",", 
   "91.54241445235087`", ",", "94.67691424618239`", ",", "98.52010419896592`",
    ",", "70.77254170526834`", ",", "0.6736532599921812`", ",", 
   "0.6715476742500281`", ",", "0.3171764923824781`", ",", "Indeterminate", 
   ",", "1.431819323092944`", ",", "0.6242424378402551`", ",", 
   "0.12423624320465787`", ",", "232.24711714941455`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934070962371035`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934070965959044`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934070966115046`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593407140790782*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934071517420015`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934072453109655`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934073015958643`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.699639415910244`", ",", "49.72941150601833`", ",", "66.29484782021159`",
    ",", "72.06148909925169`", ",", "84.09822608324346`", ",", 
   "89.37786589719228`", ",", "93.254930690653`", ",", "98.14202021520104`", 
   ",", "66.29484782021159`", ",", "0.6291181063182967`", ",", 
   "0.6267311734010352`", ",", "0.3171764923824781`", ",", "Indeterminate", 
   ",", "1.345784922977703`", ",", "0.6242424378402551`", ",", 
   "0.10997335833962985`", ",", "97.40864002496279`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934073015958643`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0004999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0004999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934073019546647`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934073019546647`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593407345899942*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593407356788761*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9897499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9897499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934074706533613`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934075265482597`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.608765398350936`", ",", "47.366755809502315`", ",", 
   "63.906959371360394`", ",", "69.8008129882521`", ",", "82.44209247289936`",
    ",", "88.18067065285832`", ",", "92.47490994134881`", ",", 
   "97.96543444611144`", ",", "63.906959371360394`", ",", 
   "0.6054772298913556`", ",", "0.6029414756409359`", ",", 
   "0.3171764923824781`", ",", "Indeterminate", ",", "1.3126460042022448`", 
   ",", "0.6242424378402551`", ",", "0.10398661874346038`", ",", 
   "50.76853295490286`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934075265482597`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0003999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0003999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.59340752692266*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59340752692266*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934075711643376`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593407582084357*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9897799999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9897799999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934076974777594`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934077533414574`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.33931580107549`", ",", "44.8004810372762`", ",", "61.35069904103333`", 
   ",", "67.38531920840147`", ",", "80.69042415524571`", ",", 
   "86.91042400100942`", ",", "91.65015102444511`", ",", "97.77974486429885`",
    ",", "61.35069904103333`", ",", "0.5801465549941974`", ",", 
   "0.577452158513179`", ",", "0.3171764923824781`", ",", "Indeterminate", 
   ",", "1.2812079963946166`", ",", "0.6242424378402551`", ",", 
   "0.10158852172515176`", ",", "20.967966627765538`"}], "}"}]], "Print",
 CellChangeTimes->{3.593407753372658*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0002999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0002999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593407753435058*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593407753435058*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934077977235355`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934078086123548`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934079245205584`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593407980212656*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.328718406279965`", ",", "42.69473673961428`", ",", "59.23604343398724`",
    ",", "65.41165347610347`", ",", "79.33354234618895`", ",", 
   "85.9699401179475`", ",", "91.09080804139722`", ",", "97.6960857902652`", 
   ",", "59.23604343398724`", ",", "0.5604400013929904`", ",", 
   "0.5576238858766817`", ",", "0.3171764923824781`", ",", "Indeterminate", 
   ",", "1.2675968924023255`", ",", "0.6242424378402551`", ",", 
   "0.09809477105791942`", ",", "11.015282291580458`"}], "}"}]], "Print",
 CellChangeTimes->{3.593407980212656*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.00019999999999999993`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.00019999999999999993`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593407980212656*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593407980212656*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408024423134*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593408035343153*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408151578957*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593408207271055*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.306526626002372`", ",", "40.68485247972843`", ",", 
   "57.296829096551924`", ",", "63.621346194490016`", ",", 
   "78.10200323319526`", ",", "85.10921319008918`", ",", "90.55881379683326`",
    ",", "97.58718188359732`", ",", "57.296829096551924`", ",", 
   "0.5420198576191552`", ",", "0.5390911808812757`", ",", 
   "0.3171764923824781`", ",", "Indeterminate", ",", "1.250766977707935`", 
   ",", "0.6242424378402551`", ",", "0.09730779973648385`", ",", 
   "13.724882759676662`"}], "}"}]], "Print",
 CellChangeTimes->{3.593408207271055*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593408223073883*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593408489693951*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "10.748474820212603`", ",", "40.69387950067541`", ",", "57.90826138781987`",
    ",", "64.32324846184837`", ",", "78.71277414879869`", ",", 
   "85.56331777507474`", ",", "90.8434325412175`", ",", "97.64310681708639`", 
   ",", "57.90826138781987`", ",", "0.5457946561365125`", ",", 
   "0.5429366148247277`", ",", "0.30511172093244526`", ",", "Indeterminate", 
   ",", "1.2734276446198711`", ",", "0.6232656485599716`", ",", 
   "0.09802075286208445`", ",", "10.47013474802247`"}], "}"}]], "Print",
 CellChangeTimes->{3.593408489693951*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0002999999999999999`", ",", "0.9898999999999999`", ",", 
     "0.9888499999999999`", ",", "0.9909499999999999`", ",", 
     "40.78585820420136`", ",", "10.510231118094355`", ",", 
     "4.083234624824825`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0002999999999999999, 0.9898999999999999, 0.9888499999999999, 
   0.9909499999999999, 40.78585820420136, 10.510231118094355`, 
   4.083234624824825}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408490193152*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "32.23173661333333333333333333333333333333`10.436949953705307"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   32.23173661333333333333333333333333333333`10.436949953705307],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408490239952*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593408490302352*^9}],

Cell[BoxData["\<\"Country: Slovakia\"\>"], "Print",
 CellChangeTimes->{3.593408490302352*^9}],

Cell[BoxData["\<\"Estimate \[Beta]middle using \[Beta]-Dist model with time \
pref factors approximated by seven points\"\>"], "Print",
 CellChangeTimes->{3.5934084905675526`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934084905675526`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593408492969957*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408492969957*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934085678812885`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934085678812885`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Evaluate with good guess of \[Beta]middle =  \"\>", 
   "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" Evaluate with good guess of \[Beta]middle =  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408578738908*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593408596023738*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593408596023738*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0007],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408599237344*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593408599237344*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593408643385421*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408643385421*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934086543366404`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9895999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9895999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408748919606*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593408803410502*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.368828960943542`", ",", "50.117502528459454`", ",", 
   "67.45163224545486`", ",", "73.33123613594395`", ",", "85.27696486975945`",
    ",", "90.29874652001584`", ",", "93.88908656729915`", ",", 
   "98.31589844586698`", ",", "67.45163224545486`", ",", 
   "0.6389352514806255`", ",", "0.6367041536277565`", ",", 
   "0.3044284600826619`", ",", "Indeterminate", ",", "1.399554701623642`", 
   ",", "0.6162804279202441`", ",", "0.11274085151324664`", ",", 
   "572.9742482779524`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934088034261017`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0008],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408803800503*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593408803800503*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.989`}\\)\[NoBreak] is at the \
edge of the search region \[NoBreak]\\!\\({0.985`10., 0.989`10.}\\)\[NoBreak] \
in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed search \
direction points outside the region. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", \
ButtonNote -> \\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593408847464979*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5934088474961796`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934088474961796`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934088583381987`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.989375`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.989375],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408929380723*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934089849324207`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.329265096292676`", ",", "52.295641051768186`", ",", "69.5919935381831`",
    ",", "75.31951209393941`", ",", "86.66755010787102`", ",", 
   "91.28860438484621`", ",", "94.5298264533023`", ",", "98.48188082788839`", 
   ",", "69.5919935381831`", ",", "0.6596546498515761`", ",", 
   "0.657553003620443`", ",", "0.3044284600826619`", ",", "Indeterminate", 
   ",", "1.4403341557015596`", ",", "0.6162804279202441`", ",", 
   "0.12261167871651713`", ",", "705.6281072709165`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934089849324207`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0006],
  Editable->False]], "Print",
 CellChangeTimes->{3.593408984948021*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593408984948021*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593409029345699*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593409040234518*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9896299999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9896299999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934091326178803`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59340918913678*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.154582114264088`", ",", "47.51481827468076`", ",", "64.91706534523442`",
    ",", "70.96399532579983`", ",", "83.55371176007516`", ",", 
   "89.03247233546597`", ",", "93.04062208118287`", ",", "98.0848201147363`", 
   ",", "64.91706534523442`", ",", "0.6134940123742737`", ",", 
   "0.6111006259814498`", ",", "0.3044284600826619`", ",", "Indeterminate", 
   ",", "1.3529742656971757`", ",", "0.6162804279202441`", ",", 
   "0.10894482898975599`", ",", "430.79198427296376`"}], "}"}]], "Print",
 CellChangeTimes->{3.59340918913678*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0004999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0004999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934091894643803`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934091894643803`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934092332536573`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934092441112757`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9897499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9897499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934093580070763`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593409415586777*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.10857011147518`", ",", "45.02111082629604`", ",", "62.403049854264516`",
    ",", "68.59072384771368`", ",", "81.81758819546602`", ",", 
   "87.76723798421676`", ",", "92.21547918053487`", ",", "97.89489592737118`",
    ",", "62.403049854264516`", ",", "0.5886858995645612`", ",", 
   "0.5861353737446074`", ",", "0.3044284600826619`", ",", "Indeterminate", 
   ",", "1.3183310464468245`", ",", "0.6162804279202441`", ",", 
   "0.10375429582059159`", ",", "309.55287015177447`"}], "}"}]], "Print",
 CellChangeTimes->{3.593409415586777*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0003999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0003999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593409415929978*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593409415929978*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593409459672455*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934094704988737`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934095863758774`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593409642021175*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "11.149603864980044`", ",", "42.680732430791345`", ",", 
   "60.03319732113218`", ",", "66.37600881706398`", ",", "80.22369642928153`",
    ",", "86.63312377122962`", ",", "91.50631585225315`", ",", 
   "97.77474062831887`", ",", "60.03319732113218`", ",", 
   "0.5660581428733505`", ",", "0.5633643966508494`", ",", 
   "0.3044284600826619`", ",", "Indeterminate", ",", "1.2970371399358271`", 
   ",", "0.6162804279202441`", ",", "0.09905670432704539`", ",", 
   "215.17024736306996`"}], "}"}]], "Print",
 CellChangeTimes->{3.593409642021175*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0002999999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0002999999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934096423643756`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934096423799753`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593409686450053*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593409697307672*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9898999999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9898999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593409812732275*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593409870857977*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "10.179211286953032`", ",", "40.30347991436418`", ",", "57.70482968310804`",
    ",", "64.18753166011318`", ",", "78.6487438902844`", ",", 
   "85.51090992008422`", ",", "90.80572248429476`", ",", "97.62721649091617`",
    ",", "57.70482968310804`", ",", "0.543417780632953`", ",", 
   "0.5405786422744834`", ",", "0.3044284600826619`", ",", "Indeterminate", 
   ",", "1.2725178945999043`", ",", "0.6162804279202441`", ",", 
   "0.09791001631158336`", ",", "139.1315021536364`"}], "}"}]], "Print",
 CellChangeTimes->{3.593409870857977*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.00019999999999999993`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.00019999999999999993`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593409870857977*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593409870857977*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593409914803254*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593409925629673*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9899149999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9899149999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934100424582787`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934101010207815`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "9.42730982395502`", ",", "38.44118946117468`", ",", "55.87128287176522`", 
   ",", "62.46463774255424`", ",", "77.45099640524612`", ",", 
   "84.69167545893114`", ",", "90.30738686759105`", ",", "97.5312847556599`", 
   ",", "55.87128287176522`", ",", "0.526087387619605`", ",", 
   "0.5231366780562351`", ",", "0.3044284600826619`", ",", "Indeterminate", 
   ",", "1.2570588612348959`", ",", "0.6162804279202441`", ",", 
   "0.09687793043443405`", ",", "92.00787725120406`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934101010363812`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.00009999999999999992`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.00009999999999999992],
  Editable->False]], "Print",
 CellChangeTimes->{3.593410101067581*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934101010831814`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934101450440583`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934101558548775`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9899449999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9899449999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934102954127226`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934103534604244`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "8.970634655341323`", ",", "37.30909311585219`", ",", "54.75182622713335`", 
   ",", "61.43288104747128`", ",", "76.77556998102406`", ",", 
   "84.25130814996585`", ",", "90.05135184471023`", ",", "97.48930531953913`",
    ",", "54.75182622713335`", ",", "0.5160523125284504`", ",", 
   "0.51303660596647`", ",", "0.3044284600826619`", ",", "Indeterminate", ",",
    "1.2503290000308216`", ",", "0.6162804279202441`", ",", 
   "0.09591767440888055`", ",", "69.13288778222103`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934103534604244`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "8.131516293641283`*^-20"}]}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", -8.131516293641283*^-20],
  Editable->False]], "Print",
 CellChangeTimes->{3.593410353491625*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934103539596252`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.593410397436902*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934104082321205`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9899749999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9899749999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934105474779654`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934106039968643`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "8.874334653178023`", ",", "36.98535324272078`", ",", "54.44856440067459`", 
   ",", "61.16220915468023`", ",", "76.62873724803772`", ",", 
   "84.17001462678203`", ",", "90.01502855666715`", ",", "97.49074405249144`",
    ",", "54.44856440067459`", ",", "0.5136750778407684`", ",", 
   "0.5106433859105489`", ",", "0.3044284600826619`", ",", "Indeterminate", 
   ",", "1.2505680517868323`", ",", "0.6162804279202441`", ",", 
   "0.09528334783817205`", ",", "64.15005401437594`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934106039968643`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "0.00010000000000000009`"}]}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", -0.00010000000000000009`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593410604340065*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593410604340065*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", "0.989`"}],
  SequenceForm[" \[Beta]middle:  ", 0.989],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934106481605425`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593410658955761*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9899449999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9899449999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593410798513606*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934108555785065`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "9.078153267433416`", ",", "37.309149045488795`", ",", "54.73122850342181`",
    ",", "61.41197174504658`", ",", "76.78548592768259`", ",", 
   "84.26629398524547`", ",", "90.06147072679612`", ",", "97.49067665284196`",
    ",", "54.73122850342181`", ",", "0.5162067116727389`", ",", 
   "0.513189985071595`", ",", "0.3044284600826619`", ",", "Indeterminate", 
   ",", "1.2503127941956818`", ",", "0.6162804279202441`", ",", 
   "0.09594537914718582`", ",", "69.08298890343197`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934108555785065`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5934108714749346`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593411134397796*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "9.501240038551206`", ",", "37.61071430993495`", ",", "54.900257362220295`",
    ",", "61.559333836831186`", ",", "76.87893049593872`", ",", 
   "84.3294144718114`", ",", "90.11423913463436`", ",", "97.51490374321588`", 
   ",", "54.900257362220295`", ",", "0.5184029688938222`", ",", 
   "0.5153622841917906`", ",", "0.30602351256581245`", ",", "Indeterminate", 
   ",", "1.2527215735859425`", ",", "0.6206824816181933`", ",", 
   "0.0953541149961199`", ",", "72.33091128513185`"}], "}"}]], "Print",
 CellChangeTimes->{3.593411134397796*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "8.131516293641283`*^-20"}], ",", "0.9899749999999999`", 
     ",", "0.9899749999999999`", ",", "0.9899749999999999`", ",", 
     "39.91228084312581`", ",", "10.365596734837098`", ",", 
     "1.6607171316621552`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", \
{-8.131516293641283*^-20, 0.9899749999999999, 0.9899749999999999, 
   0.9899749999999999, 39.91228084312581, 10.365596734837098`, 
   1.6607171316621552`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593411134397796*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "44.06825740166666666666666666666666666667`10.572792125687128"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   44.06825740166666666666666666666666666667`10.572792125687128],
  Editable->False]], "Print",
 CellChangeTimes->{3.593411134397796*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to estimate params across Euroepan economies:  \
\"\>", "\[InvisibleSpace]", 
   "587.09064451666666666666666666666666666667`11.697371408022422"}],
  SequenceForm[
  " Time spent to estimate params across Euroepan economies:  ", 
   587.09064451666666666666666666666666666667`11.697371408022422],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934111350997972`*^9}],

Cell[BoxData["\<\"============================================================\
===============\"\>"], "Print",
 CellChangeTimes->{3.5934111350997972`*^9}],

Cell[BoxData["\<\"Models with aggregate permanent shocks\"\>"], "Print",
 CellChangeTimes->{3.5934111351153975`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593411136004599*^9}],

Cell[BoxData["\<\"Run \[Beta]-Dist model with time pref factors approximated \
by seven points\"\>"], "Print",
 CellChangeTimes->{3.593411136004599*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593411136051399*^9}],

Cell[BoxData["\<\"Country: All countries\"\>"], "Print",
 CellChangeTimes->{3.593411136113799*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934111361293993`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934111383446035`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593411139327405*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594659`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934111393430047`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934111393430047`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957241`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886498`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934111394366055`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "1.138802`7.2069632190292445"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   1.138802`7.2069632190292445],
  Editable->False]], "Print",
 CellChangeTimes->{3.593411139483405*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593411139483405*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934111394990053`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934111401074066`*^9}],

Cell[BoxData["0.989375`"], "Print",
 CellChangeTimes->{3.5934111401698065`*^9}],

Cell[BoxData["0.0008`"], "Print",
 CellChangeTimes->{3.5934111401698065`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934111401698065`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934111401698065`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.37621295666666666666666666666666666667`9.796499694841755"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.37621295666666666666666666666666666667`9.796499694841755],
  Editable->False]], "Print",
 CellChangeTimes->{3.593411582711384*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.104654903225186`", ",", "0.9712469489582356`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.104654903225186, 0.9712469489582356}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934116218830523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.95`"}],
  SequenceForm[" R^2: ", 0.95],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934116218830523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65286114666666666666666666666666666667`8.743487071682463"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65286114666666666666666666666666666667`8.743487071682463],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934116219142523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934116219142523`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934116219142523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.96482345`9.221989780975171"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.96482345`9.221989780975171],
  Editable->False]], "Print",
 CellChangeTimes->{3.59341174010006*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11738422269022974`", ",", "0.9677487425394561`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11738422269022974`, 0.9677487425394561}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934117785073276`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934117788193283`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64558113333333333333333333333333333333`8.738617078156176"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64558113333333333333333333333333333333`8.738617078156176],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934117788193283`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934117788193283`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593411778834928*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.52884093`8.651991308507977"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.52884093`8.651991308507977],
  Editable->False]], "Print",
 CellChangeTimes->{3.593411810580984*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1229088521150492`", ",", "0.9662350101118311`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1229088521150492, 0.9662350101118311}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934118501894536`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934118502050533`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66586117`8.752049937764387"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.66586117`8.752049937764387],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934118502050533`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934118502050533`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934118502050533`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.64948114`8.74123279251944"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.64948114`8.74123279251944],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934118891583223`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12693674336053085`", ",", "0.9651261622178953`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12693674336053085`, 0.9651261622178953}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934119318711967`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.593411931886797*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.71188125`8.781073802567477"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.71188125`8.781073802567477],
  Editable->False]], "Print",
 CellChangeTimes->{3.593411931886797*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593411931886797*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593411931886797*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.543920955`8.664202038903502"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.543920955`8.664202038903502],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934119645064545`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12595205008364652`", ",", "0.9653987943259892`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12595205008364652`, 0.9653987943259892}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934120043333244`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934120043333244`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66378116666666666666666666666666666667`8.75069117431055"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66378116666666666666666666666666666667`8.75069117431055],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934120043333244`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934120043333244`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593412004348924*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05330009333333333333333333333333333333`7.655394217732188"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05330009333333333333333333333333333333`7.655394217732188],
  Editable->False]], "Print",
 CellChangeTimes->{3.59341200753133*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.59341200754693*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934121866508446`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593412365099558*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12481628162452757`", ",", "0.9657082939384364`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12481628162452757`, 0.9657082939384364}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59341240055842*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.593412400901621*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.550451505`9.744937483989407"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.550451505`9.744937483989407],
  Editable->False]], "Print",
 CellChangeTimes->{3.593412400901621*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593412400917221*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593412400917221*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05278009333333333333333333333333333333`7.651136402082152"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05278009333333333333333333333333333333`7.651136402082152],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934124037252254`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934124037252254`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934132252538686`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593414048638915*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934142152940073`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05723685144622608`", ",", "0.05869349437368833`", ",", 
   "1.025449389521902`", ",", "0.05599223845625599`", ",", 
   "0.9782550409653578`", ",", "0.9965148004534454`", ",", 
   "0.9953809222483918`", ",", "0.995204785140001`", ",", 
   "0.9992217401538414`", ",", "0.9586333521454319`", ",", 
   "0.9448829650099926`", ",", "0.9710592562514628`", ",", 
   "10.264766079935812`"}], "}"}]], "Print",
 CellChangeTimes->{3.593414215621608*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934142156684084`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0958364489457764`", ",", "0.8169974456020628`", ",", 
   "0.07405785824374668`", ",", "0.0450556629469023`", ",", 
   "0.4659745298942394`", ",", "0.2038115116280719`", ",", 
   "0.9262169902399224`", ",", "0.9478187387008262`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934142156684084`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593414215684008*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.914969349347539`", ",", "37.361580024216`", ",", "51.753953791613164`", 
   ",", "68.80362549489675`", ",", "74.52150589075111`", ",", 
   "85.98543768642027`", ",", "90.75835278190884`", ",", "94.16473917995103`",
    ",", "98.3813896743884`", ",", "0.6519252678388658`", ",", 
   "26.593366071428573`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934142156996083`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.11914924803698679`"}],
  SequenceForm[" MPC mean:", 0.11914924803698679`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934142157308083`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934142157308083`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.08748104086740827`", ",", "0.09057608086936308`", ",", 
   "0.0993114903603538`", ",", "0.11388633479601629`", ",", 
   "0.12287479011294955`", ",", "0.11353869811115536`", ",", 
   "0.13237967574128895`", ",", "0.12492902051616339`", ",", 
   "0.11942278688278263`", ",", "0.12245113150216913`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934142157308083`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934142157620087`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05829731315347997`", ",", "0.059470041185301636`", ",", 
   "0.0605016369924265`", ",", "0.06295642815061198`", ",", 
   "0.0666398755006159`", ",", "0.19371986114468454`", ",", 
   "0.2836933648850576`", ",", "0.13789719215780338`", ",", 
   "0.17127343180597288`", ",", "0.2510514920832375`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934142157620087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1102422843606633`", ",", "0.2312527329256412`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.1102422843606633, 0.2312527329256412}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934142157620087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12482073071124736`", ",", "0.965706228876309`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12482073071124736`, 0.965706228876309}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934142157620087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934142157620087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.19281303`10.40856982584599"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 30.19281303`10.40856982584599],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934142157620087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "51.26351003833333333333333333333333333333`10.638474587443692"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   51.26351003833333333333333333333333333333`10.638474587443692],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934142157620087`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934142157776084`*^9}],

Cell[BoxData["\<\"Country: All (low psi)\"\>"], "Print",
 CellChangeTimes->{3.5934142157776084`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934142157776084`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934142164796095`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593414217103611*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594659`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934142171348104`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934142171348104`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957241`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574745`", ",", "0.20817745725954456`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886498`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934142171348104`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6240011`6.945700353096958"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6240011`6.945700353096958],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934142171348104`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934142171348104`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934142171348104`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934142176340113`*^9}],

Cell[BoxData["0.9901999999999999`"], "Print",
 CellChangeTimes->{3.5934142176340113`*^9}],

Cell[BoxData["0.0007`"], "Print",
 CellChangeTimes->{3.5934142176340113`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934142176340113`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934142176340113`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.54183324666666666666666666666666666667`9.80614317394459"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.54183324666666666666666666666666666667`9.80614317394459],
  Editable->False]], "Print",
 CellChangeTimes->{3.593414670487207*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10485280480673376`", ",", "0.971236436749261`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10485280480673376`, 0.971236436749261}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934147118428793`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.949`"}],
  SequenceForm[" R^2: ", 0.949],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934147118428793`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69498122166666666666666666666666666667`8.770639318364418"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69498122166666666666666666666666666667`8.770639318364418],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934147118428793`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934147118428793`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934147118584795`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "2.55242448333333333333333333333333333333`9.335619150019914"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   2.55242448333333333333333333333333333333`9.335619150019914],
  Editable->False]], "Print",
 CellChangeTimes->{3.593414864988349*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11580259722887815`", ",", "0.968248979274469`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11580259722887815`, 0.968248979274469}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593414903801217*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593414903801217*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.646881135`8.739490734207154"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.646881135`8.739490734207154],
  Editable->False]], "Print",
 CellChangeTimes->{3.593414903801217*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593414903801217*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593414903801217*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.72826128`8.790953468331425"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.72826128`8.790953468331425],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934149475280933`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1190980375155534`", ",", "0.9673507334349987`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1190980375155534, 0.9673507334349987}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934149873081636`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.928`"}],
  SequenceForm[" R^2: ", 0.928],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934149873081636`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.663521165`8.750521028763648"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.663521165`8.750521028763648],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934149873081636`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934149873081636`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934149873081636`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.45162079333333333333333333333333333333`8.58344017753153"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.45162079333333333333333333333333333333`8.58344017753153],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934150144054112`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12265479057802194`", ",", "0.9663746188066957`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12265479057802194`, 0.9663746188066957}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415055776684*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415055776684*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68952121166666666666666666666666666667`8.767213879087308"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68952121166666666666666666666666666667`8.767213879087308],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415055776684*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415055776684*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593415055776684*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.33774059333333333333333333333333333333`8.45724951021668"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.33774059333333333333333333333333333333`8.45724951021668],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934150760411196`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12884938775864915`", ",", "0.9646746997862222`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12884938775864915`, 0.9646746997862222}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415117521592*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.921`"}],
  SequenceForm[" R^2: ", 0.921],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415117521592*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69134121333333333333333333333333333333`8.768358695724036"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69134121333333333333333333333333333333`8.768358695724036],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415117521592*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934151175371923`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934151175371923`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.53040093166666666666666666666666666667`8.653270526465548"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.53040093166666666666666666666666666667`8.653270526465548],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415149345648*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593415149345648*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934153330203705`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415516523493*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12542172516800965`", ",", "0.9656076221919971`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12542172516800965`, 0.9656076221919971}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415554010359*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.928`"}],
  SequenceForm[" R^2: ", 0.928],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415554010359*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.74441184666666666666666666666666666667`9.757610330780345"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.74441184666666666666666666666666666667`9.757610330780345],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415554010359*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415554010359*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593415554010359*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.04966008666666666666666666666666666667`7.624673721364547"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.04966008666666666666666666666666666667`7.624673721364547],
  Editable->False]], "Print",
 CellChangeTimes->{3.593415556989964*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593415556989964*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593416364228982*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593417170953199*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934173338798847`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057200674373980825`", ",", "0.058664529669901704`", ",", 
   "1.0255915740844264`", ",", "0.055833538042385285`", ",", 
   "0.976099296965327`", ",", "0.9965674262744671`", ",", 
   "0.9955475561878159`", ",", "0.9953144299967109`", ",", 
   "0.9991832512789263`", ",", "0.9603653438893921`", ",", 
   "0.9470316380055055`", ",", "0.9721890899950857`", ",", 
   "10.3076336899563`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934173338798847`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934173338798847`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.08472943402540696`", ",", "0.8194026687450774`", ",", 
   "0.07248684959178361`", ",", "0.04290801713226297`", ",", 
   "0.45492348330412186`", ",", "0.19354208810860207`", ",", 
   "0.9295991079243551`", ",", "0.9506356353583449`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934173338798847`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.593231574287202`", ",", "37.34367466981924`", ",", "51.93656323788374`",
    ",", "69.07438738656623`", ",", "74.75578103083222`", ",", 
   "86.03373228737138`", ",", "90.71301288396582`", ",", "94.08039151866372`",
    ",", "98.32808670296819`", ",", "0.652477330896979`", ",", 
   "25.97713392857143`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.11889151119746932`"}],
  SequenceForm[" MPC mean:", 0.11889151119746932`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.08177010802038132`", ",", "0.08561385914984088`", ",", 
   "0.09558548868413241`", ",", "0.10632194074253898`", ",", 
   "0.11414794193402156`", ",", "0.12597111315849885`", ",", 
   "0.13436332668634776`", ",", "0.12611352341753912`", ",", 
   "0.12632329418069654`", ",", "0.12488897378760067`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.055985208079347926`", ",", "0.05695992778852732`", ",", 
   "0.05789354175184691`", ",", "0.05995166494219972`", ",", 
   "0.06385308361613351`", ",", "0.1968537972702401`", ",", 
   "0.2900785812054939`", ",", "0.13841023682494935`", ",", 
   "0.1735252211282643`", ",", "0.2561761432625189`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10960837846933025`", ",", "0.23546513241147782`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.10960837846933025`, 0.23546513241147782`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12447683812025456`", ",", "0.9658639010692651`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12447683812025456`, 0.9658639010692651}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934173339266853`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934173339422855`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.614832015`10.40017552173943"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.614832015`10.40017552173943],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934173339422855`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "51.94627123833333333333333333333333333333`10.644220627082163"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   51.94627123833333333333333333333333333333`10.644220627082163],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934173339422855`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934173339578853`*^9}],

Cell[BoxData["\<\"Country: All (high theta)\"\>"], "Print",
 CellChangeTimes->{3.5934173339578853`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934173339578853`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934173345974865`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934173352682877`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594662`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934173352994876`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934173352994876`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957245`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574756`", ",", "0.20817745725954465`", ",", 
   "0.9324174143162659`", ",", "0.9486914927886495`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934173352994876`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934173353150873`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934173353150873`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934173353150873`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934173357986884`*^9}],

Cell[BoxData["0.9888753439998171`"], "Print",
 CellChangeTimes->{3.5934173357986884`*^9}],

Cell[BoxData["0.001`"], "Print",
 CellChangeTimes->{3.5934173357986884`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934173357986884`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934173357986884`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.44173307166666666666666666666666666667`9.800340336424489"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.44173307166666666666666666666666666667`9.800340336424489],
  Editable->False]], "Print",
 CellChangeTimes->{3.593417782302673*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10141690354476673`", ",", "0.9721343598839843`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10141690354476673`, 0.9721343598839843}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934178209283404`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.954`"}],
  SequenceForm[" R^2: ", 0.954],
  Editable->False]], "Print",
 CellChangeTimes->{3.59341782094394*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64376113`8.73739099885641"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.64376113`8.73739099885641],
  Editable->False]], "Print",
 CellChangeTimes->{3.59341782094394*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.59341782094394*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59341782094394*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.88890331833333333333333333333333333333`9.20487597776855"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.88890331833333333333333333333333333333`9.20487597776855],
  Editable->False]], "Print",
 CellChangeTimes->{3.59341793426254*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11781824993034784`", ",", "0.9676329552356167`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11781824993034784`, 0.9676329552356167}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934179777866163`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.921`"}],
  SequenceForm[" R^2: ", 0.921],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934179777866163`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.72540127333333333333333333333333333333`8.78924456179966"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.72540127333333333333333333333333333333`8.78924456179966],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934179777866163`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934179777866163`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934179777866163`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.76648134666666666666666666666666666667`8.813167838406274"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.76648134666666666666666666666666666667`8.813167838406274],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934180237754965`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12541935599089643`", ",", "0.9655502987334419`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12541935599089643`, 0.9655502987334419}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934180635399666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934180635399666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.657281155`8.746417428956617"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.657281155`8.746417428956617],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934180635399666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934180635399666`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934180635399666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.72228126833333333333333333333333333333`8.78737260021429"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.72228126833333333333333333333333333333`8.78737260021429],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418106549242*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12232431641775851`", ",", "0.9663996266640933`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12232431641775851`, 0.9663996266640933}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418146968913*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.929`"}],
  SequenceForm[" R^2: ", 0.929],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418146968913*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67366118333333333333333333333333333333`8.7571077720418"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67366118333333333333333333333333333333`8.7571077720418],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418146968913*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418146968913*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593418146968913*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05382009333333333333333333333333333333`7.659610694785628"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05382009333333333333333333333333333333`7.659610694785628],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934181501981187`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12381658508967673`", ",", "0.9659846695607854`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12381658508967673`, 0.9659846695607854}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418191413391*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418191413391*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68692120666666666666666666666666666667`8.765573172360511"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68692120666666666666666666666666666667`8.765573172360511],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418191413391*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418191413391*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593418191413391*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.053820095`7.6596107082345855"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.053820095`7.6596107082345855],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934181946425967`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934181946425967`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934183746981134`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934185559704313`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12426105396300641`", ",", "0.9658614098484928`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12426105396300641`, 0.9658614098484928}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418591881695*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418591881695*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.62065162833333333333333333333333333333`9.74956698458314"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.62065162833333333333333333333333333333`9.74956698458314],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418591881695*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593418591881695*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593418591881695*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05174009166666666666666666666666666667`7.642493442025847"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05174009166666666666666666666666666667`7.642493442025847],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934185949861*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934185949861*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934194088083296`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934202274509673`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593420393232458*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05720122252073831`", ",", "0.0587010600648723`", ",", 
   "1.0262203756849817`", ",", "0.05592397515306266`", ",", 
   "0.977670977797501`", ",", "0.9963209921365391`", ",", 
   "0.9952385392642858`", ",", "0.9950727395402015`", ",", 
   "0.9992219966748752`", ",", "0.957248638393288`", ",", 
   "0.9426870780406221`", ",", "0.9683890534798771`", ",", 
   "10.2677524774576`"}], "}"}]], "Print",
 CellChangeTimes->{3.593420393232458*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593420393232458*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09711038320285316`", ",", "0.8027864248618867`", ",", 
   "0.06782891844483367`", ",", "0.0410719223681768`", ",", 
   "0.4687928280707625`", ",", "0.20561779263001997`", ",", 
   "0.9198984148068807`", ",", "0.943347561762311`"}], "}"}]], "Print",
 CellChangeTimes->{3.593420393232458*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593420393232458*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.30970390158356`", ",", "37.882065543526565`", ",", "52.11849764289308`",
    ",", "68.80271977086684`", ",", "74.37471234843443`", ",", 
   "85.63696755938696`", ",", "90.45554435238667`", ",", "93.98283395173378`",
    ",", "98.40927979283656`", ",", "0.651000146442487`", ",", 
   "27.2379375`"}], "}"}]], "Print",
 CellChangeTimes->{3.593420393232458*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.14445779695736227`"}],
  SequenceForm[" MPC mean:", 0.14445779695736227`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593420393232458*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593420393232458*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09621616583244552`", ",", "0.10247342734988574`", ",", 
   "0.10781658805318772`", ",", "0.1225018302988593`", ",", 
   "0.1327248280264517`", ",", "0.16183353209257156`", ",", 
   "0.1507609540181729`", ",", "0.1556854876717988`", ",", 
   "0.16154172290822888`", ",", "0.14503161541231324`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934203932480583`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934203932480583`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05803062004951598`", ",", "0.05937532863011774`", ",", 
   "0.06079684379606776`", ",", "0.0646676128442979`", ",", 
   "0.07098151142191222`", ",", "0.24639128050201298`", ",", 
   "0.3641588565888161`", ",", "0.1708252538147763`", ",", 
   "0.2164324275395184`", ",", "0.3217160787957227`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934203932480583`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13601434933619228`", ",", "0.2508653155062941`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.13601434933619228`, 0.2508653155062941}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593420393606859*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12535618473415674`", ",", "0.9655612489085457`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12535618473415674`, 0.9655612489085457}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593420393606859*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593420393606859*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.97077264`10.405364187322878"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.97077264`10.405364187322878],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934203936224594`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "50.96606951666666666666666666666666666667`10.635947390475279"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   50.96606951666666666666666666666666666667`10.635947390475279],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934203936224594`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934203936224594`*^9}],

Cell[BoxData["\<\"Country: All (very high theta)\"\>"], "Print",
 CellChangeTimes->{3.5934203936224594`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593420393638059*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934203942932606`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593420394979662*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594664`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934203950264616`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934203950264616`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957246`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886492`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934203950264616`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6864012`6.98709303192806"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6864012`6.98709303192806],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934203950264616`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934203950264616`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934203950264616`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593420395510062*^9}],

Cell[BoxData["0.9880755965`"], "Print",
 CellChangeTimes->{3.593420395510062*^9}],

Cell[BoxData["0.0013000000000000002`"], "Print",
 CellChangeTimes->{3.593420395510062*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593420395510062*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593420395510062*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.37699295666666666666666666666666666667`9.796545617022764"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.37699295666666666666666666666666666667`9.796545617022764],
  Editable->False]], "Print",
 CellChangeTimes->{3.59342083812964*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10031062374495192`", ",", "0.9724337452846326`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10031062374495192`, 0.9724337452846326}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59342087801891*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.957`"}],
  SequenceForm[" R^2: ", 0.957],
  Editable->False]], "Print",
 CellChangeTimes->{3.59342087801891*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66482116833333333333333333333333333333`8.751371087427298"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66482116833333333333333333333333333333`8.751371087427298],
  Editable->False]], "Print",
 CellChangeTimes->{3.59342087801891*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.59342087801891*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59342087801891*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.767743105`9.176085400189473"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.767743105`9.176085400189473],
  Editable->False]], "Print",
 CellChangeTimes->{3.593420984083496*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11529868943065008`", ",", "0.9683133428543271`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11529868943065008`, 0.9683133428543271}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593421025673169*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934210256887693`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69316121666666666666666666666666666667`8.769500503557492"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69316121666666666666666666666666666667`8.769500503557492],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934210256887693`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934210256887693`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934210256887693`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.66664117166666666666666666666666666667`8.75255838033564"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.66664117166666666666666666666666666667`8.75255838033564],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934210656716394`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12134321539807771`", ",", "0.9666548580452179`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12134321539807771`, 0.9666548580452179}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211059041104`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211059041104`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67054117833333333333333333333333333333`8.755091701505128"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67054117833333333333333333333333333333`8.755091701505128],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211059041104`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211059041104`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934211059041104`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.63388111333333333333333333333333333333`8.730674060253723"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.63388111333333333333333333333333333333`8.730674060253723],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211439369774`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1224035444872877`", ",", "0.9663701075002092`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1224035444872877, 0.9663701075002092}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211845126486`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211845438485`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67626118666666666666666666666666666667`8.75878071045953"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67626118666666666666666666666666666667`8.75878071045953],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211845438485`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211845438485`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934211845594482`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.076180135`7.810507986161235"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.076180135`7.810507986161235],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934211890834565`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12362291937284461`", ",", "0.9660392405712968`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12362291937284461`, 0.9660392405712968}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934212312191305`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934212312191305`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.70226123333333333333333333333333333333`8.77516494309624"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.70226123333333333333333333333333333333`8.77516494309624],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934212312347307`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934212312347307`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934212312347307`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.12662022166666666666666666666666666667`8.03116931769587"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.12662022166666666666666666666666666667`8.03116931769587],
  Editable->False]], "Print",
 CellChangeTimes->{3.593421238863144*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593421238863144*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934214172026567`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593421598646576*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12389519083109278`", ",", "0.9659656451134292`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12389519083109278`, 0.9659656451134292}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593421634745039*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.593421634745039*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.59881159`9.748131976730585"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.59881159`9.748131976730585],
  Editable->False]], "Print",
 CellChangeTimes->{3.593421634745039*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593421634745039*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593421634745039*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05018008833333333333333333333333333333`7.62919766969493"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05018008833333333333333333333333333333`7.62919766969493],
  Editable->False]], "Print",
 CellChangeTimes->{3.593421637755844*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593421637755844*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934224441212606`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934232502682767`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593423412820562*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05711950751123988`", ",", "0.058604312720771885`", ",", 
   "1.025994713088866`", ",", "0.05589982945018762`", ",", 
   "0.9786469086622946`", ",", "0.9962487676498856`", ",", 
   "0.9951986173329034`", ",", "0.9950607780121034`", ",", 
   "0.9992301594844617`", ",", "0.9570345657066475`", ",", 
   "0.9421339672051694`", ",", "0.9673595629335752`", ",", 
   "10.269343851216078`"}], "}"}]], "Print",
 CellChangeTimes->{3.593423412820562*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593423412820562*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0983090575802872`", ",", "0.7971238041320325`", ",", 
   "0.0670653208025749`", ",", "0.047615682549677275`", ",", 
   "0.47184878688995346`", ",", "0.20893384169232146`", ",", 
   "0.9118812352932513`", ",", "0.9383108263245746`"}], "}"}]], "Print",
 CellChangeTimes->{3.593423412820562*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593423412820562*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.104236905373956`", ",", "38.97014526401266`", ",", "53.08661060956263`",
    ",", "69.35489573212031`", ",", "74.74727083049662`", ",", 
   "85.71377053002337`", ",", "90.4902405029392`", ",", "94.02736557283426`", 
   ",", "98.47614913086257`", ",", "0.6561052512639348`", ",", 
   "27.424135714285715`"}], "}"}]], "Print",
 CellChangeTimes->{3.593423412820562*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.16995617406059904`"}],
  SequenceForm[" MPC mean:", 0.16995617406059904`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934234128517623`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934234128517623`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.10687148903794064`", ",", "0.11634591797718674`", ",", 
   "0.11818916104218169`", ",", "0.13609859158542237`", ",", 
   "0.15227924599523146`", ",", "0.1959514092214183`", ",", 
   "0.17717888773218093`", ",", "0.18539444683320072`", ",", 
   "0.19666058828156807`", ",", "0.18803872818488088`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934234128517623`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934234128517623`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05753021718016238`", ",", "0.05930342315814574`", ",", 
   "0.06137221349579247`", ",", "0.06767653718100175`", ",", 
   "0.07766395410282412`", ",", "0.2951882118221033`", ",", 
   "0.4308541250749549`", ",", "0.2036301813282595`", ",", 
   "0.2594899898599081`", ",", "0.38288023691830353`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934234128517623`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.16202559867753552`", ",", "0.27006849622590023`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.16202559867753552`, 0.27006849622590023`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934234128985624`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12503007626898754`", ",", "0.9656539913304374`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12503007626898754`, 0.9656539913304374}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934234128985624`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934234128985624`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.58441196166666666666666666666666666667`10.399729189604853"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.58441196166666666666666666666666666667`10.399729189604853],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934234128985624`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "50.29734834166666666666666666666666666667`10.630211338024292"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   50.29734834166666666666666666666666666667`10.630211338024292],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934234128985624`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934234129141626`*^9}],

Cell[BoxData["\<\"Country: Austria\"\>"], "Print",
 CellChangeTimes->{3.5934234129141626`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934234129297624`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593423413584964*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934234142713647`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594659`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934234142713647`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934234142713647`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957241`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574745`", ",", "0.20817745725954456`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886495`"}], "}"}]], "Print",
 CellChangeTimes->{3.593423414286965*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.7020012`6.996852852344888"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.7020012`6.996852852344888],
  Editable->False]], "Print",
 CellChangeTimes->{3.593423414286965*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593423414286965*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593423414286965*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934234148641653`*^9}],

Cell[BoxData["0.9884768754430068`"], "Print",
 CellChangeTimes->{3.5934234148797655`*^9}],

Cell[BoxData["0.0013000000000000002`"], "Print",
 CellChangeTimes->{3.5934234148797655`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934234148797655`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934234148797655`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.99527404333333333333333333333333333333`9.8314996022851"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.99527404333333333333333333333333333333`9.8314996022851],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934238945806084`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.09593804621517815`", ",", "0.9736303706937735`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.09593804621517815, 0.9736303706937735}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934239333778763`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.962`"}],
  SequenceForm[" R^2: ", 0.962],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934239333778763`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.640901125`8.735457282135261"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.640901125`8.735457282135261],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934239333778763`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934239333778763`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934239333778763`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "2.11094370833333333333333333333333333333`9.253142900525576"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   2.11094370833333333333333333333333333333`9.253142900525576],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934240596912985`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11140695474829307`", ",", "0.9693838795804457`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11140695474829307`, 0.9693838795804457}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934240990033674`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934240990033674`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65494115`8.744868526255509"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.65494115`8.744868526255509],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934240990033674`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934240990033674`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934240990033674`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.80522141333333333333333333333333333333`8.834581563822278"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.80522141333333333333333333333333333333`8.834581563822278],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934241473166523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11619539560555163`", ",", "0.9680721543683877`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11619539560555163`, 0.9680721543683877}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934241886411247`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.928`"}],
  SequenceForm[" R^2: ", 0.928],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934241886411247`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68874121`8.766722317465156"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.68874121`8.766722317465156],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424188656725*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424188656725*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593424188656725*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.57876101666666666666666666666666666667`8.6911655190788"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.57876101666666666666666666666666666667`8.6911655190788],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424223366786*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11841474886029255`", ",", "0.9674626460013386`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11841474886029255`, 0.9674626460013386}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424263380856*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.929`"}],
  SequenceForm[" R^2: ", 0.929],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424263380856*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66690117166666666666666666666666666667`8.752727728639114"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66690117166666666666666666666666666667`8.752727728639114],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424263380856*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424263396456*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593424263396456*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.33046058`8.447785908982722"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.33046058`8.447785908982722],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424283208491*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12136514358040344`", ",", "0.9666465440516185`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12136514358040344`, 0.9666465440516185}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424323830962*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934243238465624`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67704118833333333333333333333333333333`8.759281338350943"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67704118833333333333333333333333333333`8.759281338350943],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934243238465624`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934243238465624`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934243238465624`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.46618082`8.59722064966206"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.46618082`8.59722064966206],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934243518174114`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934243518174114`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934245266469183`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934247060784335`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12153974628218009`", ",", "0.9666042467249725`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12153974628218009`, 0.9666042467249725}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934247420052967`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.928`"}],
  SequenceForm[" R^2: ", 0.928],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934247420052967`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.50339142166666666666666666666666666667`9.741806142028873"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.50339142166666666666666666666666666667`9.741806142028873],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934247420052967`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593424742020897*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593424742020897*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05174009`7.642493428036228"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05174009`7.642493428036228],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934247451409025`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934247451409025`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934255568259277`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934263719741597`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934265364920483`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05712263685152682`", ",", "0.05857181652653843`", ",", 
   "1.0253696214826062`", ",", "0.05568950414379828`", ",", 
   "0.9749112998502933`", ",", "0.9965237009766518`", ",", 
   "0.9957208817380656`", ",", "0.995505464995832`", ",", 
   "0.9991933915110313`", ",", "0.9614515597742629`", ",", 
   "0.9478509641652714`", ",", "0.9712601615815665`", ",", 
   "10.27039911702995`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934265364920483`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593426536835249*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09523043204502939`", ",", "0.8167321687114351`", ",", 
   "0.07540928918268526`", ",", "0.0435031505364131`", ",", 
   "0.45969275748004906`", ",", "0.19789602139407514`", ",", 
   "0.9265637387111294`", ",", "0.9487309373520674`"}], "}"}]], "Print",
 CellChangeTimes->{3.593426536850849*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593426536850849*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.937330542913656`", ",", "46.557461391176176`", ",", 
   "61.53616589373861`", ",", "77.60678170043921`", ",", "82.46500800594008`",
    ",", "91.11237125272953`", ",", "94.26414759194127`", ",", 
   "96.40040935686162`", ",", "98.99572205702586`", ",", 
   "0.7361595482485562`", ",", "19.156489285714287`"}], "}"}]], "Print",
 CellChangeTimes->{3.593426536882049*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.16421083833413774`"}],
  SequenceForm[" MPC mean:", 0.16421083833413774`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593426536913249*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593426536928849*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12774382487851743`", ",", "0.1316247254955455`", ",", 
   "0.13989575970329293`", ",", "0.15402421440918845`", ",", 
   "0.16319126778107695`", ",", "0.16573844689744632`", ",", 
   "0.1942855233174754`", ",", "0.17147585562663636`", ",", 
   "0.16927903945168554`", ",", "0.18051293163902726`"}], "}"}]], "Print",
 CellChangeTimes->{3.593426536928849*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593426536928849*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05598695397330011`", ",", "0.05743473143747446`", ",", 
   "0.05888565565673343`", ",", "0.06295654000576867`", ",", 
   "0.07428226451821096`", ",", "0.2866171138252679`", ",", 
   "0.3961784320686492`", ",", "0.1970402425777762`", ",", 
   "0.2534977752098362`", ",", "0.3593700979662925`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934265369444494`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.15097111020661502`", ",", "0.3258274779282283`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.15097111020661502`, 0.3258274779282283}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934265369600496`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12281187437465867`", ",", "0.9662598407291909`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12281187437465867`, 0.9662598407291909}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934265369600496`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934265369600496`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.85611244`10.403699505865491"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.85611244`10.403699505865491],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934265369756494`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "52.03675139833333333333333333333333333333`10.644976424386966"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   52.03675139833333333333333333333333333333`10.644976424386966],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934265369756494`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934265369756494`*^9}],

Cell[BoxData["\<\"Country: Belgium\"\>"], "Print",
 CellChangeTimes->{3.5934265369912496`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934265370068493`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934265376932507`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934265383484516`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594659`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934265383484516`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934265383484516`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957241`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886498`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934265383484516`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6552011`6.966889615710615"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6552011`6.966889615710615],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934265383484516`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934265383484516`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593426538364052*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934265389256525`*^9}],

Cell[BoxData["0.9897499999999999`"], "Print",
 CellChangeTimes->{3.5934265389256525`*^9}],

Cell[BoxData["0.0004999999999999999`"], "Print",
 CellChangeTimes->{3.5934265389256525`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934265389256525`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934265389256525`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "6.58373156333333333333333333333333333333`9.747138363411993"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   6.58373156333333333333333333333333333333`9.747138363411993],
  Editable->False]], "Print",
 CellChangeTimes->{3.593426933949547*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11674072913889288`", ",", "0.9679432139065204`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11674072913889288`, 0.9679432139065204}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934269724660144`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.935`"}],
  SequenceForm[" R^2: ", 0.935],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934269724660144`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64194112666666666666666666666666666667`8.73616144834645"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64194112666666666666666666666666666667`8.73616144834645],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934269724660144`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934269724660144`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593426972481614*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.59302279833333333333333333333333333333`9.1308882394088"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.59302279833333333333333333333333333333`9.1308882394088],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934270680473824`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12159202522469525`", ",", "0.9666105065274189`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12159202522469525`, 0.9666105065274189}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427107312651*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427107343851*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65442115`8.744523574911694"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.65442115`8.744523574911694],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427107343851*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427107343851*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593427107343851*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05148009`7.640305545703404"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05148009`7.640305545703404],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427110401457*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12456719845944636`", ",", "0.9657909122149172`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12456719845944636`, 0.9657909122149172}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934271535979323`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934271535979323`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.719941265`8.78596331502622"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.719941265`8.78596331502622],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934271535979323`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427153613532*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593427153613532*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05278009166666666666666666666666666667`7.651136388368189"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05278009166666666666666666666666666667`7.651136388368189],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427156764738*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1269490322353907`", ",", "0.9651359079763309`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1269490322353907, 0.9651359079763309}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427198338811*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427198354411*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69290121833333333333333333333333333333`8.769337573172908"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69290121833333333333333333333333333333`8.769337573172908],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427198354411*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427198354411*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593427198354411*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.10478018333333333333333333333333333333`7.948945402210985"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.10478018333333333333333333333333333333`7.948945402210985],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427204625622*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1270412986150608`", ",", "0.9651100053406274`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1270412986150608, 0.9651100053406274}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934272480872984`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934272480872984`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.72436127166666666666666666666666666667`8.788621470709275"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.72436127166666666666666666666666666667`8.788621470709275],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427248430499*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427248430499*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593427248430499*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05278009333333333333333333333333333333`7.651136402082152"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05278009333333333333333333333333333333`7.651136402082152],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934272512541037`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593427251269704*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427428548415*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427607199929*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12785671103208027`", ",", "0.9648859091142865`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12785671103208027`, 0.9648859091142865}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427645341996*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427645341996*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.56813153666666666666666666666666666667`9.746108089811255"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.56813153666666666666666666666666666667`9.746108089811255],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427645341996*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593427645341996*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934276453575964`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.08060014166666666666666666666666666667`7.83500205335816"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.08060014166666666666666666666666666667`7.83500205335816],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934276501780047`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934276501780047`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934284541878166`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934292626380367`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934294259391236`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05728469699313043`", ",", "0.058822508915714986`", ",", 
   "1.0268450738732016`", ",", "0.05601857237735376`", ",", 
   "0.9778976815408746`", ",", "0.9965239644713165`", ",", 
   "0.9950763162450325`", ",", "0.9949632387638965`", ",", 
   "0.9992549029560169`", ",", "0.9558352343352823`", ",", 
   "0.9418975309105485`", ",", "0.9708824052658412`", ",", 
   "10.273173292386877`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934294259391236`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934294259547234`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.099023910333752`", ",", "0.8175990393741399`", ",", 
   "0.07502741881833999`", ",", "0.042744537695697346`", ",", 
   "0.4752755842975633`", ",", "0.21257443924847816`", ",", 
   "0.9251934005747027`", ",", "0.9450309014022026`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934294259547234`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934294259547234`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.09149430062231`", ",", "30.56592757600431`", ",", "44.04835866354453`", 
   ",", "61.329301122966186`", ",", "67.5416601558457`", ",", 
   "80.98785503366581`", ",", "87.14355484318448`", ",", "91.79645364757664`",
    ",", "97.78779510154683`", ",", "0.5778923531682664`", ",", 
   "34.604955357142856`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934294259547234`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.1026509213718918`"}],
  SequenceForm[" MPC mean:", 0.1026509213718918],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934294262979245`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934294262979245`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.07097526417234745`", ",", "0.074153083002235`", ",", 
   "0.0825640891601237`", ",", "0.09716324968974521`", ",", 
   "0.1063186451488759`", ",", "0.0971281825058129`", ",", 
   "0.11371358533055198`", ",", "0.1085312188735379`", ",", 
   "0.10312179473290661`", ",", "0.10498814195516049`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934294262979245`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934294263135242`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.060491890049916175`", ",", "0.061225931063994654`", ",", 
   "0.06192453928832087`", ",", "0.06357866313660898`", ",", 
   "0.06529117977435017`", ",", "0.15657659104271304`", ",", 
   "0.23291195461560554`", ",", "0.11578106760587292`", ",", 
   "0.13974464236621986`", ",", "0.20367966447483554`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934294263135242`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.09518839066465457`", ",", "0.19748033077754634`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.09518839066465457, 0.19748033077754634`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934294263135242`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12723254038767662`", ",", "0.9650562823655426`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12723254038767662`, 0.9650562823655426}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934294263135242`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934294263291245`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.59627865`10.399903355782282"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.59627865`10.399903355782282],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934294263291245`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "48.12677119833333333333333333333333333333`10.611052974264815"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   48.12677119833333333333333333333333333333`10.611052974264815],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934294263291245`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934294263291245`*^9}],

Cell[BoxData["\<\"Country: Cyprus\"\>"], "Print",
 CellChangeTimes->{3.5934294263291245`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934294263447247`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934294270779257`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934294277331266`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594664`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.593429427748727*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593429427748727*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957246`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886492`"}], "}"}]], "Print",
 CellChangeTimes->{3.593429427748727*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.593429427748727*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593429427748727*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593429427748727*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593429428294728*^9}],

Cell[BoxData["0.989225`"], "Print",
 CellChangeTimes->{3.593429428294728*^9}],

Cell[BoxData["0.0009000000000000001`"], "Print",
 CellChangeTimes->{3.5934294283103275`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934294283103275`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934294283103275`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.59721334333333333333333333333333333333`9.809320570556661"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.59721334333333333333333333333333333333`9.809320570556661],
  Editable->False]], "Print",
 CellChangeTimes->{3.593429884127528*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10333009892009415`", ",", "0.971608571570672`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10333009892009415`, 0.971608571570672}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593429924672*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.952`"}],
  SequenceForm[" R^2: ", 0.952],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934299246875997`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67574118666666666666666666666666666667`8.758446638351108"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67574118666666666666666666666666666667`8.758446638351108],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934299246875997`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934299246875997`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934299246875997`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "2.03710357833333333333333333333333333333`9.237679359864288"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   2.03710357833333333333333333333333333333`9.237679359864288],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934300468982143`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12074268552665163`", ",", "0.9668311766667188`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12074268552665163`, 0.9668311766667188}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430085430282*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.919`"}],
  SequenceForm[" R^2: ", 0.919],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430085757882*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64220112833333333333333333333333333333`8.73633731249269"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64220112833333333333333333333333333333`8.73633731249269],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430085757882*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430085757882*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593430085757882*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.86242151333333333333333333333333333333`8.864385829784432"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.86242151333333333333333333333333333333`8.864385829784432],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430137175573*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12554110330555104`", ",", "0.9655139359412712`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12554110330555104`, 0.9655139359412712}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934301790772467`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934301790772467`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69836122666666666666666666666666666667`8.77274636735507"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69836122666666666666666666666666666667`8.77274636735507],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934301790772467`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934301790772467`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934301790928464`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.57460101`8.688032633249634"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.57460101`8.688032633249634],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430213553307*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12522565257083124`", ",", "0.9656009401580378`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12522565257083124`, 0.9656009401580378}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934302530993767`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934302530993767`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65910115833333333333333333333333333333`8.747618323104676"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65910115833333333333333333333333333333`8.747618323104676],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934302530993767`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934302530993767`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934302530993767`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05226009166666666666666666666666666667`7.646836415380619"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05226009166666666666666666666666666667`7.646836415380619],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430256234982*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12530978656437505`", ",", "0.9655859972347944`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12530978656437505`, 0.9655859972347944}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430300289459*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430300289459*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.73424128833333333333333333333333333333`8.794505050599838"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.73424128833333333333333333333333333333`8.794505050599838],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430300289459*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430300289459*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593430300289459*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05226009166666666666666666666666666667`7.646836415380619"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05226009166666666666666666666666666667`7.646836415380619],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430303425065*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593430303440665*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934304802825756`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430657623687*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1264806196575241`", ",", "0.9652603696626386`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1264806196575241, 0.9652603696626386}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934306938001504`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934306938001504`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.50625142833333333333333333333333333333`9.741997090411987"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.50625142833333333333333333333333333333`9.741997090411987],
  Editable->False]], "Print",
 CellChangeTimes->{3.59343069381575*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.59343069381575*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59343069381575*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05096009`7.635896434545847"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05096009`7.635896434545847],
  Editable->False]], "Print",
 CellChangeTimes->{3.593430696873356*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593430696873356*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.59343150211557*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593432307513785*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934324696136694`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057158899360724595`", ",", "0.05863123658296186`", ",", 
   "1.0257586699307397`", ",", "0.05586121236831543`", ",", 
   "0.9772968512878532`", ",", "0.9965279698898513`", ",", 
   "0.9953981215302132`", ",", "0.995219974830018`", ",", 
   "0.9992129921665424`", ",", "0.9586854626298043`", ",", 
   "0.9449797468989286`", ",", "0.9712988237678816`", ",", 
   "10.267760263551866`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0963406773479406`", ",", "0.8176798644412471`", ",", 
   "0.07399573014393492`", ",", "0.0499370742083697`", ",", 
   "0.4651389587659081`", ",", "0.2027924480525514`", ",", 
   "0.9264970973327886`", ",", "0.9475319009385456`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.01306177701269`", ",", "39.24699343857765`", ",", "53.86657546237854`", 
   ",", "70.84687727538957`", ",", "76.41384906373307`", ",", 
   "87.30034366903074`", ",", "91.69364210111148`", ",", "94.77841078749537`",
    ",", "98.55081064409504`", ",", "0.6719035254119414`", ",", 
   "24.647603571428572`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.12696092369127687`"}],
  SequenceForm[" MPC mean:", 0.12696092369127687`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09453993999497468`", ",", "0.09776494600102059`", ",", 
   "0.10640915801814521`", ",", "0.12097493600696607`", ",", 
   "0.13001938046538097`", ",", "0.12235812718100503`", ",", 
   "0.14266516969272525`", ",", "0.13297623030187322`", ",", 
   "0.12793456513519263`", ",", "0.13210509380011826`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05777418164093873`", ",", "0.05899295753274081`", ",", 
   "0.06008852612564042`", ",", "0.06276610572575247`", ",", 
   "0.06756221379305793`", ",", "0.21068096432954575`", ",", 
   "0.3061204862826298`", ",", "0.14825302010815034`", ",", 
   "0.1860261468956459`", ",", "0.27176328143311024`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11731314098454504`", ",", "0.24778078989897634`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.11731314098454504`, 0.24778078989897634`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12594472327936745`", ",", "0.9654042515501373`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12594472327936745`, 0.9654042515501373}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934324696292696`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.59343246964487*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.545931895`10.399163940635011"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.545931895`10.399163940635011],
  Editable->False]], "Print",
 CellChangeTimes->{3.59343246964487*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "50.69800904666666666666666666666666666667`10.633657152775317"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   50.69800904666666666666666666666666666667`10.633657152775317],
  Editable->False]], "Print",
 CellChangeTimes->{3.59343246964487*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934324699880705`*^9}],

Cell[BoxData["\<\"Country: Germany\"\>"], "Print",
 CellChangeTimes->{3.59343247000367*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.59343247000367*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934324707056713`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934324713764725`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594662`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934324713764725`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934324713764725`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957245`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574756`", ",", "0.20817745725954465`", ",", 
   "0.9324174143162659`", ",", "0.9486914927886495`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934324713764725`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6864012`6.98709303192806"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6864012`6.98709303192806],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934324713920727`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934324713920727`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934324713920727`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934324719692736`*^9}],

Cell[BoxData["0.986958894123534`"], "Print",
 CellChangeTimes->{3.593432471984874*^9}],

Cell[BoxData["0.0019000000000000004`"], "Print",
 CellChangeTimes->{3.593432471984874*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593432471984874*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593432471984874*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.81249372166666666666666666666666666667`9.821455929556997"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.81249372166666666666666666666666666667`9.821455929556997],
  Editable->False]], "Print",
 CellChangeTimes->{3.593432940718897*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.09051369216453414`", ",", "0.975121842090162`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.09051369216453414, 0.975121842090162}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934329791261644`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.971`"}],
  SequenceForm[" R^2: ", 0.971],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934329791261644`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.640121125`8.734928408046233"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.640121125`8.734928408046233],
  Editable->False]], "Print",
 CellChangeTimes->{3.593432979469365*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593432979469365*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593432979469365*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.95936344166666666666666666666666666667`9.220781248820124"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.95936344166666666666666666666666666667`9.220781248820124],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433096687971*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10561547259526613`", ",", "0.970970717962114`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10561547259526613`, 0.970970717962114}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59343313603124*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.928`"}],
  SequenceForm[" R^2: ", 0.928],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934331360468397`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65572115166666666666666666666666666667`8.745385441191216"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65572115166666666666666666666666666667`8.745385441191216],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934331360468397`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934331360468397`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934331360468397`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.777921365`8.819601947425674"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.777921365`8.819601947425674],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433182706522*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11586817791735164`", ",", "0.9681511651801109`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11586817791735164`, 0.9681511651801109}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934332229077926`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934332229077926`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67002117666666666666666666666666666667`8.754754777430097"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67002117666666666666666666666666666667`8.754754777430097],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934332229077926`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934332229077926`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934332229077926`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.03454181833333333333333333333333333333`8.94341429865899"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.03454181833333333333333333333333333333`8.94341429865899],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433284980302*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11367571408474511`", ",", "0.9687670414918654`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11367571408474511`, 0.9687670414918654}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934333241363707`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.933`"}],
  SequenceForm[" R^2: ", 0.933],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934333241363707`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.652601145`8.743314079645339"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.652601145`8.743314079645339],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934333241363707`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934333241363707`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934333241363707`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009333333333333333333333333333333`7.653270527830221"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009333333333333333333333333333333`7.653270527830221],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433327318776*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1174536431263571`", ",", "0.9677316671321222`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1174536431263571, 0.9677316671321222}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433367956847*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.928`"}],
  SequenceForm[" R^2: ", 0.928],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433367956847*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67184118`8.75593286829087"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.67184118`8.75593286829087],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433367956847*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934333679724474`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934333679724474`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05226009166666666666666666666666666667`7.646836415380619"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05226009166666666666666666666666666667`7.646836415380619],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934333707804527`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934333707804527`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433547107562*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433722919871*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12039380441020775`", ",", "0.966922202140911`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12039380441020775`, 0.966922202140911}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433758487933*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433758487933*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.46179134833333333333333333333333333333`9.739019178726341"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.46179134833333333333333333333333333333`9.739019178726341],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433758831134*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433758831134*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593433758831134*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.320840565`8.43495552067661"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.320840565`8.43495552067661],
  Editable->False]], "Print",
 CellChangeTimes->{3.593433777738367*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593433777738367*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593434622536272*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934354467013197`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934356128260117`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05712618381110462`", ",", "0.05848412740720926`", ",", 
   "1.0237709488978797`", ",", "0.05588131706577398`", ",", 
   "0.9782084735530916`", ",", "0.9964500669587781`", ",", 
   "0.9957906273520554`", ",", "0.9955923011139229`", ",", 
   "0.9991814403432983`", ",", "0.9623899636757786`", ",", 
   "0.9484996667068064`", ",", "0.9706158950373529`", ",", 
   "10.262345440837345`"}], "}"}]], "Print",
 CellChangeTimes->{3.593435613153612*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934356134188128`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0896214874004412`", ",", "0.8141210610356912`", ",", 
   "0.07871661137656806`", ",", "0.048201429169262275`", ",", 
   "0.45845234584591493`", ",", "0.19793042110590622`", ",", 
   "0.9227384815876541`", ",", "0.9464923431884738`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934356134188128`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593435613652813*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.98859497410454`", ",", "50.831588234379275`", ",", "65.40762287230501`",
    ",", "79.95112269639432`", ",", "84.13900990119485`", ",", 
   "91.64231015055876`", ",", "94.5456928773542`", ",", "96.60465734843226`", 
   ",", "99.12905511710613`", ",", "0.75729947364018`", ",", 
   "17.719889285714284`"}], "}"}]], "Print",
 CellChangeTimes->{3.593435613652813*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.21571733988764608`"}],
  SequenceForm[" MPC mean:", 0.21571733988764608`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934356137776136`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934356137776136`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.14937357047732847`", ",", "0.15507371763394595`", ",", 
   "0.16123405541876934`", ",", "0.17963198513646006`", ",", 
   "0.19335102000989868`", ",", "0.24839067257381753`", ",", 
   "0.25032231348549516`", ",", "0.23221883840927127`", ",", 
   "0.24460162089221615`", ",", "0.24098022640514905`"}], "}"}]], "Print",
 CellChangeTimes->{3.593435613793214*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934356138400135`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05466399711517922`", ",", "0.056594205125439334`", ",", 
   "0.05887537291578726`", ",", "0.06869691234895159`", ",", 
   "0.09071920335559758`", ",", "0.3784869888835103`", ",", 
   "0.517240346906689`", ",", "0.26291418387783727`", ",", 
   "0.33688961631557457`", ",", "0.4709368890402196`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934356138868136`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.2023555202000178`", ",", "0.37778742198660065`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.2023555202000178, 0.37778742198660065`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593435613933614*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12115118408189962`", ",", "0.9667119194257883`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12115118408189962`, 0.9667119194257883}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593435613964814*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.928`"}],
  SequenceForm[" R^2: ", 0.928],
  Editable->False]], "Print",
 CellChangeTimes->{3.593435613964814*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.58479407833333333333333333333333333333`10.414171809036223"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.58479407833333333333333333333333333333`10.414171809036223],
  Editable->False]], "Print",
 CellChangeTimes->{3.593435613964814*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "52.35723232`10.647642929258588"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 52.35723232`10.647642929258588],
  Editable->False]], "Print",
 CellChangeTimes->{3.593435613964814*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593435613980414*^9}],

Cell[BoxData["\<\"Country: Spain\"\>"], "Print",
 CellChangeTimes->{3.593435613996014*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593435614011614*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593435614744815*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934356154000163`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594659`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934356154312162`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934356154468164`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957241`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574745`", ",", "0.20817745725954456`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886498`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934356154468164`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6552012`6.96688968199476"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6552012`6.96688968199476],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934356154468164`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934356154624166`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934356154624166`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934356160240173`*^9}],

Cell[BoxData["0.9895999999999999`"], "Print",
 CellChangeTimes->{3.5934356160240173`*^9}],

Cell[BoxData["0.0004999999999999999`"], "Print",
 CellChangeTimes->{3.5934356160240173`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934356160240173`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934356160240173`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "6.299811065`9.72799377312027"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   6.299811065`9.72799377312027],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934359940126815`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11487954788751331`", ",", "0.9684675129961908`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11487954788751331`, 0.9684675129961908}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934360323575487`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.938`"}],
  SequenceForm[" R^2: ", 0.938],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934360327007494`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.63908112166666666666666666666666666667`8.734222236978258"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.63908112166666666666666666666666666667`8.734222236978258],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934360327163496`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934360327163496`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934360327163496`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.79114314666666666666666666666666666667`9.181796543904511"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.79114314666666666666666666666666666667`9.181796543904511],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934361398729377`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.120852508088782`", ",", "0.9668288392928449`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.120852508088782, 0.9668288392928449}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934361795750074`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436179918208*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66248116333333333333333333333333333333`8.749839782482299"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66248116333333333333333333333333333333`8.749839782482299],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436179949408*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436179965008*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593436179965008*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05538009666666666666666666666666666667`7.6720199576924175"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05538009666666666666666666666666666667`7.6720199576924175],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436182897813*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12458112310340669`", ",", "0.9658077879862494`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12458112310340669`, 0.9658077879862494}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436223691885*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934362240194855`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.679901195`8.76111205271001"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.679901195`8.76111205271001],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934362240350857`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934362240350857`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934362240350857`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.31174054666666666666666666666666666667`8.422459540862603"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.31174054666666666666666666666666666667`8.422459540862603],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934362423963175`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12470868038638896`", ",", "0.9657683067634077`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12470868038638896`, 0.9657683067634077}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436281848787*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436281848787*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.657021155`8.746245601568186"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.657021155`8.746245601568186],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436281864387*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436281879987*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593436281879987*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.054080095`7.661703694054897"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.054080095`7.661703694054897],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934362850623927`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12537814929595398`", ",", "0.9655865826180601`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12537814929595398`, 0.9655865826180601}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934363246864624`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436324702062*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65468115`8.744696084832087"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.65468115`8.744696084832087],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436324702062*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436324702062*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593436324702062*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05486009666666666666666666666666666667`7.667922816736837"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05486009666666666666666666666666666667`7.667922816736837],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934363276504674`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934363276504674`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934364999839697`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.59343667634228*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1271547017071278`", ",", "0.965095848800015`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1271547017071278, 0.965095848800015}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934367127995443`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934367131271443`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.419411275`9.736161448910227"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.419411275`9.736161448910227],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436713142744*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593436713142744*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593436713142744*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.10478018333333333333333333333333333333`7.948945402210985"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.10478018333333333333333333333333333333`7.948945402210985],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934367190863547`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934367190863547`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934375214893646`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934383262323775`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593438488535063*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057314147057905246`", ",", "0.05883753157714847`", ",", 
   "1.0265795549169412`", ",", "0.0560823617188805`", ",", 
   "0.9785081798778188`", ",", "0.9964970304927697`", ",", 
   "0.9950804060344678`", ",", "0.9949812499696723`", ",", 
   "0.9992447833215429`", ",", "0.9560696761193351`", ",", 
   "0.9420157417748395`", ",", "0.9703017184760865`", ",", 
   "10.285099155699264`"}], "}"}]], "Print",
 CellChangeTimes->{3.593438488535063*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934384889718637`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638763074942212`", ",", "0.8044021989490813`", ",", 
   "0.07998662216099384`", ",", "0.06260754409831416`", ",", 
   "0.47276990939362457`", ",", "0.21064155590764516`", ",", 
   "0.9195592548093078`", ",", "0.9434056727703969`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934384889718637`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593438488987464*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "11.249221025116514`", ",", "28.829798736367607`", ",", 
   "41.89328509635363`", ",", "59.039587668383994`", ",", 
   "65.34639501845562`", ",", "79.33914833664447`", ",", "85.94027182396019`",
    ",", "91.02200815333535`", ",", "97.64595194825098`", ",", 
   "0.5554559399205619`", ",", "37.77882857142857`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934384890030637`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.11840725113757722`"}],
  SequenceForm[" MPC mean:", 0.11840725113757722`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934384890342636`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593438489049864*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.07346575688235868`", ",", "0.0808413829200133`", ",", 
   "0.08607047974606674`", ",", "0.09954008486705401`", ",", 
   "0.10879423203282523`", ",", "0.13268070160184253`", ",", 
   "0.1191087282537373`", ",", "0.1283485421641981`", ",", 
   "0.13303101836819642`", ",", "0.11416893066586697`"}], "}"}]], "Print",
 CellChangeTimes->{3.593438489049864*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934384890810633`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.06111757942736795`", ",", "0.061925744928990945`", ",", 
   "0.06274446406856016`", ",", "0.0646532469735176`", ",", 
   "0.06675643364332018`", ",", "0.19182615461606067`", ",", 
   "0.29296337659756244`", ",", "0.1362308421067474`", ",", 
   "0.1688869609103386`", ",", "0.2548246674582645`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934384890966635`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1116867395231762`", ",", "0.20412452726969033`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.1116867395231762, 0.20412452726969033`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934384891122637`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1265999763692806`", ",", "0.9652474267039981`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1265999763692806, 0.9652474267039981}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593438489143464*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934384891590643`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.49107179833333333333333333333333333333`10.39835680468209"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.49107179833333333333333333333333333333`10.39835680468209],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934384891902637`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.88584410666666666666666666666666666667`10.608873395555579"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   47.88584410666666666666666666666666666667`10.608873395555579],
  Editable->False]], "Print",
 CellChangeTimes->{3.593438489205864*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593438489205864*^9}],

Cell[BoxData["\<\"Country: Finland\"\>"], "Print",
 CellChangeTimes->{3.593438489205864*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593438489221464*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934384899078655`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934384905630665`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594662`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934384905630665`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934384905630665`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957245`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574756`", ",", "0.20817745725954465`", ",", 
   "0.9324174143162659`", ",", "0.9486914927886495`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934384905630665`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6552011`6.966889615710615"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6552011`6.966889615710615],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934384905630665`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934384905630665`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934384905630665`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934384912026677`*^9}],

Cell[BoxData["0.9893`"], "Print",
 CellChangeTimes->{3.5934384912338676`*^9}],

Cell[BoxData["0.0009000000000000001`"], "Print",
 CellChangeTimes->{3.5934384912338676`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934384912338676`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934384912338676`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.75399361833333333333333333333333333333`9.818191687455334"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.75399361833333333333333333333333333333`9.818191687455334],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934389564422846`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10107618088651427`", ",", "0.9722512963247447`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10107618088651427`, 0.9722512963247447}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593438995925954*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.954`"}],
  SequenceForm[" R^2: ", 0.954],
  Editable->False]], "Print",
 CellChangeTimes->{3.593438995972754*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65806115666666666666666666666666666667`8.746932504699883"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65806115666666666666666666666666666667`8.746932504699883],
  Editable->False]], "Print",
 CellChangeTimes->{3.593438995972754*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593438995972754*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934389959883537`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "2.27422399333333333333333333333333333333`9.285499485294647"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   2.27422399333333333333333333333333333333`9.285499485294647],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934391327225943`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11583550878609455`", ",", "0.9682138120955319`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11583550878609455`, 0.9682138120955319}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934391710986614`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934391711142616`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.639861125`8.734751973463881"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.639861125`8.734751973463881],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934391711454616`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934391711454616`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934391711454616`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.697321225`8.772099132637658"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.697321225`8.772099132637658],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934392126103344`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12379728130090387`", ",", "0.9660292225587391`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12379728130090387`, 0.9660292225587391}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934392534980063`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.922`"}],
  SequenceForm[" R^2: ", 0.922],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934392535292063`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68146119666666666666666666666666666667`8.762107379768088"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68146119666666666666666666666666666667`8.762107379768088],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934392535448065`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934392535604067`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934392535604067`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.72670127666666666666666666666666666667`8.790022171376831"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.72670127666666666666666666666666666667`8.790022171376831],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934392971000834`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1262291060771531`", ",", "0.965360300445661`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1262291060771531, 0.965360300445661}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934393367085524`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934393367085524`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65962115833333333333333333333333333333`8.747960826055362"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65962115833333333333333333333333333333`8.747960826055362],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934393367085524`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934393367241526`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934393367241526`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.31694055666666666666666666666666666667`8.429644064589741"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.31694055666666666666666666666666666667`8.429644064589741],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439355709386*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12522989998262885`", ",", "0.9656400588478513`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12522989998262885`, 0.9656400588478513}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934393967686577`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934393967686577`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68458120166666666666666666666666666667`8.764091217623793"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68458120166666666666666666666666666667`8.764091217623793],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439396784258*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439396784258*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593439396784258*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009333333333333333333333333333333`7.653270527830221"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009333333333333333333333333333333`7.653270527830221],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934393999510636`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934393999822636`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439573017768*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439744555669*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12456720040218314`", ",", "0.9658182734881227`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12456720040218314`, 0.9658182734881227}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439780576132*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439780576132*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.33829113333333333333333333333333333333`9.730638431743992"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.33829113333333333333333333333333333333`9.730638431743992],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439780576132*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593439780576132*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593439780576132*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05044008833333333333333333333333333333`7.6314420866766595"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05044008833333333333333333333333333333`7.6314420866766595],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934397832749367`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934397832749367`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593440599764771*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593441418501009*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934415837832994`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05713617377048652`", ",", "0.058599773636372164`", ",", 
   "1.0256159936744251`", ",", "0.05580602687925865`", ",", 
   "0.9767197065632883`", ",", "0.9964961463318364`", ",", 
   "0.9954852536035089`", ",", "0.9952844465368039`", ",", 
   "0.9992006265495479`", ",", "0.9597039254959282`", ",", 
   "0.9459788193127425`", ",", "0.9710414713519309`", ",", 
   "10.292419842285723`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934415837832994`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934415841421003`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09191377159440373`", ",", "0.814794616803596`", ",", 
   "0.07625009436742052`", ",", "0.04122907820347709`", ",", 
   "0.4604180090902635`", ",", "0.1990304415716839`", ",", 
   "0.9262654677432888`", ",", "0.9482932570062335`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934415841421003`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934415841577*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.97319665361498`", ",", "39.25294729685836`", ",", "53.878337978314214`",
    ",", "70.85642414899756`", ",", "76.4260261794894`", ",", 
   "87.30589244348637`", ",", "91.69764168915447`", ",", "94.77855217019115`",
    ",", "98.55116038369836`", ",", "0.6719704615749816`", ",", 
   "24.634444642857144`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934415841577*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.12654051318823123`"}],
  SequenceForm[" MPC mean:", 0.12654051318823123`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934415841733*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934415841889*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09363345856637695`", ",", "0.09743365833882145`", ",", 
   "0.1060983879531584`", ",", "0.12059731351135339`", ",", 
   "0.12957374134765076`", ",", "0.12197581540630553`", ",", 
   "0.1422660213193837`", ",", "0.13251438774735813`", ",", 
   "0.12744792396083693`", ",", "0.13170228689005747`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934415842045*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934415842201*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05756975583706192`", ",", "0.05870508255770868`", ",", 
   "0.059812038727004024`", ",", "0.06248866371081363`", ",", 
   "0.06727644325114002`", ",", "0.21008473449229081`", ",", 
   "0.3053871317359621`", ",", "0.14778765848324837`", ",", 
   "0.1854829250968134`", ",", "0.2710617840619586`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934415842201*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11689493219444436`", ",", "0.24733786539839298`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.11689493219444436`, 0.24733786539839298`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934415842356997`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12457662232336796`", ",", "0.9658135885178765`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12457662232336796`, 0.9658135885178765}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934415842513*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934415842669*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.00873270666666666666666666666666666667`10.405913903416462"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.00873270666666666666666666666666666667`10.405913903416462],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934415842669*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "51.55393055`10.640928030368658"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 51.55393055`10.640928030368658],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934415842669*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934415842669*^9}],

Cell[BoxData["\<\"Country: France\"\>"], "Print",
 CellChangeTimes->{3.5934415842825003`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934415842981005`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934415850781016`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593441585717703*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594662`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934415857645025`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934415857645025`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957245`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574756`", ",", "0.20817745725954465`", ",", 
   "0.9324174143162659`", ",", "0.9486914927886495`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934415857645025`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6552012`6.96688968199476"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6552012`6.96688968199476],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934415857645025`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934415857645025`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934415857645025`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934415862949038`*^9}],

Cell[BoxData["0.98915`"], "Print",
 CellChangeTimes->{3.593441586341704*^9}],

Cell[BoxData["0.0009000000000000001`"], "Print",
 CellChangeTimes->{3.593441586341704*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593441586341704*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934415863573036`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "7.477873135`9.802444341123053"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.477873135`9.802444341123053],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442034967292*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1037079035228311`", ",", "0.9715114450124209`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1037079035228311, 0.9715114450124209}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934420737333603`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.951`"}],
  SequenceForm[" R^2: ", 0.951],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934420737333603`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.646101135`8.738966752177644"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.646101135`8.738966752177644],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934420737333603`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934420737333603`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934420737333603`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.921663375`9.212343561213126"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.921663375`9.212343561213126],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442189033162*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12118994894762863`", ",", "0.9667142613233721`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12118994894762863`, 0.9667142613233721}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934422300768347`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.919`"}],
  SequenceForm[" R^2: ", 0.919],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934422300768347`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68406120166666666666666666666666666667`8.763761207182926"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68406120166666666666666666666666666667`8.763761207182926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934422300768347`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934422300768347`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934422300768347`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.828881455`8.847158671262031"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.828881455`8.847158671262031],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442279809722*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12710015508084424`", ",", "0.9650885154245223`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12710015508084424`, 0.9650885154245223}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442319340191*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.922`"}],
  SequenceForm[" R^2: ", 0.922],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442319355791*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65858115666666666666666666666666666667`8.74727554872945"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65858115666666666666666666666666666667`8.74727554872945],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442319355791*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442319355791*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593442319355791*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.69992123`8.773715414942389"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.69992123`8.773715414942389],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442361319865*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1221766112708154`", ",", "0.966445908575855`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1221766112708154, 0.966445908575855}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442402238737*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.93`"}],
  SequenceForm[" R^2: ", 0.93],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442402238737*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68198119666666666666666666666666666667`8.762438648830344"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68198119666666666666666666666666666667`8.762438648830344],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442402238737*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442402238737*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593442402238737*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009333333333333333333333333333333`7.653270527830221"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009333333333333333333333333333333`7.653270527830221],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934424054211426`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12227450072641069`", ",", "0.9664238161570788`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12227450072641069`, 0.9664238161570788}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934424441716104`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934424441716104`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.645841135`8.738791950919971"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.645841135`8.738791950919971],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934424445148115`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934424445304112`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934424445304112`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05278009333333333333333333333333333333`7.651136402082152"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05278009333333333333333333333333333333`7.651136402082152],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934424473540163`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934424473540163`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934426242583265`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934427990098333`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12355987185757063`", ",", "0.9660701379281187`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12355987185757063`, 0.9660701379281187}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442834000695*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442834000695*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.44437131833333333333333333333333333333`9.737846804304406"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.44437131833333333333333333333333333333`9.737846804304406],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442834000695*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593442834000695*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593442834000695*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05070008833333333333333333333333333333`7.633674964208648"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05070008833333333333333333333333333333`7.633674964208648],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934428370427003`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934428370427003`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934436518945312`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934444672611637`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934446322626534`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057190909839334746`", ",", "0.05865995421106895`", ",", 
   "1.0256866760095469`", ",", "0.05590023383883893`", ",", 
   "0.9774321477989826`", ",", "0.9963341911414202`", ",", 
   "0.995404080239194`", ",", "0.9951756056105548`", ",", 
   "0.9992016572482534`", ",", "0.9588391440557017`", ",", 
   "0.9445362613883278`", ",", "0.9689564564721824`", ",", 
   "10.275555820138264`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934446322626534`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593444632683854*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09069710944251126`", ",", "0.8074838558359169`", ",", 
   "0.05756640945035577`", ",", "0.047878552964394035`", ",", 
   "0.4614933776588907`", ",", "0.19927579624156025`", ",", 
   "0.9227789118592608`", ",", "0.946138069860396`"}], "}"}]], "Print",
 CellChangeTimes->{3.593444632683854*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934446327306547`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.99609451716186`", ",", "37.51369108682043`", ",", "51.843358317850274`",
    ",", "68.71923620696884`", ",", "74.36179410776661`", ",", 
   "85.72386383965438`", ",", "90.53724739067685`", ",", "94.02924483308585`",
    ",", "98.39265029088831`", ",", "0.6503792076717656`", ",", 
   "27.014989285714286`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934446327462544`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.1314110687847111`"}],
  SequenceForm[" MPC mean:", 0.1314110687847111],
  Editable->False]], "Print",
 CellChangeTimes->{3.593444632761854*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593444632761854*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09154586697545952`", ",", "0.09645394986208045`", ",", 
   "0.10400001654547486`", ",", "0.11747939861998213`", ",", 
   "0.1253192884969362`", ",", "0.14048900739464354`", ",", 
   "0.14177433996019506`", ",", "0.13985684042117774`", ",", 
   "0.1402616060751175`", ",", "0.13330366330911303`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934446327774544`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934446327774544`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05814593353841202`", ",", "0.059367287187578444`", ",", 
   "0.06056356752449721`", ",", "0.06368337584676953`", ",", 
   "0.0686942076936281`", ",", "0.21947974164438377`", ",", 
   "0.32323006864119486`", ",", "0.15390609708400094`", ",", 
   "0.19333778545108182`", ",", "0.285691193546879`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934446327774544`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12260626186230217`", ",", "0.2422139227657767`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.12260626186230217`, 0.2422139227657767}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934446327930546`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12393935606131937`", ",", "0.9659644968938613`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12393935606131937`, 0.9659644968938613}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934446327930546`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934446327930546`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.92033255333333333333333333333333333333`10.404632664456015"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.92033255333333333333333333333333333333`10.404632664456015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593444632808655*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "50.77574918333333333333333333333333333333`10.634322588244956"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   50.77574918333333333333333333333333333333`10.634322588244956],
  Editable->False]], "Print",
 CellChangeTimes->{3.593444632808655*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593444632808655*^9}],

Cell[BoxData["\<\"Country: Greece\"\>"], "Print",
 CellChangeTimes->{3.593444632824255*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593444632824255*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593444633526256*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934446342438574`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594664`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934446342438574`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934446342438574`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957246`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886492`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934446342438574`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.7176012`7.006398154112369"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.7176012`7.006398154112369],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934446342438574`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934446342438574`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934446342438574`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934446348210583`*^9}],

Cell[BoxData["0.9898999999999999`"], "Print",
 CellChangeTimes->{3.5934446348366585`*^9}],

Cell[BoxData["0.0002999999999999999`"], "Print",
 CellChangeTimes->{3.593444634867858*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593444634867858*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593444634867858*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "5.37576944166666666666666666666666666667`9.659106882450555"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   5.37576944166666666666666666666666666667`9.659106882450555],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934449573672247`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12114242593733815`", ",", "0.9667474920179974`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12114242593733815`, 0.9667474920179974}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593444996429693*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.931`"}],
  SequenceForm[" R^2: ", 0.931],
  Editable->False]], "Print",
 CellChangeTimes->{3.593444996429693*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65104114333333333333333333333333333333`8.74227468341672"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65104114333333333333333333333333333333`8.74227468341672],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934449967572937`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934449967572937`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593444996772894*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.557402735`9.121067181253535"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.557402735`9.121067181253535],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445090217058*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12489052983938098`", ",", "0.9657125476694457`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12489052983938098`, 0.9657125476694457}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445130168728*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445130168728*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67158118`8.75576476543645"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.67158118`8.75576476543645],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445130168728*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445130184328*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593445130184328*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009333333333333333333333333333333`7.653270527830221"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009333333333333333333333333333333`7.653270527830221],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445133351134*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1264928269970761`", ",", "0.9652678941809695`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1264928269970761, 0.9652678941809695}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934451724760027`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934451724760027`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.652081145`8.742967890930617"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.652081145`8.742967890930617],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934451724760027`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934451724760027`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934451724760027`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05148009`7.640305545703404"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05148009`7.640305545703404],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445175564808*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12794580228324065`", ",", "0.9648702014035307`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12794580228324065`, 0.9648702014035307}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934452162808795`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934452162808795`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67860119166666666666666666666666666667`8.760280866174012"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67860119166666666666666666666666666667`8.760280866174012],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934452162964797`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934452166240797`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934452166240797`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05148009166666666666666666666666666667`7.640305559763676"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05148009166666666666666666666666666667`7.640305559763676],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934452193696847`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12842296154816843`", ",", "0.9647378828431288`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12842296154816843`, 0.9647378828431288}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445260382157*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445260382157*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.6835412`8.763430944725034"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.6835412`8.763430944725034],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445260382157*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445260382157*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593445260382157*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05330009333333333333333333333333333333`7.655394217732188"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05330009333333333333333333333333333333`7.655394217732188],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445263580162*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593445263923363*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445438035269*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934456132391768`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1279825179251285`", ",", "0.9648571158908565`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1279825179251285, 0.9648571158908565}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593445648776039*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934456487916393`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.41993127666666666666666666666666666667`9.736196627324329"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.41993127666666666666666666666666666667`9.736196627324329],
  Editable->False]], "Print",
 CellChangeTimes->{3.59344564911924*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.59344564911924*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59344564911924*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05096009`7.635896434545847"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05096009`7.635896434545847],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934456518336444`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934456518648443`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934464781202955`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934473079481535`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934474753052473`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05725494343720833`", ",", "0.05880923830161666`", ",", 
   "1.0271469111852838`", ",", "0.05595785982829307`", ",", 
   "0.977345473926845`", ",", "0.9965206395116979`", ",", 
   "0.9950574061794418`", ",", "0.9949391965921889`", ",", 
   "0.9992431511670965`", ",", "0.9556156222082277`", ",", 
   "0.9416681485889147`", ",", "0.9708460111385491`", ",", 
   "10.276179187699508`"}], "}"}]], "Print",
 CellChangeTimes->{3.593447475648448*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934474756952477`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09568274506673698`", ",", "0.8145593052027733`", ",", 
   "0.0806307563672049`", ",", "0.04038770834694342`", ",", 
   "0.4724667517765717`", ",", "0.20904322771942246`", ",", 
   "0.9247429771267572`", ",", "0.9456431889759891`"}], "}"}]], "Print",
 CellChangeTimes->{3.593447475710848*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593447475710848*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "10.168705256756375`", ",", "26.771300768664364`", ",", 
   "39.52181659289551`", ",", "56.76840757886611`", ",", 
   "63.256335201186786`", ",", "77.92311219683232`", ",", "84.9676004208339`",
    ",", "90.42815662762757`", ",", "97.52256311110482`", ",", 
   "0.5340892313835441`", ",", "40.0670875`"}], "}"}]], "Print",
 CellChangeTimes->{3.593447475710848*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.09767001129538344`"}],
  SequenceForm[" MPC mean:", 0.09767001129538344],
  Editable->False]], "Print",
 CellChangeTimes->{3.593447475726448*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934474757420483`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.06681972634282829`", ",", "0.06941093820914035`", ",", 
   "0.07732896389274935`", ",", "0.09163288774124234`", ",", 
   "0.1008152678343548`", ",", "0.09293666423931168`", ",", 
   "0.10913098985350012`", ",", "0.1036691457739638`", ",", 
   "0.09869326865425043`", ",", "0.1007684462955929`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934474757420483`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934474757732477`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0620112491303505`", ",", "0.06248376772696307`", ",", 
   "0.06287637624476994`", ",", "0.06392620851812225`", ",", 
   "0.06490830004644965`", ",", "0.14518896605595377`", ",", 
   "0.21570427734190878`", ",", "0.10893801451865548`", ",", 
   "0.12998556176048537`", ",", "0.18848402302394918`"}], "}"}]], "Print",
 CellChangeTimes->{3.593447475788848*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.09059540787476861`", ",", "0.1877988429583053`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.09059540787476861, 0.1877988429583053}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593447475835648*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12780431200120185`", ",", "0.9649042795057022`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12780431200120185`, 0.9649042795057022}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593447475835648*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593447475835648*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.39119337833333333333333333333333333333`10.411414002176759"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.39119337833333333333333333333333333333`10.411414002176759],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934474758512483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.35102316833333333333333333333333333333`10.603995615959576"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   47.35102316833333333333333333333333333333`10.603995615959576],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934474758512483`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934474758512483`*^9}],

Cell[BoxData["\<\"Country: Italy\"\>"], "Print",
 CellChangeTimes->{3.593447475866848*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593447475882448*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934474766156497`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593447477317651*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594664`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.593447477317651*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593447477317651*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957246`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886492`"}], "}"}]], "Print",
 CellChangeTimes->{3.593447477317651*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.7020012`6.996852852344888"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.7020012`6.996852852344888],
  Editable->False]], "Print",
 CellChangeTimes->{3.593447477317651*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593447477317651*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593447477317651*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593447477941652*^9}],

Cell[BoxData["0.989225`"], "Print",
 CellChangeTimes->{3.593447477957252*^9}],

Cell[BoxData["0.0007`"], "Print",
 CellChangeTimes->{3.5934474779728518`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934474779728518`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934474779728518`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "6.63443165333333333333333333333333333333`9.750469972643918"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   6.63443165333333333333333333333333333333`9.750469972643918],
  Editable->False]], "Print",
 CellChangeTimes->{3.593447876054351*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11212379681781359`", ",", "0.9692052303514498`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11212379681781359`, 0.9692052303514498}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934479157252207`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.941`"}],
  SequenceForm[" R^2: ", 0.941],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934479160684214`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66196116333333333333333333333333333333`8.749498758735706"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66196116333333333333333333333333333333`8.749498758735706],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934479160684214`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934479160684214`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934479160684214`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.563122745`9.12265933076657"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.563122745`9.12265933076657],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934480095125856`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12103556469681204`", ",", "0.9667619734672932`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12103556469681204`, 0.9667619734672932}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448050899458*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448050915058*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68978121166666666666666666666666666667`8.767377609049241"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68978121166666666666666666666666666667`8.767377609049241],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934480512426586`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448051258259*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593448051258259*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.08164014333333333333333333333333333333`7.840570006739099"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.08164014333333333333333333333333333333`7.840570006739099],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448055797867*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1261580087638269`", ",", "0.9653573419467049`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1261580087638269, 0.9653573419467049}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934480968259387`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.922`"}],
  SequenceForm[" R^2: ", 0.922],
  Editable->False]], "Print",
 CellChangeTimes->{3.59344809716914*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68380120166666666666666666666666666667`8.763596107877062"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68380120166666666666666666666666666667`8.763596107877062],
  Editable->False]], "Print",
 CellChangeTimes->{3.59344809716914*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934480971847396`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934480971847396`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.407940715`8.53926330096445"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.407940715`8.53926330096445],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934481213023815`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12673661400397734`", ",", "0.9652024318538273`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12673661400397734`, 0.9652024318538273}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448161316452*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448161316452*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66690117166666666666666666666666666667`8.752727728639114"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66690117166666666666666666666666666667`8.752727728639114],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448161316452*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448161316452*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593448161316452*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.14326025166666666666666666666666666667`8.084791957968212"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.14326025166666666666666666666666666667`8.084791957968212],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934481699120674`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12668028592010736`", ",", "0.9652125896403416`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12668028592010736`, 0.9652125896403416}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934482108465395`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934482108465395`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68224119833333333333333333333333333333`8.762604189714148"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68224119833333333333333333333333333333`8.762604189714148],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934482108465395`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934482108465395`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934482108465395`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05902010333333333333333333333333333333`7.699666213749907"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05902010333333333333333333333333333333`7.699666213749907],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448214403345*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934482144189453`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934483875324497`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934485651855617`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12555192957741884`", ",", "0.9655206984021794`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12555192957741884`, 0.9655206984021794}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448600394823*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448600394823*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.4334513`9.737110265996385"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.4334513`9.737110265996385],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448600394823*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593448600394823*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934486004104233`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05018008666666666666666666666666666667`7.629197655270401"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05018008666666666666666666666666666667`7.629197655270401],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934486034368286`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934486034368286`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934494086010427`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450215902461*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593450378953947*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057255904019152294`", ",", "0.058765538639404195`", ",", 
   "1.026366444580928`", ",", "0.05610672644300477`", ",", 
   "0.979929099088836`", ",", "0.996318437618534`", ",", 
   "0.9949955023453585`", ",", "0.9949038332306894`", ",", 
   "0.9992272172901983`", ",", "0.9554088897834775`", ",", 
   "0.9406532265666787`", ",", "0.9682187819476578`", ",", 
   "10.279560591923506`"}], "}"}]], "Print",
 CellChangeTimes->{3.593450378953947*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934503792971478`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09519300419609954`", ",", "0.7942602986207942`", ",", 
   "0.08207556099780433`", ",", "0.041763885875889126`", ",", 
   "0.4711682560620536`", ",", "0.2062443601561619`", ",", 
   "0.9142179648143097`", ",", "0.9409463386061974`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934503792971478`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593450379312748*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.981773826745313`", ",", "31.432792213637782`", ",", 
   "44.72743202931505`", ",", "61.67530818272831`", ",", "67.76595968173447`",
    ",", "81.0049364630632`", ",", "87.12772563127426`", ",", 
   "91.79589871924206`", ",", "97.84391903755497`", ",", 
   "0.5814144539559423`", ",", "35.130225`"}], "}"}]], "Print",
 CellChangeTimes->{3.593450379343948*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.13230259014317258`"}],
  SequenceForm[" MPC mean:", 0.13230259014317258`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450379343948*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593450379343948*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0818959726802394`", ",", "0.09125982063668014`", ",", 
   "0.09417168927973929`", ",", "0.10886052300857219`", ",", 
   "0.1205775042284184`", ",", "0.1496698945076851`", ",", 
   "0.13102311766612285`", ",", "0.14386618275384577`", ",", 
   "0.15102420146650486`", ",", "0.13197334178514153`"}], "}"}]], "Print",
 CellChangeTimes->{3.593450379359548*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593450379359548*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.06025995834375519`", ",", "0.061389909836101264`", ",", 
   "0.06260526398328325`", ",", "0.06560401768231938`", ",", 
   "0.06925428565635627`", ",", "0.22080071638135312`", ",", 
   "0.335307867467742`", ",", "0.1544267608376848`", ",", 
   "0.19397939637399741`", ",", "0.2928910615252466`"}], "}"}]], "Print",
 CellChangeTimes->{3.593450379359548*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12555476885820838`", ",", "0.21829756181805138`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.12555476885820838`, 0.21829756181805138`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450379359548*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12685117062662724`", ",", "0.9651662234375386`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12685117062662724`, 0.9651662234375386}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450379390748*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450379390748*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.59221197666666666666666666666666666667`10.39984367749779"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.59221197666666666666666666666666666667`10.39984367749779],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450379390748*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "48.36060494166666666666666666666666666667`10.613157973017774"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   48.36060494166666666666666666666666666667`10.613157973017774],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450379390748*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593450379390748*^9}],

Cell[BoxData["\<\"Country: Luxembourg\"\>"], "Print",
 CellChangeTimes->{3.593450379390748*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593450379406348*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593450380108349*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934503807635503`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594664`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934503807635503`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934503807635503`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957246`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886492`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934503807635503`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6552012`6.96688968199476"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6552012`6.96688968199476],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934503807791505`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934503807791505`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934503807791505`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934503813407516`*^9}],

Cell[BoxData["0.989375`"], "Print",
 CellChangeTimes->{3.5934503813407516`*^9}],

Cell[BoxData["0.0008`"], "Print",
 CellChangeTimes->{3.5934503813407516`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934503813407516`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934503813407516`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.39649299166666666666666666666666666667`9.79769209829945"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.39649299166666666666666666666666666667`9.79769209829945],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450825130331*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10620616220799195`", ",", "0.970819524100238`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10620616220799195`, 0.970819524100238}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593450863490798*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.948`"}],
  SequenceForm[" R^2: ", 0.948],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934508638339987`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.63934112166666666666666666666666666667`8.734398886856559"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.63934112166666666666666666666666666667`8.734398886856559],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934508638339987`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934508638339987`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934508638339987`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.96586345333333333333333333333333333333`9.222219597152584"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.96586345333333333333333333333333333333`9.222219597152584],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934509814426055`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11903788828246009`", ",", "0.9672982027877204`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11903788828246009`, 0.9672982027877204}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593451020099473*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.922`"}],
  SequenceForm[" R^2: ", 0.922],
  Editable->False]], "Print",
 CellChangeTimes->{3.593451020099473*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64428113166666666666666666666666666667`8.737741661056564"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64428113166666666666666666666666666667`8.737741661056564],
  Editable->False]], "Print",
 CellChangeTimes->{3.593451020115073*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934510201306734`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934510201462736`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.46748082166666666666666666666666666667`8.598430046921097"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.46748082166666666666666666666666666667`8.598430046921097],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934510481483226`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12397581626001941`", ",", "0.9659392211365899`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12397581626001941`, 0.9659392211365899}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934510891451945`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934510895663953`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.6830212`8.7631004319852"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.6830212`8.7631004319852],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934510895663953`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934510895819955`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934510895819955`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.62790110333333333333333333333333333333`8.726557494406894"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.62790110333333333333333333333333333333`8.726557494406894],
  Editable->False]], "Print",
 CellChangeTimes->{3.593451127146861*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1209991981439838`", ",", "0.9667588252981765`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1209991981439838, 0.9667588252981765}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934511686117344`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.93`"}],
  SequenceForm[" R^2: ", 0.93],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934511686117344`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69654122333333333333333333333333333333`8.771613072545469"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69654122333333333333333333333333333333`8.771613072545469],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934511686117344`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934511686117344`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934511686117344`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05226009166666666666666666666666666667`7.646836415380619"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05226009166666666666666666666666666667`7.646836415380619],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934511717473392`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12211439125068771`", ",", "0.9664482661030811`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12211439125068771`, 0.9664482661030811}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593451211106209*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934512114494095`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65598115166666666666666666666666666667`8.745557609169403"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65598115166666666666666666666666666667`8.745557609169403],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934512114494095`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934512114494095`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934512114494095`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05226009166666666666666666666666666667`7.646836415380619"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05226009166666666666666666666666666667`7.646836415380619],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934512142418146`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593451214273014*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593451386107316*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593451559673221*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12548186616697488`", ",", "0.9655236672408978`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12548186616697488`, 0.9655236672408978}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934515948044825`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934515948044825`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.34271114`9.730941181407099"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.34271114`9.730941181407099],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934515948044825`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934515948044825`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934515948044825`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.20384035833333333333333333333333333333`8.237956422322874"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.20384035833333333333333333333333333333`8.237956422322874],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934516070349045`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934516070349045`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934524175187273`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934532303737555`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593453395266045*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057300214499589636`", ",", "0.058760561496577525`", ",", 
   "1.0254858905807123`", ",", "0.056125663739292404`", ",", 
   "0.9795018086659056`", ",", "0.9965172762600704`", ",", 
   "0.9952534565907466`", ",", "0.9951321554351831`", ",", 
   "0.9992357441962886`", ",", "0.9575831034467934`", ",", 
   "0.9436812450878463`", ",", "0.9708683308245988`", ",", 
   "10.265016019922902`"}], "}"}]], "Print",
 CellChangeTimes->{3.593453395266045*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934533953284454`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09736577493281916`", ",", "0.8129975808248501`", ",", 
   "0.07844790163042495`", ",", "0.04583422106198928`", ",", 
   "0.47108844609722916`", ",", "0.2103221554324694`", ",", 
   "0.9233516724776238`", ",", "0.9448709907286539`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934533953284454`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934533953284454`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.62171666044101`", ",", "37.116782184112076`", ",", "51.56384115696674`",
    ",", "68.70270248085524`", ",", "74.44395743128253`", ",", 
   "85.94716451495367`", ",", "90.73887827377321`", ",", "94.15983919087138`",
    ",", "98.38220065649274`", ",", "0.6508166157853171`", ",", 
   "26.6123125`"}], "}"}]], "Print",
 CellChangeTimes->{3.593453395359645*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.11903076439994487`"}],
  SequenceForm[" MPC mean:", 0.11903076439994487`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453395375245*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593453395375245*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.08686548011443573`", ",", "0.09023740832198568`", ",", 
   "0.09891408141798463`", ",", "0.11355274508344815`", ",", 
   "0.12262081470374853`", ",", "0.1136250455943999`", ",", 
   "0.13244467040004837`", ",", "0.12489367035645582`", ",", 
   "0.1194569737607809`", ",", "0.12250203715902708`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934533953908453`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934533953908453`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05838501197484691`", ",", "0.05948568509172958`", ",", 
   "0.060509352175780906`", ",", "0.06294605557057131`", ",", 
   "0.0666125552484843`", ",", "0.19347937244034252`", ",", 
   "0.28336515233903614`", ",", "0.13774132826120022`", ",", 
   "0.1710667482410554`", ",", "0.2507625341037024`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934533953908453`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11013112148087345`", ",", "0.2310478970478751`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.11013112148087345`, 0.2310478970478751}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934533954376454`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12513317225653914`", ",", "0.9656198072091762`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12513317225653914`, 0.9656198072091762}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934533954376454`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934533954376454`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.80411234833333333333333333333333333333`10.40294244004255"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.80411234833333333333333333333333333333`10.40294244004255],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934533954376454`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "50.241968245`10.629732892901753"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 50.241968245`10.629732892901753],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934533954376454`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593453395453245*^9}],

Cell[BoxData["\<\"Country: Malta\"\>"], "Print",
 CellChangeTimes->{3.593453395453245*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934533954688454`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593453396170847*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934533968884478`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594659`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934533968884478`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934533968884478`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957241`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574745`", ",", "0.20817745725954456`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886498`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934533968884478`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.7176012`7.006398154112369"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.7176012`7.006398154112369],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934533968884478`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934533968884478`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593453396904048*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934533974500484`*^9}],

Cell[BoxData["0.9897499999999999`"], "Print",
 CellChangeTimes->{3.5934533974656487`*^9}],

Cell[BoxData["0.0004999999999999999`"], "Print",
 CellChangeTimes->{3.5934533974656487`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934533974656487`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934533974656487`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "6.567091535`9.746039317932242"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   6.567091535`9.746039317932242],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453791475541*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11462876151549206`", ",", "0.9685249902815989`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11462876151549206`, 0.9685249902815989}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453831817212*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.938`"}],
  SequenceForm[" R^2: ", 0.938],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453831817212*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67236118`8.756268878922267"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.67236118`8.756268878922267],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934538318328123`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934538318328123`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934538318328123`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.80258316666666666666666666666666666667`9.184561559327589"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.80258316666666666666666666666666666667`9.184561559327589],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453939972202*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11976404567179572`", ",", "0.9671123772247671`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11976404567179572`, 0.9671123772247671}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453979986272*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453979986272*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66690117166666666666666666666666666667`8.752727728639114"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66690117166666666666666666666666666667`8.752727728639114],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453979986272*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593453979986272*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593453979986272*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05226009166666666666666666666666666667`7.646836415380619"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05226009166666666666666666666666666667`7.646836415380619],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934539831218777`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12812769884439548`", ",", "0.9648120666104412`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12812769884439548`, 0.9648120666104412}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59345402460235*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.92`"}],
  SequenceForm[" R^2: ", 0.92],
  Editable->False]], "Print",
 CellChangeTimes->{3.59345402460235*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69134121333333333333333333333333333333`8.768358695724036"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69134121333333333333333333333333333333`8.768358695724036],
  Editable->False]], "Print",
 CellChangeTimes->{3.59345402460235*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.59345402460235*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59345402460235*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.52390092166666666666666666666666666667`8.647915410691818"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.52390092166666666666666666666666666667`8.647915410691818],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934540560364056`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12742086562790617`", ",", "0.9650042128657994`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12742086562790617`, 0.9650042128657994}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934540958008757`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934540958008757`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66274116333333333333333333333333333333`8.750010193989066"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66274116333333333333333333333333333333`8.750010193989066],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934540958008757`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934540958008757`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934540958008757`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05252009166666666666666666666666666667`7.648991723635601"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05252009166666666666666666666666666667`7.648991723635601],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934540989520807`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12759427648909696`", ",", "0.9649512437805751`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12759427648909696`, 0.9649512437805751}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454139902153*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454139902153*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.6825012`8.762769667522448"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.6825012`8.762769667522448],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454139902153*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454139902153*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593454139902153*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05278009166666666666666666666666666667`7.651136388368189"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05278009166666666666666666666666666667`7.651136388368189],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934541430689583`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934541431001587`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454318101266*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454497454781*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12640904480915627`", ",", "0.9652773264816261`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12640904480915627`, 0.9652773264816261}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934545341772456`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934545341928453`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.51847145`9.742812015932525"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.51847145`9.742812015932525],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454534520446*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454534520446*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593454534520446*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05148009`7.640305545703404"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05148009`7.640305545703404],
  Editable->False]], "Print",
 CellChangeTimes->{3.593454537266051*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593454537266051*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934553760639243`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934562175762024`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593456387554101*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05724517443024536`", ",", "0.05880429800754608`", ",", 
   "1.0272358953015428`", ",", "0.05593251225780358`", ",", 
   "0.9770694703002207`", ",", "0.9964937259005157`", ",", 
   "0.99505721474927`", ",", "0.9949560631651985`", ",", 
   "0.9992460309694653`", ",", "0.9555360071190918`", ",", 
   "0.9414275577849639`", ",", "0.970286670095469`", ",", 
   "10.271858292995823`"}], "}"}]], "Print",
 CellChangeTimes->{3.593456387554101*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934563875853004`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1003555654032666`", ",", "0.8096345744557462`", ",", 
   "0.07507525275855872`", ",", "0.052556034721310145`", ",", 
   "0.47480683729802803`", ",", "0.21079850797879252`", ",", 
   "0.9237464111616686`", ",", "0.9451761923906751`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934563875853004`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593456387632101*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.482003688583333`", ",", "30.93130148429023`", ",", "44.34500118649432`",
    ",", "61.536331402321466`", ",", "67.71413544038573`", ",", 
   "81.08174572109787`", ",", "87.20211499134278`", ",", "91.82997446474049`",
    ",", "97.7931076942059`", ",", "0.579989105582384`", ",", 
   "34.52735178571429`"}], "}"}]], "Print",
 CellChangeTimes->{3.593456387632101*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.10281573382580533`"}],
  SequenceForm[" MPC mean:", 0.10281573382580533`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593456387647701*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934563876633005`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.07080835060892154`", ",", "0.07433394964895323`", ",", 
   "0.08279827165480891`", ",", "0.09739054743734432`", ",", 
   "0.10651872866893597`", ",", "0.09723967487674623`", ",", 
   "0.11387066109814581`", ",", "0.10867239063663825`", ",", 
   "0.10325230331494473`", ",", "0.1051224568118605`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934563876633005`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593456387694501*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.06039865788193388`", ",", "0.06121227976021004`", ",", 
   "0.06191409245430035`", ",", "0.06358568687147959`", ",", 
   "0.06531756040720005`", ",", "0.15693324300835332`", ",", 
   "0.23341344371924866`", ",", "0.11600200879283051`", ",", 
   "0.14005447123821113`", ",", "0.20414115511328912`"}], "}"}]], "Print",
 CellChangeTimes->{3.593456387694501*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.09533201170293615`", ",", "0.19790164805159172`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.09533201170293615, 0.19790164805159172`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593456387710101*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12720263523459313`", ",", "0.9650605077305879`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12720263523459313`, 0.9650605077305879}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593456387710101*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.593456387710101*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.83813416333333333333333333333333333333`10.417754341748834"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.83813416333333333333333333333333333333`10.417754341748834],
  Editable->False]], "Print",
 CellChangeTimes->{3.593456387710101*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "49.84442754666666666666666666666666666667`10.626282860792948"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   49.84442754666666666666666666666666666667`10.626282860792948],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934563877257013`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934563877257013`*^9}],

Cell[BoxData["\<\"Country: Netherland\"\>"], "Print",
 CellChangeTimes->{3.5934563877257013`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934563877257013`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934563884277024`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934563890985036`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594659`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934563890985036`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934563890985036`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957241`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574745`", ",", "0.20817745725954456`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886498`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934563890985036`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934563890985036`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934563890985036`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934563890985036`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934563897069044`*^9}],

Cell[BoxData["0.9895999999999999`"], "Print",
 CellChangeTimes->{3.5934563897069044`*^9}],

Cell[BoxData["0.0007`"], "Print",
 CellChangeTimes->{3.5934563897069044`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934563897069044`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934563897069044`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "7.24049271833333333333333333333333333333`9.788434369326074"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   7.24049271833333333333333333333333333333`9.788434369326074],
  Editable->False]], "Print",
 CellChangeTimes->{3.593456824136468*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10800823396053949`", ",", "0.9703605137262734`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10800823396053949`, 0.9703605137262734}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934568631989365`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.946`"}],
  SequenceForm[" R^2: ", 0.946],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934568631989365`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65104114333333333333333333333333333333`8.74227468341672"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65104114333333333333333333333333333333`8.74227468341672],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934568631989365`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934568631989365`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934568631989365`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "2.218063895`9.274640300780005"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   2.218063895`9.274640300780005],
  Editable->False]], "Print",
 CellChangeTimes->{3.59345699631397*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11791089197619342`", ",", "0.9676515892329733`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11791089197619342`, 0.9676515892329733}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934570354856386`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934570358288393`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65338114833333333333333333333333333333`8.74383284806286"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65338114833333333333333333333333333333`8.74383284806286],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934570358288393`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934570358288393`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934570358288393`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.42822075166666666666666666666666666667`8.560333957684911"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.42822075166666666666666666666666666667`8.560333957684911],
  Editable->False]], "Print",
 CellChangeTimes->{3.593457061178884*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12464214269202994`", ",", "0.9658061651655078`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12464214269202994`, 0.9658061651655078}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571008497534`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571008497534`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66118116166666666666666666666666666667`8.748986719399424"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66118116166666666666666666666666666667`8.748986719399424],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571008653536`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571008653536`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934571008653536`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.56212098666666666666666666666666666667`8.678496048102739"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.56212098666666666666666666666666666667`8.678496048102739],
  Editable->False]], "Print",
 CellChangeTimes->{3.593457134577013*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12559500137696258`", ",", "0.9655369938089216`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12559500137696258`, 0.9655369938089216}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571754490843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571754490843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68120119666666666666666666666666666667`8.761941650432314"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68120119666666666666666666666666666667`8.761941650432314],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571754490843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571754490843`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934571754646845`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.15184026666666666666666666666666666667`8.110053206021188"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.15184026666666666666666666666666666667`8.110053206021188],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934571845595007`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12501918215853297`", ",", "0.9656981472849644`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12501918215853297`, 0.9656981472849644}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934572265079746`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.927`"}],
  SequenceForm[" R^2: ", 0.927],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934572265079746`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69914122833333333333333333333333333333`8.77323116142992"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69914122833333333333333333333333333333`8.77323116142992],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934572265235744`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934572265235744`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934572265235744`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05278009333333333333333333333333333333`7.651136402082152"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05278009333333333333333333333333333333`7.651136402082152],
  Editable->False]], "Print",
 CellChangeTimes->{3.59345722967478*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.59345722967478*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593457410791098*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934575903786135`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.125269050339494`", ",", "0.9656267410969968`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.125269050339494, 0.9656267410969968}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593457628629881*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934576286454806`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.64379166833333333333333333333333333333`9.751082253886652"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.64379166833333333333333333333333333333`9.751082253886652],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934576286454806`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934576286454806`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934576286454806`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05018008833333333333333333333333333333`7.62919766969493"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05018008833333333333333333333333333333`7.62919766969493],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934576313130856`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934576313130856`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593458438708103*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934592474135237`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593459411151411*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05723967307939702`", ",", "0.058731347608151865`", ",", 
   "1.0260601510893632`", ",", "0.055987117986695895`", ",", 
   "0.9781173611707441`", ",", "0.9965172152740172`", ",", 
   "0.9952414593904214`", ",", "0.9950957613095412`", ",", 
   "0.9992281403019405`", ",", "0.9576813100499905`", ",", 
   "0.9438732385044968`", ",", "0.9712281791179126`", ",", 
   "10.296299637869026`"}], "}"}]], "Print",
 CellChangeTimes->{3.593459411151411*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593459411151411*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09365143623828497`", ",", "0.8176896933130102`", ",", 
   "0.0759796844738666`", ",", "0.044902737716284266`", ",", 
   "0.46716418743034144`", ",", "0.2066003177526708`", ",", 
   "0.9255874172991441`", ",", "0.9467345710855527`"}], "}"}]], "Print",
 CellChangeTimes->{3.593459411151411*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934594111670113`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.713282882248985`", ",", "35.17885798223668`", ",", "49.30363267363013`",
    ",", "66.4776944881298`", ",", "72.36894293184169`", ",", 
   "84.47802925752174`", ",", "89.67959718396268`", ",", "93.45880558666259`",
    ",", "98.1917908646347`", ",", "0.629062823651336`", ",", 
   "28.878991071428572`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934594111670113`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.11210819596820809`"}],
  SequenceForm[" MPC mean:", 0.11210819596820809`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934594111670113`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934594111670113`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.07965634830042234`", ",", "0.08354373824570904`", ",", 
   "0.09221770590337686`", ",", "0.1067967671296578`", ",", 
   "0.11583077717099366`", ",", "0.10650229942215139`", ",", 
   "0.12415648714585081`", ",", "0.11791730710954929`", ",", 
   "0.11235318683393203`", ",", "0.11470215562327601`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934594111670113`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934594114946117`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05875317159524884`", ",", "0.05970205073863266`", ",", 
   "0.06062798576651396`", ",", "0.06285187412134197`", ",", 
   "0.06568877661029293`", ",", "0.17846711237805635`", ",", 
   "0.263242912114391`", ",", "0.1286195711180983`", ",", 
   "0.15821283835642852`", ",", "0.23221076651869488`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934594114946117`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10376007740250504`", ",", "0.2175771722613824`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.10376007740250504`, 0.2175771722613824}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934594114946117`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12506316033992215`", ",", "0.9656837977718989`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12506316033992215`, 0.9656837977718989}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934594114946117`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593459411510212*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.66397210166666666666666666666666666667`10.400895552235525"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.66397210166666666666666666666666666667`10.400895552235525],
  Editable->False]], "Print",
 CellChangeTimes->{3.593459411510212*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "50.367548465`10.63081706121285"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 50.367548465`10.63081706121285],
  Editable->False]], "Print",
 CellChangeTimes->{3.593459411510212*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593459411510212*^9}],

Cell[BoxData["\<\"Country: Portugal\"\>"], "Print",
 CellChangeTimes->{3.593459411510212*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593459411525812*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934594122278132`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934594128986144`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132316`", ",", "0.059046432572672634`", ",", 
   "1.0278100580633`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060691`", ",", "0.9965784191812694`", ",", 
   "0.9949659616594662`", ",", "0.994831556634029`", ",", 
   "0.9992603749621536`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108294`", ",", "0.9722791946884406`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934594128986144`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934594128986144`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744226`", ",", "0.8432676445957245`", ",", 
   "0.06012135698108243`", ",", "0.047622883260817664`", ",", 
   "0.47933907823574756`", ",", "0.20817745725954465`", ",", 
   "0.9324174143162659`", ",", "0.9486914927886495`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934594128986144`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934594128986144`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934594128986144`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934594128986144`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934594135070157`*^9}],

Cell[BoxData["0.9896299999999999`"], "Print",
 CellChangeTimes->{3.5934594135070157`*^9}],

Cell[BoxData["0.0006`"], "Print",
 CellChangeTimes->{3.5934594135070157`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934594135070157`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934594135070157`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "6.86687206`9.765425204135594"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   6.86687206`9.765425204135594],
  Editable->False]], "Print",
 CellChangeTimes->{3.593459825519339*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1114914923354563`", ",", "0.969374301956432`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1114914923354563, 0.969374301956432}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593459866469411*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.942`"}],
  SequenceForm[" R^2: ", 0.942],
  Editable->False]], "Print",
 CellChangeTimes->{3.593459866469411*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68250119833333333333333333333333333333`8.7627696664619"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68250119833333333333333333333333333333`8.7627696664619],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934598667970114`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934598667970114`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934598667970114`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.75708308666666666666666666666666666667`9.173458546550819"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.75708308666666666666666666666666666667`9.173458546550819],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934599719099965`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11789070604967727`", ",", "0.9676155166418147`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11789070604967727`, 0.9676155166418147}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460011144065*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460011144065*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65416114833333333333333333333333333333`8.7443509953334"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65416114833333333333333333333333333333`8.7443509953334],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460011144065*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460011144065*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593460011144065*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05174009166666666666666666666666666667`7.642493442025847"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05174009166666666666666666666666666667`7.642493442025847],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460014248471*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1250910521288158`", ",", "0.9656343994549137`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1250910521288158, 0.9656343994549137}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934600564153447`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.922`"}],
  SequenceForm[" R^2: ", 0.922],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460056742945*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.702781235`8.775486405073107"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.702781235`8.775486405073107],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460056742945*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934600567585454`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934600567585454`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.56472099166666666666666666666666666667`8.680500179737809"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.56472099166666666666666666666666666667`8.680500179737809],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934600902986045`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12726518010474464`", ",", "0.9650373391628607`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12726518010474464`, 0.9650373391628607}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934601309054756`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.923`"}],
  SequenceForm[" R^2: ", 0.923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934601309054756`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67678118833333333333333333333333333333`8.759114526856866"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67678118833333333333333333333333333333`8.759114526856866],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934601309054756`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934601309054756`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593460130921076*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.30550053666666666666666666666666666667`8.413678225711283"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.30550053666666666666666666666666666667`8.413678225711283],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934601492511077`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12605561734106996`", ",", "0.9653704070540507`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12605561734106996`, 0.9653704070540507}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59346019007638*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.59346019007638*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.680681195`8.761610000846755"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.680681195`8.761610000846755],
  Editable->False]], "Print",
 CellChangeTimes->{3.59346019007638*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.59346019007638*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59346019007638*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05200009166666666666666666666666666667`7.644670357432977"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05200009166666666666666666666666666667`7.644670357432977],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460193196385*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593460193196385*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460370865097*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460553447818*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12657695089134385`", ",", "0.9652251724171462`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12657695089134385`, 0.9652251724171462}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460589359081*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460589359081*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.60271159666666666666666666666666666667`9.748388576124466"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.60271159666666666666666666666666666667`9.748388576124466],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460589359081*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593460589359081*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593460589359081*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05122009`7.6381065854575505"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05122009`7.6381065854575505],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934605924322863`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934605924322863`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593461400732106*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934622123547316`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934623756090183`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05722391041669965`", ",", "0.05875809627363977`", ",", 
   "1.0268102240089554`", ",", "0.0559097734182736`", ",", 
   "0.9770351765746762`", ",", "0.9965277343514425`", ",", 
   "0.9951606971240584`", ",", "0.9950380347412827`", ",", 
   "0.9992455857807463`", ",", "0.9564177877078838`", ",", 
   "0.9425110229284985`", ",", "0.9709123151498485`", ",", 
   "10.263553751580236`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934623756090183`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593462375952219*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.10384050797862099`", ",", "0.8175416654855923`", ",", 
   "0.07788591806635174`", ",", "0.049716006441344204`", ",", 
   "0.47338032056266377`", ",", "0.21044725391475774`", ",", 
   "0.9257339989362983`", ",", "0.9459381984832869`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934623759678187`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934623759678187`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.384011544919561`", ",", "32.924910164038174`", ",", 
   "46.75682708430383`", ",", "63.99949345900949`", ",", "70.04737108472592`",
    ",", "82.79838509190789`", ",", "88.46070812275273`", ",", 
   "92.65922709001892`", ",", "97.98838052995379`", ",", 
   "0.6043071256281348`", ",", "31.563089285714284`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934623759834185`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.10703939433687482`"}],
  SequenceForm[" MPC mean:", 0.10703939433687482`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462376030219*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593462376030219*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0754049677597669`", ",", "0.0785349443820067`", ",", 
   "0.08716302995850944`", ",", "0.1018692658911311`", ",", 
   "0.11096878504081154`", ",", "0.10112090343258318`", ",", 
   "0.118136978022511`", ",", "0.11283773105454398`", ",", 
   "0.10719033258050825`", ",", "0.109132661567939`"}], "}"}]], "Print",
 CellChangeTimes->{3.593462376030219*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934623760458193`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05973192022060225`", ",", "0.06063117467754764`", ",", 
   "0.06145500122787362`", ",", "0.06341203845121457`", ",", 
   "0.06567041991902167`", ",", "0.16649793363479293`", ",", 
   "0.2469360709820324`", ",", "0.12170156302396251`", ",", 
   "0.14820894044613386`", ",", "0.21663437821637854`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934623760458193`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.0992216589717696`", ",", "0.20614987487688108`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.0992216589717696, 0.20614987487688108`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462376061419*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12692973251517023`", ",", "0.9651263474262759`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12692973251517023`, 0.9651263474262759}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462376092619*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462376092619*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.7196122`10.4017093863964"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.7196122`10.4017093863964],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462376092619*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "49.37850672833333333333333333333333333333`10.622204200367827"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   49.37850672833333333333333333333333333333`10.622204200367827],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462376108219*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593462376108219*^9}],

Cell[BoxData["\<\"Country: Slovenia\"\>"], "Print",
 CellChangeTimes->{3.593462376108219*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934623761394196`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934623767946205`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593462377481022*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594664`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.593462377481022*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593462377481022*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957246`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886492`"}], "}"}]], "Print",
 CellChangeTimes->{3.593462377481022*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6864012`6.98709303192806"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6864012`6.98709303192806],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462377481022*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934623774966216`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934623774966216`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934623780426226`*^9}],

Cell[BoxData["0.9898999999999999`"], "Print",
 CellChangeTimes->{3.5934623780426226`*^9}],

Cell[BoxData["0.0002999999999999999`"], "Print",
 CellChangeTimes->{3.5934623780426226`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934623780426226`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934623780582223`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "5.32740935666666666666666666666666666667`9.65518231735093"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   5.32740935666666666666666666666666666667`9.65518231735093],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934626976871843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12046220097414162`", ",", "0.9669355780374211`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12046220097414162`, 0.9669355780374211}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462736765252*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.932`"}],
  SequenceForm[" R^2: ", 0.932],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462736765252*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65130114333333333333333333333333333333`8.742448088777126"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65130114333333333333333333333333333333`8.742448088777126],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934627367808523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934627367808523`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934627367808523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.60940282833333333333333333333333333333`9.135331008427942"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.60940282833333333333333333333333333333`9.135331008427942],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462833329422*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12407046828716917`", ",", "0.9659362195689223`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12407046828716917`, 0.9659362195689223}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934628718146896`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934628718146896`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64142112666666666666666666666666666667`8.735809508522193"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64142112666666666666666666666666666667`8.735809508522193],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934628718146896`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934628718146896`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934628718146896`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009166666666666666666666666666667`7.653270514183484"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009166666666666666666666666666667`7.653270514183484],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462874997095*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1252297049610518`", ",", "0.9656150330675457`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1252297049610518, 0.9656150330675457}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934629166803684`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934629166803684`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69472122166666666666666666666666666667`8.7704768136971"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69472122166666666666666666666666666667`8.7704768136971],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934629166959686`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934629166959686`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934629166959686`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05278009166666666666666666666666666667`7.651136388368189"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05278009166666666666666666666666666667`7.651136388368189],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934629198471737`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12738284367063235`", ",", "0.9650214941344383`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12738284367063235`, 0.9650214941344383}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462960079645*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462960079645*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67054117833333333333333333333333333333`8.755091701505128"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67054117833333333333333333333333333333`8.755091701505128],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462960079645*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593462960079645*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593462960079645*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.053820095`7.6596107082345855"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.053820095`7.6596107082345855],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934629633088503`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12726704430282965`", ",", "0.9650530825143927`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12726704430282965`, 0.9650530825143927}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593463003837722*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593463003837722*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.674961185`8.757945046787386"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.674961185`8.757945046787386],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934630038533216`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934630038533216`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934630038689213`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05356009333333333333333333333333333333`7.657507573353891"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05356009333333333333333333333333333333`7.657507573353891],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934630070201273`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934630070357275`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934631850264397`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593463365050756*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12713718276045582`", ",", "0.9650863843190902`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12713718276045582`, 0.9650863843190902}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934634005876184`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934634006188183`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.55945818833333333333333333333333333333`9.745534216320722"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.55945818833333333333333333333333333333`9.745534216320722],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934634006188183`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934634006188183`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934634006188183`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05096009`7.635896434545847"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05096009`7.635896434545847],
  Editable->False]], "Print",
 CellChangeTimes->{3.593463403660824*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593463403660824*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934642028502274`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.59346500693804*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934651689287243`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057333579507918286`", ",", "0.0588642512042606`", ",", 
   "1.0266976475126748`", ",", "0.05612901065785158`", ",", 
   "0.9789901684073232`", ",", "0.9965403571054212`", ",", 
   "0.9950035422169551`", ",", "0.9948966867769601`", ",", 
   "0.9992512313589669`", ",", "0.9553224706783903`", ",", 
   "0.9414417795721616`", ",", "0.9710473152329662`", ",", 
   "10.27477590893806`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934651689287243`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934651689287243`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09651863690962516`", ",", "0.8141600041114402`", ",", 
   "0.07533813731417935`", ",", "0.04529213878327446`", ",", 
   "0.474503802995825`", ",", "0.2114299884549918`", ",", 
   "0.9248170074880413`", ",", "0.9454931421230026`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934651689287243`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5934651689287243`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "10.279106041844171`", ",", "26.866639909315982`", ",", 
   "39.58599497981355`", ",", "56.80171151149749`", ",", "63.28084634254183`",
    ",", "77.93655559560393`", ",", "84.97882263515366`", ",", 
   "90.44001434526989`", ",", "97.52604400026208`", ",", 
   "0.5345303357093494`", ",", "40.10605892857143`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934651689287243`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.09773765274681512`"}],
  SequenceForm[" MPC mean:", 0.09773765274681512],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465169287525*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5934651693031254`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.06683163759322819`", ",", "0.06946885755059606`", ",", 
   "0.07740012992218492`", ",", "0.0917476927415114`", ",", 
   "0.10096343919044282`", ",", "0.09288270648280872`", ",", 
   "0.10919674957513326`", ",", "0.10372977468445754`", ",", 
   "0.09874467204342197`", ",", "0.10082100084255319`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934651693031254`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593465169349925*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.06204581847336221`", ",", "0.06248203133222463`", ",", 
   "0.0628695155023955`", ",", "0.06392662734286436`", ",", 
   "0.06491101933547871`", ",", "0.14534752582158872`", ",", 
   "0.21599043322713685`", ",", "0.10902892619767823`", ",", 
   "0.13011577420816167`", ",", "0.1887236982247842`"}], "}"}]], "Print",
 CellChangeTimes->{3.593465169349925*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.09065521502379492`", ",", "0.18796181963799674`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.09065521502379492, 0.18796181963799674`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465169349925*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1272010004880208`", ",", "0.9650674756357387`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1272010004880208, 0.9650674756357387}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934651693655252`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934651693655252`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.421391675`10.397329459859298"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.421391675`10.397329459859298],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934651693655252`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "46.52412838166666666666666666666666666667`10.596344493692195"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   46.52412838166666666666666666666666666667`10.596344493692195],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934651693811255`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5934651693811255`*^9}],

Cell[BoxData["\<\"Country: Slovakia\"\>"], "Print",
 CellChangeTimes->{3.5934651693811255`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934651693811255`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5934651700675263`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5934651707383275`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05744877870132317`", ",", "0.05904643257267263`", ",", 
   "1.0278100580632998`", ",", "0.056099706021112646`", ",", 
   "0.9765169476060689`", ",", "0.9965784191812693`", ",", 
   "0.9949659616594659`", ",", "0.994831556634029`", ",", 
   "0.9992603749621527`", ",", "0.9545285336164773`", ",", 
   "0.9408156515108295`", ",", "0.9722791946884403`", ",", 
   "10.259205717381175`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934651707383275`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5934651707383275`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09638479524744224`", ",", "0.8432676445957241`", ",", 
   "0.06012135698108244`", ",", "0.04762288326081762`", ",", 
   "0.47933907823574734`", ",", "0.20817745725954453`", ",", 
   "0.9324174143162658`", ",", "0.9486914927886498`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934651707383275`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934651707383275`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5934651707383275`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5934651707383275`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5934651713311286`*^9}],

Cell[BoxData["0.9899749999999999`"], "Print",
 CellChangeTimes->{3.5934651713311286`*^9}],

Cell[BoxData[
 RowBox[{"-", "8.131516293641283`*^-20"}]], "Print",
 CellChangeTimes->{3.5934651713311286`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934651713311286`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593465171346729*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "2.001223515`9.229961845552499"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   2.001223515`9.229961845552499],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934652914045396`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12584682680211068`", ",", "0.9653709810060364`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12584682680211068`, 0.9653709810060364}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934653301394076`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.926`"}],
  SequenceForm[" R^2: ", 0.926],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934653301394076`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.645581135`8.738617079277374"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.645581135`8.738617079277374],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934653301394076`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934653301394076`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934653301550074`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.63852287666666666666666666666666666667`9.14311875749261"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.63852287666666666666666666666666666667`9.14311875749261],
  Editable->False]], "Print",
 CellChangeTimes->{3.59346542845078*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12656959270797763`", ",", "0.9651874896863823`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12656959270797763`, 0.9651874896863823}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465467201248*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465467201248*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.645841135`8.738791950919971"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.645841135`8.738791950919971],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465467201248*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465467201248*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593465467201248*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05200009166666666666666666666666666667`7.644670357432977"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05200009166666666666666666666666666667`7.644670357432977],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465470321254*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12701453710981941`", ",", "0.9650728075233961`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12701453710981941`, 0.9650728075233961}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655100545235`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655100545235`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66222116333333333333333333333333333333`8.749669304082026"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66222116333333333333333333333333333333`8.749669304082026],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655100545235`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655100545235`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934655100545235`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009333333333333333333333333333333`7.653270527830221"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009333333333333333333333333333333`7.653270527830221],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465513236929*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12736505819621277`", ",", "0.9649805655897205`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12736505819621277`, 0.9649805655897205}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655533445997`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.924`"}],
  SequenceForm[" R^2: ", 0.924],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655533602*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66846117333333333333333333333333333333`8.75374243513632"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66846117333333333333333333333333333333`8.75374243513632],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655533602*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655533602*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934655533602*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009333333333333333333333333333333`7.653270527830221"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009333333333333333333333333333333`7.653270527830221],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465556527005*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12743711082430914`", ",", "0.9649644870822254`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12743711082430914`, 0.9649644870822254}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655981946783`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655981946783`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69446122`8.770314247158607"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.69446122`8.770314247158607],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655982102785`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934655982102785`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5934655982102785`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05174009`7.642493428036228"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05174009`7.642493428036228],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934656012990837`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934656012990837`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465777314193*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934659568861084`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1272685466292364`", ",", "0.9650122852453833`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1272685466292364, 0.9650122852453833}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465993405772*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465993405772*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.53511147833333333333333333333333333333`9.743919248550263"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.53511147833333333333333333333333333333`9.743919248550263],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465993405772*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593465993405772*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593465993405772*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05044008833333333333333333333333333333`7.6314420866766595"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05044008833333333333333333333333333333`7.6314420866766595],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934659964321775`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5934659964477777`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934667967291837`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593467596324188*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593467758080872*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05734124641117555`", ",", "0.05886428355559567`", ",", 
   "1.026560935447739`", ",", "0.05615207017393893`", ",", 
   "0.9792614163161119`", ",", "0.9965388845439106`", ",", 
   "0.9950212478127283`", ",", "0.9949147995109651`", ",", 
   "0.9992424154203021`", ",", "0.9552330863600197`", ",", 
   "0.941313418016488`", ",", "0.9709523377331494`", ",", 
   "10.251999311266543`"}], "}"}]], "Print",
 CellChangeTimes->{3.593467758112072*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593467758112072*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.09606527869209185`", ",", "0.8126981628262937`", ",", 
   "0.07696513473875935`", ",", "0.04568541005491819`", ",", 
   "0.47276720265685707`", ",", "0.21026446595260292`", ",", 
   "0.9238512395181747`", ",", "0.9454514410044543`"}], "}"}]], "Print",
 CellChangeTimes->{3.593467758112072*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593467758112072*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "8.82764081193259`", ",", "24.092379322266176`", ",", "36.36659751351858`", 
   ",", "53.661365481172666`", ",", "60.37074340919396`", ",", 
   "75.962799129143`", ",", "83.63543864968531`", ",", "89.63303016144543`", 
   ",", "97.38048182345852`", ",", "0.5052856914071718`", ",", 
   "42.92744821428571`"}], "}"}]], "Print",
 CellChangeTimes->{3.593467758112072*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.09578619349313489`"}],
  SequenceForm[" MPC mean:", 0.09578619349313489],
  Editable->False]], "Print",
 CellChangeTimes->{3.593467758127672*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593467758127672*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0661698712680594`", ",", "0.06821799232131587`", ",", 
   "0.07563597407192513`", ",", "0.08953174019061638`", ",", 
   "0.0986663067363307`", ",", "0.09145308867667767`", ",", 
   "0.10746705762072217`", ",", "0.10176428615584909`", ",", 
   "0.09711307479796971`", ",", "0.09932291915993574`"}], "}"}]], "Print",
 CellChangeTimes->{3.593467758127672*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593467758127672*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.06383018282080921`", ",", "0.06383002375014302`", ",", 
   "0.06393585968454318`", ",", "0.06461354133950292`", ",", 
   "0.06527578034301762`", ",", "0.1401436854797221`", ",", 
   "0.2073903349347883`", ",", "0.10613423640966668`", ",", 
   "0.1258054869398787`", ",", "0.1813102796665812`"}], "}"}]], "Print",
 CellChangeTimes->{3.593467758127672*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.08891895551205542`", ",", "0.18338822261831`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.08891895551205542, 0.18338822261831}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593467758190072*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12704825674504278`", ",", "0.9650738905562637`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12704825674504278`, 0.9650738905562637}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593467758190072*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.925`"}],
  SequenceForm[" R^2: ", 0.925],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934677582056723`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.36081157`10.396434304074836"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.36081157`10.396434304074836],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934677582056723`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "43.12237574`10.563368927628781"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 43.12237574`10.563368927628781],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934677582056723`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model across Euroepan \
economies (minutes):  \"\>", "\[InvisibleSpace]", 
   "943.701271215`11.903500788281962"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model across Euroepan economies (minutes): \
 ", 943.701271215`11.903500788281962],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934677582212725`*^9}],

Cell[BoxData["\<\"============================================================\
===============\"\>"], "Print",
 CellChangeTimes->{3.5934677582212725`*^9}],

Cell[BoxData["\<\"============================================================\
===============\"\>"], "Print",
 CellChangeTimes->{3.5934677582212725`*^9}],

Cell[BoxData["\<\"cstMPC across European economies, using data on liq fin and \
retirement assets \"\>"], "Print",
 CellChangeTimes->{3.5934677582212725`*^9}],

Cell[BoxData["\<\"============================================================\
===============\"\>"], "Print",
 CellChangeTimes->{3.5934677582368727`*^9}],

Cell[BoxData["\<\"Models without aggregate shock\"\>"], "Print",
 CellChangeTimes->{3.5934677582368727`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5934677582836723`*^9}],

Cell[BoxData["\<\"Country: All (baseline)\"\>"], "Print",
 CellChangeTimes->{3.5934677583148727`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593467758689273*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593467761871679*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593467761871679*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615836071847013499791501089931`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615836071847013499791501089931`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934680808142385`*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.593468088832653*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.71964302166666666666666666666666666667`9.164104549884325"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   1.71964302166666666666666666666666666667`9.164104549884325],
  Editable->False]], "Print",
 CellChangeTimes->{3.593468191995634*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593468191995634*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593468191995634*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.59346819520924*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59346819520924*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268032434836413180107313686`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268032434836413180107313686`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593468345905504*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934683537835183`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698583289126802`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698583289126802],
  Editable->False]], "Print",
 CellChangeTimes->{3.593468478334137*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593468531857831*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.92686124524206`", ",", "60.8788892772794`", ",", "75.55214253375213`", 
   ",", "80.16848679766257`", ",", "89.01972056792337`", ",", 
   "92.64415991139559`", ",", "95.27504962508125`", ",", "98.65239899650904`",
    ",", "75.55214253375213`", ",", "0.7164027997681808`", ",", 
   "0.7145882950160927`", ",", "0.3047115776157385`", ",", "Indeterminate", 
   ",", "1.4892839894354077`", ",", "0.6234952281203399`", ",", 
   "0.23902650310197526`", ",", "28.57888702724989`"}], "}"}]], "Print",
 CellChangeTimes->{3.593468531857831*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593468531857831*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593468531857831*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108071476688383259123799779`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108071476688383259123799779`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593468727014174*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934687350793877`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696438465510807`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696438465510807],
  Editable->False]], "Print",
 CellChangeTimes->{3.593468842469977*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934688944492683`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.127253077156332`", ",", "62.34732982455442`", ",", "76.74049651904802`",
    ",", "81.20824214257979`", ",", "89.657380898896`", ",", 
   "93.07789571090322`", ",", "95.55408261205667`", ",", "98.73338642428028`",
    ",", "76.74049651904802`", ",", "0.7280752444524209`", ",", 
   "0.7263338842179606`", ",", "0.3047115776157385`", ",", "Indeterminate", 
   ",", "1.5146869718162088`", ",", "0.6234952281203399`", ",", 
   "0.24752567812265114`", ",", "19.50534960136104`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934688944492683`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934688944492683`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934688944492683`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96959487213047134701068121938440983528`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96959487213047134701068121938440983528`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934690892312098`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934690970468235`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9694198721304713`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9694198721304713],
  Editable->False]], "Print",
 CellChangeTimes->{3.593469241191077*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593469294543171*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "27.268234501209054`", ",", "63.71105228620132`", ",", "77.82285294733272`",
    ",", "82.14734894499634`", ",", "90.22387664472087`", ",", 
   "93.46047487125495`", ",", "95.80002243185037`", ",", "98.8047685254456`", 
   ",", "77.82285294733272`", ",", "0.7386952148055874`", ",", 
   "0.73702054413605`", ",", "0.3047115776157385`", ",", "Indeterminate", ",",
    "1.5389449941557025`", ",", "0.6234952281203399`", ",", 
   "0.25617314335047914`", ",", "14.675384946400023`"}], "}"}]], "Print",
 CellChangeTimes->{3.593469294558771*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593469294558771*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593469294558771*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96938962337161884771314173655938416237`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96938962337161884771314173655938416237`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593469489964714*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934694978115273`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9691946233716188`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9691946233716188],
  Editable->False]], "Print",
 CellChangeTimes->{3.593469601036909*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934696525793996`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.425083367242987`", ",", "65.03882379450773`", ",", "78.85475502687234`",
    ",", "83.03098045014839`", ",", "90.74834149416272`", ",", 
   "93.81304585919915`", ",", "96.02667648971084`", ",", "98.87055687137078`",
    ",", "78.85475502687234`", ",", "0.7488306114160406`", ",", 
   "0.7472198259059677`", ",", "0.3047115776157385`", ",", "Indeterminate", 
   ",", "1.5622122652113775`", ",", "0.6234952281203399`", ",", 
   "0.26470769279188366`", ",", "13.078761831116271`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934696525793996`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934696529382*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934696529382*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96918076512128287974549431215543285806`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96918076512128287974549431215543285806`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593469847907343*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934698557697563`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9689607651212829`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9689607651212829],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934699790099726`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593470032440067*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "29.493851765789604`", ",", "66.25352366082625`", ",", "79.78482315615963`",
    ",", "83.81859252193607`", ",", "91.20687875771804`", ",", 
   "94.11971861276255`", ",", "96.223238431969`", ",", "98.92783062141176`", 
   ",", "79.78482315615963`", ",", "0.7579185353301793`", ",", 
   "0.7563651704051507`", ",", "0.3047115776157385`", ",", "Indeterminate", 
   ",", "1.5841615085136216`", ",", "0.6234952281203399`", ",", 
   "0.27331320219932076`", ",", "14.11462369385113`"}], "}"}]], "Print",
 CellChangeTimes->{3.593470032440067*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5934700482740946`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593470301212939*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.385706853929317`", ",", "63.882515619157424`", ",", 
   "78.28708474984552`", ",", "82.59743580699165`", ",", "90.53059731349602`",
    ",", "93.6729497488226`", ",", "95.94189713962244`", ",", 
   "98.85312668838063`", ",", "78.28708474984552`", ",", 
   "0.7411137713727913`", ",", "0.7395048217192053`", ",", 
   "0.3039231884740426`", ",", "Indeterminate", ",", "1.568608225237978`", 
   ",", "0.6239130076125791`", ",", "0.26397363946939056`", ",", 
   "13.045623562567762`"}], "}"}]], "Print",
 CellChangeTimes->{3.593470301212939*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0017000000000000001`", ",", "0.9691946233716188`", ",", 
     "0.9632446233716188`", ",", "0.9751446233716188`", ",", 
     "19.792299152177787`", ",", "6.616734622464994`", ",", 
     "0.7573963431186475`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0017000000000000001`, 0.9691946233716188, 0.9632446233716188, 
   0.9751446233716188, 19.792299152177787`, 6.616734622464994, 
   0.7573963431186475}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934703012441387`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", "42.38215444`10.555849277929285"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   42.38215444`10.555849277929285],
  Editable->False]], "Print",
 CellChangeTimes->{3.593470301353339*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593470301415739*^9}],

Cell[BoxData["\<\"Country: Austria\"\>"], "Print",
 CellChangeTimes->{3.5934703014313393`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934703016809397`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5934703051285458`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934703051285458`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615811702451622977605438791215`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615811702451622977605438791215`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593470625755909*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5934706332751217`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.38892244`9.071344242822589"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 1.38892244`9.071344242822589],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934707166104684`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593470716953669*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593470716953669*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.593470719824074*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593470719870874*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268119732408424633842956206`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268119732408424633842956206`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934708703643384`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593470877899152*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698733289126811`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698733289126811],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934709837609377`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934710357090287`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.658518270015232`", ",", "60.57559521088757`", ",", "75.39116049297854`",
    ",", "80.0443728039368`", ",", "88.97434507152214`", ",", 
   "92.63657222616732`", ",", "95.28607655580271`", ",", "98.66478251868946`",
    ",", "75.39116049297854`", ",", "0.7144330255799162`", ",", 
   "0.7126183234381483`", ",", "0.30208146989814055`", ",", "Indeterminate", 
   ",", "1.4965327943027957`", ",", "0.6245754309835974`", ",", 
   "0.23837029760429507`", ",", "9.651935686284649`"}], "}"}]], "Print",
 CellChangeTimes->{3.59347103605223*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934710360990295`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934710360990295`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9697938465510806651791437004659918374`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9697938465510806651791437004659918374`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934712303349705`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593471238275385*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696588465510806`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696588465510806],
  Editable->False]], "Print",
 CellChangeTimes->{3.593471385336843*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934714384861364`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.785934494199594`", ",", "62.02062443046417`", ",", "76.57905355588849`",
    ",", "81.08556849146186`", ",", "89.60556302256279`", ",", 
   "93.06193816321678`", ",", "95.5587725944496`", ",", "98.74416759104135`", 
   ",", "76.57905355588849`", ",", "0.7259333083131284`", ",", 
   "0.7241923854855273`", ",", "0.30208146989814055`", ",", "Indeterminate", 
   ",", "1.5219536892996852`", ",", "0.6245754309835974`", ",", 
   "0.2468576341094546`", ",", "7.884166435840837`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934714384861364`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934714385641365`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934714385797367`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96959487213047139010425496846179315195`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96959487213047139010425496846179315195`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59347163418848*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934716417232933`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9694398721304713`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9694398721304713],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934717682083154`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934718215448093`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.91559226945658`", ",", "63.41338842096559`", ",", "77.69961740755596`", 
   ",", "82.05906284466275`", ",", "90.18885833074442`", ",", 
   "93.45211514720731`", ",", "95.80739678602072`", ",", "98.81634337836516`",
    ",", "77.69961740755596`", ",", "0.7367898762529608`", ",", 
   "0.735118810900095`", ",", "0.30208146989814055`", ",", "Indeterminate", 
   ",", "1.546181703655073`", ",", "0.6245754309835974`", ",", 
   "0.25536290627663494`", ",", "9.733348532384593`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934718215448093`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5934718372384367`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934720917684836`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.078834009168737`", ",", "61.99861868075771`", ",", "76.56743031556405`",
    ",", "81.0706513420252`", ",", "89.58413898988144`", ",", 
   "93.03411511540769`", ",", "95.5330485453732`", ",", "98.73321711906429`", 
   ",", "76.56743031556405`", ",", "0.7257151515551298`", ",", 
   "0.7239724153722894`", ",", "0.305969679612983`", ",", "Indeterminate", 
   ",", "1.5182347862149181`", ",", "0.6218987330127146`", ",", 
   "0.24682337321407166`", ",", "8.031218877588234`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934720917684836`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0015`", ",", "0.9696588465510806`", ",", "0.9644088465510806`", ",", 
     "0.9749088465510806`", ",", "19.762443888109487`", ",", 
     "6.61034513026793`", ",", "0.9689495833467423`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0015, 
   0.9696588465510806, 0.9644088465510806, 0.9749088465510806, 
   19.762443888109487`, 6.61034513026793, 0.9689495833467423}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934720921896844`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "29.83921240833333333333333333333333333333`10.403453604173473"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   29.83921240833333333333333333333333333333`10.403453604173473],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934720921896844`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5934720922520847`*^9}],

Cell[BoxData["\<\"Country: Belgium\"\>"], "Print",
 CellChangeTimes->{3.593472092267685*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934720925172853`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593472096011691*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934720960428915`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615836071847013499791501089931`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615836071847013499791501089931`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593472418167857*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5934724260770707`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.37748242`9.067752313129137"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 1.37748242`9.067752313129137],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934725087416162`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5934725087416162`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5934725087416162`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.593472511955222*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593472511955222*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9699883289126803225251317884354012252`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9699883289126803225251317884354012252`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934726620586853`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593472669936699*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698683289126803`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698683289126803],
  Editable->False]], "Print",
 CellChangeTimes->{3.593472754083247*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593472804752136*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.05492327637502`", ",", "60.58266443502156`", ",", "75.44068771625953`", 
   ",", "80.08353838505936`", ",", "88.94964842388616`", ",", 
   "92.58787041629309`", ",", "95.24081990243195`", ",", "98.64545047196421`",
    ",", "75.44068771625953`", ",", "0.713728719118303`", ",", 
   "0.7119293531014991`", ",", "0.30497290616306433`", ",", "Indeterminate", 
   ",", "1.4922697266269138`", ",", "0.6222571984670713`", ",", 
   "0.23806246616109872`", ",", "31.4677201350502`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934728050797367`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593472805110937*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593472805110937*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108039116940519894555603636`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108039116940519894555603636`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593472999050477*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593473006928491*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696538465510803`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696538465510803],
  Editable->False]], "Print",
 CellChangeTimes->{3.593473134427515*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934731877796087`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.066805866142207`", ",", "61.99582904257376`", ",", "76.60659485150826`",
    ",", "81.1069775050903`", ",", "89.5766205583948`", ",", 
   "93.01464321336546`", ",", "95.51567840490249`", ",", "98.72538757190259`",
    ",", "76.60659485150826`", ",", "0.7249988409524675`", ",", 
   "0.7232720684354537`", ",", "0.30497290616306433`", ",", "Indeterminate", 
   ",", "1.517418841754318`", ",", "0.6222571984670713`", ",", 
   "0.24652154799468484`", ",", "20.57105485593146`"}], "}"}]], "Print",
 CellChangeTimes->{3.593473187826409*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.593473187857609*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593473187857609*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96959487213047128068211586187063986273`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96959487213047128068211586187063986273`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593473383731553*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934733912507663`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9694348721304712`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9694348721304712],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934735365650215`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934735913055177`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.08280712763472`", ",", "63.35548916444084`", ",", "77.70882562352985`", 
   ",", "82.06384273282245`", ",", "90.15388144725499`", ",", 
   "93.40502300139758`", ",", "95.76650630259064`", ",", "98.79827700779053`",
    ",", "77.70882562352985`", ",", "0.7356357333974676`", ",", 
   "0.7339776897440646`", ",", "0.30497290616306433`", ",", "Indeterminate", 
   ",", "1.542260858795863`", ",", "0.6222571984670713`", ",", 
   "0.25500678172927205`", ",", "13.771422204333252`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934735916331177`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593473591664318*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934735916799183`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96938962337161965281329926957440961439`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96938962337161965281329926957440961439`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934737872730618`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934737951666756`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9692096233716195`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9692096233716195],
  Editable->False]], "Print",
 CellChangeTimes->{3.593473898470057*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934739511669493`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.050052853973476`", ",", "64.61955906756664`", ",", "78.71849807978083`",
    ",", "82.93127125261022`", ",", "90.66812994506509`", ",", 
   "93.74997949812465`", ",", "95.98770753485304`", ",", "98.86284542477217`",
    ",", "78.71849807978083`", ",", "0.7453388344556552`", ",", 
   "0.7437435947169412`", ",", "0.30497290616306433`", ",", "Indeterminate", 
   ",", "1.56566951389492`", ",", "0.6222571984670713`", ",", 
   "0.263527343888452`", ",", "10.480963359095817`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934739511669493`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934739513073497`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934739513073497`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96918076512128276567064512107199584787`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96918076512128276567064512107199584787`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934741469472933`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934741549345074`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9689757651212827`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9689757651212827],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934743013095646`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934743540376573`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.96985115533189`", ",", "65.79258932350487`", ",", "79.64192142236315`", 
   ",", "83.71764951931335`", ",", "91.12444853793066`", ",", 
   "94.05469260171243`", ",", "96.18224588800368`", ",", "98.91968931753127`",
    ",", "79.64192142236315`", ",", "0.754182588607603`", ",", 
   "0.7526446861627547`", ",", "0.30497290616306433`", ",", "Indeterminate", 
   ",", "1.587930474649608`", ",", "0.6222571984670713`", ",", 
   "0.27214018070751866`", ",", "9.892224773011677`"}], "}"}]], "Print",
 CellChangeTimes->{3.593474354053257*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0019000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0019000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593474354100057*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593474354115657*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96896693706165330460024799303398301663`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96896693706165330460024799303398301663`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593474550317202*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934745578520155`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9687369370616533`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9687369370616533],
  Editable->False]], "Print",
 CellChangeTimes->{3.593474681248232*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593474734101125*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "27.886972591667476`", ",", "66.92132683760097`", ",", "80.51304776597532`",
    ",", "84.45189939576481`", ",", "91.54061234668065`", ",", 
   "94.3320723301626`", ",", "96.35919277098434`", ",", "98.97138976495914`", 
   ",", "80.51304776597532`", ",", "0.7625205125814092`", ",", 
   "0.7610367889570678`", ",", "0.30497290616306433`", ",", "Indeterminate", 
   ",", "1.60914585745969`", ",", "0.6222571984670713`", ",", 
   "0.28070352921207886`", ",", "11.417867474566838`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934747344287252`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593474749701152*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934750051048007`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "27.71260960966538`", ",", "65.57635052119672`", ",", "79.49699340687734`", 
   ",", "83.61021783865756`", ",", "91.09952913725333`", ",", 
   "94.05339039385579`", ",", "96.18551627186812`", ",", "98.9229022611206`", 
   ",", "79.49699340687734`", ",", "0.7536865658943223`", ",", 
   "0.7521287780823547`", ",", "0.3056315545638283`", ",", "Indeterminate", 
   ",", "1.5884044201392677`", ",", "0.6223810163253408`", ",", 
   "0.2725045115169985`", ",", "9.525962243591216`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934750051048007`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0018000000000000002`", ",", "0.9689757651212827`", ",", 
     "0.9626757651212827`", ",", "0.9752757651212827`", ",", 
     "19.77689638463908`", ",", "6.613438623563747`", ",", 
     "0.7049703773517547`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0018000000000000002`, 0.9689757651212827, 0.9626757651212827, 
   0.9752757651212827, 19.77689638463908, 6.613438623563747, 
   0.7049703773517547}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934750051360006`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "48.54754526833333333333333333333333333333`10.614833523584903"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   48.54754526833333333333333333333333333333`10.614833523584903],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934750052140007`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593475005245201*^9}],

Cell[BoxData["\<\"Country: Cyprus\"\>"], "Print",
 CellChangeTimes->{3.5934750052608013`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934750055104017`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5934750089736075`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934750090204077`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615816065628109754470642656088`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615816065628109754470642656088`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593475330973773*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.593475338929787*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.70950300166666666666666666666666666667`9.16153611591398"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   1.70950300166666666666666666666666666667`9.16153611591398],
  Editable->False]], "Print",
 CellChangeTimes->{3.593475441499967*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593475441499967*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593475441499967*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.593475444760373*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593475444760373*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268060670758883978724164258`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268060670758883978724164258`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593475595784238*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934756033814516`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698833289126806`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698833289126806],
  Editable->False]], "Print",
 CellChangeTimes->{3.593475688432801*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934757403496923`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.501708699807065`", ",", "60.00863460767143`", ",", "75.0923795807911`", 
   ",", "79.81839256323296`", ",", "88.8587359412907`", ",", 
   "92.55383595805473`", ",", "95.2336279917943`", ",", "98.65089911046101`", 
   ",", "75.0923795807911`", ",", "0.7105316622606676`", ",", 
   "0.7087363971596126`", ",", "0.30591803727591027`", ",", "Indeterminate", 
   ",", "1.498683418844555`", ",", "0.6261687736201007`", ",", 
   "0.2380148843861628`", ",", "14.451810534717758`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934757403496923`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593475740692893*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593475740692893*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108089327567255978447595254`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108089327567255978447595254`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593475935240834*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934759431656485`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696688465510809`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696688465510809],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934760503066363`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934761043607316`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.552698216724156`", ",", "61.465752554292344`", ",", 
   "76.28614981849698`", ",", "80.86410393320081`", ",", "89.49851726347205`",
    ",", "92.98585707080295`", ",", "95.50975400723395`", ",", 
   "98.73016445544218`", ",", "76.28614981849698`", ",", 
   "0.7220741310468676`", ",", "0.7203525236882843`", ",", 
   "0.30591803727591027`", ",", "Indeterminate", ",", "1.5245822593099574`", 
   ",", "0.6261687736201007`", ",", "0.24647915107280094`", ",", 
   "9.590667130020336`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934761043919315`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.593476104423132*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593476104423132*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9695948721304704295455818694827070409`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9695948721304704295455818694827070409`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934763030270805`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934763108582945`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9694498721304704`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9694498721304704],
  Editable->False]], "Print",
 CellChangeTimes->{3.593476435580513*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593476488012205*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.59983177424771`", ",", "62.8654207324831`", ",", "77.40754607411867`", 
   ",", "81.83546172285025`", ",", "90.08525530156992`", ",", 
   "93.3803479295685`", ",", "95.76186504953743`", ",", "98.8020500842907`", 
   ",", "77.40754607411867`", ",", "0.732932972839948`", ",", 
   "0.7312808858790969`", ",", "0.30591803727591027`", ",", "Indeterminate", 
   ",", "1.5488038777005804`", ",", "0.6261687736201007`", ",", 
   "0.2549321070418348`", ",", "8.583329787147978`"}], "}"}]], "Print",
 CellChangeTimes->{3.593476488449006*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593476489556608*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934764897438087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96938962337161959443616697371952059737`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96938962337161959443616697371952059737`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934766854929523`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593476693339766*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9692196233716196`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9692196233716196],
  Editable->False]], "Print",
 CellChangeTimes->{3.593476838342021*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934768917877145`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.567320416331345`", ",", "64.14937049160609`", ",", "78.4185699885648`", 
   ",", "82.70019181740689`", ",", "90.59836603549194`", ",", 
   "93.72378231820642`", ",", "95.98051784555278`", ",", "98.86462613788397`",
    ",", "78.4185699885648`", ",", "0.7426779188901861`", ",", 
   "0.7410881775657538`", ",", "0.30591803727591027`", ",", "Indeterminate", 
   ",", "1.5719503654444351`", ",", "0.6261687736201007`", ",", 
   "0.26357578125579695`", ",", "10.593958561540965`"}], "}"}]], "Print",
 CellChangeTimes->{3.593476891865715*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5934769108509483`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934771631501913`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.987571582265193`", ",", "62.98517291059781`", ",", "77.5139314094302`", 
   ",", "81.9200179487506`", ",", "90.13453834257862`", ",", 
   "93.40845898496772`", ",", "95.77265369894022`", ",", "98.80312492387645`",
    ",", "77.5139314094302`", ",", "0.7340612661522818`", ",", 
   "0.7323945716977117`", ",", "0.3053929475371579`", ",", "Indeterminate", 
   ",", "1.5473139738249553`", ",", "0.6267051377806406`", ",", 
   "0.25479487011226065`", ",", "8.740464812936244`"}], "}"}]], "Print",
 CellChangeTimes->{3.593477163493392*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0016`", ",", "0.9694498721304704`", ",", "0.9638498721304704`", ",", 
     "0.9750498721304705`", ",", "19.791746396401333`", ",", 
     "6.616616355645619`", ",", "0.838261502872709`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0016, 
   0.9694498721304704, 0.9638498721304704, 0.9750498721304705, 
   19.791746396401333`, 6.616616355645619, 0.838261502872709}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934771643981934`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "35.97236318166666666666666666666666666667`10.484635217672876"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   35.97236318166666666666666666666666666667`10.484635217672876],
  Editable->False]], "Print",
 CellChangeTimes->{3.593477164741394*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5934771670345984`*^9}],

Cell[BoxData["\<\"Country: Germany\"\>"], "Print",
 CellChangeTimes->{3.593477167159398*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934771676117992`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593477171028205*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934771710906053`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96940687079691256755609174433629959822`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96940687079691256755609174433629959822`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593477489237564*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.593477497162378*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "2.081043655`9.246947438915845"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 2.081043655`9.246947438915845],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934776220717974`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5934776221029973`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5934776221029973`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.593477625269803*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593477625269803*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96959631207697575090471365600817336907`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96959631207697575090471365600817336907`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934778012537117`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593477809131726*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9695463120769757`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9695463120769757],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934778926230726`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934779433075614`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.386947525088335`", ",", "54.23582472164854`", ",", "69.86887625705106`",
    ",", "75.05789168722188`", ",", "85.71188162558278`", ",", 
   "90.41127290714009`", ",", "93.91987293266396`", ",", "98.39880840687407`",
    ",", "69.86887625705106`", ",", "0.6604871997969537`", ",", 
   "0.658362304083862`", ",", "0.30529681699876776`", ",", "Indeterminate", 
   ",", "1.5805967749685623`", ",", "0.63714018895312`", ",", 
   "0.2462849089108523`", ",", "38.3224219748507`"}], "}"}]], "Print",
 CellChangeTimes->{3.593477943697562*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934779438223624`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593477943837962*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96940250529054256489660953708690941015`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96940250529054256489660953708690941015`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593478137309502*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593478145093916*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9693425052905426`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9693425052905426],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934782724525394`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934783237454295`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.31829896503288`", ",", "55.63492217208705`", ",", "71.03394558272791`", 
   ",", "76.09141221740823`", ",", "86.37869375477217`", ",", 
   "90.87839937228006`", ",", "94.2263061845337`", ",", "98.48659099821944`", 
   ",", "71.03394558272791`", ",", "0.6719798249134793`", ",", 
   "0.6699275157632818`", ",", "0.30529681699876776`", ",", "Indeterminate", 
   ",", "1.6033665676445497`", ",", "0.63714018895312`", ",", 
   "0.25284262892287324`", ",", "28.41523581638186`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934783237766294`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934783239170303`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59347832393263*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96920416944767497416827557255384438104`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96920416944767497416827557255384438104`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934785196037736`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934785274505873`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9691291694476749`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9691291694476749],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934786504880037`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593478703309696*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.211868910335664`", ",", "56.93836659867178`", ",", "72.10529934501984`",
    ",", "77.0370604131307`", ",", "86.98156735120949`", ",", 
   "91.29894342685134`", ",", "94.50141170934899`", ",", "98.56508921817738`",
    ",", "72.10529934501984`", ",", "0.6825311477401172`", ",", 
   "0.680545608786733`", ",", "0.30529681699876776`", ",", "Indeterminate", 
   ",", "1.6249548613329738`", ",", "0.63714018895312`", ",", 
   "0.25957707442054645`", ",", "22.778818570413662`"}], "}"}]], "Print",
 CellChangeTimes->{3.593478703652897*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593478703808897*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593478703808897*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96900069534619028597708937185356213962`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96900069534619028597708937185356213962`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934788983880386`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593478906250453*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9689106953461902`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9689106953461902],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934790517675085`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934791050572023`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.12185304435134`", ",", "58.20741111488371`", ",", "73.13223680086818`", 
   ",", "77.93682234873391`", ",", "87.54700833855291`", ",", 
   "91.69150162250432`", ",", "94.75751616438161`", ",", "98.6377526147169`", 
   ",", "73.13223680086818`", ",", "0.6926335726683983`", ",", 
   "0.6907121339436374`", ",", "0.30529681699876776`", ",", "Indeterminate", 
   ",", "1.645948107609628`", ",", "0.63714018895312`", ",", 
   "0.2663770258077165`", ",", "20.48238499247056`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934791050572023`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593479105135202*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934791051508017`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96879313944387118193051384247333572815`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96879313944387118193051384247333572815`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593479299074743*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934793068591566`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9686881394438712`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9686881394438712],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934794087117352`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934794605350266`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.007758572420823`", ",", "59.40509661744768`", ",", "74.09393740445105`",
    ",", "78.77457554194447`", ",", "88.06760869257526`", ",", 
   "92.05137256623524`", ",", "94.99220015748546`", ",", "98.70399706978283`",
    ",", "74.09393740445105`", ",", "0.7020750452234836`", ",", 
   "0.70021367397915`", ",", "0.30529681699876776`", ",", "Indeterminate", 
   ",", "1.6664775554632951`", ",", "0.63714018895312`", ",", 
   "0.2731706695011611`", ",", "21.02166302515562`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934794605350266`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593479476883855*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593479731913103*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.84936893516683`", ",", "59.399257492274835`", ",", "73.80796188575307`",
    ",", "78.4692310779272`", ",", "87.8355162376374`", ",", 
   "91.88693667111674`", ",", "94.88406851642533`", ",", "98.66812558704454`",
    ",", "73.80796188575307`", ",", "0.7017361886764504`", ",", 
   "0.699811983906075`", ",", "0.3040391020054596`", ",", "Indeterminate", 
   ",", "1.6386465016791478`", ",", "0.621581122330036`", ",", 
   "0.2667895967268592`", ",", "21.201555633999156`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934797322875032`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0017000000000000001`", ",", "0.9689106953461902`", ",", 
     "0.9629606953461902`", ",", "0.9748606953461902`", ",", 
     "19.784271414579553`", ",", "6.615016902655761`", ",", 
     "0.6788275905080816`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0017000000000000001`, 0.9689106953461902, 0.9629606953461902, 
   0.9748606953461902, 19.784271414579553`, 6.615016902655761, 
   0.6788275905080816}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593479732490304*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "42.74797508166666666666666666666666666667`10.559581795778385"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   42.74797508166666666666666666666666666667`10.559581795778385],
  Editable->False]], "Print",
 CellChangeTimes->{3.593479732552704*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5934797325995045`*^9}],

Cell[BoxData["\<\"Country: Spain\"\>"], "Print",
 CellChangeTimes->{3.593479732615104*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934797328959045`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5934797363435106`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0055`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0055],
  Editable->False]], "Print",
 CellChangeTimes->{3.593479736374711*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.95959261839551490691491153484093956649`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.95959261839551490691491153484093956649`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934801771846848`*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5934801842670975`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "2.696204735`9.359419115272088"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 2.696204735`9.359419115272088],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934803460705814`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5934803460705814`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5934803460861816`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.005399999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.005399999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934803492841873`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934803493153877`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.95986308609041446353896985178927680663`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.95986308609041446353896985178927680663`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934805588705554`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934805663273687`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9588180860904145`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9588180860904145],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934807521392946`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934808044617867`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "42.6008937104059`", ",", "81.2751136238608`", ",", "89.02496046821452`", 
   ",", "91.12316625276594`", ",", "95.1040694815682`", ",", 
   "96.79119537125116`", ",", "98.02988703558513`", ",", "99.54278968180296`",
    ",", "89.02496046821452`", ",", "0.8543791821969642`", ",", 
   "0.8534835821986931`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.171348878814663`", ",", "0.6331344445429303`", ",", 
   "0.4806482383614049`", ",", "33.44428120874348`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934808044773865`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0055`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0055],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934808048829875`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934808048829875`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.95958867524551630494164830066657916225`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.95958867524551630494164830066657916225`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934809671544724`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934809742524853`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9585236752455164`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9585236752455164],
  Editable->False]], "Print",
 CellChangeTimes->{3.593481160813213*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593481212262103*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "42.83411011893659`", ",", "81.55556128484385`", ",", "89.1850357862284`", 
   ",", "91.24777147883829`", ",", "95.17077070803364`", ",", 
   "96.83572233917884`", ",", "98.05829428747337`", ",", "99.55049599671727`",
    ",", "89.1850357862284`", ",", "0.856062436311943`", ",", 
   "0.85517744598074`", ",", "0.3082839234150588`", ",", "Indeterminate", ",",
    "2.181751266877836`", ",", "0.6331344445429303`", ",", 
   "0.48502924083341625`", ",", "35.30505639292466`"}], "}"}]], "Print",
 CellChangeTimes->{3.593481212262103*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.005299999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.005299999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593481212340103*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934812123557034`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96012949802026866747505031459641319935`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96012949802026866747505031459641319935`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934814021924367`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59348140963365*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9591044980202688`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9591044980202688],
  Editable->False]], "Print",
 CellChangeTimes->{3.593481571390334*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593481623166825*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "42.28243545964555`", ",", "80.94298416986034`", ",", "88.83102289771553`", 
   ",", "90.97013537805341`", ",", "95.02107259680469`", ",", 
   "96.73584505823484`", ",", "97.99474683466381`", ",", "99.53347451823332`",
    ",", "88.83102289771553`", ",", "0.8523164437371984`", ",", 
   "0.8514077077242588`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.1604248640965737`", ",", "0.6331344445429303`", ",", 
   "0.4763235162707724`", ",", "31.272810344112482`"}], "}"}]], "Print",
 CellChangeTimes->{3.593481623166825*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.005199999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.005199999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934816235412254`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934816235412254`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96040147223863509594541597427768145951`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96040147223863509594541597427768145951`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934818364503994`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593481843564012*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9593964722386352`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9593964722386352],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934819835274577`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934820363335505`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "42.01139571620834`", ",", "80.6342226635051`", ",", "88.65423323259665`", 
   ",", "90.83183398682533`", ",", "94.94687253431681`", ",", 
   "96.68631153245522`", ",", "97.9631307419665`", ",", "99.52492324808986`", 
   ",", "88.65423323259665`", ",", "0.8504443665659567`", ",", 
   "0.8495237944194508`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.1497207357557113`", ",", "0.6331344445429303`", ",", 
   "0.4718525330554836`", ",", "29.373043923637393`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934820366923513`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.005099999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.005099999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593482036754751*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934820367703514`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96066985825912342663659453920825113502`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96066985825912342663659453920825113502`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934823844169617`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593482391764575*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9596848582591235`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9596848582591235],
  Editable->False]], "Print",
 CellChangeTimes->{3.593482595142132*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593482646497422*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "41.698783289701566`", ",", "80.29866666807897`", ",", "88.45979877882039`",
    ",", "90.67909370527732`", ",", "94.8642663651121`", ",", 
   "96.63114015325452`", ",", "97.92799030186556`", ",", "99.51549170018038`",
    ",", "88.45979877882039`", ",", "0.8483770615611919`", ",", 
   "0.8474433603438049`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.1385653323794602`", ",", "0.6331344445429303`", ",", 
   "0.4673665290772361`", ",", "27.3714992102449`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934826465130224`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004999999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004999999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.593482646637822*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934826466534224`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96093675787664334784078650881644310228`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96093675787664334784078650881644310228`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934831076186323`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934831151222453`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9599767578766435`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9599767578766435],
  Editable->False]], "Print",
 CellChangeTimes->{3.593483275786927*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593483327345018*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "41.41041162398522`", ",", "79.97341131456788`", ",", "88.27472078521417`", 
   ",", "90.53506352468432`", ",", "94.78685677222678`", ",", 
   "96.57939766124667`", ",", "97.89491130847146`", ",", "99.50648937904911`",
    ",", "88.27472078521417`", ",", "0.8464080671259213`", ",", 
   "0.845461918190184`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.127846291602878`", ",", "0.6331344445429303`", ",", 
   "0.46274958056911963`", ",", "25.55103804530652`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934833276882186`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004899999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004899999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934833277350187`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593483327750619*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96120189420047273680701038131219441483`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96120189420047273680701038131219441483`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59348363004795*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593483637535963*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9602618942004727`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9602618942004727],
  Editable->False]], "Print",
 CellChangeTimes->{3.593483819978283*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934838729559765`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "41.051931948198884`", ",", "79.60510801931696`", ",", "88.05957226342306`",
    ",", "90.36550830488689`", ",", "94.69431517414657`", ",", 
   "96.51754433873036`", ",", "97.85547658773787`", ",", "99.49594282910205`",
    ",", "88.05957226342306`", ",", "0.8441075899217322`", ",", 
   "0.8431467857411029`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.1165860690366958`", ",", "0.6331344445429303`", ",", 
   "0.4581491217526492`", ",", "23.541179404374784`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934838729715767`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004799999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004799999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.593483873080776*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593483873080776*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96146689900675856237944436294833949124`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96146689900675856237944436294833949124`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934841316824307`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593484139170444*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9605518990067586`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9605518990067586],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934843227359667`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593484377195662*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "40.73129957662938`", ",", "79.2557663021444`", ",", "87.85939680315776`", 
   ",", "90.20881408461894`", ",", "94.60934771603958`", ",", 
   "96.46064485444306`", ",", "97.81904424021711`", ",", "99.48603073345612`",
    ",", "87.85939680315776`", ",", "0.8419650267722446`", ",", 
   "0.8409906480683388`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.1055101365214024`", ",", "0.6331344445429303`", ",", 
   "0.4534038298590336`", ",", "21.771901106362133`"}], "}"}]], "Print",
 CellChangeTimes->{3.593484377195662*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004699999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004699999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934843775700626`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593484377585663*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96173150993106889853812262740328293495`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96173150993106889853812262740328293495`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934845904012365`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934845978580494`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.960841509931069`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.960841509931069],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934847791615677`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593484831109659*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "40.397180179580005`", ",", "78.89559568997032`", ",", "87.65352402295963`",
    ",", "90.04733846303715`", ",", "94.52171069837132`", ",", 
   "96.40184018516315`", ",", "97.78143653753085`", ",", "99.47576838136692`",
    ",", "87.65352402295963`", ",", "0.8397506360005192`", ",", 
   "0.8387622259723116`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.094595751480683`", ",", "0.6331344445429303`", ",", 
   "0.4485758310377159`", ",", "20.054347654718963`"}], "}"}]], "Print",
 CellChangeTimes->{3.59348483145286*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004599999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004599999999999997],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934848315152597`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934848315152597`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96199466208568968025971196083916197232`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96199466208568968025971196083916197232`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934850456724358`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593485053129249*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9611246620856897`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9611246620856897],
  Editable->False]], "Print",
 CellChangeTimes->{3.593485236273571*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593485287597661*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "39.99738652585043`", ",", "78.49422852604518`", ",", "87.41724105511955`", 
   ",", "89.85957207069876`", ",", "94.41860465557089`", ",", 
   "96.33265987045817`", ",", "97.73735764566169`", ",", "99.46390754927845`",
    ",", "87.41724105511955`", ",", "0.837207659515256`", ",", 
   "0.8362030351838446`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.0830563990975155`", ",", "0.6331344445429303`", ",", 
   "0.44377641885196695`", ",", "18.21399324265649`"}], "}"}]], "Print",
 CellChangeTimes->{3.593485287613261*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004499999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004499999999999997],
  Editable->False]], "Print",
 CellChangeTimes->{3.593485287644461*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593485287660061*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96225722519451014470450739510943688957`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96225722519451014470450739510943688957`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934855927809973`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59348560030021*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9614122251945103`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9614122251945103],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934857836941323`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593485837607827*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "39.62972784370277`", ",", "78.10993659515128`", ",", "87.19370066765309`", 
   ",", "89.68264044035831`", ",", "94.32188065594119`", ",", 
   "96.26771231470188`", ",", "97.6958143498581`", ",", "99.45256707129346`", 
   ",", "87.19370066765309`", ",", "0.8348006033429008`", ",", 
   "0.8337807013549995`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.0715793495096406`", ",", "0.6331344445429303`", ",", 
   "0.43880615962764324`", ",", "16.59808804208721`"}], "}"}]], "Print",
 CellChangeTimes->{3.593485837607827*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004399999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004399999999999997],
  Editable->False]], "Print",
 CellChangeTimes->{3.593485837654627*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593485837670227*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9625135381813403633435417287801193403`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9625135381813403633435417287801193403`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593486007351725*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934860148241386`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9616985381813404`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9616985381813404],
  Editable->False]], "Print",
 CellChangeTimes->{3.593486176300022*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934862307753177`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "39.24000249192862`", ",", "77.71000653355573`", ",", "86.9598666858989`", 
   ",", "89.4969854029686`", ",", "94.22015276405965`", ",", 
   "96.19944333560413`", ",", "97.6521035751484`", ",", "99.44064713377418`", 
   ",", "86.9598666858989`", ",", "0.832278387229961`", ",", 
   "0.8312424579723936`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.0597964844067365`", ",", "0.6331344445429303`", ",", 
   "0.43376167432723967`", ",", "15.039938948506945`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934862308065176`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0042999999999999965`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0042999999999999965`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934862308689175`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934862308689175`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96277841033044658457105423967012590209`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96277841033044658457105423967012590209`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593486491124175*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934864986901884`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9619834103304465`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9619834103304465],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934866411652384`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593486692707729*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "38.830687054909156`", ",", "77.29606133100759`", ",", "86.71433885622251`",
    ",", "89.30112776794135`", ",", "94.1122691144486`", ",", 
   "96.12700545016783`", ",", "97.60574202752933`", ",", "99.42802476866368`",
    ",", "86.71433885622251`", ",", "0.8296364370965742`", ",", 
   "0.8285836970039002`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.0480410166356626`", ",", "0.6331344445429303`", ",", 
   "0.4286668779055367`", ",", "13.55084472536279`"}], "}"}]], "Print",
 CellChangeTimes->{3.593486692754529*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004199999999999996`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004199999999999996],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934866927857294`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593486692801329*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96303844170907296974563811589743422195`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96303844170907296974563811589743422195`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593486907816507*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934869154605203`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.962268441709073`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.962268441709073],
  Editable->False]], "Print",
 CellChangeTimes->{3.59348709771564*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593487152627737*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "38.414223690243034`", ",", "76.87502948220357`", ",", "86.4633959750405`", 
   ",", "89.10064916384881`", ",", "94.00160402761348`", ",", 
   "96.0526338424977`", ",", "97.55812551546217`", ",", "99.41502198001366`", 
   ",", "86.4633959750405`", ",", "0.8269390322885333`", ",", 
   "0.8258691340749283`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.0361476657067885`", ",", "0.6331344445429303`", ",", 
   "0.4234690141324001`", ",", "12.183591241933877`"}], "}"}]], "Print",
 CellChangeTimes->{3.593487152627737*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.004099999999999996`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.004099999999999996],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934871529865375`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934871530177374`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96329542589771449698453254630746131039`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96329542589771449698453254630746131039`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593487368095315*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593487375333728*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9625504258977146`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9625504258977146],
  Editable->False]], "Print",
 CellChangeTimes->{3.593487538541215*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593487590894906*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "37.927891009438234`", ",", "76.40580064721436`", ",", "86.18007028226839`",
    ",", "88.87334018352145`", ",", "93.87551593553258`", ",", 
   "95.96800007240739`", ",", "97.50411072275335`", ",", "99.40039464057985`",
    ",", "86.18007028226839`", ",", "0.8238908270962783`", ",", 
   "0.8228014518493182`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.0239383297470046`", ",", "0.6331344445429303`", ",", 
   "0.41823287428522593`", ",", "10.829232294854597`"}], "}"}]], "Print",
 CellChangeTimes->{3.593487590894906*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.003999999999999996`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.003999999999999996],
  Editable->False]], "Print",
 CellChangeTimes->{3.593487591238107*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593487591238107*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96355345721491976707927806858674171212`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96355345721491976707927806858674171212`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934878979346457`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934879055318594`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9628334572149198`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9628334572149198],
  Editable->False]], "Print",
 CellChangeTimes->{3.593488068177745*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593488119579835*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "37.47659337376758`", ",", "75.95723552226342`", ",", "85.90685298232906`", 
   ",", "88.6539215082596`", ",", "93.75349579345283`", ",", 
   "95.886095432544`", ",", "97.45168180868106`", ",", "99.38605082866178`", 
   ",", "85.90685298232906`", ",", "0.8209692885932868`", ",", 
   "0.8198613119798702`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "2.011543774537036`", ",", "0.6331344445429303`", ",", 
   "0.41286009129923895`", ",", "9.713295583295878`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934881199230356`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.003899999999999996`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.003899999999999996],
  Editable->False]], "Print",
 CellChangeTimes->{3.593488119954236*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934881199698358`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96380472040345824554342506085792398032`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96380472040345824554342506085792398032`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934882902909346`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934882978881483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9631147204034582`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9631147204034582],
  Editable->False]], "Print",
 CellChangeTimes->{3.593488501608906*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934885538533983`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "37.00126764717792`", ",", "75.48954097688893`", ",", "85.61841037886563`", 
   ",", "88.42143833053291`", ",", "93.62375456133152`", ",", 
   "95.79894243118218`", ",", "97.39589771279775`", ",", "99.37080164863036`",
    ",", "85.61841037886563`", ",", "0.8178950491827249`", ",", 
   "0.8167674842753839`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "1.9986341467481212`", ",", "0.6331344445429303`", ",", 
   "0.40743502110360175`", ",", "8.739313687398793`"}], "}"}]], "Print",
 CellChangeTimes->{3.593488553868998*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.003799999999999996`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.003799999999999996],
  Editable->False]], "Print",
 CellChangeTimes->{3.593488553900198*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593488553900198*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96406431708198087407474954577082417561`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96406431708198087407474954577082417561`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934888389438987`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593488846556712*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.963394317081981`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.963394317081981],
  Editable->False]], "Print",
 CellChangeTimes->{3.593489009389798*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593489060745088*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "36.500020853669376`", ",", "75.00096105950111`", ",", "85.31466855654784`",
    ",", "88.17582255296898`", ",", "93.48656251542856`", ",", 
   "95.70671432269584`", ",", "97.33690718364157`", ",", "99.35467158811356`",
    ",", "85.31466855654784`", ",", "0.8146642007421825`", ",", 
   "0.8135160434521256`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "1.9859464856769187`", ",", "0.6331344445429303`", ",", 
   "0.4019337227618156`", ",", "7.939412521438481`"}], "}"}]], "Print",
 CellChangeTimes->{3.593489060745088*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0036999999999999963`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0036999999999999963`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593489060745088*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593489060745088*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96431426569326384762594394523636743049`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96431426569326384762594394523636743049`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593489231518588*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934892391002016`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9636742656932638`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9636742656932638],
  Editable->False]], "Print",
 CellChangeTimes->{3.59348940336849*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934894555193815`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "35.99571507438161`", ",", "74.50559902186197`", ",", "85.00331197813159`", 
   ",", "87.92287994288218`", ",", "93.34510565487882`", ",", 
   "95.61158257168138`", ",", "97.27604291854963`", ",", "99.33796398008471`",
    ",", "85.00331197813159`", ",", "0.8113694458619951`", ",", 
   "0.810200315712362`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "1.973095229379866`", ",", "0.6331344445429303`", ",", 
   "0.39633450573425477`", ",", "7.361527507807018`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934894555193815`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0035999999999999964`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0035999999999999964`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934894558781824`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593489455893782*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96457073923350334330716802466069963395`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96457073923350334330716802466069963395`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934898105292053`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934898181420183`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9639507392335034`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9639507392335034],
  Editable->False]], "Print",
 CellChangeTimes->{3.593490002315942*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934900537804327`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "35.453799913524534`", ",", "73.97909446507174`", ",", "84.66668355147202`",
    ",", "87.64766354414894`", ",", "93.19069555964839`", ",", 
   "95.50778641348845`", ",", "97.20968977664083`", ",", "99.31981092494566`",
    ",", "84.66668355147202`", ",", "0.807823906433725`", ",", 
   "0.8066321785539815`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "1.9599824116745896`", ",", "0.6331344445429303`", ",", 
   "0.39067877854169486`", ",", "7.015328636260872`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934900537804327`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0034999999999999966`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0034999999999999966`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593490054139233*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593490054139233*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96482158435007929308155998314213764063`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96482158435007929308155998314213764063`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593490408306655*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934904159194684`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9642315843500793`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9642315843500793],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934905794389553`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934906304666452`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "34.936570426732686`", ",", "73.4668951351227`", ",", "84.33938765826157`", 
   ",", "87.38025104885476`", ",", "93.04111961536161`", ",", 
   "95.40732088410138`", ",", "97.14532788203516`", ",", "99.3020270672679`", 
   ",", "84.33938765826157`", ",", "0.8043850649175698`", ",", 
   "0.8031715120270254`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "1.9466757157801282`", ",", "0.6331344445429303`", ",", 
   "0.3848297950742967`", ",", "6.948563934269686`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934906304666452`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0033999999999999968`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0033999999999999968`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593490630809846*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934906308410463`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96507122455418475145780794949564132613`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96507122455418475145780794949564132613`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593490894465909*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934909021099224`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9645062245541847`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9645062245541847],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934910454741745`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593491097266265*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "34.36480825305409`", ",", "72.90765820753309`", ",", "83.97180307133483`", 
   ",", "87.07826818214257`", ",", "92.87097930509371`", ",", 
   "95.29308262113543`", ",", "97.07230452997184`", ",", "99.28199761039008`",
    ",", "83.97180307133483`", ",", "0.8005571712265735`", ",", 
   "0.7993192446166975`", ",", "0.3082839234150588`", ",", "Indeterminate", 
   ",", "1.9331278522117585`", ",", "0.6331344445429303`", ",", 
   "0.37897914414903644`", ",", "7.20375970474541`"}], "}"}]], "Print",
 CellChangeTimes->{3.593491097266265*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593491113630694*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593491368893942*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "34.74094936424631`", ",", "72.90574619551963`", ",", "84.05662924258259`", 
   ",", "87.1675360209376`", ",", "92.93549696320396`", ",", 
   "95.33821286416222`", ",", "97.10162085717579`", ",", "99.28948581515033`",
    ",", "84.05662924258259`", ",", "0.8015012363056521`", ",", 
   "0.8002737316005734`", ",", "0.3054315665518656`", ",", "Indeterminate", 
   ",", "1.939892345294556`", ",", "0.6222939812187739`", ",", 
   "0.38465667122079883`", ",", "6.976402839070886`"}], "}"}]], "Print",
 CellChangeTimes->{3.593491368893942*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0034999999999999966`", ",", "0.9642315843500793`", ",", 
     "0.9519815843500793`", ",", "0.9764815843500793`", ",", 
     "19.797421903944418`", ",", "6.617830622390687`", ",", 
     "1.4781167266454236`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0034999999999999966`, 0.9642315843500793, 0.9519815843500793, 
   0.9764815843500793, 19.797421903944418`, 6.617830622390687, 
   1.4781167266454236`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593491368987542*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", "193.938500635`11.216330281912873"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   193.938500635`11.216330281912873],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934913690187426`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593491369252743*^9}],

Cell[BoxData["\<\"Country: Finland\"\>"], "Print",
 CellChangeTimes->{3.593491369268343*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593491369533543*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5934913729655495`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934913729655495`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615811813473925440121092833579`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615811813473925440121092833579`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934916938113127`*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.593491701892127*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.711063005`9.16193224970299"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 1.711063005`9.16193224970299],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934918045559072`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593491804602708*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593491804602708*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.593491807769513*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593491807769513*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268121353663899061231889937`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268121353663899061231889937`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934919583409777`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934919662969913`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9699033289126812`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9699033289126812],
  Editable->False]], "Print",
 CellChangeTimes->{3.593492093640015*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593492144636505*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.853406069746597`", ",", "59.76153769556297`", ",", "74.9936249672627`", 
   ",", "79.74919422041764`", ",", "88.83574061253555`", ",", 
   "92.54124183405304`", ",", "95.22327273127547`", ",", "98.64738019981161`",
    ",", "74.9936249672627`", ",", "0.7090813810668315`", ",", 
   "0.707266350896423`", ",", "0.3047595663135402`", ",", "Indeterminate", 
   ",", "1.5010505637987337`", ",", "0.6256785839906445`", ",", 
   "0.237292281746048`", ",", "49.05202077439755`"}], "}"}]], "Print",
 CellChangeTimes->{3.593492144652105*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593492144683305*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593492144683305*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108068354817472814250512514`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108068354817472814250512514`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593492339668047*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934923475616612`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696888465510807`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696888465510807],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934924318018093`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934924827358985`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.96963418944552`", ",", "61.215135381436056`", ",", "76.18988382667051`",
    ",", "80.79408621095189`", ",", "89.47158901084151`", ",", 
   "92.97087514261707`", ",", "95.49793439782415`", ",", "98.7272459037587`", 
   ",", "76.18988382667051`", ",", "0.7206527327651662`", ",", 
   "0.7189110911565819`", ",", "0.3047595663135402`", ",", "Indeterminate", 
   ",", "1.5263863297427853`", ",", "0.6256785839906445`", ",", 
   "0.24576416314902816`", ",", "32.3114746559286`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934924830634995`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.593492483079099*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593492483079099*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96959487213047182660718075777522287898`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96959487213047182660718075777522287898`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934926790778437`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593492687018257*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9694698721304718`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9694698721304718],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934927915384407`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934928433773317`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.096336884808082`", ",", "62.615974199070365`", ",", 
   "77.32021710981778`", ",", "81.77320065079013`", ",", "90.05844763786503`",
    ",", "93.36404183182384`", ",", "95.74835696768218`", ",", 
   "98.79947769407961`", ",", "77.32021710981778`", ",", 
   "0.7315865009969822`", ",", "0.7299143405168467`", ",", 
   "0.3047595663135402`", ",", "Indeterminate", ",", "1.5508633442360997`", 
   ",", "0.6256785839906445`", ",", "0.2542634791838222`", ",", 
   "20.137853854823224`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934928433773317`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934928437205324`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934928437205324`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96938962337161918402675285037853981543`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96938962337161918402675285037853981543`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934930392044764`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934930470512896`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9692446233716191`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9692446233716191],
  Editable->False]], "Print",
 CellChangeTimes->{3.59349317235071*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934932274344063`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.179201377576582`", ",", "63.91690274761894`", ",", "78.35022787597134`",
    ",", "82.65744561387017`", ",", "90.58034698470023`", ",", 
   "93.7118794859297`", ",", "95.96927790446404`", ",", "98.86249665354386`", 
   ",", "78.35022787597134`", ",", "0.741551011643771`", ",", 
   "0.7399421818131585`", ",", "0.3047595663135402`", ",", "Indeterminate", 
   ",", "1.5735346103516932`", ",", "0.6256785839906445`", ",", 
   "0.26279219469336446`", ",", "12.077707235067438`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934932274344063`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593493227481207*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593493227481207*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96918076512128289813839242622059068622`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96918076512128289813839242622059068622`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934934231835504`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934934310615644`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9690107651212829`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9690107651212829],
  Editable->False]], "Print",
 CellChangeTimes->{3.593493576079419*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934936277311096`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.21492928453708`", ",", "65.1331103168306`", ",", "79.29583963546014`", 
   ",", "83.46145087271957`", ",", "91.04630891977284`", ",", 
   "94.02140040804427`", ",", "96.16608469391714`", ",", "98.9185736649979`", 
   ",", "79.29583963546014`", ",", "0.7506851372875948`", ",", 
   "0.7491343328027747`", ",", "0.3047595663135402`", ",", "Indeterminate", 
   ",", "1.5950741214826543`", ",", "0.6256785839906445`", ",", 
   "0.27138430720863216`", ",", "7.188363629560458`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934936280743103`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0019000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0019000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934936280743103`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934936280899105`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96896693706165257997448397089262102259`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96896693706165257997448397089262102259`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593493822622252*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934938304378653`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9687719370616525`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9687719370616525],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934939333824463`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934939857673388`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "27.249636310869306`", ",", "66.31145928312135`", ",", "80.18706461788574`",
    ",", "84.21152559063987`", ",", "91.4715490585767`", ",", 
   "94.3036748876846`", ",", "96.34583910374637`", ",", "98.96996976974897`", 
   ",", "80.18706461788574`", ",", "0.7593161086097548`", ",", 
   "0.757820165464759`", ",", "0.3047595663135402`", ",", "Indeterminate", 
   ",", "1.615813688610229`", ",", "0.6256785839906445`", ",", 
   "0.27993541776845376`", ",", "4.74813530355746`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934939857829385`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.002],
  Editable->False]], "Print",
 CellChangeTimes->{3.593493986126139*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593493986126139*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96875025993973732528834750628120304857`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96875025993973732528834750628120304857`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59349418009688*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934941879592934`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9685302599397373`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9685302599397373],
  Editable->False]], "Print",
 CellChangeTimes->{3.593494311605111*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934943646296043`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.234166777228438`", ",", "67.42197581849095`", ",", "81.00947297991773`",
    ",", "84.89852125971412`", ",", "91.85350475925682`", ",", 
   "94.55729335812859`", ",", "96.50751961134615`", ",", "99.01650121617378`",
    ",", "81.00947297991773`", ",", "0.7672831560856391`", ",", 
   "0.7658378286904548`", ",", "0.3047595663135402`", ",", "Indeterminate", 
   ",", "1.6357440687914453`", ",", "0.6256785839906445`", ",", 
   "0.2883864882079692`", ",", "4.3210611582109735`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934943646296043`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0021],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934943646296043`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593494364645204*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96853068954110884653145515140900600436`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96853068954110884653145515140900600436`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593494558023144*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934945659167576`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9682856895411089`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9682856895411089],
  Editable->False]], "Print",
 CellChangeTimes->{3.593494710731812*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593494761946702*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "29.196048606686738`", ",", "68.48674464715151`", ",", "81.78364509951707`",
    ",", "85.53933706782556`", ",", "92.20214589835989`", ",", 
   "94.78795883846452`", ",", "96.65493190278544`", ",", "99.05922129985841`",
    ",", "81.78364509951707`", ",", "0.7747800640184737`", ",", 
   "0.7733823329242483`", ",", "0.3047595663135402`", ",", "Indeterminate", 
   ",", "1.6544950979269955`", ",", "0.6256785839906445`", ",", 
   "0.2966998162744663`", ",", "5.499611277135838`"}], "}"}]], "Print",
 CellChangeTimes->{3.593494761946702*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5934947778431296`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934950333091784`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.96061506330537`", ",", "67.82974668431477`", ",", "81.21096425652195`", 
   ",", "85.05693141395076`", ",", "91.93515990107656`", ",", 
   "94.60983027339613`", ",", "96.54124876512539`", ",", "99.0253737139721`", 
   ",", "81.21096425652195`", ",", "0.7699036988389636`", ",", 
   "0.7684705597964956`", ",", "0.3027078898927874`", ",", "Indeterminate", 
   ",", "1.633882689070559`", ",", "0.6191355501096883`", ",", 
   "0.28828069304103643`", ",", "4.515708371352752`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934950333091784`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.002`", ",", "0.9685302599397373`", ",", "0.9615302599397373`", ",", 
     "0.9755302599397373`", ",", "19.768684533652657`", ",", 
     "6.6116810128653`", ",", "1.42539346273369`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.002, 
   0.9685302599397373, 0.9615302599397373, 0.9755302599397373, 
   19.768684533652657`, 6.6116810128653, 1.42539346273369}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934950336835794`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "61.06760725833333333333333333333333333333`10.714477152084754"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   61.06760725833333333333333333333333333333`10.714477152084754],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934950337615795`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593495033792779*^9}],

Cell[BoxData["\<\"Country: France\"\>"], "Print",
 CellChangeTimes->{3.593495033792779*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5934950340579796`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593495037521186*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593495037536786*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96958217341018557215043382985719298893`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96958217341018557215043382985719298893`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593495311629267*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.593495319647681*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "2.04594359333333333333333333333333333333`9.239559904257575"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   2.04594359333333333333333333333333333333`9.239559904257575],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934954427474976`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5934954427474976`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5934954427474976`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934954455555024`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934954455555024`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96977724264424457195082714125328474229`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96977724264424457195082714125328474229`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934955969537687`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934956045665817`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9697122426442446`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9697122426442446],
  Editable->False]], "Print",
 CellChangeTimes->{3.593495710085167*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934957608476562`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.752699537305848`", ",", "56.94997932477144`", ",", "72.14092835874398`",
    ",", "77.10235813008337`", ",", "87.09118763202322`", ",", 
   "91.36798909904876`", ",", "94.52007213000176`", ",", "98.52355207683632`",
    ",", "72.14092835874398`", ",", "0.6834194925522804`", ",", 
   "0.6814145217933137`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.4932966567712045`", ",", "0.6284651263349117`", ",", 
   "0.24134999844817773`", ",", "101.80809796314904`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934957608632565`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593495761222057*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593495761237657*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96958222535446886429800792971863533953`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96958222535446886429800792971863533953`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593495955426798*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934959633204117`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.969502225354469`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.969502225354469],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934960698997993`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934961213330894`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.749842657997075`", ",", "58.35233713493023`", ",", "73.3074540182236`", 
   ",", "78.1259151887197`", ",", "87.73449241856672`", ",", 
   "91.81339762305643`", ",", "94.8100871625286`", ",", "98.6066083586547`", 
   ",", "73.3074540182236`", ",", "0.6948085992376604`", ",", 
   "0.6928764441225332`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.5166994630164887`", ",", "0.6284651263349117`", ",", 
   "0.2486771298609013`", ",", "76.86371838971411`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934961213330894`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.59349612170749*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59349612172309*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96938346382064140842989137880172432294`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96938346382064140842989137880172432294`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593496316614232*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934963244922466`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9692834638206415`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9692834638206415],
  Editable->False]], "Print",
 CellChangeTimes->{3.593496469681701*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593496524734198*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.734179494685936`", ",", "59.697228541196644`", ",", 
   "74.40463417613121`", ",", "79.08191796315054`", ",", "88.32382020507856`",
    ",", "92.21846318571136`", ",", "95.07290962994375`", ",", 
   "98.68141139179323`", ",", "74.40463417613121`", ",", 
   "0.7054940908203655`", ",", "0.7036301990639621`", ",", 
   "0.3081334246775697`", ",", "Indeterminate", ",", "1.539274742583603`", 
   ",", "0.6284651263349117`", ",", "0.2561584854912782`", ",", 
   "57.021136049963715`"}], "}"}]], "Print",
 CellChangeTimes->{3.593496524734198*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934965250929985`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934965250929985`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96917888152750415095576698643367423412`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96917888152750415095576698643367423412`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59349671953174*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593496727347354*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9690638815275041`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9690638815275041],
  Editable->False]], "Print",
 CellChangeTimes->{3.593496832319938*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934968862024326`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.698624161813175`", ",", "60.9812909049062`", ",", "75.4405977631211`", 
   ",", "79.98085815043017`", ",", "88.8705192440202`", ",", 
   "92.5929848000982`", ",", "95.31509902676932`", ",", "98.75008353800322`", 
   ",", "75.4405977631211`", ",", "0.715577382639407`", ",", 
   "0.7137779485720113`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.5611296404957873`", ",", "0.6284651263349117`", ",", 
   "0.2636024985109745`", ",", "41.38749210288278`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934968865456333`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934968865768337`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934968865768337`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96897056330250701780002734272177529369`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96897056330250701780002734272177529369`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593497082232377*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593497090094791*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.968835563302507`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.968835563302507],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934971928053713`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593497245034263*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.64534649634463`", ",", "62.205625888056474`", ",", "76.4107052239596`", 
   ",", "80.81658005268946`", ",", "89.36888591020232`", ",", 
   "92.93250086375822`", ",", "95.53374035238052`", ",", "98.81215439959313`",
    ",", "76.4107052239596`", ",", "0.725007000604716`", ",", 
   "0.7232678508519247`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.5818706500065574`", ",", "0.6284651263349117`", ",", 
   "0.2711201094059388`", ",", "29.494831180351575`"}], "}"}]], "Print",
 CellChangeTimes->{3.593497245034263*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0019000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0019000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593497245049863*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593497245049863*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96875725766660955971667072839065950338`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96875725766660955971667072839065950338`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934974398006053`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593497447647419*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9686022576666096`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9686022576666096],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934975726816387`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593497625862132*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.547156956214874`", ",", "63.34648057342943`", ",", "77.30360667643193`",
    ",", "81.582366048035`", ",", "89.818726437303`", ",", 
   "93.23755548960456`", ",", "95.72980365314667`", ",", "98.86761652314004`",
    ",", "77.30360667643193`", ",", "0.7336785757198676`", ",", 
   "0.7319948600022874`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.6017172897929075`", ",", "0.6284651263349117`", ",", 
   "0.2786270445569623`", ",", "20.826059423001986`"}], "}"}]], "Print",
 CellChangeTimes->{3.593497625862132*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.002],
  Editable->False]], "Print",
 CellChangeTimes->{3.593497625924532*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934976259557323`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96854112961012629916791524060702077302`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96854112961012629916791524060702077302`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593497820971675*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934978289120884`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9683661296101262`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9683661296101262],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934979736647425`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934980253008337`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.435987877469447`", ",", "64.44572993766204`", ",", "78.1529922337464`", 
   ",", "82.30624085817068`", ",", "90.23827512720037`", ",", 
   "93.52026018311824`", ",", "95.91113355803773`", ",", "98.91871358358296`",
    ",", "78.1529922337464`", ",", "0.7419164955939561`", ",", 
   "0.7402854447385526`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.6212296785836555`", ",", "0.6284651263349117`", ",", 
   "0.2860839940926444`", ",", "14.583697407363267`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934980253008337`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0021],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934980253008337`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934980253008337`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9683216905822601693535270959894455912`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9683216905822601693535270959894455912`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593498219645975*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934982274459887`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.96812169058226`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.96812169058226],
  Editable->False]], "Print",
 CellChangeTimes->{3.593498351169806*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934984034142976`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "27.27458485333206`", ",", "65.466870334813`", ",", "78.93325961024014`", 
   ",", "82.96662754164794`", ",", "90.61559511970472`", ",", 
   "93.7726270282149`", ",", "96.07252565520139`", ",", "98.96422815617628`", 
   ",", "78.93325961024014`", ",", "0.7494595995743463`", ",", 
   "0.7478767104796058`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.6395668659362075`", ",", "0.6284651263349117`", ",", 
   "0.2935959701231713`", ",", "10.555245649589432`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934984034142976`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0021999999999999997`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934984034610977`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593498403476698*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96809772781724881061893054929581645058`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96809772781724881061893054929581645058`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934985982118397`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5934986057310534`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9678727278172488`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9678727278172488],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934987502809067`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593498802104198*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.11149122592799`", ",", "66.45499950081476`", ",", "79.6773953786851`", 
   ",", "83.59333179587415`", ",", "90.96881338326321`", ",", 
   "94.00713452922822`", ",", "96.22208655805535`", ",", "99.00627756265777`",
    ",", "79.6773953786851`", ",", "0.7566591721863939`", ",", 
   "0.7551222862158069`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.6571081201841835`", ",", "0.6284651263349117`", ",", 
   "0.301044267887149`", ",", "8.210476983634914`"}], "}"}]], "Print",
 CellChangeTimes->{3.593498802104198*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0022999999999999995`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0022999999999999995`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593498802104198*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934988021197977`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96787210685893980342355169189710106445`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96787210685893980342355169189710106445`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593498996277739*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593499004062153*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9676271068589398`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9676271068589398],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934991481284056`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593499200638098*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.943142329162203`", ",", "67.41176375279932`", ",", "80.3934847037184`", 
   ",", "84.19467328383763`", ",", "91.30667514360974`", ",", 
   "94.23118267965056`", ",", "96.36454060883467`", ",", "99.0460985720381`", 
   ",", "80.3934847037184`", ",", "0.7636001892373948`", ",", 
   "0.7621077560445451`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.6744067084510015`", ",", "0.6284651263349117`", ",", 
   "0.3082529098497714`", ",", "7.293767301666044`"}], "}"}]], "Print",
 CellChangeTimes->{3.593499200638098*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0023999999999999994`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0023999999999999994`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934992009656982`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5934992009812984`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96764317269267681448766637856419506156`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96764317269267681448766637856419506156`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934993709747972`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593499378728011*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9673731726926769`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9673731726926769],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934994822029924`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934995336830826`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "29.720252824809833`", ",", "68.29057226203132`", ",", "81.04377345008409`",
    ",", "84.7375690391145`", ",", "91.60809705387327`", ",", 
   "94.43044295564516`", ",", "96.4906344051935`", ",", "99.08124275570313`", 
   ",", "81.04377345008409`", ",", "0.7699019884207748`", ",", 
   "0.7684498804067372`", ",", "0.3081334246775697`", ",", "Indeterminate", 
   ",", "1.690864096476566`", ",", "0.6284651263349117`", ",", 
   "0.31550633763053537`", ",", "7.612303111754322`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934995336830826`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5934995497355113`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5934998073387637`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "30.648163609304195`", ",", "68.02794050561444`", ",", "80.80010021418404`",
    ",", "84.54146195677481`", ",", "91.50710785587705`", ",", 
   "94.35904274318928`", ",", "96.44164747902073`", ",", "99.06425424520953`",
    ",", "80.80010021418404`", ",", "0.7684358600676321`", ",", 
   "0.7669693044425466`", ",", "0.3060464220083047`", ",", "Indeterminate", 
   ",", "1.6742852407184117`", ",", "0.6253982702059139`", ",", 
   "0.30798912240193677`", ",", "7.343313312314589`"}], "}"}]], "Print",
 CellChangeTimes->{3.5934998073387637`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0022999999999999995`", ",", "0.9676271068589398`", ",", 
     "0.9595771068589398`", ",", "0.9756771068589398`", ",", 
     "19.790236660147222`", ",", "6.616293328620321`", ",", 
     "0.8275456235973268`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0022999999999999995`, 0.9676271068589398, 0.9595771068589398, 
   0.9756771068589398, 19.790236660147222`, 6.616293328620321, 
   0.8275456235973268}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934998076663637`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "79.55909973833333333333333333333333333333`10.82935610837836"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   79.55909973833333333333333333333333333333`10.82935610837836],
  Editable->False]], "Print",
 CellChangeTimes->{3.5934998078067646`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5934998078691645`*^9}],

Cell[BoxData["\<\"Country: Greece\"\>"], "Print",
 CellChangeTimes->{3.5934998078691645`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593499808134365*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593499811613171*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593499811613171*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615816010116958523212815634906`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615816010116958523212815634906`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593500132677335*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.593500140726949*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.425582505`9.082658605202635"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 1.425582505`9.082658605202635],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935002262618995`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5935002262618995`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593500226277499*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935002295067053`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935002295067053`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268061928399425618262262154`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268061928399425618262262154`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59350038039017*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593500388377384*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698733289126805`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698733289126805],
  Editable->False]], "Print",
 CellChangeTimes->{3.593500493131568*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935005452980595`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.879933699250948`", ",", "60.053257188775724`", ",", 
   "75.04437731088835`", ",", "79.78731453919757`", ",", "88.86066550625114`",
    ",", "92.55119815257196`", ",", "95.22249333862733`", ",", 
   "98.63746456044576`", ",", "75.04437731088835`", ",", 
   "0.7108735457073316`", ",", "0.7090633095921205`", ",", 
   "0.30316134587531424`", ",", "Indeterminate", ",", "1.4918894524231503`", 
   ",", "0.6154132449896567`", ",", "0.23802504321261786`", ",", 
   "165.38830790405558`"}], "}"}]], "Print",
 CellChangeTimes->{3.59350054534486*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.59350054537606*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59350054537606*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108073811176928230459381977`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108073811176928230459381977`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593500740204802*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593500748176416*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696638465510806`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696638465510806],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935008750670385`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935009284191327`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.857699961344608`", ",", "61.498140328396644`", ",", 
   "76.22636488205048`", ",", "80.82245900147645`", ",", "89.49761308277813`",
    ",", "92.98374010869048`", ",", "95.50120806689893`", ",", 
   "98.71856641220343`", ",", "76.22636488205048`", ",", 
   "0.7223051717776109`", ",", "0.7205698209506003`", ",", 
   "0.30316134587531424`", ",", "Indeterminate", ",", "1.5175334590557807`", 
   ",", "0.6154132449896567`", ",", "0.24632898573119144`", ",", 
   "132.8934235472011`"}], "}"}]], "Print",
 CellChangeTimes->{3.593500928434733*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.593500928434733*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593500928434733*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96959487213047161228184223427889237337`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96959487213047161228184223427889237337`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593501124386677*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935011323270907`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9694448721304716`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9694448721304716],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935012362232733`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935012893257666`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.809922676784762`", ",", "62.87471317176754`", ",", "77.32767493180542`",
    ",", "81.77869467096129`", ",", "90.07492869345758`", ",", 
   "93.37178419873906`", ",", "95.74967606946034`", ",", "98.79089955117513`",
    ",", "77.32767493180542`", ",", "0.732924994125459`", ",", 
   "0.7312593960791876`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.5424708927343924`", ",", "0.6154132449896567`", ",", 
   "0.25479834867920925`", ",", "106.31122716849475`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935012893257666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935012893725667`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935012893725667`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96938962337161913338329964958711485348`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96938962337161913338329964958711485348`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593501485854912*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935014937017255`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9692196233716192`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9692196233716192],
  Editable->False]], "Print",
 CellChangeTimes->{3.593501617581543*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935016706372366`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.710820299062597`", ",", "64.15711068621255`", ",", "78.33609546388873`",
    ",", "82.64606390215965`", ",", "90.58860002086533`", ",", 
   "93.71606742252567`", ",", "95.96858412014613`", ",", "98.85481551735688`",
    ",", "78.33609546388873`", ",", "0.7426174616451051`", ",", 
   "0.7410156459262022`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.565966791668881`", ",", "0.6154132449896567`", ",", 
   "0.2632964630510035`", ",", "85.04193673229888`"}], "}"}]], "Print",
 CellChangeTimes->{3.593501670668436*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593501670668436*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935016706840363`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96918076512128251523779891200345748301`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96918076512128251523779891200345748301`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935018660275793`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593501873905593*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9689857651212823`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9689857651212823],
  Editable->False]], "Print",
 CellChangeTimes->{3.593502018081046*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935020703879385`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.562975524511184`", ",", "65.35108034179046`", ",", "79.26175932037654`",
    ",", "83.43327948884215`", ",", "91.04422964845705`", ",", 
   "94.02054494532449`", ",", "96.161791732676`", ",", "98.91088451088984`", 
   ",", "79.26175932037654`", ",", "0.7514640430412064`", ",", 
   "0.7499205366167196`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.5886148613761961`", ",", "0.6154132449896567`", ",", 
   "0.2718838662184444`", ",", "68.09324197831874`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935020704035387`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0019000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0019000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935020704659386`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593502070481539*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9689669370616531089558313648403674565`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9689669370616531089558313648403674565`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935022650918803`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935022728918943`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9687519370616532`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9687519370616532],
  Editable->False]], "Print",
 CellChangeTimes->{3.593502375727275*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935024276129656`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "27.43606115799229`", ",", "66.52444360793909`", ",", "80.16124208695126`", 
   ",", "84.18566013531681`", ",", "91.47184286799843`", ",", 
   "94.30572863927067`", ",", "96.34385274255213`", ",", "98.96352225120953`",
    ",", "80.16124208695126`", ",", "0.7600151955992359`", ",", 
   "0.7585282128904505`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.6101076991909318`", ",", "0.6154132449896567`", ",", 
   "0.2802727119356314`", ",", "53.930582449054334`"}], "}"}]], "Print",
 CellChangeTimes->{3.593502427628566*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.002],
  Editable->False]], "Print",
 CellChangeTimes->{3.593502427644166*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593502427644166*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96875025993973825092071776364965879277`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96875025993973825092071776364965879277`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593502621661707*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935026294461203`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9685102599397384`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9685102599397384],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935027521091356`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593502807567233*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.24107974143459`", ",", "67.59923443263436`", ",", "80.98085615391848`", 
   ",", "84.86293854679965`", ",", "91.84741629192177`", ",", 
   "94.55443488692352`", ",", "96.50268158871853`", ",", "99.0097161506005`", 
   ",", "80.98085615391848`", ",", "0.7677224922847828`", ",", 
   "0.7662864070291635`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.6307788565034027`", ",", "0.6154132449896567`", ",", 
   "0.28870218838671424`", ",", "42.99060650250112`"}], "}"}]], "Print",
 CellChangeTimes->{3.593502807567233*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0021],
  Editable->False]], "Print",
 CellChangeTimes->{3.593502807910434*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593502807910434*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96853068954110845678952820861568202171`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96853068954110845678952820861568202171`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593503001740774*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593503009556388*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9682656895411084`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9682656895411084],
  Editable->False]], "Print",
 CellChangeTimes->{3.593503131829403*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935031852594967`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "29.028024287725852`", ",", "68.62206719077012`", ",", "81.75547912265976`",
    ",", "85.4993277462913`", ",", "92.19085005234726`", ",", 
   "94.78082054386003`", ",", "96.64737913009009`", ",", "99.05201715625357`",
    ",", "81.75547912265976`", ",", "0.7749666619948028`", ",", 
   "0.7735784441176051`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.6504103214650172`", ",", "0.6154132449896567`", ",", 
   "0.2970178487712575`", ",", "34.327075425978634`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935031852594967`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0021999999999999997`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593503185587097*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593503185587097*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96830633444755011946387434657404111906`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96830633444755011946387434657404111906`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935033796514378`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593503387420252*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9680163344475501`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9680163344475501],
  Editable->False]], "Print",
 CellChangeTimes->{3.59350351161207*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935035643089623`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "29.785015961492505`", ",", "69.57793413066177`", ",", "82.47722012207093`",
    ",", "86.08881117339354`", ",", "92.50083138451764`", ",", 
   "94.9842675383583`", ",", "96.77710491971753`", ",", "99.09001257957722`", 
   ",", "82.47722012207093`", ",", "0.7816773602701235`", ",", 
   "0.7803335390431482`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.6689348827163315`", ",", "0.6154132449896567`", ",", 
   "0.3052508573821333`", ",", "27.697412090542265`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935035643089623`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0022999999999999995`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0022999999999999995`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593503564636563*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935035646521626`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96808046867657342015265767206846016065`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96808046867657342015265767206846016065`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593503757983302*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593503765783316*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9677654686765734`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9677654686765734],
  Editable->False]], "Print",
 CellChangeTimes->{3.593503909755969*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593503962702462*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "30.555220815012373`", ",", "70.51299679603376`", ",", "83.17830157004082`",
    ",", "86.65935190842902`", ",", "92.79597730609586`", ",", 
   "95.17744498201945`", ",", "96.90020340191175`", ",", "99.12596103079206`",
    ",", "83.17830157004082`", ",", "0.7882013392268642`", ",", 
   "0.7869007669448841`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.6870096335388343`", ",", "0.6154132449896567`", ",", 
   "0.3133211112417177`", ",", "22.534372422683745`"}], "}"}]], "Print",
 CellChangeTimes->{3.593503962718062*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0023999999999999994`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0023999999999999994`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593503962764862*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593503962764862*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96785214514002939613902114715022269287`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96785214514002939613902114715022269287`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593504156501602*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593504164317216*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9675071451400292`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9675071451400292],
  Editable->False]], "Print",
 CellChangeTimes->{3.593504328835105*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935043828112*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "31.253539421459298`", ",", "71.35776150379328`", ",", "83.80999626374356`",
    ",", "87.17146374409755`", ",", "93.05492936710088`", ",", 
   "95.34474060786614`", ",", "97.00662643678795`", ",", "99.15702529326519`",
    ",", "83.80999626374356`", ",", "0.7940337014448835`", ",", 
   "0.7927717371878562`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.7038175536338858`", ",", "0.6154132449896567`", ",", 
   "0.32136851542499867`", ",", "18.973794742442458`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935043828268003`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002499999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.002499999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935043828268003`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935043828268003`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96762016565507779311860348603614187239`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96762016565507779311860348603614187239`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593504552508298*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935045602303114`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9672501656550778`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9672501656550778],
  Editable->False]], "Print",
 CellChangeTimes->{3.593504704592965*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935047583506594`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "31.95485503344702`", ",", "72.17309642513405`", ",", "84.41439259698622`", 
   ",", "87.66317444500258`", ",", "93.30446607150705`", ",", 
   "95.50567354502331`", ",", "97.10868304872899`", ",", "99.1868368726809`", 
   ",", "84.41439259698622`", ",", "0.7996718152544022`", ",", 
   "0.7984472273850853`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.7204122530986448`", ",", "0.6154132449896567`", ",", 
   "0.3291852966190123`", ",", "16.45043187262066`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935047583506594`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002599999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.002599999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935047583974595`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935047583974595`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96738659397062743857233645831687082972`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96738659397062743857233645831687082972`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935049280633574`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935049357853713`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9669915939706275`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9669915939706275],
  Editable->False]], "Print",
 CellChangeTimes->{3.593505059212788*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935051112700796`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "32.64669656305508`", ",", "72.94788380751574`", ",", "84.9819690489558`", 
   ",", "88.12500682270293`", ",", "93.53824379254246`", ",", 
   "95.65681684089154`", ",", "97.20405527031689`", ",", "99.2145784072353`", 
   ",", "84.9819690489558`", ",", "0.8050176448735831`", ",", 
   "0.8038285355302697`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.7368538666515334`", ",", "0.6154132449896567`", ",", 
   "0.336871970064614`", ",", "14.885774790941625`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935051112700796`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002699999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.002699999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.59350511162888*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59350511162888*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96715030431477488788614294277585861464`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96715030431477488788614294277585861464`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935052803431764`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59350528808079*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9667253043147749`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9667253043147749],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935054108374057`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935054656403017`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "33.28293734868707`", ",", "73.65332459142277`", ",", "85.49468406355844`", 
   ",", "88.54180017136063`", ",", "93.74748175799229`", ",", 
   "95.79165198364261`", ",", "97.28913054453126`", ",", "99.23908834617139`",
    ",", "85.49468406355844`", ",", "0.8098604461439374`", ",", 
   "0.8087033882600653`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.752746045003173`", ",", "0.6154132449896567`", ",", 
   "0.3445638484232505`", ",", "14.153290408343063`"}], "}"}]], "Print",
 CellChangeTimes->{3.593505465655902*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0027999999999999987`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0027999999999999987`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935054659835024`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935054659835024`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96691161474928417422383267702520112012`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96691161474928417422383267702520112012`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593505634791399*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593505642482212*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9664616147492843`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9664616147492843],
  Editable->False]], "Print",
 CellChangeTimes->{3.59350576634643*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593505818871722*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "33.95978038197536`", ",", "74.36280444090946`", ",", "86.00110775913537`", 
   ",", "88.95345834094502`", ",", "93.95770643360757`", ",", 
   "95.92813856215844`", ",", "97.3756772554823`", ",", "99.26400121214925`", 
   ",", "86.00110775913537`", ",", "0.8147405731901961`", ",", 
   "0.8136159238772147`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.7684132098786252`", ",", "0.6154132449896567`", ",", 
   "0.35198929120864575`", ",", "14.01764980830183`"}], "}"}]], "Print",
 CellChangeTimes->{3.593505818871722*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0028999999999999985`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0028999999999999985`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935058188873224`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935058188873224`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9666716322394812640093982513925350031`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9666716322394812640093982513925350031`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935059875080185`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935059951832323`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9661916322394812`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9661916322394812],
  Editable->False]], "Print",
 CellChangeTimes->{3.593506117456247*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59350617035594*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "34.562626542996625`", ",", "74.9952121223545`", ",", "86.4477579646683`", 
   ",", "89.31446074351469`", ",", "94.14140555618164`", ",", 
   "96.04694177779642`", ",", "97.45105910026804`", ",", "99.28566233496142`",
    ",", "86.4477579646683`", ",", "0.819055557825497`", ",", 
   "0.8179594658946107`", ",", "0.30316134587531424`", ",", "Indeterminate", 
   ",", "1.7836688840156907`", ",", "0.6154132449896567`", ",", 
   "0.3594021980958134`", ",", "14.422786833743189`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935061703715396`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593506185753167*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935064448852215`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "35.51796530089701`", ",", "74.80010819342392`", ",", "86.26019791395196`", 
   ",", "89.16021328591141`", ",", "94.07085047652357`", ",", 
   "96.00471087777282`", ",", "97.42851787866874`", ",", "99.28288068460846`",
    ",", "86.26019791395196`", ",", "0.8182344767190504`", ",", 
   "0.81711873562558`", ",", "0.30428546308781024`", ",", "Indeterminate", 
   ",", "1.7721427921194606`", ",", "0.6200951368839737`", ",", 
   "0.35214750486186863`", ",", "14.114868497550582`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935064448852215`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0027999999999999987`", ",", "0.9664616147492843`", ",", 
     "0.9566616147492842`", ",", "0.9762616147492843`", ",", 
     "19.798235080742085`", ",", "6.618004590142314`", ",", 
     "1.102433457982714`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0027999999999999987`, 0.9664616147492843, 0.9566616147492842, 
   0.9762616147492843, 19.798235080742085`, 6.618004590142314, 
   1.102433457982714}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935064452596226`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "110.61693428833333333333333333333333333333`10.972487866187961"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   110.61693428833333333333333333333333333333`10.972487866187961],
  Editable->False]], "Print",
 CellChangeTimes->{3.593506445306422*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593506445368823*^9}],

Cell[BoxData["\<\"Country: Italy\"\>"], "Print",
 CellChangeTimes->{3.593506445368823*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593506445634023*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5935064490816293`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593506449097229*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96917299439053650189634936396032571793`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96917299439053650189634936396032571793`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593506767883789*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5935067758086033`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.71964302166666666666666666666666666667`9.164104549884325"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   1.71964302166666666666666666666666666667`9.164104549884325],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935068789871845`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5935068789871845`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5935068789871845`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.59350688220079*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59350688220079*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96936175514781789724975869445257286536`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96936175514781789724975869445257286536`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935070781995344`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593507086015148*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9693467551478178`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9693467551478178],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935071699432955`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935072207993846`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.440870601218112`", ",", "50.761073282392864`", ",", "66.9921393715559`",
    ",", "72.51523390690917`", ",", "84.12262660112343`", ",", 
   "89.34337964219591`", ",", "93.26746020912358`", ",", "98.26646272168699`",
    ",", "66.9921393715559`", ",", "0.6330967825839732`", ",", 
   "0.6308084372301563`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.6000095200399804`", ",", "0.632039912417306`", ",", 
   "0.24874765931243836`", ",", "134.05206792402257`"}], "}"}]], "Print",
 CellChangeTimes->{3.593507220814985*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593507220814985*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593507220814985*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96916837183910037955854275563066369498`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96916837183910037955854275563066369498`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935074138653235`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935074216965375`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9691433718391004`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9691433718391004],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935075053594847`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935075573387756`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.234184318713073`", ",", "52.0573777369527`", ",", "68.10597401015013`", 
   ",", "73.5031071093282`", ",", "84.76680814628216`", ",", 
   "89.79813396032044`", ",", "93.56785782607956`", ",", "98.35174137498308`",
    ",", "68.10597401015013`", ",", "0.6439789952342252`", ",", 
   "0.6417605881833175`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.6207194339745954`", ",", "0.632039912417306`", ",", 
   "0.25466306240098213`", ",", "106.47973987528957`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935075573387756`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935075576975765`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935075576975765`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96897150176367263324978513808916083494`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96897150176367263324978513808916083494`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593507751543517*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59350775903153*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9689365017636726`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9689365017636726],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935078422264757`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593507894985769*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.016058001163294`", ",", "53.306251341122`", ",", "69.16822570928787`", 
   ",", "74.44196992474744`", ",", "85.37228748210977`", ",", 
   "90.2238896893603`", ",", "93.84840079860545`", ",", "98.43092671591963`", 
   ",", "69.16822570928787`", ",", "0.6543394976884654`", ",", 
   "0.6521877285385654`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.6409597735828492`", ",", "0.632039912417306`", ",", 
   "0.26059221731219406`", ",", "83.61080270603262`"}], "}"}]], "Print",
 CellChangeTimes->{3.593507894985769*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593507894985769*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935078950013685`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96876850264305332196387188856795719528`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96876850264305332196387188856795719528`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935080885197086`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593508096319722*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9687235026430533`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9687235026430533],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935082007307057`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935082523667965`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.804782323196434`", ",", "54.526721767334394`", ",", 
   "70.18962514685259`", ",", "75.34122603257597`", ",", "85.94401917809115`",
    ",", "90.6238126819207`", ",", "94.11094320478769`", ",", 
   "98.504899261338`", ",", "70.18962514685259`", ",", "0.6642963438978428`", 
   ",", "0.662208705964622`", ",", "0.3043122155904047`", ",", 
   "Indeterminate", ",", "1.6617743781050296`", ",", "0.632039912417306`", 
   ",", "0.26663687822764814`", ",", "64.77457918362957`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935082523667965`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593508252694397*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593508252709997*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96856218919531394823944700449303870017`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96856218919531394823944700449303870017`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593508445619936*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935084534043493`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9685021891953138`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9685021891953138],
  Editable->False]], "Print",
 CellChangeTimes->{3.593508575053363*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935086270326543`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.555179244906988`", ",", "55.66811454840016`", ",", "71.13456107547772`",
    ",", "76.17149063262802`", ",", "86.46626863895257`", ",", 
   "90.98754797628288`", ",", "94.34909747517293`", ",", "98.57223957171128`",
    ",", "71.13456107547772`", ",", "0.6735008820913269`", ",", 
   "0.6714724988913559`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.6813371216674668`", ",", "0.632039912417306`", ",", 
   "0.27279127948806403`", ",", "50.022648073364394`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935086270638547`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0019000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0019000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935086270794544`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935086270794544`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96835045337445343499719320156298266557`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96835045337445343499719320156298266557`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593508821299796*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935088291466093`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9682804533744535`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9682804533744535],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935089538532286`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593509004412917*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.316081078067448`", ",", "56.79292736809206`", ",", "72.05268396681257`",
    ",", "76.97550091567172`", ",", "86.96638328762177`", ",", 
   "91.33408760256381`", ",", "94.57521991304074`", ",", "98.6359301107638`", 
   ",", "72.05268396681257`", ",", "0.6824466711609586`", ",", 
   "0.6804759754546278`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.700651869233823`", ",", "0.632039912417306`", ",", 
   "0.2788793062535343`", ",", "38.14285541933596`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935090044285173`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.002],
  Editable->False]], "Print",
 CellChangeTimes->{3.593509004771718*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593509004771718*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9681360135210497507542532116912081819`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9681360135210497507542532116912081819`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593509198929659*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593509206760873*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9680510135210497`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9680510135210497],
  Editable->False]], "Print",
 CellChangeTimes->{3.59350933043789*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593509383883584*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.069628315841985`", ",", "57.87570650099785`", ",", "72.92422077747572`",
    ",", "77.73512721337276`", ",", "87.43376112338603`", ",", 
   "91.65653441413733`", ",", "94.78481507753968`", ",", "98.69484423743916`",
    ",", "72.92422077747572`", ",", "0.6909403084516387`", ",", 
   "0.6890244131995672`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.7196787312441597`", ",", "0.632039912417306`", ",", 
   "0.28504024767388036`", ",", "29.083027357673778`"}], "}"}]], "Print",
 CellChangeTimes->{3.593509383883584*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0021],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935093842267847`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935093842267847`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96791733190170252787250929909700417375`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96791733190170252787250929909700417375`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593509577526724*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935095852643375`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9678173319017025`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9678173319017025],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935097281605883`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935097803738804`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.793856146502353`", ",", "58.89907572503369`", ",", "73.74079042150382`",
    ",", "78.44320245694453`", ",", "87.86622153505806`", ",", 
   "91.95354695808996`", ",", "94.97729318841074`", ",", "98.74874395351073`",
    ",", "73.74079042150382`", ",", "0.6988873267062928`", ",", 
   "0.6970226829165785`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.7384139756939052`", ",", "0.632039912417306`", ",", 
   "0.2912072607912868`", ",", "22.518895567837546`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935097804050803`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0021999999999999997`],
  Editable->False]], "Print",
 CellChangeTimes->{3.59350978042068*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59350978042068*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96769546858768395559253333078524105678`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96769546858768395559253333078524105678`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935099735646195`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935099812866335`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.967575468587684`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.967575468587684],
  Editable->False]], "Print",
 CellChangeTimes->{3.593510147286525*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593510200389018*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.482267504009297`", ",", "59.85385032196023`", ",", "74.49657174182455`",
    ",", "79.09517275653286`", ",", "88.26164699019475`", ",", 
   "92.223885184566`", ",", "95.15215133423509`", ",", "98.79762023322974`", 
   ",", "74.49657174182455`", ",", "0.7062365801070405`", ",", 
   "0.7044192816168183`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.7562001511134602`", ",", "0.632039912417306`", ",", 
   "0.2974562207617155`", ",", "18.08630072572042`"}], "}"}]], "Print",
 CellChangeTimes->{3.593510200389018*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0022999999999999995`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0022999999999999995`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935102004358177`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935102004358177`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96747109099293257679527915412456443789`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96747109099293257679527915412456443789`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593510393532957*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935104013017707`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9673360909929326`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9673360909929326],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935105052603536`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935105595952487`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.21497684019544`", ",", "60.82431771833764`", ",", "75.25509281819238`", 
   ",", "79.74618354416569`", ",", "88.65415445887224`", ",", 
   "92.4912496936354`", ",", "95.32446471033789`", ",", "98.84527552337521`", 
   ",", "75.25509281819238`", ",", "0.7136367868496717`", ",", 
   "0.7118673307045854`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.7739026938756315`", ",", "0.632039912417306`", ",", 
   "0.30354763725658074`", ",", "15.209533028132363`"}], "}"}]], "Print",
 CellChangeTimes->{3.59351055993845*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0023999999999999994`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0023999999999999994`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935105599540496`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935105599540496`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96724266182078710693062346197946838211`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96724266182078710693062346197946838211`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935107330363536`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593510741195168*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9670876618207871`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9670876618207871],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935108752462034`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935109314687023`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.88293629813114`", ",", "61.703636604177134`", ",", "75.93839489960905`",
    ",", "80.3309991014717`", ",", "89.00494893573166`", ",", 
   "92.72974721976077`", ",", "95.4778234831202`", ",", "98.88794662390303`", 
   ",", "75.93839489960905`", ",", "0.7202991379992599`", ",", 
   "0.7185726345522323`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.7914829005052915`", ",", "0.632039912417306`", ",", 
   "0.30972543017874754`", ",", "13.944326319369711`"}], "}"}]], "Print",
 CellChangeTimes->{3.593510931499902*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002499999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.002499999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593510931811903*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935109318275027`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96701272938441708758818521565174575436`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96701272938441708758818521565174575436`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593511105767808*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593511113443022*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.966837729384417`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.966837729384417],
  Editable->False]], "Print",
 CellChangeTimes->{3.593511279302513*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593511332592207*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.547159011016532`", ",", "62.55724505526403`", ",", "76.59522087590194`",
    ",", "80.89171750314388`", ",", "89.33891751344343`", ",", 
   "92.95658531399215`", ",", "95.62333135621236`", ",", "98.92837137661314`",
    ",", "76.59522087590194`", ",", "0.7267182825940811`", ",", 
   "0.7250332172284062`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.8087350106818467`", ",", "0.632039912417306`", ",", 
   "0.31584585840705337`", ",", "13.91091592665812`"}], "}"}]], "Print",
 CellChangeTimes->{3.593511332592207*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002599999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.002599999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593511332654607*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593511332670207*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96678035902159163690881148925246231348`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96678035902159163690881148925246231348`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935115011193027`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593511508841316*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9665903590215916`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9665903590215916],
  Editable->False]], "Print",
 CellChangeTimes->{3.593511674591607*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935117275693007`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.22853211215179`", ",", "63.40828987011942`", ",", "77.24443719853491`", 
   ",", "81.44490121989313`", ",", "89.6668199615499`", ",", 
   "93.1794974206271`", ",", "95.76593295003144`", ",", "98.96779092706744`", 
   ",", "77.24443719853491`", ",", "0.7330878756500419`", ",", 
   "0.7314440440782721`", ",", "0.3043122155904047`", ",", "Indeterminate", 
   ",", "1.8256272996053038`", ",", "0.632039912417306`", ",", 
   "0.321797156801905`", ",", "15.003403662102667`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935117276317005`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593511743325328*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59351200643539*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.11494500299283`", ",", "63.42682618555815`", ",", "77.02266265347853`", 
   ",", "81.22697283982707`", ",", "89.51335077442187`", ",", 
   "93.06751799133941`", ",", "95.68745761717065`", ",", "98.94197043249692`",
    ",", "77.02266265347853`", ",", "0.7323138657307794`", ",", 
   "0.7306476966657509`", ",", "0.30568667814672024`", ",", "Indeterminate", 
   ",", "1.8136685899824931`", ",", "0.6230905715300645`", ",", 
   "0.31643432045217446`", ",", "14.809395604394723`"}], "}"}]], "Print",
 CellChangeTimes->{3.59351200646659*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.002499999999999999`", ",", "0.966837729384417`", ",", 
     "0.958087729384417`", ",", "0.9755877293844171`", ",", 
     "19.76523136872822`", ",", "6.610941841605763`", ",", 
     "0.9821304659381864`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.002499999999999999, 0.966837729384417, 0.958087729384417, 
   0.9755877293844171, 19.76523136872822, 6.610941841605763, 
   0.9821304659381864}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935120066381903`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", "92.68444279`10.895673091556878"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   92.68444279`10.895673091556878],
  Editable->False]], "Print",
 CellChangeTimes->{3.593512006684991*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5935120067161903`*^9}],

Cell[BoxData["\<\"Country: Luxembourg\"\>"], "Print",
 CellChangeTimes->{3.593512006762991*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593512007059391*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593512010475797*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593512010475797*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615811824576155686372658237815`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615811824576155686372658237815`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593512332865964*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5935123407751775`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.38268242833333333333333333333333333333`9.06938869191151"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   1.38268242833333333333333333333333333333`9.06938869191151],
  Editable->False]], "Print",
 CellChangeTimes->{3.593512423736123*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5935124237517233`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5935124237517233`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935124269653287`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935124269653287`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268041759459774718706252646`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268041759459774718706252646`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593512578113994*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593512585680008*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698933289126804`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698933289126804],
  Editable->False]], "Print",
 CellChangeTimes->{3.593512692789796*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593512746001489*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.481478298835444`", ",", "59.51514916874336`", ",", "74.84182431688097`",
    ",", "79.6391146014348`", ",", "88.81512537421841`", ",", 
   "92.53402311914176`", ",", "95.22005959124904`", ",", "98.64815128152145`",
    ",", "74.84182431688097`", ",", "0.7077369835895508`", ",", 
   "0.7059431015080925`", ",", "0.30402620069006936`", ",", "Indeterminate", 
   ",", "1.5048843137783308`", ",", "0.621600677872024`", ",", 
   "0.23747012649975419`", ",", "11.225881994043299`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935127463290896`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.59351274637589*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59351274637589*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108102859228373993885249543`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108102859228373993885249543`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593512940986232*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593512948567845*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696788465510809`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696788465510809],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935130571440353`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593513111510131*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.488152941862715`", ",", "60.94803491170897`", ",", "76.02946767214476`",
    ",", "80.67528125036772`", ",", "89.44732504196008`", ",", 
   "92.96141238939542`", ",", "95.4935676974926`", ",", "98.7272590888766`", 
   ",", "76.02946767214476`", ",", "0.7191188573702175`", ",", 
   "0.7173983987358618`", ",", "0.30402620069006936`", ",", "Indeterminate", 
   ",", "1.5306889132247783`", ",", "0.621600677872024`", ",", 
   "0.24591457653028193`", ",", "15.148881780612616`"}], "}"}]], "Print",
 CellChangeTimes->{3.593513111510131*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0013`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0013],
  Editable->False]], "Print",
 CellChangeTimes->{3.593513111572531*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593513111588131*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5935131695266333`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593513170135034*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593513177201846*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9700799999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9700799999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935132811136293`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935133330305204`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.356813644992876`", ",", "57.89501788940057`", ",", "73.47537595591041`",
    ",", "78.43706061019168`", ",", "88.05852465223167`", ",", 
   "92.01538139604686`", ",", "94.88445376381458`", ",", "98.55013421315448`",
    ",", "73.47537595591041`", ",", "0.6944778271924587`", ",", 
   "0.6925983332884853`", ",", "0.30402620069006936`", ",", "Indeterminate", 
   ",", "1.4765031755518827`", ",", "0.621600677872024`", ",", 
   "0.2297348864991695`", ",", "11.622487312090584`"}], "}"}]], "Print",
 CellChangeTimes->{3.59351333306172*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5935133486461477`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935136135190125`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.828868621178596`", ",", "60.235778960401454`", ",", "75.2111465324908`",
    ",", "79.90444009112508`", ",", "88.90280737154069`", ",", 
   "92.57828543382618`", ",", "95.24239482827885`", ",", "98.65063991542222`",
    ",", "75.2111465324908`", ",", "0.7119543277397069`", ",", 
   "0.710146170074149`", ",", "0.30578332835821764`", ",", "Indeterminate", 
   ",", "1.497248282969074`", ",", "0.626793684630277`", ",", 
   "0.23756994921784513`", ",", "12.572226911802142`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935136135190125`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0014`", ",", "0.9698933289126804`", ",", "0.9649933289126804`", ",", 
     "0.9747933289126804`", ",", "19.7931670111627`", ",", 
     "6.6169203059080886`", ",", "0.8254507745683072`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0014, 
   0.9698933289126804, 0.9649933289126804, 0.9747933289126804, 
   19.7931670111627, 6.6169203059080886`, 0.8254507745683072}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935136138934135`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "26.78004703666666666666666666666666666667`10.356477583690232"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   26.78004703666666666666666666666666666667`10.356477583690232],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935136139558134`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5935136140182133`*^9}],

Cell[BoxData["\<\"Country: Malta\"\>"], "Print",
 CellChangeTimes->{3.5935136140338135`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593513614299014*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.59351361766862*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935136176842203`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9697975561583619397154620855872053653`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9697975561583619397154620855872053653`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593513936813981*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5935139447075944`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.35304237666666666666666666666666666667`9.059977646930246"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   1.35304237666666666666666666666666666667`9.059977646930246],
  Editable->False]], "Print",
 CellChangeTimes->{3.593514025890137*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593514025890137*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593514025905737*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.593514029088143*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593514029088143*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268031671144171080143283976`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268031671144171080143283976`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935141807204094`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935141887076235`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698733289126802`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698733289126802],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935142948034096`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935143462991*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.067931636653295`", ",", "60.39721643607338`", ",", "75.34596747600078`",
    ",", "80.00550997476715`", ",", "88.93745386777931`", ",", 
   "92.59164379436476`", ",", "95.2464673458578`", ",", "98.64998187819879`", 
   ",", "75.34596747600078`", ",", "0.7129288734068222`", ",", 
   "0.711120074448421`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.499290639457022`", ",", "0.6265737380926085`", ",", 
   "0.23849907531792047`", ",", "262.5154233536388`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935143462991*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935143463927*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935143464083*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108070738505578957768743155`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108070738505578957768743155`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935145408938417`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935145488654556`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696638465510806`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696638465510806],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935146768948803`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593514728374971*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.162244032494712`", ",", "61.84257974099053`", ",", "76.5364837201805`", 
   ",", "81.0488606394624`", ",", "89.5803290996953`", ",", 
   "93.02878361807761`", ",", "95.52745357980137`", ",", "98.73103339638672`",
    ",", "76.5364837201805`", ",", "0.7245059860500893`", ",", 
   "0.7227717844932317`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.5245700201779901`", ",", "0.6265737380926085`", ",", 
   "0.246812875087135`", ",", "307.5932219241742`"}], "}"}]], "Print",
 CellChangeTimes->{3.593514728390571*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0013`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0013],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935147287649717`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935147287649717`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593514784129469*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593514784145069*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593514792069883*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.970065`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.970065],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935148956852646`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593514948959358*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.88543125540828`", ",", "58.78785164531828`", ",", "73.99514533716815`", 
   ",", "78.81436644733228`", ",", "88.1882910051506`", ",", 
   "92.07677418949089`", ",", "94.91354111778433`", ",", "98.55383792288445`",
    ",", "73.99514533716815`", ",", "0.6997607945052015`", ",", 
   "0.6978671622567286`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.4713393935493813`", ",", "0.6265737380926085`", ",", 
   "0.230586922888683`", ",", "215.84465635340771`"}], "}"}]], "Print",
 CellChangeTimes->{3.593514948959358*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0012`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0012],
  Editable->False]], "Print",
 CellChangeTimes->{3.593514949021758*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593514949021758*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5935149814542155`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935149814542155`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935149892698293`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.97026`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.97026],
  Editable->False]], "Print",
 CellChangeTimes->{3.593515094726014*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935151466585054`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.74603876234281`", ",", "57.16406243031297`", ",", "72.60001330144422`", 
   ",", "77.57426275277439`", ",", "87.39459525966275`", ",", 
   "91.52792500598804`", ",", "94.55796337157241`", ",", "98.45055104131991`",
    ",", "72.60001330144422`", ",", "0.6861663702710786`", ",", 
   "0.6841855833055489`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.4429677646119408`", ",", "0.6265737380926085`", ",", 
   "0.22255623355522858`", ",", "172.7782999833716`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935151466741056`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0010999999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0010999999999999998`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593515146783306*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593515146783306*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593515179309363*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593515179496563*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935151795277634`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593515187452577*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9704499999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9704499999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935153326264324`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593515384137723*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.575874086253904`", ",", "55.445533540179916`", ",", "71.0987855297158`",
    ",", "76.23187244929125`", ",", "86.5175364608484`", ",", 
   "90.91551976136529`", ",", "94.15950244707044`", ",", "98.33443585028586`",
    ",", "71.0987855297158`", ",", "0.6714672697445956`", ",", 
   "0.6693924584641118`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.4138604980184106`", ",", "0.6265737380926085`", ",", 
   "0.214718155148943`", ",", "132.40990774430588`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935153841533227`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009999999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009999999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935153845277233`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935153845277233`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935154163517795`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593515424276593*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.970625`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.970625],
  Editable->False]], "Print",
 CellChangeTimes->{3.59351561052532*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593515663222213*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.39038627901243`", ",", "53.61814784075922`", ",", "69.47399379295128`", 
   ",", "74.76592494340825`", ",", "85.53737584299837`", ",", 
   "90.22515013964298`", ",", "93.70559973435022`", ",", "98.20270475048542`",
    ",", "69.47399379295128`", ",", "0.6554524190672805`", ",", 
   "0.653275397135033`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.3832105963442145`", ",", "0.6265737380926085`", ",", 
   "0.20740760178643913`", ",", "95.88402117093804`"}], "}"}]], "Print",
 CellChangeTimes->{3.593515663222213*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008999999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0008999999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.593515663315813*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593515663315813*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935156957638702`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935157033298836`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9707950000000001`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9707950000000001],
  Editable->False]], "Print",
 CellChangeTimes->{3.593515888658209*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935159408403006`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.198265542434594`", ",", "51.69736024435521`", ",", "67.74076565485984`",
    ",", "73.18736719441098`", ",", "84.46353781355684`", ",", 
   "89.46250520258602`", ",", "93.20170926751395`", ",", "98.05719973772032`",
    ",", "67.74076565485984`", ",", "0.6382713273437071`", ",", 
   "0.6359850759510521`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.352447025951914`", ",", "0.6265737380926085`", ",", 
   "0.20049791767397473`", ",", "65.39669745640113`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935159408403006`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0007999999999999997],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935159409183006`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935159409183006`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935159733507576`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593515981275572*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.970945`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.970945],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935161485858655`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935162008147573`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "15.983908099930726`", ",", "49.64795699225757`", ",", "65.8661695793733`", 
   ",", "71.47015703035439`", ",", "83.2780211012378`", ",", 
   "88.60948917516693`", ",", "92.634438506401`", ",", "97.89411854165596`", 
   ",", "65.8661695793733`", ",", "0.619587777464109`", ",", 
   "0.6171831298749748`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.3215911119903072`", ",", "0.6265737380926085`", ",", 
   "0.19446256209217372`", ",", "42.54796765130054`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935162008303576`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0006999999999999997],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935162011891575`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935162012047577`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593516233091214*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593516240969228*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9710799999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9710799999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935164298855596`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593516482488852*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.78888161526439`", ",", "47.52245602301945`", ",", "63.88805721549182`", 
   ",", "69.6511203022507`", ",", "82.00554586400305`", ",", 
   "87.68800016464063`", ",", "92.0219987888747`", ",", "97.72241575662257`", 
   ",", "63.88805721549182`", ",", "0.5998851724193587`", ",", 
   "0.5973561841649494`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.2919397437812048`", ",", "0.6265737380926085`", ",", 
   "0.18929003938548616`", ",", "30.019834664226565`"}], "}"}]], "Print",
 CellChangeTimes->{3.593516482520052*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0005999999999999996`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0005999999999999996],
  Editable->False]], "Print",
 CellChangeTimes->{3.593516482535652*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593516482535652*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935165147185087`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593516522658922*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9711999999999998`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9711999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935167122772555`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.59351676578535*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.60457263573675`", ",", "45.33126696243199`", ",", "61.82880291943379`", 
   ",", "67.74939977270724`", ",", "80.67333395943973`", ",", 
   "86.72506006703131`", ",", "91.3827235847943`", ",", "97.55262237333265`", 
   ",", "61.82880291943379`", ",", "0.5794679357792399`", ",", 
   "0.5768105646275197`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.2652965379854146`", ",", "0.6265737380926085`", ",", 
   "0.18501862489231724`", ",", "29.61805340683683`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935167658321495`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0004999999999999996`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0004999999999999996],
  Editable->False]], "Print",
 CellChangeTimes->{3.59351676586335*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59351676586335*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935167980306063`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935168058930197`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9712999999999998`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9712999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935170378654275`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935170897667184`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.456270312079146`", ",", "43.15532179280705`", ",", "59.76782212181592`",
    ",", "65.84930321440939`", ",", "79.35045967466414`", ",", 
   "85.77137352266429`", ",", "90.76234357296002`", ",", "97.40201902471382`",
    ",", "59.76782212181592`", ",", "0.5592690288764947`", ",", 
   "0.5564848750425745`", ",", "0.30722372376015666`", ",", "Indeterminate", 
   ",", "1.243128435238833`", ",", "0.6265737380926085`", ",", 
   "0.18168414137792366`", ",", "41.80197564514678`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935170901099195`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593517105990747*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935173686796083`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.237255920646179`", ",", "44.76301175057224`", ",", 
   "61.407703843255646`", ",", "67.3976206341806`", ",", "80.47684217391925`",
    ",", "86.58031875067152`", ",", "91.27809497537564`", ",", 
   "97.52025427598942`", ",", "61.407703843255646`", ",", 
   "0.5752846037714745`", ",", "0.5726132712199017`", ",", 
   "0.3046809675139576`", ",", "Indeterminate", ",", "1.2580269481675226`", 
   ",", "0.6224141140382868`", ",", "0.18477448148409736`", ",", 
   "30.411029046618594`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935173686796083`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0005999999999999996`", ",", "0.9711999999999998`", ",", 
     "0.9690999999999999`", ",", "0.9732999999999998`", ",", 
     "19.62964133792562`", ",", "6.581881094304734`", ",", 
     "0.5844781349772189`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0005999999999999996, 0.9711999999999998, 0.9690999999999999, 
   0.9732999999999998, 19.62964133792562, 6.581881094304734, 
   0.5844781349772189}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593517369178809*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "62.57742991166666666666666666666666666667`10.725083970686692"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   62.57742991166666666666666666666666666667`10.725083970686692],
  Editable->False]], "Print",
 CellChangeTimes->{3.593517369256809*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5935173692724094`*^9}],

Cell[BoxData["\<\"Country: Netherland\"\>"], "Print",
 CellChangeTimes->{3.5935173692880096`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5935173695532103`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593517373032016*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593517373032016*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9697975561583619397154620855872053653`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9697975561583619397154620855872053653`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935176915065756`*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5935176994001894`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "2.092483675`9.249328326644335"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 2.092483675`9.249328326644335],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935178249492097`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.59351782496481*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5935178252924104`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935178281628156`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935178281628156`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268031780745808395923616462`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268031780745808395923616462`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935179788746805`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593517986877494*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698633289126802`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698633289126802],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935180907268763`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593518142612568*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.741775127720256`", ",", "60.41352334618198`", ",", "75.33413528473433`",
    ",", "80.02244773912673`", ",", "88.99019094364485`", ",", 
   "92.6429133412708`", ",", "95.28693987855951`", ",", "98.66217713851941`", 
   ",", "75.33413528473433`", ",", "0.7132437710047561`", ",", 
   "0.7114406722760688`", ",", "0.30911433420411377`", ",", "Indeterminate", 
   ",", "1.5013307429447666`", ",", "0.6233479504007495`", ",", 
   "0.23879017236337552`", ",", "241.76114751370915`"}], "}"}]], "Print",
 CellChangeTimes->{3.593518142612568*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593518142612568*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593518142612568*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108070844446229231065661237`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108070844446229231065661237`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593518338502112*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593518346567326*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696538465510807`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696538465510807],
  Editable->False]], "Print",
 CellChangeTimes->{3.593518453708314*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935185061712065`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.807729614226297`", ",", "61.84931727294324`", ",", "76.51738866326376`",
    ",", "81.06026175237474`", ",", "89.6234760660184`", ",", 
   "93.07220100285167`", ",", "95.56264462502835`", ",", "98.74246330963517`",
    ",", "76.51738866326376`", ",", "0.7246947515477254`", ",", 
   "0.7229657242195836`", ",", "0.30911433420411377`", ",", "Indeterminate", 
   ",", "1.5276006912922035`", ",", "0.6233479504007495`", ",", 
   "0.24716124899652614`", ",", "283.1326735481057`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935185061712065`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0013`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0013],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935185061712065`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935185061868067`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5935185626277056`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935185629709063`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593518570505719*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.970055`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.970055],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935186762739053`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935187289395976`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.591083583487418`", ",", "58.8300487484743`", ",", "74.00689312737595`", 
   ",", "78.84908012547601`", ",", "88.25799498594678`", ",", 
   "92.13979622838984`", ",", "94.96032426408819`", ",", "98.56672884748453`",
    ",", "74.00689312737595`", ",", "0.7003123936114204`", ",", 
   "0.698425702086727`", ",", "0.30911433420411377`", ",", "Indeterminate", 
   ",", "1.473178254411138`", ",", "0.6233479504007495`", ",", 
   "0.23085327393405186`", ",", "199.76896904138624`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935187289395976`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0012`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0012],
  Editable->False]], "Print",
 CellChangeTimes->{3.593518729298398*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593518729298398*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5935187613252544`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935187613564544`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935187691564684`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.97025`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.97025],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935189137843227`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935189666840153`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.497087108703667`", ",", "57.23222890603251`", ",", "72.63215384242162`",
    ",", "77.62431257862463`", ",", "87.48000511617087`", ",", 
   "91.60004878636217`", ",", "94.6082714759041`", ",", "98.46357924553722`", 
   ",", "72.63215384242162`", ",", "0.6869301646587976`", ",", 
   "0.6849571786879882`", ",", "0.30911433420411377`", ",", "Indeterminate", 
   ",", "1.4446828879492222`", ",", "0.6233479504007495`", ",", 
   "0.222816574027216`", ",", "161.33618393491813`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935189666996155`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0010999999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0010999999999999998`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935189670428157`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935189670428157`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593518998991672*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "stop"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Further output of \
\[NoBreak]\\!\\(\\*StyleBox[\\(FindMinimum :: reged\\), \
\\\"MessageName\\\"]\\)\[NoBreak] will be suppressed during this calculation. \
\\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", \
ButtonFrame->None, ButtonData:>\\\"paclet:ref/message/General/stop\\\", \
ButtonNote -> \\\"General::stop\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593518999022872*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593518999022872*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593519006916486*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.97044`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.97044],
  Editable->False]], "Print",
 CellChangeTimes->{3.593519173945979*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593519226174871*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19.393227469978402`", ",", "55.54598118638289`", ",", "71.15204468007819`",
    ",", "76.2982632625318`", ",", "86.61827198471246`", ",", 
   "90.99665566801467`", ",", "94.21306076196988`", ",", "98.34739855775378`",
    ",", "71.15204468007819`", ",", "0.6724650551345513`", ",", 
   "0.6703988937057312`", ",", "0.30911433420411377`", ",", "Indeterminate", 
   ",", "1.415174251834982`", ",", "0.6233479504007495`", ",", 
   "0.21496482434043906`", ",", "125.85642836804426`"}], "}"}]], "Print",
 CellChangeTimes->{3.593519226206071*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0009999999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0009999999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.593519226221671*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935192262372713`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935192584669275`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935192663605413`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9706149999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9706149999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593519432469633*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935194844801245`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "18.273719543922144`", ",", "53.758072994730696`", ",", 
   "69.55474273715316`", ",", "74.85567004429107`", ",", "85.65577977179208`",
    ",", "90.31487606900096`", ",", "93.76166837530077`", ",", 
   "98.21487052773999`", ",", "69.55474273715316`", ",", 
   "0.6567205781184665`", ",", "0.6545530059236886`", ",", 
   "0.30911433420411377`", ",", "Indeterminate", ",", "1.3846728648132305`", 
   ",", "0.6233479504007495`", ",", "0.2076548980276719`", ",", 
   "94.65621979371707`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935194844801245`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0008999999999999998`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0008999999999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935194844801245`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935194844801245`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935195168345814`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593519524790595*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9707799999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9707799999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935197126461253`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593519764906217*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "17.14130380397309`", ",", "51.8630083504611`", ",", "67.84105716789206`", 
   ",", "73.29228298983072`", ",", "84.58915752803102`", ",", 
   "89.55338643899734`", ",", "93.25325311200393`", ",", "98.0664963556015`", 
   ",", "67.84105716789206`", ",", "0.6396836040681446`", ",", 
   "0.6374062792547915`", ",", "0.30911433420411377`", ",", "Indeterminate", 
   ",", "1.3535711228962637`", ",", "0.6233479504007495`", ",", 
   "0.20087803689434247`", ",", "69.62648963386766`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935197649218173`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0007999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0007999999999999997],
  Editable->False]], "Print",
 CellChangeTimes->{3.593519765249418*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593519765249418*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593519797307474*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935198051854877`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.97093`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.97093],
  Editable->False]], "Print",
 CellChangeTimes->{3.593519972214981*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935200241474724`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "16.013584974948305`", ",", "49.87497382228426`", ",", "66.00945973694864`",
    ",", "71.60757244264155`", ",", "83.41975084667624`", ",", 
   "88.70950229280828`", ",", "92.68919981545002`", ",", "97.90410262194547`",
    ",", "66.00945973694864`", ",", "0.6214012788581188`", ",", 
   "0.6190062650956706`", ",", "0.30911433420411377`", ",", "Indeterminate", 
   ",", "1.322248973841419`", ",", "0.6233479504007495`", ",", 
   "0.19481714137445394`", ",", "52.7793732929608`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935200241474724`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0006999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0006999999999999997],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935200242254725`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935200242254725`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935200566579294`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935200645827436`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9710649999999998`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9710649999999998],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935202738259115`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935203266164036`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.908419222511046`", ",", "47.82568266257839`", ",", "64.08130477297537`",
    ",", "69.82606187805102`", ",", "82.1658997859487`", ",", 
   "87.80035237534594`", ",", "92.0820619980652`", ",", "97.73393831187587`", 
   ",", "64.08130477297537`", ",", "0.6021672910297597`", ",", 
   "0.5996485635659126`", ",", "0.30911433420411377`", ",", "Indeterminate", 
   ",", "1.2924724129544811`", ",", "0.6233479504007495`", ",", 
   "0.18959774586087363`", ",", "46.247624750280146`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935203266164036`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0005999999999999996`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0005999999999999996],
  Editable->False]], "Print",
 CellChangeTimes->{3.593520326944004*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593520326944004*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59352035893966*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935203668332744`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9711849999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9711849999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.593520553830803*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935206062780952`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.822244236244567`", ",", "45.716549138142305`", ",", 
   "62.076525709970895`", ",", "67.96740408414358`", ",", "80.852511902122`", 
   ",", "86.84779028521248`", ",", "91.44787182436077`", ",", 
   "97.56505499315843`", ",", "62.076525709970895`", ",", 
   "0.5822635897190093`", ",", "0.5796165814893269`", ",", 
   "0.30911433420411377`", ",", "Indeterminate", ",", "1.2656587862998034`", 
   ",", "0.6233479504007495`", ",", "0.18528269789397567`", ",", 
   "51.60076862778338`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935206062780952`*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593520622096523*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935208811973777`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "14.14594316380299`", ",", "46.79354993170605`", ",", "63.39556543802295`", 
   ",", "69.26147635678099`", ",", "81.83425945634609`", ",", 
   "87.5787119829555`", ",", "91.94719086974298`", ",", "97.69848144238725`", 
   ",", "63.39556543802295`", ",", "0.5952217660772439`", ",", 
   "0.5926831050576693`", ",", "0.3063434035888164`", ",", "Indeterminate", 
   ",", "1.2877123909977155`", ",", "0.626514575327523`", ",", 
   "0.18942807701479947`", ",", "45.03716487245468`"}], "}"}]], "Print",
 CellChangeTimes->{3.593520881212978*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0006999999999999997`", ",", "0.9710649999999998`", ",", 
     "0.9686149999999999`", ",", "0.9735149999999998`", ",", 
     "19.61916257260801`", ",", "6.579632197670671`", ",", 
     "0.81622067041063`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0006999999999999997, 0.9710649999999998, 0.9686149999999999, 
   0.9735149999999998, 19.61916257260801, 6.579632197670671, 
   0.81622067041063}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935208815717783`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", "58.532082805`10.696060226534911"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   58.532082805`10.696060226534911],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935208815717783`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.5935208817121787`*^9}],

Cell[BoxData["\<\"Country: Portugal\"\>"], "Print",
 CellChangeTimes->{3.593520881727779*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593520881992979*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5935208854561853`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935208854561853`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615836127358164731049328111112`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615836127358164731049328111112`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593521205319147*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5935212133219614`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.37436241333333333333333333333333333333`9.066767517589048"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   1.37436241333333333333333333333333333333`9.066767517589048],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935212957837057`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5935212961269064`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5935212961269064`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935212989817114`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593521299012912*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268086372112522429995267823`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268086372112522429995267823`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935214497247763`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59352145769639*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698883289126808`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698883289126808],
  Editable->False]], "Print",
 CellChangeTimes->{3.593521585445015*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935216358799033`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.415573078651246`", ",", "59.85688885953766`", ",", "75.06753708335157`",
    ",", "79.7949085637463`", ",", "88.84729847012626`", ",", 
   "92.54076023554892`", ",", "95.21963509069684`", ",", "98.64717474705783`",
    ",", "75.06753708335157`", ",", "0.7099049346822335`", ",", 
   "0.7080966225312719`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.4989021908581952`", ",", "0.6211842906134262`", ",", 
   "0.2378704470496379`", ",", "107.52121234801027`"}], "}"}]], "Print",
 CellChangeTimes->{3.593521635895503*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935216359423037`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935216359423037`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108114441492318588323261219`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108114441492318588323261219`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935218306618457`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935218386646595`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.969673846551081`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.969673846551081],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935219654616823`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935220175033736`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.509936947357644`", ",", "61.274593371649836`", ",", "76.252088185107`", 
   ",", "80.83522657697635`", ",", "89.48459537239412`", ",", 
   "92.97218712747272`", ",", "95.49565567832641`", ",", "98.72616418081974`",
    ",", "76.252088185107`", ",", "0.7213520713867649`", ",", 
   "0.7196172083456663`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.524638055333013`", ",", "0.6211842906134262`", ",", 
   "0.24636612089315402`", ",", "81.69440645308427`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935220175033736`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935220175345736`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935220175345736`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96959487213047199196912846444947962912`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96959487213047199196912846444947962912`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935222141729193`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935222217233324`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.969454872130472`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.969454872130472],
  Editable->False]], "Print",
 CellChangeTimes->{3.59352234560315*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935223968492403`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.60832590516412`", ",", "62.63584949711043`", ",", "77.36028035756732`", 
   ",", "81.80285672588397`", ",", "90.0682454606212`", ",", 
   "93.36484749464385`", ",", "95.74668227282062`", ",", "98.79758050151266`",
    ",", "77.36028035756732`", ",", "0.7321155658341569`", ",", 
   "0.7304499609978483`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.5487657532872745`", ",", "0.6211842906134262`", ",", 
   "0.25485423753102276`", ",", "61.128107854157946`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935223968492403`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.59352239692724*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.59352239692724*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96938962337162044855499290492562359071`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96938962337162044855499290492562359071`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593522593269185*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593522601178399*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9692296233716204`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9692296233716204],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935227462898536`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935227974891434`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.656013810427503`", ",", "63.9112384638725`", ",", "78.37455501984832`", 
   ",", "82.68116152954475`", ",", "90.58945399671639`", ",", 
   "93.71409609275264`", ",", "95.96935305387132`", ",", "98.86120677407479`",
    ",", "78.37455501984832`", ",", "0.7419732671849467`", ",", 
   "0.7403711511821955`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.5716230101086301`", ",", "0.6211842906134262`", ",", 
   "0.26337829484222897`", ",", "45.329086981384094`"}], "}"}]], "Print",
 CellChangeTimes->{3.593522797816744*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935227978479443`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935227978479443`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96918076512128245331634065113176352827`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96918076512128245331634065113176352827`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935229948138905`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593523002676304*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9689957651212824`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9689957651212824],
  Editable->False]], "Print",
 CellChangeTimes->{3.59352312562012*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935231775838113`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.661103270868875`", ",", "65.10275734784149`", ",", "79.30057236042657`",
    ",", "83.4751177156975`", ",", "91.05181795111866`", ",", 
   "94.02246082090191`", ",", "96.165919204289`", ",", "98.91787107521736`", 
   ",", "79.30057236042657`", ",", "0.7509736583503563`", ",", 
   "0.7494295628448019`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.593225982586892`", ",", "0.6211842906134262`", ",", 
   "0.2719562660969451`", ",", "33.41126090838531`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935231775838113`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0019000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0019000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935231776306114`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935231776306114`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96896693706165320749171625062727126974`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96896693706165320749171625062727126974`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593523373457755*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935233813201694`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9687619370616531`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9687619370616531],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935235051219864`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593523557194878*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "27.6917436425004`", ",", "66.27876636209403`", ",", "80.19393919599342`", 
   ",", "84.23065629802983`", ",", "91.48455682249104`", ",", 
   "94.31062424203618`", ",", "96.34993393411561`", ",", "98.97105989079762`",
    ",", "80.19393919599342`", ",", "0.7596719477593982`", ",", 
   "0.7581840803051232`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.6144899195532525`", ",", "0.6211842906134262`", ",", 
   "0.28038270584841785`", ",", "24.144346123211907`"}], "}"}]], "Print",
 CellChangeTimes->{3.593523557210478*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.002],
  Editable->False]], "Print",
 CellChangeTimes->{3.593523557241678*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593523557257278*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96875025993973753146352558050037664074`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96875025993973753146352558050037664074`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935237532092223`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593523760978036*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9685202599397376`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9685202599397376],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935238857626553`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935239388495483`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.648440759330278`", ",", "67.35995453177469`", ",", "81.00358276869039`",
    ",", "84.90744211890282`", ",", "91.86358646131232`", ",", 
   "94.56173269650323`", ",", "96.51010618636464`", ",", "99.01749480004763`",
    ",", "81.00358276869039`", ",", "0.7675110603644749`", ",", 
   "0.7660738444130933`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.6348518810304224`", ",", "0.6211842906134262`", ",", 
   "0.28885080502908433`", ",", "17.624283220026054`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935239388495483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0021],
  Editable->False]], "Print",
 CellChangeTimes->{3.593523939192749*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593523939192749*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96853068954110908590496636603158901333`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96853068954110908590496636603158901333`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59352413358469*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935241410259037`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9682756895411091`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9682756895411091],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935242426444817`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935242946081734`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "29.58372977676116`", ",", "68.39624266854236`", ",", "81.76807384643256`", 
   ",", "85.5386105465321`", ",", "92.20912287234034`", ",", 
   "94.79037676183431`", ",", "96.65568834594096`", ",", "99.05975896399676`",
    ",", "81.76807384643256`", ",", "0.7748832154333304`", ",", 
   "0.7734936891586421`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.6540939176372844`", ",", "0.6211842906134262`", ",", 
   "0.297187400399028`", ",", "13.078950102267978`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935242946081734`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0021999999999999997`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593524294951374*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593524294966974*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96830633444755011984728709918552016702`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96830633444755011984728709918552016702`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593524490123317*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59352449790773*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.96802633444755`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.96802633444755],
  Editable->False]], "Print",
 CellChangeTimes->{3.593524644267187*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935246955756774`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "30.487864539854005`", ",", "69.37212058081056`", ",", "82.4791722086842`", 
   ",", "86.12035603742757`", ",", "92.52059306793178`", ",", 
   "94.99588775549732`", ",", "96.78618455768277`", ",", "99.09752185166182`",
    ",", "82.4791722086842`", ",", "0.7817242469405775`", ",", 
   "0.7803789993264518`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.672234877226553`", ",", "0.6211842906134262`", ",", 
   "0.3054316804393896`", ",", "10.225104888136501`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935246955756774`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0022999999999999995`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0022999999999999995`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593524695918878*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593524695918878*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96808046867657342532962823215909108032`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96808046867657342532962823215909108032`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.59352489059162*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935248983604336`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9677704686765733`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9677704686765733],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935250639547243`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593525116573617*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "31.370935590826747`", ",", "70.30762593416473`", ",", "83.1525383408483`", 
   ",", "86.66752687845126`", ",", "92.80612167082975`", ",", 
   "95.1831383332745`", ",", "96.90510687459187`", ",", "99.13179475018264`", 
   ",", "83.1525383408483`", ",", "0.7881804526537188`", ",", 
   "0.7868770017154189`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.6895350828683626`", ",", "0.6211842906134262`", ",", 
   "0.31363880964671176`", ",", "8.72674328807868`"}], "}"}]], "Print",
 CellChangeTimes->{3.593525116573617*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0023999999999999994`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0023999999999999994`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935251169012175`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935251169168177`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96785214514003096315075084381378365599`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96785214514003096315075084381378365599`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593525310903158*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593525318640772*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9675171451400308`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9675171451400308],
  Editable->False]], "Print",
 CellChangeTimes->{3.593525442192989*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935254936574793`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "32.23376514908316`", ",", "71.1999331859038`", ",", "83.79342910218469`", 
   ",", "87.18850552945771`", ",", "93.07563789425524`", ",", 
   "95.35936685851325`", ",", "97.01750477576185`", ",", "99.16413521797229`",
    ",", "83.79342910218469`", ",", "0.7943367045426318`", ",", 
   "0.7930731072406846`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.7064695815095416`", ",", "0.6211842906134262`", ",", 
   "0.3215984597351097`", ",", "8.334183360273801`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935254936730795`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002499999999999999`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.002499999999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935254940162797`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935254940162797`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96762016565507811823661162243777498341`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96762016565507811823661162243777498341`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935256634169774`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593525671123391*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9672551656550782`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9672551656550782],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935258148620434`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593525866435734*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "33.030140494796235`", ",", "72.01274847795354`", ",", "84.37515760993925`",
    ",", "87.65849270293704`", ",", "93.3135052233063`", ",", 
   "95.51317580296102`", ",", "97.11534250591315`", ",", "99.19231316924964`",
    ",", "84.37515760993925`", ",", "0.7998959133436983`", ",", 
   "0.7986682125530852`", ",", "0.309436955781079`", ",", "Indeterminate", 
   ",", "1.7223484405318081`", ",", "0.6211842906134262`", ",", 
   "0.3295637350162125`", ",", "8.875504373252076`"}], "}"}]], "Print",
 CellChangeTimes->{3.593525866435734*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593525882472562*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935261358482075`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "34.69080955814459`", ",", "72.43329395724541`", ",", "84.39683113878432`", 
   ",", "87.62379687539838`", ",", "93.27131596950375`", ",", 
   "95.48371769630005`", ",", "97.09687458428576`", ",", "99.1852398621989`", 
   ",", "84.39683113878432`", ",", "0.8019424556356574`", ",", 
   "0.8006818066014749`", ",", "0.305053452367583`", ",", "Indeterminate", 
   ",", "1.701226646548415`", ",", "0.6202947477812311`", ",", 
   "0.3217568734360815`", ",", "9.159780021179552`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935261358482075`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0023999999999999994`", ",", "0.9675171451400308`", ",", 
     "0.9591171451400309`", ",", "0.9759171451400308`", ",", 
     "19.789791305949574`", ",", "6.616198037804166`", ",", 
     "2.3357678207545822`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {
   0.0023999999999999994`, 0.9675171451400308, 0.9591171451400309, 
   0.9759171451400308, 19.789791305949574`, 6.616198037804166, 
   2.3357678207545822`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593526136238208*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "87.56893380833333333333333333333333333333`10.871016310222311"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   87.56893380833333333333333333333333333333`10.871016310222311],
  Editable->False]], "Print",
 CellChangeTimes->{3.593526136238208*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593526136316208*^9}],

Cell[BoxData["\<\"Country: Slovenia\"\>"], "Print",
 CellChangeTimes->{3.593526136316208*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.5935261365814085`*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.593526140060214*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593526140060214*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615816010116958523212815634906`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615816010116958523212815634906`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935264605159774`*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.5935264684875917`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "2.41280423833333333333333333333333333333`9.311188335287923"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   2.41280423833333333333333333333333333333`9.311188335287923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935266132558455`*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.5935266132714453`*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.5935266132714453`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.593526616485051*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935266165006514`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268061889133690733216608294`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268061889133690733216608294`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593526766713315*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593526774622529*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698783289126806`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698783289126806],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935268812799163`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935269324480066`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.385654149698144`", ",", "60.385807246414146`", ",", 
   "75.32749906386962`", ",", "79.99935939172165`", ",", "88.93722372482084`",
    ",", "92.59505984431947`", ",", "95.24720349959829`", ",", 
   "98.64886573714139`", ",", "75.32749906386962`", ",", 
   "0.7130485546068511`", ",", "0.7112436492376728`", ",", 
   "0.3009885644750722`", ",", "Indeterminate", ",", "1.4930680412373636`", 
   ",", "0.617946816256622`", ",", "0.23812501686405219`", ",", 
   "85.91486029665211`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935269324636064`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935269328068066`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935269328068066`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108109000530608333204823985`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108109000530608333204823985`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935271260287466`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59352713404716*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9696638465510811`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9696638465510811],
  Editable->False]], "Print",
 CellChangeTimes->{3.593527260625783*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935273117158723`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.474127438502347`", ",", "61.84656890089817`", ",", "76.5162150393282`", 
   ",", "81.03787127308681`", ",", "89.56934005028006`", ",", 
   "93.02502709132723`", ",", "95.52409676715399`", ",", "98.72872993762256`",
    ",", "76.5162150393282`", ",", "0.7245834028889856`", ",", 
   "0.7228517363020365`", ",", "0.3009885644750722`", ",", "Indeterminate", 
   ",", "1.519050945858267`", ",", "0.617946816256622`", ",", 
   "0.2466233300183921`", ",", "64.73100951846295`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935273117314725`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0016`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0016],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935273117782726`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935273117782726`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96959487213047080168636234084005698238`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96959487213047080168636234084005698238`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935275073402157`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.59352751528063*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9694448721304708`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9694448721304708],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935276202376146`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593527673121707*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.562107384579207`", ",", "63.252489954709404`", ",", 
   "77.64062786567192`", ",", "82.01030816172491`", ",", "90.15184091008842`",
    ",", "93.41792894220596`", ",", "95.77679221904896`", ",", 
   "98.80143072226839`", ",", "77.64062786567192`", ",", "0.735478427825034`",
    ",", "0.7338161280594938`", ",", "0.3009885644750722`", ",", 
   "Indeterminate", ",", "1.5434801304313588`", ",", "0.617946816256622`", 
   ",", "0.2551103346350718`", ",", "48.40823917284398`"}], "}"}]], "Print",
 CellChangeTimes->{3.593527673121707*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0017000000000000001`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0017000000000000001`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593527673449308*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935276734649076`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9693896233716192012070737558503501323`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9693896233716192012070737558503501323`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593527869416852*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593527877326066*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9692146233716191`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9692146233716191],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935280234671226`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935280753372135`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.571793017700685`", ",", "64.53765078755828`", ",", "78.64732584866613`",
    ",", "82.87270626054183`", ",", "90.65833921080258`", ",", 
   "93.75684236414696`", ",", "95.99391097063898`", ",", "98.86416172180321`",
    ",", "78.64732584866613`", ",", "0.7451975063824361`", ",", 
   "0.7435970556800757`", ",", "0.3009885644750722`", ",", "Indeterminate", 
   ",", "1.5663730263154092`", ",", "0.617946816256622`", ",", 
   "0.263776536779606`", ",", "36.83339280799862`"}], "}"}]], "Print",
 CellChangeTimes->{3.593528075384014*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0018000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0018000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593528075384014*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593528075384014*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.9691807651212829128923054626290183368`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.9691807651212829128923054626290183368`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593528271211158*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593528279057972*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9689857651212829`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9689857651212829],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935283842489567`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935284362750473`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "27.583163087179248`", ",", "65.77714694536103`", ",", "79.60208704975321`",
    ",", "83.68471636778264`", ",", "91.12922101026126`", ",", 
   "94.07130393196854`", ",", "96.19557304512881`", ",", "98.92280938732054`",
    ",", "79.60208704975321`", ",", "0.7544418492950027`", ",", 
   "0.7529004048466615`", ",", "0.3009885644750722`", ",", "Indeterminate", 
   ",", "1.5884678178143794`", ",", "0.617946816256622`", ",", 
   "0.27225016810969904`", ",", "28.40119035682`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935284362750473`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0019000000000000002`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0019000000000000002`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935284366494484`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935284366494484`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96896693706165348247452283321495159329`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96896693706165348247452283321495159329`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935286313533907`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593528639153404*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9687469370616535`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9687469370616535],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935287642032237`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935288161513147`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.561501371874137`", ",", "66.94169988737416`", ",", "80.48485916001593`",
    ",", "84.42549198694036`", ",", "91.54804289771607`", ",", 
   "94.34914677824541`", ",", "96.37327982921929`", ",", "98.9747887295606`", 
   ",", "80.48485916001593`", ",", "0.762952407562629`", ",", 
   "0.7614652705630154`", ",", "0.3009885644750722`", ",", "Indeterminate", 
   ",", "1.609636743289147`", ",", "0.617946816256622`", ",", 
   "0.28082580979998795`", ",", "22.874777978741086`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935288161513147`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.002`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.002],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935288161513147`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935288161513147`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96875025993973728744069952137158799122`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96875025993973728744069952137158799122`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935290113700576`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935290192480717`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9685052599397374`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9685052599397374],
  Editable->False]], "Print",
 CellChangeTimes->{3.593529141770687*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593529196994784*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "29.490081458167296`", ",", "68.02548930314637`", ",", "81.30113415061717`",
    ",", "85.10395098193196`", ",", "91.92479140039663`", ",", 
   "94.59814881953913`", ",", "96.53221228348133`", ",", "99.02141880606166`",
    ",", "81.30113415061717`", ",", "0.7707817988448404`", ",", 
   "0.7693446530863207`", ",", "0.3009885644750722`", ",", "Indeterminate", 
   ",", "1.6297867094191754`", ",", "0.617946816256622`", ",", 
   "0.2892983252144842`", ",", "19.625139458178587`"}], "}"}]], "Print",
 CellChangeTimes->{3.593529196994784*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0021],
  Editable->False]], "Print",
 CellChangeTimes->{3.593529197025984*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593529197025984*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96853068954110886385121332206339900401`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96853068954110886385121332206339900401`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593529391761126*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935293995767393`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9682556895411089`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9682556895411089],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935295243613586`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935295771518517`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "30.364725891776146`", ",", "69.03313164265474`", ",", "82.0528985050437`", 
   ",", "85.72110141743394`", ",", "92.25811429917887`", ",", 
   "94.81681913006722`", ",", "96.67135685543836`", ",", "99.06216228028354`",
    ",", "82.0528985050437`", ",", "0.7779241636101563`", ",", 
   "0.7765325922746199`", ",", "0.3009885644750722`", ",", "Indeterminate", 
   ",", "1.6484260748977984`", ",", "0.617946816256622`", ",", 
   "0.29775219667586905`", ",", "18.231681700367258`"}], "}"}]], "Print",
 CellChangeTimes->{3.593529577167452*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0021999999999999997`"}],
  SequenceForm[
  "Diff between approximated points of \[Beta]:  ", 0.0021999999999999997`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935295771830516`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935295771986513`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96830633444755031052549357911035086917`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96830633444755031052549357911035086917`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593529772963395*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593529780763409*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9680063344475502`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9680063344475502],
  Editable->False]], "Print",
 CellChangeTimes->{3.593529946778901*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593529998524192*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "31.23590969183151`", ",", "69.9991587919613`", ",", "82.76856375518183`", 
   ",", "86.30523530872345`", ",", "92.56851189830944`", ",", 
   "95.02046128323698`", ",", "96.80085219193948`", ",", "99.09994415758764`",
    ",", "82.76856375518183`", ",", "0.7847251181324596`", ",", 
   "0.7833770471737339`", ",", "0.3009885644750722`", ",", "Indeterminate", 
   ",", "1.6667513929501814`", ",", "0.617946816256622`", ",", 
   "0.30602864113611083`", ",", "18.25305786585923`"}], "}"}]], "Print",
 CellChangeTimes->{3.593529998524192*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.5935300143426194`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935302754558783`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "30.509580886514442`", ",", "69.20482871483257`", ",", "82.12990092218917`",
    ",", "85.79184721957071`", ",", "92.31965808264583`", ",", 
   "94.86102851334809`", ",", "96.7012955169942`", ",", "99.0721246446025`", 
   ",", "82.12990092218917`", ",", "0.7791984281018602`", ",", 
   "0.7778148648002986`", ",", "0.30625248087335477`", ",", "Indeterminate", 
   ",", "1.6532508310520846`", ",", "0.6255401589532461`", ",", 
   "0.2975723488504006`", ",", "17.91887638642376`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935302754558783`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0021`", ",", "0.9682556895411089`", ",", "0.9609056895411089`", ",", 
     "0.9756056895411088`", ",", "19.765840607648588`", ",", 
     "6.611072256333286`", ",", "0.9963760863665171`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0021, 
   0.9682556895411089, 0.9609056895411089, 0.9756056895411088, 
   19.765840607648588`, 6.611072256333286, 0.9963760863665171}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935302755338783`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", 
   "68.98566116666666666666666666666666666667`10.767425079194542"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   68.98566116666666666666666666666666666667`10.767425079194542],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935302755338783`*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593530275596278*^9}],

Cell[BoxData["\<\"Country: Slovakia\"\>"], "Print",
 CellChangeTimes->{3.5935302756118784`*^9}],

Cell[BoxData["\<\"Obtain (common) initial distribution of Pt and kRatt for \
later simulations \"\>"], "Print",
 CellChangeTimes->{3.593530275861479*^9}],

Cell[BoxData["\<\" Searching for good guess of \[Beta]middle using Reiter's \
method... \"\>"], "Print",
 CellChangeTimes->{3.5935302793558846`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm[" Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935302793558846`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979755615836127358164731049328111112`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979755615836127358164731049328111112`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593530598392045*^9}],

Cell[BoxData["\<\" Searching \[Beta]middle and obtaining (common) initial \
distribution of Pt and kRatt...\"\>"], "Print",
 CellChangeTimes->{3.593530606301259*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to search \[Beta]middle and obtain (common) \
initial distribution of Pt and kRatt (minutes):  \"\>", "\[InvisibleSpace]", 
   "1.37384241333333333333333333333333333333`9.066603168024493"}],
  SequenceForm[
  " Time spent to search \[Beta]middle and obtain (common) initial \
distribution of Pt and kRatt (minutes):  ", 
   1.37384241333333333333333333333333333333`9.066603168024493],
  Editable->False]], "Print",
 CellChangeTimes->{3.593530688731804*^9}],

Cell[BoxData["\<\" Obtained (common) initial distribution of Pt and kRatt. \"\
\>"], "Print",
 CellChangeTimes->{3.593530688747404*^9}],

Cell[BoxData["\<\"Start searching for \[Beta]middle using the simulation \
method \"\>"], "Print",
 CellChangeTimes->{3.593530688747404*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0014`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0014],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935306919610095`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935306919766097`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96998832891268086372112522429995267823`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96998832891268086372112522429995267823`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935308420176735`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593530850004887*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9698683289126807`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9698683289126807],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935309756163077`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935310262071967`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.32970116516937`", ",", "60.83204993627673`", ",", "75.52027092051135`", 
   ",", "80.13172849235421`", ",", "89.00111027299567`", ",", 
   "92.64125101524715`", ",", "95.28154071840969`", ",", "98.6634555085124`", 
   ",", "75.52027092051135`", ",", "0.7164908490606811`", ",", 
   "0.7146612971413352`", ",", "0.30395635749386274`", ",", "Indeterminate", 
   ",", "1.489985191522777`", ",", "0.6235701711497865`", ",", 
   "0.23889065053892355`", ",", "17.932384909885354`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935310262071967`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0015`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0015],
  Editable->False]], "Print",
 CellChangeTimes->{3.593531026550397*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.593531026550397*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96979384655108107302619721701072574391`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96979384655108107302619721701072574391`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935312206771383`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935312286643524`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.969653846551081`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.969653846551081],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935313562881765`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935314068946652`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.635862750202392`", ",", "62.326050670623644`", ",", 
   "76.73141950077756`", ",", "81.18657916471308`", ",", "89.64167184797775`",
    ",", "93.07284350463767`", ",", "95.55834574710508`", ",", 
   "98.74300629204855`", ",", "76.73141950077756`", ",", 
   "0.7283628411742689`", ",", "0.7266053111022072`", ",", 
   "0.30395635749386274`", ",", "Indeterminate", ",", "1.5146250171340252`", 
   ",", "0.6235701711497865`", ",", "0.24745754864189684`", ",", 
   "24.876622030271964`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935314069414654`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0013`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0013],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935314069414654`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935314069570656`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.593531463179564*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.593531463507165*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.593531471057578*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.9700599999999999`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.9700599999999999],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935315533633223`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.5935316052646136`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "23.868465605198413`", ",", "59.128714121812806`", ",", 
   "74.11902132516943`", ",", "78.90429197316391`", ",", "88.23935095257332`",
    ",", "92.12088554384475`", ",", "94.9454419988596`", ",", 
   "98.56631888678781`", ",", "74.11902132516943`", ",", 
   "0.7026842727445771`", ",", "0.7007710789493249`", ",", 
   "0.30395635749386274`", ",", "Indeterminate", ",", "1.4625870750547874`", 
   ",", "0.6235701711497865`", ",", "0.230888562777107`", ",", 
   "14.96348717714674`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935316052646136`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Diff between approximated points of \[Beta]:  \"\>", 
   "\[InvisibleSpace]", "0.0012`"}],
  SequenceForm["Diff between approximated points of \[Beta]:  ", 0.0012],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935316056078143`*^9}],

Cell[BoxData["\<\" Searching for initial guess of \[Beta]middle using \
Reiter's method... \"\>"], "Print",
 CellChangeTimes->{3.5935316056078143`*^9}]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"FindMinimum", "::", "reged"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"The point \[NoBreak]\\!\\({0.97`10.}\\)\[NoBreak] is at \
the edge of the search region \[NoBreak]\\!\\({0.95`10., 0.97`10.}\\)\
\[NoBreak] in coordinate \[NoBreak]\\!\\(1\\)\[NoBreak] and the computed \
search direction points outside the region. \\!\\(\\*ButtonBox[\\\"\
\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/FindMinimum/reged\\\", ButtonNote -> \
\\\"FindMinimum::reged\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5935316374942703`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" \[Beta]middle:  \"\>", "\[InvisibleSpace]", 
   "0.96999999999999997335464740899624302983`10."}],
  SequenceForm[
  " \[Beta]middle:  ", 0.96999999999999997335464740899624302983`10.],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935316374942703`*^9}],

Cell[BoxData["\<\" Searching for \[Beta]middle... \"\>"], "Print",
 CellChangeTimes->{3.5935316454034843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Solution of \[Beta]middle: \"\>", "\[InvisibleSpace]", 
   "0.97026`"}],
  SequenceForm[" Solution of \[Beta]middle: ", 0.97026],
  Editable->False]], "Print",
 CellChangeTimes->{3.593531769439302*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593531822463795*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.4938843511245`", ",", "57.428927862372824`", ",", "72.69060104962104`", 
   ",", "77.64207984452584`", ",", "87.44533811341107`", ",", 
   "91.57442906502244`", ",", "94.59193758494926`", ",", "98.4639411076563`", 
   ",", "72.69060104962104`", ",", "0.6886520021453786`", ",", 
   "0.6866540234233505`", ",", "0.30395635749386274`", ",", "Indeterminate", 
   ",", "1.4352414008440773`", ",", "0.6235701711497865`", ",", 
   "0.2226513187460183`", ",", "17.594948104634156`"}], "}"}]], "Print",
 CellChangeTimes->{3.593531822479395*^9}],

Cell[BoxData["\<\"Solution: \"\>"], "Print",
 CellChangeTimes->{3.593531838391423*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 10%, 20%, 25%, 40%, 50%, \
60%, 80%, 5th quintile, (* Frac between 0.5 and 1.5 * mean k (%), *) Gini \
coeff of wealth at beg-period, Gini coeff of wealth at end-period, Gini coeff \
of perm inc, std log wealth at beg-period, std log wealth at end-period, std \
log perm inc, Agg Annual MPC, Sum of Dev Squared:  \"\>"], "Print",
 CellChangeTimes->{3.593532095854275*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.91452383772687`", ",", "58.03343484743711`", ",", "73.53195696428219`", 
   ",", "78.46401967525172`", ",", "88.02708672935917`", ",", 
   "91.97396076663408`", ",", "94.84756181363502`", ",", "98.53307536244121`",
    ",", "73.53195696428219`", ",", "0.6950308183326998`", ",", 
   "0.6931332804867175`", ",", "0.3060139860820079`", ",", "Indeterminate", 
   ",", "1.467319362929334`", ",", "0.6256334409610663`", ",", 
   "0.2302625959156438`", ",", "14.557607984854265`"}], "}"}]], "Print",
 CellChangeTimes->{3.593532095854275*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Difference beteen approximated points of \[Beta] (\
\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{
    "0.0013`", ",", "0.9700599999999999`", ",", "0.9655099999999999`", ",", 
     "0.97461`", ",", "19.600013821210133`", ",", "6.575521477521069`", ",", 
     "1.4656211437814926`"}], "}"}]}],
  SequenceForm[
  " Difference beteen approximated points of \[Beta] \
(\[EmptyDownTriangle]/3.5), \[Beta]middle (\[Beta]grave in the text), \
\[Beta]middle-\[EmptyDownTriangle],  \[Beta]middle+\[EmptyDownTriangle], Agg \
K (Mean of k (level)), K/Y ratio, Average deviation of K (%): ", {0.0013, 
   0.9700599999999999, 0.9655099999999999, 0.97461, 19.600013821210133`, 
   6.575521477521069, 1.4656211437814926`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532095854275*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with seven types \
(minutes):  \"\>", "\[InvisibleSpace]", "30.337373285`10.410644223571603"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with seven types (minutes):  ", 
   30.337373285`10.410644223571603],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532095854275*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model (using liq fin and \
retirement assets) across Euroepan economies (minutes):  \"\>", 
   "\[InvisibleSpace]", "1072.299863395`11.958982498877536"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model (using liq fin and retirement \
assets) across Euroepan economies (minutes):  ", 
   1072.299863395`11.958982498877536],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532096275476*^9}],

Cell[BoxData["\<\"============================================================\
===============\"\>"], "Print",
 CellChangeTimes->{3.593532096275476*^9}],

Cell[BoxData["\<\"Models with aggregate permanent shocks\"\>"], "Print",
 CellChangeTimes->{3.593532096275476*^9}],

Cell[BoxData["\<\"============================================================\
=\"\>"], "Print",
 CellChangeTimes->{3.593532097305078*^9}],

Cell[BoxData["\<\"Run \[Beta]-Dist model with time pref factors approximated \
by seven points\"\>"], "Print",
 CellChangeTimes->{3.5935320973206778`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593532097414278*^9}],

Cell[BoxData["\<\"Country: All countries\"\>"], "Print",
 CellChangeTimes->{3.593532097414278*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593532097429878*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593532098958681*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935320997854824`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935320998634825`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593532099910282*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935320999258823`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.9516017`7.128970207048372"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.9516017`7.128970207048372],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935320999414825`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935320999414825`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5935320999414825`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5935321005186834`*^9}],

Cell[BoxData["0.9691946233716188`"], "Print",
 CellChangeTimes->{3.5935321005654836`*^9}],

Cell[BoxData["0.0017000000000000001`"], "Print",
 CellChangeTimes->{3.5935321005654836`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935321005654836`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935321005654836`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "4.06848714666666666666666666666666666667`9.538099196508291"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.06848714666666666666666666666666666667`9.538099196508291],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935323446279125`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10891004454878722`", ",", "0.963038030172576`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10891004454878722`, 0.963038030172576}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935323857651844`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.946`"}],
  SequenceForm[" R^2: ", 0.946],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935323857807846`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68562120333333333333333333333333333333`8.764750488210518"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68562120333333333333333333333333333333`8.764750488210518],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935323857807846`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532385796385*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593532385796385*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.35954238833333333333333333333333333333`9.062059000949947"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.35954238833333333333333333333333333333`9.062059000949947],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532467368928*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1303900307174414`", ",", "0.955752088999673`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1303900307174414, 0.955752088999673}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532508911801*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.893`"}],
  SequenceForm[" R^2: ", 0.893],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935325092394013`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69290121666666666666666666666666666667`8.769337572128281"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.69290121666666666666666666666666666667`8.769337572128281],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935325092394013`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935325092394013`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593532509255001*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.04806184166666666666666666666666666667`8.949053157487572"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.04806184166666666666666666666666666667`8.949053157487572],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935325717955112`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13137847586025125`", ",", "0.9554285794989141`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13137847586025125`, 0.9554285794989141}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935326121527824`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.905`"}],
  SequenceForm[" R^2: ", 0.905],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935326121527824`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67262118166666666666666666666666666667`8.756436787875936"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67262118166666666666666666666666666667`8.756436787875936],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935326121527824`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935326121527824`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935326121527824`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05772010166666666666666666666666666667`7.689993335592622"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05772010166666666666666666666666666667`7.689993335592622],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935326156159887`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13516704332478172`", ",", "0.9541419814033995`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13516704332478172`, 0.9541419814033995}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532657626862*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532657642462*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.70018122833333333333333333333333333333`8.773876711484789"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.70018122833333333333333333333333333333`8.773876711484789],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532657642462*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532657642462*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593532657642462*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.43160075833333333333333333333333333333`8.563748447293447"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.43160075833333333333333333333333333333`8.563748447293447],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935326835229073`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14153565159621934`", ",", "0.9519743829808482`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14153565159621934`, 0.9519743829808482}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935327234121776`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.894`"}],
  SequenceForm[" R^2: ", 0.894],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935327237553782`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66482116833333333333333333333333333333`8.751371087427298"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66482116833333333333333333333333333333`8.751371087427298],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935327237553782`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935327237553782`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935327237553782`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.60970107`8.713783205406958"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.60970107`8.713783205406958],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935327599942417`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935327599942417`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593532934995349*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935331074848523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14041308338524114`", ",", "0.9523630223975184`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14041308338524114`, 0.9523630223975184}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593533142647314*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.593533142647314*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.37755120166666666666666666666666666667`9.73332020221044"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.37755120166666666666666666666666666667`9.73332020221044],
  Editable->False]], "Print",
 CellChangeTimes->{3.593533142647314*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593533142647314*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593533142647314*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.053560095`7.6575075868681335"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.053560095`7.6575075868681335],
  Editable->False]], "Print",
 CellChangeTimes->{3.59353314586092*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.59353314586092*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935339578423457`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.59353476899697*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593534932610058*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057005525210562326`", ",", "0.058189578361340207`", ",", 
   "1.0207708489028795`", ",", "0.05771786909374735`", ",", 
   "1.012496049822431`", ",", "0.9964982430556182`", ",", 
   "0.9960648718363669`", ",", "0.9961515852422662`", ",", 
   "0.9992496209145199`", ",", "0.8990766895931901`", ",", 
   "0.8768689523127219`", ",", "0.9393326248486913`", ",", 
   "6.594738305000791`"}], "}"}]], "Print",
 CellChangeTimes->{3.593534932610058*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935349329688587`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.16671890356475166`", ",", "0.8173892408617802`", ",", 
   "0.12149103386570498`", ",", "0.0804897219240536`", ",", 
   "0.5695979876506864`", ",", "0.27840166087649937`", ",", 
   "0.9163369825951024`", ",", "0.939141668935143`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935349329688587`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935349330156584`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.147061526600723`", ",", "49.22359500256698`", ",", "63.08154255629685`",
    ",", "77.5723376391917`", ",", "81.97578894515802`", ",", 
   "90.17227050217083`", ",", "93.43612600901221`", ",", "95.79260563159693`",
    ",", "98.80959188363944`", ",", "0.7348528423164897`", ",", 
   "19.8165125`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935349330312586`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.25813154333631616`"}],
  SequenceForm[" MPC mean:", 0.25813154333631616`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935349330780587`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593534933124859*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.21584137081732113`", ",", "0.21960878495725844`", ",", 
   "0.22850095932536518`", ",", "0.24391428914440771`", ",", 
   "0.25377721036321965`", ",", "0.26462727766637795`", ",", 
   "0.3001839950666275`", ",", "0.26701524576861446`", ",", 
   "0.26673740902207144`", ",", "0.28357457314603685`"}], "}"}]], "Print",
 CellChangeTimes->{3.593534933124859*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935349331872587`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12285782329976158`", ",", "0.1250850764928887`", ",", 
   "0.12734084812675173`", ",", "0.134054944847654`", ",", 
   "0.15164808870687085`", ",", "0.3986617713669234`", ",", 
   "0.5121626579701255`", ",", "0.29790945681832337`", ",", 
   "0.36276862763896023`", ",", "0.47501366604366313`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935349331872587`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.24149598987609533`", ",", "0.45420248030325416`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.24149598987609533`, 0.45420248030325416`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593534933202859*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13933755622416916`", ",", "0.9527324352027938`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13933755622416916`, 0.9527324352027938}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593534933202859*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.902`"}],
  SequenceForm[" R^2: ", 0.902],
  Editable->False]], "Print",
 CellChangeTimes->{3.593534933218459*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.77915230333333333333333333333333333333`10.402578579132644"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.77915230333333333333333333333333333333`10.402578579132644],
  Editable->False]], "Print",
 CellChangeTimes->{3.593534933218459*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.21166292166666666666666666666666666667`10.60271554593527"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   47.21166292166666666666666666666666666667`10.60271554593527],
  Editable->False]], "Print",
 CellChangeTimes->{3.593534933234059*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593534933234059*^9}],

Cell[BoxData["\<\"Country: Austria\"\>"], "Print",
 CellChangeTimes->{3.593534933249659*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593534933249659*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.59353493393606*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593534934591261*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593534934606861*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593534934606861*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.593534934606861*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.593534934606861*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593534934606861*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593534934606861*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5935349351840625`*^9}],

Cell[BoxData["0.9696588465510806`"], "Print",
 CellChangeTimes->{3.5935349351840625`*^9}],

Cell[BoxData["0.0015`"], "Print",
 CellChangeTimes->{3.5935349351840625`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935349351840625`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593534935199662*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "4.12464724333333333333333333333333333333`9.544053060053148"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.12464724333333333333333333333333333333`9.544053060053148],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535182662897*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11208350070506334`", ",", "0.9619644164189903`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11208350070506334`, 0.9619644164189903}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535223519369*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.94`"}],
  SequenceForm[" R^2: ", 0.94],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535223519369*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68094119666666666666666666666666666667`8.761775857829106"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68094119666666666666666666666666666667`8.761775857829106],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935352235505686`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935352235505686`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593535223566169*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.32600232833333333333333333333333333333`9.051210534864651"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.32600232833333333333333333333333333333`9.051210534864651],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935353030795083`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1297822096044935`", ",", "0.9559698014418057`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1297822096044935, 0.9559698014418057}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535343187179*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.896`"}],
  SequenceForm[" R^2: ", 0.896],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535343187179*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.668461175`8.753742436219143"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.668461175`8.753742436219143],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535343187179*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535343187179*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593535343202779*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.89310156833333333333333333333333333333`8.879567100226643"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.89310156833333333333333333333333333333`8.879567100226643],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935353967732735`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13665781919480485`", ",", "0.9536354686664273`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13665781919480485`, 0.9536354686664273}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935354349777403`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.898`"}],
  SequenceForm[" R^2: ", 0.898],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935354350089407`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.63674111833333333333333333333333333333`8.732629144080219"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.63674111833333333333333333333333333333`8.732629144080219],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935354350089407`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935354350089407`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935354350089407`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.67314118166666666666666666666666666667`8.75677240900487"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.67314118166666666666666666666666666667`8.75677240900487],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535475366211*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13920399631117095`", ",", "0.9527702090599355`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13920399631117095`, 0.9527702090599355}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935355165482836`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.9`"}],
  SequenceForm[" R^2: ", 0.9],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935355165482836`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68636787333333333333333333333333333333`8.765223195607875"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68636787333333333333333333333333333333`8.765223195607875],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535516563884*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535516563884*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593535516563884*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.33436058666666666666666666666666666667`8.452881326785278"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.33436058666666666666666666666666666667`8.452881326785278],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935355366099186`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13841387765665486`", ",", "0.9530376549592999`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13841387765665486`, 0.9530376549592999}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535576015588*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.902`"}],
  SequenceForm[" R^2: ", 0.902],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535576015588*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.656761155`8.746073706169824"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.656761155`8.746073706169824],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535576015588*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535576015588*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593535576031188*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05278009166666666666666666666666666667`7.651136388368189"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05278009166666666666666666666666666667`7.651136388368189],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935355791823936`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935355792135935`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935357534658995`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535928404607*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13933187081075296`", ",", "0.9527374359885992`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13933187081075296`, 0.9527374359885992}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935359640350695`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.9`"}],
  SequenceForm[" R^2: ", 0.9],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935359640506697`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.41421126666666666666666666666666666667`9.735809508522193"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.41421126666666666666666666666666666667`9.735809508522193],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935359640506697`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935359640506697`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935359640506697`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05174009`7.642493428036228"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05174009`7.642493428036228],
  Editable->False]], "Print",
 CellChangeTimes->{3.593535967139475*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593535967155075*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935367929893255`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593537622988783*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593537791313079*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057008720996871085`", ",", "0.05822268497333822`", ",", 
   "1.0212943555869947`", ",", "0.057813588395993556`", ",", 
   "1.014118320584085`", ",", "0.9964898837532133`", ",", 
   "0.9959281161286196`", ",", "0.9960395049549313`", ",", 
   "0.99927008195209`", ",", "0.8958615915476985`", ",", "0.873546038970784`",
    ",", "0.9386133340939966`", ",", "6.5973695254227716`"}], "}"}]], "Print",
 CellChangeTimes->{3.593537791328679*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593537791359879*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.16558445712416528`", ",", "0.8135488068650175`", ",", 
   "0.1150380248424085`", ",", "0.08921219533532525`", ",", 
   "0.5783171119585333`", ",", "0.2843353249093379`", ",", 
   "0.9135645090315841`", ",", "0.9370300564945868`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935377913754787`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593537791406679*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "22.999996942901703`", ",", "46.45896247951989`", ",", 
   "60.339767448141714`", ",", "75.35384851093038`", ",", 
   "80.05057726218607`", ",", "89.00361540410819`", ",", "92.64469787256769`",
    ",", "95.28364529172836`", ",", "98.66204699325277`", ",", 
   "0.7133346145761026`", ",", "21.63397857142857`"}], "}"}]], "Print",
 CellChangeTimes->{3.593537791437879*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.2412258929954746`"}],
  SequenceForm[" MPC mean:", 0.2412258929954746],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935377914534793`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935377914534793`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.2010824252088924`", ",", "0.20488244875869022`", ",", 
   "0.21405860693275425`", ",", "0.22962272924614402`", ",", 
   "0.23939093336571793`", ",", "0.24397210472537867`", ",", 
   "0.2752835910169724`", ",", "0.2493009192167871`", ",", 
   "0.24728470308600425`", ",", "0.26013678714165356`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935377914534793`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935377914534793`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12346028907543938`", ",", "0.12554583304233846`", ",", 
   "0.12764363428442016`", ",", "0.13307748302738298`", ",", 
   "0.14644124107636292`", ",", "0.3683234597382907`", ",", 
   "0.4803400529254167`", ",", "0.27617518674801655`", ",", 
   "0.3345324972768615`", ",", "0.44323883757258065`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935377914534793`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.22632663067323577`", ",", "0.4194584326932327`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.22632663067323577`, 0.4194584326932327}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593537791469079*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14060120602717296`", ",", "0.9523124775960482`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14060120602717296`, 0.9523124775960482}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593537791484679*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.593537791484679*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.40315340166666666666666666666666666667`10.411584878992754"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.40315340166666666666666666666666666667`10.411584878992754],
  Editable->False]], "Print",
 CellChangeTimes->{3.593537791484679*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.61203029333333333333333333333333333333`10.606382949463113"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   47.61203029333333333333333333333333333333`10.606382949463113],
  Editable->False]], "Print",
 CellChangeTimes->{3.593537791484679*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593537791484679*^9}],

Cell[BoxData["\<\"Country: Belgium\"\>"], "Print",
 CellChangeTimes->{3.5935377915002794`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593537791515879*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5935377921866803`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935377928418818`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400663`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593537792857482*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593537792857482*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992645`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005181`", ",", "0.9488134182354926`"}], "}"}]], "Print",
 CellChangeTimes->{3.593537792857482*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708011`6.97710876393589"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708011`6.97710876393589],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935377928886814`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935377928886814`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5935377928886814`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593537793434683*^9}],

Cell[BoxData["0.9689757651212827`"], "Print",
 CellChangeTimes->{3.593537793434683*^9}],

Cell[BoxData["0.0018000000000000002`"], "Print",
 CellChangeTimes->{3.593537793434683*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593537793434683*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593537793434683*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "4.15740730333333333333333333333333333333`9.547488822860135"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.15740730333333333333333333333333333333`9.547488822860135],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538042879121*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11001257888896582`", ",", "0.9626625814900113`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11001257888896582`, 0.9626625814900113}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935380820819893`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.944`"}],
  SequenceForm[" R^2: ", 0.944],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935380820819893`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65338114666666666666666666666666666667`8.743832846955046"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65338114666666666666666666666666666667`8.743832846955046],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935380820819893`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935380820819893`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935380820819893`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.363182395`9.063220216920735"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.363182395`9.063220216920735],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935381638729334`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1249141352142987`", ",", "0.9576225523158832`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1249141352142987, 0.9576225523158832}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538203450203*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538203450203*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65962115833333333333333333333333333333`8.747960826055362"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65962115833333333333333333333333333333`8.747960826055362],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935382037934036`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935382037934036`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935382037934036`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.794041395`8.828509391891423"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.794041395`8.828509391891423],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935382510926867`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1320993373501133`", ",", "0.9551920133164238`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1320993373501133, 0.9551920133164238}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538289499954*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538289499954*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64012112333333333333333333333333333333`8.734928406915472"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64012112333333333333333333333333333333`8.734928406915472],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538289499954*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538289515554*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593538289515554*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.62712110166666666666666666666666666667`8.726017662582445"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.62712110166666666666666666666666666667`8.726017662582445],
  Editable->False]], "Print",
 CellChangeTimes->{3.59353832712722*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13371030904340794`", ",", "0.9546481262216377`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13371030904340794`, 0.9546481262216377}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538366548489*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.904`"}],
  SequenceForm[" R^2: ", 0.904],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538366548489*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.657021155`8.746245601568186"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.657021155`8.746245601568186],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935383665640893`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935383665640893`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935383665640893`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.20566036166666666666666666666666666667`8.241816843418928"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.20566036166666666666666666666666666667`8.241816843418928],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935383789037113`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13807569192781444`", ",", "0.9531578792190734`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13807569192781444`, 0.9531578792190734}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935384204465837`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935384204465837`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.692381215`8.76901152473194"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.692381215`8.76901152473194],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935384204465837`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935384204465837`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593538420462184*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.50414088666666666666666666666666666667`8.631218169088898"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.50414088666666666666666666666666666667`8.631218169088898],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538450695037*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593538451053838*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935386266789465`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538802070054*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13722801660177114`", ",", "0.9534499649090419`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13722801660177114`, 0.9534499649090419}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935388376849165`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.903`"}],
  SequenceForm[" R^2: ", 0.903],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935388376849165`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.44983132833333333333333333333333333333`9.738214605621147"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.44983132833333333333333333333333333333`9.738214605621147],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935388377005167`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935388377005167`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935388377005167`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05018008833333333333333333333333333333`7.62919766969493"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05018008833333333333333333333333333333`7.62919766969493],
  Editable->False]], "Print",
 CellChangeTimes->{3.593538840695722*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593538840695722*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593539640571527*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935404443473387`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935406065876236`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057081362544727086`", ",", "0.05827208151817229`", ",", 
   "1.0208600306713456`", ",", "0.05787054438067729`", ",", 
   "1.0138255605817368`", ",", "0.9965123279185019`", ",", 
   "0.9960083135042969`", ",", "0.9961222613416185`", ",", 
   "0.9992603494693123`", ",", "0.8978882017153625`", ",", 
   "0.8756717267478218`", ",", "0.9391356517543168`", ",", 
   "6.59560028816941`"}], "}"}]], "Print",
 CellChangeTimes->{3.593540606603224*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593540606993224*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.15989735217750603`", ",", "0.8129298882557424`", ",", 
   "0.12048667151370561`", ",", "0.08387024668018962`", ",", 
   "0.5724310785119884`", ",", "0.2821787345196049`", ",", 
   "0.9133265205381507`", ",", "0.9367917600817484`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935406070088243`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935406070244246`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "26.179971320015767`", ",", "50.56993162745591`", ",", "64.36856738405561`",
    ",", "78.57393112994653`", ",", "82.83379660909635`", ",", 
   "90.67345350681201`", ",", "93.77160960172128`", ",", "96.00598454941192`",
    ",", "98.8696833003666`", ",", "0.7445375407098914`", ",", 
   "19.026785714285715`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935406070244246`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.2665600552322518`"}],
  SequenceForm[" MPC mean:", 0.2665600552322518],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935406070400248`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935406070400248`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.22260843134172814`", ",", "0.2270299336630035`", ",", 
   "0.23562734446340083`", ",", "0.25086401634766486`", ",", 
   "0.2608449748093157`", ",", "0.27507045563934285`", ",", 
   "0.3122947849154406`", ",", "0.2758712594461268`", ",", 
   "0.27655260877365473`", ",", "0.29511427270278456`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935406070400248`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935406070556245`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12251046003299981`", ",", "0.12482513320364019`", ",", 
   "0.12714226751984903`", ",", "0.13465353135490177`", ",", 
   "0.15456943636437226`", ",", "0.41321593477044993`", ",", 
   "0.5272920928949525`", ",", "0.3087121734688276`", ",", 
   "0.37638883227161235`", ",", "0.49017555569995763`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935406070556245`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.24904837687129078`", ",", "0.4713687527798319`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.24904837687129078`, 0.4713687527798319}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935406070556245`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13816727554605732`", ",", "0.9531342787646571`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13816727554605732`, 0.9531342787646571}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540607071224*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540607071224*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.43179169333333333333333333333333333333`10.397482949283742"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.43179169333333333333333333333333333333`10.397482949283742],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540607071224*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "46.89576236833333333333333333333333333333`10.599799848645052"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   46.89576236833333333333333333333333333333`10.599799848645052],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540607071224*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593540607071224*^9}],

Cell[BoxData["\<\"Country: Cyprus\"\>"], "Print",
 CellChangeTimes->{3.593540607071224*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935406070868244`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593540607788826*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593540608444027*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400663`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593540608444027*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593540608444027*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992645`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005181`", ",", "0.9488134182354926`"}], "}"}]], "Print",
 CellChangeTimes->{3.593540608459627*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540608459627*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593540608459627*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593540608459627*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593540609068028*^9}],

Cell[BoxData["0.9694498721304704`"], "Print",
 CellChangeTimes->{3.593540609068028*^9}],

Cell[BoxData["0.0016`"], "Print",
 CellChangeTimes->{3.593540609068028*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935406090836277`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935406090836277`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "4.1555873`9.547298658555919"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.1555873`9.547298658555919],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540858403266*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1132314565500035`", ",", "0.9615703758340373`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1132314565500035, 0.9615703758340373}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540897777735*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.939`"}],
  SequenceForm[" R^2: ", 0.939],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540897793335*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65624115166666666666666666666666666667`8.745729708921838"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65624115166666666666666666666666666667`8.745729708921838],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540897793335*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540897793335*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593540897793335*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.28518225833333333333333333333333333333`9.037630969793902"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.28518225833333333333333333333333333333`9.037630969793902],
  Editable->False]], "Print",
 CellChangeTimes->{3.593540974888671*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12720142845679974`", ",", "0.9568497909174566`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12720142845679974`, 0.9568497909174566}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935410142319393`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.9`"}],
  SequenceForm[" R^2: ", 0.9],
  Editable->False]], "Print",
 CellChangeTimes->{3.59354101427874*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65572115166666666666666666666666666667`8.745385441191216"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65572115166666666666666666666666666667`8.745385441191216],
  Editable->False]], "Print",
 CellChangeTimes->{3.59354101427874*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.59354101427874*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59354101429434*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.69550122166666666666666666666666666667`8.770964145394345"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.69550122166666666666666666666666666667`8.770964145394345],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935410559620132`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13487830397104994`", ",", "0.9542591130518873`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13487830397104994`, 0.9542591130518873}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541095320882*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.898`"}],
  SequenceForm[" R^2: ", 0.898],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541095320882*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65598115166666666666666666666666666667`8.745557609169403"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65598115166666666666666666666666666667`8.745557609169403],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541095320882*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541095320882*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593541095320882*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.651561145`8.742621426038676"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.651561145`8.742621426038676],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541134414551*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13770405060609903`", ",", "0.9533055465051958`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13770405060609903`, 0.9533055465051958}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935411752398224`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935411752398224`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.680421195`8.761444081563244"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.680421195`8.761444081563244],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935411752398224`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935411752554226`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935411752554226`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.321880565`8.43636100319243"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.321880565`8.43636100319243],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541194552656*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1386181901184162`", ",", "0.9529991579484853`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1386181901184162, 0.9529991579484853}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541235206328*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541235206328*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67756119`8.759614770314624"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.67756119`8.759614770314624],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935412352219276`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935412352219276`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935412352219276`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.11232019666666666666666666666666666667`7.9791241034284734"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.11232019666666666666666666666666666667`7.9791241034284734],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935412419455395`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935412419455395`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541416353846*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935415932425566`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1404091572967406`", ",", "0.9523864139930338`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1404091572967406, 0.9523864139930338}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593541629809021*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935416298246207`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.464391355`9.739193889016205"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.464391355`9.739193889016205],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935416298558207`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935416298558207`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593541629871421*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.24596043166666666666666666666666666667`8.319531494786514"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.24596043166666666666666666666666666667`8.319531494786514],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935416445666466`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935416445666466`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593542470104497*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543302131958*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593543470066253*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057077771337142584`", ",", "0.058266736823003304`", ",", 
   "1.02083062211448`", ",", "0.057814428703298545`", ",", 
   "1.0129062040948433`", ",", "0.9965262521653588`", ",", 
   "0.9960428394890664`", ",", "0.9961362172461508`", ",", 
   "0.9992632270807643`", ",", "0.898672289075043`", ",", 
   "0.8766368444529826`", ",", "0.9396852562526431`", ",", 
   "6.597829892708499`"}], "}"}]], "Print",
 CellChangeTimes->{3.593543470066253*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935434704250536`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1609694184839418`", ",", "0.8168662436848225`", ",", 
   "0.11711502520415698`", ",", "0.0836006958933896`", ",", 
   "0.5729371801924297`", ",", "0.28183951525728024`", ",", 
   "0.9148724181304664`", ",", "0.9378043541890224`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935434704250536`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935434704718537`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "24.171178756995474`", ",", "47.949443476811545`", ",", 
   "61.84372008513234`", ",", "76.57276448709544`", ",", "81.10642531727358`",
    ",", "89.64282641783635`", ",", "93.07603536758252`", ",", 
   "95.56024910553717`", ",", "98.74227575940162`", ",", 
   "0.7250933139902704`", ",", "20.6342375`"}], "}"}]], "Print",
 CellChangeTimes->{3.593543470487454*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.2499455743050497`"}],
  SequenceForm[" MPC mean:", 0.2499455743050497],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543470487454*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593543470487454*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.20760437419938937`", ",", "0.21208345587044508`", ",", 
   "0.2213449527743825`", ",", "0.2368244697867291`", ",", 
   "0.24672486033713248`", ",", "0.25475726664038756`", ",", 
   "0.2881606804411003`", ",", "0.2584753249703271`", ",", 
   "0.25736668904077076`", ",", "0.27219948627405643`"}], "}"}]], "Print",
 CellChangeTimes->{3.593543470487454*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593543470518654*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12304917497394197`", ",", "0.12523684959568426`", ",", 
   "0.12741399133485132`", ",", "0.13342379955315664`", ",", 
   "0.14891123838949527`", ",", "0.38429167878608683`", ",", 
   "0.4973358291773131`", ",", "0.287422615411253`", ",", 
   "0.3493384380995731`", ",", "0.46010297096315955`"}], "}"}]], "Print",
 CellChangeTimes->{3.593543470518654*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.23413099381377211`", ",", "0.43767380415975066`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.23413099381377211`, 0.43767380415975066`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543470518654*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1391785083593255`", ",", "0.9528032958515957`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1391785083593255, 0.9528032958515957}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543470534254*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.902`"}],
  SequenceForm[" R^2: ", 0.902],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543470534254*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.42499343833333333333333333333333333333`10.411896741454763"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.42499343833333333333333333333333333333`10.411896741454763],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543470534254*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.69370376833333333333333333333333333333`10.607127298133056"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   47.69370376833333333333333333333333333333`10.607127298133056],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543470534254*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935434705498543`*^9}],

Cell[BoxData["\<\"Country: Germany\"\>"], "Print",
 CellChangeTimes->{3.5935434705498543`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935434705498543`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5935434711738553`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935434718290563`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935434718290563`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935434718290563`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935434718290563`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6552011`6.966889615710615"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6552011`6.966889615710615],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935434718290563`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593543471844656*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593543471844656*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593543472421857*^9}],

Cell[BoxData["0.9689106953461902`"], "Print",
 CellChangeTimes->{3.593543472421857*^9}],

Cell[BoxData["0.0017000000000000001`"], "Print",
 CellChangeTimes->{3.593543472421857*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543472421857*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593543472421857*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "3.96500696333333333333333333333333333333`9.52691020257659"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   3.96500696333333333333333333333333333333`9.52691020257659],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543710322275*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11201631496949621`", ",", "0.9619986477532123`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11201631496949621`, 0.9619986477532123}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935437512099466`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.941`"}],
  SequenceForm[" R^2: ", 0.941],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543751225547*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68146119666666666666666666666666666667`8.762107379768088"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68146119666666666666666666666666666667`8.762107379768088],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543751225547*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543751225547*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593543751225547*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.272442235`9.033304324025861"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.272442235`9.033304324025861],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543827556481*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13060604141040213`", ",", "0.9557111604850222`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13060604141040213`, 0.9557111604850222}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543866197749*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.895`"}],
  SequenceForm[" R^2: ", 0.895],
  Editable->False]], "Print",
 CellChangeTimes->{3.59354386654095*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64402113166666666666666666666666666667`8.73756636591032"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64402113166666666666666666666666666667`8.73756636591032],
  Editable->False]], "Print",
 CellChangeTimes->{3.59354386654095*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.59354386654095*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935438665565495`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.85332149833333333333333333333333333333`8.859778935474806"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.85332149833333333333333333333333333333`8.859778935474806],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935439173970385`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13283093567360532`", ",", "0.9549722536984635`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13283093567360532`, 0.9549722536984635}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935439570835085`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.904`"}],
  SequenceForm[" R^2: ", 0.904],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935439570991087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66144116166666666666666666666666666667`8.749157465900158"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66144116166666666666666666666666666667`8.749157465900158],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935439570991087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935439570991087`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935439570991087`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.19396034166666666666666666666666666667`8.216379188681891"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.19396034166666666666666666666666666667`8.216379188681891],
  Editable->False]], "Print",
 CellChangeTimes->{3.593543968721129*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1393715156055281`", ",", "0.9527516349258509`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1393715156055281, 0.9527516349258509}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935440089223995`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.897`"}],
  SequenceForm[" R^2: ", 0.897],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935440089379997`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67002117666666666666666666666666666667`8.754754777430097"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67002117666666666666666666666666666667`8.754754777430097],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935440089379997`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935440089379997`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935440089379997`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.57382100666666666666666666666666666667`8.687442691241024"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.57382100666666666666666666666666666667`8.687442691241024],
  Editable->False]], "Print",
 CellChangeTimes->{3.59354404335166*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14297722560851528`", ",", "0.9515213108446776`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14297722560851528`, 0.9515213108446776}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935440840989313`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.896`"}],
  SequenceForm[" R^2: ", 0.896],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935440840989313`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67938119333333333333333333333333333333`8.760779768752874"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67938119333333333333333333333333333333`8.760779768752874],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935440841145315`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935440841145315`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935440841145315`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.40950072`8.54092091790609"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.40950072`8.54092091790609],
  Editable->False]], "Print",
 CellChangeTimes->{3.593544108684575*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593544108700175*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593544283436082*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935444628675966`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1410801054077083`", ",", "0.9521643277387426`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1410801054077083, 0.9521643277387426}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935444984512596`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935444984512596`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.49611141`9.741319712775923"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.49611141`9.741319712775923],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935444984512596`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935444984512596`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935444984668593`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05122009`7.6381065854575505"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05122009`7.6381065854575505],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935445015244646`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935445015244646`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593545329339919*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935461597293773`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935463275076723`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.056883041277520634`", ",", "0.05812174925378082`", ",", 
   "1.0217764020425137`", ",", "0.05765227819369256`", ",", 
   "1.0135231327104854`", ",", "0.9964158694388529`", ",", 
   "0.9958763333662861`", ",", "0.9960091201371396`", ",", 
   "0.9992526400197951`", ",", "0.8944046351886956`", ",", 
   "0.8714940060634067`", ",", "0.9363449642987376`", ",", 
   "6.60153762013183`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935463275076723`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935463278352723`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.16123414119223775`", ",", "0.8035430553861918`", ",", 
   "0.12208976396759734`", ",", "0.08026879768987565`", ",", 
   "0.5753998920936019`", ",", "0.2780955046581807`", ",", 
   "0.9100250126312094`", ",", "0.9351978516206442`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935463278508725`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935463278664722`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.870604362006087`", ",", "43.29067330265804`", ",", "56.87284956904144`",
    ",", "72.02744067919457`", ",", "76.96861129376884`", ",", 
   "86.94365517417225`", ",", "91.27751937376316`", ",", "94.49342249970114`",
    ",", "98.56337615806669`", ",", "0.682416990852708`", ",", 
   "25.571114285714287`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935463278664722`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.2608699352178745`"}],
  SequenceForm[" MPC mean:", 0.2608699352178745],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935463278820724`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935463278820724`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.19867702740884563`", ",", "0.20423650033665042`", ",", 
   "0.21078667586170075`", ",", "0.2290461848823`", ",", 
   "0.24219625718104232`", ",", "0.288230694352697`", ",", 
   "0.2853338493622555`", ",", "0.27599442015922937`", ",", 
   "0.28585876659493203`", ",", "0.27732232100709553`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935463278976727`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935463278976727`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1238603439489725`", ",", "0.12644330177008045`", ",", 
   "0.1292716084653791`", ",", "0.13807450291317835`", ",", 
   "0.1524241333172075`", ",", "0.4036033133311595`", ",", 
   "0.536587157399253`", ",", "0.3007367124272965`", ",", 
   "0.3649272908547512`", ",", "0.49220728185889606`"}], "}"}]], "Print",
 CellChangeTimes->{3.593546327913273*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.24875256192878015`", ",", "0.4083503801108892`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.24875256192878015`, 0.4083503801108892}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546327913273*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14295216556743334`", ",", "0.951532828227807`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14295216556743334`, 0.951532828227807}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546327928873*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.898`"}],
  SequenceForm[" R^2: ", 0.898],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546327928873*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.43305345333333333333333333333333333333`10.41201177702484"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.43305345333333333333333333333333333333`10.41201177702484],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546327928873*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.59464359666666666666666666666666666667`10.606224327253214"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   47.59464359666666666666666666666666666667`10.606224327253214],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546327928873*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593546327944473*^9}],

Cell[BoxData["\<\"Country: Spain\"\>"], "Print",
 CellChangeTimes->{3.593546327944473*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935463279600725`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593546328615274*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935463292704754`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935463292704754`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935463292704754`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.593546329286075*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6552012`6.96688968199476"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6552012`6.96688968199476],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546329286075*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593546329286075*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593546329286075*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5935463298476763`*^9}],

Cell[BoxData["0.9642315843500793`"], "Print",
 CellChangeTimes->{3.593546329863276*^9}],

Cell[BoxData["0.0034999999999999966`"], "Print",
 CellChangeTimes->{3.593546329863276*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546329863276*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593546329863276*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "3.97202697666666666666666666666666666667`9.527678437630538"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   3.97202697666666666666666666666666666667`9.527678437630538],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546568169295*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1025457222965025`", ",", "0.965186529355087`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1025457222965025, 0.965186529355087}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935466082457647`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.957`"}],
  SequenceForm[" R^2: ", 0.957],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935466085733657`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66794117166666666666666666666666666667`8.753404462269607"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66794117166666666666666666666666666667`8.753404462269607],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935466085733657`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935466085733657`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935466085733657`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.25424220333333333333333333333333333333`9.027047658246426"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.25424220333333333333333333333333333333`9.027047658246426],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935466835002975`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12624791834118865`", ",", "0.9571343940588538`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12624791834118865`, 0.9571343940588538}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935467247623696`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.893`"}],
  SequenceForm[" R^2: ", 0.893],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935467247623696`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68770120833333333333333333333333333333`8.76606603564045"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68770120833333333333333333333333333333`8.76606603564045],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935467247623696`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935467247779694`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935467247779694`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.981761725`8.920672344888308"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.981761725`8.920672344888308],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546783668073*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12893621537627553`", ",", "0.9562375761220113`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12893621537627553`, 0.9562375761220113}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546823416943*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.906`"}],
  SequenceForm[" R^2: ", 0.906],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546823416943*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66248116333333333333333333333333333333`8.749839782482299"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66248116333333333333333333333333333333`8.749839782482299],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546823432543*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546823432543*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593546823432543*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.22308039166666666666666666666666666667`8.277127646535256"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.22308039166666666666666666666666666667`8.277127646535256],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935468368017664`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13269295953782456`", ",", "0.954960217188165`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13269295953782456`, 0.954960217188165}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546878157439*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.904`"}],
  SequenceForm[" R^2: ", 0.904],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546878157439*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68926121`8.767050086325401"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.68926121`8.767050086325401],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546878157439*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546878157439*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935468781730394`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.37206065333333333333333333333333333333`8.499279992561965"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.37206065333333333333333333333333333333`8.499279992561965],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546900481078*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13275073020965356`", ",", "0.9549533369767986`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13275073020965356`, 0.9549533369767986}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935469428039527`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.905`"}],
  SequenceForm[" R^2: ", 0.905],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935469428039527`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.70538124`8.777090153401339"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.70538124`8.777090153401339],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935469428039527`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935469428039527`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593546942819553*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05252009166666666666666666666666666667`7.648991723635601"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05252009166666666666666666666666666667`7.648991723635601],
  Editable->False]], "Print",
 CellChangeTimes->{3.593546945955158*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935469459707584`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593547121736267*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593547298827778*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13329921569460157`", ",", "0.9547703258972715`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13329921569460157`, 0.9547703258972715}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935473355502424`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.905`"}],
  SequenceForm[" R^2: ", 0.905],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935473355658426`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.493511405`9.741145855866362"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.493511405`9.741145855866362],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935473355658426`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935473355658426`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935473355658426`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05096009`7.635896434545847"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05096009`7.635896434545847],
  Editable->False]], "Print",
 CellChangeTimes->{3.593547338623448*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593547338623448*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593548169387307*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593549004675174*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593549173685871*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "35.41267712017513`", ",", "60.22775823972387`", ",", "72.29093704753315`", 
   ",", "83.61601122611`", ",", "86.81520407682926`", ",", 
   "92.75199967815203`", ",", "95.22018266835828`", ",", "97.02859883878989`",
    ",", "99.26957987051033`", ",", "0.7981381418416869`", ",", 
   "15.428233928571428`"}], "}"}]], "Print",
 CellChangeTimes->{3.593549173685871*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13547986050264882`", ",", "0.9540333708051685`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13547986050264882`, 0.9540333708051685}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935491737170715`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.902`"}],
  SequenceForm[" R^2: ", 0.902],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935491737326713`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.58437371833333333333333333333333333333`10.414165840015064"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.58437371833333333333333333333333333333`10.414165840015064],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935491737326713`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "8"}],
  SequenceForm["Iteration ", 8],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935491737326713`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935491737326713`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.14430025333333333333333333333333333333`8.087933341756612"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.14430025333333333333333333333333333333`8.087933341756612],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935491823438864`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593549182359486*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593549998318919*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935508186619606`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593550984677452*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057032032858655386`", ",", "0.058167807272182945`", ",", 
   "1.0199146752552621`", ",", "0.05765075041825983`", ",", 
   "1.0108485973336745`", ",", "0.996282696778528`", ",", 
   "0.9962653713886505`", ",", "0.9963041124004649`", ",", 
   "0.9992548973790516`", ",", "0.9039618490234668`", ",", 
   "0.8807815113583497`", ",", "0.9329628277658091`", ",", 
   "6.589675036076994`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935509846930523`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935509846930523`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.16267371819195092`", ",", "0.8090850410674225`", ",", 
   "0.10742726475426681`", ",", "0.09082270955502456`", ",", 
   "0.5667379883850322`", ",", "0.28025196184825424`", ",", 
   "0.9078398448280791`", ",", "0.9340308882866912`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935509846930523`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935509846930523`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "34.80714087243633`", ",", "60.097952673446805`", ",", "72.22615587755726`",
    ",", "83.59442232264293`", ",", "86.80129358983486`", ",", 
   "92.74542471631331`", ",", "95.21567226282194`", ",", "97.02571983976644`",
    ",", "99.26853338959768`", ",", "0.7974689366192372`", ",", 
   "15.417430357142857`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935509846930523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.3778368033490579`"}],
  SequenceForm[" MPC mean:", 0.3778368033490579],
  Editable->False]], "Print",
 CellChangeTimes->{3.593550985051852*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593550985051852*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.2862270959183866`", ",", "0.2888804478554544`", ",", 
   "0.29655388709880537`", ",", "0.32052419488845907`", ",", 
   "0.3395886843604362`", ",", "0.43206360349119455`", ",", 
   "0.4505890050489797`", ",", "0.40183337217008175`", ",", 
   "0.4232747013553806`", ",", "0.4370436155402031`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935509850674524`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593550985098653*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12030306461816642`", ",", "0.12391156309970242`", ",", 
   "0.1292826842767849`", ",", "0.15950902224755847`", ",", 
   "0.20570343823392023`", ",", "0.5816358709735329`", ",", 
   "0.7107332566400628`", ",", "0.4462923085294366`", ",", 
   "0.5365591725373839`", ",", "0.6713253149931493`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935509851142526`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.35792870574098323`", ",", "0.6008440042500309`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.35792870574098323`, 0.6008440042500309}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935509851142526`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.133552630764043`", ",", "0.9546844692197155`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.133552630764043, 0.9546844692197155}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935509851298523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.906`"}],
  SequenceForm[" R^2: ", 0.906],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935509851298523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.03915276166666666666666666666666666667`10.406353927675529"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   30.03915276166666666666666666666666666667`10.406353927675529],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935509851298523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "77.59037628`10.818474106230532"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 77.59037628`10.818474106230532],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935509851298523`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935509851454525`*^9}],

Cell[BoxData["\<\"Country: Finland\"\>"], "Print",
 CellChangeTimes->{3.5935509851454525`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935509851610527`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5935509858006535`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593550986455855*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400663`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593550986455855*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593550986471455*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992645`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005181`", ",", "0.9488134182354926`"}], "}"}]], "Print",
 CellChangeTimes->{3.593550986471455*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.593550986471455*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593550986471455*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593550986471455*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5935509870330563`*^9}],

Cell[BoxData["0.9685302599397373`"], "Print",
 CellChangeTimes->{3.5935509870330563`*^9}],

Cell[BoxData["0.002`"], "Print",
 CellChangeTimes->{3.5935509870330563`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935509870330563`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593550987048656*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "4.227867425`9.554787608711557"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.227867425`9.554787608711557],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935512407051015`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1066774239481274`", ",", "0.963802247527384`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1066774239481274, 0.963802247527384}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935512788159685`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.95`"}],
  SequenceForm[" R^2: ", 0.95],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551279143569*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.63518111666666666666666666666666666667`8.731563826662743"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.63518111666666666666666666666666666667`8.731563826662743],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551279143569*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551279159169*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593551279159169*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.46432257166666666666666666666666666667`9.094303005036073"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.46432257166666666666666666666666666667`9.094303005036073],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935513666753225`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12414650432909369`", ",", "0.9578928655603995`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12414650432909369`, 0.9578928655603995}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935514056441913`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935514056441913`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64948114`8.74123279251944"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.64948114`8.74123279251944],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551405659791*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551405659791*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593551405659791*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.867621525`8.866996566192135"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.867621525`8.866996566192135],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935514580446835`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13567413398930683`", ",", "0.953985720980632`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13567413398930683`, 0.953985720980632}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551497481552*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.896`"}],
  SequenceForm[" R^2: ", 0.896],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935514974971523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66274116333333333333333333333333333333`8.750010193989066"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66274116333333333333333333333333333333`8.750010193989066],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935514974971523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935514974971523`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935514974971523`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.82316144666666666666666666666666666667`8.84415126996513"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.82316144666666666666666666666666666667`8.84415126996513],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935515468556395`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13934637244583192`", ",", "0.9527507782208926`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13934637244583192`, 0.9527507782208926}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551589942915*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551589942915*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.71812126`8.784864032433417"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.71812126`8.784864032433417],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551589942915*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551589942915*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593551589942915*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.349440615`8.472039629243973"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.349440615`8.472039629243973],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551610909352*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13745411433765142`", ",", "0.9533894849088613`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13745411433765142`, 0.9533894849088613}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935516506426215`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.904`"}],
  SequenceForm[" R^2: ", 0.904],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935516506738214`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66222116166666666666666666666666666667`8.749669302989002"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66222116166666666666666666666666666667`8.749669302989002],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935516506738214`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935516506738214`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935516506738214`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009333333333333333333333333333333`7.653270527830221"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009333333333333333333333333333333`7.653270527830221],
  Editable->False]], "Print",
 CellChangeTimes->{3.593551653840627*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593551653856227*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935518295281353`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935520085540504`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13776114825253832`", ",", "0.9532888377027766`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13776114825253832`, 0.9532888377027766}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935520446369133`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.902`"}],
  SequenceForm[" R^2: ", 0.902],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935520446369133`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.51353144166666666666666666666666666667`9.7424827620979"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.51353144166666666666666666666666666667`9.7424827620979],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935520446525135`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935520446525135`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935520446525135`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05122009`7.6381065854575505"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05122009`7.6381065854575505],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935520477101192`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935520477101192`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593552860986347*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593553678505783*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935538435540733`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05707309366812417`", ",", "0.05824174083736795`", ",", 
   "1.0204763242034747`", ",", "0.05775455260191181`", ",", 
   "1.011940108551856`", ",", "0.9964783751987608`", ",", 
   "0.9961250082017538`", ",", "0.9961689695338574`", ",", 
   "0.999245469013611`", ",", "0.9008055717996287`", ",", 
   "0.8787352218755712`", ",", "0.9391020598339266`", ",", 
   "6.601407921960246`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935538435540733`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935538435696735`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.16144195214413726`", ",", "0.818523641689291`", ",", 
   "0.11093553704671952`", ",", "0.07836138395589565`", ",", 
   "0.5676230397229595`", ",", "0.2748756645234792`", ",", 
   "0.9171996107172632`", ",", "0.9398454517495306`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935538435696735`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935538435696735`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.435737693367187`", ",", "53.19052207159823`", ",", "66.79247251874571`",
    ",", "80.40264591241339`", ",", "84.36957595556179`", ",", 
   "91.54425256179208`", ",", "94.35258740599383`", ",", "96.37827839796286`",
    ",", "98.97838820325157`", ",", "0.7623940487932489`", ",", 
   "17.617055357142856`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935538435852733`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.28331247435467866`"}],
  SequenceForm[" MPC mean:", 0.28331247435467866`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935538436320734`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935538436320734`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.23764455550196906`", ",", "0.2421005040443146`", ",", 
   "0.2502293907585601`", ",", "0.26512437530453314`", ",", 
   "0.27514750409871147`", ",", "0.295430349741277`", ",", 
   "0.3365597068691384`", ",", "0.2933461441645233`", ",", 
   "0.2957635772810677`", ",", "0.31801967252550156`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935538436320734`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935538436632733`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1218487247197696`", ",", "0.1243070633580694`", ",", 
   "0.12683379654179605`", ",", "0.13594431881994895`", ",", 
   "0.16119224914119312`", ",", "0.4408728661729`", ",", "0.554807543827466`",
    ",", "0.3300771227589048`", ",", "0.4028480227839213`", ",", 
   "0.5185764800028906`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935538436632733`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.26414725635408753`", ",", "0.5040720325687985`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.26414725635408753`, 0.5040720325687985}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593553843678873*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13764702723911262`", ",", "0.9533317703278128`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13764702723911262`, 0.9533317703278128}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593553843678873*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.902`"}],
  SequenceForm[" R^2: ", 0.902],
  Editable->False]], "Print",
 CellChangeTimes->{3.593553843678873*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.93125257`10.40479113989233"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.93125257`10.40479113989233],
  Editable->False]], "Print",
 CellChangeTimes->{3.593553843678873*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.61856363666666666666666666666666666667`10.606442539450855"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   47.61856363666666666666666666666666666667`10.606442539450855],
  Editable->False]], "Print",
 CellChangeTimes->{3.593553843678873*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935538436944733`*^9}],

Cell[BoxData["\<\"Country: France\"\>"], "Print",
 CellChangeTimes->{3.5935538437100735`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935538437100735`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5935538443808746`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593553845067276*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593553845067276*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593553845067276*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.593553845067276*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6864013`6.987093095199286"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6864013`6.987093095199286],
  Editable->False]], "Print",
 CellChangeTimes->{3.593553845098476*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593553845098476*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593553845098476*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593553845660077*^9}],

Cell[BoxData["0.9676271068589398`"], "Print",
 CellChangeTimes->{3.593553845660077*^9}],

Cell[BoxData["0.0022999999999999995`"], "Print",
 CellChangeTimes->{3.593553845660077*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593553845660077*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593553845675677*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "4.12334724166666666666666666666666666667`9.543916158032664"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.12334724166666666666666666666666666667`9.543916158032664],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554093060911*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10590867999651349`", ",", "0.9640643556618731`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10590867999651349`, 0.9640643556618731}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935541357893867`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.951`"}],
  SequenceForm[" R^2: ", 0.951],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935541358049865`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.71214125166666666666666666666666666667`8.781232391761455"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.71214125166666666666666666666666666667`8.781232391761455],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935541361325874`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935541361325874`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935541361325874`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.40426246666666666666666666666666666667`9.076114536331202"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.40426246666666666666666666666666666667`9.076114536331202],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935542200451345`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12206065894621895`", ",", "0.9585903547452654`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12206065894621895`, 0.9585903547452654}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935542591388035`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.903`"}],
  SequenceForm[" R^2: ", 0.903],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554259154403*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65156114333333333333333333333333333333`8.742621424927767"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65156114333333333333333333333333333333`8.742621424927767],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554259154403*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554259154403*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593554259154403*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.81510143166666666666666666666666666667`8.839877904160678"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.81510143166666666666666666666666666667`8.839877904160678],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554308044889*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12507632666085305`", ",", "0.9575957752115306`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12507632666085305`, 0.9575957752115306}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554347294558*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.91`"}],
  SequenceForm[" R^2: ", 0.91],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554347294558*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65416115`8.744350996439893"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.65416115`8.744350996439893],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554347294558*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554347294558*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593554347310158*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.24128042333333333333333333333333333333`8.311188334387943"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.24128042333333333333333333333333333333`8.311188334387943],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935543617713833`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13370186477878124`", ",", "0.9546576444080739`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13370186477878124`, 0.9546576444080739}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554402674655*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554402674655*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68172119666666666666666666666666666667`8.762273045884694"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68172119666666666666666666666666666667`8.762273045884694],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554402690255*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554402690255*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593554402690255*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.70668124166666666666666666666666666667`8.777889811656038"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.70668124166666666666666666666666666667`8.777889811656038],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935544450755296`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1361050905721481`", ",", "0.9538435591177634`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1361050905721481, 0.9538435591177634}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935544857292013`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935544857292013`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67756119`8.759614770314624"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.67756119`8.759614770314624],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935544857292013`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935544857448015`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935544857448015`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.28288049666666666666666666666666666667`8.38026925406064"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.28288049666666666666666666666666666667`8.38026925406064],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554502702031*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593554502702031*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935546817903457`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935548622514625`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13574095845028186`", ",", "0.9539794444962137`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13574095845028186`, 0.9539794444962137}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935548981783257`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.903`"}],
  SequenceForm[" R^2: ", 0.903],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935548981783257`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.59127157666666666666666666666666666667`9.747635454229634"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.59127157666666666666666666666666666667`9.747635454229634],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935548981783257`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935548981783257`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593554898193926*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05174009166666666666666666666666666667`7.642493442025847"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05174009166666666666666666666666666667`7.642493442025847],
  Editable->False]], "Print",
 CellChangeTimes->{3.593554901282731*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593554901282731*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593555713763358*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593556518053971*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935566809182568`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "29.13200992052444`", ",", "53.15764045288067`", ",", "66.36880230536372`", 
   ",", "79.66102696152628`", ",", "83.5877691573227`", ",", 
   "90.9652224322551`", ",", "93.99908302207926`", ",", "96.21360215857747`", 
   ",", "99.00158439813427`", ",", "0.7568861461739531`", ",", 
   "18.48570892857143`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935566812770576`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13859928791633064`", ",", "0.9530141837366857`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13859928791633064`, 0.9530141837366857}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935566817606583`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.9`"}],
  SequenceForm[" R^2: ", 0.9],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935566818074584`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.660332095`10.400842257558653"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.660332095`10.400842257558653],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935566818542585`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "8"}],
  SequenceForm["Iteration ", 8],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935566819166584`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593556681947859*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.19916035`8.227869129039254"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.19916035`8.227869129039254],
  Editable->False]], "Print",
 CellChangeTimes->{3.593556692977078*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593556693023878*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935574959728885`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935583017299037`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593558465904592*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0570669885571643`", ",", "0.058234839252554724`", ",", 
   "1.0204645579680551`", ",", "0.05788716564125362`", ",", 
   "1.014372180919758`", ",", "0.9964612708860697`", ",", 
   "0.9960524342474506`", ",", "0.9961493108739207`", ",", 
   "0.9992452067123431`", ",", "0.8993630188925963`", ",", 
   "0.8769935195518984`", ",", "0.9386033139363971`", ",", 
   "6.600755244850161`"}], "}"}]], "Print",
 CellChangeTimes->{3.593558466076192*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935584666377935`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.16464781351521532`", ",", "0.8142112671912184`", ",", 
   "0.11689020502196773`", ",", "0.09039240077293659`", ",", 
   "0.5720797679911359`", ",", "0.2802126085959154`", ",", 
   "0.913625414257382`", ",", "0.9380524451060028`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935584667001934`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935584669809937`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "28.299472902124144`", ",", "52.89548166982525`", ",", "66.256104949226`", 
   ",", "79.63294169620363`", ",", "83.57657834024795`", ",", 
   "90.96586231913503`", ",", "94.00018449746332`", ",", "96.21369840129834`",
    ",", "99.0003794153172`", ",", "0.755967225612617`", ",", 
   "18.436283928571427`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935584670121937`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.30235049545897985`"}],
  SequenceForm[" MPC mean:", 0.30235049545897985`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935584674489946`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593558467480195*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.2397602537307999`", ",", "0.24345067218275396`", ",", 
   "0.25121280994515893`", ",", "0.2690896328847333`", ",", 
   "0.2813092617929114`", ",", "0.3330418125679686`", ",", 
   "0.3545335297343254`", ",", "0.31789115214336705`", ",", 
   "0.32771554825013527`", ",", "0.3393472498024346`"}], "}"}]], "Print",
 CellChangeTimes->{3.593558467636195*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935584679169955`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12163503874093717`", ",", "0.12445109065905413`", ",", 
   "0.12774570989242462`", ",", "0.14115935307033145`", ",", 
   "0.16849133437791608`", ",", "0.4720063043674352`", ",", 
   "0.5982744434998122`", ",", "0.35372098591587287`", ",", 
   "0.4306399486983741`", ",", "0.5583237199033828`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935584680729957`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.28464603593080773`", ",", "0.50776763989458`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.28464603593080773`, 0.50776763989458}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558468275796*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13843666178906186`", ",", "0.9530668166740983`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13843666178906186`, 0.9530668166740983}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935584683381963`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.902`"}],
  SequenceForm[" R^2: ", 0.902],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558468431796*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.55321190833333333333333333333333333333`10.399270936081354"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.55321190833333333333333333333333333333`10.399270936081354],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558468431796*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "77.03215529833333333333333333333333333333`10.815338297460427"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   77.03215529833333333333333333333333333333`10.815338297460427],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935584684473963`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593558471723402*^9}],

Cell[BoxData["\<\"Country: Greece\"\>"], "Print",
 CellChangeTimes->{3.5935584717546024`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593558473361405*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5935584888990326`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593558489835034*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593558489881834*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935584899754343`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935584899910345`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.9828017`7.142980896861013"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.9828017`7.142980896861013],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935584900222344`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935584900534344`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5935584900690346`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5935584906774354`*^9}],

Cell[BoxData["0.9664616147492843`"], "Print",
 CellChangeTimes->{3.5935584907086353`*^9}],

Cell[BoxData["0.0027999999999999987`"], "Print",
 CellChangeTimes->{3.5935584907242355`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935584907398357`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935584907398357`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "4.2125274`9.553208986933033"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.2125274`9.553208986933033],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558743429079*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10313131614148346`", ",", "0.9649984375802931`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10313131614148346`, 0.9649984375802931}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935587834743495`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.957`"}],
  SequenceForm[" R^2: ", 0.957],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935587834899497`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66716117166666666666666666666666666667`8.752897010932836"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66716117166666666666666666666666666667`8.752897010932836],
  Editable->False]], "Print",
 CellChangeTimes->{3.59355878350555*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.59355878350555*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59355878350555*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.37826242`9.067998162962558"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.37826242`9.067998162962558],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558866497696*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1181107475866077`", ",", "0.9599373200372049`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1181107475866077, 0.9599373200372049}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935589052793636`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.905`"}],
  SequenceForm[" R^2: ", 0.905],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558905294964*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.652081145`8.742967890930617"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.652081145`8.742967890930617],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558905294964*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935589053105636`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935589053261633`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.75192132166666666666666666666666666667`8.804838648177434"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.75192132166666666666666666666666666667`8.804838648177434],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558950441443*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1262296381730802`", ",", "0.9571917840778364`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1262296381730802, 0.9571917840778364}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935589900031123`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.904`"}],
  SequenceForm[" R^2: ", 0.904],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558990018712*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65442115`8.744523574911694"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.65442115`8.744523574911694],
  Editable->False]], "Print",
 CellChangeTimes->{3.593558990018712*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935589900343122`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935589900343122`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.66768117166666666666666666666666666667`8.753235377696736"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.66768117166666666666666666666666666667`8.753235377696736],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935590297519827`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12783521109969415`", ",", "0.9566530615990076`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12783521109969415`, 0.9566530615990076}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593559069376052*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.907`"}],
  SequenceForm[" R^2: ", 0.907],
  Editable->False]], "Print",
 CellChangeTimes->{3.593559069422852*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65546115166666666666666666666666666667`8.745213204933165"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65546115166666666666666666666666666667`8.745213204933165],
  Editable->False]], "Print",
 CellChangeTimes->{3.593559069422852*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593559069422852*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593559069422852*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.19578034333333333333333333333333333333`8.220435333995287"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.19578034333333333333333333333333333333`8.220435333995287],
  Editable->False]], "Print",
 CellChangeTimes->{3.593559080795272*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12919094995651284`", ",", "0.956189223587484`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12919094995651284`, 0.956189223587484}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935591215425434`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.906`"}],
  SequenceForm[" R^2: ", 0.906],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935591215581436`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67340118333333333333333333333333333333`8.756940123431772"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67340118333333333333333333333333333333`8.756940123431772],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935591215581436`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935591215581436`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593559121573744*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.202540355`8.235177815063745"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.202540355`8.235177815063745],
  Editable->False]], "Print",
 CellChangeTimes->{3.593559133382964*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935591333985643`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935593115664773`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935594892975893`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13268939298583043`", ",", "0.9550042639453842`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13268939298583043`, 0.9550042639453842}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935595262852545`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.903`"}],
  SequenceForm[" R^2: ", 0.903],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935595263164544`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.54317149333333333333333333333333333333`9.744454551309701"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.54317149333333333333333333333333333333`9.744454551309701],
  Editable->False]], "Print",
 CellChangeTimes->{3.593559526332054*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935595263476543`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935595263632545`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.39754069833333333333333333333333333333`8.528047844495703"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.39754069833333333333333333333333333333`8.528047844495703],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935595498412957`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935595498568954`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935603609491205`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561173757348*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593561338134837*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05713476530378962`", ",", "0.058228048940686`", ",", 
   "1.0191351733236902`", ",", "0.057750504020615094`", ",", 
   "1.0107769536384992`", ",", "0.9965433679451083`", ",", 
   "0.996373252272924`", ",", "0.996422764143323`", ",", 
   "0.9992463358025374`", ",", "0.9072546996453575`", ",", 
   "0.8855976253912792`", ",", "0.9400360405252773`", ",", 
   "6.600220231289633`"}], "}"}]], "Print",
 CellChangeTimes->{3.593561338134837*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593561338618438*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.17143357387342845`", ",", "0.8163656769210363`", ",", 
   "0.12757754986644843`", ",", "0.08208685716249754`", ",", 
   "0.5624089569851191`", ",", "0.2775487445205954`", ",", 
   "0.9168891014742276`", ",", "0.9403089079174032`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935613386340375`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935613386652374`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "35.30731488446317`", ",", "61.10194004900198`", ",", "73.91495614826266`", 
   ",", "85.59845951610811`", ",", "88.62132527965689`", ",", 
   "93.783906827491`", ",", "95.81585408173058`", ",", "97.30796812755507`", 
   ",", "99.24784055728033`", ",", "0.8124037446921328`", ",", 
   "13.221648214285715`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935613386808376`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.34721676649674615`"}],
  SequenceForm[" MPC mean:", 0.34721676649674615`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561338727638*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935613387432375`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.2998393617951778`", ",", "0.2978426211290337`", ",", 
   "0.30457907159705233`", ",", "0.3189111671729893`", ",", 
   "0.3296950787594297`", ",", "0.37285239864359887`", ",", 
   "0.4264855859936283`", ",", "0.36031421076178327`", ",", 
   "0.3690757415860606`", ",", "0.40362576067736256`"}], "}"}]], "Print",
 CellChangeTimes->{3.593561338774438*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593561338821238*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12030028049168828`", ",", "0.12302751067295503`", ",", 
   "0.12642280043011977`", ",", "0.14606751691509212`", ",", 
   "0.1934649656015336`", ",", "0.535287195026558`", ",", 
   "0.6421592399776737`", ",", "0.4099390280885895`", ",", 
   "0.495805406951103`", ",", "0.6098464655215381`"}], "}"}]], "Print",
 CellChangeTimes->{3.593561338836838*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.322036569098624`", ",", "0.6196312859279337`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.322036569098624, 0.6196312859279337}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561338899238*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13118691259374274`", ",", "0.955517647973517`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13118691259374274`, 0.955517647973517}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935613389460382`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.907`"}],
  SequenceForm[" R^2: ", 0.907],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935613389616385`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.80567235`10.402965171213923"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.80567235`10.402965171213923],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561338992838*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.47062337666666666666666666666666666667`10.605091183057414"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   47.47062337666666666666666666666666666667`10.605091183057414],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561339008438*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935613390240383`*^9}],

Cell[BoxData["\<\"Country: Italy\"\>"], "Print",
 CellChangeTimes->{3.5935613390240383`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935613390552387`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.59356133994444*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935613406308413`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400663`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935613406308413`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935613406308413`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992645`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005181`", ",", "0.9488134182354926`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935613406464415`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6864012`6.98709303192806"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6864012`6.98709303192806],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935613406464415`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935613406464415`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5935613406464415`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593561341286042*^9}],

Cell[BoxData["0.966837729384417`"], "Print",
 CellChangeTimes->{3.593561341286042*^9}],

Cell[BoxData["0.002499999999999999`"], "Print",
 CellChangeTimes->{3.593561341286042*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561341286042*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593561341286042*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "3.93276690666666666666666666666666666667`9.523364454948709"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   3.93276690666666666666666666666666666667`9.523364454948709],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935615775952573`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10588820007017896`", ",", "0.9640817554768708`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10588820007017896`, 0.9640817554768708}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356161890413*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.951`"}],
  SequenceForm[" R^2: ", 0.951],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356161893533*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.69420122`8.77015162078756"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.69420122`8.77015162078756],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356161898213*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356161898213*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59356161898213*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.29272227`9.040171478771015"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.29272227`9.040171478771015],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935616964986663`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12575306589521923`", ",", "0.9573654105287428`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12575306589521923`, 0.9573654105287428}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561736200736*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.897`"}],
  SequenceForm[" R^2: ", 0.897],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935617362163363`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66196116333333333333333333333333333333`8.749498758735706"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66196116333333333333333333333333333333`8.749498758735706],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561736263136*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935617362943363`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935617362943363`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.81510143166666666666666666666666666667`8.839877904160678"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.81510143166666666666666666666666666667`8.839877904160678],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935617854188223`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13815370579379246`", ",", "0.9531683028055562`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13815370579379246`, 0.9531683028055562}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935618259632936`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.893`"}],
  SequenceForm[" R^2: ", 0.893],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935618259788933`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67574118666666666666666666666666666667`8.758446638351108"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67574118666666666666666666666666666667`8.758446638351108],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935618259944935`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935618260100937`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593561826025694*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.74438130666666666666666666666666666667`8.80046170655742"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.74438130666666666666666666666666666667`8.80046170655742],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935618702985716`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1368464588597016`", ",", "0.9536274019414254`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1368464588597016, 0.9536274019414254}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561910375042*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.903`"}],
  SequenceForm[" R^2: ", 0.903],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935619107182426`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66794117333333333333333333333333333333`8.753404463353272"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.66794117333333333333333333333333333333`8.753404463353272],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935619107338424`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561910749442*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935619107650423`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009333333333333333333333333333333`7.653270527830221"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009333333333333333333333333333333`7.653270527830221],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561913588648*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13995040629231337`", ",", "0.9525797665511562`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13995040629231337`, 0.9525797665511562}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561955427921*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.898`"}],
  SequenceForm[" R^2: ", 0.898],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561955474721*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.697841225`8.772422870050528"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.697841225`8.772422870050528],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561955474721*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593561955474721*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593561955474721*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.07592013333333333333333333333333333333`7.809023210357207"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.07592013333333333333333333333333333333`7.809023210357207],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935619599831285`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935619600455294`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593562139258644*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935623171145563`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14087844872120003`", ",", "0.9522600488180567`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14087844872120003`, 0.9522600488180567}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593562352979019*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356235332222*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.549671505`9.74488576696415"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.549671505`9.74488576696415],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356235332222*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356235332222*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59356235332222*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.12298021666666666666666666666666666667`8.01850150206538"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.12298021666666666666666666666666666667`8.01850150206538],
  Editable->False]], "Print",
 CellChangeTimes->{3.593562360357832*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935623604202323`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935631542588263`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593563952231428*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935641132705107`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.0570263838732445`", ",", "0.05822214910069296`", ",", 
   "1.0209686314690118`", ",", "0.057875337299510464`", ",", 
   "1.0148870289259964`", ",", "0.9962122303606747`", ",", 
   "0.9959538505047794`", ",", "0.9960346096107561`", ",", 
   "0.9992581584972207`", ",", "0.8969406202118453`", ",", 
   "0.8731534523120755`", ",", "0.9317699909715013`", ",", 
   "6.604416241145457`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935641132705107`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593564113364111*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1633095473825822`", ",", "0.8047419830833118`", ",", 
   "0.10314725699970484`", ",", "0.07647731478721015`", ",", 
   "0.5725701132078445`", ",", "0.27905376741410787`", ",", 
   "0.906263784732206`", ",", "0.9340876563002249`"}], "}"}]], "Print",
 CellChangeTimes->{3.593564113364111*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935641134265113`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "25.01566508289444`", ",", "48.55422136233224`", ",", "61.806205210707276`",
    ",", "75.81295078846709`", ",", "80.19929454440692`", ",", 
   "88.90061899677471`", ",", "92.65359959057604`", ",", "95.42456462212652`",
    ",", "98.87192699951183`", ",", "0.7204918454043878`", ",", 
   "22.467657142857142`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935641134265113`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.310878264798228`"}],
  SequenceForm[" MPC mean:", 0.310878264798228],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564113457711*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593564113457711*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.22577363390234761`", ",", "0.23332520152031433`", ",", 
   "0.23913005692975298`", ",", "0.2620960462801589`", ",", 
   "0.2810072978206577`", ",", "0.35392739676666496`", ",", 
   "0.35225531124228127`", ",", "0.3320737850173341`", ",", 
   "0.3495744138799507`", ",", "0.34911343332350986`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935641134733114`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593564113520111*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1223176253785655`", ",", "0.12554425294407057`", ",", 
   "0.12987512769109166`", ",", "0.14635235590809204`", ",", 
   "0.17163607149228444`", ",", "0.4858859415897422`", ",", 
   "0.6270145370383894`", ",", "0.3635894966433758`", ",", 
   "0.44190191934289813`", ",", "0.5814101858336452`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935641135357113`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.2974862515870862`", ",", "0.4712532410621586`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.2974862515870862, 0.4712532410621586}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935641135981116`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14040297638145915`", ",", "0.9524164774436836`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14040297638145915`, 0.9524164774436836}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935641136137114`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935641136137114`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.21547131333333333333333333333333333333`10.394279145096005"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.21547131333333333333333333333333333333`10.394279145096005],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935641136293116`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "46.210921165`10.593410874013909"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 46.210921165`10.593410874013909],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935641136293116`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935641136293116`*^9}],

Cell[BoxData["\<\"Country: Luxembourg\"\>"], "Print",
 CellChangeTimes->{3.5935641136293116`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593564113644912*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593564114378113*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593564115064514*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400663`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593564115095714*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593564115095714*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992645`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005181`", ",", "0.9488134182354926`"}], "}"}]], "Print",
 CellChangeTimes->{3.593564115095714*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6864012`6.98709303192806"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6864012`6.98709303192806],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935641151113143`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935641151113143`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5935641151113143`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593564115688515*^9}],

Cell[BoxData["0.9698933289126804`"], "Print",
 CellChangeTimes->{3.593564115704115*^9}],

Cell[BoxData["0.0014`"], "Print",
 CellChangeTimes->{3.593564115704115*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564115704115*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593564115719715*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "4.10055222`9.541508595166933"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.10055222`9.541508595166933],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935643617684484`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11556585642739453`", ",", "0.9607820398297354`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11556585642739453`, 0.9607820398297354}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935644068770757`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.935`"}],
  SequenceForm[" R^2: ", 0.935],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564406890076*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.752290435`8.805051788230848"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.752290435`8.805051788230848],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564406900077*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564406904077*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935644069070773`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.35832769166666666666666666666666666667`9.061670802780707"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.35832769166666666666666666666666666667`9.061670802780707],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564488359736*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13082349813461516`", ",", "0.955629424727473`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13082349813461516`, 0.955629424727473}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935645327212734`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.897`"}],
  SequenceForm[" R^2: ", 0.897],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935645327242737`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.73884225833333333333333333333333333333`8.797217975314277"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.73884225833333333333333333333333333333`8.797217975314277],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564532727274*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935645327312737`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593564532734274*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.80177919333333333333333333333333333333`8.83272103006557"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.80177919333333333333333333333333333333`8.83272103006557],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935645807970233`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14131764435634614`", ",", "0.9520760152772668`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14131764435634614`, 0.9520760152772668}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564627338535*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.893`"}],
  SequenceForm[" R^2: ", 0.893],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564627338535*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.77491186166666666666666666666666666667`8.817918556956014"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.77491186166666666666666666666666666667`8.817918556956014],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935646273541346`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564627385335*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593564627385335*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.84042654333333333333333333333333333333`8.85316600856206"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.84042654333333333333333333333333333333`8.85316600856206],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935646777641277`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14179870931824864`", ",", "0.9519109060240425`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14179870931824864`, 0.9519109060240425}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935647276759253`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935647276915255`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.83238330166666666666666666666666666667`8.848989607517398"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.83238330166666666666666666666666666667`8.848989607517398],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935647277071257`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935647277071257`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593564727722726*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.07695014`7.814875662521818"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.07695014`7.814875662521818],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564732339734*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1392724930589715`", ",", "0.9527656747680113`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1392724930589715, 0.9527656747680113}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593564779778054*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.902`"}],
  SequenceForm[" R^2: ", 0.902],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935647797880545`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.79134534166666666666666666666666666667`8.827032298400978"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.79134534166666666666666666666666666667`8.827032298400978],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935647798080544`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935647798080544`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593564779818054*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05616674666666666666666666666666666667`7.678145517282562"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05616674666666666666666666666666666667`7.678145517282562],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935647831280594`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593564783158059*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356496115837*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593565139965884*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1419707050188006`", ",", "0.9518586210995801`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1419707050188006, 0.9518586210995801}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593565176485548*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.898`"}],
  SequenceForm[" R^2: ", 0.898],
  Editable->False]], "Print",
 CellChangeTimes->{3.593565176501148*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.55023813666666666666666666666666666667`9.74492333745326"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.55023813666666666666666666666666666667`9.74492333745326],
  Editable->False]], "Print",
 CellChangeTimes->{3.593565176516748*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593565176516748*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593565176516748*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05122009`7.6381065854575505"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05122009`7.6381065854575505],
  Editable->False]], "Print",
 CellChangeTimes->{3.593565179215553*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593565179215553*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935659772349544`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935667756443567`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.59356693688624*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057022164513300465`", ",", "0.058222797469576745`", ",", 
   "1.0210555486015658`", ",", "0.05783045554503431`", ",", 
   "1.0141750324392775`", ",", "0.9965159922023874`", ",", 
   "0.995956362382241`", ",", "0.9960611711095614`", ",", 
   "0.9992623431767498`", ",", "0.8966718582933841`", ",", 
   "0.874514122060135`", ",", "0.9392973817838521`", ",", 
   "6.598936882650483`"}], "}"}]], "Print",
 CellChangeTimes->{3.59356693688624*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.59356693701104*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.16539686529018843`", ",", "0.8132468144684727`", ",", 
   "0.12454153358341825`", ",", "0.08026151446572914`", ",", 
   "0.5737118552192411`", ",", "0.28332582973074477`", ",", 
   "0.9157955236397675`", ",", "0.937447165423258`"}], "}"}]], "Print",
 CellChangeTimes->{3.59356693701104*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935669370734406`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "21.977612724363187`", ",", "44.955740144743956`", ",", 
   "58.85275184965809`", ",", "74.11919390872471`", ",", "78.96381620538332`",
    ",", "88.32584999273094`", ",", "92.18358300874911`", ",", 
   "94.98738062323613`", ",", "98.57617156842142`", ",", 
   "0.7013350971434691`", ",", "22.713316071428572`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935669370890408`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.23293986406956058`"}],
  SequenceForm[" MPC mean:", 0.23293986406956058`],
  Editable->False]], "Print",
 CellChangeTimes->{3.59356693712024*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935669371514406`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.19483132535803238`", ",", "0.1977444983561386`", ",", 
   "0.20704359132847494`", ",", "0.22275221708272475`", ",", 
   "0.23243273282643062`", ",", "0.23370008970238798`", ",", 
   "0.26323628345436856`", ",", "0.24056976788658468`", ",", 
   "0.237634240197926`", ",", "0.24873387359958976`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935669371670403`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935669372138405`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12360342252074707`", ",", "0.1257517060583384`", ",", 
   "0.12775209585238867`", ",", "0.13276872352109792`", ",", 
   "0.14407445594574442`", ",", "0.3530358499770606`", ",", 
   "0.4635713592032349`", ",", "0.26548632721053267`", ",", 
   "0.3203184833123991`", ",", "0.426579822320897`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935669372294407`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.2189566184184376`", ",", "0.4014726084425049`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.2189566184184376, 0.4014726084425049}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593566937260641*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1403463056740516`", ",", "0.9524103147453215`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1403463056740516, 0.9524103147453215}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593566937276241*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935669372918406`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.294771455`10.395456362396697"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 29.294771455`10.395456362396697],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935669372918406`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.0306221`10.601046971652552"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 47.0306221`10.601046971652552],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935669372918406`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935669373074408`*^9}],

Cell[BoxData["\<\"Country: Malta\"\>"], "Print",
 CellChangeTimes->{3.5935669373074408`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935669373386407`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5935669384774427`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935669391794443`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935669391794443`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935669391794443`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935669391794443`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.7020013`6.996852914210089"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.7020013`6.996852914210089],
  Editable->False]], "Print",
 CellChangeTimes->{3.593566939241844*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593566939241844*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593566939241844*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593566939756645*^9}],

Cell[BoxData["0.9711999999999998`"], "Print",
 CellChangeTimes->{3.5935669397722454`*^9}],

Cell[BoxData["0.0005999999999999996`"], "Print",
 CellChangeTimes->{3.5935669397722454`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935669397722454`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935669397722454`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "3.42966602333333333333333333333333333333`9.46391807925394"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   3.42966602333333333333333333333333333333`9.46391807925394],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935671455366063`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13373822654643341`", ",", "0.9546480450749516`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13373822654643341`, 0.9546480450749516}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567186611479*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.909`"}],
  SequenceForm[" R^2: ", 0.909],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567186611479*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68458120166666666666666666666666666667`8.764091217623793"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68458120166666666666666666666666666667`8.764091217623793],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567186611479*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567186611479*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593567186627079*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.753481325`8.805738741030902"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.753481325`8.805738741030902],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935672318203583`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13864827675413854`", ",", "0.952984519026832`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13864827675413854`, 0.952984519026832}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935672706956263`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.898`"}],
  SequenceForm[" R^2: ", 0.898],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935672706956263`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64792113666666666666666666666666666667`8.740188396069536"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64792113666666666666666666666666666667`8.740188396069536],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935672706956263`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935672707112265`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935672707112265`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05434009666666666666666666666666666667`7.663786654873187"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05434009666666666666666666666666666667`7.663786654873187],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935672742992325`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1432748600094452`", ",", "0.9514191176368664`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1432748600094452, 0.9514191176368664}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567316653307*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.896`"}],
  SequenceForm[" R^2: ", 0.896],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567316700107*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.71162125`8.78091515645825"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.71162125`8.78091515645825],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567316700107*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567316700107*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593567316700107*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.373360655`8.500794797991798"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.373360655`8.500794797991798],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567339398147*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14291133229393288`", ",", "0.9515390149708692`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14291133229393288`, 0.9515390149708692}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935673811594205`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935673812062206`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.70174123333333333333333333333333333333`8.774843244030494"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.70174123333333333333333333333333333333`8.774843244030494],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935673812062206`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935673812218204`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935673812218204`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05304009166666666666666666666666666667`7.653270514183484"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05304009166666666666666666666666666667`7.653270514183484],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567384341826*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1449335344935885`", ",", "0.9508472237766252`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1449335344935885, 0.9508472237766252}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935674279283023`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.896`"}],
  SequenceForm[" R^2: ", 0.896],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935674279439025`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.725921275`8.789555772928209"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.725921275`8.789555772928209],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935674279751024`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935674279751024`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935674279751024`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.23088040666666666666666666666666666667`8.292053326920584"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.23088040666666666666666666666666666667`8.292053326920584],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935674417499266`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593567441765527*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935676181238365`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567799676955*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14486348532355908`", ",", "0.9508734184451594`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14486348532355908`, 0.9508734184451594}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935678361030197`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.898`"}],
  SequenceForm[" R^2: ", 0.898],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935678361186194`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.57177154166666666666666666666666666667`9.746348705608282"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.57177154166666666666666666666666666667`9.746348705608282],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935678361186194`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935678361186194`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935678361342196`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05148009166666666666666666666666666667`7.640305559763676"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05148009166666666666666666666666666667`7.640305559763676],
  Editable->False]], "Print",
 CellChangeTimes->{3.593567839145025*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593567839145025*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935686537004557`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935694725146933`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935696392789865`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057045879296064646`", ",", "0.058318929875262264`", ",", 
   "1.0223162583328862`", ",", "0.05805766970440705`", ",", 
   "1.0177364328646996`", ",", "0.9965282125677732`", ",", 
   "0.9956426098710109`", ",", "0.9958136231443738`", ",", 
   "0.9992911962501196`", ",", "0.8890319867894354`", ",", 
   "0.8666199414762793`", ",", "0.9395001966296929`", ",", 
   "6.598258178151919`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935696393257866`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593569639434987*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.17042844910736338`", ",", "0.8136995258258816`", ",", 
   "0.11982274691610975`", ",", "0.08908398724733864`", ",", 
   "0.5865294140253811`", ",", "0.29185087114608815`", ",", 
   "0.9126823503751061`", ",", "0.9351641637092278`"}], "}"}]], "Print",
 CellChangeTimes->{3.593569639434987*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593569639450587*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "12.353016495827852`", ",", "30.499714406974178`", ",", 
   "43.56983129873802`", ",", "60.34417090064152`", ",", "66.43261026359416`",
    ",", "79.82230366325767`", ",", "86.11198298542975`", ",", 
   "90.97031274427913`", ",", "97.44140104490106`", ",", 
   "0.5646595636609645`", ",", "36.72231607142857`"}], "}"}]], "Print",
 CellChangeTimes->{3.593569639450587*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.18293270953389207`"}],
  SequenceForm[" MPC mean:", 0.18293270953389207`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935696394817867`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935696394817867`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.14541961888306676`", ",", "0.15018634640505402`", ",", 
   "0.16099705879356807`", ",", "0.17786278763599284`", ",", 
   "0.18774369715740735`", ",", "0.17567619590844807`", ",", 
   "0.19402343018516754`", ",", "0.18917759793286837`", ",", 
   "0.18239958527325417`", ",", "0.1838810410499493`"}], "}"}]], "Print",
 CellChangeTimes->{3.593569639497387*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593569639512987*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1278993836795197`", ",", "0.12927073089491914`", ",", 
   "0.13049640597927992`", ",", "0.132599033941314`", ",", 
   "0.13481929841003615`", ",", "0.2513050886799406`", ",", 
   "0.34203482966833865`", ",", "0.19970157968228885`", ",", 
   "0.23011742467445828`", ",", "0.3087036018415513`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935696395285873`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.17410340947132097`", ",", "0.29364728761401626`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.17410340947132097`, 0.29364728761401626`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569639575387*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14534706048367096`", ",", "0.950710757891526`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14534706048367096`, 0.950710757891526}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569639575387*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.897`"}],
  SequenceForm[" R^2: ", 0.897],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569639590987*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "30.002232695`10.405819823303226"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 30.002232695`10.405819823303226],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569639590987*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "45.00165904`10.581894773071772"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 45.00165904`10.581894773071772],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935696396065874`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935696396065874`*^9}],

Cell[BoxData["\<\"Country: Netherland\"\>"], "Print",
 CellChangeTimes->{3.5935696396065874`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935696396065874`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.5935696403085885`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.59356964096379*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935696409793897`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935696409793897`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935696409793897`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.6708012`6.9771088286785465"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.6708012`6.9771088286785465],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935696409793897`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935696409793897`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5935696409793897`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593569641618991*^9}],

Cell[BoxData["0.9710649999999998`"], "Print",
 CellChangeTimes->{3.593569641634591*^9}],

Cell[BoxData["0.0006999999999999997`"], "Print",
 CellChangeTimes->{3.593569641634591*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569641634591*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593569641634591*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "3.4730861`9.469381798094604"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   3.4730861`9.469381798094604],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935698500041566`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1299704709259039`", ",", "0.9559111294353333`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1299704709259039, 0.9559111294353333}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935698910166287`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.913`"}],
  SequenceForm[" R^2: ", 0.913],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935698910166287`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.68354120166666666666666666666666666667`8.763430945783966"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.68354120166666666666666666666666666667`8.763430945783966],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935698910166287`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935698910166287`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935698910322285`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.87464153666666666666666666666666666667`8.870496346329414"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.87464153666666666666666666666666666667`8.870496346329414],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569943495121*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14124104729208484`", ",", "0.9520910788289958`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14124104729208484`, 0.9520910788289958}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569984101992*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.893`"}],
  SequenceForm[" R^2: ", 0.893],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569984429593*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67678118833333333333333333333333333333`8.759114526856866"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67678118833333333333333333333333333333`8.759114526856866],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569984445193*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593569984445193*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935699844607925`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.543660955`8.663994391890606"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.543660955`8.663994391890606],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935700167528496`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14348212119338255`", ",", "0.9513353057307451`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14348212119338255`, 0.9513353057307451}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357005698532*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.897`"}],
  SequenceForm[" R^2: ", 0.897],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935700570165205`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67106117833333333333333333333333333333`8.755428363319677"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67106117833333333333333333333333333333`8.755428363319677],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935700570321207`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935700570321207`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935700570321207`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.270140475`8.360255907383582"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.270140475`8.360255907383582],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570073302949*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14674172020530774`", ",", "0.9502285999533062`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14674172020530774`, 0.9502285999533062}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570113426219*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.894`"}],
  SequenceForm[" R^2: ", 0.894],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935701134886193`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67054117666666666666666666666666666667`8.755091700425666"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67054117666666666666666666666666666667`8.755091700425666],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935701134886193`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935701135042195`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935701135042195`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.39182068833333333333333333333333333333`8.521753611451174"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.39182068833333333333333333333333333333`8.521753611451174],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935701369354606`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14457487372668948`", ",", "0.950961595158018`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14457487372668948`, 0.950961595158018}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935701771835313`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570177526732*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67080117833333333333333333333333333333`8.755260065034616"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67080117833333333333333333333333333333`8.755260065034616],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570177526732*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570177526732*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935701775423317`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.053820095`7.6596107082345855"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.053820095`7.6596107082345855],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570180412737*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593570180412737*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570357488648*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935705365925627`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1441300725391135`", ",", "0.9511117954999033`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1441300725391135, 0.9511117954999033}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570572550626*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.898`"}],
  SequenceForm[" R^2: ", 0.898],
  Editable->False]], "Print",
 CellChangeTimes->{3.593570572566226*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.53563147833333333333333333333333333333`9.743953804065807"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.53563147833333333333333333333333333333`9.743953804065807],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935705728938265`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935705728938265`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935705728938265`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.05122009`7.6381065854575505"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05122009`7.6381065854575505],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935705756238313`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935705756706314`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935713825040483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935721951250753`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593572359237364*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05704628962004067`", ",", "0.05830261899237001`", ",", 
   "1.0220229813489567`", ",", "0.05799969075897065`", ",", 
   "1.016712763359022`", ",", "0.996521695906533`", ",", 
   "0.9957243341961735`", ",", "0.9958630628290187`", ",", 
   "0.9992820476373271`", ",", "0.8908932765611955`", ",", 
   "0.8686100959964785`", ",", "0.9393793626904452`", ",", 
   "6.594119528700293`"}], "}"}]], "Print",
 CellChangeTimes->{3.593572359237364*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593572359284164*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1689045751316895`", ",", "0.813682717818955`", ",", 
   "0.11830223472383705`", ",", "0.08453564273231567`", ",", 
   "0.5816068364668363`", ",", "0.2889766417359274`", ",", 
   "0.9134395909415762`", ",", "0.9364742438362762`"}], "}"}]], "Print",
 CellChangeTimes->{3.593572359284164*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935723593777637`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "13.34167441305162`", ",", "32.32642147952754`", ",", "45.659386585057014`",
    ",", "62.38418883176582`", ",", "68.33505564062868`", ",", 
   "81.18196969939304`", ",", "87.09985776441718`", ",", "91.62279017220611`",
    ",", "97.60586628278729`", ",", "0.584807759615373`", ",", 
   "34.27791071428572`"}], "}"}]], "Print",
 CellChangeTimes->{3.593572359393364*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.18702386035157648`"}],
  SequenceForm[" MPC mean:", 0.18702386035157648`],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935723594557643`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935723594557643`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.15073397078095974`", ",", "0.15456828848881987`", ",", 
   "0.16521317385751122`", ",", "0.18193106046408558`", ",", 
   "0.1917348264019919`", ",", "0.17991883619199645`", ",", 
   "0.19893329629413403`", ",", "0.19324082804478127`", ",", 
   "0.18653409083031336`", ",", "0.18847811798633696`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935723594713645`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593572359486964*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12729596295252654`", ",", "0.12874358858896706`", ",", 
   "0.13010681421376447`", ",", "0.13254024355118543`", ",", 
   "0.1353335840171661`", ",", "0.2601702087389415`", ",", 
   "0.3536090844304478`", ",", "0.2051722015411065`", ",", 
   "0.2376785587266409`", ",", "0.3195106937891845`"}], "}"}]], "Print",
 CellChangeTimes->{3.593572359486964*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1777637491548022`", ",", "0.3027787484485204`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.1777637491548022, 0.3027787484485204}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572359486964*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14347119895948945`", ",", "0.9513339933358927`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14347119895948945`, 0.9513339933358927}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572359486964*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572359486964*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.72689221333333333333333333333333333333`10.401815756641913"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.72689221333333333333333333333333333333`10.401815756641913],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935723595025644`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "45.30429957333333333333333333333333333333`10.584805668602943"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   45.30429957333333333333333333333333333333`10.584805668602943],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935723595181646`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935723595181646`*^9}],

Cell[BoxData["\<\"Country: Portugal\"\>"], "Print",
 CellChangeTimes->{3.5935723595181646`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935723595337644`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593572360188966*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593572360890967*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400663`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593572360890967*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593572360890967*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992645`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005181`", ",", "0.9488134182354926`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935723609065666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.7020012`6.996852852344888"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.7020012`6.996852852344888],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935723609065666`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935723609065666`*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.5935723609065666`*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593572361468168*^9}],

Cell[BoxData["0.9675171451400308`"], "Print",
 CellChangeTimes->{3.5935723614993677`*^9}],

Cell[BoxData["0.0023999999999999994`"], "Print",
 CellChangeTimes->{3.5935723614993677`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935723614993677`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593572361514968*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "4.20342738333333333333333333333333333333`9.552269797342422"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.20342738333333333333333333333333333333`9.552269797342422],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572613673811*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10356825254737484`", ",", "0.9648540937414547`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10356825254737484`, 0.9648540937414547}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935726528922796`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.955`"}],
  SequenceForm[" R^2: ", 0.955],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357265290788*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65364114833333333333333333333333333333`8.74400563251661"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65364114833333333333333333333333333333`8.74400563251661],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357265290788*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357265290788*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59357265290788*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.42012249333333333333333333333333333333`9.080992054489078"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.42012249333333333333333333333333333333`9.080992054489078],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357273842723*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12681446427452334`", ",", "0.9569616307864214`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12681446427452334`, 0.9569616307864214}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572777177698*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.894`"}],
  SequenceForm[" R^2: ", 0.894],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572777520899*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.651301145`8.742448089888478"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.651301145`8.742448089888478],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572777520899*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572777520899*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593572777520899*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.10604194166666666666666666666666666667`8.972437844161732"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.10604194166666666666666666666666666667`8.972437844161732],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935728435402145`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1303511682871092`", ",", "0.9557852521256109`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1303511682871092, 0.9557852521256109}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935728826182833`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.904`"}],
  SequenceForm[" R^2: ", 0.904],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935728826182833`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.651301145`8.742448089888478"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.651301145`8.742448089888478],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935728826182833`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935728826182833`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935728826338835`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.31486055166666666666666666666666666667`8.426784500256831"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.31486055166666666666666666666666666667`8.426784500256831],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935729015255165`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.12504501467580229`", ",", "0.9576039868354278`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.12504501467580229`, 0.9576039868354278}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935729441291914`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.914`"}],
  SequenceForm[" R^2: ", 0.914],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572944144791*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.70980124666666666666666666666666666667`8.779803006005423"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.70980124666666666666666666666666666667`8.779803006005423],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572944144791*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572944160391*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593572944160391*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05226009166666666666666666666666666667`7.646836415380619"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05226009166666666666666666666666666667`7.646836415380619],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572947233597*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1272769974792422`", ",", "0.9568492666384507`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1272769974792422, 0.9568492666384507}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935729887296696`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.907`"}],
  SequenceForm[" R^2: ", 0.907],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357298874527*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.691601215`8.768521995783583"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.691601215`8.768521995783583],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357298874527*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357298874527*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59357298874527*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.05226009166666666666666666666666666667`7.646836415380619"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.05226009166666666666666666666666666667`7.646836415380619],
  Editable->False]], "Print",
 CellChangeTimes->{3.593572991865275*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593572991880875*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935731688943863`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935733460014973`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1333005760911353`", ",", "0.9548097226227052`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1333005760911353, 0.9548097226227052}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935733824743614`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935733827863617`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.510151435`9.742257339183201"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 6.510151435`9.742257339183201],
  Editable->False]], "Print",
 CellChangeTimes->{3.593573382801962*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593573382801962*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593573382801962*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.22308039166666666666666666666666666667`8.277127646535256"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.22308039166666666666666666666666666667`8.277127646535256],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935733958591843`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935733958591843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935742013509994`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935750107896214`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935751743871083`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.057157369248135796`", ",", "0.05824891466950161`", ",", 
   "1.0190971949150969`", ",", "0.05813804276015946`", ",", 
   "1.0171574291281023`", ",", "0.9965626614840543`", ",", 
   "0.9961593806295332`", ",", "0.9962363903273541`", ",", 
   "0.9992439815518704`", ",", "0.9029553835257836`", ",", 
   "0.881342017522112`", ",", "0.9419211194146697`", ",", 
   "6.6036552869552585`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935751743871083`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593575174839509*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.1617969924780417`", ",", "0.822807897343096`", ",", 
   "0.11983529365490106`", ",", "0.08472297726805754`", ",", 
   "0.5656237670565517`", ",", "0.27742808580792666`", ",", 
   "0.9163581552650392`", ",", "0.9397418185457693`"}], "}"}]], "Print",
 CellChangeTimes->{3.593575174855109*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593575174886309*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "32.90356424338236`", ",", "57.72577669733151`", ",", "70.84284691787003`", 
   ",", "83.35697581644582`", ",", "86.79728076138936`", ",", 
   "92.8472644842692`", ",", "95.2086899986251`", ",", "96.92331223124567`", 
   ",", "99.13711136141333`", ",", "0.7913959620652018`", ",", 
   "15.188257142857143`"}], "}"}]], "Print",
 CellChangeTimes->{3.593575174901909*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.31634583708966446`"}],
  SequenceForm[" MPC mean:", 0.31634583708966446`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575174917509*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935751749331093`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.26817818737788246`", ",", "0.2702478902643488`", ",", 
   "0.2779840586578145`", ",", "0.2926749753682535`", ",", 
   "0.3031088282316591`", ",", "0.3358466565718281`", ",", 
   "0.3841179679809362`", ",", "0.3280839394507158`", ",", 
   "0.3338490577628068`", ",", "0.36307192751668116`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935751749331093`*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935751749331093`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12095319992470543`", ",", "0.12358651332494364`", ",", 
   "0.12662659951988753`", ",", "0.1398920607270706`", ",", 
   "0.17643580985732543`", ",", "0.49182016210590385`", ",", 
   "0.6028570962781503`", ",", "0.3716505734578113`", ",", 
   "0.45259025087116056`", ",", "0.5689345585773415`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935751749331093`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.2939409952678891`", ",", "0.5662935723679929`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.2939409952678891, 0.5662935723679929}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357517497991*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13321781474931044`", ",", "0.9548415927102671`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13321781474931044`, 0.9548415927102671}],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357517497991*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.905`"}],
  SequenceForm[" R^2: ", 0.905],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935751749955096`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.64213206333333333333333333333333333333`10.400575686053283"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.64213206333333333333333333333333333333`10.400575686053283],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935751750111094`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "46.89160236`10.599761321735372"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 46.89160236`10.599761321735372],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935751750111094`*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.5935751750111094`*^9}],

Cell[BoxData["\<\"Country: Slovenia\"\>"], "Print",
 CellChangeTimes->{3.5935751750267096`*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.5935751750267096`*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593575175744311*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.593575176461912*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.593575176461912*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593575176461912*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.593575176461912*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.7176013`7.006398214632676"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.7176013`7.006398214632676],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575176477512*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593575176477512*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.593575176477512*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.5935751770703135`*^9}],

Cell[BoxData["0.9682556895411089`"], "Print",
 CellChangeTimes->{3.5935751770703135`*^9}],

Cell[BoxData["0.0021`"], "Print",
 CellChangeTimes->{3.593575177085913*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575177085913*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593575177085913*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "4.22292741833333333333333333333333333333`9.554279865257822"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.22292741833333333333333333333333333333`9.554279865257822],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575430445958*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.10550435382911721`", ",", "0.9641966412905645`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.10550435382911721`, 0.9641966412905645}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935754704444284`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.951`"}],
  SequenceForm[" R^2: ", 0.951],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575470772029*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66664117`8.752558379249866"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.66664117`8.752558379249866],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575470787629*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575470787629*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593575470787629*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "1.44118253166666666666666666666666666667`9.087385237687013"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.44118253166666666666666666666666666667`9.087385237687013],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935755569153805`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1277232315375009`", ",", "0.9566636038825356`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1277232315375009, 0.9566636038825356}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935755956502485`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.894`"}],
  SequenceForm[" R^2: ", 0.894],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575595977849*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64558113333333333333333333333333333333`8.738617078156176"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.64558113333333333333333333333333333333`8.738617078156176],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935755959934487`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.593575596009049*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593575596009049*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.05846186`8.95334146173784"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.05846186`8.95334146173784],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357565915796*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13343161381769136`", ",", "0.954731574826103`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13343161381769136`, 0.954731574826103}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935756982048283`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.901`"}],
  SequenceForm[" R^2: ", 0.901],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935756982204285`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65078114333333333333333333333333333333`8.742101208791263"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65078114333333333333333333333333333333`8.742101208791263],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935756982204285`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935756982204285`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935756982204285`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.544440955`8.664617035322799"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.544440955`8.664617035322799],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935757308868856`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14348693808595167`", ",", "0.951325546487469`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14348693808595167`, 0.951325546487469}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935757700429544`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.892`"}],
  SequenceForm[" R^2: ", 0.892],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935757700429544`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65286114666666666666666666666666666667`8.743487071682463"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65286114666666666666666666666666666667`8.743487071682463],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935757700585546`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935757700585546`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935757700585546`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.72384127166666666666666666666666666667`8.788309590107582"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.72384127166666666666666666666666666667`8.788309590107582],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935758134734306`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1355878114514168`", ",", "0.9540107149963444`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1355878114514168, 0.9540107149963444}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935758534407015`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.909`"}],
  SequenceForm[" R^2: ", 0.909],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935758534407015`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.66612117`8.752219484406767"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.66612117`8.752219484406767],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935758534407015`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935758534407015`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593575853456301*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "0.27846049`8.373429831236178"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.27846049`8.373429831236178],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935758701483307`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935758701639304`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593576045461438*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357622297415*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13366189712702878`", ",", "0.9546707802340643`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13366189712702878`, 0.9546707802340643}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593576258869813*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.906`"}],
  SequenceForm[" R^2: ", 0.906],
  Editable->False]], "Print",
 CellChangeTimes->{3.593576258869813*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.47869137833333333333333333333333333333`9.740153540419147"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.47869137833333333333333333333333333333`9.740153540419147],
  Editable->False]], "Print",
 CellChangeTimes->{3.593576258885413*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.593576258885413*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593576258885413*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.31382055166666666666666666666666666667`8.42534762973284"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.31382055166666666666666666666666666667`8.42534762973284],
  Editable->False]], "Print",
 CellChangeTimes->{3.593576277699046*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935762777146463`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935770799616556`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935778854690704`*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935780483801565`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05702967803338749`", ",", "0.05816619265288064`", ",", 
   "1.0199284768682682`", ",", "0.057607011818867335`", ",", 
   "1.0101233919844657`", ",", "0.996504850670282`", ",", 
   "0.9962787567184928`", ",", "0.9963019718050907`", ",", 
   "0.9992352391232938`", ",", "0.9045267335749997`", ",", 
   "0.8827464980227003`", ",", "0.9395514170463152`", ",", 
   "6.600325249436788`"}], "}"}]], "Print",
 CellChangeTimes->{3.593578048707757*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.593578048738957*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.15580982857456593`", ",", "0.8178250721164231`", ",", 
   "0.11177825798868522`", ",", "0.0847689330323496`", ",", 
   "0.559022025866086`", ",", "0.27188782056069966`", ",", 
   "0.9172212234486498`", ",", "0.9405340714420299`"}], "}"}]], "Print",
 CellChangeTimes->{3.593578048770157*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.593578048832557*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "30.342463930391844`", ",", "54.77589732541789`", ",", "68.10552740011491`",
    ",", "81.30780807494254`", ",", "85.10643849382073`", ",", 
   "91.94430671268911`", ",", "94.6157484398987`", ",", "96.54583859046963`", 
   ",", "99.02821850766102`", ",", "0.771866872470481`", ",", "16.9959375`"}],
   "}"}]], "Print",
 CellChangeTimes->{3.593578048832557*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.29233020503818163`"}],
  SequenceForm[" MPC mean:", 0.29233020503818163`],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578048848157*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593578048863757*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.24682315988611858`", ",", "0.2499096600947358`", ",", 
   "0.2577659296448992`", ",", "0.2726370943740455`", ",", 
   "0.28279064788161523`", ",", "0.30646071237199357`", ",", 
   "0.3497514819558303`", ",", "0.3028502202771062`", ",", 
   "0.30618569094895853`", ",", "0.3304055321980166`"}], "}"}]], "Print",
 CellChangeTimes->{3.593578048863757*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935780488793573`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12160360872111675`", ",", "0.12416792725791992`", ",", 
   "0.12686911969978398`", ",", "0.1369222801767913`", ",", 
   "0.16517168129522253`", ",", "0.45512640642735747`", ",", 
   "0.5687986686228104`", ",", "0.34146128856292257`", ",", 
   "0.4167144222495591`", ",", "0.5329972323605363`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935780488793573`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.27218901516554617`", ",", "0.5222221736524886`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.27218901516554617`, 0.5222221736524886}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935780488949575`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1329666081012229`", ",", "0.9549126668168684`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1329666081012229, 0.9549126668168684}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935780488949575`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.905`"}],
  SequenceForm[" R^2: ", 0.905],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935780488949575`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.51135183333333333333333333333333333333`10.398655351999237"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.51135183333333333333333333333333333333`10.398655351999237],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935780489105573`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "47.86530407`10.608687070380599"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 47.86530407`10.608687070380599],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578048926157*^9}],

Cell[BoxData["\<\"==============================================\"\>"], \
"Print",
 CellChangeTimes->{3.593578048926157*^9}],

Cell[BoxData["\<\"Country: Slovakia\"\>"], "Print",
 CellChangeTimes->{3.593578048926157*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.593578048926157*^9}],

Cell[BoxData["\<\"Running rep agent model with permanent shocks ... \"\>"], \
"Print",
 CellChangeTimes->{3.593578049737359*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.59357805045496*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.05715444091405019`", ",", "0.058385571986183236`", ",", 
   "1.021540427173182`", ",", "0.0576648996942163`", ",", 
   "1.0089312181521248`", ",", "0.9965569155419628`", ",", 
   "0.9960939347223099`", ",", "0.9959747510529746`", ",", 
   "0.9991942023104342`", ",", "0.8989297032400659`", ",", 
   "0.8780178192286515`", ",", "0.9448781861526373`", ",", 
   "6.592915486195024`"}], "}"}]], "Print",
 CellChangeTimes->{3.59357805045496*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.59357805045496*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.13067142613605745`", ",", "0.8506136279992649`", ",", 
   "0.08122077816293251`", ",", "0.06823559685962945`", ",", 
   "0.5513797624004989`", ",", "0.2450038107307366`", ",", 
   "0.9316346292005178`", ",", "0.948813418235493`"}], "}"}]], "Print",
 CellChangeTimes->{3.59357805045496*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run rep agent model with permanent shocks \
(seconds): \"\>", "\[InvisibleSpace]", "0.7332013`7.015738224147161"}],
  SequenceForm[
  " Time spent to run rep agent model with permanent shocks (seconds): ", 
   0.7332013`7.015738224147161],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357805045496*^9}],

Cell[BoxData["\<\"=========================================\"\>"], "Print",
 CellChangeTimes->{3.59357805047056*^9}],

Cell[BoxData["\<\"Running \[Beta]-Dist model with time pref factors \
approximated by seven points... \"\>"], "Print",
 CellChangeTimes->{3.59357805047056*^9}],

Cell[BoxData["\<\"Params: \"\>"], "Print",
 CellChangeTimes->{3.593578051125761*^9}],

Cell[BoxData["0.9700599999999999`"], "Print",
 CellChangeTimes->{3.593578051125761*^9}],

Cell[BoxData["0.0013`"], "Print",
 CellChangeTimes->{3.593578051125761*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Iteration ", 1],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578051125761*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.593578051125761*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "4.08902718166666666666666666666666666667`9.5402862457371"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   4.08902718166666666666666666666666666667`9.5402862457371],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578296467392*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.11596749827716418`", ",", "0.9606479722118817`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.11596749827716418`, 0.9606479722118817}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935783350774603`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.935`"}],
  SequenceForm[" R^2: ", 0.935],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357833512426*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.64350113`8.737215562085927"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 0.64350113`8.737215562085927],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357833512426*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Iteration ", 2],
  Editable->False]], "Print",
 CellChangeTimes->{3.59357833512426*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.59357833512426*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", "1.30416229`9.043997886594598"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   1.30416229`9.043997886594598],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935784133583975`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13142464226758951`", ",", "0.9554087084770391`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13142464226758951`, 0.9554087084770391}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935784526860666`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.897`"}],
  SequenceForm[" R^2: ", 0.897],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935784527016664`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.65598115333333333333333333333333333333`8.745557610272824"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.65598115333333333333333333333333333333`8.745557610272824],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935784527016664`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "3"}],
  SequenceForm["Iteration ", 3],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935784527016664`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935784527016664`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.82446144666666666666666666666666666667`8.844836600262393"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.82446144666666666666666666666666666667`8.844836600262393],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935785021537533`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13603744502332693`", ",", "0.9538473822104669`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13603744502332693`, 0.9538473822104669}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935785425110245`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.9`"}],
  SequenceForm[" R^2: ", 0.9],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935785425110245`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67262118166666666666666666666666666667`8.756436787875936"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67262118166666666666666666666666666667`8.756436787875936],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935785425266247`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Iteration ", 4],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935785425266247`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935785425266247`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.50128088166666666666666666666666666667`8.628747389603069"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.50128088166666666666666666666666666667`8.628747389603069],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935785725878773`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.13862951093862746`", ",", "0.9529662985985552`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.13862951093862746`, 0.9529662985985552}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578613132348*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578613132348*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.67574118666666666666666666666666666667`8.758446638351108"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.67574118666666666666666666666666666667`8.758446638351108],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935786131479483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "5"}],
  SequenceForm["Iteration ", 5],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935786131479483`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935786131479483`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.34320060166666666666666666666666666667`8.464214288756763"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.34320060166666666666666666666666666667`8.464214288756763],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935786337243843`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.1400861215503055`", ",", "0.9524811209274159`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.1400861215503055, 0.9524811209274159}],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578677310861*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578677326461*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "0.72644127666666666666666666666666666667`8.789866761205625"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   0.72644127666666666666666666666666666667`8.789866761205625],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935786776540613`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "6"}],
  SequenceForm["Iteration ", 6],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935786776540613`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935786776696615`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.15600027333333333333333333333333333333`8.12179160751275"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.15600027333333333333333333333333333333`8.12179160751275],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935786866864777`*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.5935786866864777`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593578866507993*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593579045455908*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14268298867166584`", ",", "0.9516012257067812`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14268298867166584`, 0.9516012257067812}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935790829583735`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.897`"}],
  SequenceForm[" R^2: ", 0.897],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935790829583735`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "6.60375159833333333333333333333333333333`9.74845697702473"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   6.60375159833333333333333333333333333333`9.74845697702473],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935790829583735`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Iteration \"\>", "\[InvisibleSpace]", "7"}],
  SequenceForm["Iteration ", 7],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935790829583735`*^9}],

Cell[BoxData["\<\" Solving ind consumption function(s)...\"\>"], "Print",
 CellChangeTimes->{3.5935790829739733`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to solve consumption function(s) (minutes): \"\>", 
   "\[InvisibleSpace]", 
   "0.30056052833333333333333333333333333333`8.406598193686083"}],
  SequenceForm[
  " Time spent to solve consumption function(s) (minutes): ", 
   0.30056052833333333333333333333333333333`8.406598193686083],
  Editable->False]], "Print",
 CellChangeTimes->{3.593579100945205*^9}],

Cell[BoxData["\<\" Simulating period 1\"\>"], "Print",
 CellChangeTimes->{3.593579100945205*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "500"}],
  SequenceForm[" Simulating period ", 500],
  Editable->False]], "Print",
 CellChangeTimes->{3.593579899151807*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Simulating period \"\>", "\[InvisibleSpace]", "1000"}],
  SequenceForm[" Simulating period ", 1000],
  Editable->False]], "Print",
 CellChangeTimes->{3.593580701757617*^9}],

Cell[BoxData["\<\" \
sigmaLogY,sigmaLogC,sigmaCYRatio,sigmaLogI,sigmaIYRatio,corr(Y(t),Y(t-1)),\
corr(C(t),Y(t)),corr(C(t),Y(t-1)),corr(C(t),C(t-1)),corr(I(t),Y(t)),corr(I(t),\
Y(t-1)),corr(I(t),I(t-1)), KYRatio: \"\>"], "Print",
 CellChangeTimes->{3.5935808638575015`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.056994764875129285`", ",", "0.05821873108296493`", ",", 
   "1.0214750637276468`", ",", "0.05787845106708467`", ",", 
   "1.0155046905429204`", ",", "0.9964891704100118`", ",", 
   "0.9958499974438936`", ",", "0.9959728291050562`", ",", 
   "0.9992720521143701`", ",", "0.8939692144624084`", ",", 
   "0.8715686739135303`", ",", "0.9388443533559948`", ",", 
   "6.594132744460108`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935808638731017`*^9}],

Cell[BoxData["\<\" corr(dlC(t),dlC(t-1)), corr(dlC(t),dlY(t)), \
corr(dlC(t),dlY(t-1)), corr(dlC(t),dlY(t-2)), corr(dlC(t),r(t)), \
corr(dlC(t),r(t-1)), corr(d4lC(t),d4lY(t)), corr(d8lC(t),d8lY(t)): \"\>"], \
"Print",
 CellChangeTimes->{3.5935808642631025`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.16579641262559983`", ",", "0.8150440263652393`", ",", 
   "0.11635696763675002`", ",", "0.08664288193960602`", ",", 
   "0.5793682357025547`", ",", "0.28618190935977134`", ",", 
   "0.9135947256284787`", ",", "0.9363701486158307`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935808642631025`*^9}],

Cell[BoxData["\<\" Distribution of wealth (k) 1%, 5%, 10%, 20%, 25%, 40%, \
50%, 60%, 80%, Gini coeff, Frac between 0.5 and 1.5 * mean k (%): \"\>"], \
"Print",
 CellChangeTimes->{3.5935808642787023`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20.88081118327026`", ",", "43.46380977498909`", ",", "57.33601633540354`", 
   ",", "72.83885019778594`", ",", "77.83139056559996`", ",", 
   "87.61195507981793`", ",", "91.69140084674933`", ",", "94.66822274555614`",
    ",", "98.48448048137094`", ",", "0.6888830542746607`", ",", 
   "23.8411125`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935808642787023`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC mean:\"\>", "\[InvisibleSpace]", "0.2252611232421361`"}],
  SequenceForm[" MPC mean:", 0.2252611232421361],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935808642787023`*^9}],

Cell[BoxData["\<\" MPC by income top 1%, 10%, 20%, 40%, 60%; bottom 40%, 20%, \
3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.5935808642787023`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.18743254396485576`", ",", "0.19039157227720482`", ",", 
   "0.20014562054079665`", ",", "0.21604142493778633`", ",", 
   "0.2257978736048395`", ",", "0.2244554744559737`", ",", 
   "0.2521305641895576`", ",", "0.23263407858641338`", ",", 
   "0.2289381245595541`", ",", "0.2382866487340135`"}], "}"}]], "Print",
 CellChangeTimes->{3.593580864294302*^9}],

Cell[BoxData["\<\" MPC by wealth perm inc ratio top 1%, 10%, 20%, 40%, 60%; \
bottom 40%, 20%, 3/4, 1/2, 1/4: \"\>"], "Print",
 CellChangeTimes->{3.593580864309902*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.12414927283393096`", ",", "0.12621546583348675`", ",", 
   "0.12810790090963176`", ",", "0.13275036224061965`", ",", 
   "0.14216183836768392`", ",", "0.33840480046488386`", ",", 
   "0.4473100175242827`", ",", "0.2554765187331296`", ",", 
   "0.30685975144820354`", ",", "0.41027779773582673`"}], "}"}]], "Print",
 CellChangeTimes->{3.5935808643255024`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" MPC employed, unemployed: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.21206670724354448`", ",", "0.3853060020777177`"}], "}"}]}],
  SequenceForm[
  " MPC employed, unemployed: ", {0.21206670724354448`, 0.3853060020777177}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935808643879023`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Estimates of agg process: \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"0.14238406018509175`", ",", "0.9517018420755049`"}], "}"}]}],
  SequenceForm[
  " Estimates of agg process: ", {0.14238406018509175`, 0.9517018420755049}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935808644035025`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" R^2: \"\>", "\[InvisibleSpace]", "0.899`"}],
  SequenceForm[" R^2: ", 0.899],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935808644035025`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to simulate (minutes): \"\>", "\[InvisibleSpace]", 
   "29.38213160666666666666666666666666666667`10.396749547887424"}],
  SequenceForm[
  " Time spent to simulate (minutes): ", 
   29.38213160666666666666666666666666666667`10.396749547887424],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935808644191027`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model with time pref factors \
approximated by seven points (minutes): \"\>", "\[InvisibleSpace]", 
   "46.89030235833333333333333333333333333333`10.599749281383835"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model with time pref factors approximated \
by seven points (minutes): ", 
   46.89030235833333333333333333333333333333`10.599749281383835],
  Editable->False]], "Print",
 CellChangeTimes->{3.5935808644347024`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\" Time spent to run \[Beta]-Dist model across Euroepan \
economies (minutes):  \"\>", "\[InvisibleSpace]", 
   "812.78289373833333333333333333333333333333`11.838640802858691"}],
  SequenceForm[
  " Time spent to run \[Beta]-Dist model across Euroepan economies (minutes): \
 ", 812.78289373833333333333333333333333333333`11.838640802858691],
  Editable->False]], "Print",
 CellChangeTimes->{3.593580864450302*^9}],

Cell[BoxData["\<\"============================================================\
===============\"\>"], "Print",
 CellChangeTimes->{3.593580864450302*^9}],

Cell[BoxData["\<\"Generate tables \"\>"], "Print",
 CellChangeTimes->{3.5935808644815025`*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1044, 504},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 63019, 2915, 39981, "Input"],
Cell[CellGroupData[{
Cell[63623, 2941, 135, 2, 23, "Print"],
Cell[63761, 2945, 155, 2, 23, "Print"],
Cell[63919, 2949, 106, 1, 23, "Print"],
Cell[64028, 2952, 141, 2, 23, "Print"],
Cell[64172, 2956, 101, 1, 23, "Print"],
Cell[64276, 2959, 179, 2, 23, "Print"],
Cell[64458, 2963, 155, 2, 23, "Print"],
Cell[64616, 2967, 148, 2, 23, "Print"],
Cell[64767, 2971, 289, 6, 23, "Print"]
}, Open  ]],
Cell[65071, 2980, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[65740, 2996, 223, 5, 23, "Print"],
Cell[65966, 3003, 285, 6, 23, "Print"],
Cell[66254, 3011, 135, 2, 23, "Print"],
Cell[66392, 3015, 140, 2, 23, "Print"],
Cell[66535, 3019, 284, 6, 23, "Print"],
Cell[66822, 3027, 148, 2, 23, "Print"]
}, Open  ]],
Cell[66985, 3032, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[67654, 3048, 221, 5, 23, "Print"],
Cell[67878, 3055, 108, 1, 23, "Print"],
Cell[67989, 3058, 273, 6, 23, "Print"],
Cell[68265, 3066, 423, 5, 62, "Print"],
Cell[68691, 3073, 588, 11, 43, "Print"],
Cell[69282, 3086, 287, 6, 23, "Print"],
Cell[69572, 3094, 151, 2, 23, "Print"]
}, Open  ]],
Cell[69738, 3099, 642, 12, 48, "Message"],
Cell[70383, 3113, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[70943, 3128, 221, 5, 23, "Print"],
Cell[71167, 3135, 108, 1, 23, "Print"],
Cell[71278, 3138, 253, 6, 23, "Print"],
Cell[71534, 3146, 421, 5, 62, "Print"],
Cell[71958, 3153, 582, 10, 43, "Print"],
Cell[72543, 3165, 318, 7, 23, "Print"],
Cell[72864, 3174, 149, 2, 23, "Print"],
Cell[73016, 3178, 221, 5, 23, "Print"],
Cell[73240, 3185, 110, 1, 23, "Print"],
Cell[73353, 3188, 275, 6, 23, "Print"],
Cell[73631, 3196, 421, 5, 62, "Print"],
Cell[74055, 3203, 582, 10, 43, "Print"],
Cell[74640, 3215, 88, 1, 23, "Print"],
Cell[74731, 3218, 421, 5, 62, "Print"],
Cell[75155, 3225, 581, 10, 43, "Print"],
Cell[75739, 3237, 1023, 20, 43, "Print"],
Cell[76765, 3259, 443, 9, 23, "Print"],
Cell[77211, 3270, 141, 2, 23, "Print"],
Cell[77355, 3274, 100, 1, 23, "Print"],
Cell[77458, 3277, 177, 2, 23, "Print"],
Cell[77638, 3281, 153, 2, 23, "Print"],
Cell[77794, 3285, 148, 2, 23, "Print"],
Cell[77945, 3289, 289, 6, 23, "Print"]
}, Open  ]],
Cell[78249, 3298, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[78918, 3314, 223, 5, 23, "Print"],
Cell[79144, 3321, 283, 6, 23, "Print"],
Cell[79430, 3329, 137, 2, 23, "Print"],
Cell[79570, 3333, 142, 2, 23, "Print"],
Cell[79715, 3337, 287, 6, 23, "Print"],
Cell[80005, 3345, 151, 2, 23, "Print"]
}, Open  ]],
Cell[80171, 3350, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[80838, 3366, 221, 5, 23, "Print"],
Cell[81062, 3373, 108, 1, 23, "Print"],
Cell[81173, 3376, 275, 6, 23, "Print"],
Cell[81451, 3384, 423, 5, 62, "Print"],
Cell[81877, 3391, 586, 10, 43, "Print"],
Cell[82466, 3403, 287, 6, 23, "Print"],
Cell[82756, 3411, 151, 2, 23, "Print"]
}, Open  ]],
Cell[82922, 3416, 644, 12, 48, "Message"],
Cell[83569, 3430, 537, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[84131, 3445, 223, 5, 23, "Print"],
Cell[84357, 3452, 108, 1, 23, "Print"],
Cell[84468, 3455, 275, 6, 23, "Print"],
Cell[84746, 3463, 420, 5, 62, "Print"],
Cell[85169, 3470, 583, 10, 43, "Print"],
Cell[85755, 3482, 284, 6, 23, "Print"],
Cell[86042, 3490, 148, 2, 23, "Print"],
Cell[86193, 3494, 221, 5, 23, "Print"],
Cell[86417, 3501, 108, 1, 23, "Print"],
Cell[86528, 3504, 273, 6, 23, "Print"],
Cell[86804, 3512, 421, 5, 62, "Print"],
Cell[87228, 3519, 584, 10, 43, "Print"],
Cell[87815, 3531, 88, 1, 23, "Print"],
Cell[87906, 3534, 421, 5, 62, "Print"],
Cell[88330, 3541, 581, 10, 43, "Print"],
Cell[88914, 3553, 1075, 20, 43, "Print"],
Cell[89992, 3575, 443, 9, 23, "Print"],
Cell[90438, 3586, 139, 2, 23, "Print"],
Cell[90580, 3590, 101, 1, 23, "Print"],
Cell[90684, 3593, 177, 2, 23, "Print"],
Cell[90864, 3597, 153, 2, 23, "Print"],
Cell[91020, 3601, 145, 2, 23, "Print"],
Cell[91168, 3605, 286, 6, 23, "Print"]
}, Open  ]],
Cell[91469, 3614, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[92138, 3630, 223, 5, 23, "Print"],
Cell[92364, 3637, 285, 6, 23, "Print"],
Cell[92652, 3645, 134, 2, 23, "Print"],
Cell[92789, 3649, 139, 2, 23, "Print"],
Cell[92931, 3653, 287, 6, 23, "Print"],
Cell[93221, 3661, 151, 2, 23, "Print"]
}, Open  ]],
Cell[93387, 3666, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[94056, 3682, 223, 5, 23, "Print"],
Cell[94282, 3689, 110, 1, 23, "Print"],
Cell[94395, 3692, 275, 6, 23, "Print"],
Cell[94673, 3700, 423, 5, 62, "Print"],
Cell[95099, 3707, 583, 10, 43, "Print"],
Cell[95685, 3719, 287, 6, 23, "Print"],
Cell[95975, 3727, 151, 2, 23, "Print"]
}, Open  ]],
Cell[96141, 3732, 644, 12, 48, "Message"],
Cell[96788, 3746, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[97348, 3761, 221, 5, 23, "Print"],
Cell[97572, 3768, 110, 1, 23, "Print"],
Cell[97685, 3771, 255, 6, 23, "Print"],
Cell[97943, 3779, 421, 5, 62, "Print"],
Cell[98367, 3786, 585, 10, 43, "Print"],
Cell[98955, 3798, 318, 7, 23, "Print"],
Cell[99276, 3807, 149, 2, 23, "Print"],
Cell[99428, 3811, 221, 5, 23, "Print"],
Cell[99652, 3818, 110, 1, 23, "Print"],
Cell[99765, 3821, 255, 6, 23, "Print"],
Cell[100023, 3829, 423, 5, 62, "Print"],
Cell[100449, 3836, 583, 10, 43, "Print"],
Cell[101035, 3848, 285, 6, 23, "Print"],
Cell[101323, 3856, 151, 2, 23, "Print"],
Cell[101477, 3860, 307, 7, 23, "Print"],
Cell[101787, 3869, 108, 1, 23, "Print"],
Cell[101898, 3872, 275, 6, 23, "Print"],
Cell[102176, 3880, 423, 5, 62, "Print"],
Cell[102602, 3887, 586, 10, 43, "Print"],
Cell[103191, 3899, 287, 6, 23, "Print"],
Cell[103481, 3907, 151, 2, 23, "Print"],
Cell[103635, 3911, 307, 7, 23, "Print"],
Cell[103945, 3920, 110, 1, 23, "Print"],
Cell[104058, 3923, 273, 6, 23, "Print"],
Cell[104334, 3931, 423, 5, 62, "Print"],
Cell[104760, 3938, 583, 10, 43, "Print"],
Cell[105346, 3950, 88, 1, 23, "Print"],
Cell[105437, 3953, 421, 5, 62, "Print"],
Cell[105861, 3960, 579, 10, 43, "Print"],
Cell[106443, 3972, 1079, 20, 43, "Print"],
Cell[107525, 3994, 445, 9, 23, "Print"],
Cell[107973, 4005, 139, 2, 23, "Print"],
Cell[108115, 4009, 106, 1, 23, "Print"],
Cell[108224, 4012, 179, 2, 23, "Print"],
Cell[108406, 4016, 155, 2, 23, "Print"],
Cell[108564, 4020, 146, 2, 23, "Print"],
Cell[108713, 4024, 289, 6, 23, "Print"],
Cell[109005, 4032, 307, 7, 23, "Print"],
Cell[109315, 4041, 369, 8, 23, "Print"],
Cell[109687, 4051, 135, 2, 23, "Print"],
Cell[109825, 4055, 140, 2, 23, "Print"],
Cell[109968, 4059, 285, 6, 23, "Print"],
Cell[110256, 4067, 149, 2, 23, "Print"]
}, Open  ]],
Cell[110420, 4072, 647, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[111092, 4088, 307, 7, 23, "Print"],
Cell[111402, 4097, 110, 1, 23, "Print"],
Cell[111515, 4100, 251, 6, 23, "Print"],
Cell[111769, 4108, 421, 5, 62, "Print"],
Cell[112193, 4115, 586, 11, 43, "Print"],
Cell[112782, 4128, 285, 6, 23, "Print"],
Cell[113070, 4136, 149, 2, 23, "Print"],
Cell[113222, 4140, 307, 7, 23, "Print"],
Cell[113532, 4149, 108, 1, 23, "Print"],
Cell[113643, 4152, 275, 6, 23, "Print"],
Cell[113921, 4160, 423, 5, 62, "Print"],
Cell[114347, 4167, 588, 11, 43, "Print"],
Cell[114938, 4180, 320, 7, 23, "Print"],
Cell[115261, 4189, 151, 2, 23, "Print"],
Cell[115415, 4193, 305, 7, 23, "Print"],
Cell[115723, 4202, 108, 1, 23, "Print"],
Cell[115834, 4205, 273, 6, 23, "Print"],
Cell[116110, 4213, 421, 5, 62, "Print"],
Cell[116534, 4220, 580, 10, 43, "Print"],
Cell[117117, 4232, 283, 6, 23, "Print"],
Cell[117403, 4240, 151, 2, 23, "Print"],
Cell[117557, 4244, 307, 7, 23, "Print"],
Cell[117867, 4253, 108, 1, 23, "Print"],
Cell[117978, 4256, 273, 6, 23, "Print"],
Cell[118254, 4264, 423, 5, 62, "Print"],
Cell[118680, 4271, 581, 10, 43, "Print"],
Cell[119264, 4283, 287, 6, 23, "Print"],
Cell[119554, 4291, 151, 2, 23, "Print"],
Cell[119708, 4295, 305, 7, 23, "Print"],
Cell[120016, 4304, 108, 1, 23, "Print"],
Cell[120127, 4307, 275, 6, 23, "Print"],
Cell[120405, 4315, 421, 5, 62, "Print"],
Cell[120829, 4322, 581, 10, 43, "Print"],
Cell[121413, 4334, 321, 7, 23, "Print"],
Cell[121737, 4343, 151, 2, 23, "Print"],
Cell[121891, 4347, 307, 7, 23, "Print"],
Cell[122201, 4356, 110, 1, 23, "Print"],
Cell[122314, 4359, 271, 6, 23, "Print"],
Cell[122588, 4367, 421, 5, 62, "Print"],
Cell[123012, 4374, 580, 10, 43, "Print"],
Cell[123595, 4386, 319, 7, 23, "Print"],
Cell[123917, 4395, 151, 2, 23, "Print"],
Cell[124071, 4399, 307, 7, 23, "Print"],
Cell[124381, 4408, 110, 1, 23, "Print"],
Cell[124494, 4411, 331, 8, 23, "Print"],
Cell[124828, 4421, 421, 5, 62, "Print"],
Cell[125252, 4428, 579, 10, 43, "Print"],
Cell[125834, 4440, 319, 7, 23, "Print"],
Cell[126156, 4449, 149, 2, 23, "Print"],
Cell[126308, 4453, 305, 7, 23, "Print"],
Cell[126616, 4462, 108, 1, 23, "Print"],
Cell[126727, 4465, 273, 6, 23, "Print"],
Cell[127003, 4473, 423, 5, 62, "Print"],
Cell[127429, 4480, 584, 10, 43, "Print"],
Cell[128016, 4492, 88, 1, 23, "Print"],
Cell[128107, 4495, 423, 5, 62, "Print"],
Cell[128533, 4502, 582, 10, 43, "Print"],
Cell[129118, 4514, 1181, 23, 62, "Print"],
Cell[130302, 4539, 443, 9, 23, "Print"],
Cell[130748, 4550, 138, 2, 23, "Print"],
Cell[130889, 4554, 91, 1, 23, "Print"],
Cell[130983, 4557, 179, 2, 23, "Print"],
Cell[131165, 4561, 155, 2, 23, "Print"],
Cell[131323, 4565, 148, 2, 23, "Print"],
Cell[131474, 4569, 289, 6, 23, "Print"]
}, Open  ]],
Cell[131778, 4578, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[132447, 4594, 221, 5, 23, "Print"],
Cell[132671, 4601, 283, 6, 23, "Print"],
Cell[132957, 4609, 137, 2, 23, "Print"],
Cell[133097, 4613, 142, 2, 23, "Print"],
Cell[133242, 4617, 284, 6, 23, "Print"],
Cell[133529, 4625, 148, 2, 23, "Print"]
}, Open  ]],
Cell[133692, 4630, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[134359, 4646, 221, 5, 23, "Print"],
Cell[134583, 4653, 108, 1, 23, "Print"],
Cell[134694, 4656, 275, 6, 23, "Print"],
Cell[134972, 4664, 421, 5, 62, "Print"],
Cell[135396, 4671, 579, 10, 43, "Print"],
Cell[135978, 4683, 285, 6, 23, "Print"],
Cell[136266, 4691, 149, 2, 23, "Print"]
}, Open  ]],
Cell[136430, 4696, 642, 12, 48, "Message"],
Cell[137075, 4710, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[137635, 4725, 221, 5, 23, "Print"],
Cell[137859, 4732, 108, 1, 23, "Print"],
Cell[137970, 4735, 255, 6, 23, "Print"],
Cell[138228, 4743, 421, 5, 62, "Print"],
Cell[138652, 4750, 583, 10, 43, "Print"],
Cell[139238, 4762, 318, 7, 23, "Print"],
Cell[139559, 4771, 149, 2, 23, "Print"],
Cell[139711, 4775, 220, 5, 23, "Print"],
Cell[139934, 4782, 108, 1, 23, "Print"],
Cell[140045, 4785, 248, 6, 23, "Print"],
Cell[140296, 4793, 423, 5, 62, "Print"],
Cell[140722, 4800, 581, 10, 43, "Print"],
Cell[141306, 4812, 285, 6, 23, "Print"],
Cell[141594, 4820, 151, 2, 23, "Print"],
Cell[141748, 4824, 221, 5, 23, "Print"],
Cell[141972, 4831, 108, 1, 23, "Print"],
Cell[142083, 4834, 253, 6, 23, "Print"],
Cell[142339, 4842, 423, 5, 62, "Print"],
Cell[142765, 4849, 578, 10, 43, "Print"],
Cell[143346, 4861, 285, 6, 23, "Print"],
Cell[143634, 4869, 149, 2, 23, "Print"],
Cell[143786, 4873, 305, 7, 23, "Print"],
Cell[144094, 4882, 110, 1, 23, "Print"],
Cell[144207, 4885, 275, 6, 23, "Print"],
Cell[144485, 4893, 421, 5, 62, "Print"],
Cell[144909, 4900, 588, 11, 43, "Print"],
Cell[145500, 4913, 319, 7, 23, "Print"],
Cell[145822, 4922, 149, 2, 23, "Print"],
Cell[145974, 4926, 307, 7, 23, "Print"],
Cell[146284, 4935, 110, 1, 23, "Print"],
Cell[146397, 4938, 275, 6, 23, "Print"],
Cell[146675, 4946, 423, 5, 62, "Print"],
Cell[147101, 4953, 582, 10, 43, "Print"],
Cell[147686, 4965, 321, 7, 23, "Print"],
Cell[148010, 4974, 151, 2, 23, "Print"],
Cell[148164, 4978, 307, 7, 23, "Print"],
Cell[148474, 4987, 108, 1, 23, "Print"],
Cell[148585, 4990, 275, 6, 23, "Print"],
Cell[148863, 4998, 420, 5, 62, "Print"],
Cell[149286, 5005, 580, 10, 43, "Print"],
Cell[149869, 5017, 318, 7, 23, "Print"],
Cell[150190, 5026, 148, 2, 23, "Print"],
Cell[150341, 5030, 303, 7, 23, "Print"],
Cell[150647, 5039, 107, 1, 23, "Print"],
Cell[150757, 5042, 327, 7, 23, "Print"],
Cell[151087, 5051, 423, 5, 62, "Print"],
Cell[151513, 5058, 583, 10, 43, "Print"],
Cell[152099, 5070, 88, 1, 23, "Print"],
Cell[152190, 5073, 423, 5, 62, "Print"],
Cell[152616, 5080, 583, 10, 43, "Print"],
Cell[153202, 5092, 1126, 22, 43, "Print"],
Cell[154331, 5116, 445, 9, 23, "Print"],
Cell[154779, 5127, 141, 2, 23, "Print"],
Cell[154923, 5131, 92, 1, 23, "Print"],
Cell[155018, 5134, 179, 2, 23, "Print"],
Cell[155200, 5138, 155, 2, 23, "Print"],
Cell[155358, 5142, 148, 2, 23, "Print"],
Cell[155509, 5146, 289, 6, 23, "Print"]
}, Open  ]],
Cell[155813, 5155, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[156480, 5171, 221, 5, 23, "Print"],
Cell[156704, 5178, 283, 6, 23, "Print"],
Cell[156990, 5186, 137, 2, 23, "Print"],
Cell[157130, 5190, 142, 2, 23, "Print"],
Cell[157275, 5194, 284, 6, 23, "Print"],
Cell[157562, 5202, 148, 2, 23, "Print"]
}, Open  ]],
Cell[157725, 5207, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[158392, 5223, 221, 5, 23, "Print"],
Cell[158616, 5230, 110, 1, 23, "Print"],
Cell[158729, 5233, 273, 6, 23, "Print"],
Cell[159005, 5241, 423, 5, 62, "Print"],
Cell[159431, 5248, 585, 10, 43, "Print"],
Cell[160019, 5260, 287, 6, 23, "Print"],
Cell[160309, 5268, 151, 2, 23, "Print"]
}, Open  ]],
Cell[160475, 5273, 642, 12, 48, "Message"],
Cell[161120, 5287, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[161680, 5302, 221, 5, 23, "Print"],
Cell[161904, 5309, 108, 1, 23, "Print"],
Cell[162015, 5312, 253, 6, 23, "Print"],
Cell[162271, 5320, 421, 5, 62, "Print"],
Cell[162695, 5327, 583, 10, 43, "Print"],
Cell[163281, 5339, 285, 6, 23, "Print"],
Cell[163569, 5347, 149, 2, 23, "Print"],
Cell[163721, 5351, 223, 5, 23, "Print"],
Cell[163947, 5358, 110, 1, 23, "Print"],
Cell[164060, 5361, 273, 6, 23, "Print"],
Cell[164336, 5369, 421, 5, 62, "Print"],
Cell[164760, 5376, 578, 10, 43, "Print"],
Cell[165341, 5388, 318, 7, 23, "Print"],
Cell[165662, 5397, 149, 2, 23, "Print"],
Cell[165814, 5401, 223, 5, 23, "Print"],
Cell[166040, 5408, 110, 1, 23, "Print"],
Cell[166153, 5411, 271, 6, 23, "Print"],
Cell[166427, 5419, 421, 5, 62, "Print"],
Cell[166851, 5426, 582, 10, 43, "Print"],
Cell[167436, 5438, 318, 7, 23, "Print"],
Cell[167757, 5447, 149, 2, 23, "Print"],
Cell[167909, 5451, 221, 5, 23, "Print"],
Cell[168133, 5458, 110, 1, 23, "Print"],
Cell[168246, 5461, 275, 6, 23, "Print"],
Cell[168524, 5469, 421, 5, 62, "Print"],
Cell[168948, 5476, 583, 10, 43, "Print"],
Cell[169534, 5488, 88, 1, 23, "Print"],
Cell[169625, 5491, 423, 5, 62, "Print"],
Cell[170051, 5498, 587, 10, 43, "Print"],
Cell[170641, 5510, 1122, 22, 43, "Print"],
Cell[171766, 5534, 445, 9, 23, "Print"],
Cell[172214, 5545, 141, 2, 23, "Print"],
Cell[172358, 5549, 91, 1, 23, "Print"],
Cell[172452, 5552, 177, 2, 23, "Print"],
Cell[172632, 5556, 153, 2, 23, "Print"],
Cell[172788, 5560, 148, 2, 23, "Print"],
Cell[172939, 5564, 289, 6, 23, "Print"]
}, Open  ]],
Cell[173243, 5573, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[173910, 5589, 221, 5, 23, "Print"],
Cell[174134, 5596, 282, 6, 23, "Print"],
Cell[174419, 5604, 137, 2, 23, "Print"],
Cell[174559, 5608, 142, 2, 23, "Print"],
Cell[174704, 5612, 285, 6, 23, "Print"],
Cell[174992, 5620, 149, 2, 23, "Print"]
}, Open  ]],
Cell[175156, 5625, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[175823, 5641, 221, 5, 23, "Print"],
Cell[176047, 5648, 110, 1, 23, "Print"],
Cell[176160, 5651, 273, 6, 23, "Print"],
Cell[176436, 5659, 420, 5, 62, "Print"],
Cell[176859, 5666, 579, 10, 43, "Print"],
Cell[177441, 5678, 284, 6, 23, "Print"],
Cell[177728, 5686, 148, 2, 23, "Print"]
}, Open  ]],
Cell[177891, 5691, 642, 12, 48, "Message"],
Cell[178536, 5705, 537, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[179098, 5720, 223, 5, 23, "Print"],
Cell[179324, 5727, 110, 1, 23, "Print"],
Cell[179437, 5730, 255, 6, 23, "Print"],
Cell[179695, 5738, 423, 5, 62, "Print"],
Cell[180121, 5745, 583, 10, 43, "Print"],
Cell[180707, 5757, 320, 7, 23, "Print"],
Cell[181030, 5766, 151, 2, 23, "Print"],
Cell[181184, 5770, 221, 5, 23, "Print"],
Cell[181408, 5777, 110, 1, 23, "Print"],
Cell[181521, 5780, 252, 6, 23, "Print"],
Cell[181776, 5788, 421, 5, 62, "Print"],
Cell[182200, 5795, 585, 10, 43, "Print"],
Cell[182788, 5807, 285, 6, 23, "Print"],
Cell[183076, 5815, 151, 2, 23, "Print"],
Cell[183230, 5819, 221, 5, 23, "Print"],
Cell[183454, 5826, 108, 1, 23, "Print"],
Cell[183565, 5829, 251, 6, 23, "Print"],
Cell[183819, 5837, 421, 5, 62, "Print"],
Cell[184243, 5844, 582, 10, 43, "Print"],
Cell[184828, 5856, 88, 1, 23, "Print"],
Cell[184919, 5859, 423, 5, 62, "Print"],
Cell[185345, 5866, 592, 11, 43, "Print"],
Cell[185940, 5879, 1052, 20, 43, "Print"],
Cell[186995, 5901, 445, 9, 23, "Print"],
Cell[187443, 5912, 141, 2, 23, "Print"],
Cell[187587, 5916, 94, 1, 23, "Print"],
Cell[187684, 5919, 179, 2, 23, "Print"],
Cell[187866, 5923, 155, 2, 23, "Print"],
Cell[188024, 5927, 146, 2, 23, "Print"],
Cell[188173, 5931, 287, 6, 23, "Print"]
}, Open  ]],
Cell[188475, 5940, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[189142, 5956, 221, 5, 23, "Print"],
Cell[189366, 5963, 283, 6, 23, "Print"],
Cell[189652, 5971, 137, 2, 23, "Print"],
Cell[189792, 5975, 142, 2, 23, "Print"],
Cell[189937, 5979, 287, 6, 23, "Print"],
Cell[190227, 5987, 151, 2, 23, "Print"]
}, Open  ]],
Cell[190393, 5992, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[191062, 6008, 223, 5, 23, "Print"],
Cell[191288, 6015, 110, 1, 23, "Print"],
Cell[191401, 6018, 275, 6, 23, "Print"],
Cell[191679, 6026, 420, 5, 62, "Print"],
Cell[192102, 6033, 576, 10, 43, "Print"],
Cell[192681, 6045, 284, 6, 23, "Print"],
Cell[192968, 6053, 148, 2, 23, "Print"]
}, Open  ]],
Cell[193131, 6058, 642, 12, 48, "Message"],
Cell[193776, 6072, 537, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[194338, 6087, 223, 5, 23, "Print"],
Cell[194564, 6094, 110, 1, 23, "Print"],
Cell[194677, 6097, 255, 6, 23, "Print"],
Cell[194935, 6105, 423, 5, 62, "Print"],
Cell[195361, 6112, 584, 10, 43, "Print"],
Cell[195948, 6124, 320, 7, 23, "Print"],
Cell[196271, 6133, 151, 2, 23, "Print"],
Cell[196425, 6137, 223, 5, 23, "Print"],
Cell[196651, 6144, 108, 1, 23, "Print"],
Cell[196762, 6147, 253, 6, 23, "Print"],
Cell[197018, 6155, 423, 5, 62, "Print"],
Cell[197444, 6162, 587, 11, 43, "Print"],
Cell[198034, 6175, 285, 6, 23, "Print"],
Cell[198322, 6183, 151, 2, 23, "Print"],
Cell[198476, 6187, 307, 7, 23, "Print"],
Cell[198786, 6196, 108, 1, 23, "Print"],
Cell[198897, 6199, 273, 6, 23, "Print"],
Cell[199173, 6207, 421, 5, 62, "Print"],
Cell[199597, 6214, 582, 10, 43, "Print"],
Cell[200182, 6226, 285, 6, 23, "Print"],
Cell[200470, 6234, 149, 2, 23, "Print"],
Cell[200622, 6238, 307, 7, 23, "Print"],
Cell[200932, 6247, 110, 1, 23, "Print"],
Cell[201045, 6250, 273, 6, 23, "Print"],
Cell[201321, 6258, 423, 5, 62, "Print"],
Cell[201747, 6265, 584, 10, 43, "Print"],
Cell[202334, 6277, 321, 7, 23, "Print"],
Cell[202658, 6286, 151, 2, 23, "Print"],
Cell[202812, 6290, 307, 7, 23, "Print"],
Cell[203122, 6299, 110, 1, 23, "Print"],
Cell[203235, 6302, 275, 6, 23, "Print"],
Cell[203513, 6310, 421, 5, 62, "Print"],
Cell[203937, 6317, 584, 10, 43, "Print"],
Cell[204524, 6329, 319, 7, 23, "Print"],
Cell[204846, 6338, 149, 2, 23, "Print"],
Cell[204998, 6342, 304, 7, 23, "Print"],
Cell[205305, 6351, 110, 1, 23, "Print"],
Cell[205418, 6354, 330, 8, 23, "Print"],
Cell[205751, 6364, 421, 5, 62, "Print"],
Cell[206175, 6371, 581, 10, 43, "Print"],
Cell[206759, 6383, 319, 7, 23, "Print"],
Cell[207081, 6392, 149, 2, 23, "Print"],
Cell[207233, 6396, 305, 7, 23, "Print"],
Cell[207541, 6405, 110, 1, 23, "Print"],
Cell[207654, 6408, 273, 6, 23, "Print"],
Cell[207930, 6416, 423, 5, 62, "Print"],
Cell[208356, 6423, 584, 10, 43, "Print"],
Cell[208943, 6435, 321, 7, 23, "Print"],
Cell[209267, 6444, 149, 2, 23, "Print"],
Cell[209419, 6448, 305, 7, 23, "Print"],
Cell[209727, 6457, 108, 1, 23, "Print"],
Cell[209838, 6460, 273, 6, 23, "Print"],
Cell[210114, 6468, 421, 5, 62, "Print"],
Cell[210538, 6475, 582, 10, 43, "Print"],
Cell[211123, 6487, 319, 7, 23, "Print"],
Cell[211445, 6496, 149, 2, 23, "Print"],
Cell[211597, 6500, 305, 7, 23, "Print"],
Cell[211905, 6509, 110, 1, 23, "Print"],
Cell[212018, 6512, 275, 6, 23, "Print"],
Cell[212296, 6520, 423, 5, 62, "Print"],
Cell[212722, 6527, 587, 11, 43, "Print"],
Cell[213312, 6540, 321, 7, 23, "Print"],
Cell[213636, 6549, 151, 2, 23, "Print"],
Cell[213790, 6553, 307, 7, 23, "Print"],
Cell[214100, 6562, 110, 1, 23, "Print"],
Cell[214213, 6565, 275, 6, 23, "Print"],
Cell[214491, 6573, 423, 5, 62, "Print"],
Cell[214917, 6580, 583, 10, 43, "Print"],
Cell[215503, 6592, 321, 7, 23, "Print"],
Cell[215827, 6601, 151, 2, 23, "Print"],
Cell[215981, 6605, 305, 7, 23, "Print"],
Cell[216289, 6614, 110, 1, 23, "Print"],
Cell[216402, 6617, 273, 6, 23, "Print"],
Cell[216678, 6625, 421, 5, 62, "Print"],
Cell[217102, 6632, 585, 10, 43, "Print"],
Cell[217690, 6644, 319, 7, 23, "Print"],
Cell[218012, 6653, 149, 2, 23, "Print"],
Cell[218164, 6657, 307, 7, 23, "Print"],
Cell[218474, 6666, 110, 1, 23, "Print"],
Cell[218587, 6669, 271, 6, 23, "Print"],
Cell[218861, 6677, 423, 5, 62, "Print"],
Cell[219287, 6684, 587, 10, 43, "Print"],
Cell[219877, 6696, 321, 7, 23, "Print"],
Cell[220201, 6705, 151, 2, 23, "Print"],
Cell[220355, 6709, 305, 7, 23, "Print"],
Cell[220663, 6718, 110, 1, 23, "Print"],
Cell[220776, 6721, 275, 6, 23, "Print"],
Cell[221054, 6729, 421, 5, 62, "Print"],
Cell[221478, 6736, 585, 10, 43, "Print"],
Cell[222066, 6748, 86, 1, 23, "Print"],
Cell[222155, 6751, 421, 5, 62, "Print"],
Cell[222579, 6758, 581, 10, 43, "Print"],
Cell[223163, 6770, 1115, 22, 43, "Print"],
Cell[224281, 6794, 443, 9, 23, "Print"],
Cell[224727, 6805, 141, 2, 23, "Print"],
Cell[224871, 6809, 90, 1, 23, "Print"],
Cell[224964, 6812, 179, 2, 23, "Print"],
Cell[225146, 6816, 155, 2, 23, "Print"],
Cell[225304, 6820, 146, 2, 23, "Print"],
Cell[225453, 6824, 287, 6, 23, "Print"]
}, Open  ]],
Cell[225755, 6833, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[226424, 6849, 221, 5, 23, "Print"],
Cell[226648, 6856, 285, 6, 23, "Print"],
Cell[226936, 6864, 137, 2, 23, "Print"],
Cell[227076, 6868, 142, 2, 23, "Print"],
Cell[227221, 6872, 287, 6, 23, "Print"],
Cell[227511, 6880, 151, 2, 23, "Print"]
}, Open  ]],
Cell[227677, 6885, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[228346, 6901, 221, 5, 23, "Print"],
Cell[228570, 6908, 110, 1, 23, "Print"],
Cell[228683, 6911, 273, 6, 23, "Print"],
Cell[228959, 6919, 423, 5, 62, "Print"],
Cell[229385, 6926, 581, 10, 43, "Print"],
Cell[229969, 6938, 287, 6, 23, "Print"],
Cell[230259, 6946, 151, 2, 23, "Print"]
}, Open  ]],
Cell[230425, 6951, 644, 12, 48, "Message"],
Cell[231072, 6965, 537, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[231634, 6980, 223, 5, 23, "Print"],
Cell[231860, 6987, 108, 1, 23, "Print"],
Cell[231971, 6990, 253, 6, 23, "Print"],
Cell[232227, 6998, 421, 5, 62, "Print"],
Cell[232651, 7005, 579, 10, 43, "Print"],
Cell[233233, 7017, 285, 6, 23, "Print"],
Cell[233521, 7025, 149, 2, 23, "Print"],
Cell[233673, 7029, 223, 5, 23, "Print"],
Cell[233899, 7036, 108, 1, 23, "Print"],
Cell[234010, 7039, 275, 6, 23, "Print"],
Cell[234288, 7047, 421, 5, 62, "Print"],
Cell[234712, 7054, 580, 10, 43, "Print"],
Cell[235295, 7066, 318, 7, 23, "Print"],
Cell[235616, 7075, 149, 2, 23, "Print"],
Cell[235768, 7079, 223, 5, 23, "Print"],
Cell[235994, 7086, 110, 1, 23, "Print"],
Cell[236107, 7089, 273, 6, 23, "Print"],
Cell[236383, 7097, 423, 5, 62, "Print"],
Cell[236809, 7104, 585, 10, 43, "Print"],
Cell[237397, 7116, 320, 7, 23, "Print"],
Cell[237720, 7125, 151, 2, 23, "Print"],
Cell[237874, 7129, 221, 5, 23, "Print"],
Cell[238098, 7136, 107, 1, 23, "Print"],
Cell[238208, 7139, 275, 6, 23, "Print"],
Cell[238486, 7147, 423, 5, 62, "Print"],
Cell[238912, 7154, 589, 11, 43, "Print"],
Cell[239504, 7167, 84, 1, 23, "Print"],
Cell[239591, 7170, 423, 5, 62, "Print"],
Cell[240017, 7177, 590, 11, 43, "Print"],
Cell[240610, 7190, 1093, 20, 43, "Print"],
Cell[241706, 7212, 443, 9, 23, "Print"],
Cell[242152, 7223, 141, 2, 23, "Print"],
Cell[242296, 7227, 94, 1, 23, "Print"],
Cell[242393, 7230, 179, 2, 23, "Print"],
Cell[242575, 7234, 155, 2, 23, "Print"],
Cell[242733, 7238, 146, 2, 23, "Print"],
Cell[242882, 7242, 287, 6, 23, "Print"]
}, Open  ]],
Cell[243184, 7251, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[243851, 7267, 221, 5, 23, "Print"],
Cell[244075, 7274, 285, 6, 23, "Print"],
Cell[244363, 7282, 137, 2, 23, "Print"],
Cell[244503, 7286, 142, 2, 23, "Print"],
Cell[244648, 7290, 285, 6, 23, "Print"],
Cell[244936, 7298, 149, 2, 23, "Print"]
}, Open  ]],
Cell[245100, 7303, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[245767, 7319, 221, 5, 23, "Print"],
Cell[245991, 7326, 110, 1, 23, "Print"],
Cell[246104, 7329, 275, 6, 23, "Print"],
Cell[246382, 7337, 423, 5, 62, "Print"],
Cell[246808, 7344, 584, 10, 43, "Print"],
Cell[247395, 7356, 287, 6, 23, "Print"],
Cell[247685, 7364, 151, 2, 23, "Print"]
}, Open  ]],
Cell[247851, 7369, 642, 12, 48, "Message"],
Cell[248496, 7383, 537, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[249058, 7398, 223, 5, 23, "Print"],
Cell[249284, 7405, 108, 1, 23, "Print"],
Cell[249395, 7408, 255, 6, 23, "Print"],
Cell[249653, 7416, 423, 5, 62, "Print"],
Cell[250079, 7423, 584, 10, 43, "Print"],
Cell[250666, 7435, 320, 7, 23, "Print"],
Cell[250989, 7444, 151, 2, 23, "Print"],
Cell[251143, 7448, 223, 5, 23, "Print"],
Cell[251369, 7455, 108, 1, 23, "Print"],
Cell[251480, 7458, 251, 6, 23, "Print"],
Cell[251734, 7466, 421, 5, 62, "Print"],
Cell[252158, 7473, 583, 10, 43, "Print"],
Cell[252744, 7485, 283, 6, 23, "Print"],
Cell[253030, 7493, 149, 2, 23, "Print"],
Cell[253182, 7497, 223, 5, 23, "Print"],
Cell[253408, 7504, 110, 1, 23, "Print"],
Cell[253521, 7507, 253, 6, 23, "Print"],
Cell[253777, 7515, 421, 5, 62, "Print"],
Cell[254201, 7522, 586, 11, 43, "Print"],
Cell[254790, 7535, 88, 1, 23, "Print"],
Cell[254881, 7538, 423, 5, 62, "Print"],
Cell[255307, 7545, 583, 10, 43, "Print"],
Cell[255893, 7557, 1063, 20, 43, "Print"],
Cell[256959, 7579, 445, 9, 23, "Print"],
Cell[257407, 7590, 141, 2, 23, "Print"],
Cell[257551, 7594, 93, 1, 23, "Print"],
Cell[257647, 7597, 177, 2, 23, "Print"],
Cell[257827, 7601, 153, 2, 23, "Print"],
Cell[257983, 7605, 146, 2, 23, "Print"],
Cell[258132, 7609, 287, 6, 23, "Print"]
}, Open  ]],
Cell[258434, 7618, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[259103, 7634, 223, 5, 23, "Print"],
Cell[259329, 7641, 282, 6, 23, "Print"],
Cell[259614, 7649, 134, 2, 23, "Print"],
Cell[259751, 7653, 139, 2, 23, "Print"],
Cell[259893, 7657, 287, 6, 23, "Print"],
Cell[260183, 7665, 151, 2, 23, "Print"]
}, Open  ]],
Cell[260349, 7670, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[261018, 7686, 223, 5, 23, "Print"],
Cell[261244, 7693, 110, 1, 23, "Print"],
Cell[261357, 7696, 275, 6, 23, "Print"],
Cell[261635, 7704, 421, 5, 62, "Print"],
Cell[262059, 7711, 583, 10, 43, "Print"],
Cell[262645, 7723, 285, 6, 23, "Print"],
Cell[262933, 7731, 149, 2, 23, "Print"]
}, Open  ]],
Cell[263097, 7736, 642, 12, 48, "Message"],
Cell[263742, 7750, 537, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[264304, 7765, 223, 5, 23, "Print"],
Cell[264530, 7772, 110, 1, 23, "Print"],
Cell[264643, 7775, 251, 6, 23, "Print"],
Cell[264897, 7783, 423, 5, 62, "Print"],
Cell[265323, 7790, 590, 11, 43, "Print"],
Cell[265916, 7803, 320, 7, 23, "Print"],
Cell[266239, 7812, 151, 2, 23, "Print"],
Cell[266393, 7816, 221, 5, 23, "Print"],
Cell[266617, 7823, 110, 1, 23, "Print"],
Cell[266730, 7826, 253, 6, 23, "Print"],
Cell[266986, 7834, 423, 5, 62, "Print"],
Cell[267412, 7841, 580, 10, 43, "Print"],
Cell[267995, 7853, 285, 6, 23, "Print"],
Cell[268283, 7861, 151, 2, 23, "Print"],
Cell[268437, 7865, 305, 7, 23, "Print"],
Cell[268745, 7874, 108, 1, 23, "Print"],
Cell[268856, 7877, 275, 6, 23, "Print"],
Cell[269134, 7885, 423, 5, 62, "Print"],
Cell[269560, 7892, 586, 10, 43, "Print"],
Cell[270149, 7904, 88, 1, 23, "Print"],
Cell[270240, 7907, 423, 5, 62, "Print"],
Cell[270666, 7914, 580, 10, 43, "Print"],
Cell[271249, 7926, 1040, 20, 43, "Print"],
Cell[272292, 7948, 381, 8, 23, "Print"],
Cell[272676, 7958, 141, 2, 23, "Print"],
Cell[272820, 7962, 93, 1, 23, "Print"],
Cell[272916, 7965, 175, 2, 23, "Print"],
Cell[273094, 7969, 151, 2, 23, "Print"],
Cell[273248, 7973, 146, 2, 23, "Print"],
Cell[273397, 7977, 287, 6, 23, "Print"]
}, Open  ]],
Cell[273699, 7986, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[274366, 8002, 221, 5, 23, "Print"],
Cell[274590, 8009, 285, 6, 23, "Print"],
Cell[274878, 8017, 135, 2, 23, "Print"],
Cell[275016, 8021, 140, 2, 23, "Print"],
Cell[275159, 8025, 284, 6, 23, "Print"],
Cell[275446, 8033, 148, 2, 23, "Print"]
}, Open  ]],
Cell[275609, 8038, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[276278, 8054, 223, 5, 23, "Print"],
Cell[276504, 8061, 110, 1, 23, "Print"],
Cell[276617, 8064, 273, 6, 23, "Print"],
Cell[276893, 8072, 423, 5, 62, "Print"],
Cell[277319, 8079, 583, 10, 43, "Print"],
Cell[277905, 8091, 287, 6, 23, "Print"],
Cell[278195, 8099, 151, 2, 23, "Print"]
}, Open  ]],
Cell[278361, 8104, 644, 12, 48, "Message"],
Cell[279008, 8118, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[279568, 8133, 221, 5, 23, "Print"],
Cell[279792, 8140, 108, 1, 23, "Print"],
Cell[279903, 8143, 253, 6, 23, "Print"],
Cell[280159, 8151, 421, 5, 62, "Print"],
Cell[280583, 8158, 582, 10, 43, "Print"],
Cell[281168, 8170, 285, 6, 23, "Print"],
Cell[281456, 8178, 149, 2, 23, "Print"],
Cell[281608, 8182, 221, 5, 23, "Print"],
Cell[281832, 8189, 110, 1, 23, "Print"],
Cell[281945, 8192, 275, 6, 23, "Print"],
Cell[282223, 8200, 420, 5, 62, "Print"],
Cell[282646, 8207, 582, 10, 43, "Print"],
Cell[283231, 8219, 317, 7, 23, "Print"],
Cell[283551, 8228, 148, 2, 23, "Print"],
Cell[283702, 8232, 223, 5, 23, "Print"],
Cell[283928, 8239, 110, 1, 23, "Print"],
Cell[284041, 8242, 272, 6, 23, "Print"],
Cell[284316, 8250, 420, 5, 62, "Print"],
Cell[284739, 8257, 585, 11, 43, "Print"],
Cell[285327, 8270, 317, 7, 23, "Print"],
Cell[285647, 8279, 148, 2, 23, "Print"],
Cell[285798, 8283, 221, 5, 23, "Print"],
Cell[286022, 8290, 108, 1, 23, "Print"],
Cell[286133, 8293, 273, 6, 23, "Print"],
Cell[286409, 8301, 423, 5, 62, "Print"],
Cell[286835, 8308, 581, 10, 43, "Print"],
Cell[287419, 8320, 320, 7, 23, "Print"],
Cell[287742, 8329, 151, 2, 23, "Print"],
Cell[287896, 8333, 223, 5, 23, "Print"],
Cell[288122, 8340, 110, 1, 23, "Print"],
Cell[288235, 8343, 273, 6, 23, "Print"],
Cell[288511, 8351, 423, 5, 62, "Print"],
Cell[288937, 8358, 586, 10, 43, "Print"],
Cell[289526, 8370, 323, 7, 23, "Print"],
Cell[289852, 8379, 151, 2, 23, "Print"],
Cell[290006, 8383, 223, 5, 23, "Print"],
Cell[290232, 8390, 108, 1, 23, "Print"],
Cell[290343, 8393, 273, 6, 23, "Print"],
Cell[290619, 8401, 423, 5, 62, "Print"],
Cell[291045, 8408, 586, 10, 43, "Print"],
Cell[291634, 8420, 86, 1, 23, "Print"],
Cell[291723, 8423, 421, 5, 62, "Print"],
Cell[292147, 8430, 587, 11, 43, "Print"],
Cell[292737, 8443, 1115, 22, 43, "Print"],
Cell[293855, 8467, 443, 9, 23, "Print"],
Cell[294301, 8478, 139, 2, 23, "Print"],
Cell[294443, 8482, 90, 1, 23, "Print"],
Cell[294536, 8485, 179, 2, 23, "Print"],
Cell[294718, 8489, 155, 2, 23, "Print"],
Cell[294876, 8493, 146, 2, 23, "Print"],
Cell[295025, 8497, 287, 6, 23, "Print"]
}, Open  ]],
Cell[295327, 8506, 641, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[295993, 8522, 220, 5, 23, "Print"],
Cell[296216, 8529, 285, 6, 23, "Print"],
Cell[296504, 8537, 135, 2, 23, "Print"],
Cell[296642, 8541, 142, 2, 23, "Print"],
Cell[296787, 8545, 285, 6, 23, "Print"],
Cell[297075, 8553, 149, 2, 23, "Print"]
}, Open  ]],
Cell[297239, 8558, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[297906, 8574, 221, 5, 23, "Print"],
Cell[298130, 8581, 110, 1, 23, "Print"],
Cell[298243, 8584, 253, 6, 23, "Print"],
Cell[298499, 8592, 421, 5, 62, "Print"],
Cell[298923, 8599, 579, 10, 43, "Print"],
Cell[299505, 8611, 285, 6, 23, "Print"],
Cell[299793, 8619, 149, 2, 23, "Print"]
}, Open  ]],
Cell[299957, 8624, 642, 12, 48, "Message"],
Cell[300602, 8638, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[301162, 8653, 221, 5, 23, "Print"],
Cell[301386, 8660, 110, 1, 23, "Print"],
Cell[301499, 8663, 251, 6, 23, "Print"],
Cell[301753, 8671, 423, 5, 62, "Print"],
Cell[302179, 8678, 588, 11, 43, "Print"],
Cell[302770, 8691, 287, 6, 23, "Print"],
Cell[303060, 8699, 151, 2, 23, "Print"],
Cell[303214, 8703, 221, 5, 23, "Print"],
Cell[303438, 8710, 108, 1, 23, "Print"],
Cell[303549, 8713, 273, 6, 23, "Print"],
Cell[303825, 8721, 423, 5, 62, "Print"],
Cell[304251, 8728, 583, 10, 43, "Print"],
Cell[304837, 8740, 86, 1, 23, "Print"],
Cell[304926, 8743, 423, 5, 62, "Print"],
Cell[305352, 8750, 582, 10, 43, "Print"],
Cell[305937, 8762, 1041, 20, 43, "Print"],
Cell[306981, 8784, 445, 9, 23, "Print"],
Cell[307429, 8795, 139, 2, 23, "Print"],
Cell[307571, 8799, 95, 1, 23, "Print"],
Cell[307669, 8802, 177, 2, 23, "Print"],
Cell[307849, 8806, 153, 2, 23, "Print"],
Cell[308005, 8810, 146, 2, 23, "Print"],
Cell[308154, 8814, 287, 6, 23, "Print"]
}, Open  ]],
Cell[308456, 8823, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[309125, 8839, 223, 5, 23, "Print"],
Cell[309351, 8846, 283, 6, 23, "Print"],
Cell[309637, 8854, 135, 2, 23, "Print"],
Cell[309775, 8858, 140, 2, 23, "Print"],
Cell[309918, 8862, 285, 6, 23, "Print"],
Cell[310206, 8870, 149, 2, 23, "Print"]
}, Open  ]],
Cell[310370, 8875, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[311037, 8891, 221, 5, 23, "Print"],
Cell[311261, 8898, 108, 1, 23, "Print"],
Cell[311372, 8901, 273, 6, 23, "Print"],
Cell[311648, 8909, 421, 5, 62, "Print"],
Cell[312072, 8916, 584, 10, 43, "Print"],
Cell[312659, 8928, 285, 6, 23, "Print"],
Cell[312947, 8936, 149, 2, 23, "Print"]
}, Open  ]],
Cell[313111, 8941, 642, 12, 48, "Message"],
Cell[313756, 8955, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[314316, 8970, 221, 5, 23, "Print"],
Cell[314540, 8977, 110, 1, 23, "Print"],
Cell[314653, 8980, 255, 6, 23, "Print"],
Cell[314911, 8988, 421, 5, 62, "Print"],
Cell[315335, 8995, 580, 10, 43, "Print"],
Cell[315918, 9007, 318, 7, 23, "Print"],
Cell[316239, 9016, 149, 2, 23, "Print"],
Cell[316391, 9020, 223, 5, 23, "Print"],
Cell[316617, 9027, 110, 1, 23, "Print"],
Cell[316730, 9030, 275, 6, 23, "Print"],
Cell[317008, 9038, 423, 5, 62, "Print"],
Cell[317434, 9045, 583, 10, 43, "Print"],
Cell[318020, 9057, 88, 1, 23, "Print"],
Cell[318111, 9060, 423, 5, 62, "Print"],
Cell[318537, 9067, 586, 10, 43, "Print"],
Cell[319126, 9079, 1016, 20, 43, "Print"],
Cell[320145, 9101, 381, 8, 23, "Print"],
Cell[320529, 9111, 139, 2, 23, "Print"],
Cell[320671, 9115, 90, 1, 23, "Print"],
Cell[320764, 9118, 179, 2, 23, "Print"],
Cell[320946, 9122, 155, 2, 23, "Print"],
Cell[321104, 9126, 146, 2, 23, "Print"],
Cell[321253, 9130, 287, 6, 23, "Print"]
}, Open  ]],
Cell[321555, 9139, 641, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[322221, 9155, 220, 5, 23, "Print"],
Cell[322444, 9162, 283, 6, 23, "Print"],
Cell[322730, 9170, 135, 2, 23, "Print"],
Cell[322868, 9174, 140, 2, 23, "Print"],
Cell[323011, 9178, 287, 6, 23, "Print"],
Cell[323301, 9186, 151, 2, 23, "Print"]
}, Open  ]],
Cell[323467, 9191, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[324136, 9207, 223, 5, 23, "Print"],
Cell[324362, 9214, 110, 1, 23, "Print"],
Cell[324475, 9217, 273, 6, 23, "Print"],
Cell[324751, 9225, 423, 5, 62, "Print"],
Cell[325177, 9232, 584, 10, 43, "Print"],
Cell[325764, 9244, 287, 6, 23, "Print"],
Cell[326054, 9252, 149, 2, 23, "Print"]
}, Open  ]],
Cell[326218, 9257, 641, 12, 48, "Message"],
Cell[326862, 9271, 534, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[327421, 9286, 220, 5, 23, "Print"],
Cell[327644, 9293, 110, 1, 23, "Print"],
Cell[327757, 9296, 255, 6, 23, "Print"],
Cell[328015, 9304, 423, 5, 62, "Print"],
Cell[328441, 9311, 584, 10, 43, "Print"],
Cell[329028, 9323, 287, 6, 23, "Print"],
Cell[329318, 9331, 151, 2, 23, "Print"],
Cell[329472, 9335, 221, 5, 23, "Print"],
Cell[329696, 9342, 110, 1, 23, "Print"],
Cell[329809, 9345, 273, 6, 23, "Print"],
Cell[330085, 9353, 423, 5, 62, "Print"],
Cell[330511, 9360, 583, 10, 43, "Print"],
Cell[331097, 9372, 318, 7, 23, "Print"],
Cell[331418, 9381, 149, 2, 23, "Print"],
Cell[331570, 9385, 223, 5, 23, "Print"],
Cell[331796, 9392, 110, 1, 23, "Print"],
Cell[331909, 9395, 273, 6, 23, "Print"],
Cell[332185, 9403, 421, 5, 62, "Print"],
Cell[332609, 9410, 584, 10, 43, "Print"],
Cell[333196, 9422, 318, 7, 23, "Print"],
Cell[333517, 9431, 149, 2, 23, "Print"],
Cell[333669, 9435, 221, 5, 23, "Print"],
Cell[333893, 9442, 108, 1, 23, "Print"],
Cell[334004, 9445, 275, 6, 23, "Print"],
Cell[334282, 9453, 421, 5, 62, "Print"],
Cell[334706, 9460, 583, 10, 43, "Print"],
Cell[335292, 9472, 88, 1, 23, "Print"],
Cell[335383, 9475, 423, 5, 62, "Print"],
Cell[335809, 9482, 582, 10, 43, "Print"],
Cell[336394, 9494, 1118, 22, 43, "Print"],
Cell[337515, 9518, 445, 9, 23, "Print"],
Cell[337963, 9529, 141, 2, 23, "Print"],
Cell[338107, 9533, 97, 1, 23, "Print"],
Cell[338207, 9536, 177, 2, 23, "Print"],
Cell[338387, 9540, 153, 2, 23, "Print"],
Cell[338543, 9544, 146, 2, 23, "Print"],
Cell[338692, 9548, 289, 6, 23, "Print"]
}, Open  ]],
Cell[338996, 9557, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[339663, 9573, 221, 5, 23, "Print"],
Cell[339887, 9580, 282, 6, 23, "Print"],
Cell[340172, 9588, 137, 2, 23, "Print"],
Cell[340312, 9592, 142, 2, 23, "Print"],
Cell[340457, 9596, 287, 6, 23, "Print"],
Cell[340747, 9604, 151, 2, 23, "Print"]
}, Open  ]],
Cell[340913, 9609, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[341580, 9625, 223, 5, 23, "Print"],
Cell[341806, 9632, 108, 1, 23, "Print"],
Cell[341917, 9635, 273, 6, 23, "Print"],
Cell[342193, 9643, 423, 5, 62, "Print"],
Cell[342619, 9650, 589, 11, 43, "Print"],
Cell[343211, 9663, 285, 6, 23, "Print"],
Cell[343499, 9671, 149, 2, 23, "Print"]
}, Open  ]],
Cell[343663, 9676, 644, 12, 48, "Message"],
Cell[344310, 9690, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[344870, 9705, 221, 5, 23, "Print"],
Cell[345094, 9712, 110, 1, 23, "Print"],
Cell[345207, 9715, 253, 6, 23, "Print"],
Cell[345463, 9723, 421, 5, 62, "Print"],
Cell[345887, 9730, 583, 10, 43, "Print"],
Cell[346473, 9742, 285, 6, 23, "Print"],
Cell[346761, 9750, 149, 2, 23, "Print"],
Cell[346913, 9754, 221, 5, 23, "Print"],
Cell[347137, 9761, 108, 1, 23, "Print"],
Cell[347248, 9764, 275, 6, 23, "Print"],
Cell[347526, 9772, 421, 5, 62, "Print"],
Cell[347950, 9779, 583, 10, 43, "Print"],
Cell[348536, 9791, 86, 1, 23, "Print"],
Cell[348625, 9794, 423, 5, 62, "Print"],
Cell[349051, 9801, 591, 11, 43, "Print"],
Cell[349645, 9814, 1055, 20, 43, "Print"],
Cell[350703, 9836, 379, 8, 23, "Print"],
Cell[351085, 9846, 139, 2, 23, "Print"],
Cell[351227, 9850, 93, 1, 23, "Print"],
Cell[351323, 9853, 179, 2, 23, "Print"],
Cell[351505, 9857, 152, 2, 23, "Print"],
Cell[351660, 9861, 146, 2, 23, "Print"],
Cell[351809, 9865, 287, 6, 23, "Print"]
}, Open  ]],
Cell[352111, 9874, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[352778, 9890, 221, 5, 23, "Print"],
Cell[353002, 9897, 283, 6, 23, "Print"],
Cell[353288, 9905, 137, 2, 23, "Print"],
Cell[353428, 9909, 142, 2, 23, "Print"],
Cell[353573, 9913, 287, 6, 23, "Print"],
Cell[353863, 9921, 151, 2, 23, "Print"]
}, Open  ]],
Cell[354029, 9926, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[354698, 9942, 223, 5, 23, "Print"],
Cell[354924, 9949, 110, 1, 23, "Print"],
Cell[355037, 9952, 275, 6, 23, "Print"],
Cell[355315, 9960, 421, 5, 62, "Print"],
Cell[355739, 9967, 583, 10, 43, "Print"],
Cell[356325, 9979, 285, 6, 23, "Print"],
Cell[356613, 9987, 149, 2, 23, "Print"]
}, Open  ]],
Cell[356777, 9992, 644, 12, 48, "Message"],
Cell[357424, 10006, 537, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[357986, 10021, 221, 5, 23, "Print"],
Cell[358210, 10028, 110, 1, 23, "Print"],
Cell[358323, 10031, 253, 6, 23, "Print"],
Cell[358579, 10039, 423, 5, 62, "Print"],
Cell[359005, 10046, 583, 10, 43, "Print"],
Cell[359591, 10058, 287, 6, 23, "Print"],
Cell[359881, 10066, 151, 2, 23, "Print"],
Cell[360035, 10070, 221, 5, 23, "Print"],
Cell[360259, 10077, 110, 1, 23, "Print"],
Cell[360372, 10080, 275, 6, 23, "Print"],
Cell[360650, 10088, 423, 5, 62, "Print"],
Cell[361076, 10095, 585, 10, 43, "Print"],
Cell[361664, 10107, 318, 7, 23, "Print"],
Cell[361985, 10116, 151, 2, 23, "Print"],
Cell[362139, 10120, 223, 5, 23, "Print"],
Cell[362365, 10127, 110, 1, 23, "Print"],
Cell[362478, 10130, 273, 6, 23, "Print"],
Cell[362754, 10138, 421, 5, 62, "Print"],
Cell[363178, 10145, 581, 10, 43, "Print"],
Cell[363762, 10157, 86, 1, 23, "Print"],
Cell[363851, 10160, 421, 5, 62, "Print"],
Cell[364275, 10167, 583, 10, 43, "Print"],
Cell[364861, 10179, 1074, 20, 43, "Print"],
Cell[365938, 10201, 445, 9, 23, "Print"],
Cell[366386, 10212, 141, 2, 23, "Print"],
Cell[366530, 10216, 93, 1, 23, "Print"],
Cell[366626, 10219, 177, 2, 23, "Print"],
Cell[366806, 10223, 153, 2, 23, "Print"],
Cell[366962, 10227, 146, 2, 23, "Print"],
Cell[367111, 10231, 287, 6, 23, "Print"]
}, Open  ]],
Cell[367413, 10240, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[368082, 10256, 220, 5, 23, "Print"],
Cell[368305, 10263, 285, 6, 23, "Print"],
Cell[368593, 10271, 134, 2, 23, "Print"],
Cell[368730, 10275, 139, 2, 23, "Print"],
Cell[368872, 10279, 285, 6, 23, "Print"],
Cell[369160, 10287, 149, 2, 23, "Print"]
}, Open  ]],
Cell[369324, 10292, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[369993, 10308, 223, 5, 23, "Print"],
Cell[370219, 10315, 108, 1, 23, "Print"],
Cell[370330, 10318, 273, 6, 23, "Print"],
Cell[370606, 10326, 423, 5, 62, "Print"],
Cell[371032, 10333, 583, 10, 43, "Print"],
Cell[371618, 10345, 287, 6, 23, "Print"],
Cell[371908, 10353, 149, 2, 23, "Print"]
}, Open  ]],
Cell[372072, 10358, 644, 12, 48, "Message"],
Cell[372719, 10372, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[373279, 10387, 221, 5, 23, "Print"],
Cell[373503, 10394, 110, 1, 23, "Print"],
Cell[373616, 10397, 255, 6, 23, "Print"],
Cell[373874, 10405, 423, 5, 62, "Print"],
Cell[374300, 10412, 584, 10, 43, "Print"],
Cell[374887, 10424, 287, 6, 23, "Print"],
Cell[375177, 10432, 151, 2, 23, "Print"],
Cell[375331, 10436, 221, 5, 23, "Print"],
Cell[375555, 10443, 110, 1, 23, "Print"],
Cell[375668, 10446, 275, 6, 23, "Print"],
Cell[375946, 10454, 423, 5, 62, "Print"],
Cell[376372, 10461, 582, 10, 43, "Print"],
Cell[376957, 10473, 320, 7, 23, "Print"],
Cell[377280, 10482, 151, 2, 23, "Print"],
Cell[377434, 10486, 221, 5, 23, "Print"],
Cell[377658, 10493, 108, 1, 23, "Print"],
Cell[377769, 10496, 275, 6, 23, "Print"],
Cell[378047, 10504, 423, 5, 62, "Print"],
Cell[378473, 10511, 591, 11, 43, "Print"],
Cell[379067, 10524, 317, 7, 23, "Print"],
Cell[379387, 10533, 148, 2, 23, "Print"],
Cell[379538, 10537, 223, 5, 23, "Print"],
Cell[379764, 10544, 108, 1, 23, "Print"],
Cell[379875, 10547, 275, 6, 23, "Print"],
Cell[380153, 10555, 423, 5, 62, "Print"],
Cell[380579, 10562, 581, 10, 43, "Print"],
Cell[381163, 10574, 318, 7, 23, "Print"],
Cell[381484, 10583, 149, 2, 23, "Print"],
Cell[381636, 10587, 223, 5, 23, "Print"],
Cell[381862, 10594, 110, 1, 23, "Print"],
Cell[381975, 10597, 275, 6, 23, "Print"],
Cell[382253, 10605, 421, 5, 62, "Print"],
Cell[382677, 10612, 582, 10, 43, "Print"],
Cell[383262, 10624, 321, 7, 23, "Print"],
Cell[383586, 10633, 149, 2, 23, "Print"],
Cell[383738, 10637, 221, 5, 23, "Print"],
Cell[383962, 10644, 108, 1, 23, "Print"],
Cell[384073, 10647, 273, 6, 23, "Print"],
Cell[384349, 10655, 421, 5, 62, "Print"],
Cell[384773, 10662, 590, 11, 43, "Print"],
Cell[385366, 10675, 86, 1, 23, "Print"],
Cell[385455, 10678, 421, 5, 62, "Print"],
Cell[385879, 10685, 583, 10, 43, "Print"],
Cell[386465, 10697, 1117, 22, 43, "Print"],
Cell[387585, 10721, 443, 9, 23, "Print"],
Cell[388031, 10732, 139, 2, 23, "Print"],
Cell[388173, 10736, 93, 1, 23, "Print"],
Cell[388269, 10739, 179, 2, 23, "Print"],
Cell[388451, 10743, 155, 2, 23, "Print"],
Cell[388609, 10747, 146, 2, 23, "Print"],
Cell[388758, 10751, 287, 6, 23, "Print"]
}, Open  ]],
Cell[389060, 10760, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[389729, 10776, 223, 5, 23, "Print"],
Cell[389955, 10783, 283, 6, 23, "Print"],
Cell[390241, 10791, 135, 2, 23, "Print"],
Cell[390379, 10795, 140, 2, 23, "Print"],
Cell[390522, 10799, 285, 6, 23, "Print"],
Cell[390810, 10807, 149, 2, 23, "Print"]
}, Open  ]],
Cell[390974, 10812, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[391641, 10828, 221, 5, 23, "Print"],
Cell[391865, 10835, 110, 1, 23, "Print"],
Cell[391978, 10838, 273, 6, 23, "Print"],
Cell[392254, 10846, 421, 5, 62, "Print"],
Cell[392678, 10853, 589, 11, 43, "Print"],
Cell[393270, 10866, 285, 6, 23, "Print"],
Cell[393558, 10874, 149, 2, 23, "Print"]
}, Open  ]],
Cell[393722, 10879, 642, 12, 48, "Message"],
Cell[394367, 10893, 537, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[394929, 10908, 223, 5, 23, "Print"],
Cell[395155, 10915, 110, 1, 23, "Print"],
Cell[395268, 10918, 253, 6, 23, "Print"],
Cell[395524, 10926, 423, 5, 62, "Print"],
Cell[395950, 10933, 582, 10, 43, "Print"],
Cell[396535, 10945, 285, 6, 23, "Print"],
Cell[396823, 10953, 149, 2, 23, "Print"],
Cell[396975, 10957, 221, 5, 23, "Print"],
Cell[397199, 10964, 108, 1, 23, "Print"],
Cell[397310, 10967, 275, 6, 23, "Print"],
Cell[397588, 10975, 420, 5, 62, "Print"],
Cell[398011, 10982, 582, 10, 43, "Print"],
Cell[398596, 10994, 320, 7, 23, "Print"],
Cell[398919, 11003, 151, 2, 23, "Print"],
Cell[399073, 11007, 223, 5, 23, "Print"],
Cell[399299, 11014, 110, 1, 23, "Print"],
Cell[399412, 11017, 275, 6, 23, "Print"],
Cell[399690, 11025, 421, 5, 62, "Print"],
Cell[400114, 11032, 585, 10, 43, "Print"],
Cell[400702, 11044, 318, 7, 23, "Print"],
Cell[401023, 11053, 149, 2, 23, "Print"],
Cell[401175, 11057, 221, 5, 23, "Print"],
Cell[401399, 11064, 110, 1, 23, "Print"],
Cell[401512, 11067, 275, 6, 23, "Print"],
Cell[401790, 11075, 421, 5, 62, "Print"],
Cell[402214, 11082, 589, 11, 43, "Print"],
Cell[402806, 11095, 320, 7, 23, "Print"],
Cell[403129, 11104, 151, 2, 23, "Print"],
Cell[403283, 11108, 221, 5, 23, "Print"],
Cell[403507, 11115, 108, 1, 23, "Print"],
Cell[403618, 11118, 273, 6, 23, "Print"],
Cell[403894, 11126, 421, 5, 62, "Print"],
Cell[404318, 11133, 581, 10, 43, "Print"],
Cell[404902, 11145, 321, 7, 23, "Print"],
Cell[405226, 11154, 149, 2, 23, "Print"],
Cell[405378, 11158, 221, 5, 23, "Print"],
Cell[405602, 11165, 108, 1, 23, "Print"],
Cell[405713, 11168, 275, 6, 23, "Print"],
Cell[405991, 11176, 423, 5, 62, "Print"],
Cell[406417, 11183, 581, 10, 43, "Print"],
Cell[407001, 11195, 320, 7, 23, "Print"],
Cell[407324, 11204, 151, 2, 23, "Print"],
Cell[407478, 11208, 223, 5, 23, "Print"],
Cell[407704, 11215, 110, 1, 23, "Print"],
Cell[407817, 11218, 275, 6, 23, "Print"],
Cell[408095, 11226, 423, 5, 62, "Print"],
Cell[408521, 11233, 582, 10, 43, "Print"],
Cell[409106, 11245, 340, 8, 23, "Print"],
Cell[409449, 11255, 151, 2, 23, "Print"],
Cell[409603, 11259, 221, 5, 23, "Print"],
Cell[409827, 11266, 110, 1, 23, "Print"],
Cell[409940, 11269, 275, 6, 23, "Print"],
Cell[410218, 11277, 423, 5, 62, "Print"],
Cell[410644, 11284, 584, 10, 43, "Print"],
Cell[411231, 11296, 341, 8, 23, "Print"],
Cell[411575, 11306, 149, 2, 23, "Print"],
Cell[411727, 11310, 223, 5, 23, "Print"],
Cell[411953, 11317, 108, 1, 23, "Print"],
Cell[412064, 11320, 273, 6, 23, "Print"],
Cell[412340, 11328, 423, 5, 62, "Print"],
Cell[412766, 11335, 584, 10, 43, "Print"],
Cell[413353, 11347, 88, 1, 23, "Print"],
Cell[413444, 11350, 421, 5, 62, "Print"],
Cell[413868, 11357, 584, 10, 43, "Print"],
Cell[414455, 11369, 1137, 22, 64, "Print"],
Cell[415595, 11393, 443, 9, 23, "Print"],
Cell[416041, 11404, 431, 9, 23, "Print"],
Cell[416475, 11415, 155, 2, 23, "Print"],
Cell[416633, 11419, 116, 1, 23, "Print"],
Cell[416752, 11422, 139, 2, 23, "Print"],
Cell[416894, 11426, 152, 2, 23, "Print"],
Cell[417049, 11430, 124, 2, 23, "Print"],
Cell[417176, 11434, 98, 1, 23, "Print"],
Cell[417277, 11437, 119, 1, 23, "Print"],
Cell[417399, 11440, 130, 2, 23, "Print"],
Cell[417532, 11444, 271, 4, 62, "Print"],
Cell[417806, 11450, 490, 10, 23, "Print"],
Cell[418299, 11462, 261, 4, 43, "Print"],
Cell[418563, 11468, 340, 7, 23, "Print"],
Cell[418906, 11477, 375, 8, 23, "Print"],
Cell[419284, 11487, 117, 1, 23, "Print"],
Cell[419404, 11490, 162, 2, 23, "Print"],
Cell[419569, 11494, 86, 1, 23, "Print"],
Cell[419658, 11497, 79, 1, 23, "Print"],
Cell[419740, 11500, 77, 1, 23, "Print"],
Cell[419820, 11503, 200, 5, 23, "Print"],
Cell[420023, 11510, 117, 1, 23, "Print"],
Cell[420143, 11513, 419, 9, 23, "Print"],
Cell[420565, 11524, 359, 8, 23, "Print"],
Cell[420927, 11534, 199, 5, 23, "Print"],
Cell[421129, 11541, 375, 8, 23, "Print"],
Cell[421507, 11551, 200, 5, 23, "Print"],
Cell[421710, 11558, 117, 1, 23, "Print"],
Cell[421830, 11561, 354, 8, 23, "Print"],
Cell[422187, 11571, 364, 8, 23, "Print"],
Cell[422554, 11581, 201, 5, 23, "Print"],
Cell[422758, 11588, 375, 8, 23, "Print"],
Cell[423136, 11598, 200, 5, 23, "Print"],
Cell[423339, 11605, 115, 1, 23, "Print"],
Cell[423457, 11608, 355, 8, 23, "Print"],
Cell[423815, 11618, 361, 8, 23, "Print"],
Cell[424179, 11628, 201, 5, 23, "Print"],
Cell[424383, 11635, 311, 7, 23, "Print"],
Cell[424697, 11644, 200, 5, 23, "Print"],
Cell[424900, 11651, 117, 1, 23, "Print"],
Cell[425020, 11654, 355, 8, 23, "Print"],
Cell[425378, 11664, 364, 8, 23, "Print"],
Cell[425745, 11674, 199, 5, 23, "Print"],
Cell[425947, 11681, 309, 7, 23, "Print"],
Cell[426259, 11690, 198, 5, 23, "Print"],
Cell[426460, 11697, 115, 1, 23, "Print"],
Cell[426578, 11700, 359, 8, 23, "Print"],
Cell[426940, 11710, 364, 8, 23, "Print"],
Cell[427307, 11720, 201, 5, 23, "Print"],
Cell[427511, 11727, 373, 8, 23, "Print"],
Cell[427887, 11737, 200, 5, 23, "Print"],
Cell[428090, 11744, 115, 1, 23, "Print"],
Cell[428208, 11747, 418, 9, 23, "Print"],
Cell[428629, 11758, 95, 1, 23, "Print"],
Cell[428727, 11761, 222, 5, 23, "Print"],
Cell[428952, 11768, 222, 5, 23, "Print"],
Cell[429177, 11775, 361, 8, 23, "Print"],
Cell[429541, 11785, 199, 5, 23, "Print"],
Cell[429743, 11792, 311, 7, 23, "Print"],
Cell[430057, 11801, 198, 5, 23, "Print"],
Cell[430258, 11808, 115, 1, 23, "Print"],
Cell[430376, 11811, 421, 9, 23, "Print"],
Cell[430800, 11822, 98, 1, 23, "Print"],
Cell[430901, 11825, 222, 5, 23, "Print"],
Cell[431126, 11832, 222, 5, 23, "Print"],
Cell[431351, 11839, 273, 4, 62, "Print"],
Cell[431627, 11845, 486, 10, 23, "Print"],
Cell[432116, 11857, 261, 4, 43, "Print"],
Cell[432380, 11863, 336, 7, 23, "Print"],
Cell[432719, 11872, 202, 3, 23, "Print"],
Cell[432924, 11877, 413, 8, 23, "Print"],
Cell[433340, 11887, 238, 5, 23, "Print"],
Cell[433581, 11894, 155, 2, 23, "Print"],
Cell[433739, 11898, 404, 8, 23, "Print"],
Cell[434146, 11908, 170, 2, 23, "Print"],
Cell[434319, 11912, 402, 8, 23, "Print"],
Cell[434724, 11922, 361, 8, 23, "Print"],
Cell[435088, 11932, 362, 8, 23, "Print"],
Cell[435453, 11942, 201, 5, 23, "Print"],
Cell[435657, 11949, 313, 7, 23, "Print"],
Cell[435973, 11958, 515, 10, 23, "Print"],
Cell[436491, 11970, 126, 2, 23, "Print"],
Cell[436620, 11974, 100, 1, 23, "Print"],
Cell[436723, 11977, 119, 1, 23, "Print"],
Cell[436845, 11980, 130, 2, 23, "Print"],
Cell[436978, 11984, 271, 4, 62, "Print"],
Cell[437252, 11990, 488, 10, 23, "Print"],
Cell[437743, 12002, 261, 4, 43, "Print"],
Cell[438007, 12008, 341, 7, 23, "Print"],
Cell[438351, 12017, 377, 8, 23, "Print"],
Cell[438731, 12027, 119, 1, 23, "Print"],
Cell[438853, 12030, 162, 2, 23, "Print"],
Cell[439018, 12034, 86, 1, 23, "Print"],
Cell[439107, 12037, 89, 1, 23, "Print"],
Cell[439199, 12040, 77, 1, 23, "Print"],
Cell[439279, 12043, 200, 5, 23, "Print"],
Cell[439482, 12050, 117, 1, 23, "Print"],
Cell[439602, 12053, 417, 9, 23, "Print"],
Cell[440022, 12064, 362, 8, 23, "Print"],
Cell[440387, 12074, 201, 5, 23, "Print"],
Cell[440591, 12081, 375, 8, 23, "Print"],
Cell[440969, 12091, 200, 5, 23, "Print"],
Cell[441172, 12098, 117, 1, 23, "Print"],
Cell[441292, 12101, 419, 9, 23, "Print"],
Cell[441714, 12112, 360, 8, 23, "Print"],
Cell[442077, 12122, 199, 5, 23, "Print"],
Cell[442279, 12129, 311, 7, 23, "Print"],
Cell[442593, 12138, 198, 5, 23, "Print"],
Cell[442794, 12145, 115, 1, 23, "Print"],
Cell[442912, 12148, 357, 8, 23, "Print"],
Cell[443272, 12158, 361, 8, 23, "Print"],
Cell[443636, 12168, 201, 5, 23, "Print"],
Cell[443840, 12175, 313, 7, 23, "Print"],
Cell[444156, 12184, 200, 5, 23, "Print"],
Cell[444359, 12191, 117, 1, 23, "Print"],
Cell[444479, 12194, 419, 9, 23, "Print"],
Cell[444901, 12205, 362, 8, 23, "Print"],
Cell[445266, 12215, 199, 5, 23, "Print"],
Cell[445468, 12222, 373, 8, 23, "Print"],
Cell[445844, 12232, 198, 5, 23, "Print"],
Cell[446045, 12239, 115, 1, 23, "Print"],
Cell[446163, 12242, 419, 9, 23, "Print"],
Cell[446585, 12253, 362, 8, 23, "Print"],
Cell[446950, 12263, 199, 5, 23, "Print"],
Cell[447152, 12270, 373, 8, 23, "Print"],
Cell[447528, 12280, 200, 5, 23, "Print"],
Cell[447731, 12287, 117, 1, 23, "Print"],
Cell[447851, 12290, 419, 9, 23, "Print"],
Cell[448273, 12301, 96, 1, 23, "Print"],
Cell[448372, 12304, 222, 5, 23, "Print"],
Cell[448597, 12311, 222, 5, 23, "Print"],
Cell[448822, 12318, 362, 8, 23, "Print"],
Cell[449187, 12328, 199, 5, 23, "Print"],
Cell[449389, 12335, 373, 8, 23, "Print"],
Cell[449765, 12345, 198, 5, 23, "Print"],
Cell[449966, 12352, 115, 1, 23, "Print"],
Cell[450084, 12355, 419, 9, 23, "Print"],
Cell[450506, 12366, 96, 1, 23, "Print"],
Cell[450605, 12369, 220, 5, 23, "Print"],
Cell[450828, 12376, 222, 5, 23, "Print"],
Cell[451053, 12383, 273, 4, 62, "Print"],
Cell[451329, 12389, 490, 10, 23, "Print"],
Cell[451822, 12401, 261, 4, 43, "Print"],
Cell[452086, 12407, 340, 7, 23, "Print"],
Cell[452429, 12416, 204, 3, 23, "Print"],
Cell[452636, 12421, 413, 8, 23, "Print"],
Cell[453052, 12431, 238, 5, 23, "Print"],
Cell[453293, 12438, 155, 2, 23, "Print"],
Cell[453451, 12442, 405, 8, 23, "Print"],
Cell[453859, 12452, 170, 2, 23, "Print"],
Cell[454032, 12456, 402, 8, 23, "Print"],
Cell[454437, 12466, 367, 8, 23, "Print"],
Cell[454807, 12476, 364, 8, 23, "Print"],
Cell[455174, 12486, 201, 5, 23, "Print"],
Cell[455378, 12493, 315, 7, 23, "Print"],
Cell[455696, 12502, 515, 10, 23, "Print"],
Cell[456214, 12514, 126, 2, 23, "Print"],
Cell[456343, 12518, 103, 1, 23, "Print"],
Cell[456449, 12521, 119, 1, 23, "Print"],
Cell[456571, 12524, 130, 2, 23, "Print"],
Cell[456704, 12528, 273, 4, 62, "Print"],
Cell[456980, 12534, 488, 10, 23, "Print"],
Cell[457471, 12546, 261, 4, 43, "Print"],
Cell[457735, 12552, 341, 7, 23, "Print"],
Cell[458079, 12561, 379, 8, 23, "Print"],
Cell[458461, 12571, 119, 1, 23, "Print"],
Cell[458583, 12574, 162, 2, 23, "Print"],
Cell[458748, 12578, 86, 1, 23, "Print"],
Cell[458837, 12581, 89, 1, 23, "Print"],
Cell[458929, 12584, 76, 1, 23, "Print"],
Cell[459008, 12587, 200, 5, 23, "Print"],
Cell[459211, 12594, 117, 1, 23, "Print"],
Cell[459331, 12597, 419, 9, 23, "Print"],
Cell[459753, 12608, 364, 8, 23, "Print"],
Cell[460120, 12618, 198, 5, 23, "Print"],
Cell[460321, 12625, 306, 7, 23, "Print"],
Cell[460630, 12634, 197, 5, 23, "Print"],
Cell[460830, 12641, 114, 1, 23, "Print"],
Cell[460947, 12644, 416, 9, 23, "Print"],
Cell[461366, 12655, 364, 8, 23, "Print"],
Cell[461733, 12665, 201, 5, 23, "Print"],
Cell[461937, 12672, 373, 8, 23, "Print"],
Cell[462313, 12682, 200, 5, 23, "Print"],
Cell[462516, 12689, 117, 1, 23, "Print"],
Cell[462636, 12692, 421, 9, 23, "Print"],
Cell[463060, 12703, 364, 8, 23, "Print"],
Cell[463427, 12713, 201, 5, 23, "Print"],
Cell[463631, 12720, 313, 7, 23, "Print"],
Cell[463947, 12729, 200, 5, 23, "Print"],
Cell[464150, 12736, 117, 1, 23, "Print"],
Cell[464270, 12739, 417, 9, 23, "Print"],
Cell[464690, 12750, 362, 8, 23, "Print"],
Cell[465055, 12760, 199, 5, 23, "Print"],
Cell[465257, 12767, 369, 8, 23, "Print"],
Cell[465629, 12777, 198, 5, 23, "Print"],
Cell[465830, 12784, 115, 1, 23, "Print"],
Cell[465948, 12787, 421, 9, 23, "Print"],
Cell[466372, 12798, 362, 8, 23, "Print"],
Cell[466737, 12808, 199, 5, 23, "Print"],
Cell[466939, 12815, 373, 8, 23, "Print"],
Cell[467315, 12825, 198, 5, 23, "Print"],
Cell[467516, 12832, 115, 1, 23, "Print"],
Cell[467634, 12835, 361, 8, 23, "Print"],
Cell[467998, 12845, 98, 1, 23, "Print"],
Cell[468099, 12848, 222, 5, 23, "Print"],
Cell[468324, 12855, 224, 5, 23, "Print"],
Cell[468551, 12862, 362, 8, 23, "Print"],
Cell[468916, 12872, 199, 5, 23, "Print"],
Cell[469118, 12879, 371, 8, 23, "Print"],
Cell[469492, 12889, 198, 5, 23, "Print"],
Cell[469693, 12896, 115, 1, 23, "Print"],
Cell[469811, 12899, 417, 9, 23, "Print"],
Cell[470231, 12910, 94, 1, 23, "Print"],
Cell[470328, 12913, 222, 5, 23, "Print"],
Cell[470553, 12920, 224, 5, 23, "Print"],
Cell[470780, 12927, 271, 4, 62, "Print"],
Cell[471054, 12933, 483, 10, 23, "Print"],
Cell[471540, 12945, 259, 4, 43, "Print"],
Cell[471802, 12951, 335, 7, 23, "Print"],
Cell[472140, 12960, 202, 3, 23, "Print"],
Cell[472345, 12965, 404, 8, 23, "Print"],
Cell[472752, 12975, 236, 5, 23, "Print"],
Cell[472991, 12982, 153, 2, 23, "Print"],
Cell[473147, 12986, 401, 8, 23, "Print"],
Cell[473551, 12996, 170, 2, 23, "Print"],
Cell[473724, 13000, 400, 8, 23, "Print"],
Cell[474127, 13010, 362, 8, 23, "Print"],
Cell[474492, 13020, 362, 8, 23, "Print"],
Cell[474857, 13030, 199, 5, 23, "Print"],
Cell[475059, 13037, 315, 7, 23, "Print"],
Cell[475377, 13046, 515, 10, 23, "Print"],
Cell[475895, 13058, 126, 2, 23, "Print"],
Cell[476024, 13062, 108, 1, 23, "Print"],
Cell[476135, 13065, 117, 1, 23, "Print"],
Cell[476255, 13068, 130, 2, 23, "Print"],
Cell[476388, 13072, 271, 4, 62, "Print"],
Cell[476662, 13078, 490, 10, 23, "Print"],
Cell[477155, 13090, 261, 4, 43, "Print"],
Cell[477419, 13096, 340, 7, 23, "Print"],
Cell[477762, 13105, 375, 8, 23, "Print"],
Cell[478140, 13115, 119, 1, 23, "Print"],
Cell[478262, 13118, 162, 2, 23, "Print"],
Cell[478427, 13122, 84, 1, 23, "Print"],
Cell[478514, 13125, 81, 1, 23, "Print"],
Cell[478598, 13128, 90, 1, 23, "Print"],
Cell[478691, 13131, 198, 5, 23, "Print"],
Cell[478892, 13138, 115, 1, 23, "Print"],
Cell[479010, 13141, 418, 9, 23, "Print"],
Cell[479431, 13152, 361, 8, 23, "Print"],
Cell[479795, 13162, 198, 5, 23, "Print"],
Cell[479996, 13169, 372, 8, 23, "Print"],
Cell[480371, 13179, 197, 5, 23, "Print"],
Cell[480571, 13186, 114, 1, 23, "Print"],
Cell[480688, 13189, 357, 8, 23, "Print"],
Cell[481048, 13199, 362, 8, 23, "Print"],
Cell[481413, 13209, 201, 5, 23, "Print"],
Cell[481617, 13216, 375, 8, 23, "Print"],
Cell[481995, 13226, 200, 5, 23, "Print"],
Cell[482198, 13233, 117, 1, 23, "Print"],
Cell[482318, 13236, 419, 9, 23, "Print"],
Cell[482740, 13247, 364, 8, 23, "Print"],
Cell[483107, 13257, 201, 5, 23, "Print"],
Cell[483311, 13264, 375, 8, 23, "Print"],
Cell[483689, 13274, 200, 5, 23, "Print"],
Cell[483892, 13281, 117, 1, 23, "Print"],
Cell[484012, 13284, 421, 9, 23, "Print"],
Cell[484436, 13295, 361, 8, 23, "Print"],
Cell[484800, 13305, 201, 5, 23, "Print"],
Cell[485004, 13312, 373, 8, 23, "Print"],
Cell[485380, 13322, 200, 5, 23, "Print"],
Cell[485583, 13329, 117, 1, 23, "Print"],
Cell[485703, 13332, 359, 8, 23, "Print"],
Cell[486065, 13342, 364, 8, 23, "Print"],
Cell[486432, 13352, 201, 5, 23, "Print"],
Cell[486636, 13359, 373, 8, 23, "Print"],
Cell[487012, 13369, 200, 5, 23, "Print"],
Cell[487215, 13376, 117, 1, 23, "Print"],
Cell[487335, 13379, 417, 9, 23, "Print"],
Cell[487755, 13390, 96, 1, 23, "Print"],
Cell[487854, 13393, 222, 5, 23, "Print"],
Cell[488079, 13400, 222, 5, 23, "Print"],
Cell[488304, 13407, 362, 8, 23, "Print"],
Cell[488669, 13417, 199, 5, 23, "Print"],
Cell[488871, 13424, 309, 7, 23, "Print"],
Cell[489183, 13433, 198, 5, 23, "Print"],
Cell[489384, 13440, 115, 1, 23, "Print"],
Cell[489502, 13443, 417, 9, 23, "Print"],
Cell[489922, 13454, 96, 1, 23, "Print"],
Cell[490021, 13457, 222, 5, 23, "Print"],
Cell[490246, 13464, 224, 5, 23, "Print"],
Cell[490473, 13471, 271, 4, 62, "Print"],
Cell[490747, 13477, 488, 10, 23, "Print"],
Cell[491238, 13489, 259, 4, 43, "Print"],
Cell[491500, 13495, 337, 7, 23, "Print"],
Cell[491840, 13504, 202, 3, 23, "Print"],
Cell[492045, 13509, 412, 8, 23, "Print"],
Cell[492460, 13519, 238, 5, 23, "Print"],
Cell[492701, 13526, 155, 2, 23, "Print"],
Cell[492859, 13530, 404, 8, 23, "Print"],
Cell[493266, 13540, 170, 2, 23, "Print"],
Cell[493439, 13544, 401, 8, 23, "Print"],
Cell[493843, 13554, 367, 8, 23, "Print"],
Cell[494213, 13564, 364, 8, 23, "Print"],
Cell[494580, 13574, 201, 5, 23, "Print"],
Cell[494784, 13581, 379, 8, 23, "Print"],
Cell[495166, 13591, 515, 10, 23, "Print"],
Cell[495684, 13603, 126, 2, 23, "Print"],
Cell[495813, 13607, 94, 1, 23, "Print"],
Cell[495910, 13610, 119, 1, 23, "Print"],
Cell[496032, 13613, 128, 2, 23, "Print"],
Cell[496163, 13617, 273, 4, 62, "Print"],
Cell[496439, 13623, 488, 10, 23, "Print"],
Cell[496930, 13635, 261, 4, 43, "Print"],
Cell[497194, 13641, 339, 7, 23, "Print"],
Cell[497536, 13650, 375, 8, 23, "Print"],
Cell[497914, 13660, 117, 1, 23, "Print"],
Cell[498034, 13663, 160, 2, 23, "Print"],
Cell[498197, 13667, 86, 1, 23, "Print"],
Cell[498286, 13670, 89, 1, 23, "Print"],
Cell[498378, 13673, 92, 1, 23, "Print"],
Cell[498473, 13676, 200, 5, 23, "Print"],
Cell[498676, 13683, 117, 1, 23, "Print"],
Cell[498796, 13686, 417, 9, 23, "Print"],
Cell[499216, 13697, 363, 8, 23, "Print"],
Cell[499582, 13707, 201, 5, 23, "Print"],
Cell[499786, 13714, 313, 7, 23, "Print"],
Cell[500102, 13723, 200, 5, 23, "Print"],
Cell[500305, 13730, 117, 1, 23, "Print"],
Cell[500425, 13733, 421, 9, 23, "Print"],
Cell[500849, 13744, 364, 8, 23, "Print"],
Cell[501216, 13754, 201, 5, 23, "Print"],
Cell[501420, 13761, 311, 7, 23, "Print"],
Cell[501734, 13770, 200, 5, 23, "Print"],
Cell[501937, 13777, 117, 1, 23, "Print"],
Cell[502057, 13780, 421, 9, 23, "Print"],
Cell[502481, 13791, 364, 8, 23, "Print"],
Cell[502848, 13801, 201, 5, 23, "Print"],
Cell[503052, 13808, 309, 7, 23, "Print"],
Cell[503364, 13817, 198, 5, 23, "Print"],
Cell[503565, 13824, 115, 1, 23, "Print"],
Cell[503683, 13827, 415, 9, 23, "Print"],
Cell[504101, 13838, 362, 8, 23, "Print"],
Cell[504466, 13848, 199, 5, 23, "Print"],
Cell[504668, 13855, 373, 8, 23, "Print"],
Cell[505044, 13865, 198, 5, 23, "Print"],
Cell[505245, 13872, 115, 1, 23, "Print"],
Cell[505363, 13875, 355, 8, 23, "Print"],
Cell[505721, 13885, 362, 8, 23, "Print"],
Cell[506086, 13895, 201, 5, 23, "Print"],
Cell[506290, 13902, 375, 8, 23, "Print"],
Cell[506668, 13912, 200, 5, 23, "Print"],
Cell[506871, 13919, 117, 1, 23, "Print"],
Cell[506991, 13922, 355, 8, 23, "Print"],
Cell[507349, 13932, 98, 1, 23, "Print"],
Cell[507450, 13935, 222, 5, 23, "Print"],
Cell[507675, 13942, 224, 5, 23, "Print"],
Cell[507902, 13949, 364, 8, 23, "Print"],
Cell[508269, 13959, 201, 5, 23, "Print"],
Cell[508473, 13966, 375, 8, 23, "Print"],
Cell[508851, 13976, 198, 5, 23, "Print"],
Cell[509052, 13983, 115, 1, 23, "Print"],
Cell[509170, 13986, 357, 8, 23, "Print"],
Cell[509530, 13996, 98, 1, 23, "Print"],
Cell[509631, 13999, 222, 5, 23, "Print"],
Cell[509856, 14006, 224, 5, 23, "Print"],
Cell[510083, 14013, 273, 4, 62, "Print"],
Cell[510359, 14019, 488, 10, 23, "Print"],
Cell[510850, 14031, 259, 4, 43, "Print"],
Cell[511112, 14037, 337, 7, 23, "Print"],
Cell[511452, 14046, 202, 3, 23, "Print"],
Cell[511657, 14051, 414, 8, 23, "Print"],
Cell[512074, 14061, 236, 5, 23, "Print"],
Cell[512313, 14068, 153, 2, 23, "Print"],
Cell[512469, 14072, 401, 8, 23, "Print"],
Cell[512873, 14082, 168, 2, 23, "Print"],
Cell[513044, 14086, 400, 8, 23, "Print"],
Cell[513447, 14096, 364, 8, 23, "Print"],
Cell[513814, 14106, 364, 8, 23, "Print"],
Cell[514181, 14116, 201, 5, 23, "Print"],
Cell[514385, 14123, 315, 7, 23, "Print"],
Cell[514703, 14132, 515, 10, 23, "Print"],
Cell[515221, 14144, 126, 2, 23, "Print"],
Cell[515350, 14148, 94, 1, 23, "Print"],
Cell[515447, 14151, 119, 1, 23, "Print"],
Cell[515569, 14154, 130, 2, 23, "Print"],
Cell[515702, 14158, 273, 4, 62, "Print"],
Cell[515978, 14164, 490, 10, 23, "Print"],
Cell[516471, 14176, 261, 4, 43, "Print"],
Cell[516735, 14182, 340, 7, 23, "Print"],
Cell[517078, 14191, 377, 8, 23, "Print"],
Cell[517458, 14201, 119, 1, 23, "Print"],
Cell[517580, 14204, 160, 2, 23, "Print"],
Cell[517743, 14208, 86, 1, 23, "Print"],
Cell[517832, 14211, 89, 1, 23, "Print"],
Cell[517924, 14214, 92, 1, 23, "Print"],
Cell[518019, 14217, 200, 5, 23, "Print"],
Cell[518222, 14224, 117, 1, 23, "Print"],
Cell[518342, 14227, 419, 9, 23, "Print"],
Cell[518764, 14238, 364, 8, 23, "Print"],
Cell[519131, 14248, 201, 5, 23, "Print"],
Cell[519335, 14255, 373, 8, 23, "Print"],
Cell[519711, 14265, 200, 5, 23, "Print"],
Cell[519914, 14272, 115, 1, 23, "Print"],
Cell[520032, 14275, 417, 9, 23, "Print"],
Cell[520452, 14286, 362, 8, 23, "Print"],
Cell[520817, 14296, 199, 5, 23, "Print"],
Cell[521019, 14303, 309, 7, 23, "Print"],
Cell[521331, 14312, 198, 5, 23, "Print"],
Cell[521532, 14319, 115, 1, 23, "Print"],
Cell[521650, 14322, 355, 8, 23, "Print"],
Cell[522008, 14332, 364, 8, 23, "Print"],
Cell[522375, 14342, 201, 5, 23, "Print"],
Cell[522579, 14349, 311, 7, 23, "Print"],
Cell[522893, 14358, 198, 5, 23, "Print"],
Cell[523094, 14365, 115, 1, 23, "Print"],
Cell[523212, 14368, 419, 9, 23, "Print"],
Cell[523634, 14379, 359, 8, 23, "Print"],
Cell[523996, 14389, 199, 5, 23, "Print"],
Cell[524198, 14396, 373, 8, 23, "Print"],
Cell[524574, 14406, 198, 5, 23, "Print"],
Cell[524775, 14413, 115, 1, 23, "Print"],
Cell[524893, 14416, 419, 9, 23, "Print"],
Cell[525315, 14427, 361, 8, 23, "Print"],
Cell[525679, 14437, 201, 5, 23, "Print"],
Cell[525883, 14444, 373, 8, 23, "Print"],
Cell[526259, 14454, 198, 5, 23, "Print"],
Cell[526460, 14461, 115, 1, 23, "Print"],
Cell[526578, 14464, 421, 9, 23, "Print"],
Cell[527002, 14475, 96, 1, 23, "Print"],
Cell[527101, 14478, 220, 5, 23, "Print"],
Cell[527324, 14485, 222, 5, 23, "Print"],
Cell[527549, 14492, 362, 8, 23, "Print"],
Cell[527914, 14502, 199, 5, 23, "Print"],
Cell[528116, 14509, 373, 8, 23, "Print"],
Cell[528492, 14519, 198, 5, 23, "Print"],
Cell[528693, 14526, 117, 1, 23, "Print"],
Cell[528813, 14529, 419, 9, 23, "Print"],
Cell[529235, 14540, 98, 1, 23, "Print"],
Cell[529336, 14543, 222, 5, 23, "Print"],
Cell[529561, 14550, 224, 5, 23, "Print"],
Cell[529788, 14557, 273, 4, 62, "Print"],
Cell[530064, 14563, 491, 10, 23, "Print"],
Cell[530558, 14575, 261, 4, 43, "Print"],
Cell[530822, 14581, 338, 7, 23, "Print"],
Cell[531163, 14590, 204, 3, 23, "Print"],
Cell[531370, 14595, 414, 8, 23, "Print"],
Cell[531787, 14605, 235, 5, 23, "Print"],
Cell[532025, 14612, 155, 2, 23, "Print"],
Cell[532183, 14616, 399, 8, 23, "Print"],
Cell[532585, 14626, 170, 2, 23, "Print"],
Cell[532758, 14630, 407, 8, 23, "Print"],
Cell[533168, 14640, 366, 8, 23, "Print"],
Cell[533537, 14650, 364, 8, 23, "Print"],
Cell[533904, 14660, 201, 5, 23, "Print"],
Cell[534108, 14667, 315, 7, 23, "Print"],
Cell[534426, 14676, 515, 10, 23, "Print"],
Cell[534944, 14688, 126, 2, 23, "Print"],
Cell[535073, 14692, 93, 1, 23, "Print"],
Cell[535169, 14695, 119, 1, 23, "Print"],
Cell[535291, 14698, 130, 2, 23, "Print"],
Cell[535424, 14702, 273, 4, 62, "Print"],
Cell[535700, 14708, 488, 10, 23, "Print"],
Cell[536191, 14720, 259, 4, 43, "Print"],
Cell[536453, 14726, 338, 7, 23, "Print"],
Cell[536794, 14735, 377, 8, 23, "Print"],
Cell[537174, 14745, 117, 1, 23, "Print"],
Cell[537294, 14748, 160, 2, 23, "Print"],
Cell[537457, 14752, 84, 1, 23, "Print"],
Cell[537544, 14755, 77, 1, 23, "Print"],
Cell[537624, 14758, 92, 1, 23, "Print"],
Cell[537719, 14761, 200, 5, 23, "Print"],
Cell[537922, 14768, 117, 1, 23, "Print"],
Cell[538042, 14771, 419, 9, 23, "Print"],
Cell[538464, 14782, 357, 8, 23, "Print"],
Cell[538824, 14792, 201, 5, 23, "Print"],
Cell[539028, 14799, 375, 8, 23, "Print"],
Cell[539406, 14809, 200, 5, 23, "Print"],
Cell[539609, 14816, 117, 1, 23, "Print"],
Cell[539729, 14819, 421, 9, 23, "Print"],
Cell[540153, 14830, 362, 8, 23, "Print"],
Cell[540518, 14840, 199, 5, 23, "Print"],
Cell[540720, 14847, 371, 8, 23, "Print"],
Cell[541094, 14857, 198, 5, 23, "Print"],
Cell[541295, 14864, 115, 1, 23, "Print"],
Cell[541413, 14867, 419, 9, 23, "Print"],
Cell[541835, 14878, 364, 8, 23, "Print"],
Cell[542202, 14888, 201, 5, 23, "Print"],
Cell[542406, 14895, 373, 8, 23, "Print"],
Cell[542782, 14905, 200, 5, 23, "Print"],
Cell[542985, 14912, 117, 1, 23, "Print"],
Cell[543105, 14915, 355, 8, 23, "Print"],
Cell[543463, 14925, 364, 8, 23, "Print"],
Cell[543830, 14935, 201, 5, 23, "Print"],
Cell[544034, 14942, 375, 8, 23, "Print"],
Cell[544412, 14952, 200, 5, 23, "Print"],
Cell[544615, 14959, 117, 1, 23, "Print"],
Cell[544735, 14962, 419, 9, 23, "Print"],
Cell[545157, 14973, 362, 8, 23, "Print"],
Cell[545522, 14983, 199, 5, 23, "Print"],
Cell[545724, 14990, 373, 8, 23, "Print"],
Cell[546100, 15000, 198, 5, 23, "Print"],
Cell[546301, 15007, 115, 1, 23, "Print"],
Cell[546419, 15010, 419, 9, 23, "Print"],
Cell[546841, 15021, 96, 1, 23, "Print"],
Cell[546940, 15024, 222, 5, 23, "Print"],
Cell[547165, 15031, 222, 5, 23, "Print"],
Cell[547390, 15038, 361, 8, 23, "Print"],
Cell[547754, 15048, 201, 5, 23, "Print"],
Cell[547958, 15055, 372, 8, 23, "Print"],
Cell[548333, 15065, 197, 5, 23, "Print"],
Cell[548533, 15072, 114, 1, 23, "Print"],
Cell[548650, 15075, 355, 8, 23, "Print"],
Cell[549008, 15085, 96, 1, 23, "Print"],
Cell[549107, 15088, 219, 5, 23, "Print"],
Cell[549329, 15095, 222, 5, 23, "Print"],
Cell[549554, 15102, 273, 4, 62, "Print"],
Cell[549830, 15108, 490, 10, 23, "Print"],
Cell[550323, 15120, 261, 4, 43, "Print"],
Cell[550587, 15126, 336, 7, 23, "Print"],
Cell[550926, 15135, 204, 3, 23, "Print"],
Cell[551133, 15140, 414, 8, 23, "Print"],
Cell[551550, 15150, 238, 5, 23, "Print"],
Cell[551791, 15157, 155, 2, 23, "Print"],
Cell[551949, 15161, 405, 8, 23, "Print"],
Cell[552357, 15171, 170, 2, 23, "Print"],
Cell[552530, 15175, 403, 8, 23, "Print"],
Cell[552936, 15185, 367, 8, 23, "Print"],
Cell[553306, 15195, 364, 8, 23, "Print"],
Cell[553673, 15205, 198, 5, 23, "Print"],
Cell[553874, 15212, 314, 7, 23, "Print"],
Cell[554191, 15221, 512, 10, 23, "Print"],
Cell[554706, 15233, 126, 2, 23, "Print"],
Cell[554835, 15237, 91, 1, 23, "Print"],
Cell[554929, 15240, 116, 1, 23, "Print"],
Cell[555048, 15243, 130, 2, 23, "Print"],
Cell[555181, 15247, 273, 4, 62, "Print"],
Cell[555457, 15253, 488, 10, 23, "Print"],
Cell[555948, 15265, 261, 4, 43, "Print"],
Cell[556212, 15271, 341, 7, 23, "Print"],
Cell[556556, 15280, 375, 8, 23, "Print"],
Cell[556934, 15290, 119, 1, 23, "Print"],
Cell[557056, 15293, 162, 2, 23, "Print"],
Cell[557221, 15297, 86, 1, 23, "Print"],
Cell[557310, 15300, 86, 1, 23, "Print"],
Cell[557399, 15303, 90, 1, 23, "Print"],
Cell[557492, 15306, 198, 5, 23, "Print"],
Cell[557693, 15313, 115, 1, 23, "Print"],
Cell[557811, 15316, 419, 9, 23, "Print"],
Cell[558233, 15327, 361, 8, 23, "Print"],
Cell[558597, 15337, 201, 5, 23, "Print"],
Cell[558801, 15344, 311, 7, 23, "Print"],
Cell[559115, 15353, 198, 5, 23, "Print"],
Cell[559316, 15360, 115, 1, 23, "Print"],
Cell[559434, 15363, 419, 9, 23, "Print"],
Cell[559856, 15374, 359, 8, 23, "Print"],
Cell[560218, 15384, 201, 5, 23, "Print"],
Cell[560422, 15391, 375, 8, 23, "Print"],
Cell[560800, 15401, 200, 5, 23, "Print"],
Cell[561003, 15408, 117, 1, 23, "Print"],
Cell[561123, 15411, 357, 8, 23, "Print"],
Cell[561483, 15421, 364, 8, 23, "Print"],
Cell[561850, 15431, 201, 5, 23, "Print"],
Cell[562054, 15438, 375, 8, 23, "Print"],
Cell[562432, 15448, 200, 5, 23, "Print"],
Cell[562635, 15455, 117, 1, 23, "Print"],
Cell[562755, 15458, 417, 9, 23, "Print"],
Cell[563175, 15469, 364, 8, 23, "Print"],
Cell[563542, 15479, 201, 5, 23, "Print"],
Cell[563746, 15486, 313, 7, 23, "Print"],
Cell[564062, 15495, 200, 5, 23, "Print"],
Cell[564265, 15502, 117, 1, 23, "Print"],
Cell[564385, 15505, 419, 9, 23, "Print"],
Cell[564807, 15516, 359, 8, 23, "Print"],
Cell[565169, 15526, 199, 5, 23, "Print"],
Cell[565371, 15533, 307, 7, 23, "Print"],
Cell[565681, 15542, 200, 5, 23, "Print"],
Cell[565884, 15549, 117, 1, 23, "Print"],
Cell[566004, 15552, 421, 9, 23, "Print"],
Cell[566428, 15563, 98, 1, 23, "Print"],
Cell[566529, 15566, 220, 5, 23, "Print"],
Cell[566752, 15573, 222, 5, 23, "Print"],
Cell[566977, 15580, 360, 8, 23, "Print"],
Cell[567340, 15590, 199, 5, 23, "Print"],
Cell[567542, 15597, 373, 8, 23, "Print"],
Cell[567918, 15607, 198, 5, 23, "Print"],
Cell[568119, 15614, 115, 1, 23, "Print"],
Cell[568237, 15617, 355, 8, 23, "Print"],
Cell[568595, 15627, 96, 1, 23, "Print"],
Cell[568694, 15630, 220, 5, 23, "Print"],
Cell[568917, 15637, 224, 5, 23, "Print"],
Cell[569144, 15644, 273, 4, 62, "Print"],
Cell[569420, 15650, 488, 10, 23, "Print"],
Cell[569911, 15662, 261, 4, 43, "Print"],
Cell[570175, 15668, 340, 7, 23, "Print"],
Cell[570518, 15677, 202, 3, 23, "Print"],
Cell[570723, 15682, 410, 8, 23, "Print"],
Cell[571136, 15692, 238, 5, 23, "Print"],
Cell[571377, 15699, 155, 2, 23, "Print"],
Cell[571535, 15703, 403, 8, 23, "Print"],
Cell[571941, 15713, 170, 2, 23, "Print"],
Cell[572114, 15717, 402, 8, 23, "Print"],
Cell[572519, 15727, 362, 8, 23, "Print"],
Cell[572884, 15737, 362, 8, 23, "Print"],
Cell[573249, 15747, 199, 5, 23, "Print"],
Cell[573451, 15754, 377, 8, 23, "Print"],
Cell[573831, 15764, 449, 9, 23, "Print"],
Cell[574283, 15775, 124, 2, 23, "Print"],
Cell[574410, 15779, 90, 1, 23, "Print"],
Cell[574503, 15782, 117, 1, 23, "Print"],
Cell[574623, 15785, 128, 2, 23, "Print"],
Cell[574754, 15789, 273, 4, 62, "Print"],
Cell[575030, 15795, 488, 10, 23, "Print"],
Cell[575521, 15807, 261, 4, 43, "Print"],
Cell[575785, 15813, 341, 7, 23, "Print"],
Cell[576129, 15822, 375, 8, 23, "Print"],
Cell[576507, 15832, 119, 1, 23, "Print"],
Cell[576629, 15835, 162, 2, 23, "Print"],
Cell[576794, 15839, 86, 1, 23, "Print"],
Cell[576883, 15842, 89, 1, 23, "Print"],
Cell[576975, 15845, 92, 1, 23, "Print"],
Cell[577070, 15848, 200, 5, 23, "Print"],
Cell[577273, 15855, 117, 1, 23, "Print"],
Cell[577393, 15858, 357, 8, 23, "Print"],
Cell[577753, 15868, 364, 8, 23, "Print"],
Cell[578120, 15878, 201, 5, 23, "Print"],
Cell[578324, 15885, 375, 8, 23, "Print"],
Cell[578702, 15895, 200, 5, 23, "Print"],
Cell[578905, 15902, 117, 1, 23, "Print"],
Cell[579025, 15905, 421, 9, 23, "Print"],
Cell[579449, 15916, 359, 8, 23, "Print"],
Cell[579811, 15926, 199, 5, 23, "Print"],
Cell[580013, 15933, 373, 8, 23, "Print"],
Cell[580389, 15943, 198, 5, 23, "Print"],
Cell[580590, 15950, 115, 1, 23, "Print"],
Cell[580708, 15953, 421, 9, 23, "Print"],
Cell[581132, 15964, 362, 8, 23, "Print"],
Cell[581497, 15974, 201, 5, 23, "Print"],
Cell[581701, 15981, 311, 7, 23, "Print"],
Cell[582015, 15990, 200, 5, 23, "Print"],
Cell[582218, 15997, 117, 1, 23, "Print"],
Cell[582338, 16000, 421, 9, 23, "Print"],
Cell[582762, 16011, 362, 8, 23, "Print"],
Cell[583127, 16021, 199, 5, 23, "Print"],
Cell[583329, 16028, 311, 7, 23, "Print"],
Cell[583643, 16037, 198, 5, 23, "Print"],
Cell[583844, 16044, 115, 1, 23, "Print"],
Cell[583962, 16047, 359, 8, 23, "Print"],
Cell[584324, 16057, 364, 8, 23, "Print"],
Cell[584691, 16067, 199, 5, 23, "Print"],
Cell[584893, 16074, 309, 7, 23, "Print"],
Cell[585205, 16083, 198, 5, 23, "Print"],
Cell[585406, 16090, 115, 1, 23, "Print"],
Cell[585524, 16093, 421, 9, 23, "Print"],
Cell[585948, 16104, 98, 1, 23, "Print"],
Cell[586049, 16107, 222, 5, 23, "Print"],
Cell[586274, 16114, 221, 5, 23, "Print"],
Cell[586498, 16121, 359, 8, 23, "Print"],
Cell[586860, 16131, 201, 5, 23, "Print"],
Cell[587064, 16138, 311, 7, 23, "Print"],
Cell[587378, 16147, 198, 5, 23, "Print"],
Cell[587579, 16154, 115, 1, 23, "Print"],
Cell[587697, 16157, 421, 9, 23, "Print"],
Cell[588121, 16168, 98, 1, 23, "Print"],
Cell[588222, 16171, 222, 5, 23, "Print"],
Cell[588447, 16178, 224, 5, 23, "Print"],
Cell[588674, 16185, 271, 4, 62, "Print"],
Cell[588948, 16191, 488, 10, 23, "Print"],
Cell[589439, 16203, 261, 4, 43, "Print"],
Cell[589703, 16209, 340, 7, 23, "Print"],
Cell[590046, 16218, 202, 3, 23, "Print"],
Cell[590251, 16223, 416, 8, 23, "Print"],
Cell[590670, 16233, 238, 5, 23, "Print"],
Cell[590911, 16240, 153, 2, 23, "Print"],
Cell[591067, 16244, 400, 8, 23, "Print"],
Cell[591470, 16254, 170, 2, 23, "Print"],
Cell[591643, 16258, 402, 8, 23, "Print"],
Cell[592048, 16268, 364, 8, 23, "Print"],
Cell[592415, 16278, 359, 8, 23, "Print"],
Cell[592777, 16288, 201, 5, 23, "Print"],
Cell[592981, 16295, 377, 8, 23, "Print"],
Cell[593361, 16305, 513, 10, 23, "Print"],
Cell[593877, 16317, 124, 2, 23, "Print"],
Cell[594004, 16321, 92, 1, 23, "Print"],
Cell[594099, 16324, 117, 1, 23, "Print"],
Cell[594219, 16327, 130, 2, 23, "Print"],
Cell[594352, 16331, 273, 4, 62, "Print"],
Cell[594628, 16337, 488, 10, 23, "Print"],
Cell[595119, 16349, 261, 4, 43, "Print"],
Cell[595383, 16355, 341, 7, 23, "Print"],
Cell[595727, 16364, 377, 8, 23, "Print"],
Cell[596107, 16374, 119, 1, 23, "Print"],
Cell[596229, 16377, 162, 2, 23, "Print"],
Cell[596394, 16381, 86, 1, 23, "Print"],
Cell[596483, 16384, 77, 1, 23, "Print"],
Cell[596563, 16387, 92, 1, 23, "Print"],
Cell[596658, 16390, 200, 5, 23, "Print"],
Cell[596861, 16397, 117, 1, 23, "Print"],
Cell[596981, 16400, 421, 9, 23, "Print"],
Cell[597405, 16411, 362, 8, 23, "Print"],
Cell[597770, 16421, 199, 5, 23, "Print"],
Cell[597972, 16428, 373, 8, 23, "Print"],
Cell[598348, 16438, 198, 5, 23, "Print"],
Cell[598549, 16445, 117, 1, 23, "Print"],
Cell[598669, 16448, 421, 9, 23, "Print"],
Cell[599093, 16459, 364, 8, 23, "Print"],
Cell[599460, 16469, 201, 5, 23, "Print"],
Cell[599664, 16476, 313, 7, 23, "Print"],
Cell[599980, 16485, 200, 5, 23, "Print"],
Cell[600183, 16492, 117, 1, 23, "Print"],
Cell[600303, 16495, 359, 8, 23, "Print"],
Cell[600665, 16505, 364, 8, 23, "Print"],
Cell[601032, 16515, 201, 5, 23, "Print"],
Cell[601236, 16522, 375, 8, 23, "Print"],
Cell[601614, 16532, 200, 5, 23, "Print"],
Cell[601817, 16539, 117, 1, 23, "Print"],
Cell[601937, 16542, 421, 9, 23, "Print"],
Cell[602361, 16553, 359, 8, 23, "Print"],
Cell[602723, 16563, 201, 5, 23, "Print"],
Cell[602927, 16570, 375, 8, 23, "Print"],
Cell[603305, 16580, 200, 5, 23, "Print"],
Cell[603508, 16587, 117, 1, 23, "Print"],
Cell[603628, 16590, 419, 9, 23, "Print"],
Cell[604050, 16601, 364, 8, 23, "Print"],
Cell[604417, 16611, 201, 5, 23, "Print"],
Cell[604621, 16618, 373, 8, 23, "Print"],
Cell[604997, 16628, 198, 5, 23, "Print"],
Cell[605198, 16635, 115, 1, 23, "Print"],
Cell[605316, 16638, 421, 9, 23, "Print"],
Cell[605740, 16649, 98, 1, 23, "Print"],
Cell[605841, 16652, 220, 5, 23, "Print"],
Cell[606064, 16659, 222, 5, 23, "Print"],
Cell[606289, 16666, 362, 8, 23, "Print"],
Cell[606654, 16676, 199, 5, 23, "Print"],
Cell[606856, 16683, 373, 8, 23, "Print"],
Cell[607232, 16693, 198, 5, 23, "Print"],
Cell[607433, 16700, 115, 1, 23, "Print"],
Cell[607551, 16703, 423, 9, 23, "Print"],
Cell[607977, 16714, 98, 1, 23, "Print"],
Cell[608078, 16717, 220, 5, 23, "Print"],
Cell[608301, 16724, 222, 5, 23, "Print"],
Cell[608526, 16731, 273, 4, 62, "Print"],
Cell[608802, 16737, 491, 10, 23, "Print"],
Cell[609296, 16749, 261, 4, 43, "Print"],
Cell[609560, 16755, 337, 7, 23, "Print"],
Cell[609900, 16764, 200, 3, 23, "Print"],
Cell[610103, 16769, 410, 8, 23, "Print"],
Cell[610516, 16779, 234, 5, 23, "Print"],
Cell[610753, 16786, 151, 2, 23, "Print"],
Cell[610907, 16790, 399, 8, 23, "Print"],
Cell[611309, 16800, 166, 2, 23, "Print"],
Cell[611478, 16804, 399, 8, 23, "Print"],
Cell[611880, 16814, 367, 8, 23, "Print"],
Cell[612250, 16824, 360, 8, 23, "Print"],
Cell[612613, 16834, 197, 5, 23, "Print"],
Cell[612813, 16841, 375, 8, 23, "Print"],
Cell[613191, 16851, 447, 9, 23, "Print"],
Cell[613641, 16862, 122, 2, 23, "Print"],
Cell[613766, 16866, 93, 1, 23, "Print"],
Cell[613862, 16869, 119, 1, 23, "Print"],
Cell[613984, 16872, 130, 2, 23, "Print"],
Cell[614117, 16876, 271, 4, 62, "Print"],
Cell[614391, 16882, 488, 10, 23, "Print"],
Cell[614882, 16894, 261, 4, 43, "Print"],
Cell[615146, 16900, 341, 7, 23, "Print"],
Cell[615490, 16909, 375, 8, 23, "Print"],
Cell[615868, 16919, 119, 1, 23, "Print"],
Cell[615990, 16922, 162, 2, 23, "Print"],
Cell[616155, 16926, 86, 1, 23, "Print"],
Cell[616244, 16929, 76, 1, 23, "Print"],
Cell[616323, 16932, 90, 1, 23, "Print"],
Cell[616416, 16935, 198, 5, 23, "Print"],
Cell[616617, 16942, 117, 1, 23, "Print"],
Cell[616737, 16945, 357, 8, 23, "Print"],
Cell[617097, 16955, 361, 8, 23, "Print"],
Cell[617461, 16965, 201, 5, 23, "Print"],
Cell[617665, 16972, 313, 7, 23, "Print"],
Cell[617981, 16981, 200, 5, 23, "Print"],
Cell[618184, 16988, 117, 1, 23, "Print"],
Cell[618304, 16991, 357, 8, 23, "Print"],
Cell[618664, 17001, 364, 8, 23, "Print"],
Cell[619031, 17011, 201, 5, 23, "Print"],
Cell[619235, 17018, 375, 8, 23, "Print"],
Cell[619613, 17028, 200, 5, 23, "Print"],
Cell[619816, 17035, 117, 1, 23, "Print"],
Cell[619936, 17038, 357, 8, 23, "Print"],
Cell[620296, 17048, 362, 8, 23, "Print"],
Cell[620661, 17058, 199, 5, 23, "Print"],
Cell[620863, 17065, 371, 8, 23, "Print"],
Cell[621237, 17075, 198, 5, 23, "Print"],
Cell[621438, 17082, 115, 1, 23, "Print"],
Cell[621556, 17085, 355, 8, 23, "Print"],
Cell[621914, 17095, 357, 8, 23, "Print"],
Cell[622274, 17105, 197, 5, 23, "Print"],
Cell[622474, 17112, 373, 8, 23, "Print"],
Cell[622850, 17122, 198, 5, 23, "Print"],
Cell[623051, 17129, 115, 1, 23, "Print"],
Cell[623169, 17132, 421, 9, 23, "Print"],
Cell[623593, 17143, 364, 8, 23, "Print"],
Cell[623960, 17153, 201, 5, 23, "Print"],
Cell[624164, 17160, 313, 7, 23, "Print"],
Cell[624480, 17169, 200, 5, 23, "Print"],
Cell[624683, 17176, 117, 1, 23, "Print"],
Cell[624803, 17179, 421, 9, 23, "Print"],
Cell[625227, 17190, 98, 1, 23, "Print"],
Cell[625328, 17193, 222, 5, 23, "Print"],
Cell[625553, 17200, 224, 5, 23, "Print"],
Cell[625780, 17207, 362, 8, 23, "Print"],
Cell[626145, 17217, 199, 5, 23, "Print"],
Cell[626347, 17224, 373, 8, 23, "Print"],
Cell[626723, 17234, 198, 5, 23, "Print"],
Cell[626924, 17241, 115, 1, 23, "Print"],
Cell[627042, 17244, 421, 9, 23, "Print"],
Cell[627466, 17255, 98, 1, 23, "Print"],
Cell[627567, 17258, 222, 5, 23, "Print"],
Cell[627792, 17265, 224, 5, 23, "Print"],
Cell[628019, 17272, 273, 4, 62, "Print"],
Cell[628295, 17278, 490, 10, 23, "Print"],
Cell[628788, 17290, 259, 4, 43, "Print"],
Cell[629050, 17296, 337, 7, 23, "Print"],
Cell[629390, 17305, 204, 3, 23, "Print"],
Cell[629597, 17310, 415, 8, 23, "Print"],
Cell[630015, 17320, 233, 5, 23, "Print"],
Cell[630251, 17327, 153, 2, 23, "Print"],
Cell[630407, 17331, 403, 8, 23, "Print"],
Cell[630813, 17341, 170, 2, 23, "Print"],
Cell[630986, 17345, 403, 8, 23, "Print"],
Cell[631392, 17355, 364, 8, 23, "Print"],
Cell[631759, 17365, 364, 8, 23, "Print"],
Cell[632126, 17375, 201, 5, 23, "Print"],
Cell[632330, 17382, 377, 8, 23, "Print"],
Cell[632710, 17392, 513, 10, 23, "Print"],
Cell[633226, 17404, 124, 2, 23, "Print"],
Cell[633353, 17408, 91, 1, 23, "Print"],
Cell[633447, 17411, 117, 1, 23, "Print"],
Cell[633567, 17414, 128, 2, 23, "Print"],
Cell[633698, 17418, 273, 4, 62, "Print"],
Cell[633974, 17424, 490, 10, 23, "Print"],
Cell[634467, 17436, 261, 4, 43, "Print"],
Cell[634731, 17442, 340, 7, 23, "Print"],
Cell[635074, 17451, 377, 8, 23, "Print"],
Cell[635454, 17461, 119, 1, 23, "Print"],
Cell[635576, 17464, 162, 2, 23, "Print"],
Cell[635741, 17468, 86, 1, 23, "Print"],
Cell[635830, 17471, 89, 1, 23, "Print"],
Cell[635922, 17474, 90, 1, 23, "Print"],
Cell[636015, 17477, 198, 5, 23, "Print"],
Cell[636216, 17484, 115, 1, 23, "Print"],
Cell[636334, 17487, 421, 9, 23, "Print"],
Cell[636758, 17498, 362, 8, 23, "Print"],
Cell[637123, 17508, 199, 5, 23, "Print"],
Cell[637325, 17515, 373, 8, 23, "Print"],
Cell[637701, 17525, 200, 5, 23, "Print"],
Cell[637904, 17532, 115, 1, 23, "Print"],
Cell[638022, 17535, 357, 8, 23, "Print"],
Cell[638382, 17545, 362, 8, 23, "Print"],
Cell[638747, 17555, 199, 5, 23, "Print"],
Cell[638949, 17562, 307, 7, 23, "Print"],
Cell[639259, 17571, 198, 5, 23, "Print"],
Cell[639460, 17578, 115, 1, 23, "Print"],
Cell[639578, 17581, 419, 9, 23, "Print"],
Cell[640000, 17592, 361, 8, 23, "Print"],
Cell[640364, 17602, 201, 5, 23, "Print"],
Cell[640568, 17609, 313, 7, 23, "Print"],
Cell[640884, 17618, 200, 5, 23, "Print"],
Cell[641087, 17625, 117, 1, 23, "Print"],
Cell[641207, 17628, 355, 8, 23, "Print"],
Cell[641565, 17638, 364, 8, 23, "Print"],
Cell[641932, 17648, 201, 5, 23, "Print"],
Cell[642136, 17655, 375, 8, 23, "Print"],
Cell[642514, 17665, 200, 5, 23, "Print"],
Cell[642717, 17672, 117, 1, 23, "Print"],
Cell[642837, 17675, 421, 9, 23, "Print"],
Cell[643261, 17686, 362, 8, 23, "Print"],
Cell[643626, 17696, 199, 5, 23, "Print"],
Cell[643828, 17703, 307, 7, 23, "Print"],
Cell[644138, 17712, 198, 5, 23, "Print"],
Cell[644339, 17719, 115, 1, 23, "Print"],
Cell[644457, 17722, 419, 9, 23, "Print"],
Cell[644879, 17733, 96, 1, 23, "Print"],
Cell[644978, 17736, 220, 5, 23, "Print"],
Cell[645201, 17743, 224, 5, 23, "Print"],
Cell[645428, 17750, 359, 8, 23, "Print"],
Cell[645790, 17760, 201, 5, 23, "Print"],
Cell[645994, 17767, 372, 8, 23, "Print"],
Cell[646369, 17777, 197, 5, 23, "Print"],
Cell[646569, 17784, 114, 1, 23, "Print"],
Cell[646686, 17787, 357, 8, 23, "Print"],
Cell[647046, 17797, 98, 1, 23, "Print"],
Cell[647147, 17800, 222, 5, 23, "Print"],
Cell[647372, 17807, 224, 5, 23, "Print"],
Cell[647599, 17814, 273, 4, 62, "Print"],
Cell[647875, 17820, 487, 10, 23, "Print"],
Cell[648365, 17832, 261, 4, 43, "Print"],
Cell[648629, 17838, 336, 7, 23, "Print"],
Cell[648968, 17847, 202, 3, 23, "Print"],
Cell[649173, 17852, 406, 8, 23, "Print"],
Cell[649582, 17862, 235, 5, 23, "Print"],
Cell[649820, 17869, 155, 2, 23, "Print"],
Cell[649978, 17873, 402, 8, 23, "Print"],
Cell[650383, 17883, 170, 2, 23, "Print"],
Cell[650556, 17887, 402, 8, 23, "Print"],
Cell[650961, 17897, 361, 8, 23, "Print"],
Cell[651325, 17907, 362, 8, 23, "Print"],
Cell[651690, 17917, 199, 5, 23, "Print"],
Cell[651892, 17924, 379, 8, 23, "Print"],
Cell[652274, 17934, 515, 10, 23, "Print"],
Cell[652792, 17946, 126, 2, 23, "Print"],
Cell[652921, 17950, 90, 1, 23, "Print"],
Cell[653014, 17953, 117, 1, 23, "Print"],
Cell[653134, 17956, 130, 2, 23, "Print"],
Cell[653267, 17960, 271, 4, 62, "Print"],
Cell[653541, 17966, 488, 10, 23, "Print"],
Cell[654032, 17978, 259, 4, 43, "Print"],
Cell[654294, 17984, 338, 7, 23, "Print"],
Cell[654635, 17993, 375, 8, 23, "Print"],
Cell[655013, 18003, 117, 1, 23, "Print"],
Cell[655133, 18006, 160, 2, 23, "Print"],
Cell[655296, 18010, 84, 1, 23, "Print"],
Cell[655383, 18013, 77, 1, 23, "Print"],
Cell[655463, 18016, 77, 1, 23, "Print"],
Cell[655543, 18019, 200, 5, 23, "Print"],
Cell[655746, 18026, 117, 1, 23, "Print"],
Cell[655866, 18029, 419, 9, 23, "Print"],
Cell[656288, 18040, 364, 8, 23, "Print"],
Cell[656655, 18050, 201, 5, 23, "Print"],
Cell[656859, 18057, 375, 8, 23, "Print"],
Cell[657237, 18067, 200, 5, 23, "Print"],
Cell[657440, 18074, 117, 1, 23, "Print"],
Cell[657560, 18077, 357, 8, 23, "Print"],
Cell[657920, 18087, 362, 8, 23, "Print"],
Cell[658285, 18097, 199, 5, 23, "Print"],
Cell[658487, 18104, 375, 8, 23, "Print"],
Cell[658865, 18114, 198, 5, 23, "Print"],
Cell[659066, 18121, 115, 1, 23, "Print"],
Cell[659184, 18124, 419, 9, 23, "Print"],
Cell[659606, 18135, 361, 8, 23, "Print"],
Cell[659970, 18145, 198, 5, 23, "Print"],
Cell[660171, 18152, 372, 8, 23, "Print"],
Cell[660546, 18162, 200, 5, 23, "Print"],
Cell[660749, 18169, 117, 1, 23, "Print"],
Cell[660869, 18172, 357, 8, 23, "Print"],
Cell[661229, 18182, 362, 8, 23, "Print"],
Cell[661594, 18192, 199, 5, 23, "Print"],
Cell[661796, 18199, 373, 8, 23, "Print"],
Cell[662172, 18209, 198, 5, 23, "Print"],
Cell[662373, 18216, 115, 1, 23, "Print"],
Cell[662491, 18219, 421, 9, 23, "Print"],
Cell[662915, 18230, 364, 8, 23, "Print"],
Cell[663282, 18240, 201, 5, 23, "Print"],
Cell[663486, 18247, 375, 8, 23, "Print"],
Cell[663864, 18257, 200, 5, 23, "Print"],
Cell[664067, 18264, 117, 1, 23, "Print"],
Cell[664187, 18267, 419, 9, 23, "Print"],
Cell[664609, 18278, 98, 1, 23, "Print"],
Cell[664710, 18281, 222, 5, 23, "Print"],
Cell[664935, 18288, 224, 5, 23, "Print"],
Cell[665162, 18295, 362, 8, 23, "Print"],
Cell[665527, 18305, 199, 5, 23, "Print"],
Cell[665729, 18312, 307, 7, 23, "Print"],
Cell[666039, 18321, 198, 5, 23, "Print"],
Cell[666240, 18328, 117, 1, 23, "Print"],
Cell[666360, 18331, 421, 9, 23, "Print"],
Cell[666784, 18342, 98, 1, 23, "Print"],
Cell[666885, 18345, 222, 5, 23, "Print"],
Cell[667110, 18352, 222, 5, 23, "Print"],
Cell[667335, 18359, 271, 4, 62, "Print"],
Cell[667609, 18365, 487, 10, 23, "Print"],
Cell[668099, 18377, 261, 4, 43, "Print"],
Cell[668363, 18383, 339, 7, 23, "Print"],
Cell[668705, 18392, 202, 3, 23, "Print"],
Cell[668910, 18397, 404, 8, 23, "Print"],
Cell[669317, 18407, 236, 5, 23, "Print"],
Cell[669556, 18414, 153, 2, 23, "Print"],
Cell[669712, 18418, 400, 8, 23, "Print"],
Cell[670115, 18428, 168, 2, 23, "Print"],
Cell[670286, 18432, 400, 8, 23, "Print"],
Cell[670689, 18442, 365, 8, 23, "Print"],
Cell[671057, 18452, 362, 8, 23, "Print"],
Cell[671422, 18462, 199, 5, 23, "Print"],
Cell[671624, 18469, 375, 8, 23, "Print"],
Cell[672002, 18479, 513, 10, 23, "Print"],
Cell[672518, 18491, 124, 2, 23, "Print"],
Cell[672645, 18495, 95, 1, 23, "Print"],
Cell[672743, 18498, 117, 1, 23, "Print"],
Cell[672863, 18501, 128, 2, 23, "Print"],
Cell[672994, 18505, 273, 4, 62, "Print"],
Cell[673270, 18511, 490, 10, 23, "Print"],
Cell[673763, 18523, 261, 4, 43, "Print"],
Cell[674027, 18529, 340, 7, 23, "Print"],
Cell[674370, 18538, 375, 8, 23, "Print"],
Cell[674748, 18548, 119, 1, 23, "Print"],
Cell[674870, 18551, 162, 2, 23, "Print"],
Cell[675035, 18555, 86, 1, 23, "Print"],
Cell[675124, 18558, 79, 1, 23, "Print"],
Cell[675206, 18561, 77, 1, 23, "Print"],
Cell[675286, 18564, 200, 5, 23, "Print"],
Cell[675489, 18571, 117, 1, 23, "Print"],
Cell[675609, 18574, 417, 9, 23, "Print"],
Cell[676029, 18585, 360, 8, 23, "Print"],
Cell[676392, 18595, 201, 5, 23, "Print"],
Cell[676596, 18602, 375, 8, 23, "Print"],
Cell[676974, 18612, 200, 5, 23, "Print"],
Cell[677177, 18619, 117, 1, 23, "Print"],
Cell[677297, 18622, 421, 9, 23, "Print"],
Cell[677721, 18633, 362, 8, 23, "Print"],
Cell[678086, 18643, 199, 5, 23, "Print"],
Cell[678288, 18650, 373, 8, 23, "Print"],
Cell[678664, 18660, 200, 5, 23, "Print"],
Cell[678867, 18667, 117, 1, 23, "Print"],
Cell[678987, 18670, 421, 9, 23, "Print"],
Cell[679411, 18681, 364, 8, 23, "Print"],
Cell[679778, 18691, 201, 5, 23, "Print"],
Cell[679982, 18698, 305, 7, 23, "Print"],
Cell[680290, 18707, 200, 5, 23, "Print"],
Cell[680493, 18714, 117, 1, 23, "Print"],
Cell[680613, 18717, 419, 9, 23, "Print"],
Cell[681035, 18728, 361, 8, 23, "Print"],
Cell[681399, 18738, 199, 5, 23, "Print"],
Cell[681601, 18745, 375, 8, 23, "Print"],
Cell[681979, 18755, 200, 5, 23, "Print"],
Cell[682182, 18762, 117, 1, 23, "Print"],
Cell[682302, 18765, 421, 9, 23, "Print"],
Cell[682726, 18776, 362, 8, 23, "Print"],
Cell[683091, 18786, 201, 5, 23, "Print"],
Cell[683295, 18793, 375, 8, 23, "Print"],
Cell[683673, 18803, 200, 5, 23, "Print"],
Cell[683876, 18810, 117, 1, 23, "Print"],
Cell[683996, 18813, 421, 9, 23, "Print"],
Cell[684420, 18824, 96, 1, 23, "Print"],
Cell[684519, 18827, 220, 5, 23, "Print"],
Cell[684742, 18834, 222, 5, 23, "Print"],
Cell[684967, 18841, 364, 8, 23, "Print"],
Cell[685334, 18851, 201, 5, 23, "Print"],
Cell[685538, 18858, 311, 7, 23, "Print"],
Cell[685852, 18867, 200, 5, 23, "Print"],
Cell[686055, 18874, 117, 1, 23, "Print"],
Cell[686175, 18877, 421, 9, 23, "Print"],
Cell[686599, 18888, 98, 1, 23, "Print"],
Cell[686700, 18891, 222, 5, 23, "Print"],
Cell[686925, 18898, 224, 5, 23, "Print"],
Cell[687152, 18905, 271, 4, 62, "Print"],
Cell[687426, 18911, 491, 10, 23, "Print"],
Cell[687920, 18923, 261, 4, 43, "Print"],
Cell[688184, 18929, 339, 7, 23, "Print"],
Cell[688526, 18938, 204, 3, 23, "Print"],
Cell[688733, 18943, 405, 8, 23, "Print"],
Cell[689141, 18953, 236, 5, 23, "Print"],
Cell[689380, 18960, 153, 2, 23, "Print"],
Cell[689536, 18964, 403, 8, 23, "Print"],
Cell[689942, 18974, 170, 2, 23, "Print"],
Cell[690115, 18978, 403, 8, 23, "Print"],
Cell[690521, 18988, 364, 8, 23, "Print"],
Cell[690888, 18998, 364, 8, 23, "Print"],
Cell[691255, 19008, 201, 5, 23, "Print"],
Cell[691459, 19015, 377, 8, 23, "Print"],
Cell[691839, 19025, 453, 9, 23, "Print"],
Cell[692295, 19036, 124, 2, 23, "Print"],
Cell[692422, 19040, 90, 1, 23, "Print"],
Cell[692515, 19043, 119, 1, 23, "Print"],
Cell[692637, 19046, 128, 2, 23, "Print"],
Cell[692768, 19050, 273, 4, 62, "Print"],
Cell[693044, 19056, 488, 10, 23, "Print"],
Cell[693535, 19068, 261, 4, 43, "Print"],
Cell[693799, 19074, 341, 7, 23, "Print"],
Cell[694143, 19083, 377, 8, 23, "Print"],
Cell[694523, 19093, 119, 1, 23, "Print"],
Cell[694645, 19096, 160, 2, 23, "Print"],
Cell[694808, 19100, 86, 1, 23, "Print"],
Cell[694897, 19103, 89, 1, 23, "Print"],
Cell[694989, 19106, 92, 1, 23, "Print"],
Cell[695084, 19109, 200, 5, 23, "Print"],
Cell[695287, 19116, 117, 1, 23, "Print"],
Cell[695407, 19119, 357, 8, 23, "Print"],
Cell[695767, 19129, 362, 8, 23, "Print"],
Cell[696132, 19139, 199, 5, 23, "Print"],
Cell[696334, 19146, 311, 7, 23, "Print"],
Cell[696648, 19155, 200, 5, 23, "Print"],
Cell[696851, 19162, 117, 1, 23, "Print"],
Cell[696971, 19165, 419, 9, 23, "Print"],
Cell[697393, 19176, 362, 8, 23, "Print"],
Cell[697758, 19186, 199, 5, 23, "Print"],
Cell[697960, 19193, 373, 8, 23, "Print"],
Cell[698336, 19203, 198, 5, 23, "Print"],
Cell[698537, 19210, 115, 1, 23, "Print"],
Cell[698655, 19213, 421, 9, 23, "Print"],
Cell[699079, 19224, 361, 8, 23, "Print"],
Cell[699443, 19234, 196, 5, 23, "Print"],
Cell[699642, 19241, 372, 8, 23, "Print"],
Cell[700017, 19251, 197, 5, 23, "Print"],
Cell[700217, 19258, 114, 1, 23, "Print"],
Cell[700334, 19261, 421, 9, 23, "Print"],
Cell[700758, 19272, 364, 8, 23, "Print"],
Cell[701125, 19282, 201, 5, 23, "Print"],
Cell[701329, 19289, 375, 8, 23, "Print"],
Cell[701707, 19299, 200, 5, 23, "Print"],
Cell[701910, 19306, 117, 1, 23, "Print"],
Cell[702030, 19309, 421, 9, 23, "Print"],
Cell[702454, 19320, 362, 8, 23, "Print"],
Cell[702819, 19330, 199, 5, 23, "Print"],
Cell[703021, 19337, 307, 7, 23, "Print"],
Cell[703331, 19346, 198, 5, 23, "Print"],
Cell[703532, 19353, 115, 1, 23, "Print"],
Cell[703650, 19356, 421, 9, 23, "Print"],
Cell[704074, 19367, 98, 1, 23, "Print"],
Cell[704175, 19370, 220, 5, 23, "Print"],
Cell[704398, 19377, 222, 5, 23, "Print"],
Cell[704623, 19384, 364, 8, 23, "Print"],
Cell[704990, 19394, 201, 5, 23, "Print"],
Cell[705194, 19401, 309, 7, 23, "Print"],
Cell[705506, 19410, 198, 5, 23, "Print"],
Cell[705707, 19417, 115, 1, 23, "Print"],
Cell[705825, 19420, 355, 8, 23, "Print"],
Cell[706183, 19430, 96, 1, 23, "Print"],
Cell[706282, 19433, 222, 5, 23, "Print"],
Cell[706507, 19440, 224, 5, 23, "Print"],
Cell[706734, 19447, 271, 4, 62, "Print"],
Cell[707008, 19453, 485, 10, 23, "Print"],
Cell[707496, 19465, 261, 4, 43, "Print"],
Cell[707760, 19471, 340, 7, 23, "Print"],
Cell[708103, 19480, 202, 3, 23, "Print"],
Cell[708308, 19485, 411, 8, 23, "Print"],
Cell[708722, 19495, 236, 5, 23, "Print"],
Cell[708961, 19502, 155, 2, 23, "Print"],
Cell[709119, 19506, 404, 8, 23, "Print"],
Cell[709526, 19516, 168, 2, 23, "Print"],
Cell[709697, 19520, 403, 8, 23, "Print"],
Cell[710103, 19530, 364, 8, 23, "Print"],
Cell[710470, 19540, 362, 8, 23, "Print"],
Cell[710835, 19550, 199, 5, 23, "Print"],
Cell[711037, 19557, 377, 8, 23, "Print"],
Cell[711417, 19567, 515, 10, 23, "Print"],
Cell[711935, 19579, 126, 2, 23, "Print"],
Cell[712064, 19583, 97, 1, 23, "Print"],
Cell[712164, 19586, 119, 1, 23, "Print"],
Cell[712286, 19589, 130, 2, 23, "Print"],
Cell[712419, 19593, 273, 4, 62, "Print"],
Cell[712695, 19599, 488, 10, 23, "Print"],
Cell[713186, 19611, 261, 4, 43, "Print"],
Cell[713450, 19617, 341, 7, 23, "Print"],
Cell[713794, 19626, 379, 8, 23, "Print"],
Cell[714176, 19636, 119, 1, 23, "Print"],
Cell[714298, 19639, 162, 2, 23, "Print"],
Cell[714463, 19643, 86, 1, 23, "Print"],
Cell[714552, 19646, 89, 1, 23, "Print"],
Cell[714644, 19649, 77, 1, 23, "Print"],
Cell[714724, 19652, 200, 5, 23, "Print"],
Cell[714927, 19659, 117, 1, 23, "Print"],
Cell[715047, 19662, 419, 9, 23, "Print"],
Cell[715469, 19673, 364, 8, 23, "Print"],
Cell[715836, 19683, 201, 5, 23, "Print"],
Cell[716040, 19690, 373, 8, 23, "Print"],
Cell[716416, 19700, 200, 5, 23, "Print"],
Cell[716619, 19707, 117, 1, 23, "Print"],
Cell[716739, 19710, 356, 8, 23, "Print"],
Cell[717098, 19720, 364, 8, 23, "Print"],
Cell[717465, 19730, 201, 5, 23, "Print"],
Cell[717669, 19737, 373, 8, 23, "Print"],
Cell[718045, 19747, 200, 5, 23, "Print"],
Cell[718248, 19754, 117, 1, 23, "Print"],
Cell[718368, 19757, 419, 9, 23, "Print"],
Cell[718790, 19768, 364, 8, 23, "Print"],
Cell[719157, 19778, 201, 5, 23, "Print"],
Cell[719361, 19785, 375, 8, 23, "Print"],
Cell[719739, 19795, 200, 5, 23, "Print"],
Cell[719942, 19802, 117, 1, 23, "Print"],
Cell[720062, 19805, 419, 9, 23, "Print"],
Cell[720484, 19816, 364, 8, 23, "Print"],
Cell[720851, 19826, 201, 5, 23, "Print"],
Cell[721055, 19833, 375, 8, 23, "Print"],
Cell[721433, 19843, 200, 5, 23, "Print"],
Cell[721636, 19850, 117, 1, 23, "Print"],
Cell[721756, 19853, 421, 9, 23, "Print"],
Cell[722180, 19864, 364, 8, 23, "Print"],
Cell[722547, 19874, 201, 5, 23, "Print"],
Cell[722751, 19881, 373, 8, 23, "Print"],
Cell[723127, 19891, 200, 5, 23, "Print"],
Cell[723330, 19898, 117, 1, 23, "Print"],
Cell[723450, 19901, 418, 9, 23, "Print"],
Cell[723871, 19912, 95, 1, 23, "Print"],
Cell[723969, 19915, 220, 5, 23, "Print"],
Cell[724192, 19922, 224, 5, 23, "Print"],
Cell[724419, 19929, 357, 8, 23, "Print"],
Cell[724779, 19939, 201, 5, 23, "Print"],
Cell[724983, 19946, 375, 8, 23, "Print"],
Cell[725361, 19956, 200, 5, 23, "Print"],
Cell[725564, 19963, 117, 1, 23, "Print"],
Cell[725684, 19966, 419, 9, 23, "Print"],
Cell[726106, 19977, 98, 1, 23, "Print"],
Cell[726207, 19980, 220, 5, 23, "Print"],
Cell[726430, 19987, 224, 5, 23, "Print"],
Cell[726657, 19994, 271, 4, 62, "Print"],
Cell[726931, 20000, 490, 10, 23, "Print"],
Cell[727424, 20012, 259, 4, 43, "Print"],
Cell[727686, 20018, 337, 7, 23, "Print"],
Cell[728026, 20027, 204, 3, 23, "Print"],
Cell[728233, 20032, 412, 8, 23, "Print"],
Cell[728648, 20042, 238, 5, 23, "Print"],
Cell[728889, 20049, 155, 2, 23, "Print"],
Cell[729047, 20053, 404, 8, 23, "Print"],
Cell[729454, 20063, 170, 2, 23, "Print"],
Cell[729627, 20067, 402, 8, 23, "Print"],
Cell[730032, 20077, 364, 8, 23, "Print"],
Cell[730399, 20087, 364, 8, 23, "Print"],
Cell[730766, 20097, 199, 5, 23, "Print"],
Cell[730968, 20104, 377, 8, 23, "Print"],
Cell[731348, 20114, 449, 9, 23, "Print"],
Cell[731800, 20125, 124, 2, 23, "Print"],
Cell[731927, 20129, 93, 1, 23, "Print"],
Cell[732023, 20132, 117, 1, 23, "Print"],
Cell[732143, 20135, 130, 2, 23, "Print"],
Cell[732276, 20139, 273, 4, 62, "Print"],
Cell[732552, 20145, 488, 10, 23, "Print"],
Cell[733043, 20157, 261, 4, 43, "Print"],
Cell[733307, 20163, 341, 7, 23, "Print"],
Cell[733651, 20172, 379, 8, 23, "Print"],
Cell[734033, 20182, 119, 1, 23, "Print"],
Cell[734155, 20185, 162, 2, 23, "Print"],
Cell[734320, 20189, 86, 1, 23, "Print"],
Cell[734409, 20192, 89, 1, 23, "Print"],
Cell[734501, 20195, 77, 1, 23, "Print"],
Cell[734581, 20198, 200, 5, 23, "Print"],
Cell[734784, 20205, 117, 1, 23, "Print"],
Cell[734904, 20208, 355, 8, 23, "Print"],
Cell[735262, 20218, 357, 8, 23, "Print"],
Cell[735622, 20228, 199, 5, 23, "Print"],
Cell[735824, 20235, 371, 8, 23, "Print"],
Cell[736198, 20245, 200, 5, 23, "Print"],
Cell[736401, 20252, 117, 1, 23, "Print"],
Cell[736521, 20255, 421, 9, 23, "Print"],
Cell[736945, 20266, 362, 8, 23, "Print"],
Cell[737310, 20276, 199, 5, 23, "Print"],
Cell[737512, 20283, 369, 8, 23, "Print"],
Cell[737884, 20293, 198, 5, 23, "Print"],
Cell[738085, 20300, 115, 1, 23, "Print"],
Cell[738203, 20303, 419, 9, 23, "Print"],
Cell[738625, 20314, 361, 8, 23, "Print"],
Cell[738989, 20324, 199, 5, 23, "Print"],
Cell[739191, 20331, 311, 7, 23, "Print"],
Cell[739505, 20340, 200, 5, 23, "Print"],
Cell[739708, 20347, 117, 1, 23, "Print"],
Cell[739828, 20350, 421, 9, 23, "Print"],
Cell[740252, 20361, 364, 8, 23, "Print"],
Cell[740619, 20371, 201, 5, 23, "Print"],
Cell[740823, 20378, 375, 8, 23, "Print"],
Cell[741201, 20388, 200, 5, 23, "Print"],
Cell[741404, 20395, 115, 1, 23, "Print"],
Cell[741522, 20398, 421, 9, 23, "Print"],
Cell[741946, 20409, 361, 8, 23, "Print"],
Cell[742310, 20419, 198, 5, 23, "Print"],
Cell[742511, 20426, 310, 7, 23, "Print"],
Cell[742824, 20435, 197, 5, 23, "Print"],
Cell[743024, 20442, 114, 1, 23, "Print"],
Cell[743141, 20445, 419, 9, 23, "Print"],
Cell[743563, 20456, 96, 1, 23, "Print"],
Cell[743662, 20459, 220, 5, 23, "Print"],
Cell[743885, 20466, 222, 5, 23, "Print"],
Cell[744110, 20473, 362, 8, 23, "Print"],
Cell[744475, 20483, 199, 5, 23, "Print"],
Cell[744677, 20490, 373, 8, 23, "Print"],
Cell[745053, 20500, 198, 5, 23, "Print"],
Cell[745254, 20507, 115, 1, 23, "Print"],
Cell[745372, 20510, 359, 8, 23, "Print"],
Cell[745734, 20520, 98, 1, 23, "Print"],
Cell[745835, 20523, 220, 5, 23, "Print"],
Cell[746058, 20530, 224, 5, 23, "Print"],
Cell[746285, 20537, 273, 4, 62, "Print"],
Cell[746561, 20543, 489, 10, 23, "Print"],
Cell[747053, 20555, 259, 4, 43, "Print"],
Cell[747315, 20561, 341, 7, 23, "Print"],
Cell[747659, 20570, 204, 3, 23, "Print"],
Cell[747866, 20575, 416, 8, 23, "Print"],
Cell[748285, 20585, 236, 5, 23, "Print"],
Cell[748524, 20592, 153, 2, 23, "Print"],
Cell[748680, 20596, 396, 8, 23, "Print"],
Cell[749079, 20606, 170, 2, 23, "Print"],
Cell[749252, 20610, 404, 8, 23, "Print"],
Cell[749659, 20620, 362, 8, 23, "Print"],
Cell[750024, 20630, 362, 8, 23, "Print"],
Cell[750389, 20640, 199, 5, 23, "Print"],
Cell[750591, 20647, 307, 7, 23, "Print"],
Cell[750901, 20656, 513, 10, 23, "Print"],
Cell[751417, 20668, 124, 2, 23, "Print"],
Cell[751544, 20672, 93, 1, 23, "Print"],
Cell[751640, 20675, 119, 1, 23, "Print"],
Cell[751762, 20678, 130, 2, 23, "Print"],
Cell[751895, 20682, 271, 4, 62, "Print"],
Cell[752169, 20688, 488, 10, 23, "Print"],
Cell[752660, 20700, 259, 4, 43, "Print"],
Cell[752922, 20706, 338, 7, 23, "Print"],
Cell[753263, 20715, 373, 8, 23, "Print"],
Cell[753639, 20725, 119, 1, 23, "Print"],
Cell[753761, 20728, 162, 2, 23, "Print"],
Cell[753926, 20732, 86, 1, 23, "Print"],
Cell[754015, 20735, 89, 1, 23, "Print"],
Cell[754107, 20738, 92, 1, 23, "Print"],
Cell[754202, 20741, 200, 5, 23, "Print"],
Cell[754405, 20748, 117, 1, 23, "Print"],
Cell[754525, 20751, 419, 9, 23, "Print"],
Cell[754947, 20762, 362, 8, 23, "Print"],
Cell[755312, 20772, 199, 5, 23, "Print"],
Cell[755514, 20779, 375, 8, 23, "Print"],
Cell[755892, 20789, 200, 5, 23, "Print"],
Cell[756095, 20796, 117, 1, 23, "Print"],
Cell[756215, 20799, 419, 9, 23, "Print"],
Cell[756637, 20810, 364, 8, 23, "Print"],
Cell[757004, 20820, 201, 5, 23, "Print"],
Cell[757208, 20827, 375, 8, 23, "Print"],
Cell[757586, 20837, 200, 5, 23, "Print"],
Cell[757789, 20844, 117, 1, 23, "Print"],
Cell[757909, 20847, 419, 9, 23, "Print"],
Cell[758331, 20858, 361, 8, 23, "Print"],
Cell[758695, 20868, 201, 5, 23, "Print"],
Cell[758899, 20875, 371, 8, 23, "Print"],
Cell[759273, 20885, 200, 5, 23, "Print"],
Cell[759476, 20892, 117, 1, 23, "Print"],
Cell[759596, 20895, 421, 9, 23, "Print"],
Cell[760020, 20906, 362, 8, 23, "Print"],
Cell[760385, 20916, 199, 5, 23, "Print"],
Cell[760587, 20923, 373, 8, 23, "Print"],
Cell[760963, 20933, 198, 5, 23, "Print"],
Cell[761164, 20940, 115, 1, 23, "Print"],
Cell[761282, 20943, 361, 8, 23, "Print"],
Cell[761646, 20953, 362, 8, 23, "Print"],
Cell[762011, 20963, 199, 5, 23, "Print"],
Cell[762213, 20970, 313, 7, 23, "Print"],
Cell[762529, 20979, 200, 5, 23, "Print"],
Cell[762732, 20986, 117, 1, 23, "Print"],
Cell[762852, 20989, 421, 9, 23, "Print"],
Cell[763276, 21000, 98, 1, 23, "Print"],
Cell[763377, 21003, 222, 5, 23, "Print"],
Cell[763602, 21010, 222, 5, 23, "Print"],
Cell[763827, 21017, 364, 8, 23, "Print"],
Cell[764194, 21027, 201, 5, 23, "Print"],
Cell[764398, 21034, 375, 8, 23, "Print"],
Cell[764776, 21044, 200, 5, 23, "Print"],
Cell[764979, 21051, 117, 1, 23, "Print"],
Cell[765099, 21054, 355, 8, 23, "Print"],
Cell[765457, 21064, 96, 1, 23, "Print"],
Cell[765556, 21067, 222, 5, 23, "Print"],
Cell[765781, 21074, 221, 5, 23, "Print"],
Cell[766005, 21081, 273, 4, 62, "Print"],
Cell[766281, 21087, 489, 10, 23, "Print"],
Cell[766773, 21099, 261, 4, 43, "Print"],
Cell[767037, 21105, 337, 7, 23, "Print"],
Cell[767377, 21114, 204, 3, 23, "Print"],
Cell[767584, 21119, 415, 8, 23, "Print"],
Cell[768002, 21129, 235, 5, 23, "Print"],
Cell[768240, 21136, 155, 2, 23, "Print"],
Cell[768398, 21140, 404, 8, 23, "Print"],
Cell[768805, 21150, 168, 2, 23, "Print"],
Cell[768976, 21154, 401, 8, 23, "Print"],
Cell[769380, 21164, 364, 8, 23, "Print"],
Cell[769747, 21174, 361, 8, 23, "Print"],
Cell[770111, 21184, 201, 5, 23, "Print"],
Cell[770315, 21191, 317, 7, 23, "Print"],
Cell[770635, 21200, 515, 10, 23, "Print"],
Cell[771153, 21212, 126, 2, 23, "Print"],
Cell[771282, 21216, 95, 1, 23, "Print"],
Cell[771380, 21219, 119, 1, 23, "Print"],
Cell[771502, 21222, 130, 2, 23, "Print"],
Cell[771635, 21226, 273, 4, 62, "Print"],
Cell[771911, 21232, 490, 10, 23, "Print"],
Cell[772404, 21244, 261, 4, 43, "Print"],
Cell[772668, 21250, 340, 7, 23, "Print"],
Cell[773011, 21259, 379, 8, 23, "Print"],
Cell[773393, 21269, 119, 1, 23, "Print"],
Cell[773515, 21272, 162, 2, 23, "Print"],
Cell[773680, 21276, 86, 1, 23, "Print"],
Cell[773769, 21279, 89, 1, 23, "Print"],
Cell[773861, 21282, 110, 2, 23, "Print"],
Cell[773974, 21286, 200, 5, 23, "Print"],
Cell[774177, 21293, 115, 1, 23, "Print"],
Cell[774295, 21296, 359, 8, 23, "Print"],
Cell[774657, 21306, 364, 8, 23, "Print"],
Cell[775024, 21316, 201, 5, 23, "Print"],
Cell[775228, 21323, 313, 7, 23, "Print"],
Cell[775544, 21332, 200, 5, 23, "Print"],
Cell[775747, 21339, 117, 1, 23, "Print"],
Cell[775867, 21342, 416, 9, 23, "Print"],
Cell[776286, 21353, 362, 8, 23, "Print"],
Cell[776651, 21363, 199, 5, 23, "Print"],
Cell[776853, 21370, 311, 7, 23, "Print"],
Cell[777167, 21379, 198, 5, 23, "Print"],
Cell[777368, 21386, 115, 1, 23, "Print"],
Cell[777486, 21389, 419, 9, 23, "Print"],
Cell[777908, 21400, 364, 8, 23, "Print"],
Cell[778275, 21410, 201, 5, 23, "Print"],
Cell[778479, 21417, 375, 8, 23, "Print"],
Cell[778857, 21427, 200, 5, 23, "Print"],
Cell[779060, 21434, 117, 1, 23, "Print"],
Cell[779180, 21437, 419, 9, 23, "Print"],
Cell[779602, 21448, 364, 8, 23, "Print"],
Cell[779969, 21458, 197, 5, 23, "Print"],
Cell[780169, 21465, 369, 8, 23, "Print"],
Cell[780541, 21475, 196, 5, 23, "Print"],
Cell[780740, 21482, 113, 1, 23, "Print"],
Cell[780856, 21485, 419, 9, 23, "Print"],
Cell[781278, 21496, 364, 8, 23, "Print"],
Cell[781645, 21506, 201, 5, 23, "Print"],
Cell[781849, 21513, 311, 7, 23, "Print"],
Cell[782163, 21522, 200, 5, 23, "Print"],
Cell[782366, 21529, 117, 1, 23, "Print"],
Cell[782486, 21532, 357, 8, 23, "Print"],
Cell[782846, 21542, 98, 1, 23, "Print"],
Cell[782947, 21545, 220, 5, 23, "Print"],
Cell[783170, 21552, 224, 5, 23, "Print"],
Cell[783397, 21559, 359, 8, 23, "Print"],
Cell[783759, 21569, 199, 5, 23, "Print"],
Cell[783961, 21576, 373, 8, 23, "Print"],
Cell[784337, 21586, 198, 5, 23, "Print"],
Cell[784538, 21593, 115, 1, 23, "Print"],
Cell[784656, 21596, 423, 9, 23, "Print"],
Cell[785082, 21607, 98, 1, 23, "Print"],
Cell[785183, 21610, 222, 5, 23, "Print"],
Cell[785408, 21617, 222, 5, 23, "Print"],
Cell[785633, 21624, 271, 4, 62, "Print"],
Cell[785907, 21630, 486, 10, 23, "Print"],
Cell[786396, 21642, 259, 4, 43, "Print"],
Cell[786658, 21648, 338, 7, 23, "Print"],
Cell[786999, 21657, 202, 3, 23, "Print"],
Cell[787204, 21662, 410, 8, 23, "Print"],
Cell[787617, 21672, 235, 5, 23, "Print"],
Cell[787855, 21679, 153, 2, 23, "Print"],
Cell[788011, 21683, 401, 8, 23, "Print"],
Cell[788415, 21693, 168, 2, 23, "Print"],
Cell[788586, 21697, 399, 8, 23, "Print"],
Cell[788988, 21707, 357, 8, 23, "Print"],
Cell[789348, 21717, 362, 8, 23, "Print"],
Cell[789713, 21727, 201, 5, 23, "Print"],
Cell[789917, 21734, 315, 7, 23, "Print"],
Cell[790235, 21743, 451, 9, 23, "Print"],
Cell[790689, 21754, 405, 9, 23, "Print"],
Cell[791097, 21765, 155, 2, 23, "Print"],
Cell[791255, 21769, 155, 2, 23, "Print"],
Cell[791413, 21773, 158, 2, 23, "Print"],
Cell[791574, 21777, 155, 2, 23, "Print"],
Cell[791732, 21781, 108, 1, 23, "Print"],
Cell[791843, 21784, 141, 2, 23, "Print"],
Cell[791987, 21788, 101, 1, 23, "Print"],
Cell[792091, 21791, 153, 2, 23, "Print"],
Cell[792247, 21795, 146, 2, 23, "Print"],
Cell[792396, 21799, 287, 6, 23, "Print"],
Cell[792686, 21807, 307, 7, 23, "Print"],
Cell[792996, 21816, 165, 2, 23, "Print"],
Cell[793164, 21820, 517, 10, 23, "Print"],
Cell[793684, 21832, 135, 2, 23, "Print"],
Cell[793822, 21836, 140, 2, 23, "Print"],
Cell[793965, 21840, 284, 6, 23, "Print"],
Cell[794252, 21848, 148, 2, 23, "Print"],
Cell[794403, 21852, 305, 7, 23, "Print"],
Cell[794711, 21861, 110, 1, 23, "Print"],
Cell[794824, 21864, 273, 6, 23, "Print"],
Cell[795100, 21872, 421, 5, 62, "Print"],
Cell[795524, 21879, 581, 10, 43, "Print"],
Cell[796108, 21891, 285, 6, 23, "Print"],
Cell[796396, 21899, 149, 2, 23, "Print"],
Cell[796548, 21903, 305, 7, 23, "Print"],
Cell[796856, 21912, 110, 1, 23, "Print"],
Cell[796969, 21915, 273, 6, 23, "Print"],
Cell[797245, 21923, 423, 5, 62, "Print"],
Cell[797671, 21930, 583, 10, 43, "Print"],
Cell[798257, 21942, 287, 6, 23, "Print"],
Cell[798547, 21950, 151, 2, 23, "Print"],
Cell[798701, 21954, 307, 7, 23, "Print"],
Cell[799011, 21963, 110, 1, 23, "Print"],
Cell[799124, 21966, 273, 6, 23, "Print"],
Cell[799400, 21974, 421, 5, 62, "Print"],
Cell[799824, 21981, 581, 10, 43, "Print"],
Cell[800408, 21993, 319, 7, 23, "Print"],
Cell[800730, 22002, 149, 2, 23, "Print"],
Cell[800882, 22006, 305, 7, 23, "Print"],
Cell[801190, 22015, 110, 1, 23, "Print"],
Cell[801303, 22018, 273, 6, 23, "Print"],
Cell[801579, 22026, 423, 5, 62, "Print"],
Cell[802005, 22033, 586, 10, 43, "Print"],
Cell[802594, 22045, 317, 7, 23, "Print"],
Cell[802914, 22054, 147, 2, 23, "Print"],
Cell[803064, 22058, 305, 7, 23, "Print"],
Cell[803372, 22067, 110, 1, 23, "Print"],
Cell[803485, 22070, 275, 6, 23, "Print"],
Cell[803763, 22078, 421, 5, 62, "Print"],
Cell[804187, 22085, 581, 10, 43, "Print"],
Cell[804771, 22097, 88, 1, 23, "Print"],
Cell[804862, 22100, 421, 5, 62, "Print"],
Cell[805286, 22107, 587, 11, 43, "Print"],
Cell[805876, 22120, 1122, 22, 43, "Print"],
Cell[807001, 22144, 379, 8, 23, "Print"],
Cell[807383, 22154, 139, 2, 23, "Print"],
Cell[807525, 22158, 94, 1, 23, "Print"],
Cell[807622, 22161, 155, 2, 23, "Print"],
Cell[807780, 22165, 148, 2, 23, "Print"],
Cell[807931, 22169, 289, 6, 23, "Print"],
Cell[808223, 22177, 305, 7, 23, "Print"],
Cell[808531, 22186, 167, 2, 23, "Print"],
Cell[808701, 22190, 455, 9, 23, "Print"],
Cell[809159, 22201, 135, 2, 23, "Print"],
Cell[809297, 22205, 140, 2, 23, "Print"],
Cell[809440, 22209, 285, 6, 23, "Print"],
Cell[809728, 22217, 149, 2, 23, "Print"],
Cell[809880, 22221, 307, 7, 23, "Print"],
Cell[810190, 22230, 108, 1, 23, "Print"],
Cell[810301, 22233, 275, 6, 23, "Print"],
Cell[810579, 22241, 423, 5, 62, "Print"],
Cell[811005, 22248, 582, 10, 43, "Print"],
Cell[811590, 22260, 287, 6, 23, "Print"],
Cell[811880, 22268, 151, 2, 23, "Print"],
Cell[812034, 22272, 305, 7, 23, "Print"],
Cell[812342, 22281, 108, 1, 23, "Print"],
Cell[812453, 22284, 273, 6, 23, "Print"],
Cell[812729, 22292, 423, 5, 62, "Print"],
Cell[813155, 22299, 584, 10, 43, "Print"],
Cell[813742, 22311, 287, 6, 23, "Print"],
Cell[814032, 22319, 151, 2, 23, "Print"],
Cell[814186, 22323, 304, 7, 23, "Print"],
Cell[814493, 22332, 110, 1, 23, "Print"],
Cell[814606, 22335, 275, 6, 23, "Print"],
Cell[814884, 22343, 423, 5, 62, "Print"],
Cell[815310, 22350, 583, 10, 43, "Print"],
Cell[815896, 22362, 88, 1, 23, "Print"],
Cell[815987, 22365, 423, 5, 62, "Print"],
Cell[816413, 22372, 582, 10, 43, "Print"],
Cell[816998, 22384, 1079, 20, 43, "Print"],
Cell[818080, 22406, 445, 9, 23, "Print"],
Cell[818528, 22417, 141, 2, 23, "Print"],
Cell[818672, 22421, 92, 1, 23, "Print"],
Cell[818767, 22424, 155, 2, 23, "Print"],
Cell[818925, 22428, 146, 2, 23, "Print"],
Cell[819074, 22432, 289, 6, 23, "Print"],
Cell[819366, 22440, 305, 7, 23, "Print"],
Cell[819674, 22449, 167, 2, 23, "Print"],
Cell[819844, 22453, 455, 9, 23, "Print"],
Cell[820302, 22464, 137, 2, 23, "Print"],
Cell[820442, 22468, 142, 2, 23, "Print"],
Cell[820587, 22472, 285, 6, 23, "Print"],
Cell[820875, 22480, 149, 2, 23, "Print"],
Cell[821027, 22484, 305, 7, 23, "Print"],
Cell[821335, 22493, 108, 1, 23, "Print"],
Cell[821446, 22496, 273, 6, 23, "Print"],
Cell[821722, 22504, 421, 5, 62, "Print"],
Cell[822146, 22511, 583, 10, 43, "Print"],
Cell[822732, 22523, 285, 6, 23, "Print"],
Cell[823020, 22531, 149, 2, 23, "Print"],
Cell[823172, 22535, 305, 7, 23, "Print"],
Cell[823480, 22544, 108, 1, 23, "Print"],
Cell[823591, 22547, 273, 6, 23, "Print"],
Cell[823867, 22555, 423, 5, 62, "Print"],
Cell[824293, 22562, 581, 10, 43, "Print"],
Cell[824877, 22574, 285, 6, 23, "Print"],
Cell[825165, 22582, 149, 2, 23, "Print"],
Cell[825317, 22586, 305, 7, 23, "Print"],
Cell[825625, 22595, 110, 1, 23, "Print"],
Cell[825738, 22598, 275, 6, 23, "Print"],
Cell[826016, 22606, 423, 5, 62, "Print"],
Cell[826442, 22613, 585, 10, 43, "Print"],
Cell[827030, 22625, 319, 7, 23, "Print"],
Cell[827352, 22634, 151, 2, 23, "Print"],
Cell[827506, 22638, 307, 7, 23, "Print"],
Cell[827816, 22647, 110, 1, 23, "Print"],
Cell[827929, 22650, 273, 6, 23, "Print"],
Cell[828205, 22658, 423, 5, 62, "Print"],
Cell[828631, 22665, 583, 10, 43, "Print"],
Cell[829217, 22677, 321, 7, 23, "Print"],
Cell[829541, 22686, 151, 2, 23, "Print"],
Cell[829695, 22690, 307, 7, 23, "Print"],
Cell[830005, 22699, 110, 1, 23, "Print"],
Cell[830118, 22702, 275, 6, 23, "Print"],
Cell[830396, 22710, 423, 5, 62, "Print"],
Cell[830822, 22717, 581, 10, 43, "Print"],
Cell[831406, 22729, 319, 7, 23, "Print"],
Cell[831728, 22738, 149, 2, 23, "Print"],
Cell[831880, 22742, 305, 7, 23, "Print"],
Cell[832188, 22751, 110, 1, 23, "Print"],
Cell[832301, 22754, 273, 6, 23, "Print"],
Cell[832577, 22762, 421, 5, 62, "Print"],
Cell[833001, 22769, 584, 10, 43, "Print"],
Cell[833588, 22781, 86, 1, 23, "Print"],
Cell[833677, 22784, 423, 5, 62, "Print"],
Cell[834103, 22791, 582, 10, 43, "Print"],
Cell[834688, 22803, 1119, 22, 43, "Print"],
Cell[835810, 22827, 445, 9, 23, "Print"],
Cell[836258, 22838, 139, 2, 23, "Print"],
Cell[836400, 22842, 93, 1, 23, "Print"],
Cell[836496, 22845, 155, 2, 23, "Print"],
Cell[836654, 22849, 148, 2, 23, "Print"],
Cell[836805, 22853, 289, 6, 23, "Print"],
Cell[837097, 22861, 305, 7, 23, "Print"],
Cell[837405, 22870, 165, 2, 23, "Print"],
Cell[837573, 22874, 515, 10, 23, "Print"],
Cell[838091, 22886, 135, 2, 23, "Print"],
Cell[838229, 22890, 140, 2, 23, "Print"],
Cell[838372, 22894, 285, 6, 23, "Print"],
Cell[838660, 22902, 149, 2, 23, "Print"],
Cell[838812, 22906, 305, 7, 23, "Print"],
Cell[839120, 22915, 110, 1, 23, "Print"],
Cell[839233, 22918, 273, 6, 23, "Print"],
Cell[839509, 22926, 423, 5, 62, "Print"],
Cell[839935, 22933, 581, 10, 43, "Print"],
Cell[840519, 22945, 285, 6, 23, "Print"],
Cell[840807, 22953, 149, 2, 23, "Print"],
Cell[840959, 22957, 305, 7, 23, "Print"],
Cell[841267, 22966, 110, 1, 23, "Print"],
Cell[841380, 22969, 275, 6, 23, "Print"],
Cell[841658, 22977, 423, 5, 62, "Print"],
Cell[842084, 22984, 591, 11, 43, "Print"],
Cell[842678, 22997, 285, 6, 23, "Print"],
Cell[842966, 23005, 149, 2, 23, "Print"],
Cell[843118, 23009, 305, 7, 23, "Print"],
Cell[843426, 23018, 110, 1, 23, "Print"],
Cell[843539, 23021, 273, 6, 23, "Print"],
Cell[843815, 23029, 421, 5, 62, "Print"],
Cell[844239, 23036, 578, 10, 43, "Print"],
Cell[844820, 23048, 319, 7, 23, "Print"],
Cell[845142, 23057, 151, 2, 23, "Print"],
Cell[845296, 23061, 307, 7, 23, "Print"],
Cell[845606, 23070, 108, 1, 23, "Print"],
Cell[845717, 23073, 273, 6, 23, "Print"],
Cell[845993, 23081, 423, 5, 62, "Print"],
Cell[846419, 23088, 583, 10, 43, "Print"],
Cell[847005, 23100, 88, 1, 23, "Print"],
Cell[847096, 23103, 423, 5, 62, "Print"],
Cell[847522, 23110, 580, 10, 43, "Print"],
Cell[848105, 23122, 1079, 20, 43, "Print"],
Cell[849187, 23144, 443, 9, 23, "Print"],
Cell[849633, 23155, 141, 2, 23, "Print"],
Cell[849777, 23159, 92, 1, 23, "Print"],
Cell[849872, 23162, 155, 2, 23, "Print"],
Cell[850030, 23166, 146, 2, 23, "Print"],
Cell[850179, 23170, 289, 6, 23, "Print"],
Cell[850471, 23178, 305, 7, 23, "Print"],
Cell[850779, 23187, 165, 2, 23, "Print"],
Cell[850947, 23191, 457, 9, 23, "Print"],
Cell[851407, 23202, 137, 2, 23, "Print"],
Cell[851547, 23206, 142, 2, 23, "Print"],
Cell[851692, 23210, 285, 6, 23, "Print"],
Cell[851980, 23218, 149, 2, 23, "Print"],
Cell[852132, 23222, 307, 7, 23, "Print"],
Cell[852442, 23231, 108, 1, 23, "Print"],
Cell[852553, 23234, 275, 6, 23, "Print"],
Cell[852831, 23242, 423, 5, 62, "Print"],
Cell[853257, 23249, 579, 10, 43, "Print"],
Cell[853839, 23261, 287, 6, 23, "Print"],
Cell[854129, 23269, 149, 2, 23, "Print"],
Cell[854281, 23273, 305, 7, 23, "Print"],
Cell[854589, 23282, 108, 1, 23, "Print"],
Cell[854700, 23285, 275, 6, 23, "Print"],
Cell[854978, 23293, 423, 5, 62, "Print"],
Cell[855404, 23300, 582, 10, 43, "Print"],
Cell[855989, 23312, 287, 6, 23, "Print"],
Cell[856279, 23320, 148, 2, 23, "Print"],
Cell[856430, 23324, 307, 7, 23, "Print"],
Cell[856740, 23333, 110, 1, 23, "Print"],
Cell[856853, 23336, 275, 6, 23, "Print"],
Cell[857131, 23344, 421, 5, 62, "Print"],
Cell[857555, 23351, 581, 10, 43, "Print"],
Cell[858139, 23363, 319, 7, 23, "Print"],
Cell[858461, 23372, 149, 2, 23, "Print"],
Cell[858613, 23376, 307, 7, 23, "Print"],
Cell[858923, 23385, 108, 1, 23, "Print"],
Cell[859034, 23388, 275, 6, 23, "Print"],
Cell[859312, 23396, 423, 5, 62, "Print"],
Cell[859738, 23403, 580, 10, 43, "Print"],
Cell[860321, 23415, 319, 7, 23, "Print"],
Cell[860643, 23424, 151, 2, 23, "Print"],
Cell[860797, 23428, 305, 7, 23, "Print"],
Cell[861105, 23437, 110, 1, 23, "Print"],
Cell[861218, 23440, 275, 6, 23, "Print"],
Cell[861496, 23448, 423, 5, 62, "Print"],
Cell[861922, 23455, 581, 10, 43, "Print"],
Cell[862506, 23467, 86, 1, 23, "Print"],
Cell[862595, 23470, 421, 5, 62, "Print"],
Cell[863019, 23477, 581, 10, 43, "Print"],
Cell[863603, 23489, 1120, 22, 43, "Print"],
Cell[864726, 23513, 443, 9, 23, "Print"],
Cell[865172, 23524, 141, 2, 23, "Print"],
Cell[865316, 23528, 90, 1, 23, "Print"],
Cell[865409, 23531, 155, 2, 23, "Print"],
Cell[865567, 23535, 148, 2, 23, "Print"],
Cell[865718, 23539, 287, 6, 23, "Print"],
Cell[866008, 23547, 307, 7, 23, "Print"],
Cell[866318, 23556, 167, 2, 23, "Print"],
Cell[866488, 23560, 457, 9, 23, "Print"],
Cell[866948, 23571, 137, 2, 23, "Print"],
Cell[867088, 23575, 142, 2, 23, "Print"],
Cell[867233, 23579, 318, 7, 23, "Print"],
Cell[867554, 23588, 151, 2, 23, "Print"],
Cell[867708, 23592, 307, 7, 23, "Print"],
Cell[868018, 23601, 110, 1, 23, "Print"],
Cell[868131, 23604, 275, 6, 23, "Print"],
Cell[868409, 23612, 423, 5, 62, "Print"],
Cell[868835, 23619, 579, 10, 43, "Print"],
Cell[869417, 23631, 287, 6, 23, "Print"],
Cell[869707, 23639, 151, 2, 23, "Print"],
Cell[869861, 23643, 307, 7, 23, "Print"],
Cell[870171, 23652, 110, 1, 23, "Print"],
Cell[870284, 23655, 273, 6, 23, "Print"],
Cell[870560, 23663, 421, 5, 62, "Print"],
Cell[870984, 23670, 576, 10, 43, "Print"],
Cell[871563, 23682, 316, 7, 23, "Print"],
Cell[871882, 23691, 151, 2, 23, "Print"],
Cell[872036, 23695, 307, 7, 23, "Print"],
Cell[872346, 23704, 107, 1, 23, "Print"],
Cell[872456, 23707, 273, 6, 23, "Print"],
Cell[872732, 23715, 421, 5, 62, "Print"],
Cell[873156, 23722, 582, 10, 43, "Print"],
Cell[873741, 23734, 318, 7, 23, "Print"],
Cell[874062, 23743, 151, 2, 23, "Print"],
Cell[874216, 23747, 307, 7, 23, "Print"],
Cell[874526, 23756, 108, 1, 23, "Print"],
Cell[874637, 23759, 275, 6, 23, "Print"],
Cell[874915, 23767, 423, 5, 62, "Print"],
Cell[875341, 23774, 582, 10, 43, "Print"],
Cell[875926, 23786, 316, 7, 23, "Print"],
Cell[876245, 23795, 151, 2, 23, "Print"],
Cell[876399, 23799, 307, 7, 23, "Print"],
Cell[876709, 23808, 108, 1, 23, "Print"],
Cell[876820, 23811, 273, 6, 23, "Print"],
Cell[877096, 23819, 421, 5, 62, "Print"],
Cell[877520, 23826, 582, 10, 43, "Print"],
Cell[878105, 23838, 316, 7, 23, "Print"],
Cell[878424, 23847, 151, 2, 23, "Print"],
Cell[878578, 23851, 307, 7, 23, "Print"],
Cell[878888, 23860, 110, 1, 23, "Print"],
Cell[879001, 23863, 273, 6, 23, "Print"],
Cell[879277, 23871, 421, 5, 62, "Print"],
Cell[879701, 23878, 582, 10, 43, "Print"],
Cell[880286, 23890, 318, 7, 23, "Print"],
Cell[880607, 23899, 149, 2, 23, "Print"],
Cell[880759, 23903, 304, 7, 23, "Print"],
Cell[881066, 23912, 108, 1, 23, "Print"],
Cell[881177, 23915, 273, 6, 23, "Print"],
Cell[881453, 23923, 423, 5, 62, "Print"],
Cell[881879, 23930, 585, 10, 43, "Print"],
Cell[882467, 23942, 316, 7, 23, "Print"],
Cell[882786, 23951, 149, 2, 23, "Print"],
Cell[882938, 23955, 307, 7, 23, "Print"],
Cell[883248, 23964, 108, 1, 23, "Print"],
Cell[883359, 23967, 275, 6, 23, "Print"],
Cell[883637, 23975, 421, 5, 62, "Print"],
Cell[884061, 23982, 581, 10, 43, "Print"],
Cell[884645, 23994, 318, 7, 23, "Print"],
Cell[884966, 24003, 149, 2, 23, "Print"],
Cell[885118, 24007, 307, 7, 23, "Print"],
Cell[885428, 24016, 110, 1, 23, "Print"],
Cell[885541, 24019, 273, 6, 23, "Print"],
Cell[885817, 24027, 421, 5, 62, "Print"],
Cell[886241, 24034, 581, 10, 43, "Print"],
Cell[886825, 24046, 318, 7, 23, "Print"],
Cell[887146, 24055, 151, 2, 23, "Print"],
Cell[887300, 24059, 307, 7, 23, "Print"],
Cell[887610, 24068, 108, 1, 23, "Print"],
Cell[887721, 24071, 273, 6, 23, "Print"],
Cell[887997, 24079, 421, 5, 62, "Print"],
Cell[888421, 24086, 581, 10, 43, "Print"],
Cell[889005, 24098, 316, 7, 23, "Print"],
Cell[889324, 24107, 149, 2, 23, "Print"],
Cell[889476, 24111, 307, 7, 23, "Print"],
Cell[889786, 24120, 107, 1, 23, "Print"],
Cell[889896, 24123, 275, 6, 23, "Print"],
Cell[890174, 24131, 421, 5, 62, "Print"],
Cell[890598, 24138, 581, 10, 43, "Print"],
Cell[891182, 24150, 316, 7, 23, "Print"],
Cell[891501, 24159, 149, 2, 23, "Print"],
Cell[891653, 24163, 303, 7, 23, "Print"],
Cell[891959, 24172, 110, 1, 23, "Print"],
Cell[892072, 24175, 273, 6, 23, "Print"],
Cell[892348, 24183, 423, 5, 62, "Print"],
Cell[892774, 24190, 580, 10, 43, "Print"],
Cell[893357, 24202, 321, 7, 23, "Print"],
Cell[893681, 24211, 151, 2, 23, "Print"],
Cell[893835, 24215, 305, 7, 23, "Print"],
Cell[894143, 24224, 110, 1, 23, "Print"],
Cell[894256, 24227, 275, 6, 23, "Print"],
Cell[894534, 24235, 421, 5, 62, "Print"],
Cell[894958, 24242, 581, 10, 43, "Print"],
Cell[895542, 24254, 318, 7, 23, "Print"],
Cell[895863, 24263, 149, 2, 23, "Print"],
Cell[896015, 24267, 305, 7, 23, "Print"],
Cell[896323, 24276, 110, 1, 23, "Print"],
Cell[896436, 24279, 270, 6, 23, "Print"],
Cell[896709, 24287, 421, 5, 62, "Print"],
Cell[897133, 24294, 580, 10, 43, "Print"],
Cell[897716, 24306, 318, 7, 23, "Print"],
Cell[898037, 24315, 151, 2, 23, "Print"],
Cell[898191, 24319, 305, 7, 23, "Print"],
Cell[898499, 24328, 108, 1, 23, "Print"],
Cell[898610, 24331, 273, 6, 23, "Print"],
Cell[898886, 24339, 421, 5, 62, "Print"],
Cell[899310, 24346, 584, 10, 43, "Print"],
Cell[899897, 24358, 316, 7, 23, "Print"],
Cell[900216, 24367, 149, 2, 23, "Print"],
Cell[900368, 24371, 307, 7, 23, "Print"],
Cell[900678, 24380, 110, 1, 23, "Print"],
Cell[900791, 24383, 273, 6, 23, "Print"],
Cell[901067, 24391, 421, 5, 62, "Print"],
Cell[901491, 24398, 580, 10, 43, "Print"],
Cell[902074, 24410, 316, 7, 23, "Print"],
Cell[902393, 24419, 151, 2, 23, "Print"],
Cell[902547, 24423, 307, 7, 23, "Print"],
Cell[902857, 24432, 110, 1, 23, "Print"],
Cell[902970, 24435, 273, 6, 23, "Print"],
Cell[903246, 24443, 423, 5, 62, "Print"],
Cell[903672, 24450, 582, 10, 43, "Print"],
Cell[904257, 24462, 316, 7, 23, "Print"],
Cell[904576, 24471, 149, 2, 23, "Print"],
Cell[904728, 24475, 307, 7, 23, "Print"],
Cell[905038, 24484, 108, 1, 23, "Print"],
Cell[905149, 24487, 271, 6, 23, "Print"],
Cell[905423, 24495, 421, 5, 62, "Print"],
Cell[905847, 24502, 582, 10, 43, "Print"],
Cell[906432, 24514, 319, 7, 23, "Print"],
Cell[906754, 24523, 149, 2, 23, "Print"],
Cell[906906, 24527, 305, 7, 23, "Print"],
Cell[907214, 24536, 110, 1, 23, "Print"],
Cell[907327, 24539, 272, 6, 23, "Print"],
Cell[907602, 24547, 423, 5, 62, "Print"],
Cell[908028, 24554, 582, 10, 43, "Print"],
Cell[908613, 24566, 321, 7, 23, "Print"],
Cell[908937, 24575, 149, 2, 23, "Print"],
Cell[909089, 24579, 307, 7, 23, "Print"],
Cell[909399, 24588, 110, 1, 23, "Print"],
Cell[909512, 24591, 273, 6, 23, "Print"],
Cell[909788, 24599, 423, 5, 62, "Print"],
Cell[910214, 24606, 584, 10, 43, "Print"],
Cell[910801, 24618, 319, 7, 23, "Print"],
Cell[911123, 24627, 149, 2, 23, "Print"],
Cell[911275, 24631, 305, 7, 23, "Print"],
Cell[911583, 24640, 110, 1, 23, "Print"],
Cell[911696, 24643, 275, 6, 23, "Print"],
Cell[911974, 24651, 423, 5, 62, "Print"],
Cell[912400, 24658, 582, 10, 43, "Print"],
Cell[912985, 24670, 319, 7, 23, "Print"],
Cell[913307, 24679, 151, 2, 23, "Print"],
Cell[913461, 24683, 305, 7, 23, "Print"],
Cell[913769, 24692, 110, 1, 23, "Print"],
Cell[913882, 24695, 275, 6, 23, "Print"],
Cell[914160, 24703, 421, 5, 62, "Print"],
Cell[914584, 24710, 581, 10, 43, "Print"],
Cell[915168, 24722, 86, 1, 23, "Print"],
Cell[915257, 24725, 421, 5, 62, "Print"],
Cell[915681, 24732, 580, 10, 43, "Print"],
Cell[916264, 24744, 1121, 22, 43, "Print"],
Cell[917388, 24768, 385, 8, 23, "Print"],
Cell[917776, 24778, 139, 2, 23, "Print"],
Cell[917918, 24782, 92, 1, 23, "Print"],
Cell[918013, 24785, 153, 2, 23, "Print"],
Cell[918169, 24789, 148, 2, 23, "Print"],
Cell[918320, 24793, 289, 6, 23, "Print"],
Cell[918612, 24801, 307, 7, 23, "Print"],
Cell[918922, 24810, 165, 2, 23, "Print"],
Cell[919090, 24814, 455, 9, 23, "Print"],
Cell[919548, 24825, 135, 2, 23, "Print"],
Cell[919686, 24829, 140, 2, 23, "Print"],
Cell[919829, 24833, 285, 6, 23, "Print"],
Cell[920117, 24841, 149, 2, 23, "Print"],
Cell[920269, 24845, 307, 7, 23, "Print"],
Cell[920579, 24854, 110, 1, 23, "Print"],
Cell[920692, 24857, 273, 6, 23, "Print"],
Cell[920968, 24865, 421, 5, 62, "Print"],
Cell[921392, 24872, 578, 10, 43, "Print"],
Cell[921973, 24884, 285, 6, 23, "Print"],
Cell[922261, 24892, 149, 2, 23, "Print"],
Cell[922413, 24896, 305, 7, 23, "Print"],
Cell[922721, 24905, 110, 1, 23, "Print"],
Cell[922834, 24908, 275, 6, 23, "Print"],
Cell[923112, 24916, 423, 5, 62, "Print"],
Cell[923538, 24923, 583, 10, 43, "Print"],
Cell[924124, 24935, 285, 6, 23, "Print"],
Cell[924412, 24943, 149, 2, 23, "Print"],
Cell[924564, 24947, 307, 7, 23, "Print"],
Cell[924874, 24956, 108, 1, 23, "Print"],
Cell[924985, 24959, 275, 6, 23, "Print"],
Cell[925263, 24967, 423, 5, 62, "Print"],
Cell[925689, 24974, 590, 11, 43, "Print"],
Cell[926282, 24987, 321, 7, 23, "Print"],
Cell[926606, 24996, 151, 2, 23, "Print"],
Cell[926760, 25000, 307, 7, 23, "Print"],
Cell[927070, 25009, 110, 1, 23, "Print"],
Cell[927183, 25012, 272, 6, 23, "Print"],
Cell[927458, 25020, 423, 5, 62, "Print"],
Cell[927884, 25027, 584, 10, 43, "Print"],
Cell[928471, 25039, 319, 7, 23, "Print"],
Cell[928793, 25048, 149, 2, 23, "Print"],
Cell[928945, 25052, 307, 7, 23, "Print"],
Cell[929255, 25061, 110, 1, 23, "Print"],
Cell[929368, 25064, 273, 6, 23, "Print"],
Cell[929644, 25072, 423, 5, 62, "Print"],
Cell[930070, 25079, 582, 10, 43, "Print"],
Cell[930655, 25091, 321, 7, 23, "Print"],
Cell[930979, 25100, 151, 2, 23, "Print"],
Cell[931133, 25104, 305, 7, 23, "Print"],
Cell[931441, 25113, 110, 1, 23, "Print"],
Cell[931554, 25116, 275, 6, 23, "Print"],
Cell[931832, 25124, 423, 5, 62, "Print"],
Cell[932258, 25131, 580, 10, 43, "Print"],
Cell[932841, 25143, 283, 6, 23, "Print"],
Cell[933127, 25151, 149, 2, 23, "Print"],
Cell[933279, 25155, 304, 7, 23, "Print"],
Cell[933586, 25164, 110, 1, 23, "Print"],
Cell[933699, 25167, 273, 6, 23, "Print"],
Cell[933975, 25175, 423, 5, 62, "Print"],
Cell[934401, 25182, 585, 10, 43, "Print"],
Cell[934989, 25194, 287, 6, 23, "Print"],
Cell[935279, 25202, 149, 2, 23, "Print"],
Cell[935431, 25206, 305, 7, 23, "Print"],
Cell[935739, 25215, 110, 1, 23, "Print"],
Cell[935852, 25218, 273, 6, 23, "Print"],
Cell[936128, 25226, 421, 5, 62, "Print"],
Cell[936552, 25233, 582, 10, 43, "Print"],
Cell[937137, 25245, 88, 1, 23, "Print"],
Cell[937228, 25248, 423, 5, 62, "Print"],
Cell[937654, 25255, 582, 10, 43, "Print"],
Cell[938239, 25267, 1071, 20, 43, "Print"],
Cell[939313, 25289, 445, 9, 23, "Print"],
Cell[939761, 25300, 139, 2, 23, "Print"],
Cell[939903, 25304, 91, 1, 23, "Print"],
Cell[939997, 25307, 155, 2, 23, "Print"],
Cell[940155, 25311, 146, 2, 23, "Print"],
Cell[940304, 25315, 287, 6, 23, "Print"],
Cell[940594, 25323, 305, 7, 23, "Print"],
Cell[940902, 25332, 165, 2, 23, "Print"],
Cell[941070, 25336, 519, 10, 23, "Print"],
Cell[941592, 25348, 137, 2, 23, "Print"],
Cell[941732, 25352, 142, 2, 23, "Print"],
Cell[941877, 25356, 287, 6, 23, "Print"],
Cell[942167, 25364, 151, 2, 23, "Print"],
Cell[942321, 25368, 307, 7, 23, "Print"],
Cell[942631, 25377, 110, 1, 23, "Print"],
Cell[942744, 25380, 273, 6, 23, "Print"],
Cell[943020, 25388, 423, 5, 62, "Print"],
Cell[943446, 25395, 586, 10, 43, "Print"],
Cell[944035, 25407, 285, 6, 23, "Print"],
Cell[944323, 25415, 149, 2, 23, "Print"],
Cell[944475, 25419, 305, 7, 23, "Print"],
Cell[944783, 25428, 110, 1, 23, "Print"],
Cell[944896, 25431, 273, 6, 23, "Print"],
Cell[945172, 25439, 423, 5, 62, "Print"],
Cell[945598, 25446, 579, 10, 43, "Print"],
Cell[946180, 25458, 284, 6, 23, "Print"],
Cell[946467, 25466, 148, 2, 23, "Print"],
Cell[946618, 25470, 305, 7, 23, "Print"],
Cell[946926, 25479, 110, 1, 23, "Print"],
Cell[947039, 25482, 273, 6, 23, "Print"],
Cell[947315, 25490, 421, 5, 62, "Print"],
Cell[947739, 25497, 587, 11, 43, "Print"],
Cell[948329, 25510, 321, 7, 23, "Print"],
Cell[948653, 25519, 151, 2, 23, "Print"],
Cell[948807, 25523, 304, 7, 23, "Print"],
Cell[949114, 25532, 108, 1, 23, "Print"],
Cell[949225, 25535, 273, 6, 23, "Print"],
Cell[949501, 25543, 423, 5, 62, "Print"],
Cell[949927, 25550, 578, 10, 43, "Print"],
Cell[950508, 25562, 321, 7, 23, "Print"],
Cell[950832, 25571, 151, 2, 23, "Print"],
Cell[950986, 25575, 305, 7, 23, "Print"],
Cell[951294, 25584, 108, 1, 23, "Print"],
Cell[951405, 25587, 273, 6, 23, "Print"],
Cell[951681, 25595, 421, 5, 62, "Print"],
Cell[952105, 25602, 580, 10, 43, "Print"],
Cell[952688, 25614, 319, 7, 23, "Print"],
Cell[953010, 25623, 149, 2, 23, "Print"],
Cell[953162, 25627, 307, 7, 23, "Print"],
Cell[953472, 25636, 108, 1, 23, "Print"],
Cell[953583, 25639, 275, 6, 23, "Print"],
Cell[953861, 25647, 421, 5, 62, "Print"],
Cell[954285, 25654, 579, 10, 43, "Print"],
Cell[954867, 25666, 283, 6, 23, "Print"],
Cell[955153, 25674, 151, 2, 23, "Print"],
Cell[955307, 25678, 305, 7, 23, "Print"],
Cell[955615, 25687, 110, 1, 23, "Print"],
Cell[955728, 25690, 275, 6, 23, "Print"],
Cell[956006, 25698, 423, 5, 62, "Print"],
Cell[956432, 25705, 583, 10, 43, "Print"],
Cell[957018, 25717, 287, 6, 23, "Print"],
Cell[957308, 25725, 151, 2, 23, "Print"],
Cell[957462, 25729, 303, 7, 23, "Print"],
Cell[957768, 25738, 110, 1, 23, "Print"],
Cell[957881, 25741, 269, 6, 23, "Print"],
Cell[958153, 25749, 423, 5, 62, "Print"],
Cell[958579, 25756, 581, 10, 43, "Print"],
Cell[959163, 25768, 321, 7, 23, "Print"],
Cell[959487, 25777, 149, 2, 23, "Print"],
Cell[959639, 25781, 307, 7, 23, "Print"],
Cell[959949, 25790, 110, 1, 23, "Print"],
Cell[960062, 25793, 275, 6, 23, "Print"],
Cell[960340, 25801, 421, 5, 62, "Print"],
Cell[960764, 25808, 578, 10, 43, "Print"],
Cell[961345, 25820, 319, 7, 23, "Print"],
Cell[961667, 25829, 151, 2, 23, "Print"],
Cell[961821, 25833, 305, 7, 23, "Print"],
Cell[962129, 25842, 108, 1, 23, "Print"],
Cell[962240, 25845, 275, 6, 23, "Print"],
Cell[962518, 25853, 421, 5, 62, "Print"],
Cell[962942, 25860, 579, 10, 43, "Print"],
Cell[963524, 25872, 321, 7, 23, "Print"],
Cell[963848, 25881, 151, 2, 23, "Print"],
Cell[964002, 25885, 307, 7, 23, "Print"],
Cell[964312, 25894, 108, 1, 23, "Print"],
Cell[964423, 25897, 275, 6, 23, "Print"],
Cell[964701, 25905, 423, 5, 62, "Print"],
Cell[965127, 25912, 582, 10, 43, "Print"],
Cell[965712, 25924, 88, 1, 23, "Print"],
Cell[965803, 25927, 423, 5, 62, "Print"],
Cell[966229, 25934, 585, 10, 43, "Print"],
Cell[966817, 25946, 1122, 22, 43, "Print"],
Cell[967942, 25970, 443, 9, 23, "Print"],
Cell[968388, 25981, 141, 2, 23, "Print"],
Cell[968532, 25985, 93, 1, 23, "Print"],
Cell[968628, 25988, 153, 2, 23, "Print"],
Cell[968784, 25992, 146, 2, 23, "Print"],
Cell[968933, 25996, 287, 6, 23, "Print"],
Cell[969223, 26004, 305, 7, 23, "Print"],
Cell[969531, 26013, 165, 2, 23, "Print"],
Cell[969699, 26017, 457, 9, 23, "Print"],
Cell[970159, 26028, 137, 2, 23, "Print"],
Cell[970299, 26032, 140, 2, 23, "Print"],
Cell[970442, 26036, 287, 6, 23, "Print"],
Cell[970732, 26044, 151, 2, 23, "Print"],
Cell[970886, 26048, 304, 7, 23, "Print"],
Cell[971193, 26057, 108, 1, 23, "Print"],
Cell[971304, 26060, 273, 6, 23, "Print"],
Cell[971580, 26068, 423, 5, 62, "Print"],
Cell[972006, 26075, 589, 11, 43, "Print"],
Cell[972598, 26088, 284, 6, 23, "Print"],
Cell[972885, 26096, 148, 2, 23, "Print"],
Cell[973036, 26100, 305, 7, 23, "Print"],
Cell[973344, 26109, 108, 1, 23, "Print"],
Cell[973455, 26112, 275, 6, 23, "Print"],
Cell[973733, 26120, 423, 5, 62, "Print"],
Cell[974159, 26127, 589, 11, 43, "Print"],
Cell[974751, 26140, 285, 6, 23, "Print"],
Cell[975039, 26148, 149, 2, 23, "Print"],
Cell[975191, 26152, 305, 7, 23, "Print"],
Cell[975499, 26161, 110, 1, 23, "Print"],
Cell[975612, 26164, 275, 6, 23, "Print"],
Cell[975890, 26172, 423, 5, 62, "Print"],
Cell[976316, 26179, 586, 10, 43, "Print"],
Cell[976905, 26191, 321, 7, 23, "Print"],
Cell[977229, 26200, 151, 2, 23, "Print"],
Cell[977383, 26204, 305, 7, 23, "Print"],
Cell[977691, 26213, 110, 1, 23, "Print"],
Cell[977804, 26216, 273, 6, 23, "Print"],
Cell[978080, 26224, 423, 5, 62, "Print"],
Cell[978506, 26231, 582, 10, 43, "Print"],
Cell[979091, 26243, 319, 7, 23, "Print"],
Cell[979413, 26252, 151, 2, 23, "Print"],
Cell[979567, 26256, 307, 7, 23, "Print"],
Cell[979877, 26265, 108, 1, 23, "Print"],
Cell[979988, 26268, 273, 6, 23, "Print"],
Cell[980264, 26276, 423, 5, 62, "Print"],
Cell[980690, 26283, 583, 10, 43, "Print"],
Cell[981276, 26295, 321, 7, 23, "Print"],
Cell[981600, 26304, 149, 2, 23, "Print"],
Cell[981752, 26308, 305, 7, 23, "Print"],
Cell[982060, 26317, 110, 1, 23, "Print"],
Cell[982173, 26320, 273, 6, 23, "Print"],
Cell[982449, 26328, 423, 5, 62, "Print"],
Cell[982875, 26335, 583, 10, 43, "Print"],
Cell[983461, 26347, 283, 6, 23, "Print"],
Cell[983747, 26355, 149, 2, 23, "Print"],
Cell[983899, 26359, 305, 7, 23, "Print"],
Cell[984207, 26368, 110, 1, 23, "Print"],
Cell[984320, 26371, 275, 6, 23, "Print"],
Cell[984598, 26379, 421, 5, 62, "Print"],
Cell[985022, 26386, 582, 10, 43, "Print"],
Cell[985607, 26398, 285, 6, 23, "Print"],
Cell[985895, 26406, 149, 2, 23, "Print"],
Cell[986047, 26410, 305, 7, 23, "Print"],
Cell[986355, 26419, 108, 1, 23, "Print"],
Cell[986466, 26422, 273, 6, 23, "Print"],
Cell[986742, 26430, 423, 5, 62, "Print"],
Cell[987168, 26437, 585, 10, 43, "Print"],
Cell[987756, 26449, 319, 7, 23, "Print"],
Cell[988078, 26458, 149, 2, 23, "Print"],
Cell[988230, 26462, 307, 7, 23, "Print"],
Cell[988540, 26471, 108, 1, 23, "Print"],
Cell[988651, 26474, 272, 6, 23, "Print"],
Cell[988926, 26482, 423, 5, 62, "Print"],
Cell[989352, 26489, 585, 10, 43, "Print"],
Cell[989940, 26501, 319, 7, 23, "Print"],
Cell[990262, 26510, 151, 2, 23, "Print"],
Cell[990416, 26514, 305, 7, 23, "Print"],
Cell[990724, 26523, 108, 1, 23, "Print"],
Cell[990835, 26526, 273, 6, 23, "Print"],
Cell[991111, 26534, 421, 5, 62, "Print"],
Cell[991535, 26541, 584, 10, 43, "Print"],
Cell[992122, 26553, 319, 7, 23, "Print"],
Cell[992444, 26562, 149, 2, 23, "Print"],
Cell[992596, 26566, 305, 7, 23, "Print"],
Cell[992904, 26575, 108, 1, 23, "Print"],
Cell[993015, 26578, 273, 6, 23, "Print"],
Cell[993291, 26586, 419, 5, 62, "Print"],
Cell[993713, 26593, 587, 10, 43, "Print"],
Cell[994303, 26605, 318, 7, 23, "Print"],
Cell[994624, 26614, 151, 2, 23, "Print"],
Cell[994778, 26618, 305, 7, 23, "Print"],
Cell[995086, 26627, 110, 1, 23, "Print"],
Cell[995199, 26630, 273, 6, 23, "Print"],
Cell[995475, 26638, 423, 5, 62, "Print"],
Cell[995901, 26645, 583, 10, 43, "Print"],
Cell[996487, 26657, 318, 7, 23, "Print"],
Cell[996808, 26666, 151, 2, 23, "Print"],
Cell[996962, 26670, 307, 7, 23, "Print"],
Cell[997272, 26679, 110, 1, 23, "Print"],
Cell[997385, 26682, 273, 6, 23, "Print"],
Cell[997661, 26690, 423, 5, 62, "Print"],
Cell[998087, 26697, 581, 10, 43, "Print"],
Cell[998671, 26709, 315, 7, 23, "Print"],
Cell[998989, 26718, 148, 2, 23, "Print"],
Cell[999140, 26722, 307, 7, 23, "Print"],
Cell[999450, 26731, 107, 1, 23, "Print"],
Cell[999560, 26734, 275, 6, 23, "Print"],
Cell[999838, 26742, 423, 5, 62, "Print"],
Cell[1000264, 26749, 582, 10, 43, "Print"],
Cell[1000849, 26761, 321, 7, 23, "Print"],
Cell[1001173, 26770, 151, 2, 23, "Print"],
Cell[1001327, 26774, 305, 7, 23, "Print"],
Cell[1001635, 26783, 108, 1, 23, "Print"],
Cell[1001746, 26786, 272, 6, 23, "Print"],
Cell[1002021, 26794, 421, 5, 62, "Print"],
Cell[1002445, 26801, 582, 10, 43, "Print"],
Cell[1003030, 26813, 321, 7, 23, "Print"],
Cell[1003354, 26822, 151, 2, 23, "Print"],
Cell[1003508, 26826, 305, 7, 23, "Print"],
Cell[1003816, 26835, 110, 1, 23, "Print"],
Cell[1003929, 26838, 273, 6, 23, "Print"],
Cell[1004205, 26846, 420, 5, 62, "Print"],
Cell[1004628, 26853, 582, 10, 43, "Print"],
Cell[1005213, 26865, 86, 1, 23, "Print"],
Cell[1005302, 26868, 423, 5, 62, "Print"],
Cell[1005728, 26875, 584, 10, 43, "Print"],
Cell[1006315, 26887, 1120, 22, 43, "Print"],
Cell[1007438, 26911, 445, 9, 23, "Print"],
Cell[1007886, 26922, 139, 2, 23, "Print"],
Cell[1008028, 26926, 90, 1, 23, "Print"],
Cell[1008121, 26929, 153, 2, 23, "Print"],
Cell[1008277, 26933, 148, 2, 23, "Print"],
Cell[1008428, 26937, 287, 6, 23, "Print"],
Cell[1008718, 26945, 305, 7, 23, "Print"],
Cell[1009026, 26954, 167, 2, 23, "Print"],
Cell[1009196, 26958, 519, 10, 23, "Print"],
Cell[1009718, 26970, 137, 2, 23, "Print"],
Cell[1009858, 26974, 142, 2, 23, "Print"],
Cell[1010003, 26978, 284, 6, 23, "Print"],
Cell[1010290, 26986, 148, 2, 23, "Print"],
Cell[1010441, 26990, 307, 7, 23, "Print"],
Cell[1010751, 26999, 108, 1, 23, "Print"],
Cell[1010862, 27002, 275, 6, 23, "Print"],
Cell[1011140, 27010, 423, 5, 62, "Print"],
Cell[1011566, 27017, 582, 10, 43, "Print"],
Cell[1012151, 27029, 285, 6, 23, "Print"],
Cell[1012439, 27037, 149, 2, 23, "Print"],
Cell[1012591, 27041, 307, 7, 23, "Print"],
Cell[1012901, 27050, 110, 1, 23, "Print"],
Cell[1013014, 27053, 275, 6, 23, "Print"],
Cell[1013292, 27061, 423, 5, 62, "Print"],
Cell[1013718, 27068, 583, 10, 43, "Print"],
Cell[1014304, 27080, 287, 6, 23, "Print"],
Cell[1014594, 27088, 151, 2, 23, "Print"],
Cell[1014748, 27092, 305, 7, 23, "Print"],
Cell[1015056, 27101, 107, 1, 23, "Print"],
Cell[1015166, 27104, 275, 6, 23, "Print"],
Cell[1015444, 27112, 421, 5, 62, "Print"],
Cell[1015868, 27119, 579, 10, 43, "Print"],
Cell[1016450, 27131, 319, 7, 23, "Print"],
Cell[1016772, 27140, 151, 2, 23, "Print"],
Cell[1016926, 27144, 307, 7, 23, "Print"],
Cell[1017236, 27153, 108, 1, 23, "Print"],
Cell[1017347, 27156, 275, 6, 23, "Print"],
Cell[1017625, 27164, 423, 5, 62, "Print"],
Cell[1018051, 27171, 581, 10, 43, "Print"],
Cell[1018635, 27183, 319, 7, 23, "Print"],
Cell[1018957, 27192, 149, 2, 23, "Print"],
Cell[1019109, 27196, 305, 7, 23, "Print"],
Cell[1019417, 27205, 110, 1, 23, "Print"],
Cell[1019530, 27208, 273, 6, 23, "Print"],
Cell[1019806, 27216, 423, 5, 62, "Print"],
Cell[1020232, 27223, 585, 10, 43, "Print"],
Cell[1020820, 27235, 321, 7, 23, "Print"],
Cell[1021144, 27244, 151, 2, 23, "Print"],
Cell[1021298, 27248, 305, 7, 23, "Print"],
Cell[1021606, 27257, 110, 1, 23, "Print"],
Cell[1021719, 27260, 275, 6, 23, "Print"],
Cell[1021997, 27268, 421, 5, 62, "Print"],
Cell[1022421, 27275, 581, 10, 43, "Print"],
Cell[1023005, 27287, 283, 6, 23, "Print"],
Cell[1023291, 27295, 149, 2, 23, "Print"],
Cell[1023443, 27299, 303, 7, 23, "Print"],
Cell[1023749, 27308, 108, 1, 23, "Print"],
Cell[1023860, 27311, 272, 6, 23, "Print"],
Cell[1024135, 27319, 421, 5, 62, "Print"],
Cell[1024559, 27326, 583, 10, 43, "Print"],
Cell[1025145, 27338, 287, 6, 23, "Print"],
Cell[1025435, 27346, 151, 2, 23, "Print"],
Cell[1025589, 27350, 305, 7, 23, "Print"],
Cell[1025897, 27359, 110, 1, 23, "Print"],
Cell[1026010, 27362, 275, 6, 23, "Print"],
Cell[1026288, 27370, 423, 5, 62, "Print"],
Cell[1026714, 27377, 584, 10, 43, "Print"],
Cell[1027301, 27389, 318, 7, 23, "Print"],
Cell[1027622, 27398, 148, 2, 23, "Print"],
Cell[1027773, 27402, 307, 7, 23, "Print"],
Cell[1028083, 27411, 110, 1, 23, "Print"],
Cell[1028196, 27414, 271, 6, 23, "Print"],
Cell[1028470, 27422, 421, 5, 62, "Print"],
Cell[1028894, 27429, 579, 10, 43, "Print"],
Cell[1029476, 27441, 321, 7, 23, "Print"],
Cell[1029800, 27450, 151, 2, 23, "Print"],
Cell[1029954, 27454, 305, 7, 23, "Print"],
Cell[1030262, 27463, 110, 1, 23, "Print"],
Cell[1030375, 27466, 275, 6, 23, "Print"],
Cell[1030653, 27474, 423, 5, 62, "Print"],
Cell[1031079, 27481, 580, 10, 43, "Print"],
Cell[1031662, 27493, 321, 7, 23, "Print"],
Cell[1031986, 27502, 151, 2, 23, "Print"],
Cell[1032140, 27506, 307, 7, 23, "Print"],
Cell[1032450, 27515, 108, 1, 23, "Print"],
Cell[1032561, 27518, 275, 6, 23, "Print"],
Cell[1032839, 27526, 423, 5, 62, "Print"],
Cell[1033265, 27533, 581, 10, 43, "Print"],
Cell[1033849, 27545, 316, 7, 23, "Print"],
Cell[1034168, 27554, 151, 2, 23, "Print"],
Cell[1034322, 27558, 305, 7, 23, "Print"],
Cell[1034630, 27567, 108, 1, 23, "Print"],
Cell[1034741, 27570, 271, 6, 23, "Print"],
Cell[1035015, 27578, 421, 5, 62, "Print"],
Cell[1035439, 27585, 582, 10, 43, "Print"],
Cell[1036024, 27597, 316, 7, 23, "Print"],
Cell[1036343, 27606, 149, 2, 23, "Print"],
Cell[1036495, 27610, 307, 7, 23, "Print"],
Cell[1036805, 27619, 108, 1, 23, "Print"],
Cell[1036916, 27622, 273, 6, 23, "Print"],
Cell[1037192, 27630, 423, 5, 62, "Print"],
Cell[1037618, 27637, 580, 10, 43, "Print"],
Cell[1038201, 27649, 86, 1, 23, "Print"],
Cell[1038290, 27652, 420, 5, 62, "Print"],
Cell[1038713, 27659, 583, 10, 43, "Print"],
Cell[1039299, 27671, 1112, 22, 43, "Print"],
Cell[1040414, 27695, 379, 8, 23, "Print"],
Cell[1040796, 27705, 141, 2, 23, "Print"],
Cell[1040940, 27709, 95, 1, 23, "Print"],
Cell[1041038, 27712, 153, 2, 23, "Print"],
Cell[1041194, 27716, 146, 2, 23, "Print"],
Cell[1041343, 27720, 287, 6, 23, "Print"],
Cell[1041633, 27728, 305, 7, 23, "Print"],
Cell[1041941, 27737, 167, 2, 23, "Print"],
Cell[1042111, 27741, 515, 10, 23, "Print"],
Cell[1042629, 27753, 137, 2, 23, "Print"],
Cell[1042769, 27757, 142, 2, 23, "Print"],
Cell[1042914, 27761, 287, 6, 23, "Print"],
Cell[1043204, 27769, 151, 2, 23, "Print"],
Cell[1043358, 27773, 305, 7, 23, "Print"],
Cell[1043666, 27782, 108, 1, 23, "Print"],
Cell[1043777, 27785, 273, 6, 23, "Print"],
Cell[1044053, 27793, 421, 5, 62, "Print"],
Cell[1044477, 27800, 585, 10, 43, "Print"],
Cell[1045065, 27812, 284, 6, 23, "Print"],
Cell[1045352, 27820, 148, 2, 23, "Print"],
Cell[1045503, 27824, 305, 7, 23, "Print"],
Cell[1045811, 27833, 108, 1, 23, "Print"],
Cell[1045922, 27836, 275, 6, 23, "Print"],
Cell[1046200, 27844, 421, 5, 62, "Print"],
Cell[1046624, 27851, 582, 10, 43, "Print"],
Cell[1047209, 27863, 285, 6, 23, "Print"],
Cell[1047497, 27871, 149, 2, 23, "Print"]
}, Open  ]],
Cell[1047661, 27876, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[1048330, 27892, 305, 7, 23, "Print"],
Cell[1048638, 27901, 108, 1, 23, "Print"],
Cell[1048749, 27904, 275, 6, 23, "Print"],
Cell[1049027, 27912, 423, 5, 62, "Print"],
Cell[1049453, 27919, 582, 10, 43, "Print"],
Cell[1050038, 27931, 88, 1, 23, "Print"],
Cell[1050129, 27934, 423, 5, 62, "Print"],
Cell[1050555, 27941, 583, 10, 43, "Print"],
Cell[1051141, 27953, 1079, 20, 43, "Print"],
Cell[1052223, 27975, 445, 9, 23, "Print"],
Cell[1052671, 27986, 141, 2, 23, "Print"],
Cell[1052815, 27990, 92, 1, 23, "Print"],
Cell[1052910, 27993, 153, 2, 23, "Print"],
Cell[1053066, 27997, 145, 2, 23, "Print"],
Cell[1053214, 28001, 289, 6, 23, "Print"],
Cell[1053506, 28009, 303, 7, 23, "Print"],
Cell[1053812, 28018, 167, 2, 23, "Print"],
Cell[1053982, 28022, 517, 10, 23, "Print"],
Cell[1054502, 28034, 135, 2, 23, "Print"],
Cell[1054640, 28038, 140, 2, 23, "Print"],
Cell[1054783, 28042, 285, 6, 23, "Print"],
Cell[1055071, 28050, 149, 2, 23, "Print"],
Cell[1055223, 28054, 307, 7, 23, "Print"],
Cell[1055533, 28063, 110, 1, 23, "Print"],
Cell[1055646, 28066, 275, 6, 23, "Print"],
Cell[1055924, 28074, 419, 5, 62, "Print"],
Cell[1056346, 28081, 579, 10, 43, "Print"],
Cell[1056928, 28093, 283, 6, 23, "Print"],
Cell[1057214, 28101, 147, 2, 23, "Print"],
Cell[1057364, 28105, 307, 7, 23, "Print"],
Cell[1057674, 28114, 110, 1, 23, "Print"],
Cell[1057787, 28117, 275, 6, 23, "Print"],
Cell[1058065, 28125, 421, 5, 62, "Print"],
Cell[1058489, 28132, 578, 10, 43, "Print"],
Cell[1059070, 28144, 287, 6, 23, "Print"],
Cell[1059360, 28152, 151, 2, 23, "Print"]
}, Open  ]],
Cell[1059526, 28157, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[1060193, 28173, 305, 7, 23, "Print"],
Cell[1060501, 28182, 108, 1, 23, "Print"],
Cell[1060612, 28185, 255, 6, 23, "Print"],
Cell[1060870, 28193, 421, 5, 62, "Print"],
Cell[1061294, 28200, 581, 10, 43, "Print"],
Cell[1061878, 28212, 285, 6, 23, "Print"],
Cell[1062166, 28220, 149, 2, 23, "Print"]
}, Open  ]],
Cell[1062330, 28225, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[1062999, 28241, 307, 7, 23, "Print"],
Cell[1063309, 28250, 110, 1, 23, "Print"],
Cell[1063422, 28253, 251, 6, 23, "Print"],
Cell[1063676, 28261, 423, 5, 62, "Print"],
Cell[1064102, 28268, 585, 10, 43, "Print"],
Cell[1064690, 28280, 319, 7, 23, "Print"],
Cell[1065012, 28289, 149, 2, 23, "Print"]
}, Open  ]],
Cell[1065176, 28294, 642, 12, 48, "Message"],
Cell[1065821, 28308, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[1066381, 28323, 307, 7, 23, "Print"],
Cell[1066691, 28332, 108, 1, 23, "Print"],
Cell[1066802, 28335, 275, 6, 23, "Print"],
Cell[1067080, 28343, 421, 5, 62, "Print"],
Cell[1067504, 28350, 583, 10, 43, "Print"],
Cell[1068090, 28362, 320, 7, 23, "Print"],
Cell[1068413, 28371, 151, 2, 23, "Print"],
Cell[1068567, 28375, 307, 7, 23, "Print"],
Cell[1068877, 28384, 108, 1, 23, "Print"],
Cell[1068988, 28387, 252, 6, 23, "Print"],
Cell[1069243, 28395, 421, 5, 62, "Print"],
Cell[1069667, 28402, 582, 10, 43, "Print"],
Cell[1070252, 28414, 318, 7, 23, "Print"],
Cell[1070573, 28423, 149, 2, 23, "Print"],
Cell[1070725, 28427, 307, 7, 23, "Print"],
Cell[1071035, 28436, 110, 1, 23, "Print"],
Cell[1071148, 28439, 273, 6, 23, "Print"],
Cell[1071424, 28447, 423, 5, 62, "Print"],
Cell[1071850, 28454, 585, 10, 43, "Print"],
Cell[1072438, 28466, 320, 7, 23, "Print"],
Cell[1072761, 28475, 151, 2, 23, "Print"],
Cell[1072915, 28479, 307, 7, 23, "Print"],
Cell[1073225, 28488, 108, 1, 23, "Print"],
Cell[1073336, 28491, 255, 6, 23, "Print"],
Cell[1073594, 28499, 423, 5, 62, "Print"],
Cell[1074020, 28506, 580, 10, 43, "Print"],
Cell[1074603, 28518, 320, 7, 23, "Print"],
Cell[1074926, 28527, 151, 2, 23, "Print"],
Cell[1075080, 28531, 305, 7, 23, "Print"],
Cell[1075388, 28540, 108, 1, 23, "Print"],
Cell[1075499, 28543, 275, 6, 23, "Print"],
Cell[1075777, 28551, 421, 5, 62, "Print"],
Cell[1076201, 28558, 582, 10, 43, "Print"],
Cell[1076786, 28570, 318, 7, 23, "Print"],
Cell[1077107, 28579, 149, 2, 23, "Print"],
Cell[1077259, 28583, 307, 7, 23, "Print"],
Cell[1077569, 28592, 108, 1, 23, "Print"],
Cell[1077680, 28595, 275, 6, 23, "Print"],
Cell[1077958, 28603, 420, 5, 62, "Print"],
Cell[1078381, 28610, 584, 10, 43, "Print"],
Cell[1078968, 28622, 317, 7, 23, "Print"],
Cell[1079288, 28631, 148, 2, 23, "Print"],
Cell[1079439, 28635, 307, 7, 23, "Print"],
Cell[1079749, 28644, 110, 1, 23, "Print"],
Cell[1079862, 28647, 275, 6, 23, "Print"],
Cell[1080140, 28655, 423, 5, 62, "Print"],
Cell[1080566, 28662, 585, 10, 43, "Print"],
Cell[1081154, 28674, 86, 1, 23, "Print"],
Cell[1081243, 28677, 423, 5, 62, "Print"],
Cell[1081669, 28684, 591, 11, 43, "Print"],
Cell[1082263, 28697, 1116, 22, 43, "Print"],
Cell[1083382, 28721, 443, 9, 23, "Print"],
Cell[1083828, 28732, 141, 2, 23, "Print"],
Cell[1083972, 28736, 97, 1, 23, "Print"],
Cell[1084072, 28739, 155, 2, 23, "Print"],
Cell[1084230, 28743, 146, 2, 23, "Print"],
Cell[1084379, 28747, 287, 6, 23, "Print"],
Cell[1084669, 28755, 305, 7, 23, "Print"],
Cell[1084977, 28764, 167, 2, 23, "Print"],
Cell[1085147, 28768, 457, 9, 23, "Print"],
Cell[1085607, 28779, 134, 2, 23, "Print"],
Cell[1085744, 28783, 142, 2, 23, "Print"],
Cell[1085889, 28787, 287, 6, 23, "Print"],
Cell[1086179, 28795, 151, 2, 23, "Print"],
Cell[1086333, 28799, 307, 7, 23, "Print"],
Cell[1086643, 28808, 108, 1, 23, "Print"],
Cell[1086754, 28811, 275, 6, 23, "Print"],
Cell[1087032, 28819, 421, 5, 62, "Print"],
Cell[1087456, 28826, 584, 10, 43, "Print"],
Cell[1088043, 28838, 285, 6, 23, "Print"],
Cell[1088331, 28846, 149, 2, 23, "Print"],
Cell[1088483, 28850, 305, 7, 23, "Print"],
Cell[1088791, 28859, 108, 1, 23, "Print"],
Cell[1088902, 28862, 273, 6, 23, "Print"],
Cell[1089178, 28870, 423, 5, 62, "Print"],
Cell[1089604, 28877, 585, 10, 43, "Print"],
Cell[1090192, 28889, 287, 6, 23, "Print"],
Cell[1090482, 28897, 151, 2, 23, "Print"]
}, Open  ]],
Cell[1090648, 28902, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[1091317, 28918, 307, 7, 23, "Print"],
Cell[1091627, 28927, 108, 1, 23, "Print"],
Cell[1091738, 28930, 255, 6, 23, "Print"],
Cell[1091996, 28938, 423, 5, 62, "Print"],
Cell[1092422, 28945, 584, 10, 43, "Print"],
Cell[1093009, 28957, 285, 6, 23, "Print"],
Cell[1093297, 28965, 149, 2, 23, "Print"]
}, Open  ]],
Cell[1093461, 28970, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[1094130, 28986, 307, 7, 23, "Print"],
Cell[1094440, 28995, 110, 1, 23, "Print"],
Cell[1094553, 28998, 253, 6, 23, "Print"],
Cell[1094809, 29006, 423, 5, 62, "Print"],
Cell[1095235, 29013, 584, 10, 43, "Print"],
Cell[1095822, 29025, 321, 7, 23, "Print"],
Cell[1096146, 29034, 151, 2, 23, "Print"]
}, Open  ]],
Cell[1096312, 29039, 642, 12, 48, "Message"],
Cell[1096957, 29053, 535, 11, 25, "Message"],
Cell[CellGroupData[{
Cell[1097517, 29068, 305, 7, 23, "Print"],
Cell[1097825, 29077, 108, 1, 23, "Print"],
Cell[1097936, 29080, 251, 6, 23, "Print"],
Cell[1098190, 29088, 421, 5, 62, "Print"],
Cell[1098614, 29095, 583, 10, 43, "Print"],
Cell[1099200, 29107, 318, 7, 23, "Print"],
Cell[1099521, 29116, 151, 2, 23, "Print"],
Cell[1099675, 29120, 307, 7, 23, "Print"],
Cell[1099985, 29129, 110, 1, 23, "Print"],
Cell[1100098, 29132, 273, 6, 23, "Print"],
Cell[1100374, 29140, 423, 5, 62, "Print"],
Cell[1100800, 29147, 590, 11, 43, "Print"],
Cell[1101393, 29160, 320, 7, 23, "Print"],
Cell[1101716, 29169, 151, 2, 23, "Print"],
Cell[1101870, 29173, 307, 7, 23, "Print"],
Cell[1102180, 29182, 108, 1, 23, "Print"],
Cell[1102291, 29185, 275, 6, 23, "Print"],
Cell[1102569, 29193, 421, 5, 62, "Print"],
Cell[1102993, 29200, 583, 10, 43, "Print"],
Cell[1103579, 29212, 318, 7, 23, "Print"],
Cell[1103900, 29221, 149, 2, 23, "Print"],
Cell[1104052, 29225, 305, 7, 23, "Print"],
Cell[1104360, 29234, 110, 1, 23, "Print"],
Cell[1104473, 29237, 251, 6, 23, "Print"],
Cell[1104727, 29245, 423, 5, 62, "Print"],
Cell[1105153, 29252, 584, 10, 43, "Print"],
Cell[1105740, 29264, 320, 7, 23, "Print"],
Cell[1106063, 29273, 151, 2, 23, "Print"],
Cell[1106217, 29277, 307, 7, 23, "Print"],
Cell[1106527, 29286, 110, 1, 23, "Print"],
Cell[1106640, 29289, 275, 6, 23, "Print"],
Cell[1106918, 29297, 423, 5, 62, "Print"],
Cell[1107344, 29304, 585, 10, 43, "Print"],
Cell[1107932, 29316, 318, 7, 23, "Print"],
Cell[1108253, 29325, 149, 2, 23, "Print"],
Cell[1108405, 29329, 304, 7, 23, "Print"],
Cell[1108712, 29338, 110, 1, 23, "Print"],
Cell[1108825, 29341, 273, 6, 23, "Print"],
Cell[1109101, 29349, 423, 5, 62, "Print"],
Cell[1109527, 29356, 591, 11, 43, "Print"],
Cell[1110121, 29369, 86, 1, 23, "Print"],
Cell[1110210, 29372, 423, 5, 62, "Print"],
Cell[1110636, 29379, 580, 10, 43, "Print"],
Cell[1111219, 29391, 1114, 22, 43, "Print"],
Cell[1112336, 29415, 383, 8, 23, "Print"],
Cell[1112722, 29425, 141, 2, 23, "Print"],
Cell[1112866, 29429, 93, 1, 23, "Print"],
Cell[1112962, 29432, 153, 2, 23, "Print"],
Cell[1113118, 29436, 148, 2, 23, "Print"],
Cell[1113269, 29440, 289, 6, 23, "Print"],
Cell[1113561, 29448, 305, 7, 23, "Print"],
Cell[1113869, 29457, 167, 2, 23, "Print"],
Cell[1114039, 29461, 519, 10, 23, "Print"],
Cell[1114561, 29473, 137, 2, 23, "Print"],
Cell[1114701, 29477, 142, 2, 23, "Print"],
Cell[1114846, 29481, 287, 6, 23, "Print"],
Cell[1115136, 29489, 149, 2, 23, "Print"],
Cell[1115288, 29493, 307, 7, 23, "Print"],
Cell[1115598, 29502, 107, 1, 23, "Print"],
Cell[1115708, 29505, 273, 6, 23, "Print"],
Cell[1115984, 29513, 423, 5, 62, "Print"],
Cell[1116410, 29520, 581, 10, 43, "Print"],
Cell[1116994, 29532, 287, 6, 23, "Print"],
Cell[1117284, 29540, 151, 2, 23, "Print"],
Cell[1117438, 29544, 307, 7, 23, "Print"],
Cell[1117748, 29553, 110, 1, 23, "Print"],
Cell[1117861, 29556, 273, 6, 23, "Print"],
Cell[1118137, 29564, 423, 5, 62, "Print"],
Cell[1118563, 29571, 580, 10, 43, "Print"],
Cell[1119146, 29583, 287, 6, 23, "Print"],
Cell[1119436, 29591, 151, 2, 23, "Print"],
Cell[1119590, 29595, 307, 7, 23, "Print"],
Cell[1119900, 29604, 110, 1, 23, "Print"],
Cell[1120013, 29607, 270, 6, 23, "Print"],
Cell[1120286, 29615, 423, 5, 62, "Print"],
Cell[1120712, 29622, 583, 10, 43, "Print"],
Cell[1121298, 29634, 318, 7, 23, "Print"],
Cell[1121619, 29643, 148, 2, 23, "Print"],
Cell[1121770, 29647, 305, 7, 23, "Print"],
Cell[1122078, 29656, 108, 1, 23, "Print"],
Cell[1122189, 29659, 275, 6, 23, "Print"],
Cell[1122467, 29667, 423, 5, 62, "Print"],
Cell[1122893, 29674, 582, 10, 43, "Print"],
Cell[1123478, 29686, 321, 7, 23, "Print"],
Cell[1123802, 29695, 151, 2, 23, "Print"],
Cell[1123956, 29699, 307, 7, 23, "Print"],
Cell[1124266, 29708, 108, 1, 23, "Print"],
Cell[1124377, 29711, 272, 6, 23, "Print"],
Cell[1124652, 29719, 423, 5, 62, "Print"],
Cell[1125078, 29726, 579, 10, 43, "Print"],
Cell[1125660, 29738, 321, 7, 23, "Print"],
Cell[1125984, 29747, 151, 2, 23, "Print"],
Cell[1126138, 29751, 305, 7, 23, "Print"],
Cell[1126446, 29760, 110, 1, 23, "Print"],
Cell[1126559, 29763, 275, 6, 23, "Print"],
Cell[1126837, 29771, 421, 5, 62, "Print"],
Cell[1127261, 29778, 581, 10, 43, "Print"],
Cell[1127845, 29790, 283, 6, 23, "Print"],
Cell[1128131, 29798, 149, 2, 23, "Print"],
Cell[1128283, 29802, 307, 7, 23, "Print"],
Cell[1128593, 29811, 108, 1, 23, "Print"],
Cell[1128704, 29814, 275, 6, 23, "Print"],
Cell[1128982, 29822, 423, 5, 62, "Print"],
Cell[1129408, 29829, 585, 10, 43, "Print"],
Cell[1129996, 29841, 285, 6, 23, "Print"],
Cell[1130284, 29849, 149, 2, 23, "Print"],
Cell[1130436, 29853, 304, 7, 23, "Print"],
Cell[1130743, 29862, 110, 1, 23, "Print"],
Cell[1130856, 29865, 275, 6, 23, "Print"],
Cell[1131134, 29873, 423, 5, 62, "Print"],
Cell[1131560, 29880, 581, 10, 43, "Print"],
Cell[1132144, 29892, 319, 7, 23, "Print"],
Cell[1132466, 29901, 149, 2, 23, "Print"],
Cell[1132618, 29905, 305, 7, 23, "Print"],
Cell[1132926, 29914, 107, 1, 23, "Print"],
Cell[1133036, 29917, 269, 6, 23, "Print"],
Cell[1133308, 29925, 423, 5, 62, "Print"],
Cell[1133734, 29932, 581, 10, 43, "Print"],
Cell[1134318, 29944, 319, 7, 23, "Print"],
Cell[1134640, 29953, 149, 2, 23, "Print"],
Cell[1134792, 29957, 304, 7, 23, "Print"],
Cell[1135099, 29966, 110, 1, 23, "Print"],
Cell[1135212, 29969, 275, 6, 23, "Print"],
Cell[1135490, 29977, 421, 5, 62, "Print"],
Cell[1135914, 29984, 578, 10, 43, "Print"],
Cell[1136495, 29996, 321, 7, 23, "Print"],
Cell[1136819, 30005, 151, 2, 23, "Print"],
Cell[1136973, 30009, 305, 7, 23, "Print"],
Cell[1137281, 30018, 108, 1, 23, "Print"],
Cell[1137392, 30021, 273, 6, 23, "Print"],
Cell[1137668, 30029, 423, 5, 62, "Print"],
Cell[1138094, 30036, 581, 10, 43, "Print"],
Cell[1138678, 30048, 318, 7, 23, "Print"],
Cell[1138999, 30057, 151, 2, 23, "Print"],
Cell[1139153, 30061, 307, 7, 23, "Print"],
Cell[1139463, 30070, 108, 1, 23, "Print"],
Cell[1139574, 30073, 275, 6, 23, "Print"],
Cell[1139852, 30081, 421, 5, 62, "Print"],
Cell[1140276, 30088, 580, 10, 43, "Print"],
Cell[1140859, 30100, 86, 1, 23, "Print"],
Cell[1140948, 30103, 423, 5, 62, "Print"],
Cell[1141374, 30110, 580, 10, 43, "Print"],
Cell[1141957, 30122, 1121, 22, 43, "Print"],
Cell[1143081, 30146, 443, 9, 23, "Print"],
Cell[1143527, 30157, 139, 2, 23, "Print"],
Cell[1143669, 30161, 93, 1, 23, "Print"],
Cell[1143765, 30164, 155, 2, 23, "Print"],
Cell[1143923, 30168, 146, 2, 23, "Print"],
Cell[1144072, 30172, 287, 6, 23, "Print"],
Cell[1144362, 30180, 307, 7, 23, "Print"],
Cell[1144672, 30189, 167, 2, 23, "Print"],
Cell[1144842, 30193, 519, 10, 23, "Print"],
Cell[1145364, 30205, 137, 2, 23, "Print"],
Cell[1145504, 30209, 142, 2, 23, "Print"],
Cell[1145649, 30213, 285, 6, 23, "Print"],
Cell[1145937, 30221, 151, 2, 23, "Print"],
Cell[1146091, 30225, 305, 7, 23, "Print"],
Cell[1146399, 30234, 108, 1, 23, "Print"],
Cell[1146510, 30237, 275, 6, 23, "Print"],
Cell[1146788, 30245, 423, 5, 62, "Print"],
Cell[1147214, 30252, 589, 11, 43, "Print"],
Cell[1147806, 30265, 287, 6, 23, "Print"],
Cell[1148096, 30273, 151, 2, 23, "Print"],
Cell[1148250, 30277, 307, 7, 23, "Print"],
Cell[1148560, 30286, 107, 1, 23, "Print"],
Cell[1148670, 30289, 273, 6, 23, "Print"],
Cell[1148946, 30297, 423, 5, 62, "Print"],
Cell[1149372, 30304, 580, 10, 43, "Print"],
Cell[1149955, 30316, 287, 6, 23, "Print"],
Cell[1150245, 30324, 151, 2, 23, "Print"],
Cell[1150399, 30328, 307, 7, 23, "Print"],
Cell[1150709, 30337, 107, 1, 23, "Print"],
Cell[1150819, 30340, 275, 6, 23, "Print"],
Cell[1151097, 30348, 421, 5, 62, "Print"],
Cell[1151521, 30355, 581, 10, 43, "Print"],
Cell[1152105, 30367, 319, 7, 23, "Print"],
Cell[1152427, 30376, 151, 2, 23, "Print"],
Cell[1152581, 30380, 303, 7, 23, "Print"],
Cell[1152887, 30389, 108, 1, 23, "Print"],
Cell[1152998, 30392, 275, 6, 23, "Print"],
Cell[1153276, 30400, 423, 5, 62, "Print"],
Cell[1153702, 30407, 580, 10, 43, "Print"],
Cell[1154285, 30419, 319, 7, 23, "Print"],
Cell[1154607, 30428, 149, 2, 23, "Print"],
Cell[1154759, 30432, 303, 7, 23, "Print"],
Cell[1155065, 30441, 108, 1, 23, "Print"],
Cell[1155176, 30444, 275, 6, 23, "Print"],
Cell[1155454, 30452, 423, 5, 62, "Print"],
Cell[1155880, 30459, 581, 10, 43, "Print"],
Cell[1156464, 30471, 321, 7, 23, "Print"],
Cell[1156788, 30480, 151, 2, 23, "Print"],
Cell[1156942, 30484, 307, 7, 23, "Print"],
Cell[1157252, 30493, 108, 1, 23, "Print"],
Cell[1157363, 30496, 275, 6, 23, "Print"],
Cell[1157641, 30504, 423, 5, 62, "Print"],
Cell[1158067, 30511, 582, 10, 43, "Print"],
Cell[1158652, 30523, 285, 6, 23, "Print"],
Cell[1158940, 30531, 151, 2, 23, "Print"],
Cell[1159094, 30535, 307, 7, 23, "Print"],
Cell[1159404, 30544, 110, 1, 23, "Print"],
Cell[1159517, 30547, 273, 6, 23, "Print"],
Cell[1159793, 30555, 421, 5, 62, "Print"],
Cell[1160217, 30562, 582, 10, 43, "Print"],
Cell[1160802, 30574, 285, 6, 23, "Print"],
Cell[1161090, 30582, 149, 2, 23, "Print"],
Cell[1161242, 30586, 305, 7, 23, "Print"],
Cell[1161550, 30595, 110, 1, 23, "Print"],
Cell[1161663, 30598, 275, 6, 23, "Print"],
Cell[1161941, 30606, 423, 5, 62, "Print"],
Cell[1162367, 30613, 581, 10, 43, "Print"],
Cell[1162951, 30625, 321, 7, 23, "Print"],
Cell[1163275, 30634, 151, 2, 23, "Print"],
Cell[1163429, 30638, 305, 7, 23, "Print"],
Cell[1163737, 30647, 108, 1, 23, "Print"],
Cell[1163848, 30650, 273, 6, 23, "Print"],
Cell[1164124, 30658, 421, 5, 62, "Print"],
Cell[1164548, 30665, 580, 10, 43, "Print"],
Cell[1165131, 30677, 88, 1, 23, "Print"],
Cell[1165222, 30680, 423, 5, 62, "Print"],
Cell[1165648, 30687, 583, 10, 43, "Print"],
Cell[1166234, 30699, 1081, 20, 43, "Print"],
Cell[1167318, 30721, 445, 9, 23, "Print"],
Cell[1167766, 30732, 139, 2, 23, "Print"],
Cell[1167908, 30736, 95, 1, 23, "Print"],
Cell[1168006, 30739, 153, 2, 23, "Print"],
Cell[1168162, 30743, 148, 2, 23, "Print"],
Cell[1168313, 30747, 289, 6, 23, "Print"],
Cell[1168605, 30755, 305, 7, 23, "Print"],
Cell[1168913, 30764, 165, 2, 23, "Print"],
Cell[1169081, 30768, 517, 10, 23, "Print"],
Cell[1169601, 30780, 135, 2, 23, "Print"],
Cell[1169739, 30784, 140, 2, 23, "Print"],
Cell[1169882, 30788, 287, 6, 23, "Print"],
Cell[1170172, 30796, 151, 2, 23, "Print"],
Cell[1170326, 30800, 307, 7, 23, "Print"],
Cell[1170636, 30809, 108, 1, 23, "Print"],
Cell[1170747, 30812, 275, 6, 23, "Print"],
Cell[1171025, 30820, 423, 5, 62, "Print"],
Cell[1171451, 30827, 584, 10, 43, "Print"],
Cell[1172038, 30839, 285, 6, 23, "Print"],
Cell[1172326, 30847, 149, 2, 23, "Print"],
Cell[1172478, 30851, 307, 7, 23, "Print"],
Cell[1172788, 30860, 110, 1, 23, "Print"],
Cell[1172901, 30863, 273, 6, 23, "Print"],
Cell[1173177, 30871, 423, 5, 62, "Print"],
Cell[1173603, 30878, 592, 11, 43, "Print"],
Cell[1174198, 30891, 287, 6, 23, "Print"],
Cell[1174488, 30899, 151, 2, 23, "Print"]
}, Open  ]],
Cell[1174654, 30904, 642, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[1175321, 30920, 305, 7, 23, "Print"],
Cell[1175629, 30929, 108, 1, 23, "Print"],
Cell[1175740, 30932, 275, 6, 23, "Print"],
Cell[1176018, 30940, 423, 5, 62, "Print"],
Cell[1176444, 30947, 588, 11, 43, "Print"],
Cell[1177035, 30960, 287, 6, 23, "Print"],
Cell[1177325, 30968, 151, 2, 23, "Print"]
}, Open  ]],
Cell[1177491, 30973, 644, 12, 48, "Message"],
Cell[CellGroupData[{
Cell[1178160, 30989, 307, 7, 23, "Print"],
Cell[1178470, 30998, 110, 1, 23, "Print"],
Cell[1178583, 31001, 251, 6, 23, "Print"],
Cell[1178837, 31009, 421, 5, 62, "Print"],
Cell[1179261, 31016, 582, 10, 43, "Print"],
Cell[1179846, 31028, 86, 1, 23, "Print"],
Cell[1179935, 31031, 421, 5, 62, "Print"],
Cell[1180359, 31038, 581, 10, 43, "Print"],
Cell[1180943, 31050, 1058, 20, 43, "Print"],
Cell[1182004, 31072, 381, 8, 23, "Print"],
Cell[1182388, 31082, 485, 10, 23, "Print"],
Cell[1182876, 31094, 153, 2, 23, "Print"],
Cell[1183032, 31098, 114, 1, 23, "Print"],
Cell[1183149, 31101, 139, 2, 23, "Print"],
Cell[1183291, 31105, 154, 2, 23, "Print"],
Cell[1183448, 31109, 124, 2, 23, "Print"],
Cell[1183575, 31113, 98, 1, 23, "Print"],
Cell[1183676, 31116, 117, 1, 23, "Print"],
Cell[1183796, 31119, 128, 2, 23, "Print"],
Cell[1183927, 31123, 273, 4, 62, "Print"],
Cell[1184203, 31129, 488, 10, 23, "Print"],
Cell[1184694, 31141, 259, 4, 43, "Print"],
Cell[1184956, 31147, 337, 7, 23, "Print"],
Cell[1185296, 31156, 377, 8, 23, "Print"],
Cell[1185676, 31166, 119, 1, 23, "Print"],
Cell[1185798, 31169, 162, 2, 23, "Print"],
Cell[1185963, 31173, 86, 1, 23, "Print"],
Cell[1186052, 31176, 89, 1, 23, "Print"],
Cell[1186144, 31179, 92, 1, 23, "Print"],
Cell[1186239, 31182, 200, 5, 23, "Print"],
Cell[1186442, 31189, 117, 1, 23, "Print"],
Cell[1186562, 31192, 421, 9, 23, "Print"],
Cell[1186986, 31203, 362, 8, 23, "Print"],
Cell[1187351, 31213, 201, 5, 23, "Print"],
Cell[1187555, 31220, 375, 8, 23, "Print"],
Cell[1187933, 31230, 198, 5, 23, "Print"],
Cell[1188134, 31237, 115, 1, 23, "Print"],
Cell[1188252, 31240, 419, 9, 23, "Print"],
Cell[1188674, 31251, 357, 8, 23, "Print"],
Cell[1189034, 31261, 201, 5, 23, "Print"],
Cell[1189238, 31268, 375, 8, 23, "Print"],
Cell[1189616, 31278, 200, 5, 23, "Print"],
Cell[1189819, 31285, 115, 1, 23, "Print"],
Cell[1189937, 31288, 421, 9, 23, "Print"],
Cell[1190361, 31299, 364, 8, 23, "Print"],
Cell[1190728, 31309, 201, 5, 23, "Print"],
Cell[1190932, 31316, 375, 8, 23, "Print"],
Cell[1191310, 31326, 200, 5, 23, "Print"],
Cell[1191513, 31333, 117, 1, 23, "Print"],
Cell[1191633, 31336, 421, 9, 23, "Print"],
Cell[1192057, 31347, 362, 8, 23, "Print"],
Cell[1192422, 31357, 199, 5, 23, "Print"],
Cell[1192624, 31364, 373, 8, 23, "Print"],
Cell[1193000, 31374, 198, 5, 23, "Print"],
Cell[1193201, 31381, 115, 1, 23, "Print"],
Cell[1193319, 31384, 421, 9, 23, "Print"],
Cell[1193743, 31395, 364, 8, 23, "Print"],
Cell[1194110, 31405, 201, 5, 23, "Print"],
Cell[1194314, 31412, 375, 8, 23, "Print"],
Cell[1194692, 31422, 200, 5, 23, "Print"],
Cell[1194895, 31429, 117, 1, 23, "Print"],
Cell[1195015, 31432, 357, 8, 23, "Print"],
Cell[1195375, 31442, 98, 1, 23, "Print"],
Cell[1195476, 31445, 220, 5, 23, "Print"],
Cell[1195699, 31452, 224, 5, 23, "Print"],
Cell[1195926, 31459, 362, 8, 23, "Print"],
Cell[1196291, 31469, 199, 5, 23, "Print"],
Cell[1196493, 31476, 371, 8, 23, "Print"],
Cell[1196867, 31486, 198, 5, 23, "Print"],
Cell[1197068, 31493, 115, 1, 23, "Print"],
Cell[1197186, 31496, 358, 8, 23, "Print"],
Cell[1197547, 31506, 95, 1, 23, "Print"],
Cell[1197645, 31509, 222, 5, 23, "Print"],
Cell[1197870, 31516, 221, 5, 23, "Print"],
Cell[1198094, 31523, 271, 4, 62, "Print"],
Cell[1198368, 31529, 488, 10, 23, "Print"],
Cell[1198859, 31541, 261, 4, 43, "Print"],
Cell[1199123, 31547, 337, 7, 23, "Print"],
Cell[1199463, 31556, 204, 3, 23, "Print"],
Cell[1199670, 31561, 406, 8, 23, "Print"],
Cell[1200079, 31571, 238, 5, 23, "Print"],
Cell[1200320, 31578, 153, 2, 23, "Print"],
Cell[1200476, 31582, 402, 8, 23, "Print"],
Cell[1200881, 31592, 170, 2, 23, "Print"],
Cell[1201054, 31596, 400, 8, 23, "Print"],
Cell[1201457, 31606, 365, 8, 23, "Print"],
Cell[1201825, 31616, 362, 8, 23, "Print"],
Cell[1202190, 31626, 199, 5, 23, "Print"],
Cell[1202392, 31633, 377, 8, 23, "Print"],
Cell[1202772, 31643, 511, 10, 23, "Print"],
Cell[1203286, 31655, 124, 2, 23, "Print"],
Cell[1203413, 31659, 92, 1, 23, "Print"],
Cell[1203508, 31662, 117, 1, 23, "Print"],
Cell[1203628, 31665, 127, 2, 23, "Print"],
Cell[1203758, 31669, 271, 4, 62, "Print"],
Cell[1204032, 31675, 486, 10, 23, "Print"],
Cell[1204521, 31687, 259, 4, 43, "Print"],
Cell[1204783, 31693, 335, 7, 23, "Print"],
Cell[1205121, 31702, 377, 8, 23, "Print"],
Cell[1205501, 31712, 117, 1, 23, "Print"],
Cell[1205621, 31715, 160, 2, 23, "Print"],
Cell[1205784, 31719, 86, 1, 23, "Print"],
Cell[1205873, 31722, 89, 1, 23, "Print"],
Cell[1205965, 31725, 77, 1, 23, "Print"],
Cell[1206045, 31728, 200, 5, 23, "Print"],
Cell[1206248, 31735, 115, 1, 23, "Print"],
Cell[1206366, 31738, 419, 9, 23, "Print"],
Cell[1206788, 31749, 362, 8, 23, "Print"],
Cell[1207153, 31759, 197, 5, 23, "Print"],
Cell[1207353, 31766, 375, 8, 23, "Print"],
Cell[1207731, 31776, 200, 5, 23, "Print"],
Cell[1207934, 31783, 115, 1, 23, "Print"],
Cell[1208052, 31786, 421, 9, 23, "Print"],
Cell[1208476, 31797, 359, 8, 23, "Print"],
Cell[1208838, 31807, 199, 5, 23, "Print"],
Cell[1209040, 31814, 311, 7, 23, "Print"],
Cell[1209354, 31823, 198, 5, 23, "Print"],
Cell[1209555, 31830, 115, 1, 23, "Print"],
Cell[1209673, 31833, 421, 9, 23, "Print"],
Cell[1210097, 31844, 364, 8, 23, "Print"],
Cell[1210464, 31854, 201, 5, 23, "Print"],
Cell[1210668, 31861, 375, 8, 23, "Print"],
Cell[1211046, 31871, 200, 5, 23, "Print"],
Cell[1211249, 31878, 117, 1, 23, "Print"],
Cell[1211369, 31881, 417, 9, 23, "Print"],
Cell[1211789, 31892, 364, 8, 23, "Print"],
Cell[1212156, 31902, 197, 5, 23, "Print"],
Cell[1212356, 31909, 373, 8, 23, "Print"],
Cell[1212732, 31919, 198, 5, 23, "Print"],
Cell[1212933, 31926, 115, 1, 23, "Print"],
Cell[1213051, 31929, 421, 9, 23, "Print"],
Cell[1213475, 31940, 362, 8, 23, "Print"],
Cell[1213840, 31950, 199, 5, 23, "Print"],
Cell[1214042, 31957, 311, 7, 23, "Print"],
Cell[1214356, 31966, 198, 5, 23, "Print"],
Cell[1214557, 31973, 115, 1, 23, "Print"],
Cell[1214675, 31976, 421, 9, 23, "Print"],
Cell[1215099, 31987, 98, 1, 23, "Print"],
Cell[1215200, 31990, 222, 5, 23, "Print"],
Cell[1215425, 31997, 222, 5, 23, "Print"],
Cell[1215650, 32004, 364, 8, 23, "Print"],
Cell[1216017, 32014, 197, 5, 23, "Print"],
Cell[1216217, 32021, 375, 8, 23, "Print"],
Cell[1216595, 32031, 200, 5, 23, "Print"],
Cell[1216798, 32038, 117, 1, 23, "Print"],
Cell[1216918, 32041, 355, 8, 23, "Print"],
Cell[1217276, 32051, 96, 1, 23, "Print"],
Cell[1217375, 32054, 222, 5, 23, "Print"],
Cell[1217600, 32061, 222, 5, 23, "Print"],
Cell[1217825, 32068, 271, 4, 62, "Print"],
Cell[1218099, 32074, 482, 9, 23, "Print"],
Cell[1218584, 32085, 259, 4, 43, "Print"],
Cell[1218846, 32091, 337, 7, 23, "Print"],
Cell[1219186, 32100, 202, 3, 23, "Print"],
Cell[1219391, 32105, 413, 8, 23, "Print"],
Cell[1219807, 32115, 235, 5, 23, "Print"],
Cell[1220045, 32122, 155, 2, 23, "Print"],
Cell[1220203, 32126, 402, 8, 23, "Print"],
Cell[1220608, 32136, 170, 2, 23, "Print"],
Cell[1220781, 32140, 402, 8, 23, "Print"],
Cell[1221186, 32150, 362, 8, 23, "Print"],
Cell[1221551, 32160, 362, 8, 23, "Print"],
Cell[1221916, 32170, 199, 5, 23, "Print"],
Cell[1222118, 32177, 377, 8, 23, "Print"],
Cell[1222498, 32187, 513, 10, 23, "Print"],
Cell[1223014, 32199, 124, 2, 23, "Print"],
Cell[1223141, 32203, 94, 1, 23, "Print"],
Cell[1223238, 32206, 117, 1, 23, "Print"],
Cell[1223358, 32209, 130, 2, 23, "Print"],
Cell[1223491, 32213, 273, 4, 62, "Print"],
Cell[1223767, 32219, 486, 10, 23, "Print"],
Cell[1224256, 32231, 259, 4, 43, "Print"],
Cell[1224518, 32237, 336, 7, 23, "Print"],
Cell[1224857, 32246, 375, 8, 23, "Print"],
Cell[1225235, 32256, 119, 1, 23, "Print"],
Cell[1225357, 32259, 162, 2, 23, "Print"],
Cell[1225522, 32263, 84, 1, 23, "Print"],
Cell[1225609, 32266, 87, 1, 23, "Print"],
Cell[1225699, 32269, 90, 1, 23, "Print"],
Cell[1225792, 32272, 198, 5, 23, "Print"],
Cell[1225993, 32279, 115, 1, 23, "Print"],
Cell[1226111, 32282, 419, 9, 23, "Print"],
Cell[1226533, 32293, 364, 8, 23, "Print"],
Cell[1226900, 32303, 201, 5, 23, "Print"],
Cell[1227104, 32310, 375, 8, 23, "Print"],
Cell[1227482, 32320, 200, 5, 23, "Print"],
Cell[1227685, 32327, 117, 1, 23, "Print"],
Cell[1227805, 32330, 359, 8, 23, "Print"],
Cell[1228167, 32340, 359, 8, 23, "Print"],
Cell[1228529, 32350, 199, 5, 23, "Print"],
Cell[1228731, 32357, 375, 8, 23, "Print"],
Cell[1229109, 32367, 200, 5, 23, "Print"],
Cell[1229312, 32374, 117, 1, 23, "Print"],
Cell[1229432, 32377, 359, 8, 23, "Print"],
Cell[1229794, 32387, 359, 8, 23, "Print"],
Cell[1230156, 32397, 199, 5, 23, "Print"],
Cell[1230358, 32404, 373, 8, 23, "Print"],
Cell[1230734, 32414, 198, 5, 23, "Print"],
Cell[1230935, 32421, 115, 1, 23, "Print"],
Cell[1231053, 32424, 418, 9, 23, "Print"],
Cell[1231474, 32435, 362, 8, 23, "Print"],
Cell[1231839, 32445, 199, 5, 23, "Print"],
Cell[1232041, 32452, 313, 7, 23, "Print"],
Cell[1232357, 32461, 200, 5, 23, "Print"],
Cell[1232560, 32468, 117, 1, 23, "Print"],
Cell[1232680, 32471, 421, 9, 23, "Print"],
Cell[1233104, 32482, 364, 8, 23, "Print"],
Cell[1233471, 32492, 201, 5, 23, "Print"],
Cell[1233675, 32499, 311, 7, 23, "Print"],
Cell[1233989, 32508, 200, 5, 23, "Print"],
Cell[1234192, 32515, 115, 1, 23, "Print"],
Cell[1234310, 32518, 419, 9, 23, "Print"],
Cell[1234732, 32529, 96, 1, 23, "Print"],
Cell[1234831, 32532, 222, 5, 23, "Print"],
Cell[1235056, 32539, 222, 5, 23, "Print"],
Cell[1235281, 32546, 364, 8, 23, "Print"],
Cell[1235648, 32556, 201, 5, 23, "Print"],
Cell[1235852, 32563, 375, 8, 23, "Print"],
Cell[1236230, 32573, 200, 5, 23, "Print"],
Cell[1236433, 32580, 117, 1, 23, "Print"],
Cell[1236553, 32583, 417, 9, 23, "Print"],
Cell[1236973, 32594, 96, 1, 23, "Print"],
Cell[1237072, 32597, 220, 5, 23, "Print"],
Cell[1237295, 32604, 224, 5, 23, "Print"],
Cell[1237522, 32611, 273, 4, 62, "Print"],
Cell[1237798, 32617, 487, 10, 23, "Print"],
Cell[1238288, 32629, 259, 4, 43, "Print"],
Cell[1238550, 32635, 338, 7, 23, "Print"],
Cell[1238891, 32644, 204, 3, 23, "Print"],
Cell[1239098, 32649, 414, 8, 23, "Print"],
Cell[1239515, 32659, 235, 5, 23, "Print"],
Cell[1239753, 32666, 155, 2, 23, "Print"],
Cell[1239911, 32670, 401, 8, 23, "Print"],
Cell[1240315, 32680, 170, 2, 23, "Print"],
Cell[1240488, 32684, 403, 8, 23, "Print"],
Cell[1240894, 32694, 364, 8, 23, "Print"],
Cell[1241261, 32704, 362, 8, 23, "Print"],
Cell[1241626, 32714, 199, 5, 23, "Print"],
Cell[1241828, 32721, 377, 8, 23, "Print"],
Cell[1242208, 32731, 513, 10, 23, "Print"],
Cell[1242724, 32743, 124, 2, 23, "Print"],
Cell[1242851, 32747, 91, 1, 23, "Print"],
Cell[1242945, 32750, 119, 1, 23, "Print"],
Cell[1243067, 32753, 128, 2, 23, "Print"],
Cell[1243198, 32757, 271, 4, 62, "Print"],
Cell[1243472, 32763, 486, 10, 23, "Print"],
Cell[1243961, 32775, 259, 4, 43, "Print"],
Cell[1244223, 32781, 336, 7, 23, "Print"],
Cell[1244562, 32790, 377, 8, 23, "Print"],
Cell[1244942, 32800, 117, 1, 23, "Print"],
Cell[1245062, 32803, 160, 2, 23, "Print"],
Cell[1245225, 32807, 84, 1, 23, "Print"],
Cell[1245312, 32810, 87, 1, 23, "Print"],
Cell[1245402, 32813, 75, 1, 23, "Print"],
Cell[1245480, 32816, 200, 5, 23, "Print"],
Cell[1245683, 32823, 117, 1, 23, "Print"],
Cell[1245803, 32826, 353, 8, 23, "Print"],
Cell[1246159, 32836, 359, 8, 23, "Print"],
Cell[1246521, 32846, 199, 5, 23, "Print"],
Cell[1246723, 32853, 373, 8, 23, "Print"],
Cell[1247099, 32863, 198, 5, 23, "Print"],
Cell[1247300, 32870, 115, 1, 23, "Print"],
Cell[1247418, 32873, 419, 9, 23, "Print"],
Cell[1247840, 32884, 364, 8, 23, "Print"],
Cell[1248207, 32894, 194, 5, 23, "Print"],
Cell[1248404, 32901, 372, 8, 23, "Print"],
Cell[1248779, 32911, 197, 5, 23, "Print"],
Cell[1248979, 32918, 114, 1, 23, "Print"],
Cell[1249096, 32921, 421, 9, 23, "Print"],
Cell[1249520, 32932, 362, 8, 23, "Print"],
Cell[1249885, 32942, 199, 5, 23, "Print"],
Cell[1250087, 32949, 373, 8, 23, "Print"],
Cell[1250463, 32959, 198, 5, 23, "Print"],
Cell[1250664, 32966, 115, 1, 23, "Print"],
Cell[1250782, 32969, 357, 8, 23, "Print"],
Cell[1251142, 32979, 364, 8, 23, "Print"],
Cell[1251509, 32989, 201, 5, 23, "Print"],
Cell[1251713, 32996, 313, 7, 23, "Print"],
Cell[1252029, 33005, 200, 5, 23, "Print"],
Cell[1252232, 33012, 117, 1, 23, "Print"],
Cell[1252352, 33015, 355, 8, 23, "Print"],
Cell[1252710, 33025, 359, 8, 23, "Print"],
Cell[1253072, 33035, 199, 5, 23, "Print"],
Cell[1253274, 33042, 311, 7, 23, "Print"],
Cell[1253588, 33051, 200, 5, 23, "Print"],
Cell[1253791, 33058, 117, 1, 23, "Print"],
Cell[1253911, 33061, 423, 9, 23, "Print"],
Cell[1254337, 33072, 98, 1, 23, "Print"],
Cell[1254438, 33075, 220, 5, 23, "Print"],
Cell[1254661, 33082, 224, 5, 23, "Print"],
Cell[1254888, 33089, 359, 8, 23, "Print"],
Cell[1255250, 33099, 201, 5, 23, "Print"],
Cell[1255454, 33106, 313, 7, 23, "Print"],
Cell[1255770, 33115, 200, 5, 23, "Print"],
Cell[1255973, 33122, 115, 1, 23, "Print"],
Cell[1256091, 33125, 421, 9, 23, "Print"],
Cell[1256515, 33136, 98, 1, 23, "Print"],
Cell[1256616, 33139, 220, 5, 23, "Print"],
Cell[1256839, 33146, 222, 5, 23, "Print"],
Cell[1257064, 33153, 271, 4, 62, "Print"],
Cell[1257338, 33159, 487, 10, 23, "Print"],
Cell[1257828, 33171, 261, 4, 43, "Print"],
Cell[1258092, 33177, 337, 7, 23, "Print"],
Cell[1258432, 33186, 204, 3, 23, "Print"],
Cell[1258639, 33191, 406, 8, 23, "Print"],
Cell[1259048, 33201, 233, 5, 23, "Print"],
Cell[1259284, 33208, 153, 2, 23, "Print"],
Cell[1259440, 33212, 399, 8, 23, "Print"],
Cell[1259842, 33222, 168, 2, 23, "Print"],
Cell[1260013, 33226, 399, 8, 23, "Print"],
Cell[1260415, 33236, 365, 8, 23, "Print"],
Cell[1260783, 33246, 359, 8, 23, "Print"],
Cell[1261145, 33256, 199, 5, 23, "Print"],
Cell[1261347, 33263, 377, 8, 23, "Print"],
Cell[1261727, 33273, 513, 10, 23, "Print"],
Cell[1262243, 33285, 126, 2, 23, "Print"],
Cell[1262372, 33289, 94, 1, 23, "Print"],
Cell[1262469, 33292, 119, 1, 23, "Print"],
Cell[1262591, 33295, 130, 2, 23, "Print"],
Cell[1262724, 33299, 273, 4, 62, "Print"],
Cell[1263000, 33305, 488, 10, 23, "Print"],
Cell[1263491, 33317, 261, 4, 43, "Print"],
Cell[1263755, 33323, 337, 7, 23, "Print"],
Cell[1264095, 33332, 377, 8, 23, "Print"],
Cell[1264475, 33342, 117, 1, 23, "Print"],
Cell[1264595, 33345, 160, 2, 23, "Print"],
Cell[1264758, 33349, 84, 1, 23, "Print"],
Cell[1264845, 33352, 87, 1, 23, "Print"],
Cell[1264935, 33355, 90, 1, 23, "Print"],
Cell[1265028, 33358, 198, 5, 23, "Print"],
Cell[1265229, 33365, 115, 1, 23, "Print"],
Cell[1265347, 33368, 417, 9, 23, "Print"],
Cell[1265767, 33379, 364, 8, 23, "Print"],
Cell[1266134, 33389, 199, 5, 23, "Print"],
Cell[1266336, 33396, 373, 8, 23, "Print"],
Cell[1266712, 33406, 198, 5, 23, "Print"],
Cell[1266913, 33413, 115, 1, 23, "Print"],
Cell[1267031, 33416, 357, 8, 23, "Print"],
Cell[1267391, 33426, 362, 8, 23, "Print"],
Cell[1267756, 33436, 198, 5, 23, "Print"],
Cell[1267957, 33443, 370, 8, 23, "Print"],
Cell[1268330, 33453, 197, 5, 23, "Print"],
Cell[1268530, 33460, 117, 1, 23, "Print"],
Cell[1268650, 33463, 421, 9, 23, "Print"],
Cell[1269074, 33474, 364, 8, 23, "Print"],
Cell[1269441, 33484, 201, 5, 23, "Print"],
Cell[1269645, 33491, 375, 8, 23, "Print"],
Cell[1270023, 33501, 200, 5, 23, "Print"],
Cell[1270226, 33508, 117, 1, 23, "Print"],
Cell[1270346, 33511, 419, 9, 23, "Print"],
Cell[1270768, 33522, 361, 8, 23, "Print"],
Cell[1271132, 33532, 201, 5, 23, "Print"],
Cell[1271336, 33539, 375, 8, 23, "Print"],
Cell[1271714, 33549, 200, 5, 23, "Print"],
Cell[1271917, 33556, 117, 1, 23, "Print"],
Cell[1272037, 33559, 418, 9, 23, "Print"],
Cell[1272458, 33570, 364, 8, 23, "Print"],
Cell[1272825, 33580, 201, 5, 23, "Print"],
Cell[1273029, 33587, 375, 8, 23, "Print"],
Cell[1273407, 33597, 200, 5, 23, "Print"],
Cell[1273610, 33604, 117, 1, 23, "Print"],
Cell[1273730, 33607, 353, 8, 23, "Print"],
Cell[1274086, 33617, 96, 1, 23, "Print"],
Cell[1274185, 33620, 220, 5, 23, "Print"],
Cell[1274408, 33627, 224, 5, 23, "Print"],
Cell[1274635, 33634, 361, 8, 23, "Print"],
Cell[1274999, 33644, 201, 5, 23, "Print"],
Cell[1275203, 33651, 311, 7, 23, "Print"],
Cell[1275517, 33660, 200, 5, 23, "Print"],
Cell[1275720, 33667, 117, 1, 23, "Print"],
Cell[1275840, 33670, 359, 8, 23, "Print"],
Cell[1276202, 33680, 98, 1, 23, "Print"],
Cell[1276303, 33683, 220, 5, 23, "Print"],
Cell[1276526, 33690, 224, 5, 23, "Print"],
Cell[1276753, 33697, 273, 4, 62, "Print"],
Cell[1277029, 33703, 489, 10, 23, "Print"],
Cell[1277521, 33715, 261, 4, 43, "Print"],
Cell[1277785, 33721, 338, 7, 23, "Print"],
Cell[1278126, 33730, 204, 3, 23, "Print"],
Cell[1278333, 33735, 414, 8, 23, "Print"],
Cell[1278750, 33745, 235, 5, 23, "Print"],
Cell[1278988, 33752, 155, 2, 23, "Print"],
Cell[1279146, 33756, 398, 8, 23, "Print"],
Cell[1279547, 33766, 170, 2, 23, "Print"],
Cell[1279720, 33770, 395, 8, 23, "Print"],
Cell[1280118, 33780, 362, 8, 23, "Print"],
Cell[1280483, 33790, 360, 8, 23, "Print"],
Cell[1280846, 33800, 199, 5, 23, "Print"],
Cell[1281048, 33807, 375, 8, 23, "Print"],
Cell[1281426, 33817, 513, 10, 23, "Print"],
Cell[1281942, 33829, 124, 2, 23, "Print"],
Cell[1282069, 33833, 90, 1, 23, "Print"],
Cell[1282162, 33836, 119, 1, 23, "Print"],
Cell[1282284, 33839, 128, 2, 23, "Print"],
Cell[1282415, 33843, 273, 4, 62, "Print"],
Cell[1282691, 33849, 488, 10, 23, "Print"],
Cell[1283182, 33861, 261, 4, 43, "Print"],
Cell[1283446, 33867, 335, 7, 23, "Print"],
Cell[1283784, 33876, 373, 8, 23, "Print"],
Cell[1284160, 33886, 117, 1, 23, "Print"],
Cell[1284280, 33889, 160, 2, 23, "Print"],
Cell[1284443, 33893, 86, 1, 23, "Print"],
Cell[1284532, 33896, 87, 1, 23, "Print"],
Cell[1284622, 33899, 90, 1, 23, "Print"],
Cell[1284715, 33902, 198, 5, 23, "Print"],
Cell[1284916, 33909, 115, 1, 23, "Print"],
Cell[1285034, 33912, 419, 9, 23, "Print"],
Cell[1285456, 33923, 359, 8, 23, "Print"],
Cell[1285818, 33933, 201, 5, 23, "Print"],
Cell[1286022, 33940, 375, 8, 23, "Print"],
Cell[1286400, 33950, 200, 5, 23, "Print"],
Cell[1286603, 33957, 117, 1, 23, "Print"],
Cell[1286723, 33960, 421, 9, 23, "Print"],
Cell[1287147, 33971, 364, 8, 23, "Print"],
Cell[1287514, 33981, 201, 5, 23, "Print"],
Cell[1287718, 33988, 373, 8, 23, "Print"],
Cell[1288094, 33998, 200, 5, 23, "Print"],
Cell[1288297, 34005, 117, 1, 23, "Print"],
Cell[1288417, 34008, 357, 8, 23, "Print"],
Cell[1288777, 34018, 362, 8, 23, "Print"],
Cell[1289142, 34028, 199, 5, 23, "Print"],
Cell[1289344, 34035, 373, 8, 23, "Print"],
Cell[1289720, 34045, 198, 5, 23, "Print"],
Cell[1289921, 34052, 115, 1, 23, "Print"],
Cell[1290039, 34055, 421, 9, 23, "Print"],
Cell[1290463, 34066, 360, 8, 23, "Print"],
Cell[1290826, 34076, 199, 5, 23, "Print"],
Cell[1291028, 34083, 309, 7, 23, "Print"],
Cell[1291340, 34092, 198, 5, 23, "Print"],
Cell[1291541, 34099, 117, 1, 23, "Print"],
Cell[1291661, 34102, 419, 9, 23, "Print"],
Cell[1292083, 34113, 364, 8, 23, "Print"],
Cell[1292450, 34123, 201, 5, 23, "Print"],
Cell[1292654, 34130, 311, 7, 23, "Print"],
Cell[1292968, 34139, 200, 5, 23, "Print"],
Cell[1293171, 34146, 115, 1, 23, "Print"],
Cell[1293289, 34149, 419, 9, 23, "Print"],
Cell[1293711, 34160, 98, 1, 23, "Print"],
Cell[1293812, 34163, 220, 5, 23, "Print"],
Cell[1294035, 34170, 222, 5, 23, "Print"],
Cell[1294260, 34177, 364, 8, 23, "Print"],
Cell[1294627, 34187, 201, 5, 23, "Print"],
Cell[1294831, 34194, 313, 7, 23, "Print"],
Cell[1295147, 34203, 200, 5, 23, "Print"],
Cell[1295350, 34210, 117, 1, 23, "Print"],
Cell[1295470, 34213, 355, 8, 23, "Print"],
Cell[1295828, 34223, 96, 1, 23, "Print"],
Cell[1295927, 34226, 220, 5, 23, "Print"],
Cell[1296150, 34233, 222, 5, 23, "Print"],
Cell[1296375, 34240, 202, 3, 23, "Print"],
Cell[1296580, 34245, 409, 8, 23, "Print"],
Cell[1296992, 34255, 364, 8, 23, "Print"],
Cell[1297359, 34265, 201, 5, 23, "Print"],
Cell[1297563, 34272, 379, 8, 23, "Print"],
Cell[1297945, 34282, 200, 5, 23, "Print"],
Cell[1298148, 34289, 117, 1, 23, "Print"],
Cell[1298268, 34292, 421, 9, 23, "Print"],
Cell[1298692, 34303, 96, 1, 23, "Print"],
Cell[1298791, 34306, 220, 5, 23, "Print"],
Cell[1299014, 34313, 224, 5, 23, "Print"],
Cell[1299241, 34320, 271, 4, 62, "Print"],
Cell[1299515, 34326, 490, 10, 23, "Print"],
Cell[1300008, 34338, 261, 4, 43, "Print"],
Cell[1300272, 34344, 339, 7, 23, "Print"],
Cell[1300614, 34353, 204, 3, 23, "Print"],
Cell[1300821, 34358, 415, 8, 23, "Print"],
Cell[1301239, 34368, 233, 5, 23, "Print"],
Cell[1301475, 34375, 153, 2, 23, "Print"],
Cell[1301631, 34379, 399, 8, 23, "Print"],
Cell[1302033, 34389, 168, 2, 23, "Print"],
Cell[1302204, 34393, 399, 8, 23, "Print"],
Cell[1302606, 34403, 364, 8, 23, "Print"],
Cell[1302973, 34413, 359, 8, 23, "Print"],
Cell[1303335, 34423, 201, 5, 23, "Print"],
Cell[1303539, 34430, 379, 8, 23, "Print"],
Cell[1303921, 34440, 451, 9, 23, "Print"],
Cell[1304375, 34451, 126, 2, 23, "Print"],
Cell[1304504, 34455, 94, 1, 23, "Print"],
Cell[1304601, 34458, 119, 1, 23, "Print"],
Cell[1304723, 34461, 130, 2, 23, "Print"],
Cell[1304856, 34465, 271, 4, 62, "Print"],
Cell[1305130, 34471, 486, 10, 23, "Print"],
Cell[1305619, 34483, 259, 4, 43, "Print"],
Cell[1305881, 34489, 336, 7, 23, "Print"],
Cell[1306220, 34498, 377, 8, 23, "Print"],
Cell[1306600, 34508, 117, 1, 23, "Print"],
Cell[1306720, 34511, 160, 2, 23, "Print"],
Cell[1306883, 34515, 86, 1, 23, "Print"],
Cell[1306972, 34518, 89, 1, 23, "Print"],
Cell[1307064, 34521, 76, 1, 23, "Print"],
Cell[1307143, 34524, 200, 5, 23, "Print"],
Cell[1307346, 34531, 115, 1, 23, "Print"],
Cell[1307464, 34534, 359, 8, 23, "Print"],
Cell[1307826, 34544, 359, 8, 23, "Print"],
Cell[1308188, 34554, 197, 5, 23, "Print"],
Cell[1308388, 34561, 373, 8, 23, "Print"],
Cell[1308764, 34571, 198, 5, 23, "Print"],
Cell[1308965, 34578, 115, 1, 23, "Print"],
Cell[1309083, 34581, 421, 9, 23, "Print"],
Cell[1309507, 34592, 364, 8, 23, "Print"],
Cell[1309874, 34602, 201, 5, 23, "Print"],
Cell[1310078, 34609, 307, 7, 23, "Print"],
Cell[1310388, 34618, 198, 5, 23, "Print"],
Cell[1310589, 34625, 115, 1, 23, "Print"],
Cell[1310707, 34628, 359, 8, 23, "Print"],
Cell[1311069, 34638, 360, 8, 23, "Print"],
Cell[1311432, 34648, 201, 5, 23, "Print"],
Cell[1311636, 34655, 375, 8, 23, "Print"],
Cell[1312014, 34665, 200, 5, 23, "Print"],
Cell[1312217, 34672, 117, 1, 23, "Print"],
Cell[1312337, 34675, 419, 9, 23, "Print"],
Cell[1312759, 34686, 362, 8, 23, "Print"],
Cell[1313124, 34696, 199, 5, 23, "Print"],
Cell[1313326, 34703, 309, 7, 23, "Print"],
Cell[1313638, 34712, 198, 5, 23, "Print"],
Cell[1313839, 34719, 115, 1, 23, "Print"],
Cell[1313957, 34722, 357, 8, 23, "Print"],
Cell[1314317, 34732, 364, 8, 23, "Print"],
Cell[1314684, 34742, 201, 5, 23, "Print"],
Cell[1314888, 34749, 375, 8, 23, "Print"],
Cell[1315266, 34759, 200, 5, 23, "Print"],
Cell[1315469, 34766, 117, 1, 23, "Print"],
Cell[1315589, 34769, 419, 9, 23, "Print"],
Cell[1316011, 34780, 96, 1, 23, "Print"],
Cell[1316110, 34783, 222, 5, 23, "Print"],
Cell[1316335, 34790, 224, 5, 23, "Print"],
Cell[1316562, 34797, 364, 8, 23, "Print"],
Cell[1316929, 34807, 201, 5, 23, "Print"],
Cell[1317133, 34814, 371, 8, 23, "Print"],
Cell[1317507, 34824, 200, 5, 23, "Print"],
Cell[1317710, 34831, 117, 1, 23, "Print"],
Cell[1317830, 34834, 359, 8, 23, "Print"],
Cell[1318192, 34844, 98, 1, 23, "Print"],
Cell[1318293, 34847, 220, 5, 23, "Print"],
Cell[1318516, 34854, 222, 5, 23, "Print"],
Cell[1318741, 34861, 273, 4, 62, "Print"],
Cell[1319017, 34867, 487, 10, 23, "Print"],
Cell[1319507, 34879, 261, 4, 43, "Print"],
Cell[1319771, 34885, 337, 7, 23, "Print"],
Cell[1320111, 34894, 204, 3, 23, "Print"],
Cell[1320318, 34899, 415, 8, 23, "Print"],
Cell[1320736, 34909, 238, 5, 23, "Print"],
Cell[1320977, 34916, 155, 2, 23, "Print"],
Cell[1321135, 34920, 398, 8, 23, "Print"],
Cell[1321536, 34930, 170, 2, 23, "Print"],
Cell[1321709, 34934, 394, 8, 23, "Print"],
Cell[1322106, 34944, 362, 8, 23, "Print"],
Cell[1322471, 34954, 362, 8, 23, "Print"],
Cell[1322836, 34964, 199, 5, 23, "Print"],
Cell[1323038, 34971, 311, 7, 23, "Print"],
Cell[1323352, 34980, 513, 10, 23, "Print"],
Cell[1323868, 34992, 126, 2, 23, "Print"],
Cell[1323997, 34996, 93, 1, 23, "Print"],
Cell[1324093, 34999, 119, 1, 23, "Print"],
Cell[1324215, 35002, 130, 2, 23, "Print"],
Cell[1324348, 35006, 271, 4, 62, "Print"],
Cell[1324622, 35012, 486, 10, 23, "Print"],
Cell[1325111, 35024, 259, 4, 43, "Print"],
Cell[1325373, 35030, 335, 7, 23, "Print"],
Cell[1325711, 35039, 375, 8, 23, "Print"],
Cell[1326089, 35049, 117, 1, 23, "Print"],
Cell[1326209, 35052, 160, 2, 23, "Print"],
Cell[1326372, 35056, 84, 1, 23, "Print"],
Cell[1326459, 35059, 87, 1, 23, "Print"],
Cell[1326549, 35062, 90, 1, 23, "Print"],
Cell[1326642, 35065, 198, 5, 23, "Print"],
Cell[1326843, 35072, 115, 1, 23, "Print"],
Cell[1326961, 35075, 419, 9, 23, "Print"],
Cell[1327383, 35086, 364, 8, 23, "Print"],
Cell[1327750, 35096, 201, 5, 23, "Print"],
Cell[1327954, 35103, 375, 8, 23, "Print"],
Cell[1328332, 35113, 200, 5, 23, "Print"],
Cell[1328535, 35120, 117, 1, 23, "Print"],
Cell[1328655, 35123, 421, 9, 23, "Print"],
Cell[1329079, 35134, 364, 8, 23, "Print"],
Cell[1329446, 35144, 199, 5, 23, "Print"],
Cell[1329648, 35151, 373, 8, 23, "Print"],
Cell[1330024, 35161, 198, 5, 23, "Print"],
Cell[1330225, 35168, 115, 1, 23, "Print"],
Cell[1330343, 35171, 419, 9, 23, "Print"],
Cell[1330765, 35182, 362, 8, 23, "Print"],
Cell[1331130, 35192, 197, 5, 23, "Print"],
Cell[1331330, 35199, 309, 7, 23, "Print"],
Cell[1331642, 35208, 198, 5, 23, "Print"],
Cell[1331843, 35215, 115, 1, 23, "Print"],
Cell[1331961, 35218, 421, 9, 23, "Print"],
Cell[1332385, 35229, 362, 8, 23, "Print"],
Cell[1332750, 35239, 199, 5, 23, "Print"],
Cell[1332952, 35246, 373, 8, 23, "Print"],
Cell[1333328, 35256, 198, 5, 23, "Print"],
Cell[1333529, 35263, 115, 1, 23, "Print"],
Cell[1333647, 35266, 421, 9, 23, "Print"],
Cell[1334071, 35277, 361, 8, 23, "Print"],
Cell[1334435, 35287, 201, 5, 23, "Print"],
Cell[1334639, 35294, 311, 7, 23, "Print"],
Cell[1334953, 35303, 200, 5, 23, "Print"],
Cell[1335156, 35310, 117, 1, 23, "Print"],
Cell[1335276, 35313, 417, 9, 23, "Print"],
Cell[1335696, 35324, 96, 1, 23, "Print"],
Cell[1335795, 35327, 222, 5, 23, "Print"],
Cell[1336020, 35334, 224, 5, 23, "Print"],
Cell[1336247, 35341, 364, 8, 23, "Print"],
Cell[1336614, 35351, 201, 5, 23, "Print"],
Cell[1336818, 35358, 375, 8, 23, "Print"],
Cell[1337196, 35368, 200, 5, 23, "Print"],
Cell[1337399, 35375, 115, 1, 23, "Print"],
Cell[1337517, 35378, 419, 9, 23, "Print"],
Cell[1337939, 35389, 96, 1, 23, "Print"],
Cell[1338038, 35392, 220, 5, 23, "Print"],
Cell[1338261, 35399, 222, 5, 23, "Print"],
Cell[1338486, 35406, 204, 3, 23, "Print"],
Cell[1338693, 35411, 411, 8, 23, "Print"],
Cell[1339107, 35421, 364, 8, 23, "Print"],
Cell[1339474, 35431, 197, 5, 23, "Print"],
Cell[1339674, 35438, 317, 7, 23, "Print"],
Cell[1339994, 35447, 200, 5, 23, "Print"],
Cell[1340197, 35454, 115, 1, 23, "Print"],
Cell[1340315, 35457, 355, 8, 23, "Print"],
Cell[1340673, 35467, 96, 1, 23, "Print"],
Cell[1340772, 35470, 222, 5, 23, "Print"],
Cell[1340997, 35477, 224, 5, 23, "Print"],
Cell[1341224, 35484, 271, 4, 62, "Print"],
Cell[1341498, 35490, 486, 10, 23, "Print"],
Cell[1341987, 35502, 261, 4, 43, "Print"],
Cell[1342251, 35508, 337, 7, 23, "Print"],
Cell[1342591, 35517, 204, 3, 23, "Print"],
Cell[1342798, 35522, 411, 8, 23, "Print"],
Cell[1343212, 35532, 238, 5, 23, "Print"],
Cell[1343453, 35539, 153, 2, 23, "Print"],
Cell[1343609, 35543, 397, 8, 23, "Print"],
Cell[1344009, 35553, 170, 2, 23, "Print"],
Cell[1344182, 35557, 401, 8, 23, "Print"],
Cell[1344586, 35567, 358, 8, 23, "Print"],
Cell[1344947, 35577, 364, 8, 23, "Print"],
Cell[1345314, 35587, 199, 5, 23, "Print"],
Cell[1345516, 35594, 377, 8, 23, "Print"],
Cell[1345896, 35604, 515, 10, 23, "Print"],
Cell[1346414, 35616, 124, 2, 23, "Print"],
Cell[1346541, 35620, 93, 1, 23, "Print"],
Cell[1346637, 35623, 117, 1, 23, "Print"],
Cell[1346757, 35626, 130, 2, 23, "Print"],
Cell[1346890, 35630, 271, 4, 62, "Print"],
Cell[1347164, 35636, 486, 10, 23, "Print"],
Cell[1347653, 35648, 261, 4, 43, "Print"],
Cell[1347917, 35654, 337, 7, 23, "Print"],
Cell[1348257, 35663, 377, 8, 23, "Print"],
Cell[1348637, 35673, 119, 1, 23, "Print"],
Cell[1348759, 35676, 162, 2, 23, "Print"],
Cell[1348924, 35680, 86, 1, 23, "Print"],
Cell[1349013, 35683, 89, 1, 23, "Print"],
Cell[1349105, 35686, 92, 1, 23, "Print"],
Cell[1349200, 35689, 200, 5, 23, "Print"],
Cell[1349403, 35696, 117, 1, 23, "Print"],
Cell[1349523, 35699, 353, 8, 23, "Print"],
Cell[1349879, 35709, 364, 8, 23, "Print"],
Cell[1350246, 35719, 201, 5, 23, "Print"],
Cell[1350450, 35726, 372, 8, 23, "Print"],
Cell[1350825, 35736, 197, 5, 23, "Print"],
Cell[1351025, 35743, 114, 1, 23, "Print"],
Cell[1351142, 35746, 355, 8, 23, "Print"],
Cell[1351500, 35756, 361, 8, 23, "Print"],
Cell[1351864, 35766, 199, 5, 23, "Print"],
Cell[1352066, 35773, 311, 7, 23, "Print"],
Cell[1352380, 35782, 200, 5, 23, "Print"],
Cell[1352583, 35789, 117, 1, 23, "Print"],
Cell[1352703, 35792, 419, 9, 23, "Print"],
Cell[1353125, 35803, 361, 8, 23, "Print"],
Cell[1353489, 35813, 199, 5, 23, "Print"],
Cell[1353691, 35820, 309, 7, 23, "Print"],
Cell[1354003, 35829, 200, 5, 23, "Print"],
Cell[1354206, 35836, 117, 1, 23, "Print"],
Cell[1354326, 35839, 421, 9, 23, "Print"],
Cell[1354750, 35850, 362, 8, 23, "Print"],
Cell[1355115, 35860, 199, 5, 23, "Print"],
Cell[1355317, 35867, 373, 8, 23, "Print"],
Cell[1355693, 35877, 198, 5, 23, "Print"],
Cell[1355894, 35884, 115, 1, 23, "Print"],
Cell[1356012, 35887, 419, 9, 23, "Print"],
Cell[1356434, 35898, 362, 8, 23, "Print"],
Cell[1356799, 35908, 201, 5, 23, "Print"],
Cell[1357003, 35915, 375, 8, 23, "Print"],
Cell[1357381, 35925, 200, 5, 23, "Print"],
Cell[1357584, 35932, 115, 1, 23, "Print"],
Cell[1357702, 35935, 357, 8, 23, "Print"],
Cell[1358062, 35945, 98, 1, 23, "Print"],
Cell[1358163, 35948, 222, 5, 23, "Print"],
Cell[1358388, 35955, 224, 5, 23, "Print"],
Cell[1358615, 35962, 364, 8, 23, "Print"],
Cell[1358982, 35972, 201, 5, 23, "Print"],
Cell[1359186, 35979, 373, 8, 23, "Print"],
Cell[1359562, 35989, 200, 5, 23, "Print"],
Cell[1359765, 35996, 117, 1, 23, "Print"],
Cell[1359885, 35999, 421, 9, 23, "Print"],
Cell[1360309, 36010, 98, 1, 23, "Print"],
Cell[1360410, 36013, 222, 5, 23, "Print"],
Cell[1360635, 36020, 222, 5, 23, "Print"],
Cell[1360860, 36027, 271, 4, 62, "Print"],
Cell[1361134, 36033, 484, 10, 23, "Print"],
Cell[1361621, 36045, 259, 4, 43, "Print"],
Cell[1361883, 36051, 338, 7, 23, "Print"],
Cell[1362224, 36060, 204, 3, 23, "Print"],
Cell[1362431, 36065, 412, 8, 23, "Print"],
Cell[1362846, 36075, 236, 5, 23, "Print"],
Cell[1363085, 36082, 155, 2, 23, "Print"],
Cell[1363243, 36086, 397, 8, 23, "Print"],
Cell[1363643, 36096, 168, 2, 23, "Print"],
Cell[1363814, 36100, 395, 8, 23, "Print"],
Cell[1364212, 36110, 357, 8, 23, "Print"],
Cell[1364572, 36120, 362, 8, 23, "Print"],
Cell[1364937, 36130, 201, 5, 23, "Print"],
Cell[1365141, 36137, 313, 7, 23, "Print"],
Cell[1365457, 36146, 513, 10, 23, "Print"],
Cell[1365973, 36158, 126, 2, 23, "Print"],
Cell[1366102, 36162, 92, 1, 23, "Print"],
Cell[1366197, 36165, 119, 1, 23, "Print"],
Cell[1366319, 36168, 127, 2, 23, "Print"],
Cell[1366449, 36172, 273, 4, 62, "Print"],
Cell[1366725, 36178, 488, 10, 23, "Print"],
Cell[1367216, 36190, 261, 4, 43, "Print"],
Cell[1367480, 36196, 338, 7, 23, "Print"],
Cell[1367821, 36205, 375, 8, 23, "Print"],
Cell[1368199, 36215, 119, 1, 23, "Print"],
Cell[1368321, 36218, 162, 2, 23, "Print"],
Cell[1368486, 36222, 84, 1, 23, "Print"],
Cell[1368573, 36225, 86, 1, 23, "Print"],
Cell[1368662, 36228, 89, 1, 23, "Print"],
Cell[1368754, 36231, 198, 5, 23, "Print"],
Cell[1368955, 36238, 115, 1, 23, "Print"],
Cell[1369073, 36241, 421, 9, 23, "Print"],
Cell[1369497, 36252, 361, 8, 23, "Print"],
Cell[1369861, 36262, 198, 5, 23, "Print"],
Cell[1370062, 36269, 306, 7, 23, "Print"],
Cell[1370371, 36278, 197, 5, 23, "Print"],
Cell[1370571, 36285, 114, 1, 23, "Print"],
Cell[1370688, 36288, 357, 8, 23, "Print"],
Cell[1371048, 36298, 362, 8, 23, "Print"],
Cell[1371413, 36308, 201, 5, 23, "Print"],
Cell[1371617, 36315, 373, 8, 23, "Print"],
Cell[1371993, 36325, 200, 5, 23, "Print"],
Cell[1372196, 36332, 117, 1, 23, "Print"],
Cell[1372316, 36335, 421, 9, 23, "Print"],
Cell[1372740, 36346, 364, 8, 23, "Print"],
Cell[1373107, 36356, 201, 5, 23, "Print"],
Cell[1373311, 36363, 375, 8, 23, "Print"],
Cell[1373689, 36373, 200, 5, 23, "Print"],
Cell[1373892, 36380, 115, 1, 23, "Print"],
Cell[1374010, 36383, 419, 9, 23, "Print"],
Cell[1374432, 36394, 359, 8, 23, "Print"],
Cell[1374794, 36404, 201, 5, 23, "Print"],
Cell[1374998, 36411, 375, 8, 23, "Print"],
Cell[1375376, 36421, 198, 5, 23, "Print"],
Cell[1375577, 36428, 117, 1, 23, "Print"],
Cell[1375697, 36431, 419, 9, 23, "Print"],
Cell[1376119, 36442, 362, 8, 23, "Print"],
Cell[1376484, 36452, 199, 5, 23, "Print"],
Cell[1376686, 36459, 311, 7, 23, "Print"],
Cell[1377000, 36468, 198, 5, 23, "Print"],
Cell[1377201, 36475, 115, 1, 23, "Print"],
Cell[1377319, 36478, 421, 9, 23, "Print"],
Cell[1377743, 36489, 98, 1, 23, "Print"],
Cell[1377844, 36492, 220, 5, 23, "Print"],
Cell[1378067, 36499, 224, 5, 23, "Print"],
Cell[1378294, 36506, 362, 8, 23, "Print"],
Cell[1378659, 36516, 198, 5, 23, "Print"],
Cell[1378860, 36523, 308, 7, 23, "Print"],
Cell[1379171, 36532, 197, 5, 23, "Print"],
Cell[1379371, 36539, 114, 1, 23, "Print"],
Cell[1379488, 36542, 417, 9, 23, "Print"],
Cell[1379908, 36553, 98, 1, 23, "Print"],
Cell[1380009, 36556, 222, 5, 23, "Print"],
Cell[1380234, 36563, 222, 5, 23, "Print"],
Cell[1380459, 36570, 273, 4, 62, "Print"],
Cell[1380735, 36576, 489, 10, 23, "Print"],
Cell[1381227, 36588, 259, 4, 43, "Print"],
Cell[1381489, 36594, 335, 7, 23, "Print"],
Cell[1381827, 36603, 204, 3, 23, "Print"],
Cell[1382034, 36608, 415, 8, 23, "Print"],
Cell[1382452, 36618, 231, 5, 23, "Print"],
Cell[1382686, 36625, 153, 2, 23, "Print"],
Cell[1382842, 36629, 401, 8, 23, "Print"],
Cell[1383246, 36639, 168, 2, 23, "Print"],
Cell[1383417, 36643, 400, 8, 23, "Print"],
Cell[1383820, 36653, 361, 8, 23, "Print"],
Cell[1384184, 36663, 364, 8, 23, "Print"],
Cell[1384551, 36673, 201, 5, 23, "Print"],
Cell[1384755, 36680, 379, 8, 23, "Print"],
Cell[1385137, 36690, 453, 9, 23, "Print"],
Cell[1385593, 36701, 126, 2, 23, "Print"],
Cell[1385722, 36705, 97, 1, 23, "Print"],
Cell[1385822, 36708, 117, 1, 23, "Print"],
Cell[1385942, 36711, 128, 2, 23, "Print"],
Cell[1386073, 36715, 271, 4, 62, "Print"],
Cell[1386347, 36721, 486, 10, 23, "Print"],
Cell[1386836, 36733, 259, 4, 43, "Print"],
Cell[1387098, 36739, 336, 7, 23, "Print"],
Cell[1387437, 36748, 375, 8, 23, "Print"],
Cell[1387815, 36758, 119, 1, 23, "Print"],
Cell[1387937, 36761, 162, 2, 23, "Print"],
Cell[1388102, 36765, 84, 1, 23, "Print"],
Cell[1388189, 36768, 87, 1, 23, "Print"],
Cell[1388279, 36771, 75, 1, 23, "Print"],
Cell[1388357, 36774, 198, 5, 23, "Print"],
Cell[1388558, 36781, 115, 1, 23, "Print"],
Cell[1388676, 36784, 357, 8, 23, "Print"],
Cell[1389036, 36794, 364, 8, 23, "Print"],
Cell[1389403, 36804, 199, 5, 23, "Print"],
Cell[1389605, 36811, 311, 7, 23, "Print"],
Cell[1389919, 36820, 198, 5, 23, "Print"],
Cell[1390120, 36827, 117, 1, 23, "Print"],
Cell[1390240, 36830, 419, 9, 23, "Print"],
Cell[1390662, 36841, 362, 8, 23, "Print"],
Cell[1391027, 36851, 201, 5, 23, "Print"],
Cell[1391231, 36858, 373, 8, 23, "Print"],
Cell[1391607, 36868, 200, 5, 23, "Print"],
Cell[1391810, 36875, 115, 1, 23, "Print"],
Cell[1391928, 36878, 419, 9, 23, "Print"],
Cell[1392350, 36889, 362, 8, 23, "Print"],
Cell[1392715, 36899, 199, 5, 23, "Print"],
Cell[1392917, 36906, 375, 8, 23, "Print"],
Cell[1393295, 36916, 198, 5, 23, "Print"],
Cell[1393496, 36923, 115, 1, 23, "Print"],
Cell[1393614, 36926, 419, 9, 23, "Print"],
Cell[1394036, 36937, 364, 8, 23, "Print"],
Cell[1394403, 36947, 201, 5, 23, "Print"],
Cell[1394607, 36954, 375, 8, 23, "Print"],
Cell[1394985, 36964, 200, 5, 23, "Print"],
Cell[1395188, 36971, 115, 1, 23, "Print"],
Cell[1395306, 36974, 355, 8, 23, "Print"],
Cell[1395664, 36984, 359, 8, 23, "Print"],
Cell[1396026, 36994, 201, 5, 23, "Print"],
Cell[1396230, 37001, 375, 8, 23, "Print"],
Cell[1396608, 37011, 200, 5, 23, "Print"],
Cell[1396811, 37018, 115, 1, 23, "Print"],
Cell[1396929, 37021, 421, 9, 23, "Print"],
Cell[1397353, 37032, 96, 1, 23, "Print"],
Cell[1397452, 37035, 219, 5, 23, "Print"],
Cell[1397674, 37042, 222, 5, 23, "Print"],
Cell[1397899, 37049, 359, 8, 23, "Print"],
Cell[1398261, 37059, 199, 5, 23, "Print"],
Cell[1398463, 37066, 371, 8, 23, "Print"],
Cell[1398837, 37076, 198, 5, 23, "Print"],
Cell[1399038, 37083, 115, 1, 23, "Print"],
Cell[1399156, 37086, 357, 8, 23, "Print"],
Cell[1399516, 37096, 96, 1, 23, "Print"],
Cell[1399615, 37099, 222, 5, 23, "Print"],
Cell[1399840, 37106, 224, 5, 23, "Print"],
Cell[1400067, 37113, 270, 4, 62, "Print"],
Cell[1400340, 37119, 486, 10, 23, "Print"],
Cell[1400829, 37131, 258, 4, 43, "Print"],
Cell[1401090, 37137, 335, 7, 23, "Print"],
Cell[1401428, 37146, 204, 3, 23, "Print"],
Cell[1401635, 37151, 416, 8, 23, "Print"],
Cell[1402054, 37161, 235, 5, 23, "Print"],
Cell[1402292, 37168, 155, 2, 23, "Print"],
Cell[1402450, 37172, 402, 8, 23, "Print"],
Cell[1402855, 37182, 170, 2, 23, "Print"],
Cell[1403028, 37186, 399, 8, 23, "Print"],
Cell[1403430, 37196, 359, 8, 23, "Print"],
Cell[1403792, 37206, 359, 8, 23, "Print"],
Cell[1404154, 37216, 201, 5, 23, "Print"],
Cell[1404358, 37223, 317, 7, 23, "Print"],
Cell[1404678, 37232, 449, 9, 23, "Print"],
Cell[1405130, 37243, 126, 2, 23, "Print"],
Cell[1405259, 37247, 92, 1, 23, "Print"],
Cell[1405354, 37250, 119, 1, 23, "Print"],
Cell[1405476, 37253, 130, 2, 23, "Print"],
Cell[1405609, 37257, 273, 4, 62, "Print"],
Cell[1405885, 37263, 488, 10, 23, "Print"],
Cell[1406376, 37275, 261, 4, 43, "Print"],
Cell[1406640, 37281, 337, 7, 23, "Print"],
Cell[1406980, 37290, 375, 8, 23, "Print"],
Cell[1407358, 37300, 117, 1, 23, "Print"],
Cell[1407478, 37303, 160, 2, 23, "Print"],
Cell[1407641, 37307, 84, 1, 23, "Print"],
Cell[1407728, 37310, 89, 1, 23, "Print"],
Cell[1407820, 37313, 92, 1, 23, "Print"],
Cell[1407915, 37316, 200, 5, 23, "Print"],
Cell[1408118, 37323, 117, 1, 23, "Print"],
Cell[1408238, 37326, 419, 9, 23, "Print"],
Cell[1408660, 37337, 362, 8, 23, "Print"],
Cell[1409025, 37347, 199, 5, 23, "Print"],
Cell[1409227, 37354, 373, 8, 23, "Print"],
Cell[1409603, 37364, 198, 5, 23, "Print"],
Cell[1409804, 37371, 115, 1, 23, "Print"],
Cell[1409922, 37374, 359, 8, 23, "Print"],
Cell[1410284, 37384, 362, 8, 23, "Print"],
Cell[1410649, 37394, 201, 5, 23, "Print"],
Cell[1410853, 37401, 375, 8, 23, "Print"],
Cell[1411231, 37411, 200, 5, 23, "Print"],
Cell[1411434, 37418, 117, 1, 23, "Print"],
Cell[1411554, 37421, 421, 9, 23, "Print"],
Cell[1411978, 37432, 359, 8, 23, "Print"],
Cell[1412340, 37442, 199, 5, 23, "Print"],
Cell[1412542, 37449, 307, 7, 23, "Print"],
Cell[1412852, 37458, 198, 5, 23, "Print"],
Cell[1413053, 37465, 115, 1, 23, "Print"],
Cell[1413171, 37468, 357, 8, 23, "Print"],
Cell[1413531, 37478, 364, 8, 23, "Print"],
Cell[1413898, 37488, 201, 5, 23, "Print"],
Cell[1414102, 37495, 375, 8, 23, "Print"],
Cell[1414480, 37505, 200, 5, 23, "Print"],
Cell[1414683, 37512, 117, 1, 23, "Print"],
Cell[1414803, 37515, 419, 9, 23, "Print"],
Cell[1415225, 37526, 361, 8, 23, "Print"],
Cell[1415589, 37536, 201, 5, 23, "Print"],
Cell[1415793, 37543, 313, 7, 23, "Print"],
Cell[1416109, 37552, 200, 5, 23, "Print"],
Cell[1416312, 37559, 117, 1, 23, "Print"],
Cell[1416432, 37562, 421, 9, 23, "Print"],
Cell[1416856, 37573, 96, 1, 23, "Print"],
Cell[1416955, 37576, 222, 5, 23, "Print"],
Cell[1417180, 37583, 222, 5, 23, "Print"],
Cell[1417405, 37590, 364, 8, 23, "Print"],
Cell[1417772, 37600, 201, 5, 23, "Print"],
Cell[1417976, 37607, 375, 8, 23, "Print"],
Cell[1418354, 37617, 200, 5, 23, "Print"],
Cell[1418557, 37624, 117, 1, 23, "Print"],
Cell[1418677, 37627, 419, 9, 23, "Print"],
Cell[1419099, 37638, 96, 1, 23, "Print"],
Cell[1419198, 37641, 222, 5, 23, "Print"],
Cell[1419423, 37648, 224, 5, 23, "Print"],
Cell[1419650, 37655, 273, 4, 62, "Print"],
Cell[1419926, 37661, 491, 10, 23, "Print"],
Cell[1420420, 37673, 259, 4, 43, "Print"],
Cell[1420682, 37679, 337, 7, 23, "Print"],
Cell[1421022, 37688, 202, 3, 23, "Print"],
Cell[1421227, 37693, 413, 8, 23, "Print"],
Cell[1421643, 37703, 238, 5, 23, "Print"],
Cell[1421884, 37710, 155, 2, 23, "Print"],
Cell[1422042, 37714, 402, 8, 23, "Print"],
Cell[1422447, 37724, 168, 2, 23, "Print"],
Cell[1422618, 37728, 400, 8, 23, "Print"],
Cell[1423021, 37738, 365, 8, 23, "Print"],
Cell[1423389, 37748, 360, 8, 23, "Print"],
Cell[1423752, 37758, 199, 5, 23, "Print"],
Cell[1423954, 37765, 315, 7, 23, "Print"],
Cell[1424272, 37774, 451, 9, 23, "Print"],
Cell[1424726, 37785, 126, 2, 23, "Print"],
Cell[1424855, 37789, 97, 1, 23, "Print"],
Cell[1424955, 37792, 119, 1, 23, "Print"],
Cell[1425077, 37795, 130, 2, 23, "Print"],
Cell[1425210, 37799, 270, 4, 62, "Print"],
Cell[1425483, 37805, 488, 10, 23, "Print"],
Cell[1425974, 37817, 261, 4, 43, "Print"],
Cell[1426238, 37823, 337, 7, 23, "Print"],
Cell[1426578, 37832, 379, 8, 23, "Print"],
Cell[1426960, 37842, 119, 1, 23, "Print"],
Cell[1427082, 37845, 162, 2, 23, "Print"],
Cell[1427247, 37849, 84, 1, 23, "Print"],
Cell[1427334, 37852, 87, 1, 23, "Print"],
Cell[1427424, 37855, 90, 1, 23, "Print"],
Cell[1427517, 37858, 198, 5, 23, "Print"],
Cell[1427718, 37865, 115, 1, 23, "Print"],
Cell[1427836, 37868, 355, 8, 23, "Print"],
Cell[1428194, 37878, 361, 8, 23, "Print"],
Cell[1428558, 37888, 201, 5, 23, "Print"],
Cell[1428762, 37895, 375, 8, 23, "Print"],
Cell[1429140, 37905, 200, 5, 23, "Print"],
Cell[1429343, 37912, 117, 1, 23, "Print"],
Cell[1429463, 37915, 419, 9, 23, "Print"],
Cell[1429885, 37926, 362, 8, 23, "Print"],
Cell[1430250, 37936, 199, 5, 23, "Print"],
Cell[1430452, 37943, 373, 8, 23, "Print"],
Cell[1430828, 37953, 198, 5, 23, "Print"],
Cell[1431029, 37960, 117, 1, 23, "Print"],
Cell[1431149, 37963, 359, 8, 23, "Print"],
Cell[1431511, 37973, 361, 8, 23, "Print"],
Cell[1431875, 37983, 201, 5, 23, "Print"],
Cell[1432079, 37990, 375, 8, 23, "Print"],
Cell[1432457, 38000, 200, 5, 23, "Print"],
Cell[1432660, 38007, 117, 1, 23, "Print"],
Cell[1432780, 38010, 357, 8, 23, "Print"],
Cell[1433140, 38020, 362, 8, 23, "Print"],
Cell[1433505, 38030, 201, 5, 23, "Print"],
Cell[1433709, 38037, 375, 8, 23, "Print"],
Cell[1434087, 38047, 200, 5, 23, "Print"],
Cell[1434290, 38054, 117, 1, 23, "Print"],
Cell[1434410, 38057, 421, 9, 23, "Print"],
Cell[1434834, 38068, 362, 8, 23, "Print"],
Cell[1435199, 38078, 199, 5, 23, "Print"],
Cell[1435401, 38085, 373, 8, 23, "Print"],
Cell[1435777, 38095, 198, 5, 23, "Print"],
Cell[1435978, 38102, 117, 1, 23, "Print"],
Cell[1436098, 38105, 359, 8, 23, "Print"],
Cell[1436460, 38115, 96, 1, 23, "Print"],
Cell[1436559, 38118, 220, 5, 23, "Print"],
Cell[1436782, 38125, 224, 5, 23, "Print"],
Cell[1437009, 38132, 359, 8, 23, "Print"],
Cell[1437371, 38142, 199, 5, 23, "Print"],
Cell[1437573, 38149, 375, 8, 23, "Print"],
Cell[1437951, 38159, 200, 5, 23, "Print"],
Cell[1438154, 38166, 117, 1, 23, "Print"],
Cell[1438274, 38169, 359, 8, 23, "Print"],
Cell[1438636, 38179, 98, 1, 23, "Print"],
Cell[1438737, 38182, 222, 5, 23, "Print"],
Cell[1438962, 38189, 224, 5, 23, "Print"],
Cell[1439189, 38196, 271, 4, 62, "Print"],
Cell[1439463, 38202, 485, 10, 23, "Print"],
Cell[1439951, 38214, 259, 4, 43, "Print"],
Cell[1440213, 38220, 334, 7, 23, "Print"],
Cell[1440550, 38229, 204, 3, 23, "Print"],
Cell[1440757, 38234, 411, 8, 23, "Print"],
Cell[1441171, 38244, 238, 5, 23, "Print"],
Cell[1441412, 38251, 155, 2, 23, "Print"],
Cell[1441570, 38255, 404, 8, 23, "Print"],
Cell[1441977, 38265, 168, 2, 23, "Print"],
Cell[1442148, 38269, 397, 8, 23, "Print"],
Cell[1442548, 38279, 359, 8, 23, "Print"],
Cell[1442910, 38289, 362, 8, 23, "Print"],
Cell[1443275, 38299, 199, 5, 23, "Print"],
Cell[1443477, 38306, 379, 8, 23, "Print"],
Cell[1443859, 38316, 515, 10, 23, "Print"],
Cell[1444377, 38328, 126, 2, 23, "Print"],
Cell[1444506, 38332, 95, 1, 23, "Print"],
Cell[1444604, 38335, 119, 1, 23, "Print"],
Cell[1444726, 38338, 128, 2, 23, "Print"],
Cell[1444857, 38342, 271, 4, 62, "Print"],
Cell[1445131, 38348, 486, 10, 23, "Print"],
Cell[1445620, 38360, 259, 4, 43, "Print"],
Cell[1445882, 38366, 338, 7, 23, "Print"],
Cell[1446223, 38375, 377, 8, 23, "Print"],
Cell[1446603, 38385, 119, 1, 23, "Print"],
Cell[1446725, 38388, 162, 2, 23, "Print"],
Cell[1446890, 38392, 84, 1, 23, "Print"],
Cell[1446977, 38395, 89, 1, 23, "Print"],
Cell[1447069, 38398, 92, 1, 23, "Print"],
Cell[1447164, 38401, 200, 5, 23, "Print"],
Cell[1447367, 38408, 115, 1, 23, "Print"],
Cell[1447485, 38411, 419, 9, 23, "Print"],
Cell[1447907, 38422, 364, 8, 23, "Print"],
Cell[1448274, 38432, 198, 5, 23, "Print"],
Cell[1448475, 38439, 370, 8, 23, "Print"],
Cell[1448848, 38449, 197, 5, 23, "Print"],
Cell[1449048, 38456, 114, 1, 23, "Print"],
Cell[1449165, 38459, 418, 9, 23, "Print"],
Cell[1449586, 38470, 362, 8, 23, "Print"],
Cell[1449951, 38480, 199, 5, 23, "Print"],
Cell[1450153, 38487, 311, 7, 23, "Print"],
Cell[1450467, 38496, 198, 5, 23, "Print"],
Cell[1450668, 38503, 115, 1, 23, "Print"],
Cell[1450786, 38506, 421, 9, 23, "Print"],
Cell[1451210, 38517, 361, 8, 23, "Print"],
Cell[1451574, 38527, 201, 5, 23, "Print"],
Cell[1451778, 38534, 313, 7, 23, "Print"],
Cell[1452094, 38543, 200, 5, 23, "Print"],
Cell[1452297, 38550, 117, 1, 23, "Print"],
Cell[1452417, 38553, 421, 9, 23, "Print"],
Cell[1452841, 38564, 364, 8, 23, "Print"],
Cell[1453208, 38574, 199, 5, 23, "Print"],
Cell[1453410, 38581, 373, 8, 23, "Print"],
Cell[1453786, 38591, 198, 5, 23, "Print"],
Cell[1453987, 38598, 115, 1, 23, "Print"],
Cell[1454105, 38601, 419, 9, 23, "Print"],
Cell[1454527, 38612, 361, 8, 23, "Print"],
Cell[1454891, 38622, 198, 5, 23, "Print"],
Cell[1455092, 38629, 310, 7, 23, "Print"],
Cell[1455405, 38638, 197, 5, 23, "Print"],
Cell[1455605, 38645, 114, 1, 23, "Print"],
Cell[1455722, 38648, 419, 9, 23, "Print"],
Cell[1456144, 38659, 96, 1, 23, "Print"],
Cell[1456243, 38662, 222, 5, 23, "Print"],
Cell[1456468, 38669, 224, 5, 23, "Print"],
Cell[1456695, 38676, 361, 8, 23, "Print"],
Cell[1457059, 38686, 201, 5, 23, "Print"],
Cell[1457263, 38693, 311, 7, 23, "Print"],
Cell[1457577, 38702, 198, 5, 23, "Print"],
Cell[1457778, 38709, 115, 1, 23, "Print"],
Cell[1457896, 38712, 421, 9, 23, "Print"],
Cell[1458320, 38723, 98, 1, 23, "Print"],
Cell[1458421, 38726, 222, 5, 23, "Print"],
Cell[1458646, 38733, 224, 5, 23, "Print"],
Cell[1458873, 38740, 273, 4, 62, "Print"],
Cell[1459149, 38746, 490, 10, 23, "Print"],
Cell[1459642, 38758, 259, 4, 43, "Print"],
Cell[1459904, 38764, 335, 7, 23, "Print"],
Cell[1460242, 38773, 202, 3, 23, "Print"],
Cell[1460447, 38778, 410, 8, 23, "Print"],
Cell[1460860, 38788, 236, 5, 23, "Print"],
Cell[1461099, 38795, 155, 2, 23, "Print"],
Cell[1461257, 38799, 397, 8, 23, "Print"],
Cell[1461657, 38809, 170, 2, 23, "Print"],
Cell[1461830, 38813, 401, 8, 23, "Print"],
Cell[1462234, 38823, 358, 8, 23, "Print"],
Cell[1462595, 38833, 361, 8, 23, "Print"],
Cell[1462959, 38843, 201, 5, 23, "Print"],
Cell[1463163, 38850, 379, 8, 23, "Print"],
Cell[1463545, 38860, 451, 9, 23, "Print"],
Cell[1463999, 38871, 126, 2, 23, "Print"],
Cell[1464128, 38875, 95, 1, 23, "Print"],
Cell[1464226, 38878, 119, 1, 23, "Print"],
Cell[1464348, 38881, 128, 2, 23, "Print"],
Cell[1464479, 38885, 271, 4, 62, "Print"],
Cell[1464753, 38891, 486, 10, 23, "Print"],
Cell[1465242, 38903, 259, 4, 43, "Print"],
Cell[1465504, 38909, 335, 7, 23, "Print"],
Cell[1465842, 38918, 375, 8, 23, "Print"],
Cell[1466220, 38928, 117, 1, 23, "Print"],
Cell[1466340, 38931, 160, 2, 23, "Print"],
Cell[1466503, 38935, 86, 1, 23, "Print"],
Cell[1466592, 38938, 89, 1, 23, "Print"],
Cell[1466684, 38941, 75, 1, 23, "Print"],
Cell[1466762, 38944, 198, 5, 23, "Print"],
Cell[1466963, 38951, 115, 1, 23, "Print"],
Cell[1467081, 38954, 419, 9, 23, "Print"],
Cell[1467503, 38965, 364, 8, 23, "Print"],
Cell[1467870, 38975, 199, 5, 23, "Print"],
Cell[1468072, 38982, 309, 7, 23, "Print"],
Cell[1468384, 38991, 198, 5, 23, "Print"],
Cell[1468585, 38998, 115, 1, 23, "Print"],
Cell[1468703, 39001, 421, 9, 23, "Print"],
Cell[1469127, 39012, 361, 8, 23, "Print"],
Cell[1469491, 39022, 199, 5, 23, "Print"],
Cell[1469693, 39029, 375, 8, 23, "Print"],
Cell[1470071, 39039, 198, 5, 23, "Print"],
Cell[1470272, 39046, 115, 1, 23, "Print"],
Cell[1470390, 39049, 352, 8, 23, "Print"],
Cell[1470745, 39059, 362, 8, 23, "Print"],
Cell[1471110, 39069, 201, 5, 23, "Print"],
Cell[1471314, 39076, 375, 8, 23, "Print"],
Cell[1471692, 39086, 200, 5, 23, "Print"],
Cell[1471895, 39093, 117, 1, 23, "Print"],
Cell[1472015, 39096, 359, 8, 23, "Print"],
Cell[1472377, 39106, 362, 8, 23, "Print"],
Cell[1472742, 39116, 201, 5, 23, "Print"],
Cell[1472946, 39123, 375, 8, 23, "Print"],
Cell[1473324, 39133, 200, 5, 23, "Print"],
Cell[1473527, 39140, 117, 1, 23, "Print"],
Cell[1473647, 39143, 421, 9, 23, "Print"],
Cell[1474071, 39154, 361, 8, 23, "Print"],
Cell[1474435, 39164, 201, 5, 23, "Print"],
Cell[1474639, 39171, 311, 7, 23, "Print"],
Cell[1474953, 39180, 200, 5, 23, "Print"],
Cell[1475156, 39187, 115, 1, 23, "Print"],
Cell[1475274, 39190, 357, 8, 23, "Print"],
Cell[1475634, 39200, 98, 1, 23, "Print"],
Cell[1475735, 39203, 220, 5, 23, "Print"],
Cell[1475958, 39210, 221, 5, 23, "Print"],
Cell[1476182, 39217, 362, 8, 23, "Print"],
Cell[1476547, 39227, 199, 5, 23, "Print"],
Cell[1476749, 39234, 373, 8, 23, "Print"],
Cell[1477125, 39244, 198, 5, 23, "Print"],
Cell[1477326, 39251, 115, 1, 23, "Print"],
Cell[1477444, 39254, 417, 9, 23, "Print"],
Cell[1477864, 39265, 98, 1, 23, "Print"],
Cell[1477965, 39268, 222, 5, 23, "Print"],
Cell[1478190, 39275, 224, 5, 23, "Print"],
Cell[1478417, 39282, 273, 4, 62, "Print"],
Cell[1478693, 39288, 487, 10, 23, "Print"],
Cell[1479183, 39300, 259, 4, 43, "Print"],
Cell[1479445, 39306, 335, 7, 23, "Print"],
Cell[1479783, 39315, 202, 3, 23, "Print"],
Cell[1479988, 39320, 402, 8, 23, "Print"],
Cell[1480393, 39330, 236, 5, 23, "Print"],
Cell[1480632, 39337, 153, 2, 23, "Print"],
Cell[1480788, 39341, 397, 8, 23, "Print"],
Cell[1481188, 39351, 170, 2, 23, "Print"],
Cell[1481361, 39355, 401, 8, 23, "Print"],
Cell[1481765, 39365, 364, 8, 23, "Print"],
Cell[1482132, 39375, 361, 8, 23, "Print"],
Cell[1482496, 39385, 201, 5, 23, "Print"],
Cell[1482700, 39392, 379, 8, 23, "Print"],
Cell[1483082, 39402, 449, 9, 23, "Print"],
Cell[1483534, 39413, 124, 2, 23, "Print"],
Cell[1483661, 39417, 93, 1, 23, "Print"],
Cell[1483757, 39420, 117, 1, 23, "Print"],
Cell[1483877, 39423, 128, 2, 23, "Print"],
Cell[1484008, 39427, 270, 4, 62, "Print"],
Cell[1484281, 39433, 485, 10, 23, "Print"],
Cell[1484769, 39445, 258, 4, 43, "Print"],
Cell[1485030, 39451, 334, 7, 23, "Print"],
Cell[1485367, 39460, 374, 8, 23, "Print"],
Cell[1485744, 39470, 116, 1, 23, "Print"],
Cell[1485863, 39473, 159, 2, 23, "Print"],
Cell[1486025, 39477, 84, 1, 23, "Print"],
Cell[1486112, 39480, 87, 1, 23, "Print"],
Cell[1486202, 39483, 75, 1, 23, "Print"],
Cell[1486280, 39486, 198, 5, 23, "Print"],
Cell[1486481, 39493, 115, 1, 23, "Print"],
Cell[1486599, 39496, 415, 9, 23, "Print"],
Cell[1487017, 39507, 364, 8, 23, "Print"],
Cell[1487384, 39517, 198, 5, 23, "Print"],
Cell[1487585, 39524, 308, 7, 23, "Print"],
Cell[1487896, 39533, 197, 5, 23, "Print"],
Cell[1488096, 39540, 114, 1, 23, "Print"],
Cell[1488213, 39543, 357, 8, 23, "Print"],
Cell[1488573, 39553, 364, 8, 23, "Print"],
Cell[1488940, 39563, 201, 5, 23, "Print"],
Cell[1489144, 39570, 375, 8, 23, "Print"],
Cell[1489522, 39580, 200, 5, 23, "Print"],
Cell[1489725, 39587, 117, 1, 23, "Print"],
Cell[1489845, 39590, 421, 9, 23, "Print"],
Cell[1490269, 39601, 364, 8, 23, "Print"],
Cell[1490636, 39611, 197, 5, 23, "Print"],
Cell[1490836, 39618, 375, 8, 23, "Print"],
Cell[1491214, 39628, 200, 5, 23, "Print"],
Cell[1491417, 39635, 117, 1, 23, "Print"],
Cell[1491537, 39638, 421, 9, 23, "Print"],
Cell[1491961, 39649, 362, 8, 23, "Print"],
Cell[1492326, 39659, 199, 5, 23, "Print"],
Cell[1492528, 39666, 375, 8, 23, "Print"],
Cell[1492906, 39676, 200, 5, 23, "Print"],
Cell[1493109, 39683, 117, 1, 23, "Print"],
Cell[1493229, 39686, 421, 9, 23, "Print"],
Cell[1493653, 39697, 359, 8, 23, "Print"],
Cell[1494015, 39707, 199, 5, 23, "Print"],
Cell[1494217, 39714, 375, 8, 23, "Print"],
Cell[1494595, 39724, 200, 5, 23, "Print"],
Cell[1494798, 39731, 117, 1, 23, "Print"],
Cell[1494918, 39734, 419, 9, 23, "Print"],
Cell[1495340, 39745, 98, 1, 23, "Print"],
Cell[1495441, 39748, 220, 5, 23, "Print"],
Cell[1495664, 39755, 222, 5, 23, "Print"],
Cell[1495889, 39762, 364, 8, 23, "Print"],
Cell[1496256, 39772, 201, 5, 23, "Print"],
Cell[1496460, 39779, 373, 8, 23, "Print"],
Cell[1496836, 39789, 200, 5, 23, "Print"],
Cell[1497039, 39796, 117, 1, 23, "Print"],
Cell[1497159, 39799, 419, 9, 23, "Print"],
Cell[1497581, 39810, 96, 1, 23, "Print"],
Cell[1497680, 39813, 220, 5, 23, "Print"],
Cell[1497903, 39820, 222, 5, 23, "Print"],
Cell[1498128, 39827, 273, 4, 62, "Print"],
Cell[1498404, 39833, 490, 10, 23, "Print"],
Cell[1498897, 39845, 261, 4, 43, "Print"],
Cell[1499161, 39851, 339, 7, 23, "Print"],
Cell[1499503, 39860, 204, 3, 23, "Print"],
Cell[1499710, 39865, 406, 8, 23, "Print"],
Cell[1500119, 39875, 235, 5, 23, "Print"],
Cell[1500357, 39882, 155, 2, 23, "Print"],
Cell[1500515, 39886, 398, 8, 23, "Print"],
Cell[1500916, 39896, 168, 2, 23, "Print"],
Cell[1501087, 39900, 403, 8, 23, "Print"],
Cell[1501493, 39910, 364, 8, 23, "Print"],
Cell[1501860, 39920, 364, 8, 23, "Print"],
Cell[1502227, 39930, 201, 5, 23, "Print"],
Cell[1502431, 39937, 379, 8, 23, "Print"],
Cell[1502813, 39947, 515, 10, 23, "Print"],
Cell[1503331, 39959, 461, 9, 23, "Print"],
Cell[1503795, 39970, 153, 2, 23, "Print"],
Cell[1503951, 39974, 94, 1, 23, "Print"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)

